@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Thread Safe Resource Manager                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1999-2009, Andi Gutmans, Sascha Schumann, Zeev Suraski |
+   | Copyright (c) 1999-2006, Andi Gutmans, Sascha Schumann, Zeev Suraski |
    | This source file is subject to the TSRM license, that is bundled     |
    | with this package in the file LICENSE                                |
    +----------------------------------------------------------------------+
@@ -480,52 +480,6 @@ void ts_free_thread(void)
 }
 
 
-/* frees all resources allocated for all threads except current */
-void ts_free_worker_threads(void)
-{
-	tsrm_tls_entry *thread_resources;
-	int i;
-	THREAD_T thread_id = tsrm_thread_id();
-	int hash_value;
-	tsrm_tls_entry *last=NULL;
-
-	tsrm_mutex_lock(tsmm_mutex);
-	hash_value = THREAD_HASH_OF(thread_id, tsrm_tls_table_size);
-	thread_resources = tsrm_tls_table[hash_value];
-
-	while (thread_resources) {
-		if (thread_resources->thread_id != thread_id) {
-			for (i=0; i<thread_resources->count; i++) {
-				if (resource_types_table[i].dtor) {
-					resource_types_table[i].dtor(thread_resources->storage[i], &thread_resources->storage);
-				}
-			}
-			for (i=0; i<thread_resources->count; i++) {
-				free(thread_resources->storage[i]);
-			}
-			free(thread_resources->storage);
-			if (last) {
-				last->next = thread_resources->next;
-			} else {
-				tsrm_tls_table[hash_value] = thread_resources->next;
-			}
-			free(thread_resources);
-			if (last) {
-				thread_resources = last->next;
-			} else {
-				thread_resources = tsrm_tls_table[hash_value];
-			}
-		} else {
-			if (thread_resources->next) {
-				last = thread_resources;
-			}
-			thread_resources = thread_resources->next;
-		}
-	}
-	tsrm_mutex_unlock(tsmm_mutex);
-}
-
-
 /* deallocates all occurrences of a given id */
 void ts_free_id(ts_rsrc_id id)
 {
@@ -647,26 +601,19 @@ TSRM_API void tsrm_mutex_free(MUTEX_T mutexp)
 }
 
 
-/*
-  Lock a mutex.
-  A return value of 0 indicates success
-*/
+/* Lock a mutex */
 TSRM_API int tsrm_mutex_lock(MUTEX_T mutexp)
 {
 	TSRM_ERROR((TSRM_ERROR_LEVEL_INFO, "Mutex locked thread: %ld", tsrm_thread_id()));
 #ifdef TSRM_WIN32
 	EnterCriticalSection(mutexp);
-	return 0;
+	return 1;
 #elif defined(GNUPTH)
-	if (pth_mutex_acquire(mutexp, 0, NULL)) {
-		return 0;
-	}
-	return -1;
+	return pth_mutex_acquire(mutexp, 0, NULL);
 #elif defined(PTHREADS)
 	return pthread_mutex_lock(mutexp);
 #elif defined(NSAPI)
-	crit_enter(mutexp);
-	return 0;
+	return crit_enter(mutexp);
 #elif defined(PI3WEB)
 	return PISync_lock(mutexp);
 #elif defined(TSRM_ST)
@@ -679,26 +626,19 @@ TSRM_API int tsrm_mutex_lock(MUTEX_T mutexp)
 }
 
 
-/*
-  Unlock a mutex.
-  A return value of 0 indicates success
-*/
+/* Unlock a mutex */
 TSRM_API int tsrm_mutex_unlock(MUTEX_T mutexp)
 {
 	TSRM_ERROR((TSRM_ERROR_LEVEL_INFO, "Mutex unlocked thread: %ld", tsrm_thread_id()));
 #ifdef TSRM_WIN32
 	LeaveCriticalSection(mutexp);
-	return 0;
+	return 1;
 #elif defined(GNUPTH)
-	if (pth_mutex_release(mutexp)) {
-		return 0;
-	}
-	return -1;
+	return pth_mutex_release(mutexp);
 #elif defined(PTHREADS)
 	return pthread_mutex_unlock(mutexp);
 #elif defined(NSAPI)
-	crit_exit(mutexp);
-	return 0;
+	return crit_exit(mutexp);
 #elif defined(PI3WEB)
 	return PISync_unlock(mutexp);
 #elif defined(TSRM_ST)
