@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_constants.c,v 1.71.2.5.2.7.2.17 2009/01/12 21:54:37 stas Exp $ */
+/* $Id: zend_constants.c,v 1.71.2.1 2006/01/04 23:53:04 andi Exp $ */
 
 #include "zend.h"
 #include "zend_constants.h"
@@ -53,19 +53,17 @@ void zend_copy_constants(HashTable *target, HashTable *source)
 }
 
 
-static int clean_non_persistent_constant(const zend_constant *c TSRMLS_DC)
+static int clean_non_persistent_constant(zend_constant *c TSRMLS_DC)
 {
-	return (c->flags & CONST_PERSISTENT) ? ZEND_HASH_APPLY_STOP : ZEND_HASH_APPLY_REMOVE;
+	if (c->flags & CONST_PERSISTENT) {
+		return EG(full_tables_cleanup) ? 0 : ZEND_HASH_APPLY_STOP;
+	} else {
+		return EG(full_tables_cleanup) ? 1 : ZEND_HASH_APPLY_REMOVE;
+	}
 }
 
 
-static int clean_non_persistent_constant_full(const zend_constant *c TSRMLS_DC)
-{
-	return (c->flags & CONST_PERSISTENT) ? 0 : 1;
-}
-
-
-static int clean_module_constant(const zend_constant *c, int *module_number TSRMLS_DC)
+static int clean_module_constant(zend_constant *c, int *module_number TSRMLS_DC)
 {
 	if (c->module_number == *module_number) {
 		return 1;
@@ -96,12 +94,10 @@ int zend_startup_constants(TSRMLS_D)
 void zend_register_standard_constants(TSRMLS_D)
 {
 	REGISTER_MAIN_LONG_CONSTANT("E_ERROR", E_ERROR, CONST_PERSISTENT | CONST_CS);
-	REGISTER_MAIN_LONG_CONSTANT("E_RECOVERABLE_ERROR", E_RECOVERABLE_ERROR, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_WARNING", E_WARNING, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_PARSE", E_PARSE, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_NOTICE", E_NOTICE, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_STRICT", E_STRICT, CONST_PERSISTENT | CONST_CS);
-	REGISTER_MAIN_LONG_CONSTANT("E_DEPRECATED", E_DEPRECATED, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_CORE_ERROR", E_CORE_ERROR, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_CORE_WARNING", E_CORE_WARNING, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_COMPILE_ERROR", E_COMPILE_ERROR, CONST_PERSISTENT | CONST_CS);
@@ -109,7 +105,6 @@ void zend_register_standard_constants(TSRMLS_D)
 	REGISTER_MAIN_LONG_CONSTANT("E_USER_ERROR", E_USER_ERROR, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_USER_WARNING", E_USER_WARNING, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("E_USER_NOTICE", E_USER_NOTICE, CONST_PERSISTENT | CONST_CS);
-	REGISTER_MAIN_LONG_CONSTANT("E_USER_DEPRECATED", E_USER_DEPRECATED, CONST_PERSISTENT | CONST_CS);
 
 	REGISTER_MAIN_LONG_CONSTANT("E_ALL", E_ALL, CONST_PERSISTENT | CONST_CS);
 
@@ -117,7 +112,8 @@ void zend_register_standard_constants(TSRMLS_D)
 	{
 		zend_constant c;
 	
-		c.flags = CONST_PERSISTENT | CONST_CT_SUBST;
+		c.value.type = IS_BOOL;
+		c.flags = CONST_PERSISTENT;
 		c.module_number = 0;
 
 		c.name = zend_strndup(ZEND_STRL("TRUE"));
@@ -132,23 +128,15 @@ void zend_register_standard_constants(TSRMLS_D)
 		c.value.type = IS_BOOL;
 		zend_register_constant(&c TSRMLS_CC);
 
-		c.name = zend_strndup(ZEND_STRL("NULL"));
-		c.name_len = sizeof("NULL");
-		c.value.type = IS_NULL;
-		zend_register_constant(&c TSRMLS_CC);
-
-		c.flags = CONST_PERSISTENT;
-
 		c.name = zend_strndup(ZEND_STRL("ZEND_THREAD_SAFE"));
 		c.name_len = sizeof("ZEND_THREAD_SAFE");
 		c.value.value.lval = ZTS_V;
 		c.value.type = IS_BOOL;
 		zend_register_constant(&c TSRMLS_CC);
 
-		c.name = zend_strndup(ZEND_STRL("ZEND_DEBUG_BUILD"));
-		c.name_len = sizeof("ZEND_DEBUG_BUILD");
-		c.value.value.lval = ZEND_DEBUG;
-		c.value.type = IS_BOOL;
+		c.name = zend_strndup(ZEND_STRL("NULL"));
+		c.name_len = sizeof("NULL");
+		c.value.type = IS_NULL;
 		zend_register_constant(&c TSRMLS_CC);
 	}
 }
@@ -165,14 +153,14 @@ int zend_shutdown_constants(TSRMLS_D)
 void clean_non_persistent_constants(TSRMLS_D)
 {
 	if (EG(full_tables_cleanup)) {
-		zend_hash_apply(EG(zend_constants), (apply_func_t) clean_non_persistent_constant_full TSRMLS_CC);
+		zend_hash_apply(EG(zend_constants), (apply_func_t) clean_non_persistent_constant TSRMLS_CC);
 	} else {
 		zend_hash_reverse_apply(EG(zend_constants), (apply_func_t) clean_non_persistent_constant TSRMLS_CC);
 	}
 }
 
 
-ZEND_API void zend_register_long_constant(const char *name, uint name_len, long lval, int flags, int module_number TSRMLS_DC)
+ZEND_API void zend_register_long_constant(char *name, uint name_len, long lval, int flags, int module_number TSRMLS_DC)
 {
 	zend_constant c;
 	
@@ -186,7 +174,7 @@ ZEND_API void zend_register_long_constant(const char *name, uint name_len, long
 }
 
 
-ZEND_API void zend_register_double_constant(const char *name, uint name_len, double dval, int flags, int module_number TSRMLS_DC)
+ZEND_API void zend_register_double_constant(char *name, uint name_len, double dval, int flags, int module_number TSRMLS_DC)
 {
 	zend_constant c;
 	
@@ -200,7 +188,7 @@ ZEND_API void zend_register_double_constant(const char *name, uint name_len, dou
 }
 
 
-ZEND_API void zend_register_stringl_constant(const char *name, uint name_len, char *strval, uint strlen, int flags, int module_number TSRMLS_DC)
+ZEND_API void zend_register_stringl_constant(char *name, uint name_len, char *strval, uint strlen, int flags, int module_number TSRMLS_DC)
 {
 	zend_constant c;
 	
@@ -215,49 +203,85 @@ ZEND_API void zend_register_stringl_constant(const char *name, uint name_len, ch
 }
 
 
-ZEND_API void zend_register_string_constant(const char *name, uint name_len, char *strval, int flags, int module_number TSRMLS_DC)
+ZEND_API void zend_register_string_constant(char *name, uint name_len, char *strval, int flags, int module_number TSRMLS_DC)
 {
 	zend_register_stringl_constant(name, name_len, strval, strlen(strval), flags, module_number TSRMLS_CC);
 }
 
 
-ZEND_API int zend_get_constant(const char *name, uint name_len, zval *result TSRMLS_DC)
+ZEND_API int zend_get_constant(char *name, uint name_len, zval *result TSRMLS_DC)
 {
 	zend_constant *c;
 	int retval = 1;
 	char *lookup_name;
+	char *colon;
 
+	if ((colon = memchr(name, ':', name_len)) && colon[1] == ':') {
+		/* class constant */
+		zend_class_entry **ce = NULL, *scope;
+		int class_name_len = colon-name;
+		int const_name_len = name_len - class_name_len - 2;
+		char *constant_name = colon+2;
+		zval **ret_constant;
+		char *class_name;
+
+		if (EG(in_execution)) {
+			scope = EG(scope);
+		} else {
+			scope = CG(active_class_entry);
+		}
+	
+		class_name = estrndup(name, class_name_len);
+
+		if (class_name_len == sizeof("self")-1 && strcmp(class_name, "self") == 0) {
+			if (scope) {
+				ce = &scope;
+			} else {
+				zend_error(E_ERROR, "Cannot access self:: when no class scope is active");
+				retval = 0;
+			}
+		} else if (class_name_len == sizeof("parent")-1 && strcmp(class_name, "parent") == 0) {
+			if (!scope) {   	 
+				zend_error(E_ERROR, "Cannot access parent:: when no class scope is active");
+			} else if (!scope->parent) { 	 
+				zend_error(E_ERROR, "Cannot access parent:: when current class scope has no parent"); 	 
+			} else {
+				ce = &scope->parent;
+			}
+		} else {
+			if (zend_lookup_class(class_name, class_name_len, &ce TSRMLS_CC) != SUCCESS) {
+				retval = 0;
+			}
+		}
+		efree(class_name);
+
+		if (retval && ce) {
+			if (zend_hash_find(&((*ce)->constants_table), constant_name, const_name_len+1, (void **) &ret_constant) != SUCCESS) {
+				retval = 0;
+			}
+		} else {
+			retval = 0;
+		}
+
+		if (retval) {
+			zval_update_constant(ret_constant, (void*)1 TSRMLS_CC);
+			*result = **ret_constant;
+			zval_copy_ctor(result);
+		}
+		
+		return retval;
+	}
+	
 	if (zend_hash_find(EG(zend_constants), name, name_len+1, (void **) &c) == FAILURE) {
-		lookup_name = zend_str_tolower_dup(name, name_len);
-
+		lookup_name = estrndup(name, name_len);
+		zend_str_tolower(lookup_name, name_len);
+		 
 		if (zend_hash_find(EG(zend_constants), lookup_name, name_len+1, (void **) &c)==SUCCESS) {
-			if (c->flags & CONST_CS) {
+			if ((c->flags & CONST_CS) && memcmp(c->name, name, name_len)!=0) {
 				retval=0;
 			}
 		} else {
-			static char haltoff[] = "__COMPILER_HALT_OFFSET__";
-
-			if (!EG(in_execution)) {
-				retval = 0;
-			} else if (name_len == sizeof("__COMPILER_HALT_OFFSET__")-1 &&
-			          !memcmp(name, "__COMPILER_HALT_OFFSET__", sizeof("__COMPILER_HALT_OFFSET__")-1)) {
-				char *cfilename, *haltname;
-				int len, clen;
-
-				cfilename = zend_get_executed_filename(TSRMLS_C);
-				clen = strlen(cfilename);
-				/* check for __COMPILER_HALT_OFFSET__ */
-				zend_mangle_property_name(&haltname, &len, haltoff,
-					sizeof("__COMPILER_HALT_OFFSET__") - 1, cfilename, clen, 0);
-				if (zend_hash_find(EG(zend_constants), haltname, len+1, (void **) &c) == SUCCESS) {
-					retval = 1;
-				} else {
-					retval=0;
-				}
-				pefree(haltname, 0);
-			} else {
-				retval=0;
-			}
+			retval=0;
 		}
 		efree(lookup_name);
 	}
@@ -265,148 +289,13 @@ ZEND_API int zend_get_constant(const char *name, uint name_len, zval *result TSR
 	if (retval) {
 		*result = c->value;
 		zval_copy_ctor(result);
-		Z_SET_REFCOUNT_P(result, 1);
-		Z_UNSET_ISREF_P(result);
+		result->refcount = 1;
+		result->is_ref = 0;
 	}
 
 	return retval;
 }
 
-ZEND_API int zend_get_constant_ex(const char *name, uint name_len, zval *result, zend_class_entry *scope, ulong flags TSRMLS_DC)
-{
-	zend_constant *c;
-	int retval = 1;
-	char *colon;
-	zend_class_entry *ce = NULL;
-	char *class_name;
-	zval **ret_constant;
-
-	/* Skip leading \\ */
-	if (name[0] == '\\') {
-		name += 1;
-		name_len -= 1;
-	}
-
-
-	if ((colon = zend_memrchr(name, ':', name_len)) &&
-	    colon > name && (*(colon - 1) == ':')) {
-		int class_name_len = colon - name - 1;
-		int const_name_len = name_len - class_name_len - 2;
-		char *constant_name = colon + 1;
-		char *lcname;
-
-		class_name = estrndup(name, class_name_len);
-		lcname = zend_str_tolower_dup(class_name, class_name_len);
-		if (!scope) {
-			if (EG(in_execution)) {
-				scope = EG(scope);
-			} else {
-				scope = CG(active_class_entry);
-			}
-		}
-
-		if (class_name_len == sizeof("self")-1 &&
-		    !memcmp(lcname, "self", sizeof("self")-1)) {
-			if (scope) {
-				ce = scope;
-			} else {
-				zend_error(E_ERROR, "Cannot access self:: when no class scope is active");
-				retval = 0;
-			}
-			efree(lcname);
-		} else if (class_name_len == sizeof("parent")-1 &&
-		           !memcmp(lcname, "parent", sizeof("parent")-1)) {
-			if (!scope) {
-				zend_error(E_ERROR, "Cannot access parent:: when no class scope is active");
-			} else if (!scope->parent) {
-				zend_error(E_ERROR, "Cannot access parent:: when current class scope has no parent");
-			} else {
-				ce = scope->parent;
-			}
-			efree(lcname);
-		} else if (class_name_len == sizeof("static")-1 &&
-		           !memcmp(lcname, "static", sizeof("static")-1)) {
-			if (EG(called_scope)) {
-				ce = EG(called_scope);
-			} else {
-				zend_error(E_ERROR, "Cannot access static:: when no class scope is active");
-			}
-			efree(lcname);
-		} else {
-			efree(lcname);
-			ce = zend_fetch_class(class_name, class_name_len, flags TSRMLS_CC);
-		}
-		if (retval && ce) {
-			if (zend_hash_find(&ce->constants_table, constant_name, const_name_len+1, (void **) &ret_constant) != SUCCESS) {
-				retval = 0;
-				if ((flags & ZEND_FETCH_CLASS_SILENT) == 0) {
-					zend_error(E_ERROR, "Undefined class constant '%s::%s'", class_name, constant_name);
-				}
-			}
-		} else if (!ce) {
-			retval = 0;
-		}
-		efree(class_name);
-		goto finish;
-	}
-
-	/* non-class constant */
-	if ((colon = zend_memrchr(name, '\\', name_len)) != NULL) {
-		/* compound constant name */
-		int prefix_len = colon - name;
-		int const_name_len = name_len - prefix_len - 1;
-		char *constant_name = colon + 1;
-		char *lcname;
-		int found_const = 0;
-
-		lcname = zend_str_tolower_dup(name, prefix_len);
-		/* Check for namespace constant */
-
-		/* Concatenate lowercase namespace name and constant name */
-		lcname = erealloc(lcname, prefix_len + 1 + const_name_len + 1);
-		lcname[prefix_len] = '\\';
-		memcpy(lcname + prefix_len + 1, constant_name, const_name_len + 1);
-
-		if (zend_hash_find(EG(zend_constants), lcname, prefix_len + 1 + const_name_len + 1, (void **) &c) == SUCCESS) {
-			found_const = 1;
-		} else {
-			/* try lowercase */
-			zend_str_tolower(lcname + prefix_len + 1, const_name_len);
-			if (zend_hash_find(EG(zend_constants), lcname, prefix_len + 1 + const_name_len + 1, (void **) &c) == SUCCESS) {
-				if ((c->flags & CONST_CS) == 0) {
-					found_const = 1;
-				}
-			}
-		}
-		efree(lcname);
-		if(found_const) {
-			*result = c->value;
-			zval_update_constant_ex(&result, (void*)1, NULL TSRMLS_CC);
-			zval_copy_ctor(result);
-			Z_SET_REFCOUNT_P(result, 1);
-			Z_UNSET_ISREF_P(result);
-			return 1;
-		}
-		/* name requires runtime resolution, need to check non-namespaced name */
-		if ((flags & IS_CONSTANT_UNQUALIFIED) != 0) {
-			name = constant_name;
-			name_len = const_name_len;
-			return zend_get_constant(name, name_len, result TSRMLS_CC);
-		}
-		retval = 0;
-finish:
-		if (retval) {
-			zval_update_constant_ex(ret_constant, (void*)1, ce TSRMLS_CC);
-			*result = **ret_constant;
-			zval_copy_ctor(result);
-			INIT_PZVAL(result);
-		}
-
-		return retval;
-	}
-
-	return zend_get_constant(name, name_len, result TSRMLS_CC);
-}
 
 ZEND_API int zend_register_constant(zend_constant *c TSRMLS_DC)
 {
@@ -420,22 +309,14 @@ ZEND_API int zend_register_constant(zend_constant *c TSRMLS_DC)
 
 	if (!(c->flags & CONST_CS)) {
 		/* keep in mind that c->name_len already contains the '\0' */
-		lowercase_name = estrndup(c->name, c->name_len-1);
-		zend_str_tolower(lowercase_name, c->name_len-1);
+		lowercase_name = estrndup(c->name, c->name_len);
+		zend_str_tolower(lowercase_name, c->name_len);
 		name = lowercase_name;
 	} else {
-		char *slash = strrchr(c->name, '\\');
-		if(slash) {
-			lowercase_name = estrndup(c->name, c->name_len-1);
-			zend_str_tolower(lowercase_name, slash-c->name);
-			name = lowercase_name;
-		} else {
-			name = c->name;
-		}
+		name = c->name;
 	}
 
-	if ((strncmp(name, "__COMPILER_HALT_OFFSET__", sizeof("__COMPILER_HALT_OFFSET__") - 1) == 0) ||
-			zend_hash_add(EG(zend_constants), name, c->name_len, (void *) c, sizeof(zend_constant), NULL)==FAILURE) {
+	if (zend_hash_add(EG(zend_constants), name, c->name_len, (void *) c, sizeof(zend_constant), NULL)==FAILURE) {
 		zend_error(E_NOTICE,"Constant %s already defined", name);
 		free(c->name);
 		if (!(c->flags & CONST_PERSISTENT)) {
