@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2014 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,10 +17,9 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: zend_hash.c,v 1.121.2.1 2006/01/04 23:53:04 andi Exp $ */
 
 #include "zend.h"
-#include "zend_globals.h"
 
 #define CONNECT_TO_BUCKET_DLLIST(element, list_head)		\
 	(element)->pNext = (list_head);							\
@@ -49,7 +48,7 @@
 #define HT_DESTROYED		2
 #define HT_CLEANING			3
 
-static void _zend_is_inconsistent(const HashTable *ht, const char *file, int line)
+static void _zend_is_inconsistent(HashTable *ht, char *file, int line)
 {
 	if (ht->inconsistent==HT_OK) {
 		return;
@@ -64,9 +63,6 @@ static void _zend_is_inconsistent(const HashTable *ht, const char *file, int lin
 		case HT_CLEANING:
 			zend_output_debug_string(1, "%s(%d) : ht=%p is being cleaned", file, line, ht);
 			break;
-		default:
-			zend_output_debug_string(1, "%s(%d) : ht=%p is inconsistent", file, line, ht);
-			break;
 	}
 	zend_bailout();
 }
@@ -98,7 +94,7 @@ static void _zend_is_inconsistent(const HashTable *ht, const char *file, int lin
 
 static int zend_hash_do_resize(HashTable *ht);
 
-ZEND_API ulong zend_hash_func(const char *arKey, uint nKeyLength)
+ZEND_API ulong zend_hash_func(char *arKey, uint nKeyLength)
 {
 	return zend_inline_hash_func(arKey, nKeyLength);
 }
@@ -106,14 +102,14 @@ ZEND_API ulong zend_hash_func(const char *arKey, uint nKeyLength)
 
 #define UPDATE_DATA(ht, p, pData, nDataSize)											\
 	if (nDataSize == sizeof(void*)) {													\
-		if ((p)->pData != &(p)->pDataPtr) {												\
-			pefree_rel((p)->pData, (ht)->persistent);									\
+		if (!(p)->pDataPtr) {															\
+			pefree_rel((p)->pData, (ht)->persistent);										\
 		}																				\
 		memcpy(&(p)->pDataPtr, pData, sizeof(void *));									\
 		(p)->pData = &(p)->pDataPtr;													\
 	} else {																			\
-		if ((p)->pData == &(p)->pDataPtr) {												\
-			(p)->pData = (void *) pemalloc_rel(nDataSize, (ht)->persistent);			\
+		if ((p)->pDataPtr) {															\
+			(p)->pData = (void *) pemalloc_rel(nDataSize, (ht)->persistent);				\
 			(p)->pDataPtr=NULL;															\
 		} else {																		\
 			(p)->pData = (void *) perealloc_rel((p)->pData, nDataSize, (ht)->persistent);	\
@@ -127,43 +123,32 @@ ZEND_API ulong zend_hash_func(const char *arKey, uint nKeyLength)
 		memcpy(&(p)->pDataPtr, pData, sizeof(void *));					\
 		(p)->pData = &(p)->pDataPtr;									\
 	} else {															\
-		(p)->pData = (void *) pemalloc_rel(nDataSize, (ht)->persistent);\
+		(p)->pData = (void *) pemalloc_rel(nDataSize, (ht)->persistent);	\
 		if (!(p)->pData) {												\
-			pefree_rel(p, (ht)->persistent);							\
+			pefree_rel(p, (ht)->persistent);								\
 			return FAILURE;												\
 		}																\
 		memcpy((p)->pData, pData, nDataSize);							\
 		(p)->pDataPtr=NULL;												\
 	}
 
-#define CHECK_INIT(ht) do {												\
-	if (UNEXPECTED((ht)->nTableMask == 0)) {								\
-		(ht)->arBuckets = (Bucket **) pecalloc((ht)->nTableSize, sizeof(Bucket *), (ht)->persistent);	\
-		(ht)->nTableMask = (ht)->nTableSize - 1;						\
-	}																	\
-} while (0)
- 
-static const Bucket *uninitialized_bucket = NULL;
+
 
 ZEND_API int _zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction, dtor_func_t pDestructor, zend_bool persistent ZEND_FILE_LINE_DC)
 {
 	uint i = 3;
+	Bucket **tmp;
 
 	SET_INCONSISTENT(HT_OK);
 
-	if (nSize >= 0x80000000) {
-		/* prevent overflow */
-		ht->nTableSize = 0x80000000;
-	} else {
-		while ((1U << i) < nSize) {
-			i++;
-		}
-		ht->nTableSize = 1 << i;
+	while ((1U << i) < nSize) {
+		i++;
 	}
 
-	ht->nTableMask = 0;	/* 0 means that ht->arBuckets is uninitialized */
+	ht->nTableSize = 1 << i;
+	ht->nTableMask = ht->nTableSize - 1;
 	ht->pDestructor = pDestructor;
-	ht->arBuckets = (Bucket**)&uninitialized_bucket;
+	ht->arBuckets = NULL;
 	ht->pListHead = NULL;
 	ht->pListTail = NULL;
 	ht->nNumOfElements = 0;
@@ -172,6 +157,21 @@ ZEND_API int _zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunctio
 	ht->persistent = persistent;
 	ht->nApplyCount = 0;
 	ht->bApplyProtection = 1;
+	
+	/* Uses ecalloc() so that Bucket* == NULL */
+	if (persistent) {
+		tmp = (Bucket **) calloc(ht->nTableSize, sizeof(Bucket *));
+		if (!tmp) {
+			return FAILURE;
+		}
+		ht->arBuckets = tmp;
+	} else {
+		tmp = (Bucket **) ecalloc_rel(ht->nTableSize, sizeof(Bucket *));
+		if (tmp) {
+			ht->arBuckets = tmp;
+		}
+	}
+	
 	return SUCCESS;
 }
 
@@ -192,14 +192,11 @@ ZEND_API void zend_hash_set_apply_protection(HashTable *ht, zend_bool bApplyProt
 
 
 
-ZEND_API int _zend_hash_add_or_update(HashTable *ht, const char *arKey, uint nKeyLength, void *pData, uint nDataSize, void **pDest, int flag ZEND_FILE_LINE_DC)
+ZEND_API int _zend_hash_add_or_update(HashTable *ht, char *arKey, uint nKeyLength, void *pData, uint nDataSize, void **pDest, int flag ZEND_FILE_LINE_DC)
 {
 	ulong h;
 	uint nIndex;
 	Bucket *p;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	IS_CONSISTENT(ht);
 
@@ -210,15 +207,13 @@ ZEND_API int _zend_hash_add_or_update(HashTable *ht, const char *arKey, uint nKe
 		return FAILURE;
 	}
 
-	CHECK_INIT(ht);
-
 	h = zend_inline_hash_func(arKey, nKeyLength);
 	nIndex = h & ht->nTableMask;
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				if (flag & HASH_ADD) {
 					return FAILURE;
 				}
@@ -239,24 +234,16 @@ ZEND_API int _zend_hash_add_or_update(HashTable *ht, const char *arKey, uint nKe
 				}
 				HANDLE_UNBLOCK_INTERRUPTIONS();
 				return SUCCESS;
+			}
 		}
 		p = p->pNext;
 	}
 	
-	if (IS_INTERNED(arKey)) {
-		p = (Bucket *) pemalloc(sizeof(Bucket), ht->persistent);
-		if (!p) {
-			return FAILURE;
-		}
-		p->arKey = arKey;
-	} else {
-		p = (Bucket *) pemalloc(sizeof(Bucket) + nKeyLength, ht->persistent);
-		if (!p) {
-			return FAILURE;
-		}
-		p->arKey = (const char*)(p + 1);
-		memcpy((char*)p->arKey, arKey, nKeyLength);
+	p = (Bucket *) pemalloc(sizeof(Bucket) - 1 + nKeyLength, ht->persistent);
+	if (!p) {
+		return FAILURE;
 	}
+	memcpy(p->arKey, arKey, nKeyLength);
 	p->nKeyLength = nKeyLength;
 	INIT_DATA(ht, p, pData, nDataSize);
 	p->h = h;
@@ -275,13 +262,10 @@ ZEND_API int _zend_hash_add_or_update(HashTable *ht, const char *arKey, uint nKe
 	return SUCCESS;
 }
 
-ZEND_API int _zend_hash_quick_add_or_update(HashTable *ht, const char *arKey, uint nKeyLength, ulong h, void *pData, uint nDataSize, void **pDest, int flag ZEND_FILE_LINE_DC)
+ZEND_API int _zend_hash_quick_add_or_update(HashTable *ht, char *arKey, uint nKeyLength, ulong h, void *pData, uint nDataSize, void **pDest, int flag ZEND_FILE_LINE_DC)
 {
 	uint nIndex;
 	Bucket *p;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	IS_CONSISTENT(ht);
 
@@ -289,13 +273,12 @@ ZEND_API int _zend_hash_quick_add_or_update(HashTable *ht, const char *arKey, ui
 		return zend_hash_index_update(ht, h, pData, nDataSize, pDest);
 	}
 
-	CHECK_INIT(ht);
 	nIndex = h & ht->nTableMask;
 	
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				if (flag & HASH_ADD) {
 					return FAILURE;
 				}
@@ -316,25 +299,17 @@ ZEND_API int _zend_hash_quick_add_or_update(HashTable *ht, const char *arKey, ui
 				}
 				HANDLE_UNBLOCK_INTERRUPTIONS();
 				return SUCCESS;
+			}
 		}
 		p = p->pNext;
 	}
 	
-	if (IS_INTERNED(arKey)) {
-		p = (Bucket *) pemalloc(sizeof(Bucket), ht->persistent);
-		if (!p) {
-			return FAILURE;
-		}
-		p->arKey = arKey;
-	} else {
-		p = (Bucket *) pemalloc(sizeof(Bucket) + nKeyLength, ht->persistent);
-		if (!p) {
-			return FAILURE;
-		}
-		p->arKey = (const char*)(p + 1);
-		memcpy((char*)p->arKey, arKey, nKeyLength);
+	p = (Bucket *) pemalloc(sizeof(Bucket) - 1 + nKeyLength, ht->persistent);
+	if (!p) {
+		return FAILURE;
 	}
 
+	memcpy(p->arKey, arKey, nKeyLength);
 	p->nKeyLength = nKeyLength;
 	INIT_DATA(ht, p, pData, nDataSize);
 	p->h = h;
@@ -356,7 +331,7 @@ ZEND_API int _zend_hash_quick_add_or_update(HashTable *ht, const char *arKey, ui
 }
 
 
-ZEND_API int zend_hash_add_empty_element(HashTable *ht, const char *arKey, uint nKeyLength)
+ZEND_API int zend_hash_add_empty_element(HashTable *ht, char *arKey, uint nKeyLength)
 {
 	void *dummy = (void *) 1;
 
@@ -368,12 +343,8 @@ ZEND_API int _zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 {
 	uint nIndex;
 	Bucket *p;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	IS_CONSISTENT(ht);
-	CHECK_INIT(ht);
 
 	if (flag & HASH_NEXT_INSERT) {
 		h = ht->nNextFreeElement;
@@ -400,7 +371,7 @@ ZEND_API int _zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 			UPDATE_DATA(ht, p, pData, nDataSize);
 			HANDLE_UNBLOCK_INTERRUPTIONS();
 			if ((long)h >= (long)ht->nNextFreeElement) {
-				ht->nNextFreeElement = h < LONG_MAX ? h + 1 : LONG_MAX;
+				ht->nNextFreeElement = h + 1;
 			}
 			if (pDest) {
 				*pDest = p->pData;
@@ -409,11 +380,10 @@ ZEND_API int _zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 		}
 		p = p->pNext;
 	}
-	p = (Bucket *) pemalloc_rel(sizeof(Bucket), ht->persistent);
+	p = (Bucket *) pemalloc_rel(sizeof(Bucket) - 1, ht->persistent);
 	if (!p) {
 		return FAILURE;
 	}
-	p->arKey = NULL;
 	p->nKeyLength = 0; /* Numeric indices are marked by making the nKeyLength == 0 */
 	p->h = h;
 	INIT_DATA(ht, p, pData, nDataSize);
@@ -429,7 +399,7 @@ ZEND_API int _zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 	HANDLE_UNBLOCK_INTERRUPTIONS();
 
 	if ((long)h >= (long)ht->nNextFreeElement) {
-		ht->nNextFreeElement = h < LONG_MAX ? h + 1 : LONG_MAX;
+		ht->nNextFreeElement = h + 1;
 	}
 	ht->nNumOfElements++;
 	ZEND_HASH_IF_FULL_DO_RESIZE(ht);
@@ -440,9 +410,6 @@ ZEND_API int _zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 static int zend_hash_do_resize(HashTable *ht)
 {
 	Bucket **t;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	IS_CONSISTENT(ht);
 
@@ -468,9 +435,6 @@ ZEND_API int zend_hash_rehash(HashTable *ht)
 	uint nIndex;
 
 	IS_CONSISTENT(ht);
-	if (UNEXPECTED(ht->nNumOfElements == 0)) {
-		return SUCCESS;
-	}
 
 	memset(ht->arBuckets, 0, ht->nTableSize * sizeof(Bucket *));
 	p = ht->pListHead;
@@ -483,13 +447,10 @@ ZEND_API int zend_hash_rehash(HashTable *ht)
 	return SUCCESS;
 }
 
-ZEND_API int zend_hash_del_key_or_index(HashTable *ht, const char *arKey, uint nKeyLength, ulong h, int flag)
+ZEND_API int zend_hash_del_key_or_index(HashTable *ht, char *arKey, uint nKeyLength, ulong h, int flag)
 {
 	uint nIndex;
 	Bucket *p;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	IS_CONSISTENT(ht);
 
@@ -500,10 +461,8 @@ ZEND_API int zend_hash_del_key_or_index(HashTable *ht, const char *arKey, uint n
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if ((p->h == h) 
-			 && (p->nKeyLength == nKeyLength)
-			 && ((p->nKeyLength == 0) /* Numeric index (short circuits the memcmp() check) */
-				 || !memcmp(p->arKey, arKey, nKeyLength))) { /* String index */
+		if ((p->h == h) && ((p->nKeyLength == 0) || /* Numeric index */
+			((p->nKeyLength == nKeyLength) && (!memcmp(p->arKey, arKey, nKeyLength))))) {
 			HANDLE_BLOCK_INTERRUPTIONS();
 			if (p == ht->arBuckets[nIndex]) {
 				ht->arBuckets[nIndex] = p->pNext;
@@ -530,7 +489,7 @@ ZEND_API int zend_hash_del_key_or_index(HashTable *ht, const char *arKey, uint n
 			if (ht->pDestructor) {
 				ht->pDestructor(p->pData);
 			}
-			if (p->pData != &p->pDataPtr) {
+			if (!p->pDataPtr) {
 				pefree(p->pData, ht->persistent);
 			}
 			pefree(p, ht->persistent);
@@ -559,14 +518,12 @@ ZEND_API void zend_hash_destroy(HashTable *ht)
 		if (ht->pDestructor) {
 			ht->pDestructor(q->pData);
 		}
-		if (q->pData != &q->pDataPtr) {
+		if (!q->pDataPtr && q->pData) {
 			pefree(q->pData, ht->persistent);
 		}
 		pefree(q, ht->persistent);
 	}
-	if (ht->nTableMask) {
-		pefree(ht->arBuckets, ht->persistent);
-	}
+	pefree(ht->arBuckets, ht->persistent);
 
 	SET_INCONSISTENT(HT_DESTROYED);
 }
@@ -578,28 +535,28 @@ ZEND_API void zend_hash_clean(HashTable *ht)
 
 	IS_CONSISTENT(ht);
 
+	SET_INCONSISTENT(HT_CLEANING);
+
 	p = ht->pListHead;
-
-	if (ht->nTableMask) {
-		memset(ht->arBuckets, 0, ht->nTableSize*sizeof(Bucket *));
-	}
-	ht->pListHead = NULL;
-	ht->pListTail = NULL;
-	ht->nNumOfElements = 0;
-	ht->nNextFreeElement = 0;
-	ht->pInternalPointer = NULL;
-
 	while (p != NULL) {
 		q = p;
 		p = p->pListNext;
 		if (ht->pDestructor) {
 			ht->pDestructor(q->pData);
 		}
-		if (q->pData != &q->pDataPtr) {
+		if (!q->pDataPtr && q->pData) {
 			pefree(q->pData, ht->persistent);
 		}
 		pefree(q, ht->persistent);
 	}
+	memset(ht->arBuckets, 0, ht->nTableSize*sizeof(Bucket *));
+	ht->pListHead = NULL;
+	ht->pListTail = NULL;
+	ht->nNumOfElements = 0;
+	ht->nNextFreeElement = 0;
+	ht->pInternalPointer = NULL;
+
+	SET_INCONSISTENT(HT_OK);
 }
 
 /* This function is used by the various apply() functions.
@@ -610,11 +567,17 @@ ZEND_API void zend_hash_clean(HashTable *ht)
 static Bucket *zend_hash_apply_deleter(HashTable *ht, Bucket *p)
 {
 	Bucket *retval;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
 
 	HANDLE_BLOCK_INTERRUPTIONS();
+
+	if (ht->pDestructor) {
+		ht->pDestructor(p->pData);
+	}
+	if (!p->pDataPtr) {
+		pefree(p->pData, ht->persistent);
+	}
+	retval = p->pListNext;
+
 	if (p->pLast) {
 		p->pLast->pNext = p->pNext;
 	} else {
@@ -643,17 +606,9 @@ static Bucket *zend_hash_apply_deleter(HashTable *ht, Bucket *p)
 	if (ht->pInternalPointer == p) {
 		ht->pInternalPointer = p->pListNext;
 	}
-	ht->nNumOfElements--;
-	HANDLE_UNBLOCK_INTERRUPTIONS();
-
-	if (ht->pDestructor) {
-		ht->pDestructor(p->pData);
-	}
-	if (p->pData != &p->pDataPtr) {
-		pefree(p->pData, ht->persistent);
-	}
-	retval = p->pListNext;
 	pefree(p, ht->persistent);
+	HANDLE_UNBLOCK_INTERRUPTIONS();
+	ht->nNumOfElements--;
 
 	return retval;
 }
@@ -669,9 +624,7 @@ ZEND_API void zend_hash_graceful_destroy(HashTable *ht)
 	while (p != NULL) {
 		p = zend_hash_apply_deleter(ht, p);
 	}
-	if (ht->nTableMask) {
-		pefree(ht->arBuckets, ht->persistent);
-	}
+	pefree(ht->arBuckets, ht->persistent);
 
 	SET_INCONSISTENT(HT_DESTROYED);
 }
@@ -688,22 +641,17 @@ ZEND_API void zend_hash_graceful_reverse_destroy(HashTable *ht)
 		p = ht->pListTail;
 	}
 
-	if (ht->nTableMask) {
-		pefree(ht->arBuckets, ht->persistent);
-	}
+	pefree(ht->arBuckets, ht->persistent);
 
 	SET_INCONSISTENT(HT_DESTROYED);
 }
 
-/* This is used to recurse elements and selectively delete certain entries 
- * from a hashtable. apply_func() receives the data and decides if the entry 
- * should be deleted or recursion should be stopped. The following three 
- * return codes are possible:
- * ZEND_HASH_APPLY_KEEP   - continue
- * ZEND_HASH_APPLY_STOP   - stop iteration
- * ZEND_HASH_APPLY_REMOVE - delete the element, combineable with the former
+/* This is used to selectively delete certain entries from a hashtable.
+ * destruct() receives the data and decides if the entry should be deleted 
+ * or not
  */
 
+
 ZEND_API void zend_hash_apply(HashTable *ht, apply_func_t apply_func TSRMLS_DC)
 {
 	Bucket *p;
@@ -713,16 +661,11 @@ ZEND_API void zend_hash_apply(HashTable *ht, apply_func_t apply_func TSRMLS_DC)
 	HASH_PROTECT_RECURSION(ht);
 	p = ht->pListHead;
 	while (p != NULL) {
-		int result = apply_func(p->pData TSRMLS_CC);
-		
-		if (result & ZEND_HASH_APPLY_REMOVE) {
+		if (apply_func(p->pData TSRMLS_CC)) {
 			p = zend_hash_apply_deleter(ht, p);
 		} else {
 			p = p->pListNext;
 		}
-		if (result & ZEND_HASH_APPLY_STOP) {
-			break;
-		}
 	}
 	HASH_UNPROTECT_RECURSION(ht);
 }
@@ -737,22 +680,17 @@ ZEND_API void zend_hash_apply_with_argument(HashTable *ht, apply_func_arg_t appl
 	HASH_PROTECT_RECURSION(ht);
 	p = ht->pListHead;
 	while (p != NULL) {
-		int result = apply_func(p->pData, argument TSRMLS_CC);
-		
-		if (result & ZEND_HASH_APPLY_REMOVE) {
+		if (apply_func(p->pData, argument TSRMLS_CC)) {
 			p = zend_hash_apply_deleter(ht, p);
 		} else {
 			p = p->pListNext;
 		}
-		if (result & ZEND_HASH_APPLY_STOP) {
-			break;
-		}
 	}
 	HASH_UNPROTECT_RECURSION(ht);
 }
 
 
-ZEND_API void zend_hash_apply_with_arguments(HashTable *ht TSRMLS_DC, apply_func_args_t apply_func, int num_args, ...)
+ZEND_API void zend_hash_apply_with_arguments(HashTable *ht, apply_func_args_t destruct, int num_args, ...)
 {
 	Bucket *p;
 	va_list args;
@@ -764,22 +702,15 @@ ZEND_API void zend_hash_apply_with_arguments(HashTable *ht TSRMLS_DC, apply_func
 
 	p = ht->pListHead;
 	while (p != NULL) {
-		int result;
 		va_start(args, num_args);
 		hash_key.arKey = p->arKey;
 		hash_key.nKeyLength = p->nKeyLength;
 		hash_key.h = p->h;
-		result = apply_func(p->pData TSRMLS_CC, num_args, args, &hash_key);
-
-		if (result & ZEND_HASH_APPLY_REMOVE) {
+		if (destruct(p->pData, num_args, args, &hash_key)) {
 			p = zend_hash_apply_deleter(ht, p);
 		} else {
 			p = p->pListNext;
 		}
-		if (result & ZEND_HASH_APPLY_STOP) {
-			va_end(args);
-			break;
-		}
 		va_end(args);
 	}
 
@@ -801,7 +732,11 @@ ZEND_API void zend_hash_reverse_apply(HashTable *ht, apply_func_t apply_func TSR
 		q = p;
 		p = p->pListLast;
 		if (result & ZEND_HASH_APPLY_REMOVE) {
-			zend_hash_apply_deleter(ht, q);
+			if (q->nKeyLength>0) {
+				zend_hash_del(ht, q->arKey, q->nKeyLength);
+			} else {
+				zend_hash_index_del(ht, q->h);
+			}
 		}
 		if (result & ZEND_HASH_APPLY_STOP) {
 			break;
@@ -815,19 +750,14 @@ ZEND_API void zend_hash_copy(HashTable *target, HashTable *source, copy_ctor_fun
 {
 	Bucket *p;
 	void *new_entry;
-	zend_bool setTargetPointer;
 
 	IS_CONSISTENT(source);
 	IS_CONSISTENT(target);
 
-	setTargetPointer = !target->pInternalPointer;
 	p = source->pListHead;
 	while (p) {
-		if (setTargetPointer && source->pInternalPointer == p) {
-			target->pInternalPointer = NULL;
-		}
 		if (p->nKeyLength) {
-			zend_hash_quick_update(target, p->arKey, p->nKeyLength, p->h, p->pData, size, &new_entry);
+			zend_hash_update(target, p->arKey, p->nKeyLength, p->pData, size, &new_entry);
 		} else {
 			zend_hash_index_update(target, p->h, p->pData, size, &new_entry);
 		}
@@ -836,9 +766,7 @@ ZEND_API void zend_hash_copy(HashTable *target, HashTable *source, copy_ctor_fun
 		}
 		p = p->pListNext;
 	}
-	if (!target->pInternalPointer) {
-		target->pInternalPointer = target->pListHead;
-	}
+	target->pInternalPointer = target->pListHead;
 }
 
 
@@ -854,7 +782,7 @@ ZEND_API void _zend_hash_merge(HashTable *target, HashTable *source, copy_ctor_f
 	p = source->pListHead;
 	while (p) {
 		if (p->nKeyLength>0) {
-			if (_zend_hash_quick_add_or_update(target, p->arKey, p->nKeyLength, p->h, p->pData, size, &t, mode ZEND_FILE_LINE_RELAY_CC)==SUCCESS && pCopyConstructor) {
+			if (_zend_hash_add_or_update(target, p->arKey, p->nKeyLength, p->pData, size, &t, mode ZEND_FILE_LINE_RELAY_CC)==SUCCESS && pCopyConstructor) {
 				pCopyConstructor(t);
 			}
 		} else {
@@ -900,7 +828,7 @@ ZEND_API void zend_hash_merge_ex(HashTable *target, HashTable *source, copy_ctor
 }
 
 
-ZEND_API ulong zend_get_hash_value(const char *arKey, uint nKeyLength)
+ZEND_API ulong zend_get_hash_value(char *arKey, uint nKeyLength)
 {
 	return zend_inline_hash_func(arKey, nKeyLength);
 }
@@ -910,7 +838,7 @@ ZEND_API ulong zend_get_hash_value(const char *arKey, uint nKeyLength)
  * data is returned in pData. The reason is that there's no reason
  * someone using the hash table might not want to have NULL data
  */
-ZEND_API int zend_hash_find(const HashTable *ht, const char *arKey, uint nKeyLength, void **pData)
+ZEND_API int zend_hash_find(HashTable *ht, char *arKey, uint nKeyLength, void **pData)
 {
 	ulong h;
 	uint nIndex;
@@ -923,10 +851,11 @@ ZEND_API int zend_hash_find(const HashTable *ht, const char *arKey, uint nKeyLen
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				*pData = p->pData;
 				return SUCCESS;
+			}
 		}
 		p = p->pNext;
 	}
@@ -934,7 +863,7 @@ ZEND_API int zend_hash_find(const HashTable *ht, const char *arKey, uint nKeyLen
 }
 
 
-ZEND_API int zend_hash_quick_find(const HashTable *ht, const char *arKey, uint nKeyLength, ulong h, void **pData)
+ZEND_API int zend_hash_quick_find(HashTable *ht, char *arKey, uint nKeyLength, ulong h, void **pData)
 {
 	uint nIndex;
 	Bucket *p;
@@ -949,10 +878,11 @@ ZEND_API int zend_hash_quick_find(const HashTable *ht, const char *arKey, uint n
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				*pData = p->pData;
 				return SUCCESS;
+			}
 		}
 		p = p->pNext;
 	}
@@ -960,7 +890,7 @@ ZEND_API int zend_hash_quick_find(const HashTable *ht, const char *arKey, uint n
 }
 
 
-ZEND_API int zend_hash_exists(const HashTable *ht, const char *arKey, uint nKeyLength)
+ZEND_API int zend_hash_exists(HashTable *ht, char *arKey, uint nKeyLength)
 {
 	ulong h;
 	uint nIndex;
@@ -973,9 +903,10 @@ ZEND_API int zend_hash_exists(const HashTable *ht, const char *arKey, uint nKeyL
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				return 1;
+			}
 		}
 		p = p->pNext;
 	}
@@ -983,7 +914,7 @@ ZEND_API int zend_hash_exists(const HashTable *ht, const char *arKey, uint nKeyL
 }
 
 
-ZEND_API int zend_hash_quick_exists(const HashTable *ht, const char *arKey, uint nKeyLength, ulong h)
+ZEND_API int zend_hash_quick_exists(HashTable *ht, char *arKey, uint nKeyLength, ulong h)
 {
 	uint nIndex;
 	Bucket *p;
@@ -998,9 +929,10 @@ ZEND_API int zend_hash_quick_exists(const HashTable *ht, const char *arKey, uint
 
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
-		if (p->arKey == arKey ||
-			((p->h == h) && (p->nKeyLength == nKeyLength) && !memcmp(p->arKey, arKey, nKeyLength))) {
+		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
+			if (!memcmp(p->arKey, arKey, nKeyLength)) {
 				return 1;
+			}
 		}
 		p = p->pNext;
 	}
@@ -1009,7 +941,7 @@ ZEND_API int zend_hash_quick_exists(const HashTable *ht, const char *arKey, uint
 }
 
 
-ZEND_API int zend_hash_index_find(const HashTable *ht, ulong h, void **pData)
+ZEND_API int zend_hash_index_find(HashTable *ht, ulong h, void **pData)
 {
 	uint nIndex;
 	Bucket *p;
@@ -1030,7 +962,7 @@ ZEND_API int zend_hash_index_find(const HashTable *ht, ulong h, void **pData)
 }
 
 
-ZEND_API int zend_hash_index_exists(const HashTable *ht, ulong h)
+ZEND_API int zend_hash_index_exists(HashTable *ht, ulong h)
 {
 	uint nIndex;
 	Bucket *p;
@@ -1050,7 +982,7 @@ ZEND_API int zend_hash_index_exists(const HashTable *ht, ulong h)
 }
 
 
-ZEND_API int zend_hash_num_elements(const HashTable *ht)
+ZEND_API int zend_hash_num_elements(HashTable *ht)
 {
 	IS_CONSISTENT(ht);
 
@@ -1058,39 +990,6 @@ ZEND_API int zend_hash_num_elements(const HashTable *ht)
 }
 
 
-ZEND_API int zend_hash_get_pointer(const HashTable *ht, HashPointer *ptr)
-{
-	ptr->pos = ht->pInternalPointer;
-	if (ht->pInternalPointer) {
-		ptr->h = ht->pInternalPointer->h;
-		return 1;
-	} else {
-		ptr->h = 0;
-		return 0;
-	}
-}
-
-ZEND_API int zend_hash_set_pointer(HashTable *ht, const HashPointer *ptr)
-{
-	if (ptr->pos == NULL) {
-		ht->pInternalPointer = NULL;
-	} else if (ht->pInternalPointer != ptr->pos) {
-		Bucket *p;
-
-		IS_CONSISTENT(ht);
-		p = ht->arBuckets[ptr->h & ht->nTableMask];
-		while (p != NULL) {
-			if (p == ptr->pos) {
-				ht->pInternalPointer = p;
-				return 1;
-			}
-			p = p->pNext;
-		}
-		return 0;
-	}
-	return 1;
-}
-
 ZEND_API void zend_hash_internal_pointer_reset_ex(HashTable *ht, HashPosition *pos)
 {
 	IS_CONSISTENT(ht);
@@ -1144,7 +1043,7 @@ ZEND_API int zend_hash_move_backwards_ex(HashTable *ht, HashPosition *pos)
 
 
 /* This function should be made binary safe  */
-ZEND_API int zend_hash_get_current_key_ex(const HashTable *ht, char **str_index, uint *str_length, ulong *num_index, zend_bool duplicate, HashPosition *pos)
+ZEND_API int zend_hash_get_current_key_ex(HashTable *ht, char **str_index, uint *str_length, ulong *num_index, zend_bool duplicate, HashPosition *pos)
 {
 	Bucket *p;
 
@@ -1157,7 +1056,7 @@ ZEND_API int zend_hash_get_current_key_ex(const HashTable *ht, char **str_index,
 			if (duplicate) {
 				*str_index = estrndup(p->arKey, p->nKeyLength - 1);
 			} else {
-				*str_index = (char*)p->arKey;
+				*str_index = p->arKey;
 			}
 			if (str_length) {
 				*str_length = p->nKeyLength;
@@ -1207,16 +1106,12 @@ ZEND_API int zend_hash_get_current_data_ex(HashTable *ht, void **pData, HashPosi
 	}
 }
 
-/* This function changes key of current element without changing elements'
+/* This function changes key of currevt element without changing elements'
  * order. If element with target key already exists, it will be deleted first.
  */
-ZEND_API int zend_hash_update_current_key_ex(HashTable *ht, int key_type, const char *str_index, uint str_length, ulong num_index, int mode, HashPosition *pos)
+ZEND_API int zend_hash_update_current_key_ex(HashTable *ht, int key_type, char *str_index, uint str_length, ulong num_index, HashPosition *pos)
 {
-	Bucket *p, *q;
-	ulong h;
-#ifdef ZEND_SIGNALS
-	TSRMLS_FETCH();
-#endif
+	Bucket *p;
 
 	p = pos ? (*pos) : ht->pInternalPointer;
 
@@ -1228,143 +1123,30 @@ ZEND_API int zend_hash_update_current_key_ex(HashTable *ht, int key_type, const
 			if (!p->nKeyLength && p->h == num_index) {
 				return SUCCESS;
 			}
-
-			q = ht->arBuckets[num_index & ht->nTableMask];
-			while (q != NULL) {
-				if (!q->nKeyLength && q->h == num_index) {
-					break;
-				}
-				q = q->pNext;
-			}
+			zend_hash_index_del(ht, num_index);
 		} else if (key_type == HASH_KEY_IS_STRING) {
-			if (IS_INTERNED(str_index)) {
-				h = INTERNED_HASH(str_index);
-			} else {
-				h = zend_inline_hash_func(str_index, str_length);
-			}
-
-			if (p->arKey == str_index ||
-			    (p->nKeyLength == str_length &&
-			     p->h == h &&
-			     memcmp(p->arKey, str_index, str_length) == 0)) {
+			if (p->nKeyLength == str_length &&
+			    memcmp(p->arKey, str_index, str_length) == 0) {
 				return SUCCESS;
 			}
-
-			q = ht->arBuckets[h & ht->nTableMask];
-
-			while (q != NULL) {
-				if (q->arKey == str_index ||
-				    (q->h == h && q->nKeyLength == str_length &&
-				     memcmp(q->arKey, str_index, str_length) == 0)) {
-					break;
-				}
-				q = q->pNext;
-			}
+			zend_hash_del(ht, str_index, str_length);
 		} else {
 			return FAILURE;
 		}
 
 		HANDLE_BLOCK_INTERRUPTIONS();
 
-		if (q) {
-			if (mode != HASH_UPDATE_KEY_ANYWAY) {
-				Bucket *r = p->pListLast;
-				int found = HASH_UPDATE_KEY_IF_BEFORE;
-
-				while (r) {
-					if (r == q) {
-						found = HASH_UPDATE_KEY_IF_AFTER;
-						break;
-					}
-					r = r->pListLast;
-				}
-				if (mode & found) {
-					/* delete current bucket */
-					if (p == ht->arBuckets[p->h & ht->nTableMask]) {
-						ht->arBuckets[p->h & ht->nTableMask] = p->pNext;
-					} else {
-						p->pLast->pNext = p->pNext;
-					}
-					if (p->pNext) {
-						p->pNext->pLast = p->pLast;
-					}
-					if (p->pListLast != NULL) {
-						p->pListLast->pListNext = p->pListNext;
-					} else {
-						/* Deleting the head of the list */
-						ht->pListHead = p->pListNext;
-					}
-					if (p->pListNext != NULL) {
-						p->pListNext->pListLast = p->pListLast;
-					} else {
-						ht->pListTail = p->pListLast;
-					}
-					if (ht->pInternalPointer == p) {
-						ht->pInternalPointer = p->pListNext;
-					}
-					if (ht->pDestructor) {
-						ht->pDestructor(p->pData);
-					}
-					if (p->pData != &p->pDataPtr) {
-						pefree(p->pData, ht->persistent);
-					}
-					pefree(p, ht->persistent);
-					ht->nNumOfElements--;
-					HANDLE_UNBLOCK_INTERRUPTIONS();
-					return FAILURE;
-				}
-			}
-			/* delete another bucket with the same key */
-			if (q == ht->arBuckets[q->h & ht->nTableMask]) {
-				ht->arBuckets[q->h & ht->nTableMask] = q->pNext;
-			} else {
-				q->pLast->pNext = q->pNext;
-			}
-			if (q->pNext) {
-				q->pNext->pLast = q->pLast;
-			}
-			if (q->pListLast != NULL) {
-				q->pListLast->pListNext = q->pListNext;
-			} else {
-				/* Deleting the head of the list */
-				ht->pListHead = q->pListNext;
-			}
-			if (q->pListNext != NULL) {
-				q->pListNext->pListLast = q->pListLast;
-			} else {
-				ht->pListTail = q->pListLast;
-			}
-			if (ht->pInternalPointer == q) {
-				ht->pInternalPointer = q->pListNext;
-			}
-			if (ht->pDestructor) {
-				ht->pDestructor(q->pData);
-			}
-			if (q->pData != &q->pDataPtr) {
-				pefree(q->pData, ht->persistent);
-			}
-			pefree(q, ht->persistent);
-			ht->nNumOfElements--;
-		}
-
 		if (p->pNext) {
 			p->pNext->pLast = p->pLast;
 		}
 		if (p->pLast) {
 			p->pLast->pNext = p->pNext;
-		} else {
+		} else{
 			ht->arBuckets[p->h & ht->nTableMask] = p->pNext;
 		}
 
-		if ((IS_INTERNED(p->arKey) != IS_INTERNED(str_index)) ||
-		    (!IS_INTERNED(p->arKey) && p->nKeyLength != str_length)) {
-			Bucket *q;
-
-			if (IS_INTERNED(str_index)) {
-				q = (Bucket *) pemalloc(sizeof(Bucket), ht->persistent);
-			} else {
-				q = (Bucket *) pemalloc(sizeof(Bucket) + str_length, ht->persistent);
-			}
+		if (p->nKeyLength != str_length) {
+			Bucket *q = (Bucket *) pemalloc(sizeof(Bucket) - 1 + str_length, ht->persistent);
 
 			q->nKeyLength = str_length;
 			if (p->pData == &p->pDataPtr) {
@@ -1398,14 +1180,8 @@ ZEND_API int zend_hash_update_current_key_ex(HashTable *ht, int key_type, const
 		if (key_type == HASH_KEY_IS_LONG) {
 			p->h = num_index;
 		} else {
-			p->h = h;
-			p->nKeyLength = str_length;
-			if (IS_INTERNED(str_index)) {
-				p->arKey = str_index;
-			} else {
-				p->arKey = (const char*)(p+1);
-				memcpy((char*)p->arKey, str_index, str_length);
-			}
+			memcpy(p->arKey, str_index, str_length);
+			p->h = zend_inline_hash_func(str_index, str_length);
 		}
 
 		CONNECT_TO_BUCKET_DLLIST(p, ht->arBuckets[p->h & ht->nTableMask]);
@@ -1483,7 +1259,7 @@ ZEND_API int zend_hash_sort(HashTable *ht, sort_func_t sort_func,
 
 ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t compar, zend_bool ordered TSRMLS_DC)
 {
-	Bucket *p1, *p2 = NULL;
+	Bucket *p1, *p2;
 	int result;
 	void *pData2;
 
@@ -1542,7 +1318,7 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 					return 1;
 				}
 			} else { /* string index */
-				if (zend_hash_quick_find(ht2, p1->arKey, p1->nKeyLength, p1->h, &pData2)==FAILURE) {
+				if (zend_hash_find(ht2, p1->arKey, p1->nKeyLength, &pData2)==FAILURE) {
 					HASH_UNPROTECT_RECURSION(ht1); 
 					HASH_UNPROTECT_RECURSION(ht2); 
 					return 1;
@@ -1567,7 +1343,7 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 }
 
 
-ZEND_API int zend_hash_minmax(const HashTable *ht, compare_func_t compar, int flag, void **pData TSRMLS_DC)
+ZEND_API int zend_hash_minmax(HashTable *ht, compare_func_t compar, int flag, void **pData TSRMLS_DC)
 {
 	Bucket *p, *res;
 
@@ -1594,7 +1370,7 @@ ZEND_API int zend_hash_minmax(const HashTable *ht, compare_func_t compar, int fl
 	return SUCCESS;
 }
 
-ZEND_API ulong zend_hash_next_free_element(const HashTable *ht)
+ZEND_API ulong zend_hash_next_free_element(HashTable *ht)
 {
 	IS_CONSISTENT(ht);
 
@@ -1604,7 +1380,7 @@ ZEND_API ulong zend_hash_next_free_element(const HashTable *ht)
 
 
 #if ZEND_DEBUG
-void zend_hash_display_pListTail(const HashTable *ht)
+void zend_hash_display_pListTail(HashTable *ht)
 {
 	Bucket *p;
 
@@ -1615,15 +1391,11 @@ void zend_hash_display_pListTail(const HashTable *ht)
 	}
 }
 
-void zend_hash_display(const HashTable *ht)
+void zend_hash_display(HashTable *ht)
 {
 	Bucket *p;
 	uint i;
 
-	if (UNEXPECTED(ht->nNumOfElements == 0)) {
-		zend_output_debug_string(0, "The hash is empty");
-		return;
-	}
 	for (i = 0; i < ht->nTableSize; i++) {
 		p = ht->arBuckets[i];
 		while (p != NULL) {
