@@ -2,10 +2,10 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
-   | that is bundled with this package in the file LICENSE, and is        |
+   | that is bundled with this package in the file LICENSE, and is        | 
    | available through the world-wide-web at the following url:           |
    | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
@@ -17,14 +17,13 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_modules.h,v 1.67.2.3.2.4.2.9 2009/06/26 15:44:19 johannes Exp $ */
+/* $Id: zend_modules.h,v 1.67.2.2 2006/01/04 23:53:04 andi Exp $ */
 
 #ifndef MODULES_H
 #define MODULES_H
 
 #include "zend.h"
 #include "zend_compile.h"
-#include "zend_build.h"
 
 #define INIT_FUNC_ARGS		int type, int module_number TSRMLS_DC
 #define INIT_FUNC_ARGS_PASSTHRU	type, module_number TSRMLS_CC
@@ -33,7 +32,13 @@
 #define ZEND_MODULE_INFO_FUNC_ARGS zend_module_entry *zend_module TSRMLS_DC
 #define ZEND_MODULE_INFO_FUNC_ARGS_PASSTHRU zend_module TSRMLS_CC
 
-#define ZEND_MODULE_API_NO 20090626
+extern struct _zend_arg_info first_arg_force_ref[2];
+extern struct _zend_arg_info second_arg_force_ref[3];
+extern struct _zend_arg_info third_arg_force_ref[4];
+extern struct _zend_arg_info fourth_arg_force_ref[5];
+extern struct _zend_arg_info all_args_by_ref[1];
+
+#define ZEND_MODULE_API_NO 20050922
 #ifdef ZTS
 #define USING_ZTS 1
 #else
@@ -46,20 +51,10 @@
 #define ZE2_STANDARD_MODULE_HEADER \
 	STANDARD_MODULE_HEADER_EX, ini_entries, NULL
 
-#define ZEND_MODULE_BUILD_ID "API" ZEND_TOSTR(ZEND_MODULE_API_NO) ZEND_BUILD_TS ZEND_BUILD_DEBUG ZEND_BUILD_SYSTEM ZEND_BUILD_EXTRA
-
-#define STANDARD_MODULE_PROPERTIES_EX 0, 0, NULL, 0, ZEND_MODULE_BUILD_ID
-
-#define NO_MODULE_GLOBALS 0, NULL, NULL, NULL
-
-#ifdef ZTS
-# define ZEND_MODULE_GLOBALS(module_name) sizeof(zend_##module_name##_globals), &module_name##_globals_id
-#else
-# define ZEND_MODULE_GLOBALS(module_name) sizeof(zend_##module_name##_globals), &module_name##_globals
-#endif
+#define STANDARD_MODULE_PROPERTIES_EX 0, 0, 0, NULL, 0
 
 #define STANDARD_MODULE_PROPERTIES \
-	NO_MODULE_GLOBALS, NULL, STANDARD_MODULE_PROPERTIES_EX
+	NULL, STANDARD_MODULE_PROPERTIES_EX
 
 #define NO_VERSION_YET NULL
 
@@ -75,30 +70,22 @@ struct _zend_module_entry {
 	unsigned int zend_api;
 	unsigned char zend_debug;
 	unsigned char zts;
-	const struct _zend_ini_entry *ini_entry;
-	const struct _zend_module_dep *deps;
-	const char *name;
-	const struct _zend_function_entry *functions;
+	struct _zend_ini_entry *ini_entry;
+	struct _zend_module_dep *deps;
+	char *name;
+	struct _zend_function_entry *functions;
 	int (*module_startup_func)(INIT_FUNC_ARGS);
 	int (*module_shutdown_func)(SHUTDOWN_FUNC_ARGS);
 	int (*request_startup_func)(INIT_FUNC_ARGS);
 	int (*request_shutdown_func)(SHUTDOWN_FUNC_ARGS);
 	void (*info_func)(ZEND_MODULE_INFO_FUNC_ARGS);
-	const char *version;
-	size_t globals_size;
-#ifdef ZTS
-	ts_rsrc_id* globals_id_ptr;
-#else
-	void* globals_ptr;
-#endif
-	void (*globals_ctor)(void *global TSRMLS_DC);
-	void (*globals_dtor)(void *global TSRMLS_DC);
+	char *version;
 	int (*post_deactivate_func)(void);
+	int globals_id;
 	int module_started;
 	unsigned char type;
 	void *handle;
 	int module_number;
-	char *build_id;
 };
 
 #define MODULE_DEP_REQUIRED		1
@@ -114,10 +101,10 @@ struct _zend_module_entry {
 #define ZEND_MOD_OPTIONAL(name)		ZEND_MOD_OPTIONAL_EX(name, NULL, NULL)
 
 struct _zend_module_dep {
-	const char *name;		/* module name */
-	const char *rel;		/* version relationship: NULL (exists), lt|le|eq|ge|gt (to given version) */
-	const char *version;	/* version */
-	unsigned char type;		/* dependency type */
+	char *name;			/* module name */
+	char *rel;			/* version relationship: NULL (exists), lt|le|eq|ge|gt (to given version) */
+	char *version;		/* version */
+	unsigned char type;	/* dependency type */
 };
 
 extern ZEND_API HashTable module_registry;
@@ -125,7 +112,7 @@ extern ZEND_API HashTable module_registry;
 void module_destructor(zend_module_entry *module);
 int module_registry_cleanup(zend_module_entry *module TSRMLS_DC);
 int module_registry_request_startup(zend_module_entry *module TSRMLS_DC);
-int module_registry_unload_temp(const zend_module_entry *module TSRMLS_DC);
+int module_registry_unload_temp(zend_module_entry *module TSRMLS_DC);
 
 #define ZEND_MODULE_DTOR (void (*)(void *)) module_destructor
 #endif
