@@ -1,8 +1,8 @@
-/*
+/* 
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,44 +17,23 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_objects.c,v 1.56.2.3.2.6.2.8 2008/12/31 11:15:32 sebastian Exp $ */
+/* $Id: zend_objects.c,v 1.56.2.2 2006/01/04 23:53:04 andi Exp $ */
 
 #include "zend.h"
 #include "zend_globals.h"
 #include "zend_variables.h"
 #include "zend_API.h"
 #include "zend_interfaces.h"
-#include "zend_exceptions.h"
 
-ZEND_API void zend_object_std_init(zend_object *object, zend_class_entry *ce TSRMLS_DC)
-{
-	ALLOC_HASHTABLE(object->properties);
-	zend_hash_init(object->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-
-	object->ce = ce;	
-	object->guards = NULL;
-}
-
-ZEND_API void zend_object_std_dtor(zend_object *object TSRMLS_DC)
-{
-	if (object->guards) {
-		zend_hash_destroy(object->guards);
-		FREE_HASHTABLE(object->guards);		
-	}
-	if (object->properties) {
-		zend_hash_destroy(object->properties);
-		FREE_HASHTABLE(object->properties);
-	}
-}
 
 ZEND_API void zend_objects_destroy_object(zend_object *object, zend_object_handle handle TSRMLS_DC)
 {
 	zend_function *destructor = object->ce->destructor;
 
 	if (destructor) {
-		zval *obj;
-		zend_object_store_bucket *obj_bucket;
-
+		zval zobj, *obj = &zobj;
+		zval *old_exception;
+		
 		if (destructor->op_array.fn_flags & (ZEND_ACC_PRIVATE|ZEND_ACC_PROTECTED)) {
 			if (destructor->op_array.fn_flags & ZEND_ACC_PRIVATE) {
 				/* Ensure that if we're calling a private function, we're allowed to do so.
@@ -85,33 +64,36 @@ ZEND_API void zend_objects_destroy_object(zend_object *object, zend_object_handl
 			}
 		}
 
-		MAKE_STD_ZVAL(obj);
-		Z_TYPE_P(obj) = IS_OBJECT;
-		Z_OBJ_HANDLE_P(obj) = handle;
-		obj_bucket = &EG(objects_store).object_buckets[handle];
-		if (!obj_bucket->bucket.obj.handlers) {
-			obj_bucket->bucket.obj.handlers = &std_object_handlers;
-		}
-		Z_OBJ_HT_P(obj) = obj_bucket->bucket.obj.handlers;
-		zval_copy_ctor(obj);
+		zobj.type = IS_OBJECT;
+		zobj.value.obj.handle = handle;
+		zobj.value.obj.handlers = &std_object_handlers;
+		INIT_PZVAL(obj);
 
 		/* Make sure that destructors are protected from previously thrown exceptions.
 		 * For example, if an exception was thrown in a function and when the function's
 		 * local variable destruction results in a destructor being called.
 		 */
-		if (EG(exception) && Z_OBJ_HANDLE_P(EG(exception)) == handle) {
-			zend_error(E_ERROR, "Attempt to destruct pending exception");
+		old_exception = EG(exception);
+		EG(exception) = NULL;
+		zend_call_method_with_0_params(&obj, object->ce, &object->ce->destructor, ZEND_DESTRUCTOR_FUNC_NAME, NULL);
+		if (old_exception) {
+			if (EG(exception)) {
+				zend_error(E_ERROR, "Ignoring exception from %s::__destruct() while an exception is already active", object->ce->name);
+				zval_ptr_dtor(&EG(exception));
+			}
+			EG(exception) = old_exception;
 		}
-		zend_exception_save(TSRMLS_C);
-		zend_call_method_with_0_params(&obj, object->ce, &destructor, ZEND_DESTRUCTOR_FUNC_NAME, NULL);
-		zend_exception_restore(TSRMLS_C);
-		zval_ptr_dtor(&obj);
 	}
 }
 
 ZEND_API void zend_objects_free_object_storage(zend_object *object TSRMLS_DC)
 {
-	zend_object_std_dtor(object TSRMLS_CC);
+	if (object->guards) {
+		zend_hash_destroy(object->guards);
+		FREE_HASHTABLE(object->guards);		
+	}
+	zend_hash_destroy(object->properties);
+	FREE_HASHTABLE(object->properties);
 	efree(object);
 }
 
@@ -127,15 +109,38 @@ ZEND_API zend_object_value zend_objects_new(zend_object **object, zend_class_ent
 	return retval;
 }
 
-ZEND_API zend_object *zend_objects_get_address(const zval *zobject TSRMLS_DC)
+ZEND_API zend_object *zend_objects_get_address(zval *zobject TSRMLS_DC)
 {
 	return (zend_object *)zend_object_store_get_object(zobject TSRMLS_CC);
 }
 
+static void zval_add_ref_or_clone(zval **p)
+{
+	if (Z_TYPE_PP(p) == IS_OBJECT && !PZVAL_IS_REF(*p)) {
+		TSRMLS_FETCH();
+
+		if (Z_OBJ_HANDLER_PP(p, clone_obj) == NULL) {
+			zend_error(E_ERROR, "Trying to clone an uncloneable object of class %s",  Z_OBJCE_PP(p)->name);
+		} else {
+			zval *orig = *p;
+
+			ALLOC_ZVAL(*p);
+			**p = *orig;
+			INIT_PZVAL(*p);
+			(*p)->value.obj = Z_OBJ_HT_PP(p)->clone_obj(orig TSRMLS_CC);
+		}
+	} else {
+		(*p)->refcount++;
+	}
+}
+
 ZEND_API void zend_objects_clone_members(zend_object *new_object, zend_object_value new_obj_val, zend_object *old_object, zend_object_handle handle TSRMLS_DC)
 {
-	zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref, (void *) NULL /* Not used anymore */, sizeof(zval *));
-
+	if (EG(ze1_compatibility_mode)) {
+		zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref_or_clone, (void *) NULL /* Not used anymore */, sizeof(zval *));
+	} else {
+		zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref, (void *) NULL /* Not used anymore */, sizeof(zval *));
+	}
 	if (old_object->ce->clone) {
 		zval *new_obj;
 
