@@ -1,8 +1,8 @@
-/*
+/* 
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_objects_API.c,v 1.47.2.6.2.6.2.9 2009/02/11 09:58:23 tony2001 Exp $ */
+/* $Id: zend_objects_API.c,v 1.47.2.4 2006/01/04 23:53:04 andi Exp $ */
 
 #include "zend.h"
 #include "zend_globals.h"
@@ -33,7 +33,6 @@ ZEND_API void zend_objects_store_init(zend_objects_store *objects, zend_uint ini
 	objects->top = 1; /* Skip 0 so that handles are true */
 	objects->size = init_size;
 	objects->free_list_head = -1;
-	memset(&objects->object_buckets[0], 0, sizeof(zend_object_store_bucket));
 }
 
 ZEND_API void zend_objects_store_destroy(zend_objects_store *objects)
@@ -53,10 +52,7 @@ ZEND_API void zend_objects_store_call_destructors(zend_objects_store *objects TS
 			if (!objects->object_buckets[i].destructor_called) {
 				objects->object_buckets[i].destructor_called = 1;
 				if (obj->dtor && obj->object) {
-					obj->refcount++;
 					obj->dtor(obj->object, i TSRMLS_CC);
-					obj = &objects->object_buckets[i].bucket.obj;
-					obj->refcount--;
 				}
 			}
 		}
@@ -65,11 +61,8 @@ ZEND_API void zend_objects_store_call_destructors(zend_objects_store *objects TS
 
 ZEND_API void zend_objects_store_mark_destructed(zend_objects_store *objects TSRMLS_DC)
 {
-	zend_uint i;
+	zend_uint i = 1;
 
-	if (!objects->object_buckets) {
-		return;
-	}
 	for (i = 1; i < objects->top ; i++) {
 		if (objects->object_buckets[i].valid) {
 			objects->object_buckets[i].destructor_called = 1;
@@ -85,8 +78,6 @@ ZEND_API void zend_objects_store_free_object_storage(zend_objects_store *objects
 		if (objects->object_buckets[i].valid) {
 			struct _store_object *obj = &objects->object_buckets[i].bucket.obj;
 
-			GC_REMOVE_ZOBJ_FROM_BUFFER(obj);
-
 			objects->object_buckets[i].valid = 0;
 			if (obj->free_storage) {
 				obj->free_storage(obj->object TSRMLS_CC);
@@ -103,7 +94,7 @@ ZEND_API zend_object_handle zend_objects_store_put(void *object, zend_objects_st
 {
 	zend_object_handle handle;
 	struct _store_object *obj;
-
+	
 	if (EG(objects_store).free_list_head != -1) {
 		handle = EG(objects_store).free_list_head;
 		EG(objects_store).free_list_head = EG(objects_store).object_buckets[handle].bucket.free_list.next;
@@ -119,12 +110,11 @@ ZEND_API zend_object_handle zend_objects_store_put(void *object, zend_objects_st
 	EG(objects_store).object_buckets[handle].valid = 1;
 
 	obj->refcount = 1;
-	GC_OBJ_INIT(obj);
 	obj->object = object;
 	obj->dtor = dtor?dtor:(zend_objects_store_dtor_t)zend_objects_destroy_object;
 	obj->free_storage = free_storage;
+
 	obj->clone = clone;
-	obj->handlers = NULL;
 
 #if ZEND_DEBUG_OBJECTS
 	fprintf(stderr, "Allocated object id #%d\n", handle);
@@ -132,13 +122,6 @@ ZEND_API zend_object_handle zend_objects_store_put(void *object, zend_objects_st
 	return handle;
 }
 
-ZEND_API zend_uint zend_objects_store_get_refcount(zval *object TSRMLS_DC)
-{
-	zend_object_handle handle = Z_OBJ_HANDLE_P(object);
-
-	return EG(objects_store).object_buckets[handle].bucket.obj.refcount;
-}
-
 ZEND_API void zend_objects_store_add_ref(zval *object TSRMLS_DC)
 {
 	zend_object_handle handle = Z_OBJ_HANDLE_P(object);
@@ -149,14 +132,6 @@ ZEND_API void zend_objects_store_add_ref(zval *object TSRMLS_DC)
 #endif
 }
 
-/*
- * Add a reference to an objects store entry given the object handle.
- */
-ZEND_API void zend_objects_store_add_ref_by_handle(zend_object_handle handle TSRMLS_DC)
-{
-	EG(objects_store).object_buckets[handle].bucket.obj.refcount++;
-}
-
 #define ZEND_OBJECTS_STORE_ADD_TO_FREE_LIST()																	\
 			EG(objects_store).object_buckets[handle].bucket.free_list.next = EG(objects_store).free_list_head;	\
 			EG(objects_store).free_list_head = handle;															\
@@ -165,62 +140,33 @@ ZEND_API void zend_objects_store_add_ref_by_handle(zend_object_handle handle TSR
 ZEND_API void zend_objects_store_del_ref(zval *zobject TSRMLS_DC)
 {
 	zend_object_handle handle;
-
-	handle = Z_OBJ_HANDLE_P(zobject);
-
-	Z_ADDREF_P(zobject);
-	zend_objects_store_del_ref_by_handle_ex(handle, Z_OBJ_HT_P(zobject) TSRMLS_CC);
-	Z_DELREF_P(zobject);
-
-	GC_ZOBJ_CHECK_POSSIBLE_ROOT(zobject);
-}
-
-/*
- * Delete a reference to an objects store entry given the object handle.
- */
-ZEND_API void zend_objects_store_del_ref_by_handle_ex(zend_object_handle handle, const zend_object_handlers *handlers TSRMLS_DC) /* {{{ */
-{
 	struct _store_object *obj;
-	int failure = 0;
 
 	if (!EG(objects_store).object_buckets) {
 		return;
 	}
 
+	handle = Z_OBJ_HANDLE_P(zobject);
 	obj = &EG(objects_store).object_buckets[handle].bucket.obj;
 
 	/*	Make sure we hold a reference count during the destructor call
 		otherwise, when the destructor ends the storage might be freed
 		when the refcount reaches 0 a second time
-	 */
+	*/
 	if (EG(objects_store).object_buckets[handle].valid) {
 		if (obj->refcount == 1) {
 			if (!EG(objects_store).object_buckets[handle].destructor_called) {
 				EG(objects_store).object_buckets[handle].destructor_called = 1;
 
 				if (obj->dtor) {
-					if (handlers && !obj->handlers) {
-						obj->handlers = handlers;
-					}
-					zend_try {
-						obj->dtor(obj->object, handle TSRMLS_CC);
-					} zend_catch {
-						failure = 1;
-					} zend_end_try();
+					zobject->refcount++;
+					obj->dtor(obj->object, handle TSRMLS_CC);
+					zobject->refcount--;
 				}
 			}
-			
-			/* re-read the object from the object store as the store might have been reallocated in the dtor */
-			obj = &EG(objects_store).object_buckets[handle].bucket.obj;
-
 			if (obj->refcount == 1) {
-				GC_REMOVE_ZOBJ_FROM_BUFFER(obj);
 				if (obj->free_storage) {
-					zend_try {
-						obj->free_storage(obj->object TSRMLS_CC);
-					} zend_catch {
-						failure = 1;
-					} zend_end_try();
+					obj->free_storage(obj->object TSRMLS_CC);
 				}
 				ZEND_OBJECTS_STORE_ADD_TO_FREE_LIST();
 			}
@@ -236,11 +182,7 @@ ZEND_API void zend_objects_store_del_ref_by_handle_ex(zend_object_handle handle,
 		fprintf(stderr, "Decreased refcount of object id #%d\n", handle);
 	}
 #endif
-	if (failure) {
-		zend_bailout();
-	}
 }
-/* }}} */
 
 ZEND_API zend_object_value zend_objects_store_clone_obj(zval *zobject TSRMLS_DC)
 {
@@ -250,36 +192,26 @@ ZEND_API zend_object_value zend_objects_store_clone_obj(zval *zobject TSRMLS_DC)
 	zend_object_handle handle = Z_OBJ_HANDLE_P(zobject);
 
 	obj = &EG(objects_store).object_buckets[handle].bucket.obj;
-
+	
 	if (obj->clone == NULL) {
 		zend_error(E_CORE_ERROR, "Trying to clone uncloneable object of class %s", Z_OBJCE_P(zobject)->name);
-	}
+	}		
 
 	obj->clone(obj->object, &new_object TSRMLS_CC);
-	obj = &EG(objects_store).object_buckets[handle].bucket.obj;
 
 	retval.handle = zend_objects_store_put(new_object, obj->dtor, obj->free_storage, obj->clone TSRMLS_CC);
 	retval.handlers = Z_OBJ_HT_P(zobject);
-	EG(objects_store).object_buckets[handle].bucket.obj.handlers = retval.handlers;
-
+	
 	return retval;
 }
 
-ZEND_API void *zend_object_store_get_object(const zval *zobject TSRMLS_DC)
+ZEND_API void *zend_object_store_get_object(zval *zobject TSRMLS_DC)
 {
 	zend_object_handle handle = Z_OBJ_HANDLE_P(zobject);
 
 	return EG(objects_store).object_buckets[handle].bucket.obj.object;
 }
 
-/*
- * Retrieve an entry from the objects store given the object handle.
- */
-ZEND_API void *zend_object_store_get_object_by_handle(zend_object_handle handle TSRMLS_DC)
-{
-	return EG(objects_store).object_buckets[handle].bucket.obj.object;
-}
-
 /* zend_object_store_set_object:
  * It is ONLY valid to call this function from within the constructor of an
  * overloaded object.  Its purpose is to set the object pointer for the object
@@ -299,10 +231,8 @@ ZEND_API void zend_object_store_set_object(zval *zobject, void *object TSRMLS_DC
 ZEND_API void zend_object_store_ctor_failed(zval *zobject TSRMLS_DC)
 {
 	zend_object_handle handle = Z_OBJ_HANDLE_P(zobject);
-	zend_object_store_bucket *obj_bucket = &EG(objects_store).object_buckets[handle];
-	
-	obj_bucket->bucket.obj.handlers = Z_OBJ_HT_P(zobject);;
-	obj_bucket->destructor_called = 1;
+
+	EG(objects_store).object_buckets[handle].destructor_called = 1;
 }
 
 
@@ -341,10 +271,10 @@ ZEND_API zval *zend_object_create_proxy(zval *object, zval *member TSRMLS_DC)
 	zval_add_ref(&pobj->object);
 
 	MAKE_STD_ZVAL(retval);
-	Z_TYPE_P(retval) = IS_OBJECT;
+	retval->type = IS_OBJECT;
 	Z_OBJ_HANDLE_P(retval) = zend_objects_store_put(pobj, NULL, (zend_objects_free_object_storage_t) zend_objects_proxy_free_storage, (zend_objects_store_clone_t) zend_objects_proxy_clone TSRMLS_CC);
 	Z_OBJ_HT_P(retval) = &zend_object_proxy_handlers;
-
+	
 	return retval;
 }
 
@@ -372,14 +302,14 @@ ZEND_API zval* zend_object_proxy_get(zval *property TSRMLS_DC)
 	return NULL;
 }
 
-ZEND_API zend_object_handlers *zend_get_std_object_handlers(void)
+ZEND_API zend_object_handlers *zend_get_std_object_handlers()
 {
 	return &std_object_handlers;
 }
 
 static zend_object_handlers zend_object_proxy_handlers = {
 	ZEND_OBJECTS_STORE_HANDLERS,
-
+	
 	NULL,						/* read_property */
 	NULL,						/* write_property */
 	NULL,						/* read dimension */
