@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    |          Wez Furlong               <wez@thebrainroom.com>            |
    +----------------------------------------------------------------------+
  */
-/* $Id: calendar.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id: calendar.c,v 1.46.2.2 2006/01/01 12:50:00 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -35,120 +35,26 @@
 
 #include <stdio.h>
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_unixtojd, 0, 0, 0)
-	ZEND_ARG_INFO(0, timestamp)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jdtounix, 0)
-	ZEND_ARG_INFO(0, jday)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_cal_info, 0, 0, 0)
-	ZEND_ARG_INFO(0, calendar)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_cal_days_in_month, 0)
-	ZEND_ARG_INFO(0, calendar)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_cal_to_jd, 0)
-	ZEND_ARG_INFO(0, calendar)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, day)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_cal_from_jd, 0)
-	ZEND_ARG_INFO(0, jd)
-	ZEND_ARG_INFO(0, calendar)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jdtogregorian, 0)
-	ZEND_ARG_INFO(0, juliandaycount)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_gregoriantojd, 0)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, day)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jdtojulian, 0)
-	ZEND_ARG_INFO(0, juliandaycount)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_juliantojd, 0)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, day)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_jdtojewish, 0, 0, 1)
-	ZEND_ARG_INFO(0, juliandaycount)
-	ZEND_ARG_INFO(0, hebrew)
-	ZEND_ARG_INFO(0, fl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jewishtojd, 0)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, day)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jdtofrench, 0)
-	ZEND_ARG_INFO(0, juliandaycount)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_frenchtojd, 0)
-	ZEND_ARG_INFO(0, month)
-	ZEND_ARG_INFO(0, day)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_jddayofweek, 0, 0, 1)
-	ZEND_ARG_INFO(0, juliandaycount)
-	ZEND_ARG_INFO(0, mode)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_jdmonthname, 0)
-	ZEND_ARG_INFO(0, juliandaycount)
-	ZEND_ARG_INFO(0, mode)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_easter_date, 0, 0, 0)
-	ZEND_ARG_INFO(0, year)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_easter_days, 0, 0, 0)
-	ZEND_ARG_INFO(0, year)
-	ZEND_ARG_INFO(0, method)
-ZEND_END_ARG_INFO()
-
-/* }}} */
-
-const zend_function_entry calendar_functions[] = {
-	PHP_FE(jdtogregorian, arginfo_jdtogregorian)
-	PHP_FE(gregoriantojd, arginfo_gregoriantojd)
-	PHP_FE(jdtojulian, arginfo_jdtojulian)
-	PHP_FE(juliantojd, arginfo_juliantojd)
-	PHP_FE(jdtojewish, arginfo_jdtojewish)
-	PHP_FE(jewishtojd, arginfo_jewishtojd)
-	PHP_FE(jdtofrench, arginfo_jdtofrench)
-	PHP_FE(frenchtojd, arginfo_frenchtojd)
-	PHP_FE(jddayofweek, arginfo_jddayofweek)
-	PHP_FE(jdmonthname, arginfo_jdmonthname)
-	PHP_FE(easter_date, arginfo_easter_date) 
-	PHP_FE(easter_days, arginfo_easter_days)
-	PHP_FE(unixtojd, arginfo_unixtojd)
-	PHP_FE(jdtounix, arginfo_jdtounix)
-	PHP_FE(cal_to_jd, arginfo_cal_to_jd)
-	PHP_FE(cal_from_jd, arginfo_cal_from_jd)
-	PHP_FE(cal_days_in_month, arginfo_cal_days_in_month)
-	PHP_FE(cal_info, arginfo_cal_info)
-	PHP_FE_END
+zend_function_entry calendar_functions[] = {
+	PHP_FE(jdtogregorian, NULL)
+	PHP_FE(gregoriantojd, NULL)
+	PHP_FE(jdtojulian, NULL)
+	PHP_FE(juliantojd, NULL)
+	PHP_FE(jdtojewish, NULL)
+	PHP_FE(jewishtojd, NULL)
+	PHP_FE(jdtofrench, NULL)
+	PHP_FE(frenchtojd, NULL)
+	PHP_FE(jddayofweek, NULL)
+	PHP_FE(jdmonthname, NULL)
+	PHP_FE(easter_date, NULL)
+	PHP_FE(easter_days, NULL)
+	PHP_FE(unixtojd, NULL)
+	PHP_FE(jdtounix, NULL)
+	PHP_FE(cal_to_jd, NULL)
+	PHP_FE(cal_from_jd, NULL)
+	PHP_FE(cal_days_in_month, NULL)
+	PHP_FE(cal_info, NULL)
+	{NULL, NULL, NULL}
 };
 
 
@@ -283,7 +189,7 @@ static void _php_cal_info(int cal, zval **ret)
 	
 }
 
-/* {{{ proto array cal_info([int calendar])
+/* {{{ proto array cal_info(int calendar)
    Returns information about a particular calendar */
 PHP_FUNCTION(cal_info)
 {
@@ -348,15 +254,8 @@ PHP_FUNCTION(cal_days_in_month)
 	sdn_next = calendar->to_jd(year, 1 + month, 1);
 
 	if (sdn_next == 0) {
-		/* If the next month is invalid, then we need to try the first month of
-		 * the next year, bearing in mind that the next year after 1 BCE is
-		 * actually 1 AD and not 0. */
-		if (year == -1) {
-			sdn_next = calendar->to_jd(1, 1, 1);
-		}
-		else {
-			sdn_next = calendar->to_jd(year + 1, 1, 1);
-		}
+/* if invalid, try first month of the next year... */
+		sdn_next = calendar->to_jd(year + 1, 1, 1);
 	}
 
 	RETURN_LONG(sdn_next - sdn_start);
@@ -405,7 +304,7 @@ PHP_FUNCTION(cal_from_jd)
 
 	calendar->from_jd(jd, &year, &month, &day);
 
-	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
+	sprintf(date, "%i/%i/%i", month, day, year);
 	add_assoc_string(return_value, "date", date, 1);
 
 	add_assoc_long(return_value, "month", month);
@@ -436,7 +335,7 @@ PHP_FUNCTION(jdtogregorian)
 	}
 
 	SdnToGregorian(julday, &year, &month, &day);
-	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
+	sprintf(date, "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
@@ -469,7 +368,7 @@ PHP_FUNCTION(jdtojulian)
 	}
 
 	SdnToJulian(julday, &year, &month, &day);
-	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
+	sprintf(date, "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
@@ -591,7 +490,7 @@ PHP_FUNCTION(jdtojewish)
 	long julday, fl = 0;
 	zend_bool heb   = 0;
 	int year, month, day;
-	char date[16], hebdate[32];
+	char date[16], hebdate[25];
 	char *dayp, *yearp;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|bl", &julday, &heb, &fl) == FAILURE) {
@@ -600,7 +499,7 @@ PHP_FUNCTION(jdtojewish)
 
 	SdnToJewish(julday, &year, &month, &day);
 	if (!heb) {
-		snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
+		sprintf(date, "%i/%i/%i", month, day, year);
 		RETURN_STRING(date, 1);
 	} else {
 		if (year <= 0 || year > 9999) {
@@ -608,7 +507,7 @@ PHP_FUNCTION(jdtojewish)
 			RETURN_FALSE;
 		}
 
-		snprintf(hebdate, sizeof(hebdate), "%s %s %s", heb_number_to_chars(day, fl, &dayp), JewishMonthHebName[month], heb_number_to_chars(year, fl, &yearp));
+		sprintf(hebdate, "%s %s %s", heb_number_to_chars(day, fl, &dayp), JewishMonthHebName[month], heb_number_to_chars(year, fl, &yearp));
 
 		if (dayp) {
 			efree(dayp);
@@ -650,7 +549,7 @@ PHP_FUNCTION(jdtofrench)
 	}
 
 	SdnToFrench(julday, &year, &month, &day);
-	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
+	sprintf(date, "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
