@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_iterator.c,v 1.9.2.2.2.2.2.3 2009/05/19 17:38:29 kalle Exp $ */
+/* $Id: com_iterator.c,v 1.9.2.1 2006/01/01 12:50:00 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,7 +38,6 @@ struct php_com_iterator {
 	VARIANT safe_array;
 	VARTYPE sa_type;
 	LONG sa_max;
-	zval *zdata;
 };
 
 static void com_iter_dtor(zend_object_iterator *iter TSRMLS_DC)
@@ -50,9 +49,6 @@ static void com_iter_dtor(zend_object_iterator *iter TSRMLS_DC)
 	}
 	VariantClear(&I->v);
 	VariantClear(&I->safe_array);
-	if (I->zdata) {
-		zval_ptr_dtor((zval**)&I->zdata);
-	}
 	efree(I);
 }
 
@@ -60,18 +56,32 @@ static int com_iter_valid(zend_object_iterator *iter TSRMLS_DC)
 {
 	struct php_com_iterator *I = (struct php_com_iterator*)iter->data;
 
-	if (I->zdata) {
-		return SUCCESS;
+	if (I->key == (ulong)-1) {
+		return FAILURE;
 	}
-
-	return FAILURE;
+	return SUCCESS;
 }
 
 static void com_iter_get_data(zend_object_iterator *iter, zval ***data TSRMLS_DC)
 {
 	struct php_com_iterator *I = (struct php_com_iterator*)iter->data;
+	zval **ptr_ptr;
+	zval *ptr;
 
-	*data = &I->zdata;
+	/* sanity */
+	if (I->key == (ulong)-1) {
+		*data = NULL;
+		return;
+	}
+
+	MAKE_STD_ZVAL(ptr);
+	php_com_zval_from_variant(ptr, &I->v, I->code_page TSRMLS_CC);
+	/* php_com_wrap_variant(ptr, &I->v, I->code_page TSRMLS_CC); */
+	ptr_ptr = emalloc(sizeof(*ptr_ptr));
+	*ptr_ptr = ptr;
+	*data = ptr_ptr;
+
+	return;
 }
 
 static int com_iter_get_key(zend_object_iterator *iter, char **str_key, uint *str_key_len,
@@ -90,20 +100,15 @@ static int com_iter_move_forwards(zend_object_iterator *iter TSRMLS_DC)
 {
 	struct php_com_iterator *I = (struct php_com_iterator*)iter->data;
 	unsigned long n_fetched;
-	zval *ptr;
 
 	/* release current cached element */
 	VariantClear(&I->v);
 
-	if (I->zdata) {
-		zval_ptr_dtor((zval**)&I->zdata);
-		I->zdata = NULL;
-	}
-
 	if (I->ev) {
 		/* Get the next element */
 		if (SUCCEEDED(IEnumVARIANT_Next(I->ev, 1, &I->v, &n_fetched)) && n_fetched > 0) {
 			I->key++;
+			return SUCCESS;
 		} else {
 			/* indicate that there are no more items */
 			I->key = (ulong)-1;
@@ -111,22 +116,18 @@ static int com_iter_move_forwards(zend_object_iterator *iter TSRMLS_DC)
 		}
 	} else {
 		/* safe array */
-		if (I->key >= (ULONG) I->sa_max) {
+		if (I->key >= I->sa_max) {
 			I->key = (ulong)-1;
 			return FAILURE;
 		}
 		I->key++;
-		if (php_com_safearray_get_elem(&I->safe_array, &I->v, (LONG)I->key TSRMLS_CC) == 0) {
+		if (php_com_safearray_get_elem(&I->safe_array, &I->v, (LONG)I->key TSRMLS_CC)) {
+			return SUCCESS;
+		} else {
 			I->key = (ulong)-1;
 			return FAILURE;
 		}
 	}
-
-	MAKE_STD_ZVAL(ptr);
-	php_com_zval_from_variant(ptr, &I->v, I->code_page TSRMLS_CC);
-	/* php_com_wrap_variant(ptr, &I->v, I->code_page TSRMLS_CC); */
-	I->zdata = ptr;
-	return SUCCESS;
 }
 
 
@@ -139,7 +140,7 @@ static zend_object_iterator_funcs com_iter_funcs = {
 	NULL
 };
 
-zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC)
+zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object TSRMLS_DC)
 {
 	php_com_dotnet_object *obj;
 	struct php_com_iterator *I;
@@ -147,11 +148,6 @@ zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object, int b
 	DISPPARAMS dp;
 	VARIANT v;
 	unsigned long n_fetched;
-	zval *ptr;
-
-	if (by_ref) {
-		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
-	}
 
 	obj = CDNO_FETCH(object);
 
@@ -167,7 +163,6 @@ zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object, int b
 	I->iter.funcs = &com_iter_funcs;
 	I->iter.data = I;
 	I->code_page = obj->code_page;
-	I->zdata = NULL;
 	VariantInit(&I->safe_array);
 	VariantInit(&I->v);
 
@@ -194,9 +189,6 @@ zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object, int b
 		/* pre-fetch the element */
 		if (php_com_safearray_get_elem(&I->safe_array, &I->v, bound TSRMLS_CC)) {
 			I->key = bound;
-			MAKE_STD_ZVAL(ptr);
-			php_com_zval_from_variant(ptr, &I->v, I->code_page TSRMLS_CC);
-			I->zdata = ptr;
 		} else {
 			I->key = (ulong)-1;
 		}
@@ -228,9 +220,6 @@ zend_object_iterator *php_com_iter_get(zend_class_entry *ce, zval *object, int b
 		if (SUCCEEDED(IEnumVARIANT_Next(I->ev, 1, &I->v, &n_fetched)) && n_fetched > 0) {
 			/* indicate that we have element 0 */
 			I->key = 0;
-			MAKE_STD_ZVAL(ptr);
-			php_com_zval_from_variant(ptr, &I->v, I->code_page TSRMLS_CC);
-			I->zdata = ptr;
 		} else {
 			/* indicate that there are no more items */
 			I->key = (ulong)-1;
