@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_persist.c,v 1.5.2.3.2.2.2.4 2009/05/19 17:38:29 kalle Exp $ */
+/* $Id: com_persist.c,v 1.5.2.1 2006/01/01 12:50:00 sniper Exp $ */
 
 /* Infrastructure for working with persistent COM objects.
  * Implements: IStream* wrapper for PHP streams.
@@ -157,7 +157,7 @@ static HRESULT STDMETHODCALLTYPE stm_seek(IStream *This, LARGE_INTEGER dlibMove,
 		return STG_E_INVALIDFUNCTION;
 	}
 	
-	offset = (off_t) dlibMove.QuadPart;
+	offset = dlibMove.QuadPart;
 
 	ret = php_stream_seek(stm->stream, offset, whence);
 
@@ -390,16 +390,15 @@ CPH_METHOD(SaveToFile)
 
 		if (filename) {
 			fullpath = expand_filepath(filename, NULL TSRMLS_CC);
-			if (!fullpath) {
-				RETURN_FALSE;
-			}
 	
-			if ((PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) || 
-					php_check_open_basedir(fullpath TSRMLS_CC)) {
-				efree(fullpath);
+			if (PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 				RETURN_FALSE;
 			}
 
+			if (php_check_open_basedir(fullpath TSRMLS_CC)) {
+				RETURN_FALSE;
+			}
+			
 			olefilename = php_com_string_to_olestring(filename, strlen(fullpath), helper->codepage TSRMLS_CC);
 			efree(fullpath);
 		}
@@ -453,13 +452,13 @@ CPH_METHOD(LoadFromFile)
 			return;
 		}
 
-		if (!(fullpath = expand_filepath(filename, NULL TSRMLS_CC))) {
+		fullpath = expand_filepath(filename, NULL TSRMLS_CC);
+
+		if (PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 			RETURN_FALSE;
 		}
 
-		if ((PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) ||
-				php_check_open_basedir(fullpath TSRMLS_CC)) {
-			efree(fullpath);
+		if (php_check_open_basedir(fullpath TSRMLS_CC)) {
 			RETURN_FALSE;
 		}
 
@@ -681,7 +680,7 @@ CPH_METHOD(__construct)
 
 
 
-static const zend_function_entry com_persist_helper_methods[] = {
+static zend_function_entry com_persist_helper_methods[] = {
 	CPH_ME(__construct, NULL)
 	CPH_ME(GetCurFileName, NULL)
 	CPH_ME(SaveToFile, NULL)
@@ -709,7 +708,8 @@ static void helper_free_storage(void *obj TSRMLS_DC)
 	if (object->unk) {
 		IUnknown_Release(object->unk);
 	}
-	zend_object_std_dtor(&object->std TSRMLS_CC);
+	zend_hash_destroy(object->std.properties);
+	FREE_HASHTABLE(object->std.properties);
 	efree(object);
 }
 
@@ -722,8 +722,9 @@ static void helper_clone(void *obj, void **clone_ptr TSRMLS_DC)
 	memcpy(clone, object, sizeof(*object));
 	*clone_ptr = clone;
 
-	zend_object_std_init(&clone->std, object->std.ce TSRMLS_CC);
-
+	ALLOC_HASHTABLE(clone->std.properties);
+	zend_hash_init(clone->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
+	
 	if (clone->ipf) {
 		IPersistFile_AddRef(clone->ipf);
 	}
@@ -746,7 +747,9 @@ static zend_object_value helper_new(zend_class_entry *ce TSRMLS_DC)
 	helper = emalloc(sizeof(*helper));
 	memset(helper, 0, sizeof(*helper));
 
-	zend_object_std_init(&helper->std, helper_ce TSRMLS_CC);
+	ALLOC_HASHTABLE(helper->std.properties);
+	zend_hash_init(helper->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
+	helper->std.ce = helper_ce;
 	
 	retval.handle = zend_objects_store_put(helper, NULL, helper_free_storage, helper_clone TSRMLS_CC);
 	retval.handlers = &helper_handlers;
