@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_saproxy.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id: com_saproxy.c,v 1.15.2.2 2006/01/01 12:50:00 sniper Exp $ */
 
 /* This module implements a SafeArray proxy which is used internally
  * by the engine when resolving multi-dimensional array accesses on
@@ -71,7 +71,7 @@ static inline void clone_indices(php_com_saproxy *dest, php_com_saproxy *src, in
 	}
 }
 
-static zval *saproxy_property_read(zval *object, zval *member, int type, const zend_literal *key TSRMLS_DC)
+static zval *saproxy_property_read(zval *object, zval *member, int type TSRMLS_DC)
 {
 	zval *return_value;
 	
@@ -83,7 +83,7 @@ static zval *saproxy_property_read(zval *object, zval *member, int type, const z
 	return return_value;
 }
 
-static void saproxy_property_write(zval *object, zval *member, zval *value, const zend_literal *key TSRMLS_DC)
+static void saproxy_property_write(zval *object, zval *member, zval *value TSRMLS_DC)
 {
 	php_com_throw_exception(E_INVALIDARG, "safearray has no properties" TSRMLS_CC);
 }
@@ -92,9 +92,10 @@ static zval *saproxy_read_dimension(zval *object, zval *offset, int type TSRMLS_
 {
 	php_com_saproxy *proxy = SA_FETCH(object);
 	zval *return_value;
-	UINT dims, i;
+	UINT dims;
 	SAFEARRAY *sa;
 	LONG ubound, lbound;
+	int i;
 	HRESULT res;
 	
 	MAKE_STD_ZVAL(return_value);
@@ -109,7 +110,7 @@ static zval *saproxy_read_dimension(zval *object, zval *offset, int type TSRMLS_
 
 		args = safe_emalloc(proxy->dimensions + 1, sizeof(zval *), 0);
 
-		for (i = 1; i < (UINT) proxy->dimensions; i++) {
+		for (i = 1; i < proxy->dimensions; i++) {
 			args[i-1] = proxy->indices[i];
 		}
 		args[i-1] = offset;
@@ -119,7 +120,7 @@ static zval *saproxy_read_dimension(zval *object, zval *offset, int type TSRMLS_
 
 		res = php_com_do_invoke(proxy->obj, Z_STRVAL_P(proxy->indices[0]),
 			   	Z_STRLEN_P(proxy->indices[0]), DISPATCH_METHOD|DISPATCH_PROPERTYGET, &v,
-			   	proxy->dimensions, args, 0 TSRMLS_CC);
+			   	proxy->dimensions, args TSRMLS_CC);
 
 		if (res == SUCCESS) {
 			php_com_zval_from_variant(return_value, &v, proxy->obj->code_page TSRMLS_CC);
@@ -144,7 +145,7 @@ static zval *saproxy_read_dimension(zval *object, zval *offset, int type TSRMLS_
 	sa = V_ARRAY(&proxy->obj->v);
 	dims = SafeArrayGetDim(sa);
 
-	if ((UINT) proxy->dimensions >= dims) {
+	if (proxy->dimensions >= dims) {
 		/* too many dimensions */
 		php_com_throw_exception(E_INVALIDARG, "too many dimensions!" TSRMLS_CC);
 		return return_value;
@@ -211,7 +212,8 @@ static zval *saproxy_read_dimension(zval *object, zval *offset, int type TSRMLS_
 static void saproxy_write_dimension(zval *object, zval *offset, zval *value TSRMLS_DC)
 {
 	php_com_saproxy *proxy = SA_FETCH(object);
-	UINT dims, i;
+	UINT dims;
+	int i;
 	HRESULT res;
 	VARIANT v;
 	
@@ -221,7 +223,7 @@ static void saproxy_write_dimension(zval *object, zval *offset, zval *value TSRM
 		 * the final value */
 		zval **args = safe_emalloc(proxy->dimensions + 2, sizeof(zval *), 0);
 
-		for (i = 1; i < (UINT) proxy->dimensions; i++) {
+		for (i = 1; i < proxy->dimensions; i++) {
 			args[i-1] = proxy->indices[i];
 		}
 		args[i-1] = offset;
@@ -231,7 +233,7 @@ static void saproxy_write_dimension(zval *object, zval *offset, zval *value TSRM
 		VariantInit(&v);
 		if (SUCCESS == php_com_do_invoke(proxy->obj, Z_STRVAL_P(proxy->indices[0]),
 					Z_STRLEN_P(proxy->indices[0]), DISPATCH_PROPERTYPUT, &v, proxy->dimensions + 1,
-					args, 0 TSRMLS_CC)) {
+					args TSRMLS_CC)) {
 			VariantClear(&v);
 		}
 
@@ -293,7 +295,7 @@ static zval *saproxy_object_get(zval *property TSRMLS_DC)
 }
 #endif
 
-static int saproxy_property_exists(zval *object, zval *member, int check_empty, const zend_literal *key TSRMLS_DC)
+static int saproxy_property_exists(zval *object, zval *member, int check_empty TSRMLS_DC)
 {
 	/* no properties */
 	return 0;
@@ -305,7 +307,7 @@ static int saproxy_dimension_exists(zval *object, zval *member, int check_empty
 	return 0;
 }
 
-static void saproxy_property_delete(zval *object, zval *member, const zend_literal *key TSRMLS_DC)
+static void saproxy_property_delete(zval *object, zval *member TSRMLS_DC)
 {
 	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot delete properties from a COM object");
 }
@@ -321,13 +323,13 @@ static HashTable *saproxy_properties_get(zval *object TSRMLS_DC)
 	return NULL;
 }
 
-static union _zend_function *saproxy_method_get(zval **object, const char *name, int len, const zend_literal *key TSRMLS_DC)
+static union _zend_function *saproxy_method_get(zval **object, char *name, int len TSRMLS_DC)
 {
 	/* no methods */
 	return NULL;
 }
 
-static int saproxy_call_method(const char *method, INTERNAL_FUNCTION_PARAMETERS)
+static int saproxy_call_method(char *method, INTERNAL_FUNCTION_PARAMETERS)
 {
 	return FAILURE;
 }
@@ -338,12 +340,12 @@ static union _zend_function *saproxy_constructor_get(zval *object TSRMLS_DC)
 	return NULL;
 }
 
-static zend_class_entry *saproxy_class_entry_get(const zval *object TSRMLS_DC)
+static zend_class_entry *saproxy_class_entry_get(zval *object TSRMLS_DC)
 {
 	return php_com_saproxy_class_entry;
 }
 
-static int saproxy_class_name_get(const zval *object, const char **class_name, zend_uint *class_name_len, int parent TSRMLS_DC)
+static int saproxy_class_name_get(zval *object, char **class_name, zend_uint *class_name_len, int parent TSRMLS_DC)
 {
 	*class_name = estrndup(php_com_saproxy_class_entry->name, php_com_saproxy_class_entry->name_length);
 	*class_name_len = php_com_saproxy_class_entry->name_length;
@@ -355,7 +357,7 @@ static int saproxy_objects_compare(zval *object1, zval *object2 TSRMLS_DC)
 	return -1;
 }
 
-static int saproxy_object_cast(zval *readobj, zval *writeobj, int type TSRMLS_DC)
+static int saproxy_object_cast(zval *readobj, zval *writeobj, int type, int should_free TSRMLS_DC)
 {
 	return FAILURE;
 }
@@ -384,8 +386,8 @@ zend_object_handlers php_com_saproxy_handlers = {
 	saproxy_read_dimension,
 	saproxy_write_dimension,
 	NULL,
-	NULL, /* saproxy_object_get, */
-	NULL, /* saproxy_object_set, */
+	NULL, //saproxy_object_get,
+	NULL, //saproxy_object_set,
 	saproxy_property_exists,
 	saproxy_property_delete,
 	saproxy_dimension_exists,
@@ -425,7 +427,7 @@ static void saproxy_clone(void *object, void **clone_ptr TSRMLS_DC)
 	cloneproxy = emalloc(sizeof(*cloneproxy));
 	memcpy(cloneproxy, proxy, sizeof(*cloneproxy));
 
-	Z_ADDREF_P(cloneproxy->zobj);
+	ZVAL_ADDREF(cloneproxy->zobj);
 	cloneproxy->indices = safe_emalloc(cloneproxy->dimensions, sizeof(zval *), 0);
 	clone_indices(cloneproxy, proxy, proxy->dimensions);
 
@@ -449,7 +451,7 @@ int php_com_saproxy_create(zval *com_object, zval *proxy_out, zval *index TSRMLS
 		proxy->zobj = com_object;
 	}
 
-	Z_ADDREF_P(proxy->zobj);
+	ZVAL_ADDREF(proxy->zobj);
 	proxy->indices = safe_emalloc(proxy->dimensions, sizeof(zval *), 0);
 
 	if (rel) {
@@ -552,23 +554,19 @@ static zend_object_iterator_funcs saproxy_iter_funcs = {
 };
 
 
-zend_object_iterator *php_com_saproxy_iter_get(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC)
+zend_object_iterator *php_com_saproxy_iter_get(zend_class_entry *ce, zval *object TSRMLS_DC)
 {
 	php_com_saproxy *proxy = SA_FETCH(object);
 	php_com_saproxy_iter *I;
 	int i;
 
-	if (by_ref) {
-		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
-	}
-
 	I = ecalloc(1, sizeof(*I));
 	I->iter.funcs = &saproxy_iter_funcs;
 	I->iter.data = I;
 
 	I->proxy = proxy;
 	I->proxy_obj = object;
-	Z_ADDREF_P(I->proxy_obj);
+	ZVAL_ADDREF(I->proxy_obj);
 
 	I->indices = safe_emalloc(proxy->dimensions + 1, sizeof(LONG), 0);
 	for (i = 0; i < proxy->dimensions; i++) {
