@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: dba_db4.c,v 1.15.2.3.2.1.2.4 2009/05/13 02:14:53 felipe Exp $ */
+/* $Id: dba_db4.c,v 1.15.2.2 2006/01/01 12:50:05 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,11 +36,7 @@
 #include <db.h>
 #endif
 
-static void php_dba_db4_errcall_fcn(
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-	const DB_ENV *dbenv, 
-#endif
-	const char *errpfx, const char *msg)
+static void php_dba_db4_errcall_fcn(const char *errpfx, char *msg)
 {
 	TSRMLS_FETCH();
 	
@@ -85,6 +81,7 @@ DBA_OPEN_FUNC(db4)
 		return FAILURE; /* not possible */
 	}
 
+	gmode |= DB_INIT_LOCK;
 	if (info->flags & DBA_PERSISTENT) {
 		gmode |= DB_THREAD;
 	}
@@ -94,6 +91,10 @@ DBA_OPEN_FUNC(db4)
 		filemode = Z_LVAL_PP(info->argv[0]);
 	}
 
+#ifdef DB_FCNTL_LOCKING
+	gmode |= DB_FCNTL_LOCKING;
+#endif
+
 	if ((err=db_create(&dbp, NULL, 0)) == 0) {
 	    dbp->set_errcall(dbp, php_dba_db4_errcall_fcn);
 	    if (
@@ -138,15 +139,9 @@ DBA_FETCH_FUNC(db4)
 	DB4_GKEY;
 	
 	memset(&gval, 0, sizeof(gval));
-	if (info->flags & DBA_PERSISTENT) {
-		gval.flags |= DB_DBT_MALLOC;
-	}
 	if (!dba->dbp->get(dba->dbp, NULL, &gkey, &gval, 0)) {
 		if (newlen) *newlen = gval.size;
 		new = estrndup(gval.data, gval.size);
-		if (info->flags & DBA_PERSISTENT) {
-			free(gval.data);
-		}
 	}
 	return new;
 }
@@ -215,23 +210,11 @@ DBA_NEXTKEY_FUNC(db4)
 	memset(&gkey, 0, sizeof(gkey));
 	memset(&gval, 0, sizeof(gval));
 
-	if (info->flags & DBA_PERSISTENT) {
-		gkey.flags |= DB_DBT_MALLOC;
-		gval.flags |= DB_DBT_MALLOC;
-	}
-	if (dba->cursor && dba->cursor->c_get(dba->cursor, &gkey, &gval, DB_NEXT) == 0) {
+	if (dba->cursor->c_get(dba->cursor, &gkey, &gval, DB_NEXT) == 0) {
 		if (gkey.data) {
 			nkey = estrndup(gkey.data, gkey.size);
 			if (newlen) *newlen = gkey.size;
 		}
-		if (info->flags & DBA_PERSISTENT) {
-			if (gkey.data) {
-				free(gkey.data);
-			}
-			if (gval.data) {
-				free(gval.data);
-			}
-		}
 	}
 
 	return nkey;
