@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: dom_iterators.c,v 1.9.2.2 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -42,8 +42,7 @@ struct _notationIterator {
 	xmlNotation *notation;
 };
 
-static void itemHashScanner (void *payload, void *data, xmlChar *name) /* {{{ */
-{
+static void itemHashScanner (void *payload, void *data, xmlChar *name) {
 	nodeIterator *priv = (nodeIterator *)data;
 
 	if(priv->cur < priv->index) {
@@ -54,10 +53,9 @@ static void itemHashScanner (void *payload, void *data, xmlChar *name) /* {{{ */
 		}
 	}
 }
-/* }}} */
 
-xmlNodePtr create_notation(const xmlChar *name, const xmlChar *ExternalID, const xmlChar *SystemID) /* {{{ */
-{
+xmlNodePtr create_notation(const xmlChar *name, 
+									const xmlChar *ExternalID, const xmlChar *SystemID) {
 	xmlEntityPtr ret;
 
 	ret = (xmlEntityPtr) xmlMalloc(sizeof(xmlEntity));
@@ -78,9 +76,8 @@ xmlNodePtr create_notation(const xmlChar *name, const xmlChar *ExternalID, const
 	ret->prev = NULL;
 	return((xmlNodePtr) ret);
 }
-/* }}} */
 
-xmlNode *php_dom_libxml_hash_iter(xmlHashTable *ht, int index) /* {{{ */
+xmlNode *php_dom_libxml_hash_iter(xmlHashTable *ht, int index)
 {
 	xmlNode *nodep = NULL;
 	nodeIterator *iter;
@@ -99,9 +96,8 @@ xmlNode *php_dom_libxml_hash_iter(xmlHashTable *ht, int index) /* {{{ */
 		return NULL;
 	}
 }
-/* }}} */
 
-xmlNode *php_dom_libxml_notation_iter(xmlHashTable *ht, int index) /* {{{ */
+xmlNode *php_dom_libxml_notation_iter(xmlHashTable *ht, int index)
 {
 	notationIterator *iter;
 	xmlNotation *notep = NULL;
@@ -120,9 +116,8 @@ xmlNode *php_dom_libxml_notation_iter(xmlHashTable *ht, int index) /* {{{ */
 		return NULL;
 	}
 }
-/* }}} */
 
-static void php_dom_iterator_dtor(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static void php_dom_iterator_dtor(zend_object_iterator *iter TSRMLS_DC)
 {
 	php_dom_iterator *iterator = (php_dom_iterator *)iter;
 
@@ -134,9 +129,8 @@ static void php_dom_iterator_dtor(zend_object_iterator *iter TSRMLS_DC) /* {{{ *
 
 	efree(iterator);
 }
-/* }}} */
 
-static int php_dom_iterator_valid(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static int php_dom_iterator_valid(zend_object_iterator *iter TSRMLS_DC)
 {
 
 	php_dom_iterator *iterator = (php_dom_iterator *)iter;
@@ -147,17 +141,15 @@ static int php_dom_iterator_valid(zend_object_iterator *iter TSRMLS_DC) /* {{{ *
 		return FAILURE;
 	}
 }
-/* }}} */
 
-static void php_dom_iterator_current_data(zend_object_iterator *iter, zval ***data TSRMLS_DC) /* {{{ */
+static void php_dom_iterator_current_data(zend_object_iterator *iter, zval ***data TSRMLS_DC)
 {
 	php_dom_iterator *iterator = (php_dom_iterator *)iter;
 
 	*data = &iterator->curobj;
 }
-/* }}} */
 
-static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC) /* {{{ */
+static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC)
 {
 	zval *curobj;
 	xmlNodePtr curnode = NULL;
@@ -170,7 +162,7 @@ static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_k
 	object = (zval *)iterator->intern.data;
 
 	if (instanceof_function(Z_OBJCE_P(object), dom_nodelist_class_entry TSRMLS_CC)) {
-		*int_key = iter->index;
+		*int_key = iter->index - 1;
 		return HASH_KEY_IS_LONG;
 	} else {
 		curobj = iterator->curobj;
@@ -178,8 +170,6 @@ static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_k
 		intern = (dom_object *)zend_object_store_get_object(curobj TSRMLS_CC);
 		if (intern != NULL && intern->ptr != NULL) {
 			curnode = (xmlNodePtr)((php_libxml_node_ptr *)intern->ptr)->node;
-		} else {
-			return HASH_KEY_NON_EXISTANT;
 		}
 
 		namelen = xmlStrlen(curnode->name);
@@ -188,9 +178,8 @@ static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_k
 		return HASH_KEY_IS_STRING;
 	}
 }
-/* }}} */
 
-static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 {
 	zval *curobj, *curattr = NULL;
 	zval *object;
@@ -198,7 +187,7 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 	dom_object *intern;
 	dom_object *nnmap;
 	dom_nnodemap_object *objmap;
-	int ret, previndex=0;
+	int ret, previndex=1;
 	HashTable *nodeht;
 	zval **entry;
 
@@ -211,30 +200,25 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 	curobj = iterator->curobj;
 	intern = (dom_object *)zend_object_store_get_object(curobj TSRMLS_CC);
 	if (intern != NULL && intern->ptr != NULL) {
-		if (objmap->nodetype != XML_ENTITY_NODE && 
-			objmap->nodetype != XML_NOTATION_NODE) {
+		if (objmap->ht == NULL) {
 			if (objmap->nodetype == DOM_NODESET) {
 				nodeht = HASH_OF(objmap->baseobjptr);
 				zend_hash_move_forward(nodeht);
 				if (zend_hash_get_current_data(nodeht, (void **) &entry)==SUCCESS) {
 					curattr = *entry;
-					Z_ADDREF_P(curattr);
+					curattr->refcount++;
 				}
 			} else {
 				curnode = (xmlNodePtr)((php_libxml_node_ptr *)intern->ptr)->node;
-				if (objmap->nodetype == XML_ATTRIBUTE_NODE || 
-					objmap->nodetype == XML_ELEMENT_NODE) {
+				if (objmap->nodetype == XML_ATTRIBUTE_NODE || objmap->nodetype == XML_ELEMENT_NODE) {
 					curnode = curnode->next;
 				} else {
 					/* Nav the tree evey time as this is LIVE */
 					basenode = dom_object_get_node(objmap->baseobj);
-					if (basenode && (basenode->type == XML_DOCUMENT_NODE || 
-						basenode->type == XML_HTML_DOCUMENT_NODE)) {
+					if (basenode && (basenode->type == XML_DOCUMENT_NODE || basenode->type == XML_HTML_DOCUMENT_NODE)) {
 						basenode = xmlDocGetRootElement((xmlDoc *) basenode);
-					} else if (basenode) {
-						basenode = basenode->children;
 					} else {
-						goto err;
+						basenode = basenode->children;
 					}
 					curnode = dom_get_elements_by_tag_name_ns_raw(basenode, objmap->ns, objmap->local, &previndex, iter->index);
 				}
@@ -247,16 +231,15 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 			}
 		}
 	}
-err:
+
 	zval_ptr_dtor((zval**)&curobj);
 	if (curnode) {
 		MAKE_STD_ZVAL(curattr);
-		curattr = php_dom_create_object(curnode, &ret, curattr, objmap->baseobj TSRMLS_CC);
+		curattr = php_dom_create_object(curnode, &ret, NULL, curattr, objmap->baseobj TSRMLS_CC);
 	}
 
 	iterator->curobj = curattr;
 }
-/* }}} */
 
 zend_object_iterator_funcs php_dom_iterator_funcs = {
 	php_dom_iterator_dtor,
@@ -267,7 +250,7 @@ zend_object_iterator_funcs php_dom_iterator_funcs = {
 	NULL
 };
 
-zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC) /* {{{ */
+zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TSRMLS_DC)
 {
 	dom_object *intern;
 	dom_nnodemap_object *objmap;
@@ -276,34 +259,26 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object, i
 	int ret, curindex = 0;
 	HashTable *nodeht;
 	zval **entry;
-	php_dom_iterator *iterator;
 
-	if (by_ref) {
-		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
-	}
-	iterator = emalloc(sizeof(php_dom_iterator));
+	php_dom_iterator *iterator = emalloc(sizeof(php_dom_iterator));
 
-	Z_ADDREF_P(object);
+	object->refcount++;
 	iterator->intern.data = (void*)object;
 	iterator->intern.funcs = &php_dom_iterator_funcs;
 
 	intern = (dom_object *)zend_object_store_get_object(object TSRMLS_CC);
 	objmap = (dom_nnodemap_object *)intern->ptr;
 	if (objmap != NULL) {
-		if (objmap->nodetype != XML_ENTITY_NODE && 
-			objmap->nodetype != XML_NOTATION_NODE) {
+		if (objmap->ht == NULL) {
 			if (objmap->nodetype == DOM_NODESET) {
 				nodeht = HASH_OF(objmap->baseobjptr);
 				zend_hash_internal_pointer_reset(nodeht);
 				if (zend_hash_get_current_data(nodeht, (void **) &entry)==SUCCESS) {
 					curattr = *entry;
-					Z_ADDREF_P(curattr);
+					curattr->refcount++;
 				}
 			} else {
 				nodep = (xmlNode *)dom_object_get_node(objmap->baseobj);
-				if (!nodep) {
-					goto err;
-				}
 				if (objmap->nodetype == XML_ATTRIBUTE_NODE || objmap->nodetype == XML_ELEMENT_NODE) {
 					if (objmap->nodetype == XML_ATTRIBUTE_NODE) {
 						curnode = (xmlNodePtr) nodep->properties;
@@ -320,32 +295,18 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object, i
 				}
 			}
 		} else {
-			if (objmap->nodetype == XML_ENTITY_NODE) {
-				curnode = php_dom_libxml_hash_iter(objmap->ht, 0);
-			} else {
-				curnode = php_dom_libxml_notation_iter(objmap->ht, 0);
-			}
+			curnode = php_dom_libxml_hash_iter(objmap->ht, 0);
 		}
 	}
-err:
+
 	if (curnode) {
 		MAKE_STD_ZVAL(curattr);
-		curattr = php_dom_create_object(curnode, &ret, curattr, objmap->baseobj TSRMLS_CC);
+		curattr = php_dom_create_object(curnode, &ret, NULL, curattr, objmap->baseobj TSRMLS_CC);
 	}
 
 	iterator->curobj = curattr;
 
 	return (zend_object_iterator*)iterator;
 }
-/* }}} */
 
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
