@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: domimplementation.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id: domimplementation.c,v 1.15.2.1 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,28 +27,6 @@
 #if HAVE_LIBXML && HAVE_DOM
 #include "php_dom.h"
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_implementation_get_feature, 0, 0, 2)
-	ZEND_ARG_INFO(0, feature)
-	ZEND_ARG_INFO(0, version)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_implementation_has_feature, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_implementation_create_documenttype, 0, 0, 3)
-	ZEND_ARG_INFO(0, qualifiedName)
-	ZEND_ARG_INFO(0, publicId)
-	ZEND_ARG_INFO(0, systemId)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_implementation_create_document, 0, 0, 3)
-	ZEND_ARG_INFO(0, namespaceURI)
-	ZEND_ARG_INFO(0, qualifiedName)
-	ZEND_ARG_OBJ_INFO(0, docType, DOMDocumentType, 0)
-ZEND_END_ARG_INFO();
-/* }}} */
-
 /*
 * class DOMImplementation 
 *
@@ -56,12 +34,12 @@ ZEND_END_ARG_INFO();
 * Since: 
 */
 
-const zend_function_entry php_dom_domimplementation_class_functions[] = {
-	PHP_ME(domimplementation, getFeature, arginfo_dom_implementation_get_feature, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(domimplementation, hasFeature, arginfo_dom_implementation_has_feature, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(domimplementation, createDocumentType, arginfo_dom_implementation_create_documenttype, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(domimplementation, createDocument, arginfo_dom_implementation_create_document, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FE_END
+zend_function_entry php_dom_domimplementation_class_functions[] = {
+	PHP_ME(domimplementation, getFeature, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(domimplementation, hasFeature, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(domimplementation, createDocumentType, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(domimplementation, createDocument, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	{NULL, NULL, NULL}
 };
 
 /* {{{ proto boolean dom_domimplementation_has_feature(string feature, string version);
@@ -85,15 +63,17 @@ PHP_METHOD(domimplementation, hasFeature)
 }
 /* }}} end dom_domimplementation_has_feature */
 
+
 /* {{{ proto DOMDocumentType dom_domimplementation_create_document_type(string qualifiedName, string publicId, string systemId);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Level-2-Core-DOM-createDocType
 Since: DOM Level 2
 */
 PHP_METHOD(domimplementation, createDocumentType)
 {
+	zval *rv = NULL;
 	xmlDtd *doctype;
 	int ret, name_len = 0, publicid_len = 0, systemid_len = 0;
-	char *name = NULL, *publicid = NULL, *systemid = NULL;
+	char *name, *publicid, *systemid;
 	xmlChar *pch1 = NULL, *pch2 = NULL, *localname = NULL;
 	xmlURIPtr uri;
 
@@ -112,7 +92,7 @@ PHP_METHOD(domimplementation, createDocumentType)
 		pch2 = systemid;
 
 	uri = xmlParseURI(name);
-	if (uri != NULL && uri->opaque != NULL) {
+	if (uri->opaque != NULL) {
 		localname = xmlStrdup(uri->opaque);
 		if (xmlStrchr(localname, (xmlChar) ':') != NULL) {
 			php_dom_throw_error(NAMESPACE_ERR, 1 TSRMLS_CC);
@@ -128,35 +108,29 @@ PHP_METHOD(domimplementation, createDocumentType)
 	php_dom_throw_error(INVALID_CHARACTER_ERR, TSRMLS_CC);
 	*/
 
-	if (uri) {
-		xmlFreeURI(uri);
-	}
+	xmlFreeURI(uri);
 
 	doctype = xmlCreateIntSubset(NULL, localname, pch1, pch2);
 	xmlFree(localname);
 
-	if (doctype == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to create DocumentType");
-		RETURN_FALSE;
-	}
-
-	DOM_RET_OBJ((xmlNodePtr) doctype, &ret, NULL);
+	DOM_RET_OBJ(rv, (xmlNodePtr) doctype, &ret, NULL);
 }
 /* }}} end dom_domimplementation_create_document_type */
 
+
 /* {{{ proto DOMDocument dom_domimplementation_create_document(string namespaceURI, string qualifiedName, DOMDocumentType doctype);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Level-2-Core-DOM-createDocument
 Since: DOM Level 2
 */
 PHP_METHOD(domimplementation, createDocument)
 {
-	zval *node = NULL;
+	zval *node = NULL, *rv = NULL;
 	xmlDoc *docp;
 	xmlNode *nodep;
 	xmlDtdPtr doctype = NULL;
 	xmlNsPtr nsptr = NULL;
 	int ret, uri_len = 0, name_len = 0, errorcode = 0;
-	char *uri = NULL, *name = NULL;
+	char *uri, *name;
 	char *prefix = NULL, *localname = NULL;
 	dom_object *doctobj;
 
@@ -237,7 +211,7 @@ PHP_METHOD(domimplementation, createDocument)
 		xmlFree(localname);
 	}
 
-	DOM_RET_OBJ((xmlNodePtr) docp, &ret, NULL);
+	DOM_RET_OBJ(rv, (xmlNodePtr) docp, &ret, NULL);
 
 	if (doctobj != NULL) {
 		doctobj->document = ((dom_object *)((php_libxml_node_ptr *)docp->_private)->_private)->document;
@@ -246,6 +220,7 @@ PHP_METHOD(domimplementation, createDocument)
 }
 /* }}} end dom_domimplementation_create_document */
 
+
 /* {{{ proto DOMNode dom_domimplementation_get_feature(string feature, string version);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#DOMImplementation3-getFeature
 Since: DOM Level 3
@@ -255,14 +230,4 @@ PHP_METHOD(domimplementation, getFeature)
  DOM_NOT_IMPLEMENTED();
 }
 /* }}} end dom_domimplementation_get_feature */
-
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
