@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: element.c,v 1.30.2.4 2005/05/20 15:02:48 rrichards Exp $ */
+/* $Id: element.c,v 1.36.2.3 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -368,6 +368,11 @@ PHP_FUNCTION(dom_element_set_attribute_node)
 		RETURN_FALSE;
 	}
 
+	if (!(attrp->doc == NULL || attrp->doc == nodep->doc)) {
+		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		RETURN_FALSE;
+	}
+
 	existattrp = xmlHasProp(nodep, attrp->name);
 	if (existattrp != NULL && existattrp->type != XML_ATTRIBUTE_DECL) {
 		if ((oldobj = php_dom_object_get_data((xmlNodePtr) existattrp)) != NULL && 
@@ -501,6 +506,42 @@ PHP_FUNCTION(dom_element_get_attribute_ns)
 }
 /* }}} end dom_element_get_attribute_ns */
 
+static xmlNsPtr _dom_new_reconNs(xmlDocPtr doc, xmlNodePtr tree, xmlNsPtr ns) {
+    xmlNsPtr def;
+    xmlChar prefix[50];
+    int counter = 1;
+
+	if ((tree == NULL) || (ns == NULL) || (ns->type != XML_NAMESPACE_DECL)) {
+		return NULL;
+	}
+
+	/* Code taken from libxml2 (2.6.20) xmlNewReconciliedNs
+	 *
+	 * Find a close prefix which is not already in use.
+	 * Let's strip namespace prefixes longer than 20 chars !
+	 */
+	if (ns->prefix == NULL)
+		snprintf((char *) prefix, sizeof(prefix), "default");
+	else
+		snprintf((char *) prefix, sizeof(prefix), "%.20s", (char *)ns->prefix);
+
+	def = xmlSearchNs(doc, tree, prefix);
+	while (def != NULL) {
+		if (counter > 1000) return(NULL);
+		if (ns->prefix == NULL)
+			snprintf((char *) prefix, sizeof(prefix), "default%d", counter++);
+		else
+			snprintf((char *) prefix, sizeof(prefix), "%.20s%d", 
+			(char *)ns->prefix, counter++);
+		def = xmlSearchNs(doc, tree, prefix);
+	}
+
+	/*
+	 * OK, now we are ready to create a new one.
+	 */
+	def = xmlNewNs(tree, ns->href, prefix);
+	return(def);
+}
 
 /* {{{ proto void dom_element_set_attribute_ns(string namespaceURI, string qualifiedName, string value);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-ElSetAttrNS
@@ -550,8 +591,21 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 				nsptr = dom_get_nsdecl(elemp, localname);
 			} else {
 				nsptr = xmlSearchNsByHref(elemp->doc, elemp, uri);
-				while (nsptr && nsptr->prefix == NULL) {
-					nsptr = nsptr->next;
+				if (nsptr && nsptr->prefix == NULL) {
+					xmlNsPtr tmpnsptr;
+
+					tmpnsptr = nsptr->next;
+					while (tmpnsptr) {
+						if ((tmpnsptr->prefix != NULL) && (tmpnsptr->href != NULL) && 
+							(xmlStrEqual(tmpnsptr->href, (xmlChar *) uri))) {
+							nsptr = tmpnsptr;
+							break;
+						}
+						tmpnsptr = tmpnsptr->next;
+					}
+					if (tmpnsptr == NULL) {
+						nsptr = _dom_new_reconNs(elemp->doc, elemp, nsptr);
+					}
 				}
 			}
 
@@ -671,7 +725,7 @@ PHP_FUNCTION(dom_element_get_attribute_node_ns)
 	int uri_len, name_len, ret;
 	char *uri, *name;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oss", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -720,6 +774,11 @@ PHP_FUNCTION(dom_element_set_attribute_node_ns)
 		RETURN_FALSE;
 	}
 
+	if (!(attrp->doc == NULL || attrp->doc == nodep->doc)) {
+		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		RETURN_FALSE;
+	}
+
     nsp = attrp->ns;
     if (nsp != NULL) {
         existattrp = xmlHasNsProp(nodep, nsp->href, attrp->name);
