@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: namednodemap.c,v 1.15.2.1 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,37 +27,6 @@
 #if HAVE_LIBXML && HAVE_DOM
 #include "php_dom.h"
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_get_named_item, 0, 0, 1)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_set_named_item, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, arg, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_remove_named_item, 0, 0, 0)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_item, 0, 0, 0)
-	ZEND_ARG_INFO(0, index)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_get_named_item_ns, 0, 0, 0)
-	ZEND_ARG_INFO(0, namespaceURI)
-	ZEND_ARG_INFO(0, localName)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_set_named_item_ns, 0, 0, 0)
-	ZEND_ARG_OBJ_INFO(0, arg, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_namednodemap_remove_named_item_ns, 0, 0, 0)
-	ZEND_ARG_INFO(0, namespaceURI)
-	ZEND_ARG_INFO(0, localName)
-ZEND_END_ARG_INFO();
-/* }}} */
 
 /*
 * class DOMNamedNodeMap 
@@ -66,17 +35,16 @@ ZEND_END_ARG_INFO();
 * Since: 
 */
 
-const zend_function_entry php_dom_namednodemap_class_functions[] = { /* {{{ */
-	PHP_FALIAS(getNamedItem, dom_namednodemap_get_named_item, arginfo_dom_namednodemap_get_named_item)
-	PHP_FALIAS(setNamedItem, dom_namednodemap_set_named_item, arginfo_dom_namednodemap_set_named_item)
-	PHP_FALIAS(removeNamedItem, dom_namednodemap_remove_named_item, arginfo_dom_namednodemap_remove_named_item)
-	PHP_FALIAS(item, dom_namednodemap_item, arginfo_dom_namednodemap_item)
-	PHP_FALIAS(getNamedItemNS, dom_namednodemap_get_named_item_ns, arginfo_dom_namednodemap_get_named_item_ns)
-	PHP_FALIAS(setNamedItemNS, dom_namednodemap_set_named_item_ns, arginfo_dom_namednodemap_set_named_item_ns)
-	PHP_FALIAS(removeNamedItemNS, dom_namednodemap_remove_named_item_ns, arginfo_dom_namednodemap_remove_named_item_ns)
-	PHP_FE_END
+zend_function_entry php_dom_namednodemap_class_functions[] = {
+	PHP_FALIAS(getNamedItem, dom_namednodemap_get_named_item, NULL)
+	PHP_FALIAS(setNamedItem, dom_namednodemap_set_named_item, NULL)
+	PHP_FALIAS(removeNamedItem, dom_namednodemap_remove_named_item, NULL)
+	PHP_FALIAS(item, dom_namednodemap_item, NULL)
+	PHP_FALIAS(getNamedItemNS, dom_namednodemap_get_named_item_ns, NULL)
+	PHP_FALIAS(setNamedItemNS, dom_namednodemap_set_named_item_ns, NULL)
+	PHP_FALIAS(removeNamedItemNS, dom_namednodemap_remove_named_item_ns, NULL)
+	{NULL, NULL, NULL}
 };
-/* }}} */
 
 /* {{{ length	int	
 readonly=yes 
@@ -93,11 +61,8 @@ int dom_namednodemap_length_read(dom_object *obj, zval **retval TSRMLS_DC)
 	objmap = (dom_nnodemap_object *)obj->ptr;
 
 	if (objmap != NULL) {
-		if ((objmap->nodetype == XML_NOTATION_NODE) || 
-			objmap->nodetype == XML_ENTITY_NODE) {
-			if (objmap->ht) {
-				count = xmlHashSize(objmap->ht);
-			}
+		if (objmap->ht) {
+			count = xmlHashSize(objmap->ht);
 		} else {
 			nodep = dom_object_get_node(objmap->baseobj);
 			if (nodep) {
@@ -120,13 +85,16 @@ int dom_namednodemap_length_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
+
 /* {{{ proto DOMNode dom_namednodemap_get_named_item(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1074577549
 Since: 
 */
 PHP_FUNCTION(dom_namednodemap_get_named_item)
 {
-	zval *id;
+	zval *id, *rv = NULL;
 	int ret, namedlen=0;
 	dom_object *intern;
 	xmlNodePtr itemnode = NULL;
@@ -145,17 +113,12 @@ PHP_FUNCTION(dom_namednodemap_get_named_item)
 	objmap = (dom_nnodemap_object *)intern->ptr;
 
 	if (objmap != NULL) {
-		if ((objmap->nodetype == XML_NOTATION_NODE) || 
-			objmap->nodetype == XML_ENTITY_NODE) {
-			if (objmap->ht) {
-				if (objmap->nodetype == XML_ENTITY_NODE) {
-					itemnode = (xmlNodePtr)xmlHashLookup(objmap->ht, named);
-				} else {
-					notep = (xmlNotation *)xmlHashLookup(objmap->ht, named);
-					if (notep) {
-						itemnode = create_notation(notep->name, notep->PublicID, notep->SystemID);
-					}
-				}
+		if (objmap->ht) {
+			if (objmap->nodetype == XML_ENTITY_NODE) {
+				itemnode = (xmlNodePtr)xmlHashLookup(objmap->ht, named);
+			} else {
+				notep = (xmlNotation *)xmlHashLookup(objmap->ht, named);
+				itemnode = create_notation(notep->name, notep->PublicID, notep->SystemID);
 			}
 		} else {
 			nodep = dom_object_get_node(objmap->baseobj);
@@ -166,7 +129,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item)
 	}
 
 	if (itemnode) {
-		DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
+		DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
 		return;
 	} else {
 		RETVAL_NULL();
@@ -174,6 +137,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item)
 }
 /* }}} end dom_namednodemap_get_named_item */
 
+
 /* {{{ proto DOMNode dom_namednodemap_set_named_item(DOMNode arg);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1025163788
 Since: 
@@ -184,6 +148,7 @@ PHP_FUNCTION(dom_namednodemap_set_named_item)
 }
 /* }}} end dom_namednodemap_set_named_item */
 
+
 /* {{{ proto DOMNode dom_namednodemap_remove_named_item(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-D58B193
 Since: 
@@ -194,13 +159,14 @@ PHP_FUNCTION(dom_namednodemap_remove_named_item)
 }
 /* }}} end dom_namednodemap_remove_named_item */
 
+
 /* {{{ proto DOMNode dom_namednodemap_item(int index);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-349467F9
 Since: 
 */
 PHP_FUNCTION(dom_namednodemap_item)
 {
-	zval *id;
+	zval *id, *rv = NULL;
 	long index;
 	int ret;
 	dom_object *intern;
@@ -219,14 +185,11 @@ PHP_FUNCTION(dom_namednodemap_item)
 		objmap = (dom_nnodemap_object *)intern->ptr;
 
 		if (objmap != NULL) {
-			if ((objmap->nodetype == XML_NOTATION_NODE) || 
-				objmap->nodetype == XML_ENTITY_NODE) {
-				if (objmap->ht) {
-					if (objmap->nodetype == XML_ENTITY_NODE) {
-						itemnode = php_dom_libxml_hash_iter(objmap->ht, index);
-					} else {
-						itemnode = php_dom_libxml_notation_iter(objmap->ht, index);
-					}
+			if (objmap->ht) {
+				if (objmap->nodetype == XML_ENTITY_NODE) {
+					itemnode = php_dom_libxml_hash_iter(objmap->ht, index);
+				} else {
+					itemnode = php_dom_libxml_notation_iter(objmap->ht, index);
 				}
 			} else {
 				nodep = dom_object_get_node(objmap->baseobj);
@@ -243,7 +206,7 @@ PHP_FUNCTION(dom_namednodemap_item)
 		}
 
 		if (itemnode) {
-			DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
+			DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
 			return;
 		}
 	}
@@ -252,13 +215,14 @@ PHP_FUNCTION(dom_namednodemap_item)
 }
 /* }}} end dom_namednodemap_item */
 
+
 /* {{{ proto DOMNode dom_namednodemap_get_named_item_ns(string namespaceURI, string localName);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-getNamedItemNS
 Since: DOM Level 2
 */
 PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 {
-	zval *id;
+	zval *id, *rv = NULL;
 	int ret, namedlen=0, urilen=0;
 	dom_object *intern;
 	xmlNodePtr itemnode = NULL;
@@ -277,17 +241,12 @@ PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 	objmap = (dom_nnodemap_object *)intern->ptr;
 
 	if (objmap != NULL) {
-		if ((objmap->nodetype == XML_NOTATION_NODE) || 
-			objmap->nodetype == XML_ENTITY_NODE) {
-			if (objmap->ht) {
-				if (objmap->nodetype == XML_ENTITY_NODE) {
-					itemnode = (xmlNodePtr)xmlHashLookup(objmap->ht, named);
-				} else {
-					notep = (xmlNotation *)xmlHashLookup(objmap->ht, named);
-					if (notep) {
-						itemnode = create_notation(notep->name, notep->PublicID, notep->SystemID);
-					}
-				}
+		if (objmap->ht) {
+			if (objmap->nodetype == XML_ENTITY_NODE) {
+				itemnode = (xmlNodePtr)xmlHashLookup(objmap->ht, named);
+			} else {
+				notep = (xmlNotation *)xmlHashLookup(objmap->ht, named);
+				itemnode = create_notation(notep->name, notep->PublicID, notep->SystemID);
 			}
 		} else {
 			nodep = dom_object_get_node(objmap->baseobj);
@@ -298,7 +257,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 	}
 
 	if (itemnode) {
-		DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
+		DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
 		return;
 	} else {
 		RETVAL_NULL();
@@ -306,6 +265,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 }
 /* }}} end dom_namednodemap_get_named_item_ns */
 
+
 /* {{{ proto DOMNode dom_namednodemap_set_named_item_ns(DOMNode arg);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-setNamedItemNS
 Since: DOM Level 2
@@ -316,6 +276,7 @@ PHP_FUNCTION(dom_namednodemap_set_named_item_ns)
 }
 /* }}} end dom_namednodemap_set_named_item_ns */
 
+
 /* {{{ proto DOMNode dom_namednodemap_remove_named_item_ns(string namespaceURI, string localName);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-removeNamedItemNS
 Since: DOM Level 2
@@ -325,14 +286,4 @@ PHP_FUNCTION(dom_namednodemap_remove_named_item_ns)
  DOM_NOT_IMPLEMENTED();
 }
 /* }}} end dom_namednodemap_remove_named_item_ns */
-
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
