@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: node.c,v 1.37.2.3.2.8.2.14 2009/03/13 13:43:29 rrichards Exp $ */
+/* $Id: node.c,v 1.37.2.2 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,104 +27,6 @@
 #if HAVE_LIBXML && HAVE_DOM
 #include "php_dom.h"
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_insert_before, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
-	ZEND_ARG_OBJ_INFO(0, refChild, DOMNode, 1)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_replace_child, 0, 0, 2)
-	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
-	ZEND_ARG_OBJ_INFO(0, oldChild, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_remove_child, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, oldChild, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_append_child, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_has_child_nodes, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_clone_node, 0, 0, 1)
-	ZEND_ARG_INFO(0, deep)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_normalize, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_supported, 0, 0, 2)
-	ZEND_ARG_INFO(0, feature)
-	ZEND_ARG_INFO(0, version)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_has_attributes, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_compare_document_position, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, other, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_same_node, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, other, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_lookup_prefix, 0, 0, 1)
-	ZEND_ARG_INFO(0, namespaceURI)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_default_namespace, 0, 0, 1)
-	ZEND_ARG_INFO(0, namespaceURI)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_lookup_namespace_uri, 0, 0, 1)
-	ZEND_ARG_INFO(0, prefix)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_equal_node, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, arg, DOMNode, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_get_feature, 0, 0, 2)
-	ZEND_ARG_INFO(0, feature)
-	ZEND_ARG_INFO(0, version)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_set_user_data, 0, 0, 3)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, data)
-	ZEND_ARG_INFO(0, handler)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_get_user_data, 0, 0, 1)
-	ZEND_ARG_INFO(0, key)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_getNodePath, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_getLineNo, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_C14N, 0, 0, 0)
-	ZEND_ARG_INFO(0, exclusive)
-	ZEND_ARG_INFO(0, with_comments)
-	ZEND_ARG_ARRAY_INFO(0, xpath, 1)
-	ZEND_ARG_ARRAY_INFO(0, ns_prefixes, 1)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_C14NFile, 0, 0, 1)
-	ZEND_ARG_INFO(0, uri)
-	ZEND_ARG_INFO(0, exclusive)
-	ZEND_ARG_INFO(0, with_comments)
-	ZEND_ARG_ARRAY_INFO(0, xpath, 1)
-	ZEND_ARG_ARRAY_INFO(0, ns_prefixes, 1)
-ZEND_END_ARG_INFO();
-/* }}} */
-
 /*
 * class DOMNode 
 *
@@ -132,64 +34,43 @@ ZEND_END_ARG_INFO();
 * Since: 
 */
 
-const zend_function_entry php_dom_node_class_functions[] = { /* {{{ */
-	PHP_FALIAS(insertBefore, dom_node_insert_before, arginfo_dom_node_insert_before)
-	PHP_FALIAS(replaceChild, dom_node_replace_child, arginfo_dom_node_replace_child)
-	PHP_FALIAS(removeChild, dom_node_remove_child, arginfo_dom_node_remove_child)
-	PHP_FALIAS(appendChild, dom_node_append_child, arginfo_dom_node_append_child)
-	PHP_FALIAS(hasChildNodes, dom_node_has_child_nodes, arginfo_dom_node_has_child_nodes)
-	PHP_FALIAS(cloneNode, dom_node_clone_node, arginfo_dom_node_clone_node)
-	PHP_FALIAS(normalize, dom_node_normalize, arginfo_dom_node_normalize)
-	PHP_FALIAS(isSupported, dom_node_is_supported, arginfo_dom_node_is_supported)
-	PHP_FALIAS(hasAttributes, dom_node_has_attributes, arginfo_dom_node_has_attributes)
-	PHP_FALIAS(compareDocumentPosition, dom_node_compare_document_position, arginfo_dom_node_compare_document_position)
-	PHP_FALIAS(isSameNode, dom_node_is_same_node, arginfo_dom_node_is_same_node)
-	PHP_FALIAS(lookupPrefix, dom_node_lookup_prefix, arginfo_dom_node_lookup_prefix)
-	PHP_FALIAS(isDefaultNamespace, dom_node_is_default_namespace, arginfo_dom_node_is_default_namespace)
-	PHP_FALIAS(lookupNamespaceUri, dom_node_lookup_namespace_uri, arginfo_dom_node_lookup_namespace_uri)
-	PHP_FALIAS(isEqualNode, dom_node_is_equal_node, arginfo_dom_node_is_equal_node)
-	PHP_FALIAS(getFeature, dom_node_get_feature, arginfo_dom_node_get_feature)
-	PHP_FALIAS(setUserData, dom_node_set_user_data, arginfo_dom_node_set_user_data)
-	PHP_FALIAS(getUserData, dom_node_get_user_data, arginfo_dom_node_get_user_data)
-	PHP_ME(domnode, getNodePath, arginfo_dom_node_getNodePath, ZEND_ACC_PUBLIC)
-	PHP_ME(domnode, getLineNo, arginfo_dom_node_getLineNo, ZEND_ACC_PUBLIC)
-	PHP_ME(domnode, C14N, arginfo_dom_node_C14N, ZEND_ACC_PUBLIC)
-	PHP_ME(domnode, C14NFile, arginfo_dom_node_C14NFile, ZEND_ACC_PUBLIC)
+zend_function_entry php_dom_node_class_functions[] = {
+	PHP_FALIAS(insertBefore, dom_node_insert_before, NULL)
+	PHP_FALIAS(replaceChild, dom_node_replace_child, NULL)
+	PHP_FALIAS(removeChild, dom_node_remove_child, NULL)
+	PHP_FALIAS(appendChild, dom_node_append_child, NULL)
+	PHP_FALIAS(hasChildNodes, dom_node_has_child_nodes, NULL)
+	PHP_FALIAS(cloneNode, dom_node_clone_node, NULL)
+	PHP_FALIAS(normalize, dom_node_normalize, NULL)
+	PHP_FALIAS(isSupported, dom_node_is_supported, NULL)
+	PHP_FALIAS(hasAttributes, dom_node_has_attributes, NULL)
+	PHP_FALIAS(compareDocumentPosition, dom_node_compare_document_position, NULL)
+	PHP_FALIAS(isSameNode, dom_node_is_same_node, NULL)
+	PHP_FALIAS(lookupPrefix, dom_node_lookup_prefix, NULL)
+	PHP_FALIAS(isDefaultNamespace, dom_node_is_default_namespace, NULL)
+	PHP_FALIAS(lookupNamespaceUri, dom_node_lookup_namespace_uri, NULL)
+	PHP_FALIAS(isEqualNode, dom_node_is_equal_node, NULL)
+	PHP_FALIAS(getFeature, dom_node_get_feature, NULL)
+	PHP_FALIAS(setUserData, dom_node_set_user_data, NULL)
+	PHP_FALIAS(getUserData, dom_node_get_user_data, NULL)
 	{NULL, NULL, NULL}
 };
-/* }}} */
 
-static void dom_reconcile_ns(xmlDocPtr doc, xmlNodePtr nodep) /* {{{ */
-{
-	xmlNsPtr nsptr, nsdftptr, curns, prevns = NULL;
+static void dom_reconcile_ns(xmlDocPtr doc, xmlNodePtr nodep) {
+	xmlNsPtr nsptr;
 
 	if (nodep->type == XML_ELEMENT_NODE) {
 		/* Following if block primarily used for inserting nodes created via createElementNS */
-		if (nodep->nsDef != NULL) {
-			curns = nodep->nsDef;
-			while (curns) {
-				nsdftptr = curns->next;
-				if (curns->href != NULL) {
-					if((nsptr = xmlSearchNsByHref(doc, nodep->parent, curns->href)) && 
-						(curns->prefix == NULL || xmlStrEqual(nsptr->prefix, curns->prefix))) {
-						curns->next = NULL;
-						if (prevns == NULL) {
-							nodep->nsDef = nsdftptr;
-						} else {
-							prevns->next = nsdftptr;
-						}
-						dom_set_old_ns(doc, curns);
-						curns = prevns;
-					}
-				}
-				prevns = curns;
-				curns = nsdftptr;
+		if (nodep->nsDef != NULL && nodep->nsDef->href != NULL) {
+			if((nsptr = xmlSearchNsByHref(doc, nodep->parent, nodep->nsDef->href)) && 
+				(nodep->nsDef->prefix == NULL || xmlStrEqual(nsptr->prefix, nodep->nsDef->prefix))) {
+				dom_set_old_ns(doc, nodep->nsDef);
+				nodep->nsDef = NULL;
 			}
 		}
 		xmlReconciliateNs(doc, nodep);
 	}
 }
-/* }}} */
 
 /* {{{ nodeName	string	
 readonly=yes 
@@ -260,6 +141,7 @@ int dom_node_node_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 			break;
 		default:
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid Node Type");
+			return FAILURE;
 	}
 
 	ALLOC_ZVAL(*retval);
@@ -280,6 +162,8 @@ int dom_node_node_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ nodeValue	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-F68D080
@@ -353,7 +237,7 @@ int dom_node_node_value_write(dom_object *obj, zval *newval TSRMLS_DC)
 		case XML_CDATA_SECTION_NODE:
 		case XML_PI_NODE:
 			if (newval->type != IS_STRING) {
-				if(Z_REFCOUNT_P(newval) > 1) {
+				if(newval->refcount > 1) {
 					value_copy = *newval;
 					zval_copy_ctor(&value_copy);
 					newval = &value_copy;
@@ -374,6 +258,8 @@ int dom_node_node_value_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ nodeType	int
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-111237558
@@ -404,6 +290,8 @@ int dom_node_node_type_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ parentNode	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1060184317
@@ -421,14 +309,13 @@ int dom_node_parent_node_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	nodeparent = nodep->parent;
 	if (!nodeparent) {
-		ZVAL_NULL(*retval);
-		return SUCCESS;
+		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	if (NULL == (*retval = php_dom_create_object(nodeparent, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -438,6 +325,8 @@ int dom_node_parent_node_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ childNodes	DomNodeList	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1451460987
@@ -470,6 +359,8 @@ int dom_node_child_nodes_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ firstChild DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-169727388
@@ -491,13 +382,12 @@ int dom_node_first_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 		first = nodep->children;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	if (!first) {
-		ZVAL_NULL(*retval);
-		return SUCCESS;
+		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	if (NULL == (*retval = php_dom_create_object(first, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -507,6 +397,8 @@ int dom_node_first_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ lastChild	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-61AD09FB
@@ -528,13 +420,12 @@ int dom_node_last_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 		last = nodep->last;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	if (!last) {
-		ZVAL_NULL(*retval);
-		return SUCCESS;
+		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	if (NULL == (*retval = php_dom_create_object(last, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -544,6 +435,8 @@ int dom_node_last_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ previousSibling	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-640FB3C8
@@ -561,14 +454,13 @@ int dom_node_previous_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	prevsib = nodep->prev;
 	if (!prevsib) {
-		ZVAL_NULL(*retval);
-		return SUCCESS;
+		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	if (NULL == (*retval = php_dom_create_object(prevsib, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -578,6 +470,8 @@ int dom_node_previous_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ nextSibling	DomNode
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-6AC54C2F
@@ -611,6 +505,8 @@ int dom_node_next_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ attributes	DomNamedNodeMap	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-84CF096
@@ -643,6 +539,8 @@ int dom_node_attributes_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ ownerDocument	DomDocument	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-node-ownerDoc
@@ -683,6 +581,8 @@ int dom_node_owner_document_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ namespaceUri	string	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSname
@@ -726,6 +626,8 @@ int dom_node_namespace_uri_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ prefix	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSPrefix
@@ -795,7 +697,7 @@ int dom_node_prefix_write(dom_object *obj, zval *newval TSRMLS_DC)
 				}
 			}
 			if (newval->type != IS_STRING) {
-				if(Z_REFCOUNT_P(newval) > 1) {
+				if(newval->refcount > 1) {
 					value_copy = *newval;
 					zval_copy_ctor(&value_copy);
 					newval = &value_copy;
@@ -848,6 +750,8 @@ int dom_node_prefix_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ localName	string	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSLocalN
@@ -877,6 +781,8 @@ int dom_node_local_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ baseURI	string
 readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-baseURI
@@ -909,6 +815,8 @@ int dom_node_base_uri_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
+
+
 /* {{{ textContent	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-textContent
@@ -932,10 +840,10 @@ int dom_node_text_content_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 	if(str != NULL) {
 		ZVAL_STRING(*retval, str, 1);
-		xmlFree(str);
 	} else {
 		ZVAL_EMPTY_STRING(*retval);
 	}
+	xmlFree(str);
 
 	return SUCCESS;
 }
@@ -947,7 +855,10 @@ int dom_node_text_content_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib, xmlNodePtr nextsib, xmlNodePtr fragment, dom_object *intern, dom_object *childobj TSRMLS_DC) /* {{{ */
+
+static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib,
+					xmlNodePtr nextsib, xmlNodePtr fragment, 
+					dom_object *intern, dom_object *childobj TSRMLS_DC)
 {
 	xmlNodePtr newchild, node;
 
@@ -990,7 +901,6 @@ static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib,
 
 	return newchild;
 }
-/* }}} */
 
 /* {{{ proto domnode dom_node_insert_before(DomNode newChild, DomNode refChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-952280727
@@ -1099,7 +1009,7 @@ PHP_FUNCTION(dom_node_insert_before)
 			new_child = xmlAddPrevSibling(refp, child);
 		}
 	} else {
-		if (child->parent != NULL){
+		if (child->parent == parentp){
 			xmlUnlinkNode(child);
 		}
 		if (child->type == XML_TEXT_NODE && parentp->last != NULL && parentp->last->type == XML_TEXT_NODE) {
@@ -1153,6 +1063,7 @@ PHP_FUNCTION(dom_node_insert_before)
 }
 /* }}} end dom_node_insert_before */
 
+
 /* {{{ proto DomNode dom_node_replace_child(DomNode newChild, DomNode oldChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-785887307
 Since: 
@@ -1244,6 +1155,7 @@ PHP_FUNCTION(dom_node_replace_child)
 }
 /* }}} end dom_node_replace_child */
 
+
 /* {{{ proto DomNode dom_node_remove_child(DomNode oldChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1734834066
 Since: 
@@ -1296,6 +1208,7 @@ PHP_FUNCTION(dom_node_remove_child)
 }
 /* }}} end dom_node_remove_child */
 
+
 /* {{{ proto DomNode dom_node_append_child(DomNode newChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-184E7107
 Since: 
@@ -1397,6 +1310,7 @@ PHP_FUNCTION(dom_node_append_child)
 }
 /* }}} end dom_node_append_child */
 
+
 /* {{{ proto boolean dom_node_has_child_nodes();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-810594187
 Since: 
@@ -1425,6 +1339,7 @@ PHP_FUNCTION(dom_node_has_child_nodes)
 }
 /* }}} end dom_node_has_child_nodes */
 
+
 /* {{{ proto DomNode dom_node_clone_node(boolean deep);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-3A0ED0A4
 Since: 
@@ -1487,6 +1402,8 @@ PHP_FUNCTION(dom_node_clone_node)
 }
 /* }}} end dom_node_clone_node */
 
+
+
 /* {{{ proto void dom_node_normalize();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-normalize
 Since: 
@@ -1508,6 +1425,7 @@ PHP_FUNCTION(dom_node_normalize)
 }
 /* }}} end dom_node_normalize */
 
+
 /* {{{ proto boolean dom_node_is_supported(string feature, string version);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Level-2-Core-Node-supports
 Since: DOM Level 2
@@ -1530,6 +1448,7 @@ PHP_FUNCTION(dom_node_is_supported)
 }
 /* }}} end dom_node_is_supported */
 
+
 /* {{{ proto boolean dom_node_has_attributes();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeHasAttrs
 Since: DOM Level 2
@@ -1567,6 +1486,7 @@ PHP_FUNCTION(dom_node_compare_document_position)
 }
 /* }}} end dom_node_compare_document_position */
 
+
 /* {{{ proto boolean dom_node_is_same_node(DomNode other);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-isSameNode
 Since: DOM Level 3
@@ -1593,6 +1513,7 @@ PHP_FUNCTION(dom_node_is_same_node)
 }
 /* }}} end dom_node_is_same_node */
 
+
 /* {{{ proto string dom_node_lookup_prefix(string namespaceURI);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-lookupNamespacePrefix
 Since: DOM Level 3
@@ -1643,6 +1564,7 @@ PHP_FUNCTION(dom_node_lookup_prefix)
 }
 /* }}} end dom_node_lookup_prefix */
 
+
 /* {{{ proto boolean dom_node_is_default_namespace(string namespaceURI);
 URL: http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-isDefaultNamespace
 Since: DOM Level 3
@@ -1661,11 +1583,8 @@ PHP_FUNCTION(dom_node_is_default_namespace)
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
-	if (nodep->type == XML_DOCUMENT_NODE || nodep->type == XML_HTML_DOCUMENT_NODE) {
-		nodep = xmlDocGetRootElement((xmlDocPtr) nodep);
-	}
 
-	if (nodep && uri_len > 0) {
+	if (uri_len > 0) {
 		nsptr = xmlSearchNs(nodep->doc, nodep, NULL);
 		if (nsptr && xmlStrEqual(nsptr->href, uri)) {
 			RETURN_TRUE;
@@ -1676,6 +1595,7 @@ PHP_FUNCTION(dom_node_is_default_namespace)
 }
 /* }}} end dom_node_is_default_namespace */
 
+
 /* {{{ proto string dom_node_lookup_namespace_uri(string prefix);
 URL: http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespaceURI
 Since: DOM Level 3
@@ -1687,29 +1607,26 @@ PHP_FUNCTION(dom_node_lookup_namespace_uri)
 	dom_object *intern;
 	xmlNsPtr nsptr;
 	int prefix_len = 0;
-	char *prefix=NULL;
+	char *prefix;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!", &id, dom_node_class_entry, &prefix, &prefix_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_node_class_entry, &prefix, &prefix_len) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
-	if (nodep->type == XML_DOCUMENT_NODE || nodep->type == XML_HTML_DOCUMENT_NODE) {
-		nodep = xmlDocGetRootElement((xmlDocPtr) nodep);
-		if (nodep == NULL) {
-			RETURN_NULL();
-		}
-	}
 
-	nsptr = xmlSearchNs(nodep->doc, nodep, prefix);
-	if (nsptr && nsptr->href != NULL) {
-		RETURN_STRING((char *) nsptr->href, 1);
+	if (prefix_len > 0) {
+		nsptr = xmlSearchNs(nodep->doc, nodep, prefix);
+		if (nsptr && nsptr->href != NULL) {
+			RETURN_STRING((char *) nsptr->href, 1);
+		}
 	}
 
 	RETURN_NULL();
 }
 /* }}} end dom_node_lookup_namespace_uri */
 
+
 /* {{{ proto boolean dom_node_is_equal_node(DomNode arg);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-isEqualNode
 Since: DOM Level 3
@@ -1720,6 +1637,7 @@ PHP_FUNCTION(dom_node_is_equal_node)
 }
 /* }}} end dom_node_is_equal_node */
 
+
 /* {{{ proto DomNode dom_node_get_feature(string feature, string version);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-getFeature
 Since: DOM Level 3
@@ -1730,6 +1648,7 @@ PHP_FUNCTION(dom_node_get_feature)
 }
 /* }}} end dom_node_get_feature */
 
+
 /* {{{ proto DomUserData dom_node_set_user_data(string key, DomUserData data, userdatahandler handler);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-setUserData
 Since: DOM Level 3
@@ -1740,6 +1659,7 @@ PHP_FUNCTION(dom_node_set_user_data)
 }
 /* }}} end dom_node_set_user_data */
 
+
 /* {{{ proto DomUserData dom_node_get_user_data(string key);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-getUserData
 Since: DOM Level 3
@@ -1749,239 +1669,4 @@ PHP_FUNCTION(dom_node_get_user_data)
  DOM_NOT_IMPLEMENTED();
 }
 /* }}} end dom_node_get_user_data */
-
-static void dom_canonicalization(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
-{
-	zval *id;
-	zval *xpath_array=NULL, *ns_prefixes=NULL;
-	xmlNodePtr nodep;
-	xmlDocPtr docp;
-	xmlNodeSetPtr nodeset = NULL;
-	dom_object *intern;
-	zend_bool exclusive=0, with_comments=0;
-	xmlChar **inclusive_ns_prefixes = NULL;
-	char *file = NULL;
-	int ret = -1, file_len = 0;
-	xmlOutputBufferPtr buf;
-	xmlXPathContextPtr ctxp=NULL;
-	xmlXPathObjectPtr xpathobjp=NULL;
-
-	if (mode == 0) {
-		if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), 
-			"O|bba!a!", &id, dom_node_class_entry, &exclusive, &with_comments, 
-			&xpath_array, &ns_prefixes) == FAILURE) {
-			return;
-		}
-	} else {
-		if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), 
-			"Os|bba!a!", &id, dom_node_class_entry, &file, &file_len, &exclusive, 
-			&with_comments, &xpath_array, &ns_prefixes) == FAILURE) {
-			return;
-		}
-	}
-
-	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
-
-	docp = nodep->doc;
-
-	if (! docp) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Node must be associated with a document");
-		RETURN_FALSE;
-	}
-
-	if (xpath_array == NULL) {
-		if (nodep->type != XML_DOCUMENT_NODE) {
-			ctxp = xmlXPathNewContext(docp);
-			ctxp->node = nodep;
-			xpathobjp = xmlXPathEvalExpression("(.//. | .//@* | .//namespace::*)", ctxp);
-			ctxp->node = NULL;
-			if (xpathobjp && xpathobjp->type == XPATH_NODESET) {
-				nodeset = xpathobjp->nodesetval;
-			} else {
-				if (xpathobjp) {
-					xmlXPathFreeObject(xpathobjp);
-				}
-				xmlXPathFreeContext(ctxp);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "XPath query did not return a nodeset.");
-				RETURN_FALSE;
-			}
-		}
-	} else {
-		/*xpath query from xpath_array */
-		HashTable *ht = Z_ARRVAL_P(xpath_array);
-		zval **tmp;
-		char *xquery;
-
-		if (zend_hash_find(ht, "query", sizeof("query"), (void**)&tmp) == SUCCESS &&
-		    Z_TYPE_PP(tmp) == IS_STRING) {
-			xquery = Z_STRVAL_PP(tmp);
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "'query' missing from xpath array or is not a string");
-			RETURN_FALSE;
-		}
-
-		ctxp = xmlXPathNewContext(docp);
-		ctxp->node = nodep;
-
-		if (zend_hash_find(ht, "namespaces", sizeof("namespaces"), (void**)&tmp) == SUCCESS &&
-		    Z_TYPE_PP(tmp) == IS_ARRAY) {
-			zval **tmpns;
-			while (zend_hash_get_current_data(Z_ARRVAL_PP(tmp), (void **)&tmpns) == SUCCESS) {
-				if (Z_TYPE_PP(tmpns) == IS_STRING) {
-					char *prefix;
-					ulong idx;
-					int prefix_key_len;
-
-					if (zend_hash_get_current_key_ex(Z_ARRVAL_PP(tmp), 
-						&prefix, &prefix_key_len, &idx, 0, NULL) == HASH_KEY_IS_STRING) {
-						xmlXPathRegisterNs(ctxp, prefix, Z_STRVAL_PP(tmpns));
-					}
-				}
-				zend_hash_move_forward(Z_ARRVAL_PP(tmp));
-			}
-		}
-
-		xpathobjp = xmlXPathEvalExpression(xquery, ctxp);
-		ctxp->node = NULL;
-		if (xpathobjp && xpathobjp->type == XPATH_NODESET) {
-			nodeset = xpathobjp->nodesetval;
-		} else {
-			if (xpathobjp) {
-				xmlXPathFreeObject(xpathobjp);
-			}
-			xmlXPathFreeContext(ctxp);
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "XPath query did not return a nodeset.");
-			RETURN_FALSE;
-		}
-	}
-
-	if (ns_prefixes != NULL) {
-		if (exclusive) {
-			zval **tmpns;
-			int nscount = 0;
-
-			inclusive_ns_prefixes = safe_emalloc(zend_hash_num_elements(Z_ARRVAL_P(ns_prefixes)) + 1,
-				sizeof(xmlChar *), 0);
-			while (zend_hash_get_current_data(Z_ARRVAL_P(ns_prefixes), (void **)&tmpns) == SUCCESS) {
-				if (Z_TYPE_PP(tmpns) == IS_STRING) {
-					inclusive_ns_prefixes[nscount++] = Z_STRVAL_PP(tmpns);
-				}
-				zend_hash_move_forward(Z_ARRVAL_P(ns_prefixes));
-			}
-			inclusive_ns_prefixes[nscount] = NULL;
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_NOTICE, 
-				"Inclusive namespace prefixes only allowed in exlcusive mode.");
-		}
-	}
-
-	if (mode == 1) {
-		buf = xmlOutputBufferCreateFilename(file, NULL, 0);
-	} else {
-		buf = xmlAllocOutputBuffer(NULL);
-	}
-
-    if (buf != NULL) {
-		ret = xmlC14NDocSaveTo(docp, nodeset, exclusive, inclusive_ns_prefixes,
-			with_comments, buf);
-	}
-
-	if (inclusive_ns_prefixes != NULL) {
-		efree(inclusive_ns_prefixes);
-	}
-	if (xpathobjp != NULL) {
-		xmlXPathFreeObject(xpathobjp);
-	}
-	if (ctxp != NULL) {
-		xmlXPathFreeContext(ctxp);
-	}
-
-    if (buf == NULL || ret < 0) {
-        RETVAL_FALSE;
-    } else {
-		if (mode == 0) {
-			ret = buf->buffer->use;
-			if (ret > 0) {
-				RETVAL_STRINGL((char *) buf->buffer->content, ret, 1);
-			} else {
-				RETVAL_EMPTY_STRING();
-			}
-		}
-    }
-
-	if (buf) {
-		int bytes;
-
-		bytes = xmlOutputBufferClose(buf);
-		if (mode == 1 && (ret >= 0)) {
-			RETURN_LONG(bytes);
-		}
-	}
-}
-/* }}} */
-
-/* {{{ proto string DOMNode::C14N([bool exclusive [, bool with_comments [, array xpath [, array ns_prefixes]]]])
-   Canonicalize nodes to a string */
-PHP_METHOD(domnode, C14N)
-{
-	dom_canonicalization(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
-}
-/* }}} */
-
-/* {{{ proto int DOMNode::C14NFile(string uri [, bool exclusive [, bool with_comments [, array xpath [, array ns_prefixes]]]])
-   Canonicalize nodes to a file */
-PHP_METHOD(domnode, C14NFile)
-{
-	dom_canonicalization(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
-}
-/* }}} */
-
-/* {{{ proto int DOMNode::getNodePath()
-   Gets an xpath for a node */
-PHP_METHOD(domnode, getNodePath)
-{
-	zval *id;
-	xmlNode *nodep;
-	dom_object *intern;
-	char *value;
-	
-	DOM_GET_THIS_OBJ(nodep, id, xmlNodePtr, intern);
-
-	value = xmlGetNodePath(nodep);
-	if (value == NULL) {
-		RETURN_NULL();
-	} else {
-		RETVAL_STRING(value, 1);
-		xmlFree(value);
-	}
-}
-/* }}} */
-
-/* {{{ proto int DOMNode::getLineNo()
-   Gets line number for a node */
-PHP_METHOD(domnode, getLineNo)
-{
-	zval *id;
-	xmlNode *nodep;
-	dom_object *intern;
-	
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
-
-	DOM_GET_THIS_OBJ(nodep, id, xmlNodePtr, intern);
-
-	RETURN_LONG(xmlGetLineNo(nodep));
-}
-/* }}} */
-
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
