@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
+/* $Id: ftp.c,v 1.112.2.3 2006/01/01 12:50:06 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -39,7 +39,7 @@
 #ifdef PHP_WIN32
 #include <winsock2.h>
 #elif defined(NETWARE)
-#ifdef USE_WINSOCK    /* Modified to use Winsock (NOVSOCK2.H), at least for now */
+#ifdef USE_WINSOCK    /* Modified to use Winsock (NOVSOCK2.H), atleast for now */
 #include <novsock2.h>
 #else
 #include <sys/socket.h>
@@ -147,7 +147,7 @@ ftp_open(const char *host, short port, long timeout_sec TSRMLS_DC)
 
 	size = sizeof(ftp->localaddr);
 	memset(&ftp->localaddr, 0, size);
-	if (getsockname(ftp->fd, (struct sockaddr*) &ftp->localaddr, &size) != 0) {
+	if (getsockname(ftp->fd, (struct sockaddr*) &ftp->localaddr, &size) == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "getsockname failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
@@ -178,15 +178,10 @@ ftp_close(ftpbuf_t *ftp)
 	if (ftp->data) {
 		data_close(ftp, ftp->data);
 	}
-	if (ftp->stream && ftp->closestream) {
-		TSRMLS_FETCH();
-		php_stream_close(ftp->stream);
-	}
 	if (ftp->fd != -1) {
 #if HAVE_OPENSSL_EXT
 		if (ftp->ssl_active) {
 			SSL_shutdown(ftp->ssl_handle);
-			SSL_free(ftp->ssl_handle);
 		}
 #endif		
 		closesocket(ftp->fd);
@@ -248,7 +243,6 @@ ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
 {
 #if HAVE_OPENSSL_EXT
 	SSL_CTX	*ctx = NULL;
-	long ssl_ctx_options = SSL_OP_ALL;
 #endif
 	if (ftp == NULL) {
 		return 0;
@@ -272,61 +266,60 @@ ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
 			}
 				
 			if (ftp->resp != 334) {
-				return 0;
+				ftp->use_ssl = 0;
 			} else {
 				ftp->old_ssl = 1;
 				ftp->use_ssl_for_data = 1;
 			}
 		}
 		
-		ctx = SSL_CTX_new(SSLv23_client_method());
-		if (ctx == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL context");
-			return 0;
-		}
-
-#if OPENSSL_VERSION_NUMBER >= 0x0090605fL
-		ssl_ctx_options &= ~SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS;
-#endif
-		SSL_CTX_set_options(ctx, ssl_ctx_options);
-
-		ftp->ssl_handle = SSL_new(ctx);
-		if (ftp->ssl_handle == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL handle");
-			SSL_CTX_free(ctx);
-			return 0;
-		}
-
-		SSL_set_fd(ftp->ssl_handle, ftp->fd);
-
-		if (SSL_connect(ftp->ssl_handle) <= 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS handshake failed");
-			SSL_shutdown(ftp->ssl_handle);
-			SSL_free(ftp->ssl_handle);
-			return 0;
-		}
-
-		ftp->ssl_active = 1;
-
-		if (!ftp->old_ssl) {
-
-			/* set protection buffersize to zero */
-			if (!ftp_putcmd(ftp, "PBSZ", "0")) {
-				return 0;
-			}
-			if (!ftp_getresp(ftp)) {
+		/* now enable ssl if we still need to */
+		if (ftp->use_ssl) {
+			ctx = SSL_CTX_new(SSLv23_client_method());
+			if (ctx == NULL) {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL context");
 				return 0;
 			}
 
-			/* enable data conn encryption */
-			if (!ftp_putcmd(ftp, "PROT", "P")) {
-				return 0;
-			}
-			if (!ftp_getresp(ftp)) {
+			SSL_CTX_set_options(ctx, SSL_OP_ALL);
+
+			ftp->ssl_handle = SSL_new(ctx);
+			if (ftp->ssl_handle == NULL) {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL handle");
+				SSL_CTX_free(ctx);
 				return 0;
 			}
 			
-			ftp->use_ssl_for_data = (ftp->resp >= 200 && ftp->resp <=299);		
+			SSL_set_fd(ftp->ssl_handle, ftp->fd);
+			
+			if (SSL_connect(ftp->ssl_handle) <= 0) {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS handshake failed");
+				SSL_shutdown(ftp->ssl_handle);
+				return 0;
+			}
+			
+			ftp->ssl_active = 1;
+			
+			if (!ftp->old_ssl) {
+				
+				/* set protection buffersize to zero */
+				if (!ftp_putcmd(ftp, "PBSZ", "0")) {
+					return 0;
+				}
+				if (!ftp_getresp(ftp)) {
+					return 0;
+				}
+					
+				/* enable data conn encryption */
+				if (!ftp_putcmd(ftp, "PROT", "P")) {
+					return 0;
+				}
+				if (!ftp_getresp(ftp)) {
+					return 0;
+				}
+				
+				ftp->use_ssl_for_data = (ftp->resp >= 200 && ftp->resp <=299);		
+			}
 		}
 	}
 #endif
@@ -596,7 +589,11 @@ ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filenam
 		return 0;
 	}
 
-	spprintf(&buffer, 0, "CHMOD %o %s", mode, filename);
+	if (!(buffer = emalloc(32 + filename_len + 1))) {
+		return 0;
+	}
+
+	sprintf(buffer, "CHMOD %o %s", mode, filename);
 
 	if (!ftp_putcmd(ftp, "SITE", buffer)) {
 		efree(buffer);
@@ -616,7 +613,7 @@ ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filenam
 /* {{{ ftp_alloc
  */
 int
-ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
+ftp_alloc(ftpbuf_t *ftp, const int size, char **response)
 {
 	char buffer[64];
 
@@ -624,8 +621,8 @@ ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
 		return 0;
 	}
 
-	snprintf(buffer, sizeof(buffer) - 1, "%ld", size);
-    
+	snprintf(buffer, sizeof(buffer) - 1, "%d", size);
+
 	if (!ftp_putcmd(ftp, "ALLO", buffer)) {
 		return 0;
 	}
@@ -634,7 +631,7 @@ ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
 		return 0;
 	}
 
-	if (response) {
+	if (response && ftp->inbuf) {
 		*response = estrdup(ftp->inbuf);
 	}
 
@@ -791,7 +788,7 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 /* {{{ ftp_get
  */
 int
-ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, long resumepos TSRMLS_DC)
+ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, int resumepos TSRMLS_DC)
 {
 	databuf_t		*data = NULL;
 	int			lastch;
@@ -812,7 +809,11 @@ ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type,
 	ftp->data = data;
 
 	if (resumepos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", resumepos);
+		if (resumepos > 2147483647) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files greater then 2147483647 bytes.");
+			goto bail;
+		}
+		sprintf(arg, "%u", resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -885,10 +886,10 @@ bail:
 /* {{{ ftp_put
  */
 int
-ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, long startpos TSRMLS_DC)
+ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, int startpos TSRMLS_DC)
 {
 	databuf_t		*data = NULL;
-	long			size;
+	int			size;
 	char			*ptr;
 	int			ch;
 	char			arg[11];
@@ -905,7 +906,11 @@ ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, l
 	ftp->data = data;	
 
 	if (startpos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", startpos);
+		if (startpos > 2147483647) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files with a size greater then 2147483647 bytes.");
+			goto bail;
+		}
+		sprintf(arg, "%u", startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -950,7 +955,7 @@ ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, l
 	}
 	ftp->data = data = data_close(ftp, data);
 
-	if (!ftp_getresp(ftp) || (ftp->resp != 226 && ftp->resp != 250 && ftp->resp != 200)) {
+	if (!ftp_getresp(ftp) || (ftp->resp != 226 && ftp->resp != 250)) {
 		goto bail;
 	}
 	return 1;
@@ -962,7 +967,7 @@ bail:
 
 /* {{{ ftp_size
  */
-long
+int
 ftp_size(ftpbuf_t *ftp, const char *path)
 {
 	if (ftp == NULL) {
@@ -977,7 +982,7 @@ ftp_size(ftpbuf_t *ftp, const char *path)
 	if (!ftp_getresp(ftp) || ftp->resp != 213) {
 		return -1;
 	}
-	return atol(ftp->inbuf);
+	return atoi(ftp->inbuf);
 }
 /* }}} */
 
@@ -1014,9 +1019,6 @@ ftp_mdtm(ftpbuf_t *ftp, const char *path)
 	/* figure out the GMT offset */
 	stamp = time(NULL);
 	gmt = php_gmtime_r(&stamp, &tmbuf);
-	if (!gmt) {
-		return -1;
-	}
 	gmt->tm_isdst = -1;
 
 	/* apply the GMT offset */
@@ -1101,32 +1103,23 @@ ftp_putcmd(ftpbuf_t *ftp, const char *cmd, const char *args)
 	int		size;
 	char		*data;
 
-	if (strpbrk(cmd, "\r\n")) {
-		return 0;
-	} 
 	/* build the output buffer */
 	if (args && args[0]) {
 		/* "cmd args\r\n\0" */
 		if (strlen(cmd) + strlen(args) + 4 > FTP_BUFSIZE) {
 			return 0;
 		}
-		if (strpbrk(args, "\r\n")) {
-			return 0;
-		}
-		size = slprintf(ftp->outbuf, sizeof(ftp->outbuf), "%s %s\r\n", cmd, args);
+		size = sprintf(ftp->outbuf, "%s %s\r\n", cmd, args);
 	} else {
 		/* "cmd\r\n\0" */
 		if (strlen(cmd) + 3 > FTP_BUFSIZE) {
 			return 0;
 		}
-		size = slprintf(ftp->outbuf, sizeof(ftp->outbuf), "%s\r\n", cmd);
+		size = sprintf(ftp->outbuf, "%s\r\n", cmd);
 	}
 
 	data = ftp->outbuf;
 
-	/* Clear the extra-lines buffer */
-	ftp->extra = NULL;
-
 	if (my_send(ftp, ftp->fd, data, size) != size) {
 		return 0;
 	}
@@ -1139,7 +1132,7 @@ ftp_putcmd(ftpbuf_t *ftp, const char *cmd, const char *args)
 int
 ftp_readline(ftpbuf_t *ftp)
 {
-	long		size, rcvd;
+	int		size, rcvd;
 	char		*data, *eol;
 
 	/* shift the extra to the front */
@@ -1232,8 +1225,7 @@ ftp_getresp(ftpbuf_t *ftp)
 int
 my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
-	long		size, sent;
-    int         n;
+	int		n, size, sent;
 
 	size = len;
 	while (size) {
@@ -1393,7 +1385,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 
 	sa = (struct sockaddr *) &ftp->localaddr;
 	/* bind/listen */
-	if ((fd = socket(sa->sa_family, SOCK_STREAM, 0)) == SOCK_ERR) {
+	if ((fd = socket(sa->sa_family, SOCK_STREAM, 0)) == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "socket() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
@@ -1426,30 +1418,30 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 	php_any_addr(sa->sa_family, &addr, 0);
 	size = php_sockaddr_size(&addr);
 
-	if (bind(fd, (struct sockaddr*) &addr, size) != 0) {
+	if (bind(fd, (struct sockaddr*) &addr, size) == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "bind() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
-	if (getsockname(fd, (struct sockaddr*) &addr, &size) != 0) {
+	if (getsockname(fd, (struct sockaddr*) &addr, &size) == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "getsockname() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
-	if (listen(fd, 5) != 0) {
+	if (listen(fd, 5) == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "listen() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
 	data->listener = fd;
 
-#if HAVE_IPV6 && HAVE_INET_NTOP
+#if HAVE_IPV6
 	if (sa->sa_family == AF_INET6) {
 		/* need to use EPRT */
 		char eprtarg[INET6_ADDRSTRLEN + sizeof("|x||xxxxx|")];
 		char out[INET6_ADDRSTRLEN];
 		inet_ntop(AF_INET6, &((struct sockaddr_in6*) sa)->sin6_addr, out, sizeof(out));
-		snprintf(eprtarg, sizeof(eprtarg), "|2|%s|%hu|", out, ntohs(((struct sockaddr_in6 *) &addr)->sin6_port));
+		sprintf(eprtarg, "|2|%s|%hu|", out, ntohs(((struct sockaddr_in6 *) &addr)->sin6_port));
 
 		if (!ftp_putcmd(ftp, "EPRT", eprtarg)) {
 			goto bail;
@@ -1467,7 +1459,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 	/* send the PORT */
 	ipbox.ia[0] = ((struct sockaddr_in*) sa)->sin_addr;
 	ipbox.s[2] = ((struct sockaddr_in*) &addr)->sin_port;
-	snprintf(arg, sizeof(arg), "%u,%u,%u,%u,%u,%u", ipbox.c[0], ipbox.c[1], ipbox.c[2], ipbox.c[3], ipbox.c[4], ipbox.c[5]);
+	sprintf(arg, "%u,%u,%u,%u,%u,%u", ipbox.c[0], ipbox.c[1], ipbox.c[2], ipbox.c[3], ipbox.c[4], ipbox.c[5]);
 
 	if (!ftp_putcmd(ftp, "PORT", arg)) {
 		goto bail;
@@ -1498,7 +1490,6 @@ data_accept(databuf_t *data, ftpbuf_t *ftp TSRMLS_DC)
 
 #if HAVE_OPENSSL_EXT
 	SSL_CTX		*ctx;
-	long ssl_ctx_options = SSL_OP_ALL;
 #endif
 
 	if (data->fd != -1) {
@@ -1525,10 +1516,7 @@ data_accepted:
 			return 0;
 		}
 
-#if OPENSSL_VERSION_NUMBER >= 0x0090605fL
-		ssl_ctx_options &= ~SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS;
-#endif
-		SSL_CTX_set_options(ctx, ssl_ctx_options);
+		SSL_CTX_set_options(ctx, SSL_OP_ALL);
 
 		data->ssl_handle = SSL_new(ctx);
 		if (data->ssl_handle == NULL) {
@@ -1547,7 +1535,6 @@ data_accepted:
 		if (SSL_connect(data->ssl_handle) <= 0) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: SSL/TLS handshake failed");
 			SSL_shutdown(data->ssl_handle);
-			SSL_free(data->ssl_handle);
 			return 0;
 		}
 			
@@ -1565,21 +1552,13 @@ data_accepted:
 databuf_t*
 data_close(ftpbuf_t *ftp, databuf_t *data)
 {
-#if HAVE_OPENSSL_EXT
-	SSL_CTX		*ctx;
-#endif				
 	if (data == NULL) {
 		return NULL;
 	}
 	if (data->listener != -1) {
 #if HAVE_OPENSSL_EXT
 		if (data->ssl_active) {
-		
-			ctx = SSL_get_SSL_CTX(data->ssl_handle);
-			SSL_CTX_free(ctx);
-
 			SSL_shutdown(data->ssl_handle);
-			SSL_free(data->ssl_handle);
 			data->ssl_active = 0;
 		}
 #endif				
@@ -1588,11 +1567,7 @@ data_close(ftpbuf_t *ftp, databuf_t *data)
 	if (data->fd != -1) {
 #if HAVE_OPENSSL_EXT
 		if (data->ssl_active) {
-			ctx = SSL_get_SSL_CTX(data->ssl_handle);
-			SSL_CTX_free(ctx);
-
 			SSL_shutdown(data->ssl_handle);
-			SSL_free(data->ssl_handle);
 			data->ssl_active = 0;
 		}
 #endif				
@@ -1639,17 +1614,10 @@ ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
 	if (!ftp_putcmd(ftp, cmd, path)) {
 		goto bail;
 	}
-	if (!ftp_getresp(ftp) || (ftp->resp != 150 && ftp->resp != 125 && ftp->resp != 226)) {
+	if (!ftp_getresp(ftp) || (ftp->resp != 150 && ftp->resp != 125)) {
 		goto bail;
 	}
 
-	/* some servers don't open a ftp-data connection if the directory is empty */
-	if (ftp->resp == 226) {
-		ftp->data = data_close(ftp, data);
-		php_stream_close(tmpstream);
-		return ecalloc(1, sizeof(char*));
-	}
-
 	/* pull data buffer into tmpfile */
 	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
 		goto bail;
@@ -1675,11 +1643,11 @@ ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
 		}
 	}
 
-	ftp->data = data_close(ftp, data);
+	ftp->data = data = data_close(ftp, data);
 
 	php_stream_rewind(tmpstream);
 
-	ret = safe_emalloc((lines + 1), sizeof(char*), size);
+	ret = safe_emalloc((lines + 1), sizeof(char**), size * sizeof(char*));
 
 	entry = ret;
 	text = (char*) (ret + lines + 1);
@@ -1716,13 +1684,13 @@ bail:
 /* {{{ ftp_nb_get
  */
 int
-ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, long resumepos TSRMLS_DC)
+ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, int resumepos TSRMLS_DC)
 {
 	databuf_t		*data = NULL;
 	char			arg[11];
 
 	if (ftp == NULL) {
-		return PHP_FTP_FAILED;
+		goto bail;
 	}
 
 	if (!ftp_type(ftp, type)) {
@@ -1734,7 +1702,14 @@ ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t typ
 	}
 
 	if (resumepos>0) {
-		snprintf(arg, sizeof(arg), "%ld", resumepos);
+		/* We are working on an architecture that supports 64-bit integers
+		 * since php is 32 bit by design, we bail out with warning
+		 */
+		if (resumepos > 2147483647) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files greater then 2147483648 bytes.");
+			goto bail;
+		}
+		sprintf(arg, "%u", resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1833,7 +1808,7 @@ bail:
 /* {{{ ftp_nb_put
  */
 int
-ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, long startpos TSRMLS_DC)
+ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, int startpos TSRMLS_DC)
 {
 	databuf_t		*data = NULL;
 	char			arg[11];
@@ -1848,7 +1823,11 @@ ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type
 		goto bail;
 	}
 	if (startpos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", startpos);
+		if (startpos > 2147483647) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files with a size greater then 2147483647 bytes.");
+			goto bail;
+		}
+		sprintf(arg, "%u", startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1885,7 +1864,7 @@ bail:
 int
 ftp_nb_continue_write(ftpbuf_t *ftp TSRMLS_DC)
 {
-	long			size;
+	int			size;
 	char			*ptr;
 	int 			ch;
 
