@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
+/* $Id: gd_ctx.c,v 1.22.2.4 2006/01/01 12:50:06 sniper Exp $ */
 
 #include "php_gd.h"
 
@@ -46,102 +46,61 @@ static void _php_image_output_ctxfree(struct gdIOCtx *ctx)
 	}
 }
 
-static void _php_image_stream_putc(struct gdIOCtx *ctx, int c)  {
-	char ch = (char) c;
-	php_stream * stream = (php_stream *)ctx->data;
-	TSRMLS_FETCH();
-	php_stream_write(stream, &ch, 1);
-}
-
-static int _php_image_stream_putbuf(struct gdIOCtx *ctx, const void* buf, int l)
-{
-	php_stream * stream = (php_stream *)ctx->data;
-	TSRMLS_FETCH();
-	return php_stream_write(stream, (void *)buf, l);
-}
-
-static void _php_image_stream_ctxfree(struct gdIOCtx *ctx)
-{
-	TSRMLS_FETCH();
-
-	if(ctx->data) {
-		php_stream_close((php_stream *) ctx->data);
-		ctx->data = NULL;
-	}
-	if(ctx) {
-		efree(ctx);
-	}
-}
-
 /* {{{ _php_image_output_ctx */
 static void _php_image_output_ctx(INTERNAL_FUNCTION_PARAMETERS, int image_type, char *tn, void (*func_p)())
 {
-	zval *imgind;
-	char *file = NULL;
-	int file_len = 0;
-	long quality, basefilter;
+	zval **imgind, **file, **quality;
 	gdImagePtr im;
+	char *fn = NULL;
+	FILE *fp = NULL;
 	int argc = ZEND_NUM_ARGS();
 	int q = -1, i;
-	int f = -1;
-	gdIOCtx *ctx = NULL;
-	zval *to_zval = NULL;
-	php_stream *stream;
+	gdIOCtx *ctx;
 
 	/* The third (quality) parameter for Wbmp stands for the threshold when called from image2wbmp().
 	 * The third (quality) parameter for Wbmp and Xbm stands for the foreground color index when called
 	 * from imagey<type>().
 	 */
-	if (image_type == PHP_GDIMG_TYPE_XBM) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rp!|ll", &imgind, &file, &file_len, &quality, &basefilter) == FAILURE) {
-			return;
-		}
-	} else {
-		/* PHP_GDIMG_TYPE_GIF
-		 * PHP_GDIMG_TYPE_PNG 
-		 * PHP_GDIMG_TYPE_JPG 
-		 * PHP_GDIMG_TYPE_WBM 
-		 * PHP_GDIMG_TYPE_WEBP 
-		 * */
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|z/!ll", &imgind, &to_zval, &quality, &basefilter) == FAILURE) {
-			return;
+
+	if (argc < 2 && image_type == PHP_GDIMG_TYPE_XBM) {
+		WRONG_PARAM_COUNT;
+	}
+	if (argc < 1 || argc > 3 || zend_get_parameters_ex(argc, &imgind, &file, &quality) == FAILURE)
+	{
+		WRONG_PARAM_COUNT;
+	}
+
+	ZEND_FETCH_RESOURCE(im, gdImagePtr, imgind, -1, "Image", phpi_get_le_gd());
+
+	if (argc > 1) {
+		convert_to_string_ex(file);
+		fn = Z_STRVAL_PP(file);
+		if (argc == 3) {
+			convert_to_long_ex(quality);
+			q = Z_LVAL_PP(quality);/* or colorindex for foreground of BW images (defaults to black) */
 		}
 	}
 
-	ZEND_FETCH_RESOURCE(im, gdImagePtr, &imgind, -1, "Image", phpi_get_le_gd());
+	if ((argc == 2) || (argc > 2 && Z_STRLEN_PP(file))) {
 
-	if (argc >= 3) {
-		q = quality; /* or colorindex for foreground of BW images (defaults to black) */
-		if (argc == 4) {
-			f = basefilter;
-		}
-	}
+		PHP_GD_CHECK_OPEN_BASEDIR(fn, "Invalid filename");
 
-	if (argc > 1 && to_zval != NULL) {
-		if (Z_TYPE_P(to_zval) == IS_RESOURCE) {
-			php_stream_from_zval_no_verify(stream, &to_zval);
-			if (stream == NULL) {
-				RETURN_FALSE;
-			}
-		} else if (Z_TYPE_P(to_zval) == IS_STRING) {
-			if (CHECK_ZVAL_NULL_PATH(to_zval)) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid 2nd parameter, filename must not contain null bytes");
-				RETURN_FALSE;
-			}
-
-			stream = php_stream_open_wrapper(Z_STRVAL_P(to_zval), "wb", REPORT_ERRORS|IGNORE_PATH|IGNORE_URL_WIN, NULL);
-			if (stream == NULL) {
-				RETURN_FALSE;
-			}
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid 2nd parameter, it must a filename or a stream");
+		fp = VCWD_FOPEN(fn, "wb");
+		if (!fp) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to open '%s' for writing", fn);
 			RETURN_FALSE;
 		}
+
+		ctx = gdNewFileCtx(fp);
 	} else {
 		ctx = emalloc(sizeof(gdIOCtx));
 		ctx->putC = _php_image_output_putc;
 		ctx->putBuf = _php_image_output_putbuf;
+#if HAVE_LIBGD204
 		ctx->gd_free = _php_image_output_ctxfree;
+#else
+		ctx->free = _php_image_output_ctxfree;
+#endif
 
 #if APACHE && defined(CHARSET_EBCDIC)
 		/* XXX this is unlikely to work any more thies@thieso.net */
@@ -150,30 +109,14 @@ static void _php_image_output_ctx(INTERNAL_FUNCTION_PARAMETERS, int image_type,
 #endif
 	}
 
-	if (!ctx)	{
-		ctx = emalloc(sizeof(gdIOCtx));
-		ctx->putC = _php_image_stream_putc;
-		ctx->putBuf = _php_image_stream_putbuf;
-		ctx->gd_free = _php_image_stream_ctxfree;
-		ctx->data = (void *)stream;
-	}
-
 	switch(image_type) {
 		case PHP_GDIMG_CONVERT_WBM:
 			if(q<0||q>255) {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid threshold value '%d'. It must be between 0 and 255", q);
 			}
 		case PHP_GDIMG_TYPE_JPG:
-			(*func_p)(im, ctx, q);
-			break;
-		case PHP_GDIMG_TYPE_WEBP:
-			if (q == -1) {
-				q = 80;
-			}
-			(*func_p)(im, ctx, q);
-			break;
 		case PHP_GDIMG_TYPE_PNG:
-			(*func_p)(im, ctx, q, f);
+			(*func_p)(im, ctx, q);
 			break;
 		case PHP_GDIMG_TYPE_XBM:
 		case PHP_GDIMG_TYPE_WBM:
@@ -184,7 +127,7 @@ static void _php_image_output_ctx(INTERNAL_FUNCTION_PARAMETERS, int image_type,
 				q = i;
 			}
 			if (image_type == PHP_GDIMG_TYPE_XBM) {
-				(*func_p)(im, file, q, ctx);
+				(*func_p)(im, fn, q, ctx);
 			} else {
 				(*func_p)(im, q, ctx);
 			}
@@ -200,7 +143,12 @@ static void _php_image_output_ctx(INTERNAL_FUNCTION_PARAMETERS, int image_type,
 	ctx->free(ctx);
 #endif
 
-	RETURN_TRUE;
+	if(fp) {
+		fflush(fp);
+		fclose(fp);
+	}
+
+    RETURN_TRUE;
 }
 /* }}} */
 
