@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2009 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: hash_adler32.c,v 1.3.2.4.2.2.2.3 2008/12/31 11:15:37 sebastian Exp $ */
+/* $Id: hash_adler32.c,v 1.3.2.3 2006/01/01 12:50:07 sniper Exp $ */
 
 #include "php_hash.h"
 #include "php_hash_adler32.h"
@@ -29,10 +29,8 @@ PHP_HASH_API void PHP_ADLER32Init(PHP_ADLER32_CTX *context)
 
 PHP_HASH_API void PHP_ADLER32Update(PHP_ADLER32_CTX *context, const unsigned char *input, size_t len)
 {
-	php_hash_uint32 i, s[2];
+	php_hash_uint32 i, s[2] = { context->state & 0xffff, (context->state >> 16) & 0xffff };
 	
-	s[0] = context->state & 0xffff;
-	s[1] = (context->state >> 16) & 0xffff;
 	for (i = 0; i < len; ++i) {
 		s[0] = (s[0] + input[i]) % 65521;
 		s[1] = (s[1] + s[0]) % 65521;
@@ -49,17 +47,10 @@ PHP_HASH_API void PHP_ADLER32Final(unsigned char digest[4], PHP_ADLER32_CTX *con
 	context->state = 0;
 }
 
-PHP_HASH_API int PHP_ADLER32Copy(const php_hash_ops *ops, PHP_ADLER32_CTX *orig_context, PHP_ADLER32_CTX *copy_context)
-{
-	copy_context->state = orig_context->state;
-	return SUCCESS;
-}
-
-const php_hash_ops php_hash_adler32_ops = {
+php_hash_ops php_hash_adler32_ops = {
 	(php_hash_init_func_t) PHP_ADLER32Init,
 	(php_hash_update_func_t) PHP_ADLER32Update,
 	(php_hash_final_func_t) PHP_ADLER32Final,
-	(php_hash_copy_func_t) PHP_ADLER32Copy,
 	4, /* what to say here? */
 	4,
 	sizeof(PHP_ADLER32_CTX)
