@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2013 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,22 +17,12 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: hash_tiger.c,v 1.4.2.3 2006/01/01 12:50:07 sniper Exp $ */
 
 #include "php_hash.h"
 #include "php_hash_tiger.h"
 #include "php_hash_tiger_tables.h"
 
-#if (defined(__APPLE__) || defined(__APPLE_CC__)) && (defined(__BIG_ENDIAN__) || defined(__LITTLE_ENDIAN__))
-# if defined(__LITTLE_ENDIAN__)
-#  undef WORDS_BIGENDIAN
-# else 
-#  if defined(__BIG_ENDIAN__)
-#   define WORDS_BIGENDIAN
-#  endif
-# endif
-#endif
-
 /* {{{ */
 #define save_abc \
 	aa = a; \
@@ -98,24 +88,6 @@
 	} \
 	feedforward
 
-#define split_ex(str) \
-	x0=str[0]; x1=str[1]; x2=str[2]; x3=str[3]; \
-	x4=str[4]; x5=str[5]; x6=str[6]; x7=str[7];
-#ifdef WORDS_BIGENDIAN
-#	define split(str) \
-	{ \
-		int i; \
-		php_hash_uint64 tmp[8]; \
-		 \
-		for (i = 0; i < 64; ++i) { \
-			((unsigned char *) tmp)[i^7] = ((unsigned char *) str)[i]; \
-		} \
-		split_ex(tmp); \
-	}
-#else
-#	define split split_ex
-#endif
-
 #define tiger_compress(passes, str, state) \
 { \
 	register php_hash_uint64 a, b, c, tmpa, x0, x1, x2, x3, x4, x5, x6, x7; \
@@ -126,7 +98,8 @@
 	b = state[1]; \
 	c = state[2]; \
 	\
-	split(str); \
+	x0=str[0]; x1=str[1]; x2=str[2]; x3=str[3]; \
+	x4=str[4]; x5=str[5]; x6=str[6]; x7=str[7]; \
 	\
 	compress(passes); \
 	\
@@ -145,7 +118,7 @@ static inline void TigerFinalize(PHP_TIGER_CTX *context)
 		memset(&context->buffer[context->length], 0, 8-context->length%8);
 		context->length += 8-context->length%8;
 	}
-	
+    
 	if (context->length > 56) {
 		memset(&context->buffer[context->length], 0, 64 - context->length);
 		tiger_compress(context->passes, ((php_hash_uint64 *) context->buffer), context->state);
@@ -153,19 +126,8 @@ static inline void TigerFinalize(PHP_TIGER_CTX *context)
 	} else {
 		memset(&context->buffer[context->length], 0, 56 - context->length);
 	}
-
-#ifndef WORDS_BIGENDIAN	
+	
 	memcpy(&context->buffer[56], &context->passed, sizeof(php_hash_uint64));
-#else
-	context->buffer[56] = (unsigned char) (context->passed & 0xff);
-	context->buffer[57] = (unsigned char) ((context->passed >> 8) & 0xff);
-	context->buffer[58] = (unsigned char) ((context->passed >> 16) & 0xff);
-	context->buffer[59] = (unsigned char) ((context->passed >> 24) & 0xff);
-	context->buffer[60] = (unsigned char) ((context->passed >> 32) & 0xff);
-	context->buffer[61] = (unsigned char) ((context->passed >> 40) & 0xff);
-	context->buffer[62] = (unsigned char) ((context->passed >> 48) & 0xff);
-	context->buffer[63] = (unsigned char) ((context->passed >> 56) & 0xff);
-#endif
 	tiger_compress(context->passes, ((php_hash_uint64 *) context->buffer), context->state);
 }
 
@@ -203,12 +165,11 @@ PHP_HASH_API void PHP_TIGERUpdate(PHP_TIGER_CTX *context, const unsigned char *i
 		}
 		
 		for (; i + 64 <= len; i += 64) {
-			memcpy(context->buffer, &input[i], 64);
-			tiger_compress(context->passes, ((const php_hash_uint64 *) context->buffer), context->state);
+			tiger_compress(context->passes, ((const php_hash_uint64 *) (input + i)), context->state);
 			context->passed += 512;
 		}
-		memset(&context->buffer[r], 0, 64-r);
-		memcpy(context->buffer, &input[i], r);
+		
+		memcpy(context->buffer, input + i, r);
 		context->length = r;
 	}
 }
@@ -298,11 +259,10 @@ PHP_HASH_API void PHP_TIGER192Final(unsigned char digest[24], PHP_TIGER_CTX *con
 }
 
 #define PHP_HASH_TIGER_OPS(p, b) \
-	const php_hash_ops php_hash_##p##tiger##b##_ops = { \
+	php_hash_ops php_hash_##p##tiger##b##_ops = { \
 		(php_hash_init_func_t) PHP_##p##TIGERInit, \
 		(php_hash_update_func_t) PHP_TIGERUpdate, \
 		(php_hash_final_func_t) PHP_TIGER##b##Final, \
-		(php_hash_copy_func_t) php_hash_copy, \
 		b/8, \
 		64, \
 		sizeof(PHP_TIGER_CTX) \
