@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Revision: 1.28.2.2 $ */
+/* $Revision: 1.28.2.1 $ */
 
 #ifndef PHP_ICONV_H
 #define PHP_ICONV_H
@@ -93,7 +93,7 @@ typedef enum _php_iconv_err_t {
 } php_iconv_err_t;
 /* }}} */
 
-PHP_ICONV_API php_iconv_err_t php_iconv_string(const char * in_p, size_t in_len, char **out, size_t *out_len, const char *out_charset, const char *in_charset);
+PHP_ICONV_API php_iconv_err_t php_iconv_string(const char * in_p, size_t in_len, char **out, size_t *out_len, const char *in_charset, const char *out_charset);
 
 #else
 
