@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -22,7 +22,7 @@
    +----------------------------------------------------------------------+
  */
  
-/* $Id: ldap.c,v 1.154.2.7 2005/07/09 01:00:11 sniper Exp $ */
+/* $Id: ldap.c,v 1.161.2.3 2006/01/01 12:50:08 sniper Exp $ */
 #define IS_EXT_MODULE
 
 #ifdef HAVE_CONFIG_H
@@ -30,7 +30,7 @@
 #endif
 
 /* Additional headers for NetWare */
-#if defined(NETWARE)
+#if defined(NETWARE) && (NEW_LIBC)
 #include <sys/select.h>
 #include <sys/timeval.h>
 #endif
@@ -97,7 +97,7 @@ static int le_link, le_result, le_result_entry, le_ber_entry;
 */
 /* {{{ ldap_functions[]
  */
-function_entry ldap_functions[] = {
+zend_function_entry ldap_functions[] = {
 	PHP_FE(ldap_connect,								NULL)
 	PHP_FALIAS(ldap_close,		ldap_unbind,			NULL)
 	PHP_FE(ldap_bind,									NULL)
@@ -314,7 +314,7 @@ PHP_MINFO_FUNCTION(ldap)
 
 	php_info_print_table_start();
 	php_info_print_table_row(2, "LDAP Support", "enabled");
-	php_info_print_table_row(2, "RCS Version", "$Id: ldap.c,v 1.154.2.7 2005/07/09 01:00:11 sniper Exp $");
+	php_info_print_table_row(2, "RCS Version", "$Id: ldap.c,v 1.161.2.3 2006/01/01 12:50:08 sniper Exp $");
 
 	if (LDAPG(max_links) == -1) {
 		snprintf(tmp, 31, "%ld/unlimited", LDAPG(num_links));
@@ -525,8 +525,9 @@ static php_ldap_bictx *_php_sasl_setdefs(LDAP *ld, char *sasl_mech, char *sasl_r
 
 	return ctx;
 }
+/* }}} */
 
-/* {{{ _php_sasl_setdefs
+/* {{{ _php_sasl_freedefs
  */
 static void _php_sasl_freedefs(php_ldap_bictx *ctx)
 {
@@ -537,6 +538,7 @@ static void _php_sasl_freedefs(php_ldap_bictx *ctx)
 	if (ctx->authzid) ber_memfree(ctx->authzid);
 	ber_memfree(ctx);
 }
+/* }}} */
 
 /* {{{ _php_sasl_interact
    Internal interact function for SASL */
@@ -569,6 +571,7 @@ static int _php_sasl_interact(LDAP *ld, unsigned flags, void *defaults, void *in
 	}
 	return LDAP_SUCCESS;
 }
+/* }}} */
 
 /* {{{ proto bool ldap_sasl_bind(resource link [, string binddn, string password, string sasl_mech, string sasl_realm, string sasl_authz_id, string props])
    Bind to LDAP directory using SASL */
