@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: oci8_interface.c 294441 2010-02-03 19:37:35Z pajoye $ */
+/* $Id: oci8_interface.c,v 1.8.2.4 2006/01/05 13:42:35 tony2001 Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,10 +40,6 @@
 #include "php_oci8.h"
 #include "php_oci8_int.h"
 
-#ifndef OCI_STMT_CALL
-#define OCI_STMT_CALL 10
-#endif
-
 /* {{{ proto bool oci_define_by_name(resource stmt, string name, mixed &var [, int type])
    Define a PHP variable to an Oracle column by name */
 /* if you want to define a LOB/CLOB etc make sure you allocate it via OCINewDescriptor BEFORE defining!!! */
@@ -52,7 +48,7 @@ PHP_FUNCTION(oci_define_by_name)
 	zval *stmt, *var;
 	char *name;
 	int name_len;
-	long type = 0;
+	long type = SQLT_CHR;
 	php_oci_statement *statement;
 	php_oci_define *define, *tmp_define;
 
@@ -442,7 +438,8 @@ PHP_FUNCTION(oci_lob_seek)
 	}
 	
 	PHP_OCI_ZVAL_TO_DESCRIPTOR(*tmp, descriptor);
-
+	
+	
 	if (php_oci_lob_get_length(descriptor, &lob_length TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
@@ -452,9 +449,9 @@ PHP_FUNCTION(oci_lob_seek)
 			descriptor->lob_current_position += offset;
 			break;
 		case PHP_OCI_SEEK_END:
-			if ((descriptor->lob_size + offset) >= 0) {
+			if (descriptor->lob_size + offset >= 0) {
 				descriptor->lob_current_position = descriptor->lob_size + offset;
-			}
+			} 
 			else {
 				descriptor->lob_current_position = 0;
 			}
@@ -463,7 +460,8 @@ PHP_FUNCTION(oci_lob_seek)
 		default:
 				descriptor->lob_current_position = (offset > 0) ? offset : 0;
 			break;
-	}	
+	}
+	
 	RETURN_TRUE;
 }
 /* }}} */
@@ -503,7 +501,7 @@ PHP_FUNCTION(oci_lob_write)
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
 	int data_len;
-	long write_len = 0;
+	long write_len = 0; 
 	ub4 bytes_written;
 	char *data;
 	
@@ -589,8 +587,7 @@ PHP_FUNCTION(oci_lob_truncate)
 {
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
-	long trim_length = 0;
-	ub4 ub_trim_length;
+	ub4 trim_length = 0;
 	
 	if (getThis()) {
 		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &trim_length) == FAILURE) {
@@ -607,16 +604,10 @@ PHP_FUNCTION(oci_lob_truncate)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find descriptor property");
 		RETURN_FALSE;
 	}
-
-	if (trim_length < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length must be greater than or equal to zero");
-		RETURN_FALSE;
-	}
-
-	ub_trim_length = (ub4) trim_length;
+	
 	PHP_OCI_ZVAL_TO_DESCRIPTOR(*tmp, descriptor);
 	
-	if (php_oci_lob_truncate(descriptor, ub_trim_length TSRMLS_CC)) {
+	if (php_oci_lob_truncate(descriptor, trim_length TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 	RETURN_TRUE;
@@ -683,7 +674,7 @@ PHP_FUNCTION(oci_lob_flush)
 {
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
-	long flush_flag = 0;
+	ub4 flush_flag = 0;
 	
 	if (getThis()) {
 		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &flush_flag) == FAILURE) {
@@ -859,8 +850,7 @@ PHP_FUNCTION(oci_lob_export)
 {	
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
-	char *filename;
-	char *buffer;
+	char *filename, *buffer;
 	int filename_len;
 	long start = -1, length = -1, block_length;
 	php_stream *stream;
@@ -963,6 +953,7 @@ PHP_FUNCTION(oci_lob_export)
 }
 /* }}} */
 
+#ifdef HAVE_OCI8_TEMP_LOB
 /* {{{ proto bool oci_lob_write_temporary(string var [, int lob_type])
    Writes temporary blob */
 PHP_FUNCTION(oci_lob_write_temporary)
@@ -1024,6 +1015,7 @@ PHP_FUNCTION(oci_lob_close)
 	RETURN_TRUE;
 }
 /* }}} */
+#endif 
 
 /* {{{ proto object oci_new_descriptor(resource connection [, int type])
    Initialize a new empty descriptor LOB/FILE (LOB is default) */
@@ -1066,7 +1058,9 @@ PHP_FUNCTION(oci_rollback)
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 
 	if (connection->descriptors) {
-		php_oci_connection_descriptors_free(connection TSRMLS_CC);
+		zend_hash_destroy(connection->descriptors);
+		efree(connection->descriptors);
+		connection->descriptors = NULL;
 	}
 
 	if (php_oci_connection_rollback(connection TSRMLS_CC)) {
@@ -1090,7 +1084,9 @@ PHP_FUNCTION(oci_commit)
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 
 	if (connection->descriptors) {
-		php_oci_connection_descriptors_free(connection TSRMLS_CC);
+		zend_hash_destroy(connection->descriptors);
+		efree(connection->descriptors);
+		connection->descriptors = NULL;
 	}
 	
 	if (php_oci_connection_commit(connection TSRMLS_CC)) {
@@ -1106,7 +1102,7 @@ PHP_FUNCTION(oci_field_name)
 {
 	php_oci_out_column *column;
 
-	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0) ) ) {
+	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU) ) ) {
 		RETURN_STRINGL(column->name, column->name_len, 1);
 	}
 	RETURN_FALSE;
@@ -1119,7 +1115,7 @@ PHP_FUNCTION(oci_field_size)
 {
 	php_oci_out_column *column;
 
-	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0) ) ) {
+	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU) ) ) {
 		/* Handle data type of LONG */
 		if (column->data_type == SQLT_LNG){
 			RETURN_LONG(column->storage_size4);
@@ -1136,7 +1132,7 @@ PHP_FUNCTION(oci_field_scale)
 {
 	php_oci_out_column *column;
 
-	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0) ) ) {
+	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU) ) ) {
 		RETURN_LONG(column->scale);
 	}
 	RETURN_FALSE;
@@ -1149,7 +1145,7 @@ PHP_FUNCTION(oci_field_precision)
 {
 	php_oci_out_column *column;
 
-	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0) ) ) {
+	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU) ) ) {
 		RETURN_LONG(column->precision);
 	}
 	RETURN_FALSE;
@@ -1162,7 +1158,7 @@ PHP_FUNCTION(oci_field_type)
 {
 	php_oci_out_column *column;
 
-	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
+	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU);
 
 	if (!column) {
 		RETURN_FALSE;
@@ -1176,22 +1172,7 @@ PHP_FUNCTION(oci_field_type)
 #endif
 #ifdef SQLT_TIMESTAMP_TZ
 		case SQLT_TIMESTAMP_TZ:
-			RETVAL_STRING("TIMESTAMP WITH TIMEZONE",1);
-			break;
-#endif
-#ifdef SQLT_TIMESTAMP_LTZ
-		case SQLT_TIMESTAMP_LTZ:
-			RETVAL_STRING("TIMESTAMP WITH LOCAL TIMEZONE",1);
-			break;
-#endif
-#ifdef SQLT_INTERVAL_YM
-		case SQLT_INTERVAL_YM:
-			RETVAL_STRING("INTERVAL YEAR TO MONTH",1);
-			break;
-#endif
-#ifdef SQLT_INTERVAL_DS
-		case SQLT_INTERVAL_DS:
-			RETVAL_STRING("INTERVAL DAY TO SECOND",1);
+			RETVAL_STRING("TIMESTAMP_TZ",1);
 			break;
 #endif
 		case SQLT_DAT:
@@ -1210,7 +1191,7 @@ PHP_FUNCTION(oci_field_type)
 			RETVAL_STRING("LONG RAW",1);
 			break;
 		case SQLT_CHR:
-			RETVAL_STRING("VARCHAR2",1);
+			RETVAL_STRING("VARCHAR",1);
 			break;
 		case SQLT_RSET:
 			RETVAL_STRING("REFCURSOR",1);
@@ -1242,7 +1223,7 @@ PHP_FUNCTION(oci_field_type_raw)
 {
 	php_oci_out_column *column;
 
-	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
+	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU); 
 	if (column) {
 		RETURN_LONG(column->data_type);
 	}
@@ -1256,7 +1237,7 @@ PHP_FUNCTION(oci_field_is_null)
 {
 	php_oci_out_column *column;
 
-	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0) ) ) {
+	if ( ( column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU) ) ) {
 		if (column->indicator == -1) {
 			RETURN_TRUE;
 		}
@@ -1395,13 +1376,7 @@ PHP_FUNCTION(oci_fetch_all)
 				if (flags & PHP_OCI_NUM) {
 					zend_hash_next_index_insert(Z_ARRVAL_P(row), &element, sizeof(zval*), NULL);
 				} else { /* default to ASSOC */
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION > 1) || (PHP_MAJOR_VERSION > 5)
-					/* zend_symtable_update is only available in 5.2+ */
-					zend_symtable_update(Z_ARRVAL_P(row), columns[ i ]->name, columns[ i ]->name_len+1, &element, sizeof(zval*), NULL);
-#else
-					/* This code path means Bug #45458 will remain broken when OCI8 is built with PHP 4 */
 					zend_hash_update(Z_ARRVAL_P(row), columns[ i ]->name, columns[ i ]->name_len+1, &element, sizeof(zval*), NULL);
-#endif
 				}
 			}
 
@@ -1433,13 +1408,7 @@ PHP_FUNCTION(oci_fetch_all)
 				
 				MAKE_STD_ZVAL(tmp);
 				array_init(tmp);
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION > 1) || (PHP_MAJOR_VERSION > 5)
-				/* zend_symtable_update is only available in 5.2+ */
-				zend_symtable_update(Z_ARRVAL_P(array), columns[ i ]->name, columns[ i ]->name_len+1, (void *) &tmp, sizeof(zval*), (void **) &(outarrs[ i ]));
-#else
-				/* This code path means Bug #45458 will remain broken when OCI8 is built with PHP 4 */
 				zend_hash_update(Z_ARRVAL_P(array), columns[ i ]->name, columns[ i ]->name_len+1, (void *) &tmp, sizeof(zval*), (void **) &(outarrs[ i ]));
-#endif
 			}
 		}
 
@@ -1460,7 +1429,7 @@ PHP_FUNCTION(oci_fetch_all)
 		
 		efree(columns);
 		efree(outarrs);
-	}
+	} 
 
 	RETURN_LONG(rows);
 }
@@ -1470,7 +1439,7 @@ PHP_FUNCTION(oci_fetch_all)
    Fetch a result row as an object */
 PHP_FUNCTION(oci_fetch_object)
 {
-	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_ASSOC | PHP_OCI_RETURN_NULLS, 2);
+	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_ASSOC, 2);
 
 	if (Z_TYPE_P(return_value) == IS_ARRAY) {
 		object_and_properties_init(return_value, ZEND_STANDARD_CLASS_DEF_PTR, Z_ARRVAL_P(return_value));
@@ -1482,7 +1451,7 @@ PHP_FUNCTION(oci_fetch_object)
    Fetch a result row as an enumerated array */
 PHP_FUNCTION(oci_fetch_row)
 {
-	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_NUM | PHP_OCI_RETURN_NULLS, 1);
+	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_NUM, 1);
 }
 /* }}} */
 
@@ -1490,7 +1459,7 @@ PHP_FUNCTION(oci_fetch_row)
    Fetch a result row as an associative array */
 PHP_FUNCTION(oci_fetch_assoc)
 {
-	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_ASSOC | PHP_OCI_RETURN_NULLS, 1);
+	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_ASSOC, 1);
 }
 /* }}} */
 
@@ -1498,7 +1467,7 @@ PHP_FUNCTION(oci_fetch_assoc)
    Fetch a result row as an array */
 PHP_FUNCTION(oci_fetch_array)
 {
-	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_BOTH | PHP_OCI_RETURN_NULLS, 2);
+	php_oci_fetch_row(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_OCI_BOTH, 2);
 }
 /* }}} */
 
@@ -1514,8 +1483,8 @@ PHP_FUNCTION(oci_free_statement)
 	}
 
 	PHP_OCI_ZVAL_TO_STATEMENT(z_statement, statement);
-
 	zend_list_delete(statement->id);
+	
 	RETURN_TRUE;
 }
 /* }}} */
@@ -1524,13 +1493,6 @@ PHP_FUNCTION(oci_free_statement)
    Disconnect from database */
 PHP_FUNCTION(oci_close)
 {
-	/* oci_close for pconnect (if old_oci_close_semantics not set) would
-	 * release the connection back to the client-side session pool (and to the
-	 * server-side pool if Database Resident Connection Pool is being used).
-	 * Subsequent pconnects in the same script are not guaranteed to get the
-	 * same database session.
-	 */
-
 	zval *z_connection;
 	php_oci_connection *connection;
 
@@ -1545,7 +1507,6 @@ PHP_FUNCTION(oci_close)
 
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 	zend_list_delete(connection->rsrc_id);
-
 	ZVAL_NULL(z_connection);
 	
 	RETURN_TRUE;
@@ -1560,7 +1521,7 @@ PHP_FUNCTION(oci_new_connect)
 }
 /* }}} */
 
-/* {{{ proto resource oci_connect(string user, string pass [, string db [, string charset [, int session_mode ]])
+/* {{{ proto resource oci_connect(string user, string pass [, string db])
    Connect to an Oracle database and log on. Returns a new session. */
 PHP_FUNCTION(oci_connect)
 {
@@ -1568,7 +1529,7 @@ PHP_FUNCTION(oci_connect)
 }
 /* }}} */
 
-/* {{{ proto resource oci_pconnect(string user, string pass [, string db [, string charset ]])
+/* {{{ proto resource oci_pconnect(string user, string pass [, string db])
    Connect to an Oracle database using a persistent connection and log on. Returns a new session. */
 PHP_FUNCTION(oci_pconnect)
 {
@@ -1580,15 +1541,17 @@ PHP_FUNCTION(oci_pconnect)
    Return the last error of stmt|connection|global. If no error happened returns false. */
 PHP_FUNCTION(oci_error)
 {
-	zval *arg = NULL;
+	zval *arg;
 	php_oci_statement *statement;
 	php_oci_connection *connection;
 	text *errbuf;
 	sb4 errcode = 0;
-	sword error = OCI_SUCCESS;
+	sword error = 0;
 	dvoid *errh = NULL;
+#ifdef HAVE_OCI8_ATTR_STATEMENT
 	ub2 error_offset = 0;
 	text *sqltext = NULL;
+#endif
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &arg) == FAILURE) {
 		return;
@@ -1601,37 +1564,30 @@ PHP_FUNCTION(oci_error)
 			errh = statement->err;
 			error = statement->errcode;
 
+#ifdef HAVE_OCI8_ATTR_STATEMENT
 			if (php_oci_fetch_sqltext_offset(statement, &sqltext, &error_offset TSRMLS_CC)) {
 				RETURN_FALSE;
 			}
-			goto go_out;
-		}
-
-		connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_connection);
-		if (connection) {
-			errh = connection->err;
-			error = connection->errcode;
-			goto go_out;
-		}
-
-		connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_pconnection);
-		if (connection) {
-			errh = connection->err;
-			error = connection->errcode;
-			goto go_out;
+#endif
+		} else {
+			connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_connection);
+			
+			if (connection) {
+				errh = connection->err;
+				error = connection->errcode;
+			}
 		}
 	} else {
 		errh = OCI_G(err);
 		error = OCI_G(errcode);
 	}
 
-go_out:
 	if (error == OCI_SUCCESS) { /* no error set in the handle */
 		RETURN_FALSE;
 	}
 
 	if (!errh) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Oci_error: unable to find error handle");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "OCIError: unable to find error handle");
 		RETURN_FALSE;
 	}
 
@@ -1641,8 +1597,10 @@ go_out:
 		array_init(return_value);
 		add_assoc_long(return_value, "code", errcode);
 		add_assoc_string(return_value, "message", (char*) errbuf, 0);
+#ifdef HAVE_OCI8_ATTR_STATEMENT
 		add_assoc_long(return_value, "offset", error_offset);
 		add_assoc_string(return_value, "sqltext", sqltext ? (char *) sqltext : "", 1);
+#endif
 	} else {
 		RETURN_FALSE;
 	}
@@ -1675,14 +1633,15 @@ PHP_FUNCTION(oci_parse)
 	php_oci_statement *statement;
 	char *query;
 	int query_len;
+	zend_bool cached = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &z_connection, &query, &query_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &z_connection, &query, &query_len, &cached) == FAILURE) {
 		return;
 	}
 
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 
-	statement = php_oci_statement_create(connection, query, query_len TSRMLS_CC);
+	statement = php_oci_statement_create(connection, query, query_len, cached TSRMLS_CC);
 
 	if (statement) {
 		RETURN_RESOURCE(statement->id);
@@ -1712,166 +1671,16 @@ PHP_FUNCTION(oci_set_prefetch)
 }
 /* }}} */
 
-/* {{{ proto bool oci_set_client_identifier(resource connection, string value)
-  Sets the client identifier attribute on the connection */
-PHP_FUNCTION(oci_set_client_identifier)
-{
-	zval *z_connection;
-	php_oci_connection *connection;
-	char *client_id;
-	long client_id_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &z_connection, &client_id, &client_id_len) == FAILURE) {
-		return;
-	}
-
-	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
-
-	PHP_OCI_CALL_RETURN(OCI_G(errcode), OCIAttrSet, ((dvoid *) connection->session, (ub4) OCI_HTYPE_SESSION, (dvoid *) client_id, (ub4) client_id_len, (ub4) OCI_ATTR_CLIENT_IDENTIFIER, OCI_G(err)));
-
-	if (OCI_G(errcode) != OCI_SUCCESS) {
-		php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
-		RETURN_FALSE;
-	}
-
-	RETURN_TRUE;
-}
-/* }}} */
-
-/* {{{ proto bool oci_set_edition(string value)
-  Sets the edition attribute for all subsequent connections created */
-PHP_FUNCTION(oci_set_edition)
-{
-#if ((OCI_MAJOR_VERSION > 11) || ((OCI_MAJOR_VERSION == 11) && (OCI_MINOR_VERSION >= 2)))
-	char *edition;
-	long edition_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &edition, &edition_len) == FAILURE) {
-		return;
-	}
-
-	if (OCI_G(edition)) {
-		efree(OCI_G(edition));
-		OCI_G(edition) = NULL;
-	}
-
-	if (edition) {
-		OCI_G(edition) = (char *)safe_emalloc(edition_len+1, sizeof(text), 0);
-		memcpy(OCI_G(edition), edition, edition_len);
-		OCI_G(edition)[edition_len] = '\0';
-	}
-
-	RETURN_TRUE;
-#else
-	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Unsupported attribute type");
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto bool oci_set_module_name(resource connection, string value)
-  Sets the module attribute on the connection */
-PHP_FUNCTION(oci_set_module_name)
-{
-#if (OCI_MAJOR_VERSION >= 10)
-	zval *z_connection;
-	php_oci_connection *connection;
-	char *module;
-	long module_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &z_connection, &module, &module_len) == FAILURE) {
-		return;
-	}
-
-	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
-
-	PHP_OCI_CALL_RETURN(OCI_G(errcode), OCIAttrSet, ((dvoid *) connection->session, (ub4) OCI_HTYPE_SESSION, (dvoid *) module, (ub4) module_len, (ub4) OCI_ATTR_MODULE, OCI_G(err)));
-
-	if (OCI_G(errcode) != OCI_SUCCESS) {
-		php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
-		RETURN_FALSE;
-	}
-
-	RETURN_TRUE;
-#else
-	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Unsupported attribute type");
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto bool oci_set_action(resource connection, string value)
-  Sets the action attribute on the connection */
-PHP_FUNCTION(oci_set_action)
-{
-#if (OCI_MAJOR_VERSION >= 10)
-	zval *z_connection;
-	php_oci_connection *connection;
-	char *action;
-	long action_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &z_connection, &action, &action_len) == FAILURE) {
-		return;
-	}
-
-	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
-
-	PHP_OCI_CALL_RETURN(OCI_G(errcode), OCIAttrSet, ((dvoid *) connection->session, (ub4) OCI_HTYPE_SESSION, (dvoid *) action, (ub4) action_len, (ub4) OCI_ATTR_ACTION, OCI_G(err)));
-
-	if (OCI_G(errcode) != OCI_SUCCESS) {
-		php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
-		RETURN_FALSE;
-	}
-
-	RETURN_TRUE;
-#else
-	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Unsupported attribute type");
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto bool oci_set_client_info(resource connection, string value)
-  Sets the client info attribute on the connection */
-PHP_FUNCTION(oci_set_client_info)
-{
-#if (OCI_MAJOR_VERSION >= 10)
-	zval *z_connection;
-	php_oci_connection *connection;
-	char *client_info;
-	long client_info_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &z_connection, &client_info, &client_info_len) == FAILURE) {
-		return;
-	}
-
-	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
-
-	PHP_OCI_CALL_RETURN(OCI_G(errcode), OCIAttrSet, ((dvoid *) connection->session, (ub4) OCI_HTYPE_SESSION, (dvoid *) client_info, (ub4) client_info_len, (ub4) OCI_ATTR_CLIENT_INFO, OCI_G(err)));
-
-	if (OCI_G(errcode) != OCI_SUCCESS) {
-		php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
-		RETURN_FALSE;
-	}
-
-	RETURN_TRUE;
-#else
-	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Unsupported attribute type");
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
 /* {{{ proto bool oci_password_change(resource connection, string username, string old_password, string new_password)
   Changes the password of an account */
 PHP_FUNCTION(oci_password_change)
 {
 	zval *z_connection;
-	char *user, *pass_old, *pass_new, *dbname;
+	text *user, *pass_old, *pass_new, *dbname;
 	int user_len, pass_old_len, pass_new_len, dbname_len;
 	php_oci_connection *connection;
 
-	/*	Disable in Safe Mode  */
+	/*  Disable in Safe Mode  */
 	if (PG(safe_mode)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "is disabled in Safe Mode");
 		RETURN_FALSE;
@@ -1880,38 +1689,12 @@ PHP_FUNCTION(oci_password_change)
 	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "rsss", &z_connection, &user, &user_len, &pass_old, &pass_old_len, &pass_new, &pass_new_len) == SUCCESS) {
 		PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 
-		if (!user_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "username cannot be empty");
-			RETURN_FALSE;
-		}
-		if (!pass_old_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "old password cannot be empty");
-			RETURN_FALSE;
-		}
-		if (!pass_new_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "new password cannot be empty");
-			RETURN_FALSE;
-		}
-
 		if (php_oci_password_change(connection, user, user_len, pass_old, pass_old_len, pass_new, pass_new_len TSRMLS_CC)) {
 			RETURN_FALSE;
 		}
 		RETURN_TRUE;
 	} else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ssss", &dbname, &dbname_len, &user, &user_len, &pass_old, &pass_old_len, &pass_new, &pass_new_len) == SUCCESS) {
 
-		if (!user_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "username cannot be empty");
-			RETURN_FALSE;
-		}
-		if (!pass_old_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "old password cannot be empty");
-			RETURN_FALSE;
-		}
-		if (!pass_new_len) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "new password cannot be empty");
-			RETURN_FALSE;
-		}
-
 		connection = php_oci_do_connect_ex(user, user_len, pass_old, pass_old_len, pass_new, pass_new_len, dbname, dbname_len, NULL, OCI_DEFAULT, 0, 0 TSRMLS_CC);
 		if (!connection) {
 			RETURN_FALSE;
@@ -1936,7 +1719,7 @@ PHP_FUNCTION(oci_new_cursor)
 
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 
-	statement = php_oci_statement_create(connection, NULL, 0 TSRMLS_CC);
+	statement = php_oci_statement_create(connection, NULL, 0, 0 TSRMLS_CC);
 	
 	if (statement) {
 		RETURN_RESOURCE(statement->id);
@@ -1951,7 +1734,7 @@ PHP_FUNCTION(oci_result)
 {
 	php_oci_out_column *column;
 	
-	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
+	column = php_oci_statement_get_column_helper(INTERNAL_FUNCTION_PARAM_PASSTHRU);
 	if(column) {
 		php_oci_column_to_zval(column, return_value, 0 TSRMLS_CC);
 	}
@@ -2029,9 +1812,6 @@ PHP_FUNCTION(oci_statement_type)
 		case OCI_STMT_DECLARE:
 			RETVAL_STRING("DECLARE",1);
 			break;
-		case OCI_STMT_CALL:
-			RETVAL_STRING("CALL",1);
-			break;
 		default:
 			RETVAL_STRING("UNKNOWN",1);
 	}
@@ -2059,6 +1839,7 @@ PHP_FUNCTION(oci_num_rows)
 }
 /* }}} */
 
+#ifdef PHP_OCI8_HAVE_COLLECTIONS
 /* {{{ proto bool oci_free_collection()
    Deletes collection object*/
 PHP_FUNCTION(oci_free_collection)
@@ -2330,7 +2111,7 @@ PHP_FUNCTION(oci_new_collection)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|s", &z_connection, &tdo, &tdo_len, &schema, &schema_len) == FAILURE) {
 		return;
 	}
-	
+    
 	PHP_OCI_ZVAL_TO_CONNECTION(z_connection, connection);
 	
 	if ( (collection = php_oci_collection_create(connection, tdo, tdo_len, schema, schema_len TSRMLS_CC)) ) {
@@ -2343,13 +2124,6 @@ PHP_FUNCTION(oci_new_collection)
 }
 /* }}} */
 
-#endif /* HAVE_OCI8 */
+#endif
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+#endif /* HAVE_OCI8 */
