@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: oci8_lob.c 294441 2010-02-03 19:37:35Z pajoye $ */
+/* $Id: oci8_lob.c,v 1.7.2.4 2006/01/05 13:42:35 tony2001 Exp $ */
 
 
 
@@ -62,56 +62,45 @@ php_oci_descriptor *php_oci_lob_create (php_oci_connection *connection, long typ
 			/* these three are allowed */
 			break;
 		default:
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown descriptor type %ld", type);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown descriptor type %ld.", type);
 			return NULL;
 			break;
 	}
 
 	descriptor = ecalloc(1, sizeof(php_oci_descriptor));
 	descriptor->type = type;
-	descriptor->connection = connection;
-	zend_list_addref(descriptor->connection->rsrc_id);
 
-	PHP_OCI_CALL_RETURN(OCI_G(errcode), OCIDescriptorAlloc, (connection->env, (dvoid*)&(descriptor->descriptor), descriptor->type, (size_t) 0, (dvoid **) 0));
+	OCI_G(errcode) = PHP_OCI_CALL(OCIDescriptorAlloc, (connection->env, (dvoid*)&(descriptor->descriptor), descriptor->type, (size_t) 0, (dvoid **) 0));
 
 	if (OCI_G(errcode) != OCI_SUCCESS) {
-		OCI_G(errcode) = php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
-		PHP_OCI_HANDLE_ERROR(connection, OCI_G(errcode));
+		connection->errcode = php_oci_error(OCI_G(err), OCI_G(errcode) TSRMLS_CC);
+		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		efree(descriptor);
 		return NULL;
 	}
 
+	descriptor->connection = connection;
+
 	PHP_OCI_REGISTER_RESOURCE(descriptor, le_descriptor);
 	
 	descriptor->lob_current_position = 0;
 	descriptor->lob_size = -1;				/* we should set it to -1 to know, that it's just not initialized */
 	descriptor->buffering = PHP_OCI_LOB_BUFFER_DISABLED;				/* buffering is off by default */
-	descriptor->charset_form = SQLCS_IMPLICIT;	/* default value */
-	descriptor->charset_id = connection->charset;
-	descriptor->is_open = 0;
 
 	if (descriptor->type == OCI_DTYPE_LOB || descriptor->type == OCI_DTYPE_FILE) {
 		/* add Lobs & Files to hash. we'll flush them at the end */
 		if (!connection->descriptors) {
 			ALLOC_HASHTABLE(connection->descriptors);
 			zend_hash_init(connection->descriptors, 0, NULL, php_oci_descriptor_flush_hash_dtor, 0);
-			connection->descriptor_count = 0;
-		}
-		
-		descriptor->index = (connection->descriptor_count)++;
-		if (connection->descriptor_count == LONG_MAX) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Internal descriptor counter has reached limit");
-			php_oci_connection_descriptors_free(connection TSRMLS_CC);
-			return NULL;
 		}
 
-		zend_hash_index_update(connection->descriptors,descriptor->index,&descriptor,sizeof(php_oci_descriptor *),NULL);
+		zend_hash_next_index_insert(connection->descriptors,&descriptor,sizeof(php_oci_descriptor *),NULL);
 	}
 	return descriptor;
 
 } /* }}} */
 
-/* {{{ php_oci_lob_get_length()
+/* {{{ php_oci_lob_get_length() 
  Get length of the LOB. The length is cached so we don't need to ask Oracle every time */
 int php_oci_lob_get_length (php_oci_descriptor *descriptor, ub4 *length TSRMLS_DC)
 {
@@ -124,18 +113,18 @@ int php_oci_lob_get_length (php_oci_descriptor *descriptor, ub4 *length TSRMLS_D
 		return 0;
 	} else {
 		if (descriptor->type == OCI_DTYPE_FILE) {
-			PHP_OCI_CALL_RETURN(connection->errcode, OCILobFileOpen, (connection->svc, connection->err, descriptor->descriptor, OCI_FILE_READONLY));
+			connection->errcode = PHP_OCI_CALL(OCILobFileOpen, (connection->svc, connection->err, descriptor->descriptor, OCI_FILE_READONLY));
 			if (connection->errcode != OCI_SUCCESS) {
-				connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+				php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 				PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 				return 1;
 			}
 		}
 		
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobGetLength, (connection->svc, connection->err, descriptor->descriptor, (ub4 *)length));
+		connection->errcode = PHP_OCI_CALL(OCILobGetLength, (connection->svc, connection->err, descriptor->descriptor, (ub4 *)length));
 
 		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+			php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 			return 1;
 		}
@@ -143,10 +132,10 @@ int php_oci_lob_get_length (php_oci_descriptor *descriptor, ub4 *length TSRMLS_D
 		descriptor->lob_size = *length;
 
 		if (descriptor->type == OCI_DTYPE_FILE) {
-			PHP_OCI_CALL_RETURN(connection->errcode, OCILobFileClose, (connection->svc, connection->err, descriptor->descriptor));
+			connection->errcode = PHP_OCI_CALL(OCILobFileClose, (connection->svc, connection->err, descriptor->descriptor));
 
 			if (connection->errcode != OCI_SUCCESS) {
-				connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+				php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 				PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 				return 1;
 			}
@@ -155,109 +144,19 @@ int php_oci_lob_get_length (php_oci_descriptor *descriptor, ub4 *length TSRMLS_D
 	return 0;	
 } /* }}} */
 
-/* {{{ php_oci_lob_callback()
-   Append LOB portion to a memory buffer */
-#if defined(HAVE_OCI_LOB_READ2)
-sb4 php_oci_lob_callback (dvoid *ctxp, CONST dvoid *bufxp, oraub8 len, ub1 piece, dvoid **changed_bufpp, oraub8 *changed_lenp)
-#else
-sb4 php_oci_lob_callback (dvoid *ctxp, CONST dvoid *bufxp, ub4 len, ub1 piece)
-#endif
-{
-	ub4 lenp = (ub4) len;
-	php_oci_lob_ctx *ctx = (php_oci_lob_ctx *)ctxp;
-
-	switch (piece)
-	{
-		case OCI_LAST_PIECE:
-			if ((*(ctx->lob_len) + lenp) > (ctx->alloc_len)) {
-				/* this should not happen ever */
-				*(ctx->lob_data) = NULL;
-				*(ctx->lob_len) = 0;
-				return OCI_ERROR;
-			}
-			memcpy(*(ctx->lob_data) + *(ctx->lob_len), bufxp, (size_t) lenp);
-			*(ctx->lob_len) += lenp;
-			*(*(ctx->lob_data) + *(ctx->lob_len)) = 0x00;
-			return OCI_CONTINUE;
-
-		case OCI_FIRST_PIECE:
-		case OCI_NEXT_PIECE:
-			if ((*(ctx->lob_len) + lenp) > ctx->alloc_len) {
-				/* this should not happen ever */
-				*(ctx->lob_data) = NULL;
-				*(ctx->lob_len) = 0;
-				return OCI_ERROR;
-			}
-			memcpy(*(ctx->lob_data) + *(ctx->lob_len), bufxp, (size_t) lenp);
-			*(ctx->lob_len) += lenp;
-			return OCI_CONTINUE;
-
-		default: {
-			TSRMLS_FETCH();
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unexpected LOB piece id received (value:%d)", piece);
-			*(ctx->lob_data) = NULL;
-			*(ctx->lob_len) = 0;
-			return OCI_ERROR;
-		}
-	}
-}
-/* }}} */
-
-/* {{{ php_oci_lob_calculate_buffer() */
-static inline int php_oci_lob_calculate_buffer(php_oci_descriptor *descriptor, long read_length TSRMLS_DC)
-{
-	php_oci_connection *connection = descriptor->connection;
-	ub4 chunk_size;
-
-	if (descriptor->type == OCI_DTYPE_FILE) {
-		return read_length;
-	}
-
-	if (!descriptor->chunk_size) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobGetChunkSize, (connection->svc, connection->err, descriptor->descriptor, &chunk_size));
-
-		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
-			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
-			return read_length; /* we have to return original length here */
-		}
-		descriptor->chunk_size = chunk_size;
-	}
-	
-	if ((read_length % descriptor->chunk_size) != 0) {
-		return descriptor->chunk_size * ((read_length / descriptor->chunk_size) + 1);
-	}
-	return read_length;
-}
-/* }}} */
-
-/* {{{ php_oci_lob_read()
+/* {{{ php_oci_lob_read() 
  Read specified portion of the LOB into the buffer */
 int php_oci_lob_read (php_oci_descriptor *descriptor, long read_length, long initial_offset, char **data, ub4 *data_len TSRMLS_DC)
 {
 	php_oci_connection *connection = descriptor->connection;
 	ub4 length = 0;
-	int buffer_size = PHP_OCI_LOB_BUFFER_SIZE;
-	php_oci_lob_ctx ctx;
-	ub1 *bufp;
-#if defined(HAVE_OCI_LOB_READ2)
-	oraub8 bytes_read, offset = 0;
-	oraub8 requested_len = read_length; /* this is by default */
-	oraub8 chars_read = 0;
-#else
-	int bytes_read, offset = 0;
+	ub4 block_length = PHP_OCI_LOB_BUFFER_SIZE;
+	int bytes_read, bytes_total = 0, offset = 0, data_len_chars = 0;
 	int requested_len = read_length; /* this is by default */
-#endif
-	int is_clob = 0;
-	sb4 bytes_per_char = 1;
 
 	*data_len = 0;
 	*data = NULL;
 
-	ctx.lob_len = data_len;
-	ctx.lob_data = data;
-	ctx.alloc_len = 0;
-
 	if (php_oci_lob_get_length(descriptor, &length TSRMLS_CC)) {
 		return 1;
 	}
@@ -265,7 +164,7 @@ int php_oci_lob_read (php_oci_descriptor *descriptor, long read_length, long ini
 	if (length <= 0) {
 		return 0;
 	}
-	
+ 	
 	if (initial_offset > length) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Offset must be less than size of the LOB");
 		return 1;
@@ -279,154 +178,94 @@ int php_oci_lob_read (php_oci_descriptor *descriptor, long read_length, long ini
 		requested_len = length - initial_offset;
 	}
 	
-	if (requested_len <= 0) {
+	if (requested_len == 0) {
 		return 0;
 	}
-	
-	offset = initial_offset;
 
+	if (requested_len < block_length) {
+		block_length = requested_len;
+	}
+	
 	if (descriptor->type == OCI_DTYPE_FILE) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobFileOpen, (connection->svc, connection->err, descriptor->descriptor, OCI_FILE_READONLY));
+		connection->errcode = PHP_OCI_CALL(OCILobFileOpen, (connection->svc, connection->err, descriptor->descriptor, OCI_FILE_READONLY));
 
 		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+			php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 			return 1;
 		}
-	} else {
-		ub2 charset_id = 0;
+	}
 
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobCharSetId, (connection->env, connection->err, descriptor->descriptor, &charset_id));
+	*data = (char *)emalloc(block_length + 1);
+	bytes_read = block_length;
+	offset = initial_offset;
+		
+	do {
+		connection->errcode = PHP_OCI_CALL(OCILobRead, 
+			(
+				connection->svc, 
+				connection->err, 
+				descriptor->descriptor, 
+				&bytes_read,								/* IN/OUT bytes toread/read */ 
+				offset + 1,								/* offset (starts with 1) */ 
+				(dvoid *) ((char *) *data + *data_len),	
+				block_length,									/* size of buffer */ 
+				(dvoid *)0, 
+				(OCICallbackLobRead) 0,					/* callback... */ 
+				(ub2) connection->charset,	/* The character set ID of the buffer data. */ 
+				(ub1) SQLCS_IMPLICIT					/* The character set form of the buffer data. */
+			)
+		);
 
-		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
-			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
-			return 1;
+		bytes_total += bytes_read;
+		/* 
+		 * Oracle doesn't tell use how many CHARS were read, 
+		 * so we have to count them to get the correct offset for CLOBS */
+		data_len_chars = OCIMultiByteStrnDisplayLength(connection->env, *data, bytes_total); 
+		offset = initial_offset + data_len_chars;
+		
+		*data_len += bytes_read;
+		block_length = PHP_OCI_LOB_BUFFER_SIZE;
+		
+		if (connection->errcode != OCI_NEED_DATA) {
+			break;
 		}
+		*data = erealloc(*data, *data_len + PHP_OCI_LOB_BUFFER_SIZE + 1);	
+	} while (connection->errcode == OCI_NEED_DATA);
 
-		if (charset_id > 0) { /* charset_id is always > 0 for [N]CLOBs */
-			is_clob = 1;
-		}
-	}
-
-	if (is_clob) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCINlsNumericInfoGet, (connection->env, connection->err, &bytes_per_char, OCI_NLS_CHARSET_MAXBYTESZ));
-
-		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
-			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
-			return 1;
-		}
-	} else {
-		/* BLOBs don't have encoding, so bytes_per_char == 1 */
-	}
-
-	ctx.alloc_len = (requested_len + 1) * bytes_per_char;
-	*data = ecalloc(bytes_per_char, requested_len + 1);
-
-#ifdef HAVE_OCI_LOB_READ2
-	if (is_clob) {
-		chars_read = requested_len;
-		bytes_read = 0;
-	} else {
-		chars_read = 0;
-		bytes_read = requested_len;
-	}
-
-	buffer_size = (requested_len < buffer_size ) ? requested_len : buffer_size;		/* optimize buffer size */
-	buffer_size = php_oci_lob_calculate_buffer(descriptor, buffer_size TSRMLS_CC);	/* use chunk size */
-
-	bufp = (ub1 *) ecalloc(1, buffer_size);
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobRead2,
-		(
-			connection->svc,
-			connection->err,
-			descriptor->descriptor,
-			(oraub8 *)&bytes_read,							/* IN/OUT bytes toread/read */
-			(oraub8 *)&chars_read,							/* IN/OUT chars toread/read */
-			(oraub8) offset + 1,							/* offset (starts with 1) */
-			(dvoid *) bufp,
-			(oraub8) buffer_size,							/* size of buffer */
-			OCI_FIRST_PIECE,
-			(dvoid *)&ctx,
-			(OCICallbackLobRead2) php_oci_lob_callback,				/* callback... */
-			(ub2) descriptor->charset_id,			   /* The character set ID of the buffer data. */
-			(ub1) descriptor->charset_form					  /* The character set form of the buffer data. */
-		)
-	);
-	
-	efree(bufp);
-	
-	if (is_clob) {
-		offset = descriptor->lob_current_position + chars_read;
-	} else {
-		offset = descriptor->lob_current_position + bytes_read;
-	}
-
-#else
-
-	bytes_read = requested_len;
-	buffer_size = (requested_len < buffer_size ) ? requested_len : buffer_size;		/* optimize buffer size */
-	buffer_size = php_oci_lob_calculate_buffer(descriptor, buffer_size TSRMLS_CC);	/* use chunk size */
-
-	bufp = (ub1 *) ecalloc(1, buffer_size);
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobRead,
-		(
-			 connection->svc,
-			 connection->err,
-			 descriptor->descriptor,
-			 &bytes_read,								 /* IN/OUT bytes toread/read */
-			 offset + 1,							 /* offset (starts with 1) */
-			 (dvoid *) bufp,
-			 (ub4) buffer_size,							 /* size of buffer */
-			 (dvoid *)&ctx,
-			 (OCICallbackLobRead) php_oci_lob_callback,				 /* callback... */
-			 (ub2) descriptor->charset_id,				/* The character set ID of the buffer data. */
-			 (ub1) descriptor->charset_form					   /* The character set form of the buffer data. */
-		)
-	);
-	
-	efree(bufp);
-	offset = descriptor->lob_current_position + bytes_read;
-
-#endif
-	
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
-		if (*data) {
-			efree(*data);
-			*data = NULL;
-		}
-		*data_len = 0;
+		efree(*data);
+		*data = NULL;
 		return 1;
 	}
 	
-	descriptor->lob_current_position = (int)offset;
+	descriptor->lob_current_position += data_len_chars; 
 
 	if (descriptor->type == OCI_DTYPE_FILE) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobFileClose, (connection->svc, connection->err, descriptor->descriptor));
+		connection->errcode = PHP_OCI_CALL(OCILobFileClose, (connection->svc, connection->err, descriptor->descriptor));
 
 		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+			php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
-			if (*data) {
-				efree(*data);
-				*data = NULL;
-			}
-			*data_len = 0;
+			efree(*data);
+			*data = NULL;
 			return 1;
 		}
 	}
 
+	*data = erealloc(*data, *data_len + 1);
+	(*data)[ *data_len ] = 0;
+
 	return 0;
 } /* }}} */
 
-/* {{{ php_oci_lob_write()
+/* {{{ php_oci_lob_write() 
  Write data to the LOB */
 int php_oci_lob_write (php_oci_descriptor *descriptor, ub4 offset, char *data, int data_len, ub4 *bytes_written TSRMLS_DC)
 {
-	OCILobLocator *lob		   = (OCILobLocator *) descriptor->descriptor;
+	OCILobLocator *lob         = (OCILobLocator *) descriptor->descriptor;
 	php_oci_connection *connection = (php_oci_connection *) descriptor->connection;
 	ub4 lob_length;
 	
@@ -447,25 +286,10 @@ int php_oci_lob_write (php_oci_descriptor *descriptor, ub4 offset, char *data, i
 		offset = descriptor->lob_current_position;
 	}
 	
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobWrite,
-			(
-				connection->svc,
-				connection->err,
-				lob,
-				(ub4 *)&data_len,
-				(ub4) offset + 1,
-				(dvoid *) data,
-				(ub4) data_len,
-				OCI_ONE_PIECE,
-				(dvoid *)0,
-				(OCICallbackLobWrite) 0,
-				(ub2) descriptor->charset_id,
-				(ub1) descriptor->charset_form
-			)
-		);
+	connection->errcode = PHP_OCI_CALL(OCILobWrite, (connection->svc, connection->err, lob, (ub4 *)&data_len, (ub4) offset + 1, (dvoid *) data, (ub4) data_len, OCI_ONE_PIECE, (dvoid *)0, (OCICallbackLobWrite) 0, (ub2) 0, (ub1) SQLCS_IMPLICIT));
 
 	if (connection->errcode) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		*bytes_written = 0;
 		return 1;
@@ -485,7 +309,7 @@ int php_oci_lob_write (php_oci_descriptor *descriptor, ub4 offset, char *data, i
 	return 0;
 } /* }}} */
 
-/* {{{ php_oci_lob_set_buffering()
+/* {{{ php_oci_lob_set_buffering() 
  Turn buffering off/onn for this particular LOB */
 int php_oci_lob_set_buffering (php_oci_descriptor *descriptor, int on_off TSRMLS_DC)
 {
@@ -502,13 +326,14 @@ int php_oci_lob_set_buffering (php_oci_descriptor *descriptor, int on_off TSRMLS
 	}
 	
 	if (on_off) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobEnableBuffering, (connection->svc, connection->err, descriptor->descriptor));
-	} else {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobDisableBuffering, (connection->svc, connection->err, descriptor->descriptor));
+		connection->errcode = PHP_OCI_CALL(OCILobEnableBuffering, (connection->svc, connection->err, descriptor->descriptor));
+	}
+	else {
+		connection->errcode = PHP_OCI_CALL(OCILobDisableBuffering, (connection->svc, connection->err, descriptor->descriptor));
 	}
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -518,11 +343,12 @@ int php_oci_lob_set_buffering (php_oci_descriptor *descriptor, int on_off TSRMLS
 
 /* {{{ php_oci_lob_get_buffering()
  Return current buffering state for the LOB */
-int php_oci_lob_get_buffering (php_oci_descriptor *descriptor)
+int php_oci_lob_get_buffering (php_oci_descriptor *descriptor TSRMLS_DC)
 {
 	if (descriptor->buffering != PHP_OCI_LOB_BUFFER_DISABLED) {
 		return 1;
-	} else {
+	} 
+	else {
 		return 0;
 	}
 } /* }}} */
@@ -532,7 +358,7 @@ int php_oci_lob_get_buffering (php_oci_descriptor *descriptor)
 int php_oci_lob_copy (php_oci_descriptor *descriptor_dest, php_oci_descriptor *descriptor_from, long length TSRMLS_DC)
 {
 	php_oci_connection *connection = descriptor_dest->connection;
-	ub4 length_dest, length_from, copy_len;
+	int length_dest, length_from, copy_len;
 	
 	if (php_oci_lob_get_length(descriptor_dest, &length_dest TSRMLS_CC)) {
 		return 1;
@@ -544,7 +370,8 @@ int php_oci_lob_copy (php_oci_descriptor *descriptor_dest, php_oci_descriptor *d
 
 	if (length == -1) {
 		copy_len = length_from - descriptor_from->lob_current_position;
-	} else {
+	}
+	else {
 		copy_len = length;
 	}
 
@@ -553,20 +380,10 @@ int php_oci_lob_copy (php_oci_descriptor *descriptor_dest, php_oci_descriptor *d
 		return 1;
 	}
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobCopy,
-			(
-			 connection->svc,
-			 connection->err,
-			 descriptor_dest->descriptor,
-			 descriptor_from->descriptor,
-			 copy_len,
-			 descriptor_dest->lob_current_position+1,
-			 descriptor_from->lob_current_position+1
-			)
-	);
+	connection->errcode = PHP_OCI_CALL(OCILobCopy, (connection->svc, connection->err, descriptor_dest->descriptor, descriptor_from->descriptor, copy_len, descriptor_dest->lob_current_position+1, descriptor_from->lob_current_position+1));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -574,49 +391,35 @@ int php_oci_lob_copy (php_oci_descriptor *descriptor_dest, php_oci_descriptor *d
 	return 0;
 } /* }}} */
 
-/* {{{ php_oci_lob_close()
+/* {{{ php_oci_lob_close() 
  Close LOB */
 int php_oci_lob_close (php_oci_descriptor *descriptor TSRMLS_DC)
 {
 	php_oci_connection *connection = descriptor->connection;
+	int is_temporary;
 	
-	if (descriptor->is_open) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobClose, (connection->svc, connection->err, descriptor->descriptor));
-	}
+	connection->errcode = PHP_OCI_CALL(OCILobClose, (connection->svc, connection->err, descriptor->descriptor));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
-
-	if (php_oci_temp_lob_close(descriptor TSRMLS_CC)) {
-		return 1;
-	}
 	
-	return 0;
-} /* }}} */
-
-/* {{{ php_oci_temp_lob_close()
-   Close Temporary LOB */
-int php_oci_temp_lob_close (php_oci_descriptor *descriptor TSRMLS_DC)
-{
-	php_oci_connection *connection = descriptor->connection;
-	int is_temporary;
-
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobIsTemporary, (connection->env,connection->err, descriptor->descriptor, &is_temporary));
+	connection->errcode = PHP_OCI_CALL(OCILobIsTemporary, (connection->env,connection->err, descriptor->descriptor, &is_temporary));
 	
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
 	
 	if (is_temporary) {
-		PHP_OCI_CALL_RETURN(connection->errcode, OCILobFreeTemporary, (connection->svc, connection->err, descriptor->descriptor));
+		
+		connection->errcode = PHP_OCI_CALL(OCILobFreeTemporary, (connection->svc, connection->err, descriptor->descriptor));
 		
 		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+			php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 			return 1;
 		}
@@ -624,10 +427,9 @@ int php_oci_temp_lob_close (php_oci_descriptor *descriptor TSRMLS_DC)
 	return 0;
 } /* }}} */
 
-
-/* {{{ php_oci_lob_flush()
+/* {{{ php_oci_lob_flush() 
  Flush buffers for the LOB (only if they have been used) */
-int php_oci_lob_flush(php_oci_descriptor *descriptor, long flush_flag TSRMLS_DC)
+int php_oci_lob_flush(php_oci_descriptor *descriptor, int flush_flag TSRMLS_DC)
 {
 	OCILobLocator *lob = descriptor->descriptor;
 	php_oci_connection *connection = descriptor->connection;
@@ -642,7 +444,7 @@ int php_oci_lob_flush(php_oci_descriptor *descriptor, long flush_flag TSRMLS_DC)
 			/* only these two are allowed */
 			break;
 		default:
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid flag value: %ld", flush_flag);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid flag value: %d", flush_flag);
 			return 1;
 			break;
 	}
@@ -654,10 +456,10 @@ int php_oci_lob_flush(php_oci_descriptor *descriptor, long flush_flag TSRMLS_DC)
 		return 0;
 	}
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobFlushBuffer, (connection->svc, connection->err, lob, flush_flag));
+	connection->errcode = PHP_OCI_CALL(OCILobFlushBuffer, (connection->svc, connection->err, lob, flush_flag));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -671,31 +473,14 @@ int php_oci_lob_flush(php_oci_descriptor *descriptor, long flush_flag TSRMLS_DC)
  Close LOB descriptor and free associated resources */
 void php_oci_lob_free (php_oci_descriptor *descriptor TSRMLS_DC)
 {
+
 	if (!descriptor || !descriptor->connection) {
 		return;
 	}
 
 	if (descriptor->connection->descriptors) {
 		/* delete descriptor from the hash */
-		zend_hash_index_del(descriptor->connection->descriptors, descriptor->index);
-		if (zend_hash_num_elements(descriptor->connection->descriptors) == 0) {
-			descriptor->connection->descriptor_count = 0;
-		} else {
-			if (descriptor->index + 1 == descriptor->connection->descriptor_count) {
-				/* If the descriptor being freed is the end-most one
-				 * allocated, then the descriptor_count is reduced so
-				 * a future descriptor can reuse the hash table index.
-				 * This can prevent the hash index range increasing in
-				 * the common case that each descriptor is
-				 * allocated/used/freed before another descriptor is
-				 * needed.  However it is possible that a script frees
-				 * descriptors in arbitrary order which would prevent
-				 * descriptor_count ever being reduced to zero until
-				 * zend_hash_num_elements() returns 0.
-				 */
-				descriptor->connection->descriptor_count--;
-			}
-		}
+		zend_hash_apply_with_argument(descriptor->connection->descriptors, php_oci_descriptor_delete_from_hash, (void *)&descriptor->id TSRMLS_CC);
 	}
 	
 	/* flushing Lobs & Files with buffering enabled */
@@ -703,10 +488,6 @@ void php_oci_lob_free (php_oci_descriptor *descriptor TSRMLS_DC)
 		php_oci_lob_flush(descriptor, OCI_LOB_BUFFER_FREE TSRMLS_CC);
 	}
 
-	if (descriptor->type == OCI_DTYPE_LOB) {
-		php_oci_temp_lob_close(descriptor TSRMLS_CC);
-	}
-
 	PHP_OCI_CALL(OCIDescriptorFree, (descriptor->descriptor, descriptor->type));
 
 	zend_list_delete(descriptor->connection->rsrc_id);
@@ -717,8 +498,7 @@ void php_oci_lob_free (php_oci_descriptor *descriptor TSRMLS_DC)
  Import LOB contents from the given file */
 int php_oci_lob_import (php_oci_descriptor *descriptor, char *filename TSRMLS_DC)
 {
-	int fp;
-	ub4 loblen;
+	int fp, loblen;
 	OCILobLocator *lob = (OCILobLocator *)descriptor->descriptor;
 	php_oci_connection *connection = descriptor->connection;
 	char buf[8192];
@@ -734,26 +514,26 @@ int php_oci_lob_import (php_oci_descriptor *descriptor, char *filename TSRMLS_DC
 	}
 
 	while ((loblen = read(fp, &buf, sizeof(buf))) > 0) {	
-		PHP_OCI_CALL_RETURN(connection->errcode,
+		connection->errcode = PHP_OCI_CALL(
 				OCILobWrite,
 				(
-					connection->svc,
-					connection->err,
-					lob,
-					&loblen,
-					offset,
-					(dvoid *) &buf,
-					loblen,
-					OCI_ONE_PIECE,
-					(dvoid *)0,
-					(OCICallbackLobWrite) 0,
-					(ub2) descriptor->charset_id,
-					(ub1) descriptor->charset_form
+					connection->svc, 
+					connection->err, 
+					lob, 
+					&loblen, 
+					(ub4) offset, 
+					(dvoid *) &buf, 
+					(ub4) loblen, 
+					OCI_ONE_PIECE, 
+					(dvoid *)0, 
+					(OCICallbackLobWrite) 0, 
+					(ub2) 0, 
+					(ub1) SQLCS_IMPLICIT
 				)
 		);
 
 		if (connection->errcode != OCI_SUCCESS) {
-			connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+			php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 			PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 			close(fp);
 			return 1;
@@ -786,10 +566,10 @@ int php_oci_lob_append (php_oci_descriptor *descriptor_dest, php_oci_descriptor
 		return 0;
 	}
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobAppend, (connection->svc, connection->err, lob_dest, lob_from));
+	connection->errcode = PHP_OCI_CALL(OCILobAppend, (connection->svc, connection->err, lob_dest, lob_from));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -822,10 +602,10 @@ int php_oci_lob_truncate (php_oci_descriptor *descriptor, long new_lob_length TS
 		return 1;
 	}
 	
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobTrim, (connection->svc, connection->err, lob, new_lob_length));
+	connection->errcode = PHP_OCI_CALL(OCILobTrim, (connection->svc, connection->err, lob, new_lob_length));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -836,7 +616,7 @@ int php_oci_lob_truncate (php_oci_descriptor *descriptor, long new_lob_length TS
 
 /* {{{ php_oci_lob_erase()
  Erase (or fill with whitespaces, depending on LOB type) the LOB (or its part) */
-int php_oci_lob_erase (php_oci_descriptor *descriptor, long offset, ub4 length, ub4 *bytes_erased TSRMLS_DC)
+int php_oci_lob_erase (php_oci_descriptor *descriptor, long offset, long length, ub4 *bytes_erased TSRMLS_DC)
 {
 	php_oci_connection *connection = descriptor->connection;
 	OCILobLocator *lob = descriptor->descriptor;
@@ -856,10 +636,10 @@ int php_oci_lob_erase (php_oci_descriptor *descriptor, long offset, ub4 length,
 		length = lob_length;
 	}
 	
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobErase, (connection->svc, connection->err, lob, (ub4 *)&length, offset+1));
+	connection->errcode = PHP_OCI_CALL(OCILobErase, (connection->svc, connection->err, lob, (ub4 *)&length, offset+1));
 
 	if (connection->errcode != OCI_SUCCESS) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
@@ -868,7 +648,7 @@ int php_oci_lob_erase (php_oci_descriptor *descriptor, long offset, ub4 length,
 	return 0;
 } /* }}} */
 
-/* {{{ php_oci_lob_is_equal()
+/* {{{ php_oci_lob_is_equal() 
  Compare two LOB descriptors and figure out if they are pointing to the same LOB */
 int php_oci_lob_is_equal (php_oci_descriptor *descriptor_first, php_oci_descriptor *descriptor_second, boolean *result TSRMLS_DC)
 {
@@ -876,22 +656,22 @@ int php_oci_lob_is_equal (php_oci_descriptor *descriptor_first, php_oci_descript
 	OCILobLocator *first_lob   = descriptor_first->descriptor;
 	OCILobLocator *second_lob  = descriptor_second->descriptor;
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobIsEqual, (connection->env, first_lob, second_lob, result));
+	connection->errcode = PHP_OCI_CALL(OCILobIsEqual, (connection->env, first_lob, second_lob, result));
 
 	if (connection->errcode) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
 	return 0;
 } /* }}} */
 
-/* {{{ php_oci_lob_write_tmp()
+/* {{{ php_oci_lob_write_tmp() 
  Create temporary LOB and write data to it */
 int php_oci_lob_write_tmp (php_oci_descriptor *descriptor, ub1 type, char *data, int data_len TSRMLS_DC)
 {
 	php_oci_connection *connection = descriptor->connection;
-	OCILobLocator *lob		   = descriptor->descriptor;
+	OCILobLocator *lob         = descriptor->descriptor;
 	ub4 bytes_written = 0;
 	
 	switch (type) {
@@ -905,49 +685,28 @@ int php_oci_lob_write_tmp (php_oci_descriptor *descriptor, ub1 type, char *data,
 			break;
 	}
 
-	if (data_len < 0) {
+	if (!data || data_len <= 0) {
+		/* nothing to write, silently fail */
 		return 1;
 	}
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobCreateTemporary,
-			(
-			 connection->svc,
-			 connection->err,
-			 lob,
-			 OCI_DEFAULT,
-			 OCI_DEFAULT,
-			 type,
-			 OCI_ATTR_NOCACHE,
-			 OCI_DURATION_SESSION
-			)
-	);
+	connection->errcode = PHP_OCI_CALL(OCILobCreateTemporary, (connection->svc, connection->err, lob, OCI_DEFAULT, OCI_DEFAULT, type, OCI_ATTR_NOCACHE, OCI_DURATION_SESSION));
 
 	if (connection->errcode) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
 
-	PHP_OCI_CALL_RETURN(connection->errcode, OCILobOpen, (connection->svc, connection->err, lob, OCI_LOB_READWRITE));
+	connection->errcode = PHP_OCI_CALL(OCILobOpen, (connection->svc, connection->err, lob, OCI_LOB_READWRITE));
 
 	if (connection->errcode) {
-		connection->errcode = php_oci_error(connection->err, connection->errcode TSRMLS_CC);
+		php_oci_error(connection->err, connection->errcode TSRMLS_CC);
 		PHP_OCI_HANDLE_ERROR(connection, connection->errcode);
 		return 1;
 	}
 
-	descriptor->is_open = 1;
-
 	return php_oci_lob_write(descriptor, 0, data, data_len, &bytes_written TSRMLS_CC);
 } /* }}} */
 
 #endif /* HAVE_OCI8 */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
