@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,30 +16,23 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: xp_ssl.c,v 1.22.2.2 2006/01/01 12:50:10 sniper Exp $ */
 
 #include "php.h"
 #include "ext/standard/file.h"
-#include "ext/standard/url.h"
 #include "streams/php_streams_int.h"
-#include "ext/standard/php_smart_str.h"
 #include "php_network.h"
 #include "php_openssl.h"
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
 #include <openssl/err.h>
 
-#ifdef PHP_WIN32
-#include "win32/time.h"
-#endif
-
 #ifdef NETWARE
 #include <sys/select.h>
 #endif
 
 int php_openssl_apply_verification_policy(SSL *ssl, X509 *peer, php_stream *stream TSRMLS_DC);
 SSL *php_SSL_new_from_context(SSL_CTX *ctx, php_stream *stream TSRMLS_DC);
-int php_openssl_get_x509_list_id(void);
 
 /* This implementation is very closely tied to the that of the native
  * sockets implemented in the core.
@@ -49,13 +42,10 @@ int php_openssl_get_x509_list_id(void);
 typedef struct _php_openssl_netstream_data_t {
 	php_netstream_data_t s;
 	SSL *ssl_handle;
-	SSL_CTX *ctx;
-	struct timeval connect_timeout;
 	int enable_on_connect;
 	int is_client;
 	int ssl_active;
 	php_stream_xport_crypt_method_t method;
-	char *sni;
 	unsigned state_set:1;
 	unsigned _spare:31;
 } php_openssl_netstream_data_t;
@@ -66,20 +56,17 @@ php_stream_ops php_openssl_socket_ops;
  * in an error condition arising from a network connection problem */
 static int is_http_stream_talking_to_iis(php_stream *stream TSRMLS_DC)
 {
-	if (stream->wrapperdata && stream->wrapper && strcasecmp(stream->wrapper->wops->label, "HTTP") == 0) {
+	if (stream->wrapperdata && stream->wrapper && strcmp(stream->wrapper->wops->label, "HTTP") == 0) {
 		/* the wrapperdata is an array zval containing the headers */
 		zval **tmp;
 
 #define SERVER_MICROSOFT_IIS	"Server: Microsoft-IIS"
-#define SERVER_GOOGLE "Server: GFE/"
 		
 		zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream->wrapperdata));
 		while (SUCCESS == zend_hash_get_current_data(Z_ARRVAL_P(stream->wrapperdata), (void**)&tmp)) {
 
 			if (strncasecmp(Z_STRVAL_PP(tmp), SERVER_MICROSOFT_IIS, sizeof(SERVER_MICROSOFT_IIS)-1) == 0) {
 				return 1;
-			} else if (strncasecmp(Z_STRVAL_PP(tmp), SERVER_GOOGLE, sizeof(SERVER_GOOGLE)-1) == 0) {
-				return 1;
 			}
 			
 			zend_hash_move_forward(Z_ARRVAL_P(stream->wrapperdata));
@@ -88,13 +75,14 @@ static int is_http_stream_talking_to_iis(php_stream *stream TSRMLS_DC)
 	return 0;
 }
 
-static int handle_ssl_error(php_stream *stream, int nr_bytes, zend_bool is_init TSRMLS_DC)
+static int handle_ssl_error(php_stream *stream, int nr_bytes TSRMLS_DC)
 {
 	php_openssl_netstream_data_t *sslsock = (php_openssl_netstream_data_t*)stream->abstract;
 	int err = SSL_get_error(sslsock->ssl_handle, nr_bytes);
 	char esbuf[512];
-	smart_str ebuf = {0};
-	unsigned long ecode;
+	char *ebuf = NULL, *wptr = NULL;
+	size_t ebuf_size = 0;
+	unsigned long code, ecode;
 	int retry = 1;
 
 	switch(err) {
@@ -107,12 +95,12 @@ static int handle_ssl_error(php_stream *stream, int nr_bytes, zend_bool is_init
 			/* re-negotiation, or perhaps the SSL layer needs more
 			 * packets: retry in next iteration */
 			errno = EAGAIN;
-			retry = is_init ? 1 : sslsock->s.is_blocked;
+			retry = sslsock->s.is_blocked;
 			break;
 		case SSL_ERROR_SYSCALL:
 			if (ERR_peek_error() == 0) {
 				if (nr_bytes == 0) {
-					if (!is_http_stream_talking_to_iis(stream TSRMLS_CC) && ERR_get_error() != 0) {
+					if (!is_http_stream_talking_to_iis(stream TSRMLS_CC)) {
 						php_error_docref(NULL TSRMLS_CC, E_WARNING,
 								"SSL: fatal protocol error");
 					}
@@ -145,24 +133,33 @@ static int handle_ssl_error(php_stream *stream, int nr_bytes, zend_bool is_init
 
 				default:
 					do {
-						/* NULL is automatically added */
-						ERR_error_string_n(ecode, esbuf, sizeof(esbuf));
-						if (ebuf.c) {
-							smart_str_appendc(&ebuf, '\n');
+						/* allow room for a NUL and an optional \n */
+						if (ebuf) {
+							esbuf[0] = '\n';
+							esbuf[1] = '\0';
+							ERR_error_string_n(ecode, esbuf + 1, sizeof(esbuf) - 2);
+						} else {
+							esbuf[0] = '\0';
+							ERR_error_string_n(ecode, esbuf, sizeof(esbuf) - 1);
 						}
-						smart_str_appends(&ebuf, esbuf);
-					} while ((ecode = ERR_get_error()) != 0);
+						code = strlen(esbuf);
+						esbuf[code] = '\0';
 
-					smart_str_0(&ebuf);
+						ebuf = erealloc(ebuf, ebuf_size + code + 1);
+						if (wptr == NULL) {
+							wptr = ebuf;
+						}	
+
+						/* also copies the NUL */
+						memcpy(wptr, esbuf, code + 1);
+						wptr += code;
+					} while ((ecode = ERR_get_error()) != 0);
 
 					php_error_docref(NULL TSRMLS_CC, E_WARNING,
 							"SSL operation failed with code %d. %s%s",
 							err,
-							ebuf.c ? "OpenSSL Error messages:\n" : "",
-							ebuf.c ? ebuf.c : "");
-					if (ebuf.c) {
-						smart_str_free(&ebuf);
-					}
+							ebuf ? "OpenSSL Error messages:\n" : "",
+							ebuf ? ebuf : "");
 			}
 				
 			retry = 0;
@@ -184,18 +181,19 @@ static size_t php_openssl_sockop_write(php_stream *stream, const char *buf, size
 			didwrite = SSL_write(sslsock->ssl_handle, buf, count);
 
 			if (didwrite <= 0) {
-				retry = handle_ssl_error(stream, didwrite, 0 TSRMLS_CC);
+				retry = handle_ssl_error(stream, didwrite TSRMLS_CC);
 			} else {
 				break;
 			}
 		} while(retry);
-
-		if (didwrite > 0) {
-			php_stream_notify_progress_increment(stream->context, didwrite, 0);
-		}
+		
 	} else {
 		didwrite = php_stream_socket_ops.write(stream, buf, count TSRMLS_CC);
 	}
+	
+	if (didwrite > 0) {
+		php_stream_notify_progress_increment(stream->context, didwrite, 0);
+	}
 
 	if (didwrite < 0) {
 		didwrite = 0;
@@ -204,65 +202,19 @@ static size_t php_openssl_sockop_write(php_stream *stream, const char *buf, size
 	return didwrite;
 }
 
-static void php_openssl_stream_wait_for_data(php_netstream_data_t *sock)
-{
-	int retval;
-	struct timeval *ptimeout;
-
-	if (sock->socket == -1) {
-		return;
-	}
-	
-	sock->timeout_event = 0;
-
-	if (sock->timeout.tv_sec == -1)
-		ptimeout = NULL;
-	else
-		ptimeout = &sock->timeout;
-
-	while(1) {
-		retval = php_pollfd_for(sock->socket, PHP_POLLREADABLE, ptimeout);
-
-		if (retval == 0)
-			sock->timeout_event = 1;
-
-		if (retval >= 0)
-			break;
-
-		if (php_socket_errno() != EINTR)
-			break;
-	}
-}
-
 static size_t php_openssl_sockop_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
 {
 	php_openssl_netstream_data_t *sslsock = (php_openssl_netstream_data_t*)stream->abstract;
-	php_netstream_data_t *sock;
 	int nr_bytes = 0;
 
 	if (sslsock->ssl_active) {
 		int retry = 1;
-		sock = (php_netstream_data_t*)stream->abstract;
-
-		/* The SSL_read() function will block indefinitely waiting for data on a blocking
-		   socket. If we don't poll for readability first this operation has the potential
-		   to hang forever. To avoid this scenario we poll with a timeout before performing
-		   the actual read. If it times out we're finished.
-		*/
-		if (sock->is_blocked) {
-			php_openssl_stream_wait_for_data(sock);
-			if (sock->timeout_event) {
-				stream->eof = 1;
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL read operation timed out");
-				return nr_bytes;
-			}
-		}
 
 		do {
 			nr_bytes = SSL_read(sslsock->ssl_handle, buf, count);
 
 			if (nr_bytes <= 0) {
-				retry = handle_ssl_error(stream, nr_bytes, 0 TSRMLS_CC);
+				retry = handle_ssl_error(stream, nr_bytes TSRMLS_CC);
 				stream->eof = (retry == 0 && errno != EAGAIN && !SSL_pending(sslsock->ssl_handle));
 				
 			} else {
@@ -270,16 +222,16 @@ static size_t php_openssl_sockop_read(php_stream *stream, char *buf, size_t coun
 				break;
 			}
 		} while (retry);
-
-		if (nr_bytes > 0) {
-			php_stream_notify_progress_increment(stream->context, nr_bytes, 0);
-		}
 	}
 	else
 	{
 		nr_bytes = php_stream_socket_ops.read(stream, buf, count TSRMLS_CC);
 	}
 
+	if (nr_bytes > 0) {
+		php_stream_notify_progress_increment(stream->context, nr_bytes, 0);
+	}
+
 	if (nr_bytes < 0) {
 		nr_bytes = 0;
 	}
@@ -303,14 +255,6 @@ static int php_openssl_sockop_close(php_stream *stream, int close_handle TSRMLS_
 			SSL_free(sslsock->ssl_handle);
 			sslsock->ssl_handle = NULL;
 		}
-		if (sslsock->ctx) {
-			SSL_CTX_free(sslsock->ctx);
-			sslsock->ctx = NULL;
-		}
-#ifdef PHP_WIN32
-		if (sslsock->s.socket == -1)
-			sslsock->s.socket = SOCK_ERR;
-#endif
 		if (sslsock->s.socket != SOCK_ERR) {
 #ifdef PHP_WIN32
 			/* prevent more data from coming in */
@@ -320,7 +264,7 @@ static int php_openssl_sockop_close(php_stream *stream, int close_handle TSRMLS_
 			 * Essentially, we are waiting for the socket to become writeable, which means
 			 * that all pending data has been sent.
 			 * We use a small timeout which should encourage the OS to send the data,
-			 * but at the same time avoid hanging indefinitely.
+			 * but at the same time avoid hanging indefintely.
 			 * */
 			do {
 				n = php_pollfd_for_ms(sslsock->s.socket, POLLOUT, 500);
@@ -331,9 +275,6 @@ static int php_openssl_sockop_close(php_stream *stream, int close_handle TSRMLS_
 		}
 	}
 
-	if (sslsock->sni) {
-		pefree(sslsock->sni, php_stream_is_persistent(stream));
-	}
 	pefree(sslsock, php_stream_is_persistent(stream));
 	
 	return 0;
@@ -355,16 +296,12 @@ static inline int php_openssl_setup_crypto(php_stream *stream,
 		php_stream_xport_crypto_param *cparam
 		TSRMLS_DC)
 {
+	SSL_CTX *ctx;
 	SSL_METHOD *method;
-	long ssl_ctx_options = SSL_OP_ALL;
 	
 	if (sslsock->ssl_handle) {
-		if (sslsock->s.is_blocked) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS already set-up for this stream");
-			return -1;
-		} else {
-			return 0;
-		}
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS already set-up for this stream");
+		return -1;
 	}
 
 	/* need to do slightly different things, based on client/server method,
@@ -376,14 +313,9 @@ static inline int php_openssl_setup_crypto(php_stream *stream,
 			method = SSLv23_client_method();
 			break;
 		case STREAM_CRYPTO_METHOD_SSLv2_CLIENT:
-#ifdef OPENSSL_NO_SSL2
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSLv2 support is not compiled into the OpenSSL library PHP is linked against");
-			return -1;
-#else
 			sslsock->is_client = 1;
 			method = SSLv2_client_method();
 			break;
-#endif
 		case STREAM_CRYPTO_METHOD_SSLv3_CLIENT:
 			sslsock->is_client = 1;
 			method = SSLv3_client_method();
@@ -401,14 +333,9 @@ static inline int php_openssl_setup_crypto(php_stream *stream,
 			method = SSLv3_server_method();
 			break;
 		case STREAM_CRYPTO_METHOD_SSLv2_SERVER:
-#ifdef OPENSSL_NO_SSL2
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSLv2 support is not compiled into the OpenSSL library PHP is linked against");
-			return -1;
-#else
 			sslsock->is_client = 0;
 			method = SSLv2_server_method();
 			break;
-#endif
 		case STREAM_CRYPTO_METHOD_TLS_SERVER:
 			sslsock->is_client = 0;
 			method = TLSv1_server_method();
@@ -418,58 +345,28 @@ static inline int php_openssl_setup_crypto(php_stream *stream,
 
 	}
 
-	sslsock->ctx = SSL_CTX_new(method);
-	if (sslsock->ctx == NULL) {
+	ctx = SSL_CTX_new(method);
+	if (ctx == NULL) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create an SSL context");
 		return -1;
 	}
 
-#if OPENSSL_VERSION_NUMBER >= 0x0090605fL
-	ssl_ctx_options &= ~SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS;
-#endif
-	SSL_CTX_set_options(sslsock->ctx, ssl_ctx_options);
+	SSL_CTX_set_options(ctx, SSL_OP_ALL);
 
-#if OPENSSL_VERSION_NUMBER >= 0x0090806fL
-	{
-		zval **val;
-
-		if (stream->context && SUCCESS == php_stream_context_get_option(
-					stream->context, "ssl", "no_ticket", &val) && 
-				zval_is_true(*val)) {
-			SSL_CTX_set_options(sslsock->ctx, SSL_OP_NO_TICKET);
-		}
-	}
-#endif
-
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-	{
-		zval **val;
-
-		if (stream->context && SUCCESS == php_stream_context_get_option(
-					stream->context, "ssl", "disable_compression", &val) &&
-				zval_is_true(*val)) {
-			SSL_CTX_set_options(sslsock->ctx, SSL_OP_NO_COMPRESSION);
-		}
-	}
-#endif
-
-	sslsock->ssl_handle = php_SSL_new_from_context(sslsock->ctx, stream TSRMLS_CC);
+	sslsock->ssl_handle = php_SSL_new_from_context(ctx, stream TSRMLS_CC);
 	if (sslsock->ssl_handle == NULL) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create an SSL handle");
-		SSL_CTX_free(sslsock->ctx);
-		sslsock->ctx = NULL;
+		SSL_CTX_free(ctx);
 		return -1;
 	}
 
 	if (!SSL_set_fd(sslsock->ssl_handle, sslsock->s.socket)) {
-		handle_ssl_error(stream, 0, 1 TSRMLS_CC);
+		handle_ssl_error(stream, 0 TSRMLS_CC);
 	}
 
 	if (cparam->inputs.session) {
 		if (cparam->inputs.session->ops != &php_openssl_socket_ops) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "supplied session stream must be an SSL enabled stream");
- 		} else if (((php_openssl_netstream_data_t*)cparam->inputs.session->abstract)->ssl_handle == NULL) {
- 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "supplied SSL session stream is not initialized");
 		} else {
 			SSL_copy_session_id(sslsock->ssl_handle, ((php_openssl_netstream_data_t*)cparam->inputs.session->abstract)->ssl_handle);
 		}
@@ -485,17 +382,6 @@ static inline int php_openssl_enable_crypto(php_stream *stream,
 	int n, retry = 1;
 
 	if (cparam->inputs.activate && !sslsock->ssl_active) {
-		struct timeval	start_time,
-						*timeout;
-		int				blocked		= sslsock->s.is_blocked,
-						has_timeout = 0;
-
-#if OPENSSL_VERSION_NUMBER >= 0x0090806fL && !defined(OPENSSL_NO_TLSEXT)
-		if (sslsock->is_client && sslsock->sni) {
-			SSL_set_tlsext_host_name(sslsock->ssl_handle, sslsock->sni);
-		}
-#endif
-
 		if (!sslsock->state_set) {
 			if (sslsock->is_client) {
 				SSL_set_connect_state(sslsock->ssl_handle);
@@ -505,73 +391,20 @@ static inline int php_openssl_enable_crypto(php_stream *stream,
 			sslsock->state_set = 1;
 		}
 	
-		if (SUCCESS == php_set_sock_blocking(sslsock->s.socket, 0 TSRMLS_CC)) {
-			sslsock->s.is_blocked = 0;
-		}
-		
-		timeout = sslsock->is_client ? &sslsock->connect_timeout : &sslsock->s.timeout;
-		has_timeout = !sslsock->s.is_blocked && (timeout->tv_sec || timeout->tv_usec);
-		/* gettimeofday is not monotonic; using it here is not strictly correct */
-		if (has_timeout) {
-			gettimeofday(&start_time, NULL);
-		}
-		
 		do {
-			struct timeval	cur_time,
-							elapsed_time = {0};
-			
 			if (sslsock->is_client) {
 				n = SSL_connect(sslsock->ssl_handle);
 			} else {
 				n = SSL_accept(sslsock->ssl_handle);
 			}
 
-			if (has_timeout) {
-				gettimeofday(&cur_time, NULL);
-				elapsed_time.tv_sec  = cur_time.tv_sec  - start_time.tv_sec;
-				elapsed_time.tv_usec = cur_time.tv_usec - start_time.tv_usec;
-				if (cur_time.tv_usec < start_time.tv_usec) {
-					elapsed_time.tv_sec  -= 1L;
-					elapsed_time.tv_usec += 1000000L;
-				}
-			
-				if (elapsed_time.tv_sec > timeout->tv_sec ||
-						(elapsed_time.tv_sec == timeout->tv_sec &&
-						elapsed_time.tv_usec > timeout->tv_usec)) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL: crypto enabling timeout");
-					return -1;
-				}
-			}
-
 			if (n <= 0) {
-				/* in case of SSL_ERROR_WANT_READ/WRITE, do not retry in non-blocking mode */
-				retry = handle_ssl_error(stream, n, blocked TSRMLS_CC);
-				if (retry) {
-					/* wait until something interesting happens in the socket. It may be a
-					 * timeout. Also consider the unlikely of possibility of a write block  */
-					int err = SSL_get_error(sslsock->ssl_handle, n);
-					struct timeval left_time;
-					
-					if (has_timeout) {
-						left_time.tv_sec  = timeout->tv_sec  - elapsed_time.tv_sec;
-						left_time.tv_usec =	timeout->tv_usec - elapsed_time.tv_usec;
-						if (timeout->tv_usec < elapsed_time.tv_usec) {
-							left_time.tv_sec  -= 1L;
-							left_time.tv_usec += 1000000L;
-						}
-					}
-					php_pollfd_for(sslsock->s.socket, (err == SSL_ERROR_WANT_READ) ?
-						(POLLIN|POLLPRI) : POLLOUT, has_timeout ? &left_time : NULL);
-				}
+				retry = handle_ssl_error(stream, n TSRMLS_CC);
 			} else {
-				retry = 0;
+				break;
 			}
 		} while (retry);
 
-		if (sslsock->s.is_blocked != blocked && SUCCESS == php_set_sock_blocking(sslsock->s.socket, blocked TSRMLS_CC)) {
-			sslsock->s.is_blocked = blocked;
-		}
-
 		if (n == 1) {
 			X509 *peer_cert;
 
@@ -579,70 +412,11 @@ static inline int php_openssl_enable_crypto(php_stream *stream,
 
 			if (FAILURE == php_openssl_apply_verification_policy(sslsock->ssl_handle, peer_cert, stream TSRMLS_CC)) {
 				SSL_shutdown(sslsock->ssl_handle);
-				n = -1;
 			} else {	
 				sslsock->ssl_active = 1;
-
-				/* allow the script to capture the peer cert
-				 * and/or the certificate chain */
-				if (stream->context) {
-					zval **val, *zcert;
-
-					if (SUCCESS == php_stream_context_get_option(
-								stream->context, "ssl",
-								"capture_peer_cert", &val) &&
-							zval_is_true(*val)) {
-						MAKE_STD_ZVAL(zcert);
-						ZVAL_RESOURCE(zcert, zend_list_insert(peer_cert, 
-									php_openssl_get_x509_list_id() TSRMLS_CC));
-						php_stream_context_set_option(stream->context,
-								"ssl", "peer_certificate",
-								zcert);
-						peer_cert = NULL;
-						FREE_ZVAL(zcert);
-					}
-
-					if (SUCCESS == php_stream_context_get_option(
-								stream->context, "ssl",
-								"capture_peer_cert_chain", &val) &&
-							zval_is_true(*val)) {
-						zval *arr;
-						STACK_OF(X509) *chain;
-
-						MAKE_STD_ZVAL(arr);
-						chain = SSL_get_peer_cert_chain(
-									sslsock->ssl_handle);
-
-						if (chain && sk_X509_num(chain) > 0) {
-							int i;
-							array_init(arr);
-
-							for (i = 0; i < sk_X509_num(chain); i++) {
-								X509 *mycert = X509_dup(
-										sk_X509_value(chain, i));
-								MAKE_STD_ZVAL(zcert);
-								ZVAL_RESOURCE(zcert,
-										zend_list_insert(mycert,
-											php_openssl_get_x509_list_id() TSRMLS_CC));
-								add_next_index_zval(arr, zcert);
-							}
-
-						} else {
-							ZVAL_NULL(arr);
-						}
-
-						php_stream_context_set_option(stream->context,
-								"ssl", "peer_certificate_chain",
-								arr);
-						zval_dtor(arr);
-						efree(arr);
-					}
-				}
 			}
 
-			if (peer_cert) {
-				X509_free(peer_cert);
-			}
+			X509_free(peer_cert);
 		} else  {
 			n = errno == EAGAIN ? 0 : -1;
 		}
@@ -692,9 +466,6 @@ static inline int php_openssl_tcp_sockop_accept(php_stream *stream, php_openssl_
 			xparam->outputs.client = php_stream_alloc_rel(stream->ops, clisockdata, NULL, "r+");
 			if (xparam->outputs.client) {
 				xparam->outputs.client->context = stream->context;
-				if (stream->context) {
-					zend_list_addref(stream->context->rsrc_id);
-				}
 			}
 		}
 
@@ -713,8 +484,6 @@ static inline int php_openssl_tcp_sockop_accept(php_stream *stream, php_openssl_
 				case STREAM_CRYPTO_METHOD_TLS_CLIENT:
 					sock->method = STREAM_CRYPTO_METHOD_TLS_SERVER;
 					break;
-				default:
-					break;
 			}
 
 			clisockdata->method = sock->method;
@@ -751,7 +520,7 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 						tv.tv_sec = FG(default_socket_timeout);
 						tv.tv_usec = 0;
 					} else {
-						tv = sslsock->connect_timeout;
+						tv = sslsock->s.timeout;
 					}
 				} else {
 					tv.tv_sec = value;
@@ -821,11 +590,7 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 					 * we notice that the connect has actually been established */
 					php_stream_socket_ops.set_option(stream, option, value, ptrparam TSRMLS_CC);
 
-					if ((sslsock->enable_on_connect) &&
-						((xparam->outputs.returncode == 0) ||
-						(xparam->op == STREAM_XPORT_OP_CONNECT_ASYNC && 
-						xparam->outputs.returncode == 1 && xparam->outputs.error_code == EINPROGRESS)))
-					{
+					if (xparam->outputs.returncode == 0 && sslsock->enable_on_connect) {
 						if (php_stream_xport_crypto_setup(stream, sslsock->method, NULL TSRMLS_CC) < 0 ||
 								php_stream_xport_crypto_enable(stream, 1 TSRMLS_CC) < 0) {
 							php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to enable crypto");
@@ -871,20 +636,7 @@ static int php_openssl_sockop_cast(php_stream *stream, int castas, void **ret TS
 
 		case PHP_STREAM_AS_FD_FOR_SELECT:
 			if (ret) {
-				if (sslsock->ssl_active) {
-					/* OpenSSL has an internal buffer which select() cannot see. If we don't
-					   fetch it into the stream's buffer, no activity will be reported on the
-					   stream even though there is data waiting to be read - but we only fetch
-					   the number of bytes OpenSSL has ready to give us since we weren't asked
-					   for any data at this stage. This is only likely to cause issues with
-					   non-blocking streams, but it's harmless to always do it. */
-					int bytes;
-					while ((bytes = SSL_pending(sslsock->ssl_handle)) > 0) {
-						php_stream_fill_read_buffer(stream, (size_t)bytes);
-					}
-				}
-
-				*(int *)ret = sslsock->s.socket;
+				*ret = (void*)sslsock->s.socket;
 			}
 			return SUCCESS;
 
@@ -894,7 +646,7 @@ static int php_openssl_sockop_cast(php_stream *stream, int castas, void **ret TS
 				return FAILURE;
 			}
 			if (ret) {
-				*(int *)ret = sslsock->s.socket;
+				*ret = (void*)sslsock->s.socket;
 			}
 			return SUCCESS;
 		default:
@@ -912,52 +664,6 @@ php_stream_ops php_openssl_socket_ops = {
 	php_openssl_sockop_set_option,
 };
 
-static char * get_sni(php_stream_context *ctx, char *resourcename, long resourcenamelen, int is_persistent TSRMLS_DC) {
-
-	php_url *url;
-
-	if (ctx) {
-		zval **val = NULL;
-
-		if (php_stream_context_get_option(ctx, "ssl", "SNI_enabled", &val) == SUCCESS && !zend_is_true(*val)) {
-			return NULL;
-		}
-		if (php_stream_context_get_option(ctx, "ssl", "SNI_server_name", &val) == SUCCESS) {
-			convert_to_string_ex(val);
-			return pestrdup(Z_STRVAL_PP(val), is_persistent);
-		}
-	}
-
-	if (!resourcename) {
-		return NULL;
-	}
-
-	url = php_url_parse_ex(resourcename, resourcenamelen);
-	if (!url) {
-		return NULL;
-	}
-
-	if (url->host) {
-		const char * host = url->host;
-		char * sni = NULL;
-		size_t len = strlen(host);
-
-		/* skip trailing dots */
-		while (len && host[len-1] == '.') {
-			--len;
-		}
-
-		if (len) {
-			sni = pestrndup(host, len, is_persistent);
-		}
-
-		php_url_free(url);
-		return sni;
-	}
-
-	php_url_free(url);
-	return NULL;
-}
 
 php_stream *php_openssl_ssl_socket_factory(const char *proto, long protolen,
 		char *resourcename, long resourcenamelen,
@@ -972,21 +678,13 @@ php_stream *php_openssl_ssl_socket_factory(const char *proto, long protolen,
 	memset(sslsock, 0, sizeof(*sslsock));
 
 	sslsock->s.is_blocked = 1;
-	/* this timeout is used by standard stream funcs, therefor it should use the default value */
 	sslsock->s.timeout.tv_sec = FG(default_socket_timeout);
 	sslsock->s.timeout.tv_usec = 0;
 
-	/* use separate timeout for our private funcs */
-	sslsock->connect_timeout.tv_sec = timeout->tv_sec;
-	sslsock->connect_timeout.tv_usec = timeout->tv_usec;
-
 	/* we don't know the socket until we have determined if we are binding or
 	 * connecting */
 	sslsock->s.socket = -1;
 	
-	/* Initialize context as NULL */
-	sslsock->ctx = NULL;	
-	
 	stream = php_stream_alloc_rel(&php_openssl_socket_ops, sslsock, persistent_id, "r+");
 
 	if (stream == NULL)	{
@@ -994,19 +692,12 @@ php_stream *php_openssl_ssl_socket_factory(const char *proto, long protolen,
 		return NULL;
 	}
 
-	sslsock->sni = get_sni(context, resourcename, resourcenamelen, !!persistent_id TSRMLS_CC);
-	
 	if (strncmp(proto, "ssl", protolen) == 0) {
 		sslsock->enable_on_connect = 1;
 		sslsock->method = STREAM_CRYPTO_METHOD_SSLv23_CLIENT;
 	} else if (strncmp(proto, "sslv2", protolen) == 0) {
-#ifdef OPENSSL_NO_SSL2
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSLv2 support is not compiled into the OpenSSL library PHP is linked against");
-		return NULL;
-#else
 		sslsock->enable_on_connect = 1;
 		sslsock->method = STREAM_CRYPTO_METHOD_SSLv2_CLIENT;
-#endif
 	} else if (strncmp(proto, "sslv3", protolen) == 0) {
 		sslsock->enable_on_connect = 1;
 		sslsock->method = STREAM_CRYPTO_METHOD_SSLv3_CLIENT;
