@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: pdo_dbh.c,v 1.82.2.26 2006/01/01 12:50:11 sniper Exp $ */
 
 /* The PDO Database Handle Class */
 
@@ -36,15 +36,14 @@
 #include "zend_object_handlers.h"
 #include "zend_hash.h"
 
-static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_DC);
-
-void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate, const char *supp TSRMLS_DC) /* {{{ */
+void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate, const char *supp TSRMLS_DC)
 {
 	pdo_error_type *pdo_err = &dbh->error_code;
 	char *message = NULL;
 	const char *msg;
+	zval *info = NULL;
 
-	if (dbh && dbh->error_mode == PDO_ERRMODE_SILENT) {
+	if (dbh->error_mode == PDO_ERRMODE_SILENT) {
 #if 0
 		/* BUG: if user is running in silent mode and hits an error at the driver level
 		 * when they use the PDO methods to call up the error information, they may
@@ -57,7 +56,7 @@ void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate
 		pdo_err = &stmt->error_code;
 	}
 
-	strncpy(*pdo_err, sqlstate, 6);
+	strcpy(*pdo_err, sqlstate);
 
 	/* hash sqlstate to error messages */
 	msg = pdo_sqlstate_state_to_description(*pdo_err);
@@ -65,16 +64,26 @@ void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate
 		msg = "<<Unknown error>>";
 	}
 
+	MAKE_STD_ZVAL(info);
+	array_init(info);
+
+	add_next_index_string(info, *pdo_err, 1);
+	add_next_index_long(info, 0);
+		
 	if (supp) {
 		spprintf(&message, 0, "SQLSTATE[%s]: %s: %s", *pdo_err, msg, supp);
 	} else {
 		spprintf(&message, 0, "SQLSTATE[%s]: %s", *pdo_err, msg);
 	}
 
-	if (dbh && dbh->error_mode != PDO_ERRMODE_EXCEPTION) {
+	if (dbh->error_mode != PDO_ERRMODE_EXCEPTION) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", message);
+
+		if (info) {
+			zval_ptr_dtor(&info);
+		}
 	} else {
-		zval *ex, *info;
+		zval *ex;
 		zend_class_entry *def_ex = php_pdo_get_exception_base(1 TSRMLS_CC), *pdo_ex = php_pdo_get_exception();
 
 		MAKE_STD_ZVAL(ex);
@@ -83,14 +92,10 @@ void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate
 		zend_update_property_string(def_ex, ex, "message", sizeof("message")-1, message TSRMLS_CC);
 		zend_update_property_string(def_ex, ex, "code", sizeof("code")-1, *pdo_err TSRMLS_CC);
 		
-		MAKE_STD_ZVAL(info);
-		array_init(info);
-
-		add_next_index_string(info, *pdo_err, 1);
-		add_next_index_long(info, 0);
-
-		zend_update_property(pdo_ex, ex, "errorInfo", sizeof("errorInfo")-1, info TSRMLS_CC);
-		zval_ptr_dtor(&info);
+		if (info) {
+			zend_update_property(pdo_ex, ex, "errorInfo", sizeof("errorInfo")-1, info TSRMLS_CC);
+			zval_ptr_dtor(&info);
+		}
 
 		zend_throw_exception_object(ex TSRMLS_CC);
 	}
@@ -99,9 +104,8 @@ void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate
 		efree(message);
 	}
 }
-/* }}} */
 
-void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
+void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC)
 {
 	pdo_error_type *pdo_err = &dbh->error_code;
 	const char *msg = "<<Unknown>>";
@@ -110,7 +114,7 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 	char *message = NULL;
 	zval *info = NULL;
 
-	if (dbh == NULL || dbh->error_mode == PDO_ERRMODE_SILENT) {
+	if (dbh->error_mode == PDO_ERRMODE_SILENT) {
 		return;
 	}
 	
@@ -125,6 +129,7 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 	}
 
 	if (dbh->methods->fetch_err) {
+		
 		MAKE_STD_ZVAL(info);
 		array_init(info);
 
@@ -141,6 +146,9 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 				supp = estrndup(Z_STRVAL_PP(item), Z_STRLEN_PP(item));
 			}
 		}
+
+		zval_ptr_dtor(&info);
+		info = NULL;
 	}
 
 	if (supp) {
@@ -151,6 +159,10 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 
 	if (dbh->error_mode == PDO_ERRMODE_WARNING) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", message);
+
+		if (info) {
+			zval_ptr_dtor(&info);
+		}
 	} else if (EG(exception) == NULL) {
 		zval *ex;
 		zend_class_entry *def_ex = php_pdo_get_exception_base(1 TSRMLS_CC), *pdo_ex = php_pdo_get_exception();
@@ -163,15 +175,12 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 		
 		if (info) {
 			zend_update_property(pdo_ex, ex, "errorInfo", sizeof("errorInfo")-1, info TSRMLS_CC);
+			zval_ptr_dtor(&info);
 		}
 
 		zend_throw_exception_object(ex TSRMLS_CC);
 	}
-
-	if (info) {
-		zval_ptr_dtor(&info);
-	}
-
+	
 	if (message) {
 		efree(message);
 	}
@@ -180,23 +189,21 @@ void pdo_handle_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 		efree(supp);
 	}
 }
-/* }}} */
 
-static char *dsn_from_uri(char *uri, char *buf, size_t buflen TSRMLS_DC) /* {{{ */
+static char *dsn_from_uri(char *uri, char *buf, size_t buflen TSRMLS_DC)
 {
 	php_stream *stream;
 	char *dsn = NULL;
 
-	stream = php_stream_open_wrapper(uri, "rb", REPORT_ERRORS, NULL);
+	stream = php_stream_open_wrapper(uri, "rb", ENFORCE_SAFE_MODE|REPORT_ERRORS, NULL);
 	if (stream) {
 		dsn = php_stream_get_line(stream, buf, buflen, NULL);
 		php_stream_close(stream);
 	}
 	return dsn;
 }
-/* }}} */
 
-/* {{{ proto void PDO::__construct(string dsn[, string username[, string passwd [, array options]]])
+/* {{{ proto object PDO::__construct(string dsn, string username, string passwd [, array options])
    */
 static PHP_METHOD(PDO, dbh_constructor)
 {
@@ -229,7 +236,6 @@ static PHP_METHOD(PDO, dbh_constructor)
 		snprintf(alt_dsn, sizeof(alt_dsn), "pdo.dsn.%s", data_source);
 		if (FAILURE == cfg_get_string(alt_dsn, &ini_dsn)) {
 			zend_throw_exception_ex(php_pdo_get_exception(), 0 TSRMLS_CC, "invalid data source name");
-			zval_dtor(object);
 			ZVAL_NULL(object);
 			return;
 		}
@@ -275,13 +281,13 @@ static PHP_METHOD(PDO, dbh_constructor)
 	/* is this supposed to be a persistent connection ? */
 	if (options) {
 		zval **v;
-		int plen = 0;
+		int plen;
 		char *hashkey = NULL;
 		zend_rsrc_list_entry *le;
 		pdo_dbh_t *pdbh = NULL;
 
 		if (SUCCESS == zend_hash_index_find(Z_ARRVAL_P(options), PDO_ATTR_PERSISTENT, (void**)&v)) {
-			if (Z_TYPE_PP(v) == IS_STRING && !is_numeric_string(Z_STRVAL_PP(v), Z_STRLEN_PP(v), NULL, NULL, 0) && Z_STRLEN_PP(v) > 0) {
+			if (Z_TYPE_PP(v) == IS_STRING) {
 				/* user specified key */
 				plen = spprintf(&hashkey, 0, "PDO:DBH:DSN=%s:%s:%s:%s", data_source,
 						username ? username : "",
@@ -329,25 +335,15 @@ static PHP_METHOD(PDO, dbh_constructor)
 				memcpy((char *)pdbh->persistent_id, hashkey, plen+1);
 				pdbh->persistent_id_len = plen+1;
 				pdbh->refcount = 1;
-				pdbh->std.properties = NULL;
 			}
 		}
 
 		if (pdbh) {
 			/* let's copy the emalloc bits over from the other handle */
-			if (pdbh->std.properties) {
-				zend_hash_destroy(dbh->std.properties);	
-				efree(dbh->std.properties);
-				if (dbh->std.properties_table) {
-					efree(dbh->std.properties_table);
-				}
-			} else {
-				pdbh->std.ce = dbh->std.ce;
-				pdbh->def_stmt_ce = dbh->def_stmt_ce;
-				pdbh->def_stmt_ctor_args = dbh->def_stmt_ctor_args;
-				pdbh->std.properties = dbh->std.properties;
-				pdbh->std.properties_table = dbh->std.properties_table;
-			}
+			pdbh->ce = dbh->ce;
+			pdbh->def_stmt_ce = dbh->def_stmt_ce;
+			pdbh->def_stmt_ctor_args = dbh->def_stmt_ctor_args;
+			pdbh->properties = dbh->properties;
 			/* kill the non-persistent thingamy */
 			efree(dbh);
 			/* switch over to the persistent one */
@@ -361,13 +357,10 @@ static PHP_METHOD(PDO, dbh_constructor)
 		}
 	}
 	
-	if (call_factory) {
-		dbh->data_source_len = strlen(colon + 1);
-		dbh->data_source = (const char*)pestrdup(colon + 1, is_persistent);
-		dbh->username = username ? pestrdup(username, is_persistent) : NULL;
-		dbh->password = password ? pestrdup(password, is_persistent) : NULL;
-		dbh->default_fetch_type = PDO_FETCH_BOTH;
-	}	
+	dbh->data_source_len = strlen(colon + 1);
+	dbh->data_source = (const char*)pestrdup(colon + 1, is_persistent);
+	dbh->username = username ? pestrdup(username, is_persistent) : NULL;
+	dbh->password = password ? pestrdup(password, is_persistent) : NULL;
 
 	dbh->auto_commit = pdo_attr_lval(options, PDO_ATTR_AUTOCOMMIT, 1 TSRMLS_CC);
 
@@ -377,7 +370,7 @@ static PHP_METHOD(PDO, dbh_constructor)
 
 	if (!call_factory) {
 		/* we got a persistent guy from our cache */
-		goto options;
+		return;
 	}
 
 	if (driver->db_handle_factory(dbh, options TSRMLS_CC)) {
@@ -401,22 +394,7 @@ static PHP_METHOD(PDO, dbh_constructor)
 		}
 
 		dbh->driver = driver;
-options:
-		if (options) {
-			zval **attr_value;
-			char *str_key;
-			ulong long_key;
-			
-			zend_hash_internal_pointer_reset(Z_ARRVAL_P(options));
-			while (SUCCESS == zend_hash_get_current_data(Z_ARRVAL_P(options), (void**)&attr_value) 
-				&& HASH_KEY_IS_LONG == zend_hash_get_current_key(Z_ARRVAL_P(options), &str_key, &long_key, 0)) {
-				
-				pdo_dbh_attribute_set(dbh, long_key, *attr_value TSRMLS_CC);
-				zend_hash_move_forward(Z_ARRVAL_P(options));
-			}
-		}
-
-		return;
+		return;	
 	}
 
 	/* the connection failed; things will tidy up in free_storage */
@@ -440,8 +418,8 @@ static zval *pdo_stmt_instantiate(pdo_dbh_t *dbh, zval *object, zend_class_entry
 
 	Z_TYPE_P(object) = IS_OBJECT;
 	object_init_ex(object, dbstmt_ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	object->refcount = 1;
+	object->is_ref = 1;
 	
 	return object;
 } /* }}} */
@@ -454,18 +432,18 @@ static void pdo_stmt_construct(pdo_stmt_t *stmt, zval *object, zend_class_entry
 	MAKE_STD_ZVAL(query_string);
 	ZVAL_STRINGL(query_string, stmt->query_string, stmt->query_stringlen, 1);
 	ZVAL_STRINGL(&z_key, "queryString", sizeof("queryString")-1, 0);
-	std_object_handlers.write_property(object, &z_key, query_string, 0 TSRMLS_CC);
+	std_object_handlers.write_property(object, &z_key, query_string TSRMLS_CC);
 	zval_ptr_dtor(&query_string);
 
 	if (dbstmt_ce->constructor) {
 		zend_fcall_info fci;
 		zend_fcall_info_cache fcc;
-		zval *retval = NULL;
+		zval *retval;
 
 		fci.size = sizeof(zend_fcall_info);
 		fci.function_table = &dbstmt_ce->function_table;
 		fci.function_name = NULL;
-		fci.object_ptr = object;
+		fci.object_pp = &object;
 		fci.symbol_table = NULL;
 		fci.retval_ptr_ptr = &retval;
 		if (ctor_args) {
@@ -488,14 +466,13 @@ static void pdo_stmt_construct(pdo_stmt_t *stmt, zval *object, zend_class_entry
 		fcc.initialized = 1;
 		fcc.function_handler = dbstmt_ce->constructor;
 		fcc.calling_scope = EG(scope);
-		fcc.called_scope = Z_OBJCE_P(object);
-		fcc.object_ptr = object;
+		fcc.object_pp = &object;
 
 		if (zend_call_function(&fci, &fcc TSRMLS_CC) == FAILURE) {
 			zval_dtor(object);
 			ZVAL_NULL(object);
 			object = NULL; /* marks failure */
-		} else if (retval) {
+		} else {
 			zval_ptr_dtor(&retval);
 		}
 			
@@ -531,7 +508,7 @@ static PHP_METHOD(PDO, prepare)
 			|| zend_lookup_class(Z_STRVAL_PP(item), Z_STRLEN_PP(item), &pce TSRMLS_CC) == FAILURE
 		) {
 			pdo_raise_impl_error(dbh, NULL, "HY000", 
-				"PDO::ATTR_STATEMENT_CLASS requires format array(classname, array(ctor_args)); "
+				"PDO::ATTR_STATEMENT_CLASS requires format array(classname, ctor_args); "
 				"the classname must be a string specifying an existing class"
 				TSRMLS_CC);
 			PDO_HANDLE_DBH_ERR();
@@ -580,11 +557,10 @@ static PHP_METHOD(PDO, prepare)
 	/* unconditionally keep this for later reference */
 	stmt->query_string = estrndup(statement, statement_len);
 	stmt->query_stringlen = statement_len;
-	stmt->default_fetch_type = dbh->default_fetch_type;
+	stmt->default_fetch_type = PDO_FETCH_BOTH;
 	stmt->dbh = dbh;
 	/* give it a reference to me */
 	zend_objects_store_add_ref(getThis() TSRMLS_CC);
-	php_pdo_dbh_addref(dbh TSRMLS_CC);
 	stmt->database_object_handle = *getThis();
 	/* we haven't created a lazy object yet */
 	ZVAL_NULL(&stmt->lazy_object_ref);
@@ -609,9 +585,6 @@ static PHP_METHOD(PDO, beginTransaction)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (dbh->in_txn) {
@@ -642,9 +615,6 @@ static PHP_METHOD(PDO, commit)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (!dbh->in_txn) {
@@ -668,9 +638,6 @@ static PHP_METHOD(PDO, rollBack)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (!dbh->in_txn) {
@@ -688,99 +655,60 @@ static PHP_METHOD(PDO, rollBack)
 }
 /* }}} */
 
-/* {{{ proto bool PDO::inTransaction()
-   determine if inside a transaction */
-static PHP_METHOD(PDO, inTransaction)
+/* {{{ proto bool PDO::setAttribute(long attribute, mixed value)
+   Set an attribute */
+static PHP_METHOD(PDO, setAttribute)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
+	long attr;
+	zval *value = NULL;
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lz!", &attr, &value)) {
+		RETURN_FALSE;
 	}
+
 	PDO_CONSTRUCT_CHECK;
 
-	if (!dbh->methods->in_transaction) {
-		RETURN_BOOL(dbh->in_txn);
-	}	
-
-	RETURN_BOOL(dbh->methods->in_transaction(dbh TSRMLS_CC));
-}
-/* }}} */
-
-static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_DC) /* {{{ */
-{
-
-#define PDO_LONG_PARAM_CHECK \
-	if (Z_TYPE_P(value) != IS_LONG && Z_TYPE_P(value) != IS_STRING && Z_TYPE_P(value) != IS_BOOL) { \
-		pdo_raise_impl_error(dbh, NULL, "HY000", "attribute value must be an integer" TSRMLS_CC); \
-		PDO_HANDLE_DBH_ERR(); \
-		return FAILURE; \
-	} \
-
 	switch (attr) {
 		case PDO_ATTR_ERRMODE:
-			PDO_LONG_PARAM_CHECK;
 			convert_to_long(value);
 			switch (Z_LVAL_P(value)) {
 				case PDO_ERRMODE_SILENT:
 				case PDO_ERRMODE_WARNING:
 				case PDO_ERRMODE_EXCEPTION:
 					dbh->error_mode = Z_LVAL_P(value);
-					return SUCCESS;
+					RETURN_TRUE;
 				default:
 					pdo_raise_impl_error(dbh, NULL, "HY000", "invalid error mode" TSRMLS_CC);
 					PDO_HANDLE_DBH_ERR();
-					return FAILURE;
+					RETURN_FALSE;
 			}
-			return FAILURE;
+			RETURN_FALSE;
 
 		case PDO_ATTR_CASE:
-			PDO_LONG_PARAM_CHECK;
 			convert_to_long(value);
 			switch (Z_LVAL_P(value)) {
 				case PDO_CASE_NATURAL:
 				case PDO_CASE_UPPER:
 				case PDO_CASE_LOWER:
 					dbh->desired_case = Z_LVAL_P(value);
-					return SUCCESS;
+					RETURN_TRUE;
 				default:
 					pdo_raise_impl_error(dbh, NULL, "HY000", "invalid case folding mode" TSRMLS_CC);
 					PDO_HANDLE_DBH_ERR();
-					return FAILURE;
+					RETURN_FALSE;
 			}
-			return FAILURE;
+			RETURN_FALSE;
 
 		case PDO_ATTR_ORACLE_NULLS:
-			PDO_LONG_PARAM_CHECK;
 			convert_to_long(value);
 			dbh->oracle_nulls = Z_LVAL_P(value);
-			return SUCCESS;
-
-		case PDO_ATTR_DEFAULT_FETCH_MODE:
-			if (Z_TYPE_P(value) == IS_ARRAY) {
-				zval **tmp;
-				if (zend_hash_index_find(Z_ARRVAL_P(value), 0, (void**)&tmp) == SUCCESS && Z_TYPE_PP(tmp) == IS_LONG) {
-					if (Z_LVAL_PP(tmp) == PDO_FETCH_INTO || Z_LVAL_PP(tmp) == PDO_FETCH_CLASS) {
-						pdo_raise_impl_error(dbh, NULL, "HY000", "FETCH_INTO and FETCH_CLASS are not yet supported as default fetch modes" TSRMLS_CC);
-						return FAILURE;
-					}
-				}
-			} else {
-				PDO_LONG_PARAM_CHECK;
-			}
-			convert_to_long(value);
-			if (Z_LVAL_P(value) == PDO_FETCH_USE_DEFAULT) {
-				pdo_raise_impl_error(dbh, NULL, "HY000", "invalid fetch mode type" TSRMLS_CC);
-				return FAILURE;
-			}
-			dbh->default_fetch_type = Z_LVAL_P(value);
-			return SUCCESS;
+			RETURN_TRUE;
 
 		case PDO_ATTR_STRINGIFY_FETCHES:
-			PDO_LONG_PARAM_CHECK;
 			convert_to_long(value);
 			dbh->stringify = Z_LVAL_P(value) ? 1 : 0;
-			return SUCCESS;
+			RETURN_TRUE;
 			
 		case PDO_ATTR_STATEMENT_CLASS: {
 			/* array(string classname, array(mixed ctor_args)) */
@@ -792,7 +720,7 @@ static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_D
 					"PDO::ATTR_STATEMENT_CLASS cannot be used with persistent PDO instances"
 					TSRMLS_CC);
 				PDO_HANDLE_DBH_ERR();
-				return FAILURE;
+				RETURN_FALSE;
 			}
 			if (Z_TYPE_P(value) != IS_ARRAY
 				|| zend_hash_index_find(Z_ARRVAL_P(value), 0, (void**)&item) == FAILURE
@@ -804,19 +732,19 @@ static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_D
 					"the classname must be a string specifying an existing class"
 					TSRMLS_CC);
 				PDO_HANDLE_DBH_ERR();
-				return FAILURE;
+				RETURN_FALSE;
 			}
 			if (!instanceof_function(*pce, pdo_dbstmt_ce TSRMLS_CC)) {
 				pdo_raise_impl_error(dbh, NULL, "HY000", 
 					"user-supplied statement class must be derived from PDOStatement" TSRMLS_CC);
 				PDO_HANDLE_DBH_ERR();
-				return FAILURE;
+				RETURN_FALSE;
 			}
 			if ((*pce)->constructor && !((*pce)->constructor->common.fn_flags & (ZEND_ACC_PRIVATE|ZEND_ACC_PROTECTED))) {
 				pdo_raise_impl_error(dbh, NULL, "HY000", 
 					"user-supplied statement class cannot have a public constructor" TSRMLS_CC);
 				PDO_HANDLE_DBH_ERR();
-				return FAILURE;
+				RETURN_FALSE;
 			}
 			dbh->def_stmt_ce = *pce;
 			if (dbh->def_stmt_ctor_args) {
@@ -830,12 +758,12 @@ static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_D
 						"ctor_args must be an array"
 					TSRMLS_CC);
 					PDO_HANDLE_DBH_ERR();
-					return FAILURE;
+					RETURN_FALSE;
 				}
-				Z_ADDREF_PP(item);
+				(*item)->refcount++;
 				dbh->def_stmt_ctor_args = *item;
 			}
-			return SUCCESS;
+			RETURN_TRUE;
 		}
 			
 		default:
@@ -848,7 +776,7 @@ static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_D
 
 	PDO_DBH_CLEAR_ERR();
 	if (dbh->methods->set_attribute(dbh, attr, value TSRMLS_CC)) {
-		return SUCCESS;
+		RETURN_TRUE;
 	}
 
 fail:
@@ -859,29 +787,7 @@ fail:
 	} else {
 		PDO_HANDLE_DBH_ERR();
 	}
-	return FAILURE;
-}
-/* }}} */
- 
-/* {{{ proto bool PDO::setAttribute(long attribute, mixed value)
-   Set an attribute */
-static PHP_METHOD(PDO, setAttribute)
-{
-	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
-	long attr;
-	zval *value;
-
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lz", &attr, &value)) {
-		RETURN_FALSE;
-	}
-
-	PDO_DBH_CLEAR_ERR();
-	PDO_CONSTRUCT_CHECK;
-
-	if (pdo_dbh_attribute_set(dbh, attr, value TSRMLS_CC) != FAILURE) {
- 		RETURN_TRUE;
- 	}
- 	RETURN_FALSE;
+	RETURN_FALSE;
 }
 /* }}} */
 
@@ -899,7 +805,7 @@ static PHP_METHOD(PDO, getAttribute)
 	PDO_DBH_CLEAR_ERR();
 	PDO_CONSTRUCT_CHECK;
 
-	/* handle generic PDO-level attributes */
+	/* handle generic PDO-level atributes */
 	switch (attr) {
 		case PDO_ATTR_PERSISTENT:
 			RETURN_BOOL(dbh->is_persistent);
@@ -920,13 +826,10 @@ static PHP_METHOD(PDO, getAttribute)
 			array_init(return_value);
 			add_next_index_string(return_value, dbh->def_stmt_ce->name, 1);
 			if (dbh->def_stmt_ctor_args) {
-				Z_ADDREF_P(dbh->def_stmt_ctor_args);
+				dbh->def_stmt_ctor_args->refcount++;
 				add_next_index_zval(return_value, dbh->def_stmt_ctor_args);
 			}
 			return;
-		case PDO_ATTR_DEFAULT_FETCH_MODE:
-			RETURN_LONG(dbh->default_fetch_type);
-
 	}
 	
 	if (!dbh->methods->get_attribute) {
@@ -963,7 +866,6 @@ static PHP_METHOD(PDO, exec)
 	}
 
 	if (!statement_len) {
-		pdo_raise_impl_error(dbh, NULL, "HY000",  "trying to execute an empty query" TSRMLS_CC);
 		RETURN_FALSE;
 	}
 	PDO_DBH_CLEAR_ERR();
@@ -997,7 +899,7 @@ static PHP_METHOD(PDO, lastInsertId)
 		pdo_raise_impl_error(dbh, NULL, "IM001", "driver does not support lastInsertId()" TSRMLS_CC);
 		RETURN_FALSE;
 	} else {
-		Z_STRVAL_P(return_value) = dbh->methods->last_id(dbh, name, (unsigned int *)&Z_STRLEN_P(return_value) TSRMLS_CC);
+		Z_STRVAL_P(return_value) = dbh->methods->last_id(dbh, name, &Z_STRLEN_P(return_value) TSRMLS_CC);
 		if (!Z_STRVAL_P(return_value)) {
 			PDO_HANDLE_DBH_ERR();
 			RETURN_FALSE;
@@ -1014,8 +916,8 @@ static PHP_METHOD(PDO, errorCode)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS()) {
+		RETURN_FALSE;
 	}
 	PDO_CONSTRUCT_CHECK;
 
@@ -1023,14 +925,6 @@ static PHP_METHOD(PDO, errorCode)
 		RETURN_STRING(dbh->query_stmt->error_code, 1);
 	}
 	
-	if (dbh->error_code[0] == '\0') {
-		RETURN_NULL();
-	}
-
-	/**
-	 * Making sure that we fallback to the default implementation
-	 * if the dbh->error_code is not null.
-	 */
 	RETURN_STRING(dbh->error_code, 1);
 }
 /* }}} */
@@ -1039,16 +933,11 @@ static PHP_METHOD(PDO, errorCode)
    Fetch extended error information associated with the last operation on the database handle */
 static PHP_METHOD(PDO, errorInfo)
 {
-	int error_count;
-	int error_count_diff 	 = 0;
-	int error_expected_count = 3;
-
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS()) {
+		RETURN_FALSE;
 	}
-
 	PDO_CONSTRUCT_CHECK;
 
 	array_init(return_value);
@@ -1058,26 +947,9 @@ static PHP_METHOD(PDO, errorInfo)
 	} else {
 		add_next_index_string(return_value, dbh->error_code, 1);
 	}
-
 	if (dbh->methods->fetch_err) {
 		dbh->methods->fetch_err(dbh, dbh->query_stmt, return_value TSRMLS_CC);
 	}
-	
-	/**
-	 * In order to be consistent, we have to make sure we add the good amount
-	 * of nulls depending on the current number of elements. We make a simple
-	 * difference and add the needed elements
-	 */
-	error_count = zend_hash_num_elements(Z_ARRVAL_P(return_value));
-
-	if (error_expected_count > error_count) {
-		int current_index;
-
-		error_count_diff = error_expected_count - error_count;
-		for (current_index = 0; current_index < error_count_diff; current_index++) {
-			add_next_index_null(return_value);
-		}
-	}
 }
 /* }}} */
 
@@ -1090,12 +962,6 @@ static PHP_METHOD(PDO, query)
 	char *statement;
 	int statement_len;
 
-	/* Return a meaningful error when no parameters were passed */
-	if (!ZEND_NUM_ARGS()) {
-		zend_parse_parameters(0 TSRMLS_CC, "z|z", NULL, NULL);
-		RETURN_FALSE;
-	}
-	
 	if (FAILURE == zend_parse_parameters(1 TSRMLS_CC, "s", &statement,
 			&statement_len)) {
 		RETURN_FALSE;
@@ -1113,22 +979,18 @@ static PHP_METHOD(PDO, query)
 	/* unconditionally keep this for later reference */
 	stmt->query_string = estrndup(statement, statement_len);
 	stmt->query_stringlen = statement_len;
-
-	stmt->default_fetch_type = dbh->default_fetch_type;
+	stmt->default_fetch_type = PDO_FETCH_BOTH;
 	stmt->active_query_string = stmt->query_string;
 	stmt->active_query_stringlen = statement_len;
 	stmt->dbh = dbh;
 	/* give it a reference to me */
 	zend_objects_store_add_ref(getThis() TSRMLS_CC);
-	php_pdo_dbh_addref(dbh TSRMLS_CC);
 	stmt->database_object_handle = *getThis();
 	/* we haven't created a lazy object yet */
 	ZVAL_NULL(&stmt->lazy_object_ref);
 
 	if (dbh->methods->preparer(dbh, statement, statement_len, stmt, NULL TSRMLS_CC)) {
-		PDO_STMT_CLEAR_ERR();
 		if (ZEND_NUM_ARGS() == 1 || SUCCESS == pdo_stmt_setup_fetch_mode(INTERNAL_FUNCTION_PARAM_PASSTHRU, stmt, 1)) {
-
 			/* now execute the statement */
 			PDO_STMT_CLEAR_ERR();
 			if (stmt->methods->executer(stmt TSRMLS_CC)) {
@@ -1169,7 +1031,8 @@ static PHP_METHOD(PDO, quote)
 	char *qstr;
 	int qlen;
 
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &str, &str_len, &paramtype)) {
+	if (FAILURE == zend_parse_parameters(1 TSRMLS_CC, "s|l", &str, &str_len,
+			&paramtype)) {
 		RETURN_FALSE;
 	}
 	
@@ -1184,7 +1047,6 @@ static PHP_METHOD(PDO, quote)
 		RETURN_STRINGL(qstr, qlen, 0);
 	}
 	PDO_HANDLE_DBH_ERR();
-	RETURN_FALSE;
 }
 /* }}} */
 
@@ -1204,16 +1066,12 @@ static PHP_METHOD(PDO, __sleep)
 }
 /* }}} */
 
-/* {{{ proto array PDO::getAvailableDrivers()
+/* {{{ proto array pdo_drivers()
    Return array of available PDO drivers */
 static PHP_METHOD(PDO, getAvailableDrivers)
 {
 	HashPosition pos;
 	pdo_driver_t **pdriver;
-
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
 	
 	array_init(return_value);
 
@@ -1225,70 +1083,30 @@ static PHP_METHOD(PDO, getAvailableDrivers)
 }
 /* }}} */
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, dsn)
-	ZEND_ARG_INFO(0, username)
-	ZEND_ARG_INFO(0, passwd)
-	ZEND_ARG_INFO(0, options) /* array */
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_prepare, 0, 0, 1)
-	ZEND_ARG_INFO(0, statement)
-	ZEND_ARG_INFO(0, options) /* array */
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_pdo_setattribute, 0)
-	ZEND_ARG_INFO(0, attribute)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_pdo_getattribute, 0)
-	ZEND_ARG_INFO(0, attribute)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_pdo_exec, 0)
-	ZEND_ARG_INFO(0, query)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_lastinsertid, 0, 0, 0)
-	ZEND_ARG_INFO(0, seqname)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_quote, 0, 0, 1)
-	ZEND_ARG_INFO(0, string)
-	ZEND_ARG_INFO(0, paramtype)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_pdo__void, 0)
-ZEND_END_ARG_INFO()
-/* }}} */
-
-const zend_function_entry pdo_dbh_functions[] = {
-	ZEND_MALIAS(PDO, __construct, dbh_constructor,	arginfo_pdo___construct,	ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, prepare, 				arginfo_pdo_prepare,		ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, beginTransaction,       arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, commit,                 arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, rollBack,               arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, inTransaction,          arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, setAttribute,	arginfo_pdo_setattribute,	ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, exec,			arginfo_pdo_exec,		ZEND_ACC_PUBLIC)
+zend_function_entry pdo_dbh_functions[] = {
+	ZEND_MALIAS(PDO, __construct, dbh_constructor,	NULL, 			ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, prepare, 		NULL, 					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, beginTransaction,NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, commit,			NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, rollBack,		NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, setAttribute,	NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, exec,			NULL,					ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, query,			NULL,					ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, lastInsertId,	arginfo_pdo_lastinsertid,	ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, errorCode,              arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, errorInfo,              arginfo_pdo__void,         ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, getAttribute,	arginfo_pdo_getattribute,	ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, quote,			arginfo_pdo_quote,		ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, __wakeup,               arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(PDO, __sleep,                arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(PDO, getAvailableDrivers,    arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_STATIC)
+	PHP_ME(PDO, lastInsertId,	NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, errorCode,		NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, errorInfo,		NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, getAttribute,	NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, quote,			NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, __wakeup,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(PDO, __sleep,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(PDO, getAvailableDrivers, NULL,				ZEND_ACC_PUBLIC|ZEND_ACC_STATIC)
 	{NULL, NULL, NULL}
 };
 
 /* {{{ overloaded object handlers for PDO class */
 int pdo_hash_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
 {
-	const zend_function_entry *funcs;
+	zend_function_entry *funcs;
 	zend_function func;
 	zend_internal_function *ifunc = (zend_internal_function*)&func;
 	int namelen;
@@ -1297,7 +1115,8 @@ int pdo_hash_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
 	if (!dbh || !dbh->methods || !dbh->methods->get_driver_methods) {
 		return 0;
 	}
-	funcs =	dbh->methods->get_driver_methods(dbh, kind TSRMLS_CC);
+	funcs =	dbh->methods->get_driver_methods(dbh,
+			PDO_DBH_DRIVER_METHOD_KIND_DBH TSRMLS_CC);
 	if (!funcs) {
 		return 0;
 	}
@@ -1310,38 +1129,30 @@ int pdo_hash_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
 	while (funcs->fname) {
 		ifunc->type = ZEND_INTERNAL_FUNCTION;
 		ifunc->handler = funcs->handler;
-		ifunc->function_name = (char*)funcs->fname;
-		ifunc->scope = dbh->std.ce;
+		ifunc->function_name = funcs->fname;
+		ifunc->scope = dbh->ce;
 		ifunc->prototype = NULL;
-		if (funcs->flags) {
-			ifunc->fn_flags = funcs->flags | ZEND_ACC_NEVER_CACHE;
-		} else {
-			ifunc->fn_flags = ZEND_ACC_PUBLIC | ZEND_ACC_NEVER_CACHE;
-		}
 		if (funcs->arg_info) {
-			zend_internal_function_info *info = (zend_internal_function_info*)funcs->arg_info;
-
-			ifunc->arg_info = (zend_arg_info*)funcs->arg_info + 1;
+			ifunc->arg_info = funcs->arg_info + 1;
 			ifunc->num_args = funcs->num_args;
-			if (info->required_num_args == -1) {
+			if (funcs->arg_info[0].required_num_args == -1) {
 				ifunc->required_num_args = funcs->num_args;
 			} else {
-				ifunc->required_num_args = info->required_num_args;
-			}
-			if (info->pass_rest_by_reference) {
-				if (info->pass_rest_by_reference == ZEND_SEND_PREFER_REF) {
-					ifunc->fn_flags |= ZEND_ACC_PASS_REST_PREFER_REF;
-				} else {
-					ifunc->fn_flags |= ZEND_ACC_PASS_REST_BY_REFERENCE;
-				}
-			}
-			if (info->return_reference) {
-				ifunc->fn_flags |= ZEND_ACC_RETURN_REFERENCE;
+				ifunc->required_num_args = funcs->arg_info[0].required_num_args;
 			}
+			ifunc->pass_rest_by_reference = funcs->arg_info[0].pass_by_reference;
+			ifunc->return_reference = funcs->arg_info[0].return_reference;
 		} else {
 			ifunc->arg_info = NULL;
 			ifunc->num_args = 0;
 			ifunc->required_num_args = 0;
+			ifunc->pass_rest_by_reference = 0;
+			ifunc->return_reference = 0;
+		}
+		if (funcs->flags) {
+			ifunc->fn_flags = funcs->flags;
+		} else {
+			ifunc->fn_flags = ZEND_ACC_PUBLIC;
 		}
 		namelen = strlen(funcs->fname);
 		lc_name = emalloc(namelen+1);
@@ -1360,7 +1171,7 @@ static union _zend_function *dbh_method_get(
 #else
 	zval *object,
 #endif
-	char *method_name, int method_len, const zend_literal *key TSRMLS_DC)
+	char *method_name, int method_len TSRMLS_DC)
 {
 	zend_function *fbc = NULL;
 	char *lc_method_name;
@@ -1372,7 +1183,7 @@ static union _zend_function *dbh_method_get(
 	lc_method_name = emalloc(method_len + 1);
 	zend_str_tolower_copy(lc_method_name, method_name, method_len);
 
-	if ((fbc = std_object_handlers.get_method(object_pp, method_name, method_len, key TSRMLS_CC)) == NULL) {
+	if (zend_hash_find(&dbh->ce->function_table, lc_method_name, method_len+1, (void**)&fbc) == FAILURE) {
 		/* not a pre-defined method, nor a user-defined method; check
 		 * the driver specific methods */
 		if (!dbh->cls_methods[PDO_DBH_DRIVER_METHOD_KIND_DBH]) {
@@ -1385,12 +1196,12 @@ static union _zend_function *dbh_method_get(
 
 		if (zend_hash_find(dbh->cls_methods[PDO_DBH_DRIVER_METHOD_KIND_DBH],
 				lc_method_name, method_len+1, (void**)&fbc) == FAILURE) {
-			if (!fbc) {
-				fbc = NULL;
-			}
+			fbc = NULL;
+			goto out;
 		}
+		/* got it */
 	}
-
+	
 out:
 	efree(lc_method_name);
 	return fbc;
@@ -1403,6 +1214,32 @@ static int dbh_compare(zval *object1, zval *object2 TSRMLS_DC)
 
 static zend_object_handlers pdo_dbh_object_handlers;
 
+PDO_API void php_pdo_declare_stringl_constant(const char *const_name,
+		size_t name_len, const char *value, size_t value_len TSRMLS_DC)
+{
+#if PHP_MAJOR_VERSION > 5 || PHP_MINOR_VERSION >= 1
+	zend_declare_class_constant_stringl(pdo_dbh_ce, (char*)const_name, name_len, (char*)value, value_len TSRMLS_CC);
+#else
+	zval *constant = malloc(sizeof(*constant));
+	ZVAL_STRINGL(constant, zend_strndup(value, value_len), value_len, 0);
+	INIT_PZVAL(constant);
+	zend_hash_update(&pdo_dbh_ce->constants_table, (char*)const_name, name_len+1, &constant, sizeof(zval*), NULL);
+#endif
+}
+
+PDO_API void php_pdo_declare_long_constant(const char *const_name,
+		size_t name_len, long value TSRMLS_DC)
+{
+#if PHP_MAJOR_VERSION > 5 || PHP_MINOR_VERSION >= 1
+	zend_declare_class_constant_long(pdo_dbh_ce, (char*)const_name, name_len, value TSRMLS_CC);
+#else
+	zval *constant = malloc(sizeof(*constant));
+	ZVAL_LONG(constant, value);
+	INIT_PZVAL(constant);
+	zend_hash_update(&pdo_dbh_ce->constants_table, (char*)const_name, name_len+1, &constant, sizeof(zval*), NULL);
+#endif
+}
+
 void pdo_dbh_init(TSRMLS_D)
 {
 	zend_class_entry ce;
@@ -1414,7 +1251,7 @@ void pdo_dbh_init(TSRMLS_D)
 	memcpy(&pdo_dbh_object_handlers, &std_object_handlers, sizeof(zend_object_handlers));
 	pdo_dbh_object_handlers.get_method = dbh_method_get;
 	pdo_dbh_object_handlers.compare_objects = dbh_compare;
-	
+
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_BOOL", (long)PDO_PARAM_BOOL);
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_NULL", (long)PDO_PARAM_NULL);
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_INT",  (long)PDO_PARAM_INT);
@@ -1423,14 +1260,6 @@ void pdo_dbh_init(TSRMLS_D)
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_STMT", (long)PDO_PARAM_STMT);
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_INPUT_OUTPUT", (long)PDO_PARAM_INPUT_OUTPUT);
 
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_ALLOC",		(long)PDO_PARAM_EVT_ALLOC);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_FREE",			(long)PDO_PARAM_EVT_FREE);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_EXEC_PRE",		(long)PDO_PARAM_EVT_EXEC_PRE);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_EXEC_POST",	(long)PDO_PARAM_EVT_EXEC_POST);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_FETCH_PRE",	(long)PDO_PARAM_EVT_FETCH_PRE);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_FETCH_POST",	(long)PDO_PARAM_EVT_FETCH_POST);
-	REGISTER_PDO_CLASS_CONST_LONG("PARAM_EVT_NORMALIZE",	(long)PDO_PARAM_EVT_NORMALIZE);
-
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_LAZY", (long)PDO_FETCH_LAZY);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_ASSOC",(long)PDO_FETCH_ASSOC);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_NUM",  (long)PDO_FETCH_NUM);
@@ -1443,12 +1272,10 @@ void pdo_dbh_init(TSRMLS_D)
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_FUNC", (long)PDO_FETCH_FUNC);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_GROUP",(long)PDO_FETCH_GROUP);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_UNIQUE",(long)PDO_FETCH_UNIQUE);
-	REGISTER_PDO_CLASS_CONST_LONG("FETCH_KEY_PAIR",(long)PDO_FETCH_KEY_PAIR);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_CLASSTYPE",(long)PDO_FETCH_CLASSTYPE);
 #if PHP_MAJOR_VERSION > 5 || PHP_MINOR_VERSION >= 1
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_SERIALIZE",(long)PDO_FETCH_SERIALIZE);
 #endif
-	REGISTER_PDO_CLASS_CONST_LONG("FETCH_PROPS_LATE",(long)PDO_FETCH_PROPS_LATE);
 	REGISTER_PDO_CLASS_CONST_LONG("FETCH_NAMED",(long)PDO_FETCH_NAMED);
 
 	REGISTER_PDO_CLASS_CONST_LONG("ATTR_AUTOCOMMIT",	(long)PDO_ATTR_AUTOCOMMIT);
@@ -1470,8 +1297,6 @@ void pdo_dbh_init(TSRMLS_D)
 	REGISTER_PDO_CLASS_CONST_LONG("ATTR_DRIVER_NAME",		(long)PDO_ATTR_DRIVER_NAME);
 	REGISTER_PDO_CLASS_CONST_LONG("ATTR_STRINGIFY_FETCHES",(long)PDO_ATTR_STRINGIFY_FETCHES);
 	REGISTER_PDO_CLASS_CONST_LONG("ATTR_MAX_COLUMN_LEN",(long)PDO_ATTR_MAX_COLUMN_LEN);
-	REGISTER_PDO_CLASS_CONST_LONG("ATTR_EMULATE_PREPARES",(long)PDO_ATTR_EMULATE_PREPARES);
-	REGISTER_PDO_CLASS_CONST_LONG("ATTR_DEFAULT_FETCH_MODE",(long)PDO_ATTR_DEFAULT_FETCH_MODE);
 	
 	REGISTER_PDO_CLASS_CONST_LONG("ERRMODE_SILENT",	(long)PDO_ERRMODE_SILENT);
 	REGISTER_PDO_CLASS_CONST_LONG("ERRMODE_WARNING",	(long)PDO_ERRMODE_WARNING);
@@ -1515,7 +1340,7 @@ void pdo_dbh_init(TSRMLS_D)
 static void dbh_free(pdo_dbh_t *dbh TSRMLS_DC)
 {
 	int i;
-
+	
 	if (--dbh->refcount)
 		return;
 
@@ -1537,10 +1362,6 @@ static void dbh_free(pdo_dbh_t *dbh TSRMLS_DC)
 	if (dbh->password) {
 		pefree(dbh->password, dbh->is_persistent);
 	}
-	
-	if (dbh->persistent_id) {
-		pefree((char *)dbh->persistent_id, dbh->is_persistent);
-	}
 
 	if (dbh->def_stmt_ctor_args) {
 		zval_ptr_dtor(&dbh->def_stmt_ctor_args);
@@ -1573,26 +1394,32 @@ static void pdo_dbh_free_storage(pdo_dbh_t *dbh TSRMLS_DC)
 		dbh->in_txn = 0;
 	}
 	
-	if (dbh->is_persistent && dbh->methods && dbh->methods->persistent_shutdown) {
+	if (dbh->properties) {
+		zend_hash_destroy(dbh->properties);
+		efree(dbh->properties);
+		dbh->properties = NULL;
+	}
+
+	if (!dbh->is_persistent) {
+		dbh_free(dbh TSRMLS_CC);
+	} else if (dbh->methods && dbh->methods->persistent_shutdown) {
 		dbh->methods->persistent_shutdown(dbh TSRMLS_CC);
 	}
-	zend_object_std_dtor(&dbh->std TSRMLS_CC);
-	dbh->std.properties = NULL;
-	dbh->std.properties_table = NULL;
-	dbh_free(dbh TSRMLS_CC);
 }
 
 zend_object_value pdo_dbh_new(zend_class_entry *ce TSRMLS_DC)
 {
 	zend_object_value retval;
 	pdo_dbh_t *dbh;
+	zval *tmp;
 
 	dbh = emalloc(sizeof(*dbh));
 	memset(dbh, 0, sizeof(*dbh));
-	zend_object_std_init(&dbh->std, ce TSRMLS_CC);
-	object_properties_init(&dbh->std, ce);
-	rebuild_object_properties(&dbh->std);
+	dbh->ce = ce;
 	dbh->refcount = 1;
+	ALLOC_HASHTABLE(dbh->properties);
+	zend_hash_init(dbh->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
+	zend_hash_copy(dbh->properties, &ce->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 	dbh->def_stmt_ce = pdo_dbstmt_ce;
 	
 	retval.handle = zend_objects_store_put(dbh, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t)pdo_dbh_free_storage, NULL TSRMLS_CC);
