@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2013 The PHP Group                                |
+  | Copyright (c) 1997-2005 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: odbc_driver.c,v 1.27.2.2 2005/12/14 04:56:22 wez Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -104,7 +104,7 @@ void pdo_odbc_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, PDO_ODBC_HSTMT statement,
 	strcpy(*pdo_err, einfo->last_state);
 /* printf("@@ SQLSTATE[%s] %s\n", *pdo_err, einfo->last_err_msg); */
 	if (!dbh->methods) {
-		zend_throw_exception_ex(php_pdo_get_exception(), einfo->last_error TSRMLS_CC, "SQLSTATE[%s] %s: %d %s",
+		zend_throw_exception_ex(php_pdo_get_exception(), 0 TSRMLS_CC, "SQLSTATE[%s] %s: %d %s",
 				*pdo_err, what, einfo->last_error, einfo->last_err_msg);
 	}
 
@@ -114,7 +114,7 @@ void pdo_odbc_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, PDO_ODBC_HSTMT statement,
 	 * diagnostic records (which can be generated by PRINT statements
 	 * in the query, for instance). */
 	while (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO) {
-		char discard_state[6];
+		char discard_state[5];
 		char discard_buf[1024];
 		SQLINTEGER code;
 		rc = SQLGetDiagRec(htype, eh, recno++, discard_state, &code,
@@ -153,7 +153,6 @@ static int odbc_handle_preparer(pdo_dbh_t *dbh, const char *sql, long sql_len, p
 	int nsql_len = 0;
 
 	S->H = H;
-	S->assume_utf8 = H->assume_utf8;
 
 	/* before we prepare, we need to peek at the query; if it uses named parameters,
 	 * we want PDO to rewrite them for us */
@@ -199,21 +198,13 @@ static int odbc_handle_preparer(pdo_dbh_t *dbh, const char *sql, long sql_len, p
 		efree(nsql);
 	}
 
-	stmt->driver_data = S;
-	stmt->methods = &odbc_stmt_methods;
-
 	if (rc != SQL_SUCCESS) {
 		pdo_odbc_stmt_error("SQLPrepare");
-        if (rc != SQL_SUCCESS_WITH_INFO) {
-            /* clone error information into the db handle */
-            strcpy(H->einfo.last_err_msg, S->einfo.last_err_msg);
-            H->einfo.file = S->einfo.file;
-            H->einfo.line = S->einfo.line;
-            H->einfo.what = S->einfo.what;
-            strcpy(dbh->error_code, stmt->error_code);
-        }
 	}
 
+	stmt->driver_data = S;
+	stmt->methods = &odbc_stmt_methods;
+
 	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
 		return 0;
 	}
@@ -224,7 +215,7 @@ static long odbc_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRML
 {
 	pdo_odbc_db_handle *H = (pdo_odbc_db_handle *)dbh->driver_data;
 	RETCODE rc;
-	SQLLEN row_count = -1;
+	long row_count = -1;
 	PDO_ODBC_HSTMT	stmt;
 	
 	rc = SQLAllocHandle(SQL_HANDLE_STMT, H->dbc, &stmt);
@@ -235,14 +226,6 @@ static long odbc_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRML
 
 	rc = SQLExecDirect(stmt, (char *)sql, sql_len);
 
-	if (rc == SQL_NO_DATA) {
-		/* If SQLExecDirect executes a searched update or delete statement that
-		 * does not affect any rows at the data source, the call to
-		 * SQLExecDirect returns SQL_NO_DATA. */
-		row_count = 0;
-		goto out;
-	}
-
 	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
 		pdo_odbc_doer_error("SQLExecDirect");
 		goto out;
@@ -336,24 +319,8 @@ static int odbc_handle_rollback(pdo_dbh_t *dbh TSRMLS_DC)
 	return 1;
 }
 
-static int odbc_handle_set_attr(pdo_dbh_t *dbh, long attr, zval *val TSRMLS_DC)
-{
-	pdo_odbc_db_handle *H = (pdo_odbc_db_handle *)dbh->driver_data;
-	switch (attr) {
-		case PDO_ODBC_ATTR_ASSUME_UTF8:
-			H->assume_utf8 = zval_is_true(val);
-			return 1;
-		default:
-			strcpy(H->einfo.last_err_msg, "Unknown Attribute");
-			H->einfo.what = "setAttribute";
-			strcpy(H->einfo.last_state, "IM001");
-			return -1;
-	}
-}
-
 static int odbc_handle_get_attr(pdo_dbh_t *dbh, long attr, zval *val TSRMLS_DC)
 {
-	pdo_odbc_db_handle *H = (pdo_odbc_db_handle *)dbh->driver_data;
 	switch (attr) {
 		case PDO_ATTR_CLIENT_VERSION:
 			ZVAL_STRING(val, "ODBC-" PDO_ODBC_TYPE, 1);
@@ -365,9 +332,6 @@ static int odbc_handle_get_attr(pdo_dbh_t *dbh, long attr, zval *val TSRMLS_DC)
 		case PDO_ATTR_SERVER_INFO:
 		case PDO_ATTR_CONNECTION_STATUS:
 			break;
-		case PDO_ODBC_ATTR_ASSUME_UTF8:
-			ZVAL_BOOL(val, H->assume_utf8 ? 1 : 0);
-			return 1;
 
 	}
 	return 0;
@@ -381,7 +345,7 @@ static struct pdo_dbh_methods odbc_methods = {
 	odbc_handle_begin,
 	odbc_handle_commit,
 	odbc_handle_rollback,
-	odbc_handle_set_attr,
+	NULL, 	/* set attr */
 	NULL,	/* last id */
 	pdo_odbc_fetch_error_func,
 	odbc_handle_get_attr,	/* get attr */
@@ -447,8 +411,8 @@ static int pdo_odbc_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_D
 		/* Force UID and PWD to be set in the DSN */
 		if (dbh->username && *dbh->username && !strstr(dbh->data_source, "uid")
 				&& !strstr(dbh->data_source, "UID")) {
-			char *dsn;
-			spprintf(&dsn, 0, "%s;UID=%s;PWD=%s", dbh->data_source, dbh->username, dbh->password);
+			char *dsn = pemalloc(strlen(dbh->data_source) + strlen(dbh->username) + strlen(dbh->password) + sizeof(";UID=;PWD="), dbh->is_persistent);
+			sprintf(dsn, "%s;UID=%s;PWD=%s", dbh->data_source, dbh->username, dbh->password);
 			pefree((char*)dbh->data_source, dbh->is_persistent);
 			dbh->data_source = dsn;
 		}
