@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2009 The PHP Group                                |
+  | Copyright (c) 1997-2005 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: odbc_stmt.c,v 1.26.2.2.2.3.2.5 2009/05/19 10:25:53 kalle Exp $ */
+/* $Id: odbc_stmt.c,v 1.26.2.1 2005/09/30 04:19:24 wez Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -30,99 +30,6 @@
 #include "php_pdo_odbc.h"
 #include "php_pdo_odbc_int.h"
 
-enum pdo_odbc_conv_result {
-	PDO_ODBC_CONV_NOT_REQUIRED,
-	PDO_ODBC_CONV_OK,
-	PDO_ODBC_CONV_FAIL
-};
-
-static int pdo_odbc_sqltype_is_unicode(pdo_odbc_stmt *S, SWORD sqltype)
-{
-	if (!S->assume_utf8) return 0;
-	switch (sqltype) {
-#ifdef SQL_WCHAR
-		case SQL_WCHAR:
-			return 1;
-#endif
-#ifdef SQL_WLONGVARCHAR
-		case SQL_WLONGVARCHAR:
-			return 1;
-#endif
-#ifdef SQL_WVARCHAR
-		case SQL_WVARCHAR:
-			return 1;
-#endif
-		default:
-			return 0;
-	}
-}
-
-static int pdo_odbc_utf82ucs2(pdo_stmt_t *stmt, int is_unicode, const char *buf, 
-	unsigned long buflen, unsigned long *outlen)
-{
-#ifdef PHP_WIN32
-	if (is_unicode && buflen) {
-		pdo_odbc_stmt *S = (pdo_odbc_stmt*)stmt->driver_data;
-		DWORD ret;
-
-		ret = MultiByteToWideChar(CP_UTF8, 0, buf, buflen, NULL, 0);
-		if (ret == 0) {
-			/*printf("%s:%d %d [%d] %.*s\n", __FILE__, __LINE__, GetLastError(), buflen, buflen, buf);*/
-			return PDO_ODBC_CONV_FAIL;
-		}
-
-		ret *= sizeof(WCHAR);
-
-		if (S->convbufsize <= ret) {
-			S->convbufsize = ret + sizeof(WCHAR);
-			S->convbuf = erealloc(S->convbuf, S->convbufsize);
-		}
-		
-		ret = MultiByteToWideChar(CP_UTF8, 0, buf, buflen, (LPWSTR)S->convbuf, S->convbufsize / sizeof(WCHAR));
-		if (ret == 0) {
-			/*printf("%s:%d %d [%d] %.*s\n", __FILE__, __LINE__, GetLastError(), buflen, buflen, buf);*/
-			return PDO_ODBC_CONV_FAIL;
-		}
-
-		ret *= sizeof(WCHAR);
-		*outlen = ret;
-		return PDO_ODBC_CONV_OK;
-	}
-#endif
-	return PDO_ODBC_CONV_NOT_REQUIRED;
-}
-
-static int pdo_odbc_ucs22utf8(pdo_stmt_t *stmt, int is_unicode, const char *buf, 
-	unsigned long buflen, unsigned long *outlen)
-{
-#ifdef PHP_WIN32
-	if (is_unicode && buflen) {
-		pdo_odbc_stmt *S = (pdo_odbc_stmt*)stmt->driver_data;
-		DWORD ret;
-
-		ret = WideCharToMultiByte(CP_UTF8, 0, (LPCWSTR)buf, buflen/sizeof(WCHAR), NULL, 0, NULL, NULL);
-		if (ret == 0) {
-			return PDO_ODBC_CONV_FAIL;
-		}
-
-		if (S->convbufsize <= ret) {
-			S->convbufsize = ret + 1;
-			S->convbuf = erealloc(S->convbuf, S->convbufsize);
-		}
-		
-		ret = WideCharToMultiByte(CP_UTF8, 0, (LPCWSTR)buf, buflen/sizeof(WCHAR), S->convbuf, S->convbufsize, NULL, NULL);
-		if (ret == 0) {
-			return PDO_ODBC_CONV_FAIL;
-		}
-
-		*outlen = ret;
-		S->convbuf[*outlen] = '\0';
-		return PDO_ODBC_CONV_OK;
-	}
-#endif
-	return PDO_ODBC_CONV_NOT_REQUIRED;
-}
-
 static void free_cols(pdo_stmt_t *stmt, pdo_odbc_stmt *S TSRMLS_DC)
 {
 	if (S->cols) {
@@ -151,9 +58,7 @@ static int odbc_stmt_dtor(pdo_stmt_t *stmt TSRMLS_DC)
 	}
 
 	free_cols(stmt, S TSRMLS_CC);
-	if (S->convbuf) {
-		efree(S->convbuf);
-	}
+
 	efree(S);
 
 	return 1;
@@ -174,44 +79,18 @@ static int odbc_stmt_execute(pdo_stmt_t *stmt TSRMLS_DC)
 
 	while (rc == SQL_NEED_DATA) {
 		struct pdo_bound_param_data *param;
-
 		rc = SQLParamData(S->stmt, (SQLPOINTER*)&param);
 		if (rc == SQL_NEED_DATA) {
 			php_stream *stm;
 			int len;
-			pdo_odbc_param *P;
-	
-			P = (pdo_odbc_param*)param->driver_data;
+
 			if (Z_TYPE_P(param->parameter) != IS_RESOURCE) {
 				/* they passed in a string */
-				unsigned long ulen;
 				convert_to_string(param->parameter);
-
-				switch (pdo_odbc_utf82ucs2(stmt, P->is_unicode, 
-							Z_STRVAL_P(param->parameter),
-							Z_STRLEN_P(param->parameter),
-							&ulen)) {
-					case PDO_ODBC_CONV_NOT_REQUIRED:
-						SQLPutData(S->stmt, Z_STRVAL_P(param->parameter),
-							Z_STRLEN_P(param->parameter));
-						break;
-					case PDO_ODBC_CONV_OK:
-						SQLPutData(S->stmt, S->convbuf, ulen);
-						break;
-					case PDO_ODBC_CONV_FAIL:
-						pdo_odbc_stmt_error("error converting input string");
-						SQLCloseCursor(S->stmt);
-						if (buf) {
-							efree(buf);
-						}
-						return 0;
-				}
+				SQLPutData(S->stmt, Z_STRVAL_P(param->parameter), Z_STRLEN_P(param->parameter));
 				continue;
 			}
 
-			/* we assume that LOBs are binary and don't need charset
-			 * conversion */
-
 			php_stream_from_zval_no_verify(stm, &param->parameter);
 			if (!stm) {
 				/* shouldn't happen either */
@@ -267,7 +146,6 @@ static int odbc_stmt_execute(pdo_stmt_t *stmt TSRMLS_DC)
 
 		stmt->column_count = (int)colcount;
 		S->cols = ecalloc(colcount, sizeof(pdo_odbc_column));
-		S->going_long = 0;
 	}
 
 	return 1;
@@ -307,7 +185,7 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 						break;
 				}
 
-				rc = SQLDescribeParam(S->stmt, (SQLUSMALLINT) param->paramno+1, &sqltype, &precision, &scale, &nullable);
+				rc = SQLDescribeParam(S->stmt, param->paramno+1, &sqltype, &precision, &scale, &nullable);
 				if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
 					/* MS Access, for instance, doesn't support SQLDescribeParam,
 					 * so we need to guess */
@@ -334,12 +212,6 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 				P->len = 0; /* is re-populated each EXEC_PRE */
 				P->outbuf = NULL;
 
-				P->is_unicode = pdo_odbc_sqltype_is_unicode(S, sqltype);
-				if (P->is_unicode) {
-					/* avoid driver auto-translation: we'll do it ourselves */
-					ctype = SQL_C_BINARY;
-				}
-
 				if ((param->param_type & PDO_PARAM_INPUT_OUTPUT) == PDO_PARAM_INPUT_OUTPUT) {
 					P->paramtype = SQL_PARAM_INPUT_OUTPUT;
 				} else if (param->max_value_len <= 0) {
@@ -352,10 +224,7 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 					if (PDO_PARAM_TYPE(param->param_type) != PDO_PARAM_NULL) {
 						/* need an explicit buffer to hold result */
 						P->len = param->max_value_len > 0 ? param->max_value_len : precision;
-						if (P->is_unicode) {
-							P->len *= 2;
-						}
-						P->outbuf = emalloc(P->len + (P->is_unicode ? 2:1));
+						P->outbuf = emalloc(P->len + 1);
 					}
 				}
 				
@@ -364,7 +233,7 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 					return 0;
 				}
 
-				rc = SQLBindParameter(S->stmt, (SQLUSMALLINT) param->paramno+1,
+				rc = SQLBindParameter(S->stmt, param->paramno+1,
 						P->paramtype, ctype, sqltype, precision, scale,
 						P->paramtype == SQL_PARAM_INPUT ? 
 							(SQLPOINTER)param :
@@ -439,21 +308,8 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 				} else {
 					convert_to_string(param->parameter);
 					if (P->outbuf) {
-						unsigned long ulen;
-						switch (pdo_odbc_utf82ucs2(stmt, P->is_unicode,
-								Z_STRVAL_P(param->parameter),
-								Z_STRLEN_P(param->parameter),
-								&ulen)) {
-							case PDO_ODBC_CONV_FAIL:
-							case PDO_ODBC_CONV_NOT_REQUIRED:
-								P->len = Z_STRLEN_P(param->parameter);
-								memcpy(P->outbuf, Z_STRVAL_P(param->parameter), P->len);
-								break;
-							case PDO_ODBC_CONV_OK:
-								P->len = ulen;
-								memcpy(P->outbuf, S->convbuf, P->len);
-								break;
-						}
+						P->len = Z_STRLEN_P(param->parameter);
+						memcpy(P->outbuf, Z_STRVAL_P(param->parameter), P->len);
 					} else {
 						P->len = SQL_LEN_DATA_AT_EXEC(Z_STRLEN_P(param->parameter));
 					}
@@ -463,36 +319,17 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 			case PDO_PARAM_EVT_EXEC_POST:
 				P = param->driver_data;
 				if (P->outbuf) {
-					if (P->outbuf) {
-						unsigned long ulen;
-						char *srcbuf;
-						unsigned long srclen;
-
-						switch (P->len) {
-							case SQL_NULL_DATA:
-								zval_dtor(param->parameter);
-								ZVAL_NULL(param->parameter);
-								break;
-							default:
-								convert_to_string(param->parameter);
-								switch (pdo_odbc_ucs22utf8(stmt, P->is_unicode, P->outbuf, P->len, &ulen)) {
-									case PDO_ODBC_CONV_FAIL:
-										/* something fishy, but allow it to come back as binary */
-									case PDO_ODBC_CONV_NOT_REQUIRED:
-										srcbuf = P->outbuf;
-										srclen = P->len;
-										break;
-									case PDO_ODBC_CONV_OK:
-										srcbuf = S->convbuf;
-										srclen = ulen;
-										break;
-								}
-										
-								Z_STRVAL_P(param->parameter) = erealloc(Z_STRVAL_P(param->parameter), srclen+1);
-								memcpy(Z_STRVAL_P(param->parameter), srcbuf, srclen);
-								Z_STRLEN_P(param->parameter) = srclen;
-								Z_STRVAL_P(param->parameter)[srclen] = '\0';
-						}
+					switch (P->len) {
+						case SQL_NULL_DATA:
+							zval_dtor(param->parameter);
+							ZVAL_NULL(param->parameter);
+							break;
+						default:
+							convert_to_string(param->parameter);
+							Z_STRVAL_P(param->parameter) = erealloc(Z_STRVAL_P(param->parameter), P->len+1);
+							memcpy(Z_STRVAL_P(param->parameter), P->outbuf, P->len);
+							Z_STRLEN_P(param->parameter) = P->len;
+							Z_STRVAL_P(param->parameter)[P->len] = '\0';
 					}
 				}
 				return 1;
@@ -504,9 +341,9 @@ static int odbc_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *p
 static int odbc_stmt_fetch(pdo_stmt_t *stmt,
 	enum pdo_fetch_orientation ori, long offset TSRMLS_DC)
 {
+	pdo_odbc_stmt *S = (pdo_odbc_stmt*)stmt->driver_data;
 	RETCODE rc;
 	SQLSMALLINT odbcori;
-	pdo_odbc_stmt *S = (pdo_odbc_stmt*)stmt->driver_data;
 
 	switch (ori) {
 		case PDO_FETCH_ORI_NEXT:	odbcori = SQL_FETCH_NEXT; break;
@@ -521,10 +358,7 @@ static int odbc_stmt_fetch(pdo_stmt_t *stmt,
 	}
 	rc = SQLFetchScroll(S->stmt, odbcori, offset);
 
-	if (rc == SQL_SUCCESS) {
-		return 1;
-	}
-	if (rc == SQL_SUCCESS_WITH_INFO) {
+	if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO) {
 		pdo_odbc_stmt_error("SQLFetchScroll");
 		return 1;
 	}
@@ -546,50 +380,31 @@ static int odbc_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC)
 	zend_bool dyn = FALSE;
 	RETCODE rc;
 	SWORD	colnamelen;
-	SDWORD	colsize, displaysize;
+	SDWORD	colsize;
 
 	rc = SQLDescribeCol(S->stmt, colno+1, S->cols[colno].colname,
 			sizeof(S->cols[colno].colname)-1, &colnamelen,
 			&S->cols[colno].coltype, &colsize, NULL, NULL);
 
 	if (rc != SQL_SUCCESS) {
-		pdo_odbc_stmt_error("SQLDescribeCol");
-		if (rc != SQL_SUCCESS_WITH_INFO) {
-			return 0;
-		}
+		pdo_odbc_stmt_error("SQLBindCol");
+		return 0;
 	}
 
-	rc = SQLColAttribute(S->stmt, colno+1,
-			SQL_DESC_DISPLAY_SIZE,
-			NULL, 0, NULL, &displaysize);
-
-	if (rc != SQL_SUCCESS) {
-		pdo_odbc_stmt_error("SQLColAttribute");
-		if (rc != SQL_SUCCESS_WITH_INFO) {
-			return 0;
-		}
-	}
-	colsize = displaysize;
-
 	col->maxlen = S->cols[colno].datalen = colsize;
 	col->namelen = colnamelen;
 	col->name = estrdup(S->cols[colno].colname);
-	S->cols[colno].is_unicode = pdo_odbc_sqltype_is_unicode(S, S->cols[colno].coltype);
 
 	/* returning data as a string */
 	col->param_type = PDO_PARAM_STR;
 
 	/* tell ODBC to put it straight into our buffer, but only if it
-	 * isn't "long" data, and only if we haven't already bound a long
-	 * column. */
-	if (colsize < 256 && !S->going_long) {
+	 * isn't "long" data */
+	if (colsize < 256) {
 		S->cols[colno].data = emalloc(colsize+1);
-		S->cols[colno].is_long = 0;
 
-		rc = SQLBindCol(S->stmt, colno+1,
-			S->cols[colno].is_unicode ? SQL_C_BINARY : SQL_C_CHAR,
-			S->cols[colno].data,
- 			S->cols[colno].datalen+1, &S->cols[colno].fetched_len);
+		rc = SQLBindCol(S->stmt, colno+1, SQL_C_CHAR, S->cols[colno].data,
+			S->cols[colno].datalen+1, &S->cols[colno].fetched_len);
 
 		if (rc != SQL_SUCCESS) {
 			pdo_odbc_stmt_error("SQLBindCol");
@@ -599,8 +414,6 @@ static int odbc_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC)
 		/* allocate a smaller buffer to keep around for smaller
 		 * "long" columns */
 		S->cols[colno].data = emalloc(256);
-		S->going_long = 1;
-		S->cols[colno].is_long = 1;
 	}
 
 	return 1;
@@ -610,10 +423,9 @@ static int odbc_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned l
 {
 	pdo_odbc_stmt *S = (pdo_odbc_stmt*)stmt->driver_data;
 	pdo_odbc_column *C = &S->cols[colno];
-	unsigned long ulen;
 
 	/* if it is a column containing "long" data, perform late binding now */
-	if (C->is_long) {
+	if (C->datalen > 255) {
 		unsigned long alloced = 4096;
 		unsigned long used = 0;
 		char *buf;
@@ -623,8 +435,8 @@ static int odbc_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned l
 		 * of 256 bytes; if there is more to be had, we then allocate
 		 * bigger buffer for the caller to free */
 
-		rc = SQLGetData(S->stmt, colno+1, C->is_unicode ? SQL_C_BINARY : SQL_C_CHAR, C->data,
- 			256, &C->fetched_len);
+		rc = SQLGetData(S->stmt, colno+1, SQL_C_CHAR, C->data,
+			256, &C->fetched_len);
 
 		if (rc == SQL_SUCCESS) {
 			/* all the data fit into our little buffer;
@@ -653,11 +465,6 @@ static int odbc_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned l
 				if (rc == SQL_NO_DATA) {
 					/* we got the lot */
 					break;
-				} else if (rc != SQL_SUCCESS) {
-					pdo_odbc_stmt_error("SQLGetData");
-					if (rc != SQL_SUCCESS_WITH_INFO) {
-						break;
-					}
 				}
 
 				if (C->fetched_len == SQL_NO_TOTAL) {
@@ -666,11 +473,6 @@ static int odbc_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned l
 					used += C->fetched_len;
 				}
 
-				if (rc == SQL_SUCCESS) {
-					/* this was the final fetch */
-					break;
-				}
-
 				/* we need to fetch another chunk; resize the
 				 * buffer */
 				alloced *= 2;
@@ -686,9 +488,6 @@ static int odbc_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned l
 			*ptr = buf;
 			*caller_frees = 1;
 			*len = used;
-			if (C->is_unicode) {
-				goto unicode_conv;
-			}
 			return 1;
 		}
 
@@ -709,9 +508,6 @@ in_data:
 		/* it was stored perfectly */
 		*ptr = C->data;
 		*len = C->fetched_len;
-		if (C->is_unicode) {
-			goto unicode_conv;
-		}
 		return 1;
 	} else {
 		/* no data? */
@@ -719,26 +515,6 @@ in_data:
 		*len = 0;
 		return 1;
 	}
-
-	unicode_conv:
-	switch (pdo_odbc_ucs22utf8(stmt, C->is_unicode, *ptr, *len, &ulen)) {
-		case PDO_ODBC_CONV_FAIL:
-			/* oh well.  They can have the binary version of it */
-		case PDO_ODBC_CONV_NOT_REQUIRED:
-			/* shouldn't happen... */
-			return 1;
-
-		case PDO_ODBC_CONV_OK:
-			if (*caller_frees) {
-				efree(*ptr);
-			}
-			*ptr = emalloc(ulen + 1);
-			*len = ulen;
-			memcpy(*ptr, S->convbuf, ulen+1);
-			*caller_frees = 1;
-			return 1;
-	}
-	return 1;
 }
 
 static int odbc_stmt_set_param(pdo_stmt_t *stmt, long attr, zval *val TSRMLS_DC)
@@ -757,9 +533,6 @@ static int odbc_stmt_set_param(pdo_stmt_t *stmt, long attr, zval *val TSRMLS_DC)
 			pdo_odbc_stmt_error("SQLSetCursorName");
 			return 0;
 
-		case PDO_ODBC_ATTR_ASSUME_UTF8:
-			S->assume_utf8 = zval_is_true(val);
-			return 0;
 		default:
 			strcpy(S->einfo.last_err_msg, "Unknown Attribute");
 			S->einfo.what = "setAttribute";
@@ -788,10 +561,6 @@ static int odbc_stmt_get_attr(pdo_stmt_t *stmt, long attr, zval *val TSRMLS_DC)
 			return 0;
 		}
 
-		case PDO_ODBC_ATTR_ASSUME_UTF8:
-			ZVAL_BOOL(val, S->assume_utf8 ? 1 : 0);
-			return 0;
-
 		default:
 			strcpy(S->einfo.last_err_msg, "Unknown Attribute");
 			S->einfo.what = "getAttribute";
@@ -820,7 +589,6 @@ static int odbc_stmt_next_rowset(pdo_stmt_t *stmt TSRMLS_DC)
 	SQLNumResultCols(S->stmt, &colcount);
 	stmt->column_count = (int)colcount;
 	S->cols = ecalloc(colcount, sizeof(pdo_odbc_column));
-	S->going_long = 0;
 
 	return 1;
 }
