@@ -44,6 +44,7 @@ int sqlite3_exec(
   char **azCols = 0;
 
   int nRetry = 0;
+  int nChange = 0;
   int nCallback;
 
   if( zSql==0 ) return SQLITE_OK;
@@ -53,8 +54,8 @@ int sqlite3_exec(
 
     pStmt = 0;
     rc = sqlite3_prepare(db, zSql, -1, &pStmt, &zLeftover);
-    assert( rc==SQLITE_OK || pStmt==0 );
     if( rc!=SQLITE_OK ){
+      if( pStmt ) sqlite3_finalize(pStmt);
       continue;
     }
     if( !pStmt ){
@@ -63,11 +64,13 @@ int sqlite3_exec(
       continue;
     }
 
+    db->nChange += nChange;
     nCallback = 0;
 
     nCol = sqlite3_column_count(pStmt);
-    azCols = sqliteMalloc(2*nCol*sizeof(const char *) + 1);
-    if( azCols==0 ){
+    azCols = sqliteMalloc(2*nCol*sizeof(const char *));
+    if( nCol && !azCols ){
+      rc = SQLITE_NOMEM;
       goto exec_out;
     }
 
@@ -99,6 +102,9 @@ int sqlite3_exec(
       if( rc!=SQLITE_ROW ){
         rc = sqlite3_finalize(pStmt);
         pStmt = 0;
+        if( db->pVdbe==0 ){
+          nChange = db->nChange;
+        }
         if( rc!=SQLITE_SCHEMA ){
           nRetry = 0;
           zSql = zLeftover;
@@ -116,9 +122,11 @@ exec_out:
   if( pStmt ) sqlite3_finalize(pStmt);
   if( azCols ) sqliteFree(azCols);
 
-  rc = sqlite3ApiExit(0, rc);
+  if( sqlite3_malloc_failed ){
+    rc = SQLITE_NOMEM;
+  }
   if( rc!=SQLITE_OK && rc==sqlite3_errcode(db) && pzErrMsg ){
-    *pzErrMsg = sqlite3_malloc(1+strlen(sqlite3_errmsg(db)));
+    *pzErrMsg = malloc(1+strlen(sqlite3_errmsg(db)));
     if( *pzErrMsg ){
       strcpy(*pzErrMsg, sqlite3_errmsg(db));
     }
@@ -126,6 +134,5 @@ exec_out:
     *pzErrMsg = 0;
   }
 
-  assert( (rc&db->errMask)==rc );
   return rc;
 }
