@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: 967584c6fadb3467f31abe8e13caa8764df85867 $ */
+/* $Id: posix.c,v 1.70.2.3 2006/01/01 12:50:12 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,11 +36,6 @@
 #endif
 
 #include <sys/resource.h>
-
-#if defined(_GNU_SOURCE) && !defined(__USE_GNU)
-# define __USE_GNU
-#endif
-
 #include <sys/utsname.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -54,254 +49,92 @@
 #endif
 
 ZEND_DECLARE_MODULE_GLOBALS(posix)
-static PHP_MINFO_FUNCTION(posix);
-
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_kill, 0, 0, 2)
-	ZEND_ARG_INFO(0, pid)
-	ZEND_ARG_INFO(0, sig)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getpid, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getppid, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getuid, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setuid, 0, 0, 1)
-	ZEND_ARG_INFO(0, uid)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_geteuid, 0)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_SETEUID
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_seteuid, 0, 0, 1)
-	ZEND_ARG_INFO(0, uid)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getgid, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setgid, 0, 0, 1)
-	ZEND_ARG_INFO(0, gid)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getegid, 0)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_SETEGID
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setegid, 0, 0, 1)
-	ZEND_ARG_INFO(0, gid)
-ZEND_END_ARG_INFO()
-#endif
-
-#ifdef HAVE_GETGROUPS
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getgroups, 0)
-ZEND_END_ARG_INFO()
-#endif
-
-#ifdef HAVE_GETLOGIN
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getlogin, 0)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getpgrp, 0)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_SETSID
-ZEND_BEGIN_ARG_INFO(arginfo_posix_setsid, 0)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setpgid, 0, 0, 2)
-	ZEND_ARG_INFO(0, pid)
-	ZEND_ARG_INFO(0, pgid)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_GETPGID
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpgid, 0, 0, 1)
-	ZEND_ARG_INFO(0, pid)
-ZEND_END_ARG_INFO()
-#endif
-
-#ifdef HAVE_GETSID
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getsid, 0, 0, 1)
-	ZEND_ARG_INFO(0, pid)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_uname, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_times, 0)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_CTERMID
-ZEND_BEGIN_ARG_INFO(arginfo_posix_ctermid, 0)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_ttyname, 0, 0, 1)
-	ZEND_ARG_INFO(0, fd)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_isatty, 0, 0, 1)
-	ZEND_ARG_INFO(0, fd)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getcwd, 0)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_MKFIFO
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_mkfifo, 0, 0, 2)
-	ZEND_ARG_INFO(0, pathname)
-	ZEND_ARG_INFO(0, mode)
-ZEND_END_ARG_INFO()
-#endif
-
-#ifdef HAVE_MKNOD
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_mknod, 0, 0, 2)
-	ZEND_ARG_INFO(0, pathname)
-	ZEND_ARG_INFO(0, mode)
-	ZEND_ARG_INFO(0, major)
-	ZEND_ARG_INFO(0, minor)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_access, 0, 0, 1)
-	ZEND_ARG_INFO(0, file)
-	ZEND_ARG_INFO(0, mode)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getgrnam, 0, 0, 1)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getgrgid, 0, 0, 1)
-	ZEND_ARG_INFO(0, gid)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpwnam, 0, 0, 1)
-	ZEND_ARG_INFO(0, username)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpwuid, 0, 0, 1)
-	ZEND_ARG_INFO(0, uid)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_GETRLIMIT
-ZEND_BEGIN_ARG_INFO(arginfo_posix_getrlimit, 0)
-ZEND_END_ARG_INFO()
-#endif
-
-ZEND_BEGIN_ARG_INFO(arginfo_posix_get_last_error, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_strerror, 0, 0, 1)
-	ZEND_ARG_INFO(0, errno)
-ZEND_END_ARG_INFO()
-
-#ifdef HAVE_INITGROUPS
-ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_initgroups, 0, 0, 2)
-	ZEND_ARG_INFO(0, name)
-	ZEND_ARG_INFO(0, base_group_id)
-ZEND_END_ARG_INFO()
-#endif
-/* }}} */
 
 /* {{{ posix_functions[]
  */
-const zend_function_entry posix_functions[] = {
+zend_function_entry posix_functions[] = {
     /* POSIX.1, 3.3 */
-	PHP_FE(posix_kill,		arginfo_posix_kill)
+	PHP_FE(posix_kill,		NULL)
 
 	/* POSIX.1, 4.1 */
-	PHP_FE(posix_getpid,	arginfo_posix_getpid)
-	PHP_FE(posix_getppid,	arginfo_posix_getppid)
+	PHP_FE(posix_getpid,	NULL)
+	PHP_FE(posix_getppid,	NULL)
 
 	/* POSIX.1,  4.2 */
-	PHP_FE(posix_getuid,	arginfo_posix_getuid)
-	PHP_FE(posix_setuid,	arginfo_posix_setuid)
-	PHP_FE(posix_geteuid,	arginfo_posix_geteuid)
+	PHP_FE(posix_getuid,	NULL)
+	PHP_FE(posix_setuid,	NULL)
+	PHP_FE(posix_geteuid,	NULL)
 #ifdef HAVE_SETEUID
-	PHP_FE(posix_seteuid,	arginfo_posix_seteuid)
+	PHP_FE(posix_seteuid,	NULL)
 #endif
-	PHP_FE(posix_getgid,	arginfo_posix_getgid)
-	PHP_FE(posix_setgid,	arginfo_posix_setgid)
-	PHP_FE(posix_getegid,	arginfo_posix_getegid)
+	PHP_FE(posix_getgid,	NULL)
+	PHP_FE(posix_setgid,	NULL)
+	PHP_FE(posix_getegid,	NULL)
 #ifdef HAVE_SETEGID
-	PHP_FE(posix_setegid,	arginfo_posix_setegid)
+	PHP_FE(posix_setegid,	NULL)
 #endif
 #ifdef HAVE_GETGROUPS
-	PHP_FE(posix_getgroups,	arginfo_posix_getgroups)
+	PHP_FE(posix_getgroups,	NULL)
 #endif
 #ifdef HAVE_GETLOGIN
-	PHP_FE(posix_getlogin,	arginfo_posix_getlogin)
+	PHP_FE(posix_getlogin,	NULL)
 #endif
 
 	/* POSIX.1, 4.3 */
-	PHP_FE(posix_getpgrp,	arginfo_posix_getpgrp)
+	PHP_FE(posix_getpgrp,	NULL)
 #ifdef HAVE_SETSID
-	PHP_FE(posix_setsid,	arginfo_posix_setsid)
+	PHP_FE(posix_setsid,	NULL)
 #endif
-	PHP_FE(posix_setpgid,	arginfo_posix_setpgid)
+	PHP_FE(posix_setpgid,	NULL)
 	/* Non-Posix functions which are common */
 #ifdef HAVE_GETPGID
-	PHP_FE(posix_getpgid,	arginfo_posix_getpgid)
+	PHP_FE(posix_getpgid,	NULL)
 #endif /* HAVE_GETPGID */
 #ifdef HAVE_GETSID
-	PHP_FE(posix_getsid,	arginfo_posix_getsid)
+	PHP_FE(posix_getsid,	NULL)
 #endif /* HAVE_GETSID */
 
 	/* POSIX.1, 4.4 */
-	PHP_FE(posix_uname,		arginfo_posix_uname)
+	PHP_FE(posix_uname,		NULL)
 
 	/* POSIX.1, 4.5 */
-	PHP_FE(posix_times,		arginfo_posix_times)
+	PHP_FE(posix_times,		NULL)
 
 	/* POSIX.1, 4.7 */
 #ifdef HAVE_CTERMID
-	PHP_FE(posix_ctermid,	arginfo_posix_ctermid)
+	PHP_FE(posix_ctermid,	NULL)
 #endif
-	PHP_FE(posix_ttyname,	arginfo_posix_ttyname)
-	PHP_FE(posix_isatty,	arginfo_posix_isatty)
+	PHP_FE(posix_ttyname,	NULL)
+	PHP_FE(posix_isatty,	NULL)
 
     /* POSIX.1, 5.2 */
-	PHP_FE(posix_getcwd,	arginfo_posix_getcwd)
+	PHP_FE(posix_getcwd,	NULL)
 
 	/* POSIX.1, 5.4 */
 #ifdef HAVE_MKFIFO
-	PHP_FE(posix_mkfifo,	arginfo_posix_mkfifo)
+	PHP_FE(posix_mkfifo,	NULL)
 #endif
 #ifdef HAVE_MKNOD
-	PHP_FE(posix_mknod,		arginfo_posix_mknod)
+	PHP_FE(posix_mknod,	NULL)
 #endif
 
 	/* POSIX.1, 5.6 */
-	PHP_FE(posix_access,	arginfo_posix_access)
+	PHP_FE(posix_access,	NULL)
 	/* POSIX.1, 9.2 */
-	PHP_FE(posix_getgrnam,	arginfo_posix_getgrnam)
-	PHP_FE(posix_getgrgid,	arginfo_posix_getgrgid)
-	PHP_FE(posix_getpwnam,	arginfo_posix_getpwnam)
-	PHP_FE(posix_getpwuid,	arginfo_posix_getpwuid)
+	PHP_FE(posix_getgrnam,	NULL)
+	PHP_FE(posix_getgrgid,	NULL)
+	PHP_FE(posix_getpwnam,	NULL)
+	PHP_FE(posix_getpwuid,	NULL)
 
 #ifdef HAVE_GETRLIMIT
-	PHP_FE(posix_getrlimit,	arginfo_posix_getrlimit)
+	PHP_FE(posix_getrlimit,	NULL)
 #endif
 
-	PHP_FE(posix_get_last_error,					arginfo_posix_get_last_error)
-	PHP_FALIAS(posix_errno, posix_get_last_error,	arginfo_posix_get_last_error)
-	PHP_FE(posix_strerror,							arginfo_posix_strerror)
-#ifdef HAVE_INITGROUPS
-	PHP_FE(posix_initgroups,	arginfo_posix_initgroups)
-#endif
+	PHP_FE(posix_get_last_error,					NULL)
+	PHP_FALIAS(posix_errno, posix_get_last_error,	NULL)
+	PHP_FE(posix_strerror,							NULL)
 
-	PHP_FE_END
+	{NULL, NULL, NULL}
 };
 /* }}} */
 
@@ -310,21 +143,21 @@ const zend_function_entry posix_functions[] = {
 static PHP_MINFO_FUNCTION(posix)
 {
 	php_info_print_table_start();
-	php_info_print_table_row(2, "Revision", "$Id: 967584c6fadb3467f31abe8e13caa8764df85867 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 1.70.2.3 $");
 	php_info_print_table_end();
 }
 /* }}} */
 
-static PHP_GINIT_FUNCTION(posix) /* {{{ */
+static void php_posix_init_globals(zend_posix_globals *posix_globals TSRMLS_DC)
 {
 	posix_globals->last_error = 0;
 }
-/* }}} */
 
 /* {{{ PHP_MINIT_FUNCTION(posix)
  */
 static PHP_MINIT_FUNCTION(posix)
 {
+	ZEND_INIT_MODULE_GLOBALS(posix, php_posix_init_globals, NULL);
 	REGISTER_LONG_CONSTANT("POSIX_F_OK", F_OK, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("POSIX_X_OK", X_OK, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("POSIX_W_OK", W_OK, CONST_CS | CONST_PERSISTENT);
@@ -349,6 +182,8 @@ static PHP_MINIT_FUNCTION(posix)
 }
 /* }}} */
 
+static PHP_MINFO_FUNCTION(posix);
+
 /* {{{ posix_module_entry
  */
 zend_module_entry posix_module_entry = {
@@ -360,12 +195,8 @@ zend_module_entry posix_module_entry = {
 	NULL,
 	NULL, 
 	PHP_MINFO(posix),
-	NO_VERSION_YET,
-	PHP_MODULE_GLOBALS(posix),
-	PHP_GINIT(posix),
-	NULL,
-	NULL,
-	STANDARD_MODULE_PROPERTIES_EX
+    NO_VERSION_YET,
+	STANDARD_MODULE_PROPERTIES
 };
 /* }}} */
 
@@ -373,7 +204,7 @@ zend_module_entry posix_module_entry = {
 ZEND_GET_MODULE(posix)
 #endif
 
-#define PHP_POSIX_NO_ARGS	if (zend_parse_parameters_none() == FAILURE) return;
+#define PHP_POSIX_NO_ARGS	if (ZEND_NUM_ARGS()) return;
 
 #define PHP_POSIX_RETURN_LONG_FUNC(func_name)	\
 	PHP_POSIX_NO_ARGS	\
@@ -631,8 +462,7 @@ PHP_FUNCTION(posix_uname)
 	add_assoc_string(return_value, "release",  u.release,  1);
 	add_assoc_string(return_value, "version",  u.version,  1);
 	add_assoc_string(return_value, "machine",  u.machine,  1);
-
-#if defined(_GNU_SOURCE) && !defined(DARWIN) && defined(HAVE_UTSNAME_DOMAINNAME)
+#ifdef _GNU_SOURCE
 	add_assoc_string(return_value, "domainname", u.domainname, 1);
 #endif
 }
@@ -651,7 +481,7 @@ PHP_FUNCTION(posix_times)
 
 	PHP_POSIX_NO_ARGS;
 
-	if ((ticks = times(&t)) == -1) {
+	if((ticks = times(&t)) < 0) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
@@ -693,7 +523,7 @@ PHP_FUNCTION(posix_ctermid)
 /* }}} */
 
 /* Checks if the provides resource is a stream and if it provides a file descriptor */
-static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC) /* {{{ */
+static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC)
 {
 	php_stream *stream;
 
@@ -703,9 +533,7 @@ static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC) /* {{{ */
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "expects argument 1 to be a valid stream resource");
 		return 0;
 	}
-	if (php_stream_can_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT) == SUCCESS) {
-		php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT, (void*)fd, 0);
-	} else if (php_stream_can_cast(stream, PHP_STREAM_AS_FD) == SUCCESS) {
+	if (php_stream_can_cast(stream, PHP_STREAM_AS_FD) == SUCCESS) {
 		php_stream_cast(stream, PHP_STREAM_AS_FD, (void*)fd, 0);
 	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "could not use stream of type '%s'", 
@@ -714,52 +542,35 @@ static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC) /* {{{ */
 	}
 	return 1;
 }
-/* }}} */
 
 /* {{{ proto string posix_ttyname(int fd)
    Determine terminal device name (POSIX.1, 4.7.2) */
 PHP_FUNCTION(posix_ttyname)
 {
-	zval **z_fd;
+	zval *z_fd;
 	char *p;
 	int fd;
-#if defined(ZTS) && defined(HAVE_TTYNAME_R) && defined(_SC_TTY_NAME_MAX)
-	long buflen;
-#endif
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &z_fd) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-	switch (Z_TYPE_PP(z_fd)) {
+	switch (Z_TYPE_P(z_fd)) {
 		case IS_RESOURCE:
-			if (!php_posix_stream_get_fd(*z_fd, &fd TSRMLS_CC)) {
+			if (!php_posix_stream_get_fd(z_fd, &fd TSRMLS_CC)) {
 				RETURN_FALSE;
 			}
 			break;
 		default:
-			convert_to_long_ex(z_fd);
-			fd = Z_LVAL_PP(z_fd);
+			convert_to_long(z_fd);
+			fd = Z_LVAL_P(z_fd);
 	}
-#if defined(ZTS) && defined(HAVE_TTYNAME_R) && defined(_SC_TTY_NAME_MAX)
-	buflen = sysconf(_SC_TTY_NAME_MAX);
-	if (buflen < 1) {
-		RETURN_FALSE;
-	}
-	p = emalloc(buflen);
 
-	if (ttyname_r(fd, p, buflen)) {
-		POSIX_G(last_error) = errno;
-		efree(p);
-		RETURN_FALSE;
-	}
-	RETURN_STRING(p, 0);
-#else
 	if (NULL == (p = ttyname(fd))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
-#endif	
+	
 	RETURN_STRING(p, 1);
 }
 /* }}} */
@@ -768,22 +579,22 @@ PHP_FUNCTION(posix_ttyname)
    Determine if filedesc is a tty (POSIX.1, 4.7.1) */
 PHP_FUNCTION(posix_isatty)
 {
-	zval **z_fd;
+	zval *z_fd;
 	int fd;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &z_fd) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
-	switch (Z_TYPE_PP(z_fd)) {
+	switch (Z_TYPE_P(z_fd)) {
 		case IS_RESOURCE:
-			if (!php_posix_stream_get_fd(*z_fd, &fd TSRMLS_CC)) {
+			if (!php_posix_stream_get_fd(z_fd, &fd TSRMLS_CC)) {
 				RETURN_FALSE;
 			}
 			break;
 		default:
-			convert_to_long_ex(z_fd);
-			fd = Z_LVAL_PP(z_fd);
+			convert_to_long(z_fd);
+			fd = Z_LVAL_P(z_fd);
 	}
 
 	if (isatty(fd)) {
@@ -838,11 +649,11 @@ PHP_FUNCTION(posix_mkfifo)
 	long mode;
 	int     result;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "pl", &path, &path_len, &mode) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &path, &path_len, &mode) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir_ex(path, 0 TSRMLS_CC)) {
+	if (PG(safe_mode) && (!php_checkuid(path, NULL, CHECKUID_ALLOW_ONLY_DIR))) {
 		RETURN_FALSE;
 	}
 
@@ -871,12 +682,13 @@ PHP_FUNCTION(posix_mknod)
 
 	php_dev = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "pl|ll", &path, &path_len,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl|ll", &path, &path_len,
 			&mode, &major, &minor) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir_ex(path, 0 TSRMLS_CC)) {
+	if (php_check_open_basedir_ex(path, 0 TSRMLS_CC) ||
+			(PG(safe_mode) && (!php_checkuid(path, NULL, CHECKUID_ALLOW_ONLY_DIR)))) {
 		RETURN_FALSE;
 	}
 
@@ -893,7 +705,7 @@ PHP_FUNCTION(posix_mknod)
 #if defined(HAVE_MAKEDEV) || defined(makedev)
 			php_dev = makedev(major, minor);
 #else
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create a block or character device, creating a normal file instead");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Can not create a block or character device, creating a normal file instead");
 #endif
 		}
 	}
@@ -911,8 +723,7 @@ PHP_FUNCTION(posix_mknod)
 
 /* Takes a pointer to posix group and a pointer to an already initialized ZVAL
  * array container and fills the array with the posix group member data. */
-int php_posix_group_to_array(struct group *g, zval *array_group) /* {{{ */
-{
+int php_posix_group_to_array(struct group *g, zval *array_group) {
 	zval *array_members;
 	int count;
 
@@ -934,7 +745,6 @@ int php_posix_group_to_array(struct group *g, zval *array_group) /* {{{ */
 	add_assoc_long(array_group, "gid", g->gr_gid);
 	return 1;
 }
-/* }}} */
 
 /*
 	POSIX.1, 5.5.1 unlink()
@@ -951,17 +761,14 @@ PHP_FUNCTION(posix_access)
 	int filename_len, ret;
 	char *filename, *path;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|l", &filename, &filename_len, &mode) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &filename, &filename_len, &mode) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	path = expand_filepath(filename, NULL TSRMLS_CC);
-	if (!path) {
-		POSIX_G(last_error) = EIO;
-		RETURN_FALSE;
-	}
 
-	if (php_check_open_basedir_ex(path, 0 TSRMLS_CC)) {
+	if (php_check_open_basedir_ex(path, 0 TSRMLS_CC) ||
+			(PG(safe_mode) && (!php_checkuid_ex(filename, NULL, CHECKUID_CHECK_FILE_AND_DIR, CHECKUID_NO_ERRORS)))) {
 		efree(path);
 		POSIX_G(last_error) = EPERM;
 		RETURN_FALSE;
@@ -993,45 +800,22 @@ PHP_FUNCTION(posix_getgrnam)
 	char *name;
 	struct group *g;
 	int name_len;
-#if defined(ZTS) && defined(HAVE_GETGRNAM_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	struct group gbuf;
-	long buflen;
-	char *buf;
-#endif
 	
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-#if defined(ZTS) && defined(HAVE_GETGRNAM_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	buflen = sysconf(_SC_GETGR_R_SIZE_MAX);
-	if (buflen < 1) {
-		RETURN_FALSE;
-	}
-	buf = emalloc(buflen);
-	g = &gbuf;
-
-	if (getgrnam_r(name, g, buf, buflen, &g) || g == NULL) {
-		POSIX_G(last_error) = errno;
-		efree(buf);
-		RETURN_FALSE;
-	}
-#else
 	if (NULL == (g = getgrnam(name))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
-#endif
+	
 	array_init(return_value);
 
 	if (!php_posix_group_to_array(g, return_value)) {
-		zval_dtor(return_value);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to convert posix group to array");
-		RETVAL_FALSE;
+		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(HAVE_GETGRNAM_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	efree(buf);
-#endif
 }
 /* }}} */
 
@@ -1040,55 +824,27 @@ PHP_FUNCTION(posix_getgrnam)
 PHP_FUNCTION(posix_getgrgid)
 {
 	long gid;
-#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	int ret;
-	struct group _g;
-	struct group *retgrptr = NULL;
-	long grbuflen;
-	char *grbuf;
-#endif
 	struct group *g;
 	
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &gid) == FAILURE) {
 		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	
-	grbuflen = sysconf(_SC_GETGR_R_SIZE_MAX);
-	if (grbuflen < 1) {
-		RETURN_FALSE;
-	}
 
-	grbuf = emalloc(grbuflen);
-
-	ret = getgrgid_r(gid, &_g, grbuf, grbuflen, &retgrptr);
-	if (ret || retgrptr == NULL) {
-		POSIX_G(last_error) = ret;
-		efree(grbuf);
-		RETURN_FALSE;
-	}
-	g = &_g;
-#else
 	if (NULL == (g = getgrgid(gid))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
-#endif
+
 	array_init(return_value);
 
 	if (!php_posix_group_to_array(g, return_value)) {
-		zval_dtor(return_value);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to convert posix group struct to array");
-		RETVAL_FALSE;
+		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
-	efree(grbuf);
-#endif
 }
 /* }}} */
 
-int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) /* {{{ */
-{
+int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) {
 	if (NULL == pw)
 		return 0;
 	if (NULL == return_value || Z_TYPE_P(return_value) != IS_ARRAY)
@@ -1103,7 +859,6 @@ int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) /* {{{ */
 	add_assoc_string(return_value, "shell",     pw->pw_shell, 1);
 	return 1;
 }
-/* }}} */
 
 /* {{{ proto array posix_getpwnam(string groupname) 
    User database access (POSIX.1, 9.2.2) */
@@ -1112,45 +867,23 @@ PHP_FUNCTION(posix_getpwnam)
 	struct passwd *pw;
 	char *name;
 	int name_len;
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWNAM_R)
-	struct passwd pwbuf;
-	long buflen;
-	char *buf;
-#endif
-
+	
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWNAM_R)
-	buflen = sysconf(_SC_GETPW_R_SIZE_MAX);
-	if (buflen < 1) {
-		RETURN_FALSE;
-	}
-	buf = emalloc(buflen);
-	pw = &pwbuf;
-
-	if (getpwnam_r(name, pw, buf, buflen, &pw) || pw == NULL) {
-		efree(buf);
-		POSIX_G(last_error) = errno;
-		RETURN_FALSE;
-	}
-#else
 	if (NULL == (pw = getpwnam(name))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
-#endif	
+	
 	array_init(return_value);
 
 	if (!php_posix_passwd_to_array(pw, return_value)) {
-		zval_dtor(return_value);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to convert posix passwd struct to array");
-		RETVAL_FALSE;
+		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWNAM_R)
-	efree(buf);
-#endif
+
 }
 /* }}} */
 
@@ -1159,48 +892,23 @@ PHP_FUNCTION(posix_getpwnam)
 PHP_FUNCTION(posix_getpwuid)
 {
 	long uid;
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWUID_R)
-	struct passwd _pw;
-	struct passwd *retpwptr = NULL;
-	long pwbuflen;
-	char *pwbuf;
-	int ret;
-#endif
 	struct passwd *pw;
-
+	
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &uid) == FAILURE) {
 		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWUID_R)
-	pwbuflen = sysconf(_SC_GETPW_R_SIZE_MAX);
-	if (pwbuflen < 1) {
-		RETURN_FALSE;
-	}
-	pwbuf = emalloc(pwbuflen);
 
-	ret = getpwuid_r(uid, &_pw, pwbuf, pwbuflen, &retpwptr);
-	if (ret || retpwptr == NULL) {
-		POSIX_G(last_error) = ret;
-		efree(pwbuf);
-		RETURN_FALSE;
-	}
-	pw = &_pw;
-#else
 	if (NULL == (pw = getpwuid(uid))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
 	}
-#endif
+
 	array_init(return_value);
 
 	if (!php_posix_passwd_to_array(pw, return_value)) {
-		zval_dtor(return_value);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to convert posix passwd struct to array");
-		RETVAL_FALSE;
+		RETURN_FALSE;
 	}
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWUID_R)
-	efree(pwbuf);
-#endif
 }
 /* }}} */
 
@@ -1312,10 +1020,8 @@ PHP_FUNCTION(posix_getrlimit)
 	array_init(return_value);
 
 	for (l=limits; l->name; l++) {
-		if (posix_addlimit(l->limit, l->name, return_value TSRMLS_CC) == FAILURE) {
-			zval_dtor(return_value);
+		if (posix_addlimit(l->limit, l->name, return_value TSRMLS_CC) == FAILURE)
 			RETURN_FALSE;
-		}
 	}
 }
 /* }}} */
@@ -1348,28 +1054,6 @@ PHP_FUNCTION(posix_strerror)
 
 #endif
 
-#ifdef HAVE_INITGROUPS
-/* {{{ proto bool posix_initgroups(string name, int base_group_id)
-   Calculate the group access list for the user specified in name. */
-PHP_FUNCTION(posix_initgroups)
-{
-	long basegid;
-	char *name;
-	int name_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &name, &name_len, &basegid) == FAILURE) {
-		RETURN_FALSE;
-	}
-
-	if (name_len == 0) {
-		RETURN_FALSE;
-	}
-
-	RETURN_BOOL(!initgroups((const char *)name, basegid));
-}
-/* }}} */
-#endif
-
 /*
  * Local variables:
  * tab-width: 4
