@@ -1,13 +1,13 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2003 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,6 +16,8 @@
    +----------------------------------------------------------------------+
  */
 
+/* $Id: php_session.h,v 1.101.2.1 2006/01/01 12:50:12 sniper Exp $ */
+
 #ifndef PHP_SESSION_H
 #define PHP_SESSION_H
 
@@ -32,7 +34,7 @@
 #define PS_CREATE_SID_ARGS void **mod_data, int *newlen TSRMLS_DC
 
 /* default create id function */
-char *php_session_create_id(PS_CREATE_SID_ARGS);
+PHPAPI char *php_session_create_id(PS_CREATE_SID_ARGS);
 
 typedef struct ps_module_struct {
 	const char *s_name;
@@ -118,6 +120,9 @@ typedef struct _php_ps_globals {
 	zend_bool use_only_cookies;
 	zend_bool use_trans_sid;	/* contains the INI value of whether to use trans-sid */
 	zend_bool apply_trans_sid;	/* whether or not to enable trans-sid for the current request */
+
+	long hash_func;
+	long hash_bits_per_character;
 	int send_cookie;
 	int define_sid;
 } php_ps_globals;
@@ -211,7 +216,7 @@ PHPAPI void php_session_start(TSRMLS_D);
 	ulong num_key;												\
 	zval **struc;
 
-#define PS_ENCODE_LOOP(code) do {										\
+#define PS_ENCODE_LOOP(code) do {									\
 		HashTable *_ht = Z_ARRVAL_P(PS(http_session_vars)); \
 																	\
 		for (zend_hash_internal_pointer_reset(_ht);			\
