@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2009 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_simplexml.h,v 1.20.2.2.2.3.2.7 2008/12/31 11:15:43 sebastian Exp $ */
+/* $Id: php_simplexml.h,v 1.20.2.1 2006/01/01 12:50:13 sniper Exp $ */
 
 #ifndef PHP_SIMPLEXML_H
 #define PHP_SIMPLEXML_H
@@ -24,6 +24,12 @@
 extern zend_module_entry simplexml_module_entry;
 #define phpext_simplexml_ptr &simplexml_module_entry
 
+#ifdef PHP_WIN32
+#define PHP_SIMPLEXML_API __declspec(dllexport)
+#else
+#define PHP_SIMPLEXML_API
+#endif
+
 #ifdef ZTS
 #include "TSRM.h"
 #endif
@@ -47,13 +53,6 @@ PHP_RINIT_FUNCTION(simplexml);
 #endif
 PHP_MINFO_FUNCTION(simplexml);
 
-typedef enum {
-	SXE_ITER_NONE     = 0,
-	SXE_ITER_ELEMENT  = 1,
-	SXE_ITER_CHILD    = 2,
-	SXE_ITER_ATTRLIST = 3
-} SXE_ITER;
-
 typedef struct {
 	zend_object zo;
 	php_libxml_node_ptr *node;
@@ -61,33 +60,26 @@ typedef struct {
 	HashTable *properties;
 	xmlXPathContextPtr xpath;
 	struct {
-		xmlChar               *name;
-		xmlChar               *nsprefix;
-		int                   isprefix;
-		SXE_ITER              type;
+		int                   itertype;
+		char                  *name;
+		char                  *nsprefix;
+		int                   type;
 		zval                  *data;
 	} iter;
-	zval *tmp;
-	zend_function *fptr_count;
 } php_sxe_object;
 
+#define SXE_ITER_NONE 0
+#define SXE_ITER_ELEMENT 1
+#define SXE_ITER_CHILD 2
+#define SXE_ITER_ATTRLIST 3
+
 #ifdef ZTS
 #define SIMPLEXML_G(v) TSRMG(simplexml_globals_id, zend_simplexml_globals *, v)
 #else
 #define SIMPLEXML_G(v) (simplexml_globals.v)
 #endif
 
-#ifdef PHP_WIN32
-#	ifdef PHP_SIMPLEXML_EXPORTS
-#		define PHP_SXE_API __declspec(dllexport)
-#	else
-#		define PHP_SXE_API __declspec(dllimport)
-#	endif
-#else
-#	define PHP_SXE_API ZEND_API
-#endif
-
-PHP_SXE_API zend_class_entry *sxe_get_element_class_entry();
+ZEND_API zend_class_entry *sxe_get_element_class_entry();
 
 #endif
 
