@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2012 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,12 +17,11 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_http.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id: php_http.c,v 1.77.2.6 2006/01/01 12:50:13 sniper Exp $ */
 
 #include "php_soap.h"
 #include "ext/standard/base64.h"
 #include "ext/standard/md5.h"
-#include "ext/standard/php_rand.h"
 
 static char *get_http_header_value(char *headers, char *type);
 static int get_http_body(php_stream *socketd, int close, char *headers,  char **response, int *out_size TSRMLS_DC);
@@ -31,13 +30,37 @@ static int get_http_headers(php_stream *socketd,char **response, int *out_size T
 #define smart_str_append_const(str, const) \
 	smart_str_appendl(str,const,sizeof(const)-1)
 
+static int stream_alive(php_stream *stream  TSRMLS_DC)
+{
+	long socket;
+	char buf;
+
+	/* maybe better to use:
+	 * php_stream_set_option(stream, PHP_STREAM_OPTION_CHECK_LIVENESS, 0, NULL)
+	 * here instead */
+
+	if (stream == NULL || stream->eof || php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT, (void**)&socket, 0) != SUCCESS) {
+		return FALSE;
+	}
+	if (socket == -1) {
+		return FALSE;
+	} else {
+		if (php_pollfd_for_ms(socket, PHP_POLLREADABLE, 0) > 0) {
+			if (0 == recv(socket, &buf, sizeof(buf), MSG_PEEK) && php_socket_errno() != EAGAIN) {
+				return FALSE;
+			}
+		}
+	}
+	return TRUE;
+}
+
 /* Proxy HTTP Authentication */
 void proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 {
 	zval **login, **password;
 
 	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_proxy_login", sizeof("_proxy_login"), (void **)&login) == SUCCESS) {
-		unsigned char* buf;
+		char* buf;
 		int len;
 		smart_str auth = {0};
 
@@ -47,9 +70,9 @@ void proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 			smart_str_appendl(&auth, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
 		}
 		smart_str_0(&auth);
-		buf = php_base64_encode((unsigned char*)auth.c, auth.len, &len);
+		buf = php_base64_encode(auth.c, auth.len, &len);
 		smart_str_append_const(soap_headers, "Proxy-Authorization: Basic ");
-		smart_str_appendl(soap_headers, (char*)buf, len);
+		smart_str_appendl(soap_headers, buf, len);
 		smart_str_append_const(soap_headers, "\r\n");
 		efree(buf);
 		smart_str_free(&auth);
@@ -63,7 +86,7 @@ void basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 
 	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_login", sizeof("_login"), (void **)&login) == SUCCESS &&
 			!zend_hash_exists(Z_OBJPROP_P(this_ptr), "_digest", sizeof("_digest"))) {
-		unsigned char* buf;
+		char* buf;
 		int len;
 		smart_str auth = {0};
 
@@ -73,20 +96,21 @@ void basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 			smart_str_appendl(&auth, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
 		}
 		smart_str_0(&auth);
-		buf = php_base64_encode((unsigned char*)auth.c, auth.len, &len);
+		buf = php_base64_encode(auth.c, auth.len, &len);
 		smart_str_append_const(soap_headers, "Authorization: Basic ");
-		smart_str_appendl(soap_headers, (char*)buf, len);
+		smart_str_appendl(soap_headers, buf, len);
 		smart_str_append_const(soap_headers, "\r\n");
 		efree(buf);
 		smart_str_free(&auth);
 	}
 }
 
-static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, php_stream_context *context, int *use_proxy TSRMLS_DC)
+static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, int *use_proxy TSRMLS_DC)
 {
 	php_stream *stream;
 	zval **proxy_host, **proxy_port, **tmp;
 	char *host;
+	php_stream_context *context = NULL;
 	char *name;
 	long namelen;
 	int port;
@@ -115,10 +139,15 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, ph
 	old_error_reporting = EG(error_reporting);
 	EG(error_reporting) &= ~(E_WARNING|E_NOTICE|E_USER_WARNING|E_USER_NOTICE);
 
+	if (SUCCESS == zend_hash_find(Z_OBJPROP_P(this_ptr),
+			"_stream_context", sizeof("_stream_context"), (void**)&tmp)) {
+		context = php_stream_context_from_zval(*tmp, 0);
+	}
+
 	namelen = spprintf(&name, 0, "%s://%s:%d", (use_ssl && !*use_proxy)? "ssl" : "tcp", host, port);
 
 	stream = php_stream_xport_create(name, namelen,
-		REPORT_ERRORS,
+		ENFORCE_SAFE_MODE | REPORT_ERRORS,
 		STREAM_XPORT_CLIENT | STREAM_XPORT_CONNECT,
 		NULL /*persistent_id*/,
 		timeout,
@@ -137,13 +166,6 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, ph
 		smart_str_appendc(&soap_headers, ':');
 		smart_str_append_unsigned(&soap_headers, phpurl->port);
 		smart_str_append_const(&soap_headers, " HTTP/1.1\r\n");
-		smart_str_append_const(&soap_headers, "Host: ");
-		smart_str_appends(&soap_headers, phpurl->host);
-		if (phpurl->port != 80) {
-			smart_str_appendc(&soap_headers, ':');
-			smart_str_append_unsigned(&soap_headers, phpurl->port);
-		}
-		smart_str_append_const(&soap_headers, "\r\n");
 		proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
 		smart_str_append_const(&soap_headers, "\r\n");
 		if (php_stream_write(stream, soap_headers.c, soap_headers.len) != soap_headers.len) {
@@ -201,7 +223,6 @@ int make_http_soap_request(zval  *this_ptr,
 {
 	char *request;
 	smart_str soap_headers = {0};
-	smart_str soap_headers_z = {0};
 	int request_size, err;
 	php_url *phpurl = NULL;
 	php_stream *stream;
@@ -214,14 +235,8 @@ int make_http_soap_request(zval  *this_ptr,
 	int http_1_1;
 	int http_status;
 	int content_type_xml = 0;
-	long redirect_max = 20;
 	char *content_encoding;
-	char *http_msg = NULL;
 	zend_bool old_allow_url_fopen;
-	php_stream_context *context = NULL;
-	zend_bool has_authorization = 0;
-	zend_bool has_proxy_authorization = 0;
-	zend_bool has_cookies = 0;
 
 	if (this_ptr == NULL || Z_TYPE_P(this_ptr) != IS_OBJECT) {
 		return FALSE;
@@ -234,10 +249,8 @@ int make_http_soap_request(zval  *this_ptr,
 		int level = Z_LVAL_PP(tmp) & 0x0f;
 		int kind  = Z_LVAL_PP(tmp) & SOAP_COMPRESSION_DEFLATE;
 
-		if (level > 9) {level = 9;}
-		
 	  if ((Z_LVAL_PP(tmp) & SOAP_COMPRESSION_ACCEPT) != 0) {
-			smart_str_append_const(&soap_headers_z,"Accept-Encoding: gzip, deflate\r\n");
+			smart_str_append_const(&soap_headers,"Accept-Encoding: gzip, deflate\r\n");
 	  }
 	  if (level > 0) {
 			zval func;
@@ -257,11 +270,11 @@ int make_http_soap_request(zval  *this_ptr,
 	    if (kind == SOAP_COMPRESSION_DEFLATE) {
 	    	n = 2;
 				ZVAL_STRING(&func, "gzcompress", 0);
-				smart_str_append_const(&soap_headers_z,"Content-Encoding: deflate\r\n");
+				smart_str_append_const(&soap_headers,"Content-Encoding: deflate\r\n");
 	    } else {
 	      n = 3;
 				ZVAL_STRING(&func, "gzencode", 0);
-				smart_str_append_const(&soap_headers_z,"Content-Encoding: gzip\r\n");
+				smart_str_append_const(&soap_headers,"Content-Encoding: gzip\r\n");
 				ZVAL_LONG(params[2], 1);
 	    }
 			if (call_user_function(CG(function_table), (zval**)NULL, &func, &retval, n, params TSRMLS_CC) == SUCCESS &&
@@ -270,7 +283,6 @@ int make_http_soap_request(zval  *this_ptr,
 				request_size = Z_STRLEN(retval);
 			} else {
 				if (request != buf) {efree(request);}
-				smart_str_free(&soap_headers_z);
 				return FALSE;
 			}
 	  }
@@ -289,25 +301,11 @@ int make_http_soap_request(zval  *this_ptr,
 		phpurl = php_url_parse(location);
 	}
 
-	if (SUCCESS == zend_hash_find(Z_OBJPROP_P(this_ptr),
-			"_stream_context", sizeof("_stream_context"), (void**)&tmp)) {
-		context = php_stream_context_from_zval(*tmp, 0);
-	}
-
-	if (context && 
-		php_stream_context_get_option(context, "http", "max_redirects", &tmp) == SUCCESS) {
-		if (Z_TYPE_PP(tmp) != IS_STRING || !is_numeric_string(Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp), &redirect_max, NULL, 1)) {
-			if (Z_TYPE_PP(tmp) == IS_LONG)
-				redirect_max = Z_LVAL_PP(tmp);
-		}
-	}
-
 try_again:
 	if (phpurl == NULL || phpurl->host == NULL) {
 	  if (phpurl != NULL) {php_url_free(phpurl);}
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "Unable to parse URL", NULL, NULL TSRMLS_CC);
-		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -318,7 +316,6 @@ try_again:
 		php_url_free(phpurl);
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "Unknown protocol. Only http and https are allowed.", NULL, NULL TSRMLS_CC);
-		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -329,7 +326,6 @@ try_again:
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "SSL support is not available in this build", NULL, NULL TSRMLS_CC);
 		PG(allow_url_fopen) = old_allow_url_fopen;
-		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -359,7 +355,7 @@ try_again:
 	}
 
 	/* Check if keep-alive connection is still opened */
-	if (stream != NULL && php_stream_eof(stream)) {
+	if (stream != NULL && !stream_alive(stream TSRMLS_CC)) {
 		php_stream_close(stream);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpurl", sizeof("httpurl"));
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
@@ -369,7 +365,7 @@ try_again:
 	}
 
 	if (!stream) {
-		stream = http_connect(this_ptr, phpurl, use_ssl, context, &use_proxy TSRMLS_CC);
+		stream = http_connect(this_ptr, phpurl, use_ssl, &use_proxy TSRMLS_CC);
 		if (stream) {
 			php_stream_auto_cleanup(stream);
 			add_property_resource(this_ptr, "httpsocket", php_stream_get_resource_id(stream));
@@ -379,7 +375,6 @@ try_again:
 			if (request != buf) {efree(request);}
 			add_soap_fault(this_ptr, "HTTP", "Could not connect to host", NULL, NULL TSRMLS_CC);
 			PG(allow_url_fopen) = old_allow_url_fopen;
-			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 	}
@@ -387,20 +382,11 @@ try_again:
 
 	if (stream) {
 		zval **cookies, **login, **password;
-	  int ret = zend_list_insert(phpurl, le_url TSRMLS_CC);
+	  int ret = zend_list_insert(phpurl, le_url);
 
 		add_property_resource(this_ptr, "httpurl", ret);
 		/*zend_list_addref(ret);*/
 
-		if (context && 
-		    php_stream_context_get_option(context, "http", "protocol_version", &tmp) == SUCCESS &&
-		    Z_TYPE_PP(tmp) == IS_DOUBLE &&
-		    Z_DVAL_PP(tmp) == 1.0) {
-			http_1_1 = 0;
-		} else {
-			http_1_1 = 1;
-		}
-
 		smart_str_append_const(&soap_headers, "POST ");
 		if (use_proxy && !use_ssl) {
 			smart_str_appends(&soap_headers, phpurl->scheme);
@@ -411,8 +397,6 @@ try_again:
 		}
 		if (phpurl->path) {
 			smart_str_appends(&soap_headers, phpurl->path);
-		} else {
-			smart_str_appendc(&soap_headers, '/');
 		}
 		if (phpurl->query) {
 			smart_str_appendc(&soap_headers, '?');
@@ -422,51 +406,20 @@ try_again:
 			smart_str_appendc(&soap_headers, '#');
 			smart_str_appends(&soap_headers, phpurl->fragment);
 		}
-		if (http_1_1) {
-			smart_str_append_const(&soap_headers, " HTTP/1.1\r\n");
-		} else {
-			smart_str_append_const(&soap_headers, " HTTP/1.0\r\n");
-		}
-		smart_str_append_const(&soap_headers, "Host: ");
+		smart_str_append_const(&soap_headers, " HTTP/1.1\r\n"
+			"Host: ");
 		smart_str_appends(&soap_headers, phpurl->host);
 		if (phpurl->port != (use_ssl?443:80)) {
 			smart_str_appendc(&soap_headers, ':');
 			smart_str_append_unsigned(&soap_headers, phpurl->port);
 		}
-		if (!http_1_1 ||
-			(zend_hash_find(Z_OBJPROP_P(this_ptr), "_keep_alive", sizeof("_keep_alive"), (void **)&tmp) == SUCCESS &&
-			 Z_LVAL_PP(tmp) == 0)) {
-			smart_str_append_const(&soap_headers, "\r\n"
-				"Connection: close\r\n");
-		} else {
-			smart_str_append_const(&soap_headers, "\r\n"
-				"Connection: Keep-Alive\r\n");
-		}
-		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_user_agent", sizeof("_user_agent"), (void **)&tmp) == SUCCESS &&
-		    Z_TYPE_PP(tmp) == IS_STRING) {
-			if (Z_STRLEN_PP(tmp) > 0) {
-				smart_str_append_const(&soap_headers, "User-Agent: ");
-				smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
-				smart_str_append_const(&soap_headers, "\r\n");
-			}
-		} else if (context && 
-		           php_stream_context_get_option(context, "http", "user_agent", &tmp) == SUCCESS &&
-		           Z_TYPE_PP(tmp) == IS_STRING) {
-			if (Z_STRLEN_PP(tmp) > 0) {
-				smart_str_append_const(&soap_headers, "User-Agent: ");
-				smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
-				smart_str_append_const(&soap_headers, "\r\n");
-			}
-		} else if (FG(user_agent)) {
-			smart_str_append_const(&soap_headers, "User-Agent: ");
-			smart_str_appends(&soap_headers, FG(user_agent));
-			smart_str_append_const(&soap_headers, "\r\n");
-		} else {
-			smart_str_append_const(&soap_headers, "User-Agent: PHP-SOAP/"PHP_VERSION"\r\n");
-		}
-
-		smart_str_append(&soap_headers, &soap_headers_z);
-
+		smart_str_append_const(&soap_headers, "\r\n"
+			"Connection: Keep-Alive\r\n"
+/*
+			"Connection: close\r\n"
+			"Accept: text/html; text/xml; text/plain\r\n"
+*/
+			"User-Agent: PHP SOAP 0.1\r\n");
 		if (soap_version == SOAP_1_2) {
 			smart_str_append_const(&soap_headers,"Content-Type: application/soap+xml; charset=utf-8");
 			if (soapaction) {
@@ -492,7 +445,6 @@ try_again:
 		    Z_TYPE_PP(login) == IS_STRING) {
 			zval **digest;
 
-			has_authorization = 1;
 			if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_digest", sizeof("_digest"), (void **)&digest) == SUCCESS) {
 				if (Z_TYPE_PP(digest) == IS_ARRAY) {
 					char          HA1[33], HA2[33], response[33], cnonce[33], nc[9];
@@ -500,31 +452,31 @@ try_again:
 					unsigned char hash[16];
 
 					PHP_MD5Init(&md5ctx);
-					snprintf(cnonce, sizeof(cnonce), "%ld", php_rand(TSRMLS_C));
-					PHP_MD5Update(&md5ctx, (unsigned char*)cnonce, strlen(cnonce));
+					sprintf(cnonce, "%d", rand());
+					PHP_MD5Update(&md5ctx, cnonce, strlen(cnonce));
 					PHP_MD5Final(hash, &md5ctx);
 					make_digest(cnonce, hash);
 
 					if (zend_hash_find(Z_ARRVAL_PP(digest), "nc", sizeof("nc"), (void **)&tmp) == SUCCESS &&
 					    Z_TYPE_PP(tmp) == IS_LONG) {
 						Z_LVAL_PP(tmp)++;
-						snprintf(nc, sizeof(nc), "%08ld", Z_LVAL_PP(tmp));
+						sprintf(nc, "%08ld", Z_LVAL_PP(tmp));
 					} else {
 						add_assoc_long(*digest, "nc", 1);
 						strcpy(nc, "00000001");
 					}
 
 					PHP_MD5Init(&md5ctx);
-					PHP_MD5Update(&md5ctx, (unsigned char*)Z_STRVAL_PP(login), Z_STRLEN_PP(login));
-					PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+					PHP_MD5Update(&md5ctx, Z_STRVAL_PP(login), Z_STRLEN_PP(login));
+					PHP_MD5Update(&md5ctx, ":", 1);
 					if (zend_hash_find(Z_ARRVAL_PP(digest), "realm", sizeof("realm"), (void **)&tmp) == SUCCESS &&
 					    Z_TYPE_PP(tmp) == IS_STRING) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 					}
-					PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+					PHP_MD5Update(&md5ctx, ":", 1);
 					if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_password", sizeof("_password"), (void **)&password) == SUCCESS &&
 					    Z_TYPE_PP(password) == IS_STRING) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)Z_STRVAL_PP(password), Z_STRLEN_PP(password));
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
 					}
 					PHP_MD5Final(hash, &md5ctx);
 					make_digest(HA1, hash);
@@ -533,28 +485,26 @@ try_again:
 					    Z_STRLEN_PP(tmp) == sizeof("md5-sess")-1 &&
 					    stricmp(Z_STRVAL_PP(tmp), "md5-sess") == 0) {
 						PHP_MD5Init(&md5ctx);
-						PHP_MD5Update(&md5ctx, (unsigned char*)HA1, 32);
-						PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+						PHP_MD5Update(&md5ctx, HA1, 32);
+						PHP_MD5Update(&md5ctx, ":", 1);
 						if (zend_hash_find(Z_ARRVAL_PP(digest), "nonce", sizeof("nonce"), (void **)&tmp) == SUCCESS &&
 						    Z_TYPE_PP(tmp) == IS_STRING) {
-							PHP_MD5Update(&md5ctx, (unsigned char*)Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+							PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 						}
-						PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
-						PHP_MD5Update(&md5ctx, (unsigned char*)cnonce, 8);
+						PHP_MD5Update(&md5ctx, ":", 1);
+						PHP_MD5Update(&md5ctx, cnonce, 8);
 						PHP_MD5Final(hash, &md5ctx);
 						make_digest(HA1, hash);
 					}
 
 					PHP_MD5Init(&md5ctx);
-					PHP_MD5Update(&md5ctx, (unsigned char*)"POST:", sizeof("POST:")-1);
+					PHP_MD5Update(&md5ctx, "POST:", sizeof("POST:")-1);
 					if (phpurl->path) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)phpurl->path, strlen(phpurl->path));
-					} else {
-						PHP_MD5Update(&md5ctx, (unsigned char*)"/", 1);
+						PHP_MD5Update(&md5ctx, phpurl->path, strlen(phpurl->path));
 					}
 					if (phpurl->query) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)"?", 1);
-						PHP_MD5Update(&md5ctx, (unsigned char*)phpurl->query, strlen(phpurl->query));
+						PHP_MD5Update(&md5ctx, "?", 1);
+						PHP_MD5Update(&md5ctx, phpurl->query, strlen(phpurl->query));
 					}
 
 					/* TODO: Support for qop="auth-int" */
@@ -571,24 +521,24 @@ try_again:
 					make_digest(HA2, hash);
 
 					PHP_MD5Init(&md5ctx);
-					PHP_MD5Update(&md5ctx, (unsigned char*)HA1, 32);
-					PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+					PHP_MD5Update(&md5ctx, HA1, 32);
+					PHP_MD5Update(&md5ctx, ":", 1);
 					if (zend_hash_find(Z_ARRVAL_PP(digest), "nonce", sizeof("nonce"), (void **)&tmp) == SUCCESS &&
 					    Z_TYPE_PP(tmp) == IS_STRING) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 					}
-					PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+					PHP_MD5Update(&md5ctx, ":", 1);
 					if (zend_hash_find(Z_ARRVAL_PP(digest), "qop", sizeof("qop"), (void **)&tmp) == SUCCESS &&
 					    Z_TYPE_PP(tmp) == IS_STRING) {
-						PHP_MD5Update(&md5ctx, (unsigned char*)nc, 8);
-						PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
-						PHP_MD5Update(&md5ctx, (unsigned char*)cnonce, 8);
-						PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+						PHP_MD5Update(&md5ctx, nc, 8);
+						PHP_MD5Update(&md5ctx, ":", 1);
+						PHP_MD5Update(&md5ctx, cnonce, 8);
+						PHP_MD5Update(&md5ctx, ":", 1);
 						/* TODO: Support for qop="auth-int" */
-						PHP_MD5Update(&md5ctx, (unsigned char*)"auth", sizeof("auth")-1);
-						PHP_MD5Update(&md5ctx, (unsigned char*)":", 1);
+						PHP_MD5Update(&md5ctx, "auth", sizeof("auth")-1);
+						PHP_MD5Update(&md5ctx, ":", 1);
 					}
-					PHP_MD5Update(&md5ctx, (unsigned char*)HA2, 32);
+					PHP_MD5Update(&md5ctx, HA2, 32);
 					PHP_MD5Final(hash, &md5ctx);
 					make_digest(response, hash);
 	
@@ -607,9 +557,7 @@ try_again:
 					smart_str_append_const(&soap_headers, "\", uri=\"");
 					if (phpurl->path) {
 						smart_str_appends(&soap_headers, phpurl->path);
-					} else {
-						smart_str_appendc(&soap_headers, '/');
-					} 
+					}
 					if (phpurl->query) {
 						smart_str_appendc(&soap_headers, '?');
 						smart_str_appends(&soap_headers, phpurl->query);
@@ -634,15 +582,10 @@ try_again:
 						smart_str_append_const(&soap_headers, "\", opaque=\"");
 						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 					}
-					if (zend_hash_find(Z_ARRVAL_PP(digest), "algorithm", sizeof("algorithm"), (void **)&tmp) == SUCCESS &&
-						Z_TYPE_PP(tmp) == IS_STRING) {
-						smart_str_append_const(&soap_headers, "\", algorithm=\"");
-						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
-					}
 					smart_str_append_const(&soap_headers, "\"\r\n");
 				}
 			} else {
-				unsigned char* buf;
+				char* buf;
 				int len;
 
 				smart_str auth = {0};
@@ -653,9 +596,9 @@ try_again:
 					smart_str_appendl(&auth, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
 				}
 				smart_str_0(&auth);
-				buf = php_base64_encode((unsigned char*)auth.c, auth.len, &len);
+				buf = php_base64_encode(auth.c, auth.len, &len);
 				smart_str_append_const(&soap_headers, "Authorization: Basic ");
-				smart_str_appendl(&soap_headers, (char*)buf, len);
+				smart_str_appendl(&soap_headers, buf, len);
 				smart_str_append_const(&soap_headers, "\r\n");
 				efree(buf);
 				smart_str_free(&auth);
@@ -664,7 +607,6 @@ try_again:
 
 		/* Proxy HTTP Authentication */
 		if (use_proxy && !use_ssl) {
-			has_proxy_authorization = 1;
 			proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
 		}
 
@@ -674,7 +616,6 @@ try_again:
 			char *key;
 			int i, n;
 
-			has_cookies = 1;
 			n = zend_hash_num_elements(Z_ARRVAL_PP(cookies));
 			if (n > 0) {
 				zend_hash_internal_pointer_reset(Z_ARRVAL_PP(cookies));
@@ -706,67 +647,6 @@ try_again:
 				smart_str_append_const(&soap_headers, "\r\n");
 			}
 		}
-
-		if (context &&
-			php_stream_context_get_option(context, "http", "header", &tmp) == SUCCESS &&
-			Z_TYPE_PP(tmp) == IS_STRING && Z_STRLEN_PP(tmp)) {
-			char *s = Z_STRVAL_PP(tmp);
-			char *p;
-			int name_len;
-
-			while (*s) {
-				/* skip leading newlines and spaces */
-				while (*s == ' ' || *s == '\t' || *s == '\r' || *s == '\n') {
-					s++;
-				}
-				/* extract header name */
-				p = s;
-				name_len = -1;
-				while (*p) {
-					if (*p == ':') {
-						if (name_len < 0) name_len = p - s;
-						break;
-					} else if (*p == ' ' || *p == '\t') {
-						if (name_len < 0) name_len = p - s;
-					} else if (*p == '\r' || *p == '\n') {
-						break;
-					}
-					p++;
-				}
-				if (*p == ':') {
-					/* extract header value */
-					while (*p && *p != '\r' && *p != '\n') {
-						p++;
-					}
-					/* skip some predefined headers */
-					if ((name_len != sizeof("host")-1 ||
-					     strncasecmp(s, "host", sizeof("host")-1) != 0) &&
-					    (name_len != sizeof("connection")-1 ||
-					     strncasecmp(s, "connection", sizeof("connection")-1) != 0) &&
-					    (name_len != sizeof("user-agent")-1 ||
-					     strncasecmp(s, "user-agent", sizeof("user-agent")-1) != 0) &&
-					    (name_len != sizeof("content-length")-1 ||
-					     strncasecmp(s, "content-length", sizeof("content-length")-1) != 0) &&
-					    (name_len != sizeof("content-type")-1 ||
-					     strncasecmp(s, "content-type", sizeof("content-type")-1) != 0) &&
-					    (!has_cookies ||
-					     name_len != sizeof("cookie")-1 ||
-					     strncasecmp(s, "cookie", sizeof("cookie")-1) != 0) &&
-					    (!has_authorization ||
-					     name_len != sizeof("authorization")-1 ||
-					     strncasecmp(s, "authorization", sizeof("authorization")-1) != 0) &&
-					    (!has_proxy_authorization ||
-					     name_len != sizeof("proxy-authorization")-1 ||
-					     strncasecmp(s, "proxy-authorization", sizeof("proxy-authorization")-1) != 0)) {
-					    /* add header */
-						smart_str_appendl(&soap_headers, s, p-s);
-						smart_str_append_const(&soap_headers, "\r\n");
-					}
-				}
-				s = (*p) ? (p + 1) : p;
-			}
-		}
-
 		smart_str_append_const(&soap_headers, "\r\n");
 		smart_str_0(&soap_headers);
 		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "trace", sizeof("trace"), (void **) &trace) == SUCCESS &&
@@ -779,29 +659,21 @@ try_again:
 		err = php_stream_write(stream, soap_headers.c, soap_headers.len);
 		if (err != soap_headers.len) {
 			if (request != buf) {efree(request);}
+			smart_str_free(&soap_headers);
 			php_stream_close(stream);
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpurl", sizeof("httpurl"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 			add_soap_fault(this_ptr, "HTTP", "Failed Sending HTTP SOAP request", NULL, NULL TSRMLS_CC);
-			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 		smart_str_free(&soap_headers);
+
 	} else {
 		add_soap_fault(this_ptr, "HTTP", "Failed to create stream??", NULL, NULL TSRMLS_CC);
-		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
-	if (!buffer) {
-		php_stream_close(stream);
-		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
-		zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
-		smart_str_free(&soap_headers_z);
-		return TRUE;
-	}
-
 	do {
 		if (!get_http_headers(stream, &http_headers, &http_header_size TSRMLS_CC)) {
 			if (http_headers) {efree(http_headers);}
@@ -810,7 +682,6 @@ try_again:
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 			add_soap_fault(this_ptr, "HTTP", "Error Fetching http headers", NULL, NULL TSRMLS_CC);
-			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 
@@ -835,14 +706,6 @@ try_again:
 				tmp++;
 				http_status = atoi(tmp);
 			}
-			tmp = strstr(tmp," ");
-			if (tmp != NULL) {
-				tmp++;
-				if (http_msg) {
-					efree(http_msg);
-				}
-				http_msg = estrdup(tmp);
-			}
 			efree(http_version);
 
 			/* Try and get headers again */
@@ -934,65 +797,43 @@ try_again:
 		efree(cookie);
 	}
 
-	/* See if the server requested a close */
-	if (http_1_1) {
-		http_close = FALSE;
-		if (use_proxy && !use_ssl) {
-			connection = get_http_header_value(http_headers,"Proxy-Connection: ");
-			if (connection) {
-				if (strncasecmp(connection, "close", sizeof("close")-1) == 0) {
-					http_close = TRUE;
-				}
-				efree(connection);
-			}
-		}
-		if (http_close == FALSE) {
-			connection = get_http_header_value(http_headers,"Connection: ");
-			if (connection) {
-				if (strncasecmp(connection, "close", sizeof("close")-1) == 0) {
-					http_close = TRUE;
-				}
-				efree(connection);
-			}
-		}
-	} else {
-		http_close = TRUE;
-		if (use_proxy && !use_ssl) {
-			connection = get_http_header_value(http_headers,"Proxy-Connection: ");
-			if (connection) {
-				if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
-					http_close = FALSE;
-				}
-				efree(connection);
-			}
-		}
-		if (http_close == TRUE) {
-			connection = get_http_header_value(http_headers,"Connection: ");
-			if (connection) {
-				if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
-					http_close = FALSE;
-				}
-				efree(connection);
-			}
-		}
-	}	
-
-	if (!get_http_body(stream, http_close, http_headers, &http_body, &http_body_size TSRMLS_CC)) {
+	if (!get_http_body(stream, !http_1_1, http_headers, &http_body, &http_body_size TSRMLS_CC)) {
 		if (request != buf) {efree(request);}
 		php_stream_close(stream);
 		efree(http_headers);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 		add_soap_fault(this_ptr, "HTTP", "Error Fetching http body, No Content-Length, connection closed or chunked data", NULL, NULL TSRMLS_CC);
-		if (http_msg) {
-			efree(http_msg);
-		}
-		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
 	if (request != buf) {efree(request);}
 
+	/* See if the server requested a close */
+	http_close = TRUE;
+	connection = get_http_header_value(http_headers,"Proxy-Connection: ");
+	if (connection) {
+		if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
+			http_close = FALSE;
+		}
+		efree(connection);
+/*
+	} else if (http_1_1) {
+		http_close = FALSE;
+*/
+	}
+	connection = get_http_header_value(http_headers,"Connection: ");
+	if (connection) {
+		if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
+			http_close = FALSE;
+		}
+		efree(connection);
+/*
+	} else if (http_1_1) {
+		http_close = FALSE;
+*/
+	}
+
 	if (http_close) {
 		php_stream_close(stream);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
@@ -1012,38 +853,23 @@ try_again:
 				efree(http_body);
 				efree(loc);
 				if (new_url->scheme == NULL && new_url->path != NULL) {
-					new_url->scheme = phpurl->scheme ? estrdup(phpurl->scheme) : NULL;
-					new_url->host = phpurl->host ? estrdup(phpurl->host) : NULL;
+					new_url->scheme = estrdup(phpurl->scheme);
+					new_url->host = estrdup(phpurl->host);
 					new_url->port = phpurl->port;
 					if (new_url->path && new_url->path[0] != '/') {
-						if (phpurl->path) {
-							char *t = phpurl->path;
-							char *p = strrchr(t, '/');
-							if (p) {
-								char *s = emalloc((p - t) + strlen(new_url->path) + 2);
-								strncpy(s, t, (p - t) + 1);
-								s[(p - t) + 1] = 0;
-								strcat(s, new_url->path);
-								efree(new_url->path);
-								new_url->path = s;
-							} 
-						} else {
-							char *s = emalloc(strlen(new_url->path) + 2);
-							s[0] = '/'; s[1] = 0;
-							strcat(s, new_url->path);
-							efree(new_url->path);
-							new_url->path = s;
-						}
+						char *t = phpurl->path?phpurl->path:"/";
+						char *p = strrchr(t, '/');
+						char *s = emalloc((p - t) + strlen(new_url->path) + 2);
+
+						strncpy(s, t, (p - t) + 1);
+						s[(p - t) + 1] = 0;
+						strcat(s, new_url->path);
+						efree(new_url->path);
+						new_url->path = s;
 					}
 				}
 				phpurl = new_url;
 
-				if (--redirect_max < 1) {
-					add_soap_fault(this_ptr, "HTTP", "Redirection limit reached, aborting", NULL, NULL TSRMLS_CC);
-					smart_str_free(&soap_headers_z);
-					return FALSE;
-				}
-
 				goto try_again;
 			}
 		}
@@ -1102,7 +928,7 @@ try_again:
 			if (digest != NULL) {
 				php_url *new_url  = emalloc(sizeof(php_url));
 
-				Z_DELREF_P(digest);
+				digest->refcount--;
 				add_property_zval_ex(this_ptr, "_digest", sizeof("_digest"), digest TSRMLS_CC);
 
 				*new_url = *phpurl;
@@ -1124,7 +950,6 @@ try_again:
 		}
 		if (auth) efree(auth);
 	}
-	smart_str_free(&soap_headers_z);
 
 	/* Check and see if the server even sent a xml document */
 	content_type = get_http_header_value(http_headers,"Content-Type: ");
@@ -1181,9 +1006,6 @@ try_again:
 			efree(content_encoding);
 			efree(http_headers);
 			efree(http_body);
-			if (http_msg) {
-				efree(http_msg);
-			}
 			add_soap_fault(this_ptr, "HTTP", "Unknown Content-Encoding", NULL, NULL TSRMLS_CC);
 			return FALSE;
 		}
@@ -1197,9 +1019,6 @@ try_again:
 			efree(http_headers);
 			efree(http_body);
 			add_soap_fault(this_ptr, "HTTP", "Can't uncompress compressed response", NULL, NULL TSRMLS_CC);
-			if (http_msg) {
-				efree(http_msg);
-			}
 			return FALSE;
 		}
 		efree(content_encoding);
@@ -1230,16 +1049,27 @@ try_again:
 
 		if (error) {
 			efree(*buffer);
-			add_soap_fault(this_ptr, "HTTP", http_msg, NULL, NULL TSRMLS_CC);
-			efree(http_msg);
+			if (http_status == 400) {
+				add_soap_fault(this_ptr, "HTTP", "Bad Request", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 401) {
+				add_soap_fault(this_ptr, "HTTP", "Unauthorized Request", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 405) {
+				add_soap_fault(this_ptr, "HTTP", "Method not allowed", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 415) {
+				add_soap_fault(this_ptr, "HTTP", "Unsupported Media Type", NULL, NULL TSRMLS_CC);
+			} else if (http_status >= 400 && http_status < 500) {
+				add_soap_fault(this_ptr, "HTTP", "Client Error", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 500) {
+				add_soap_fault(this_ptr, "HTTP", "Internal Server Error", NULL, NULL TSRMLS_CC);
+			} else if (http_status >= 500 && http_status < 600) {
+				add_soap_fault(this_ptr, "HTTP", "Server Error", NULL, NULL TSRMLS_CC);
+			} else {
+				add_soap_fault(this_ptr, "HTTP", "Unsupported HTTP status code", NULL, NULL TSRMLS_CC);
+			}
 			return FALSE;
 		}
 	}
 
-	if (http_msg) {
-		efree(http_msg);
-	}
-
 	return TRUE;
 }
 
@@ -1261,19 +1091,17 @@ static char *get_http_header_value(char *headers, char *type)
 
 			/* match */
 			tmp = pos + typelen;
-			eol = strchr(tmp, '\n');
+			eol = strstr(tmp, "\r\n");
 			if (eol == NULL) {
 				eol = headers + headerslen;
-			} else if (eol > tmp && *(eol-1) == '\r') {
-				eol--;
 			}
 			return estrndup(tmp, eol - tmp);
 		}
 
 		/* find next line */
-		pos = strchr(pos, '\n');
+		pos = strstr(pos, "\r\n");
 		if (pos) {
-			pos++;
+			pos += 2;
 		}
 
 	} while (pos);
@@ -1313,7 +1141,7 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 	}
 
 	if (header_chunked) {
-		char ch, done, chunk_size[10], headerbuf[8192];
+		char done, chunk_size[10];
 
 		done = FALSE;
 
@@ -1341,20 +1169,11 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 						len_size += len_read;
 	 					http_buf_size += len_read;
  					}
-
-					/* Eat up '\r' '\n' */
-					ch = php_stream_getc(stream);
-					if (ch == '\r') {
-						ch = php_stream_getc(stream);
-					}
-					if (ch != '\n') {
-						/* Somthing wrong in chunked encoding */
-						if (http_buf) {
-							efree(http_buf);
-						}
-						return FALSE;
-					}
  				}
+
+				/* Eat up '\r' '\n' */
+				php_stream_getc(stream);
+				php_stream_getc(stream);
 			} else {
 				/* Somthing wrong in chunked encoding */
 				if (http_buf) {
@@ -1367,28 +1186,15 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 			}
 		}
 
-		/* Ignore trailer headers */
-		while (1) {
-			if (!php_stream_gets(stream, headerbuf, sizeof(headerbuf))) {
-				break;
-			}
-
-			if ((headerbuf[0] == '\r' && headerbuf[1] == '\n') ||
-			    (headerbuf[0] == '\n')) {
-				/* empty line marks end of headers */
-				break;
-			}
-		}
-
 		if (http_buf == NULL) {
 			http_buf = emalloc(1);
 		}
 
 	} else if (header_length) {
-		if (header_length < 0 || header_length >= INT_MAX) {
+		if (header_length < 0) {
 			return FALSE;
 		}
-		http_buf = safe_emalloc(1, header_length, 1);
+		http_buf = emalloc(header_length + 1);
 		while (http_buf_size < header_length) {
 			int len_read = php_stream_read(stream, http_buf + http_buf_size, header_length - http_buf_size);
 			if (len_read <= 0) {
@@ -1426,8 +1232,7 @@ static int get_http_headers(php_stream *stream, char **response, int *out_size T
 			break;
 		}
 
-		if ((headerbuf[0] == '\r' && headerbuf[1] == '\n') ||
-		    (headerbuf[0] == '\n')) {
+		if (strcmp(headerbuf, "\r\n") == 0) {
 			/* empty line marks end of headers */
 			done = TRUE;
 			break;
@@ -1441,11 +1246,3 @@ static int get_http_headers(php_stream *stream, char **response, int *out_size T
 	(*out_size) = tmp_response.len;
 	return done;
 }
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
