@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_soap.h,v 1.38.2.6 2006/04/19 10:48:54 dmitry Exp $ */
+/* $Id: php_soap.h,v 1.38.2.1 2006/01/01 12:50:13 sniper Exp $ */
 
 #ifndef PHP_SOAP_H
 #define PHP_SOAP_H
@@ -97,8 +97,6 @@ struct _soapMapping {
 	} map_class;
 };
 
-struct _soapHeader;
-
 struct _soapService {
 	sdlPtr sdl;
 
@@ -121,8 +119,6 @@ struct _soapService {
 	char      *uri;
 	xmlCharEncodingHandlerPtr encoding;
 	HashTable *class_map;
-	int        features;
-	struct _soapHeader **soap_headers_ptr;
 };
 
 #define SOAP_CLASS 1
@@ -155,14 +151,6 @@ struct _soapService {
 #define SOAP_AUTHENTICATION_BASIC   0
 #define SOAP_AUTHENTICATION_DIGEST  1
 
-#define SOAP_SINGLE_ELEMENT_ARRAYS  (1<<0)
-#define SOAP_WAIT_ONE_WAY_CALLS     (2<<0)
-
-#define WSDL_CACHE_NONE     0x0
-#define WSDL_CACHE_DISK     0x1
-#define WSDL_CACHE_MEMORY   0x2
-#define WSDL_CACHE_BOTH     0x3
-
 ZEND_BEGIN_MODULE_GLOBALS(soap)
 	HashTable  defEncNs;     /* mapping of default namespaces to prefixes */
 	HashTable  defEnc;
@@ -174,15 +162,11 @@ ZEND_BEGIN_MODULE_GLOBALS(soap)
 	zend_bool  use_soap_error_handler;
 	char*      error_code;
 	zval*      error_object;
-	long       cache;
+	zend_bool  cache_enabled;
 	char*      cache_dir;
 	long       cache_ttl;
-	long       cache_limit;
-	HashTable *mem_cache;
 	xmlCharEncodingHandlerPtr encoding;
 	HashTable *class_map;
-	int        features;
-	HashTable  wsdl_cache;
 ZEND_END_MODULE_GLOBALS(soap)
 
 #ifdef PHP_WIN32
