@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: spl_engine.h,v 1.19.2.3.2.1.2.4 2008/12/31 11:15:43 sebastian Exp $ */
+/* $Id: spl_engine.h,v 1.19.2.1 2006/01/01 12:50:13 sniper Exp $ */
 
 #ifndef SPL_ENGINE_H
 #define SPL_ENGINE_H
@@ -25,9 +25,18 @@
 #include "php_spl.h"
 #include "zend_interfaces.h"
 
-PHPAPI void spl_instantiate(zend_class_entry *pce, zval **object, int alloc TSRMLS_DC);
+/* {{{ zend_class_entry */
+static inline zend_class_entry *spl_get_class_entry(zval *obj TSRMLS_DC)
+{
+	if (obj && Z_TYPE_P(obj) == IS_OBJECT && Z_OBJ_HT_P(obj)->get_class_entry) {
+		return Z_OBJ_HT_P(obj)->get_class_entry(obj TSRMLS_CC);
+	} else {
+		return NULL;
+	}
+}
+/* }}} */
 
-PHPAPI long spl_offset_convert_to_long(zval *offset TSRMLS_DC);
+void spl_instantiate(zend_class_entry *pce, zval **object, int alloc TSRMLS_DC);
 
 /* {{{ spl_instantiate_arg_ex1 */
 static inline int spl_instantiate_arg_ex1(zend_class_entry *pce, zval **retval, int alloc, zval *arg1 TSRMLS_DC)
@@ -49,6 +58,8 @@ static inline int spl_instantiate_arg_ex2(zend_class_entry *pce, zval **retval,
 }
 /* }}} */
 
+int spl_is_instance_of(zval **obj, zend_class_entry *ce TSRMLS_DC);
+
 #endif /* SPL_ENGINE_H */
 
 /*
