@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: spl_functions.c,v 1.28.2.3.2.3.2.5 2008/12/31 11:15:43 sebastian Exp $ */
+/* $Id: spl_functions.c,v 1.28.2.2 2006/01/01 12:50:13 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 	#include "config.h"
@@ -27,22 +27,35 @@
 #include "ext/standard/info.h"
 #include "php_spl.h"
 
+/* {{{ spl_destroy_class */
+void spl_destroy_class(zend_class_entry ** ppce)
+{                           
+	SPL_DEBUG(fprintf(stderr, "Destroy(%s): %s\n", (*ppce)->type == ZEND_USER_CLASS ? "user" : "other", (*ppce)->name);)
+	destroy_zend_class(ppce);
+}
+/* }}} */
+
 /* {{{ spl_register_interface */
-void spl_register_interface(zend_class_entry ** ppce, char * class_name, const zend_function_entry * functions TSRMLS_DC)
+void spl_register_interface(zend_class_entry ** ppce, char * class_name, zend_function_entry *functions TSRMLS_DC)
 {
 	zend_class_entry ce;
 	
-	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), functions);
-	*ppce = zend_register_internal_interface(&ce TSRMLS_CC);
+	INIT_CLASS_ENTRY(ce, class_name, functions);
+	ce.name_length = strlen(class_name);
+	*ppce = zend_register_internal_class(&ce TSRMLS_CC);
+
+	/* entries changed by initialize */
+	(*ppce)->ce_flags = ZEND_ACC_INTERFACE;
 }
 /* }}} */
 
 /* {{{ spl_register_std_class */
-PHPAPI void spl_register_std_class(zend_class_entry ** ppce, char * class_name, void * obj_ctor, const zend_function_entry * function_list TSRMLS_DC)
+void spl_register_std_class(zend_class_entry ** ppce, char * class_name, void * obj_ctor, zend_function_entry * function_list TSRMLS_DC)
 {
 	zend_class_entry ce;
 	
-	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), function_list);
+	INIT_CLASS_ENTRY(ce, class_name, function_list);
+	ce.name_length = strlen(class_name);
 	*ppce = zend_register_internal_class(&ce TSRMLS_CC);
 
 	/* entries changed by initialize */
@@ -53,11 +66,12 @@ PHPAPI void spl_register_std_class(zend_class_entry ** ppce, char * class_name,
 /* }}} */
 
 /* {{{ spl_register_sub_class */
-PHPAPI void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry * parent_ce, char * class_name, void *obj_ctor, const zend_function_entry * function_list TSRMLS_DC)
+void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry * parent_ce, char * class_name, void *obj_ctor, zend_function_entry * function_list TSRMLS_DC)
 {
 	zend_class_entry ce;
 	
-	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), function_list);
+	INIT_CLASS_ENTRY(ce, class_name, function_list);
+	ce.name_length = strlen(class_name);
 	*ppce = zend_register_internal_class_ex(&ce, parent_ce, NULL TSRMLS_CC);
 
 	/* entries changed by initialize */
@@ -69,10 +83,29 @@ PHPAPI void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry *
 }
 /* }}} */
 
-/* {{{ spl_register_property */
-void spl_register_property( zend_class_entry * class_entry, char *prop_name, int prop_name_len, int prop_flags TSRMLS_DC)
+/* {{{ spl_register_parent_ce */
+void spl_register_parent_ce(zend_class_entry * class_entry, zend_class_entry * parent_class TSRMLS_DC)
 {
-	zend_declare_property_null(class_entry, prop_name, prop_name_len, prop_flags TSRMLS_CC);
+	class_entry->parent = parent_class;
+}
+/* }}} */
+
+/* {{{ spl_register_functions */
+void spl_register_functions(zend_class_entry * class_entry, zend_function_entry * function_list TSRMLS_DC)
+{
+	zend_register_functions(class_entry, function_list, &class_entry->function_table, MODULE_PERSISTENT TSRMLS_CC);
+}
+/* }}} */
+
+/* {{{ spl_register_property */
+void spl_register_property( zend_class_entry * class_entry, char *prop_name, zval *prop_val, int prop_flags TSRMLS_DC)
+{
+	if (!prop_val) {
+		INIT_PZVAL(prop_val);
+		prop_val->type = IS_NULL;
+	}
+
+	zend_declare_property(class_entry, prop_name, strlen(prop_name), prop_val, prop_flags TSRMLS_CC);
 }
 /* }}} */
 
@@ -80,12 +113,12 @@ void spl_register_property( zend_class_entry * class_entry, char *prop_name, int
 void spl_add_class_name(zval *list, zend_class_entry * pce, int allow, int ce_flags TSRMLS_DC)
 {
 	if (!allow || (allow > 0 && pce->ce_flags & ce_flags) || (allow < 0 && !(pce->ce_flags & ce_flags))) {
-		size_t len = pce->name_length;
+		size_t len = strlen(pce->name);
 		zval *tmp;
 
 		if (zend_hash_find(Z_ARRVAL_P(list), pce->name, len+1, (void*)&tmp) == FAILURE) {
 			MAKE_STD_ZVAL(tmp);
-			ZVAL_STRINGL(tmp, pce->name, pce->name_length, 1);
+			ZVAL_STRING(tmp, pce->name, 1);
 			zend_hash_add(Z_ARRVAL_P(list), pce->name, len+1, &tmp, sizeof(zval *), NULL);
 		}
 	}
@@ -104,8 +137,10 @@ void spl_add_interfaces(zval *list, zend_class_entry * pce, int allow, int ce_fl
 /* }}} */
 
 /* {{{ spl_add_classes */
-int spl_add_classes(zend_class_entry *pce, zval *list, int sub, int allow, int ce_flags TSRMLS_DC)
+int spl_add_classes(zend_class_entry ** ppce, zval *list, int sub, int allow, int ce_flags TSRMLS_DC)
 {
+	zend_class_entry *pce = *ppce;
+
 	if (!pce) {
 		return 0;
 	}
@@ -114,23 +149,13 @@ int spl_add_classes(zend_class_entry *pce, zval *list, int sub, int allow, int c
 		spl_add_interfaces(list, pce, allow, ce_flags TSRMLS_CC);
 		while (pce->parent) {
 			pce = pce->parent;
-			spl_add_classes(pce, list, sub, allow, ce_flags TSRMLS_CC);
+			spl_add_classes(&pce, list, sub, allow, ce_flags TSRMLS_CC);
 		}
 	}
 	return 0;
 }
 /* }}} */
 
-char * spl_gen_private_prop_name(zend_class_entry *ce, char *prop_name, int prop_len, int *name_len TSRMLS_DC) /* {{{ */
-{
-	char *rv;
-
-	zend_mangle_property_name(&rv, name_len, ce->name, ce->name_length, prop_name, prop_len, 0);
-
-	return rv;
-}
-/* }}} */
-
 /*
  * Local variables:
  * tab-width: 4
