@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    |          Marcus Boerger <helly@php.net>                              |
    +----------------------------------------------------------------------+
 
-   $Id: sqlite.c,v 1.146.2.7 2005/06/07 15:38:37 dmitry Exp $ 
+   $Id: sqlite.c,v 1.166.2.11 2006/01/01 12:50:14 sniper Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -29,7 +29,9 @@
 #include "php.h"
 #include "php_ini.h"
 #include "ext/standard/info.h"
+#if HAVE_PHP_SESSION && !defined(COMPILE_DL_SESSION)
 #include "ext/session/php_session.h"
+#endif
 #include "php_sqlite.h"
 
 #if HAVE_TIME_H
@@ -42,6 +44,18 @@
 #include <sqlite.h>
 
 #include "zend_exceptions.h"
+#include "zend_interfaces.h"
+
+#if defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+extern PHPAPI zend_class_entry *spl_ce_RuntimeException;
+extern PHPAPI zend_class_entry *spl_ce_Countable;
+#endif
+
+#if PHP_SQLITE2_HAVE_PDO
+# include "pdo/php_pdo.h"
+# include "pdo/php_pdo_driver.h"
+extern pdo_driver_t pdo_sqlite2_driver;
+#endif
 
 #ifndef safe_emalloc
 # define safe_emalloc(a,b,c) emalloc((a)*(b)+(c))
@@ -60,6 +74,8 @@ extern int sqlite_decode_binary(const unsigned char *in, unsigned char *out);
 #define php_sqlite_encode_binary(in, n, out) sqlite_encode_binary((const unsigned char *)in, n, (unsigned char *)out)
 #define php_sqlite_decode_binary(in, out)    sqlite_decode_binary((const unsigned char *)in, (unsigned char *)out)
 
+static int sqlite_count_elements(zval *object, long *count TSRMLS_DC);
+
 static int le_sqlite_db, le_sqlite_result, le_sqlite_pdb;
 
 static inline void php_sqlite_strtoupper(char *s)
@@ -85,7 +101,6 @@ STD_PHP_INI_ENTRY_EX("sqlite.assoc_case", "0", PHP_INI_ALL, OnUpdateLong, assoc_
 PHP_INI_END()
 /* }}} */
 
-
 #define DB_FROM_ZVAL(db, zv)	ZEND_FETCH_RESOURCE2(db, struct php_sqlite_db *, zv, -1, "sqlite database", le_sqlite_db, le_sqlite_pdb)
 
 #define DB_FROM_OBJECT(db, object) \
@@ -108,10 +123,6 @@ PHP_INI_END()
 		} \
 	}
 
-#define SQLITE_THROW(message) \
-	PG(suppress_errors) = 0; \
-	EG(exception) = zend_throw_exception(sqlite_ce_exception, message, 0 TSRMLS_CC);
-
 #define PHP_SQLITE_EMPTY_QUERY \
 	if (!sql_len) { \
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot execute empty query."); \
@@ -152,12 +163,12 @@ static int php_sqlite_fetch(struct php_sqlite_result *rres TSRMLS_DC);
 
 enum { PHPSQLITE_ASSOC = 1, PHPSQLITE_NUM = 2, PHPSQLITE_BOTH = PHPSQLITE_ASSOC|PHPSQLITE_NUM };
 
-function_entry sqlite_functions[] = {
+zend_function_entry sqlite_functions[] = {
 	PHP_FE(sqlite_open, third_arg_force_ref)
 	PHP_FE(sqlite_popen, third_arg_force_ref)
 	PHP_FE(sqlite_close, NULL)
-	PHP_FE(sqlite_query, NULL)
-	PHP_FE(sqlite_exec, NULL)
+	PHP_FE(sqlite_query, fourth_arg_force_ref)
+	PHP_FE(sqlite_exec, third_arg_force_ref)
 	PHP_FE(sqlite_array_query, NULL)
 	PHP_FE(sqlite_single_query, NULL)
 	PHP_FE(sqlite_fetch_array, NULL)
@@ -185,7 +196,7 @@ function_entry sqlite_functions[] = {
 	PHP_FE(sqlite_busy_timeout, NULL)
 	PHP_FE(sqlite_last_error, NULL)
 	PHP_FE(sqlite_error_string, NULL)
-	PHP_FE(sqlite_unbuffered_query, NULL)
+	PHP_FE(sqlite_unbuffered_query, fourth_arg_force_ref)
 	PHP_FE(sqlite_create_aggregate, NULL)
 	PHP_FE(sqlite_create_function, NULL)
 	PHP_FE(sqlite_factory, third_arg_force_ref)
@@ -195,14 +206,14 @@ function_entry sqlite_functions[] = {
 	{NULL, NULL, NULL}
 };
 
-function_entry sqlite_funcs_db[] = {
-	PHP_ME_MAPPING(__construct, sqlite_open, NULL)
+zend_function_entry sqlite_funcs_db[] = {
+	PHP_ME_MAPPING(__construct, sqlite_open, third_arg_force_ref)
 /*	PHP_ME_MAPPING(close, sqlite_close, NULL)*/
-	PHP_ME_MAPPING(query, sqlite_query, NULL)
-	PHP_ME_MAPPING(queryExec, sqlite_exec, NULL)
+	PHP_ME_MAPPING(query, sqlite_query, third_arg_force_ref)
+	PHP_ME_MAPPING(queryExec, sqlite_exec, second_arg_force_ref)
 	PHP_ME_MAPPING(arrayQuery, sqlite_array_query, NULL)
 	PHP_ME_MAPPING(singleQuery, sqlite_single_query, NULL)
-	PHP_ME_MAPPING(unbufferedQuery, sqlite_unbuffered_query, NULL)
+	PHP_ME_MAPPING(unbufferedQuery, sqlite_unbuffered_query, third_arg_force_ref)
 	PHP_ME_MAPPING(lastInsertRowid, sqlite_last_insert_rowid, NULL)
 	PHP_ME_MAPPING(changes, sqlite_changes, NULL)
 	PHP_ME_MAPPING(createAggregate, sqlite_create_aggregate, NULL)
@@ -215,7 +226,7 @@ function_entry sqlite_funcs_db[] = {
 	{NULL, NULL, NULL}
 };
 
-function_entry sqlite_funcs_query[] = {
+zend_function_entry sqlite_funcs_query[] = {
 	PHP_ME_MAPPING(fetch, sqlite_fetch_array, NULL)
 	PHP_ME_MAPPING(fetchObject, sqlite_fetch_object, NULL)
 	PHP_ME_MAPPING(fetchSingle, sqlite_fetch_single, NULL)
@@ -223,12 +234,14 @@ function_entry sqlite_funcs_query[] = {
 	PHP_ME_MAPPING(column, sqlite_column, NULL)
 	PHP_ME_MAPPING(numFields, sqlite_num_fields, NULL)
 	PHP_ME_MAPPING(fieldName, sqlite_field_name, NULL)
-	/* spl_forward */
+	/* iterator */
 	PHP_ME_MAPPING(current, sqlite_current, NULL)
+	PHP_ME_MAPPING(key, sqlite_key, NULL)
 	PHP_ME_MAPPING(next, sqlite_next, NULL)
 	PHP_ME_MAPPING(valid, sqlite_valid, NULL)
-	/* spl_sequence */
 	PHP_ME_MAPPING(rewind, sqlite_rewind, NULL)
+	/* countable */
+	PHP_ME_MAPPING(count, sqlite_num_rows, NULL)
 	/* additional */
 	PHP_ME_MAPPING(prev, sqlite_prev, NULL)
 	PHP_ME_MAPPING(hasPrev, sqlite_has_prev, NULL)
@@ -237,7 +250,7 @@ function_entry sqlite_funcs_query[] = {
 	{NULL, NULL, NULL}
 };
 
-function_entry sqlite_funcs_ub_query[] = {
+zend_function_entry sqlite_funcs_ub_query[] = {
 	PHP_ME_MAPPING(fetch, sqlite_fetch_array, NULL)
 	PHP_ME_MAPPING(fetchObject, sqlite_fetch_object, NULL)
 	PHP_ME_MAPPING(fetchSingle, sqlite_fetch_single, NULL)
@@ -245,19 +258,36 @@ function_entry sqlite_funcs_ub_query[] = {
 	PHP_ME_MAPPING(column, sqlite_column, NULL)
 	PHP_ME_MAPPING(numFields, sqlite_num_fields, NULL)
 	PHP_ME_MAPPING(fieldName, sqlite_field_name, NULL)
-	/* spl_forward */
+	/* iterator */
 	PHP_ME_MAPPING(current, sqlite_current, NULL)
 	PHP_ME_MAPPING(next, sqlite_next, NULL)
 	PHP_ME_MAPPING(valid, sqlite_valid, NULL)
 	{NULL, NULL, NULL}
 };
 
-function_entry sqlite_funcs_exception[] = {
+zend_function_entry sqlite_funcs_exception[] = {
+	{NULL, NULL, NULL}
+};
+
+/* Dependancies */
+static zend_module_dep sqlite_deps[] = {
+#if defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+	ZEND_MOD_REQUIRED("spl")
+#endif
+#if HAVE_PHP_SESSION && !defined(COMPILE_DL_SESSION)
+	ZEND_MOD_REQUIRED("session")
+#endif
+#ifdef PHP_SQLITE2_HAVE_PDO
+	ZEND_MOD_REQUIRED("pdo")
+#endif
 	{NULL, NULL, NULL}
 };
 
 zend_module_entry sqlite_module_entry = {
-#if ZEND_MODULE_API_NO >= 20010901
+#if ZEND_MODULE_API_NO >= 20050922
+	STANDARD_MODULE_HEADER_EX, NULL,
+	sqlite_deps,
+#elif ZEND_MODULE_API_NO >= 20010901
 	STANDARD_MODULE_HEADER,
 #endif
 	"SQLite",
@@ -318,11 +348,11 @@ static ZEND_RSRC_DTOR_FUNC(php_sqlite_db_dtor)
 {
 	if (rsrc->ptr) {
 		struct php_sqlite_db *db = (struct php_sqlite_db*)rsrc->ptr;
-	
+
 		sqlite_close(db->db);
 
 		zend_hash_destroy(&db->callbacks);
-		
+
 		pefree(db, db->is_persistent);
 
 		rsrc->ptr = NULL;
@@ -385,7 +415,7 @@ static int php_sqlite_forget_persistent_id_numbers(zend_rsrc_list_entry *rsrc TS
 
 	/* don't leave pending commits hanging around */
 	sqlite_exec(db->db, "ROLLBACK", NULL, NULL, NULL);
-	
+
 	return 0;
 }
 
@@ -410,7 +440,7 @@ static void php_sqlite_generic_function_callback(sqlite_func *func, int argc, co
 		sqlite_set_result_error(func, "not enough parameters", -1);
 		return;
 	}
-	
+
 	ZVAL_STRING(&funcname, (char*)argv[0], 1);
 
 	if (!zend_make_callable(&funcname, &callable TSRMLS_CC)) {
@@ -421,10 +451,10 @@ static void php_sqlite_generic_function_callback(sqlite_func *func, int argc, co
 		zval_dtor(&funcname);
 		return;
 	}
-	
+
 	if (argc > 1) {
 		zargs = (zval ***)safe_emalloc((argc - 1), sizeof(zval **), 0);
-		
+
 		for (i = 0; i < argc-1; i++) {
 			zargs[i] = emalloc(sizeof(zval *));
 			MAKE_STD_ZVAL(*zargs[i]);
@@ -501,7 +531,7 @@ static void php_sqlite_function_callback(sqlite_func *func, int argc, const char
 
 	if (argc > 0) {
 		zargs = (zval ***)safe_emalloc(argc, sizeof(zval **), 0);
-		
+
 		for (i = 0; i < argc; i++) {
 			zargs[i] = emalloc(sizeof(zval *));
 			MAKE_STD_ZVAL(*zargs[i]);
@@ -580,10 +610,10 @@ static void php_sqlite_agg_step_function_callback(sqlite_func *func, int argc, c
 	if (argc < 1) {
 		return;
 	}
-	
+
 	zargc = argc + 1;
 	zargs = (zval ***)safe_emalloc(zargc, sizeof(zval **), 0);
-		
+
 	/* first arg is always the context zval */
 	context_p = (zval **)sqlite_aggregate_context(func, sizeof(*context_p));
 
@@ -645,9 +675,9 @@ static void php_sqlite_agg_fini_function_callback(sqlite_func *func)
 		sqlite_set_result_error(func, "this function has not been correctly defined for this request", -1);
 		return;
 	}
-	
+
 	context_p = (zval **)sqlite_aggregate_context(func, sizeof(*context_p));
-	
+
 	res = call_user_function_ex(EG(function_table),
 			NULL,
 			funcs->fini,
@@ -763,7 +793,7 @@ typedef struct _sqlite_object {
 	} u;
 } sqlite_object;
 
-static int sqlite_free_persistent(list_entry *le, void *ptr TSRMLS_DC)
+static int sqlite_free_persistent(zend_rsrc_list_entry *le, void *ptr TSRMLS_DC)
 {
 	return le->ptr == ptr ? ZEND_HASH_APPLY_REMOVE : ZEND_HASH_APPLY_KEEP;
 }
@@ -802,14 +832,14 @@ static void sqlite_object_new(zend_class_entry *class_type, zend_object_handlers
 	zend_hash_init(intern->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
 	zend_hash_copy(intern->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 
-	retval->handle = zend_objects_store_put(intern, NULL, (zend_objects_free_object_storage_t) sqlite_object_free_storage, NULL TSRMLS_CC);
+	retval->handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t) sqlite_object_free_storage, NULL TSRMLS_CC);
 	retval->handlers = handlers;
 }
 
 static zend_object_value sqlite_object_new_db(zend_class_entry *class_type TSRMLS_DC)
 {
 	zend_object_value retval;
-	
+
 	sqlite_object_new(class_type, &sqlite_object_handlers_db, &retval TSRMLS_CC);
 	return retval;
 }
@@ -817,7 +847,7 @@ static zend_object_value sqlite_object_new_db(zend_class_entry *class_type TSRML
 static zend_object_value sqlite_object_new_query(zend_class_entry *class_type TSRMLS_DC)
 {
 	zend_object_value retval;
-	
+
 	sqlite_object_new(class_type, &sqlite_object_handlers_query, &retval TSRMLS_CC);
 	return retval;
 }
@@ -825,7 +855,7 @@ static zend_object_value sqlite_object_new_query(zend_class_entry *class_type TS
 static zend_object_value sqlite_object_new_ub_query(zend_class_entry *class_type TSRMLS_DC)
 {
 	zend_object_value retval;
-	
+
 	sqlite_object_new(class_type, &sqlite_object_handlers_ub_query, &retval TSRMLS_CC);
 	return retval;
 }
@@ -833,7 +863,7 @@ static zend_object_value sqlite_object_new_ub_query(zend_class_entry *class_type
 static zend_object_value sqlite_object_new_exception(zend_class_entry *class_type TSRMLS_DC)
 {
 	zend_object_value retval;
-	
+
 	sqlite_object_new(class_type, &sqlite_object_handlers_exception, &retval TSRMLS_CC);
 	return retval;
 }
@@ -919,7 +949,7 @@ void sqlite_iterator_get_current_data(zend_object_iterator *iter, zval ***data T
 		MAKE_STD_ZVAL(**data);
 		php_sqlite_fetch_array(res, res->mode, 1, 0, **data TSRMLS_CC);
 	}
-	
+
 }
 
 int sqlite_iterator_get_current_key(zend_object_iterator *iter, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC)
@@ -948,7 +978,7 @@ void sqlite_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 			/* php_error_docref(NULL TSRMLS_CC, E_WARNING, "no more rows available"); */
 			return;
 		}
-	
+
 		res->curr_row++;
 	}
 }
@@ -997,17 +1027,27 @@ PHP_MINIT_FUNCTION(sqlite)
 	REGISTER_SQLITE_CLASS(Database,   db,        NULL);
 	REGISTER_SQLITE_CLASS(Result,     query,     NULL);
 	REGISTER_SQLITE_CLASS(Unbuffered, ub_query,  NULL);
+#if defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+	REGISTER_SQLITE_CLASS(Exception,  exception, spl_ce_RuntimeException);
+#else
 	REGISTER_SQLITE_CLASS(Exception,  exception, zend_exception_get_default());
+#endif
 
 	sqlite_ce_db->ce_flags &= ~ZEND_ACC_FINAL_CLASS;
 	sqlite_ce_db->constructor->common.fn_flags |= ZEND_ACC_FINAL;
 
 	sqlite_object_handlers_query.get_class_entry = sqlite_get_ce_query;
 	sqlite_object_handlers_ub_query.get_class_entry = sqlite_get_ce_ub_query;
-	
+	sqlite_object_handlers_ub_query.count_elements = sqlite_count_elements;
+
 	sqlite_ce_ub_query->get_iterator = sqlite_get_iterator;
 	sqlite_ce_ub_query->iterator_funcs.funcs = &sqlite_ub_query_iterator_funcs;
 
+#if defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+	zend_class_implements(sqlite_ce_query TSRMLS_CC, 2, zend_ce_iterator, spl_ce_Countable);
+#else
+	zend_class_implements(sqlite_ce_query TSRMLS_CC, 1, zend_ce_iterator);
+#endif
 	sqlite_ce_query->get_iterator = sqlite_get_iterator;
 	sqlite_ce_query->iterator_funcs.funcs = &sqlite_query_iterator_funcs;
 
@@ -1018,7 +1058,7 @@ PHP_MINIT_FUNCTION(sqlite)
 #if HAVE_PHP_SESSION && !defined(COMPILE_DL_SESSION)
 	php_session_register_module(ps_sqlite_ptr);
 #endif
-	
+
 	le_sqlite_db = zend_register_list_destructors_ex(php_sqlite_db_dtor, NULL, "sqlite database", module_number);
 	le_sqlite_pdb = zend_register_list_destructors_ex(NULL, php_sqlite_db_dtor, "sqlite database (persistent)", module_number);
 	le_sqlite_result = zend_register_list_destructors_ex(php_sqlite_result_dtor, NULL, "sqlite result", module_number);
@@ -1026,7 +1066,7 @@ PHP_MINIT_FUNCTION(sqlite)
 	REGISTER_LONG_CONSTANT("SQLITE_BOTH",	PHPSQLITE_BOTH, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SQLITE_NUM",	PHPSQLITE_NUM, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SQLITE_ASSOC",	PHPSQLITE_ASSOC, CONST_CS|CONST_PERSISTENT);
-	
+
 	REGISTER_LONG_CONSTANT("SQLITE_OK",				SQLITE_OK, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SQLITE_ERROR",			SQLITE_ERROR, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SQLITE_INTERNAL",		SQLITE_INTERNAL, CONST_CS|CONST_PERSISTENT);
@@ -1058,12 +1098,23 @@ PHP_MINIT_FUNCTION(sqlite)
 	REGISTER_LONG_CONSTANT("SQLITE_ROW",			SQLITE_ROW, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SQLITE_DONE",			SQLITE_DONE, CONST_CS|CONST_PERSISTENT);
 
+#ifdef PHP_SQLITE2_HAVE_PDO
+    if (FAILURE == php_pdo_register_driver(&pdo_sqlite2_driver)) {
+	return FAILURE;
+    }
+#endif
+
 	return SUCCESS;
 }
 
 PHP_MSHUTDOWN_FUNCTION(sqlite)
 {
 	UNREGISTER_INI_ENTRIES();
+
+#ifdef PHP_SQLITE2_HAVE_PDO
+    php_pdo_unregister_driver(&pdo_sqlite2_driver);
+#endif
+
 	return SUCCESS;
 }
 
@@ -1076,7 +1127,7 @@ PHP_MINFO_FUNCTION(sqlite)
 {
 	php_info_print_table_start();
 	php_info_print_table_header(2, "SQLite support", "enabled");
-	php_info_print_table_row(2, "PECL Module version", PHP_SQLITE_MODULE_VERSION " $Id: sqlite.c,v 1.146.2.7 2005/06/07 15:38:37 dmitry Exp $");
+	php_info_print_table_row(2, "PECL Module version", PHP_SQLITE_MODULE_VERSION " $Id: sqlite.c,v 1.166.2.11 2006/01/01 12:50:14 sniper Exp $");
 	php_info_print_table_row(2, "SQLite Library", sqlite_libversion());
 	php_info_print_table_row(2, "SQLite Encoding", sqlite_libencoding());
 	php_info_print_table_end();
@@ -1117,7 +1168,7 @@ static struct php_sqlite_db *php_sqlite_open(char *filename, int mode, char *per
 	db->db = sdb;
 
 	zend_hash_init(&db->callbacks, 0, NULL, php_sqlite_callback_dtor, db->is_persistent);
-	
+
 	/* register the PHP functions */
 	sqlite_create_function(sdb, "php", -1, php_sqlite_generic_function_callback, 0);
 
@@ -1128,8 +1179,10 @@ static struct php_sqlite_db *php_sqlite_open(char *filename, int mode, char *per
 	/* authorizer hook so we can enforce safe mode
 	 * Note: the declaration of php_sqlite_authorizer is correct for 2.8.2 of libsqlite,
 	 * and IS backwards binary compatible with earlier versions */
-	sqlite_set_authorizer(sdb, php_sqlite_authorizer, NULL);
-	
+	if (PG(safe_mode) || (PG(open_basedir) && *PG(open_basedir))) {
+		sqlite_set_authorizer(sdb, php_sqlite_authorizer, NULL);
+	}
+
 	db->rsrc_id = ZEND_REGISTER_RESOURCE(object ? NULL : return_value, db, persistent_id ? le_sqlite_pdb : le_sqlite_db);
 	if (object) {
 		/* if object is not an object then we're called from the factory() function */
@@ -1141,7 +1194,7 @@ static struct php_sqlite_db *php_sqlite_open(char *filename, int mode, char *per
 	}
 
 	if (persistent_id) {
-		list_entry le;
+		zend_rsrc_list_entry le;
 
 		Z_TYPE(le) = le_sqlite_pdb;
 		le.ptr = db;
@@ -1152,7 +1205,7 @@ static struct php_sqlite_db *php_sqlite_open(char *filename, int mode, char *per
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to register persistent resource");
 		}
 	}
-	
+
 	return db;
 }
 
@@ -1165,8 +1218,8 @@ PHP_FUNCTION(sqlite_popen)
 	int filename_len, hashkeylen;
 	zval *errmsg = NULL;
 	struct php_sqlite_db *db = NULL;
-	list_entry *le;
-	
+	zend_rsrc_list_entry *le;
+
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz/",
 				&filename, &filename_len, &mode, &errmsg)) {
 		return;
@@ -1179,7 +1232,7 @@ PHP_FUNCTION(sqlite_popen)
 	if (strncmp(filename, ":memory:", sizeof(":memory:") - 1)) {
 		/* resolve the fully-qualified path name to use as the hash key */
 		fullpath = expand_filepath(filename, NULL TSRMLS_CC);
-	
+
 		if (PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 			RETURN_FALSE;
 		}
@@ -1192,12 +1245,12 @@ PHP_FUNCTION(sqlite_popen)
 	}
 
 	hashkeylen = spprintf(&hashkey, 0, "sqlite_pdb_%s:%ld", fullpath, mode);
-	
+
 	/* do we have an existing persistent connection ? */
 	if (SUCCESS == zend_hash_find(&EG(persistent_list), hashkey, hashkeylen+1, (void*)&le)) {
 		if (Z_TYPE_P(le) == le_sqlite_pdb) {
 			db = (struct php_sqlite_db*)le->ptr;
-			
+
 			if (db->rsrc_id == FAILURE) {
 				/* give it a valid resource id for this request */
 				db->rsrc_id = ZEND_REGISTER_RESOURCE(return_value, db, le_sqlite_pdb);
@@ -1255,7 +1308,7 @@ PHP_FUNCTION(sqlite_open)
 	if (strncmp(filename, ":memory:", sizeof(":memory:") - 1)) {
 		/* resolve the fully-qualified path name to use as the hash key */
 		fullpath = expand_filepath(filename, NULL TSRMLS_CC);
-	
+
 		if (PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 			php_std_error_handling();
 			efree(fullpath);
@@ -1310,13 +1363,13 @@ PHP_FUNCTION(sqlite_factory)
 	if (strncmp(filename, ":memory:", sizeof(":memory:") - 1)) {
 		/* resolve the fully-qualified path name to use as the hash key */
 		fullpath = expand_filepath(filename, NULL TSRMLS_CC);
-	
+
 		if (PG(safe_mode) && (!php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 			efree(fullpath);
 			php_std_error_handling();
 			RETURN_NULL();
 		}
-	
+
 		if (php_check_open_basedir(fullpath TSRMLS_CC)) {
 			efree(fullpath);
 			php_std_error_handling();
@@ -1383,7 +1436,7 @@ static int php_sqlite_fetch(struct php_sqlite_result *rres TSRMLS_DC)
 {
 	const char **rowdata, **colnames;
 	int ret, i, base;
-	char *errtext = NULL, *colname;
+	char *errtext = NULL;
 
 next_row:
 	ret = sqlite_step(rres->vm, &rres->ncolumns, &rowdata, &colnames);
@@ -1391,14 +1444,13 @@ next_row:
 		/* first row - lets copy the column names */
 		rres->col_names = safe_emalloc(rres->ncolumns, sizeof(char *), 0);
 		for (i = 0; i < rres->ncolumns; i++) {
-			colname = (char*)colnames[i];
+			rres->col_names[i] = estrdup((char*)colnames[i]);
 
 			if (SQLITE_G(assoc_case) == 1) {
-				php_sqlite_strtoupper(colname);
+				php_sqlite_strtoupper(rres->col_names[i]);
 			} else if (SQLITE_G(assoc_case) == 2) {
-				php_sqlite_strtolower(colname);
+				php_sqlite_strtolower(rres->col_names[i]);
 			}
-			rres->col_names[i] = estrdup(colname);
 		}
 		if (!rres->buffered) {
 			/* non buffered mode - also fetch memory for on single row */
@@ -1466,7 +1518,7 @@ next_row:
 /* }}} */
 
 /* {{{ sqlite_query */
-void sqlite_query(zval *object, struct php_sqlite_db *db, char *sql, long sql_len, int mode, int buffered, zval *return_value, struct php_sqlite_result **prres TSRMLS_DC)
+void sqlite_query(zval *object, struct php_sqlite_db *db, char *sql, long sql_len, int mode, int buffered, zval *return_value, struct php_sqlite_result **prres, zval *errmsg TSRMLS_DC)
 {
 	struct php_sqlite_result res, *rres;
 	int ret;
@@ -1482,6 +1534,9 @@ void sqlite_query(zval *object, struct php_sqlite_db *db, char *sql, long sql_le
 
 	if (ret != SQLITE_OK) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errtext);
+		if (errmsg) {
+			ZVAL_STRING(errmsg, errtext, 1);
+		}
 		sqlite_freemem(errtext);
 		goto terminate;
 	} else if (!res.vm) { /* empty query */
@@ -1503,7 +1558,7 @@ terminate:
 	memcpy(*prres, &res, sizeof(**prres));
 	(*prres)->db = db;
 	zend_list_addref(db->rsrc_id);
-	
+
 
 	/* now the result set is ready for stepping: get first row */
 	if (php_sqlite_fetch((*prres) TSRMLS_CC) != SQLITE_OK) {
@@ -1512,10 +1567,10 @@ terminate:
 		if (return_value) {
 			RETURN_FALSE;
 		} else {
-			return;	
+			return;
 		}
 	}
-	
+
 	(*prres)->curr_row = 0;
 
 	if (object) {
@@ -1534,7 +1589,7 @@ terminate:
 }
 /* }}} */
 
-/* {{{ proto resource sqlite_unbuffered_query(string query, resource db [ , int result_type ])
+/* {{{ proto resource sqlite_unbuffered_query(string query, resource db [ , int result_type [, string &error_message]])
    Executes a query that does not prefetch and buffer all data. */
 PHP_FUNCTION(sqlite_unbuffered_query)
 {
@@ -1544,22 +1599,28 @@ PHP_FUNCTION(sqlite_unbuffered_query)
 	int sql_len;
 	long mode = PHPSQLITE_BOTH;
 	char *errtext = NULL;
+	zval *errmsg = NULL;
 	zval *object = getThis();
 
 	if (object) {
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &sql, &sql_len, &mode)) {
+		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz/", &sql, &sql_len, &mode, &errmsg)) {
 			return;
 		}
 		DB_FROM_OBJECT(db, object);
 	} else {
 		if (FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-				ZEND_NUM_ARGS() TSRMLS_CC, "sr|l", &sql, &sql_len, &zdb, &mode) && 
-			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &zdb, &sql, &sql_len, &mode)) {
+				ZEND_NUM_ARGS() TSRMLS_CC, "sr|lz/", &sql, &sql_len, &zdb, &mode, &errmsg) &&
+			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|lz/", &zdb, &sql, &sql_len, &mode, &errmsg)) {
 			return;
 		}
 		DB_FROM_ZVAL(db, &zdb);
 	}
 
+	if (errmsg) {
+		zval_dtor(errmsg);
+		ZVAL_NULL(errmsg);
+	}
+
 	PHP_SQLITE_EMPTY_QUERY;
 
 	/* avoid doing work if we can */
@@ -1568,16 +1629,19 @@ PHP_FUNCTION(sqlite_unbuffered_query)
 
 		if (db->last_err_code != SQLITE_OK) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errtext);
+			if (errmsg) {
+				ZVAL_STRING(errmsg, errtext, 1);
+			}
 			sqlite_freemem(errtext);
 		}
 		return;
 	}
 
-	sqlite_query(object, db, sql, sql_len, (int)mode, 0, return_value, NULL TSRMLS_CC);
+	sqlite_query(object, db, sql, sql_len, (int)mode, 0, return_value, NULL, errmsg TSRMLS_CC);
 }
 /* }}} */
 
-/* {{{ proto resource sqlite_fetch_column_types(string table_name, resource db)
+/* {{{ proto resource sqlite_fetch_column_types(string table_name, resource db [, int result_type])
    Return an array of column types from a particular table. */
 PHP_FUNCTION(sqlite_fetch_column_types)
 {
@@ -1590,16 +1654,17 @@ PHP_FUNCTION(sqlite_fetch_column_types)
 	struct php_sqlite_result res;
 	const char **rowdata, **colnames, *tail;
 	int i, ncols;
+	long result_type = PHPSQLITE_ASSOC;
 
 	if (object) {
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &tbl, &tbl_len)) {
+		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &tbl, &tbl_len, &result_type)) {
 			return;
 		}
 		DB_FROM_OBJECT(db, object);
 	} else {
 		if (FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-				ZEND_NUM_ARGS() TSRMLS_CC, "sr", &tbl, &tbl_len, &zdb) && 
-			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &zdb, &tbl, &tbl_len)) {
+				ZEND_NUM_ARGS() TSRMLS_CC, "sr|l", &tbl, &tbl_len, &zdb, &result_type) &&
+			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &zdb, &tbl, &tbl_len, &result_type)) {
 			return;
 		}
 		DB_FROM_ZVAL(db, &zdb);
@@ -1627,15 +1692,21 @@ PHP_FUNCTION(sqlite_fetch_column_types)
 	array_init(return_value);
 
 	for (i = 0; i < ncols; i++) {
-		char *colname = (char *)colnames[i];
+		if (result_type == PHPSQLITE_ASSOC) {
+			char *colname = estrdup((char *)colnames[i]);
 
-		if (SQLITE_G(assoc_case) == 1) {
-			php_sqlite_strtoupper(colname);
-		} else if (SQLITE_G(assoc_case) == 2) {
-			php_sqlite_strtolower(colname);
+			if (SQLITE_G(assoc_case) == 1) {
+				php_sqlite_strtoupper(colname);
+			} else if (SQLITE_G(assoc_case) == 2) {
+				php_sqlite_strtolower(colname);
+			}
+
+			add_assoc_string(return_value, colname, colnames[ncols + i] ? (char *)colnames[ncols + i] : "", 1);
+			efree(colname);
+		}
+		if (result_type == PHPSQLITE_NUM) {
+			add_index_string(return_value, i, colnames[ncols + i] ? (char *)colnames[ncols + i] : "", 1);
 		}
-
-		add_assoc_string(return_value, colname, colnames[ncols + i] ? (char *)colnames[ncols + i] : "", 1);
 	}
 
 done:
@@ -1643,7 +1714,7 @@ done:
 }
 /* }}} */
 
-/* {{{ proto resource sqlite_query(string query, resource db [, int result_type ])
+/* {{{ proto resource sqlite_query(string query, resource db [, int result_type [, string &error_message]])
    Executes a query against a given database and returns a result handle. */
 PHP_FUNCTION(sqlite_query)
 {
@@ -1653,22 +1724,28 @@ PHP_FUNCTION(sqlite_query)
 	int sql_len;
 	long mode = PHPSQLITE_BOTH;
 	char *errtext = NULL;
+	zval *errmsg = NULL;
 	zval *object = getThis();
 
 	if (object) {
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &sql, &sql_len, &mode)) {
+		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz/", &sql, &sql_len, &mode, &errmsg)) {
 			return;
 		}
 		DB_FROM_OBJECT(db, object);
 	} else {
 		if (FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-				ZEND_NUM_ARGS() TSRMLS_CC, "sr|l", &sql, &sql_len, &zdb, &mode) && 
-			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &zdb, &sql, &sql_len, &mode)) {
+				ZEND_NUM_ARGS() TSRMLS_CC, "sr|lz/", &sql, &sql_len, &zdb, &mode, &errmsg) &&
+			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|lz/", &zdb, &sql, &sql_len, &mode, &errmsg)) {
 			return;
 		}
 		DB_FROM_ZVAL(db, &zdb);
 	}
 
+	if (errmsg) {
+		zval_dtor(errmsg);
+		ZVAL_NULL(errmsg);
+	}
+
 	PHP_SQLITE_EMPTY_QUERY;
 
 	/* avoid doing work if we can */
@@ -1677,16 +1754,19 @@ PHP_FUNCTION(sqlite_query)
 
 		if (db->last_err_code != SQLITE_OK) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errtext);
+			if (errmsg) {
+				ZVAL_STRING(errmsg, errtext, 1);
+			}
 			sqlite_freemem(errtext);
 		}
 		return;
 	}
 
-	sqlite_query(object, db, sql, sql_len, (int)mode, 1, return_value, NULL TSRMLS_CC);
+	sqlite_query(object, db, sql, sql_len, (int)mode, 1, return_value, NULL, errmsg TSRMLS_CC);
 }
 /* }}} */
 
-/* {{{ proto boolean sqlite_exec(string query, resource db)
+/* {{{ proto boolean sqlite_exec(string query, resource db[, string &error_message])
    Executes a result-less query against a given database */
 PHP_FUNCTION(sqlite_exec)
 {
@@ -1695,28 +1775,37 @@ PHP_FUNCTION(sqlite_exec)
 	char *sql;
 	int sql_len;
 	char *errtext = NULL;
+	zval *errmsg = NULL;
 	zval *object = getThis();
 
 	if (object) {
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &sql, &sql_len)) {
+		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z/", &sql, &sql_len, &errmsg)) {
 			return;
 		}
 		DB_FROM_OBJECT(db, object);
 	} else {
 		if(FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-			ZEND_NUM_ARGS() TSRMLS_CC, "sr", &sql, &sql_len, &zdb) && 
-		   FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &zdb, &sql, &sql_len)) {
+			ZEND_NUM_ARGS() TSRMLS_CC, "sr", &sql, &sql_len, &zdb) &&
+		   FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|z/", &zdb, &sql, &sql_len, &errmsg)) {
 			return;
 		}
 		DB_FROM_ZVAL(db, &zdb);
 	}
-	
+
+	if (errmsg) {
+		zval_dtor(errmsg);
+		ZVAL_NULL(errmsg);
+	}
+
 	PHP_SQLITE_EMPTY_QUERY;
 
 	db->last_err_code = sqlite_exec(db->db, sql, NULL, NULL, &errtext);
 
 	if (db->last_err_code != SQLITE_OK) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errtext);
+		if (errmsg) {
+			ZVAL_STRING(errmsg, errtext, 1);
+		}
 		sqlite_freemem(errtext);
 		RETURN_FALSE;
 	}
@@ -1837,12 +1926,12 @@ static void php_sqlite_fetch_column(struct php_sqlite_result *res, zval *which,
 		if (!res->buffered) {
 			efree((char*)rowdata[j]);
 			rowdata[j] = NULL;
-		}		
+		}
 	} else {
 		RETVAL_STRING((char*)rowdata[j], res->buffered);
 		if (!res->buffered) {
 			rowdata[j] = NULL;
-		}		
+		}
 	}
 }
 /* }}} */
@@ -1940,7 +2029,7 @@ PHP_FUNCTION(sqlite_fetch_object)
 	zval dataset;
 	zend_fcall_info fci;
 	zend_fcall_info_cache fcc;
-	zval *retval_ptr; 
+	zval *retval_ptr;
 	zval *ctor_params = NULL;
 
 	php_set_error_handling(object ? EH_THROW : EH_NORMAL, sqlite_ce_exception TSRMLS_CC);
@@ -2008,7 +2097,7 @@ PHP_FUNCTION(sqlite_fetch_object)
 			} else {
 				/* Two problems why we throw exceptions here: PHP is typeless
 				 * and hence passing one argument that's not an array could be
-				 * by mistake and the other way round is possible, too. The 
+				 * by mistake and the other way round is possible, too. The
 				 * single value is an array. Also we'd have to make that one
 				 * argument passed by reference.
 				 */
@@ -2063,7 +2152,7 @@ PHP_FUNCTION(sqlite_array_query)
 		DB_FROM_OBJECT(db, object);
 	} else {
 		if (FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-				ZEND_NUM_ARGS() TSRMLS_CC, "sr|lb", &sql, &sql_len, &zdb, &mode, &decode_binary) && 
+				ZEND_NUM_ARGS() TSRMLS_CC, "sr|lb", &sql, &sql_len, &zdb, &mode, &decode_binary) &&
 			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|lb", &zdb, &sql, &sql_len, &mode, &decode_binary)) {
 			return;
 		}
@@ -2082,15 +2171,15 @@ PHP_FUNCTION(sqlite_array_query)
 		}
 		return;
 	}
-	
+
 	rres = (struct php_sqlite_result *)emalloc(sizeof(*rres));
-	sqlite_query(NULL, db, sql, sql_len, (int)mode, 0, NULL, &rres TSRMLS_CC);
- 	if (db->last_err_code != SQLITE_OK) {
- 		if (rres) {
-	 		efree(rres);
-	 	}
- 		RETURN_FALSE;
- 	}
+	sqlite_query(NULL, db, sql, sql_len, (int)mode, 0, NULL, &rres, NULL TSRMLS_CC);
+	if (db->last_err_code != SQLITE_OK) {
+		if (rres) {
+			efree(rres);
+		}
+		RETURN_FALSE;
+	}
 
 	array_init(return_value);
 
@@ -2109,13 +2198,13 @@ static void php_sqlite_fetch_single(struct php_sqlite_result *res, zend_bool dec
 	const char **rowdata;
 	char *decoded;
 	int decoded_len;
-	
+
 	/* check range of the row */
 	if (res->curr_row >= res->nrows) {
 		/* no more */
 		RETURN_FALSE;
 	}
-	
+
 	if (res->buffered) {
 		rowdata = (const char**)&res->table[res->curr_row * res->ncolumns];
 	} else {
@@ -2179,7 +2268,7 @@ PHP_FUNCTION(sqlite_single_query)
 		RES_FROM_OBJECT(db, object);
 	} else {
 		if (FAILURE == zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-				ZEND_NUM_ARGS() TSRMLS_CC, "sr|bb", &sql, &sql_len, &zdb, &srow, &decode_binary) && 
+				ZEND_NUM_ARGS() TSRMLS_CC, "sr|bb", &sql, &sql_len, &zdb, &srow, &decode_binary) &&
 			FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|bb", &zdb, &sql, &sql_len, &srow, &decode_binary)) {
 			return;
 		}
@@ -2200,7 +2289,7 @@ PHP_FUNCTION(sqlite_single_query)
 	}
 
 	rres = (struct php_sqlite_result *)emalloc(sizeof(*rres));
-	sqlite_query(NULL, db, sql, sql_len, PHPSQLITE_NUM, 0, NULL, &rres TSRMLS_CC);
+	sqlite_query(NULL, db, sql, sql_len, PHPSQLITE_NUM, 0, NULL, &rres, NULL TSRMLS_CC);
 	if (db->last_err_code != SQLITE_OK) {
 		if (rres) {
 			efree(rres);
@@ -2390,6 +2479,19 @@ PHP_FUNCTION(sqlite_last_insert_rowid)
 }
 /* }}} */
 
+static int sqlite_count_elements(zval *object, long *count TSRMLS_DC) /* {{{ */
+{
+	sqlite_object *obj = (sqlite_object*) zend_object_store_get_object(object TSRMLS_CC);
+
+	if (obj->u.res->buffered) {
+		* count = obj->u.res->nrows;
+		return SUCCESS;
+	} else {
+		zend_throw_exception(sqlite_ce_exception, "Row count is not available for unbuffered queries", 0 TSRMLS_CC);
+		return FAILURE;
+	}
+} /* }}} */
+
 /* {{{ proto int sqlite_num_rows(resource result)
    Returns the number of rows in a buffered result set. */
 PHP_FUNCTION(sqlite_num_rows)
@@ -2468,7 +2570,7 @@ PHP_FUNCTION(sqlite_has_prev)
 		RETURN_FALSE;
 	}
 
-	RETURN_BOOL(res->curr_row); 
+	RETURN_BOOL(res->curr_row);
 }
 /* }}} */
 
@@ -2551,7 +2653,7 @@ PHP_FUNCTION(sqlite_seek)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot seek an unbuffered result set");
 		RETURN_FALSE;
 	}
-	
+
 	if (row < 0 || row >= res->nrows) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "row %ld out of range", row);
 		RETURN_FALSE;
@@ -2586,7 +2688,7 @@ PHP_FUNCTION(sqlite_rewind)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot rewind an unbuffered result set");
 		RETURN_FALSE;
 	}
-	
+
 	if (!res->nrows) {
 		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "no rows received");
 		RETURN_FALSE;
@@ -2632,6 +2734,30 @@ PHP_FUNCTION(sqlite_next)
 }
 /* }}} */
 
+/* {{{ proto int sqlite_key(resource result)
+   Return the current row index of a buffered result. */
+PHP_FUNCTION(sqlite_key)
+{
+	zval *zres;
+	struct php_sqlite_result *res;
+	zval *object = getThis();
+
+	if (object) {
+		if (ZEND_NUM_ARGS() != 0) {
+			WRONG_PARAM_COUNT
+		}
+		RES_FROM_OBJECT(res, object);
+	} else {
+		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zres)) {
+			return;
+		}
+		ZEND_FETCH_RESOURCE(res, struct php_sqlite_result *, &zres, -1, "sqlite result", le_sqlite_result);
+	}
+
+	RETURN_LONG(res->curr_row);
+}
+/* }}} */
+
 /* {{{ proto bool sqlite_prev(resource result)
  * Seek to the previous row number of a result set. */
 PHP_FUNCTION(sqlite_prev)
@@ -2683,12 +2809,12 @@ PHP_FUNCTION(sqlite_escape_string)
 	if (stringlen && (string[0] == '\x01' || memchr(string, '\0', stringlen) != NULL)) {
 		/* binary string */
 		int enclen;
-		
+
 		ret = safe_emalloc(1 + stringlen / 254, 257, 3);
 		ret[0] = '\x01';
 		enclen = php_sqlite_encode_binary(string, stringlen, ret+1);
 		RETVAL_STRINGL(ret, enclen+1, 0);
-		
+
 	} else if (stringlen) {
 		ret = sqlite_mprintf("%q", string);
 		if (ret) {
@@ -2735,7 +2861,7 @@ PHP_FUNCTION(sqlite_error_string)
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &code)) {
 		return;
 	}
-	
+
 	msg = sqlite_error_string(code);
 
 	if (msg) {
@@ -2803,7 +2929,7 @@ static enum callback_prep_t prep_callback_struct(struct php_sqlite_db *db, int i
 	}
 	alloc_funcs->is_valid = 1;
 	*funcs = alloc_funcs;
-	
+
 	return ret;
 }
 
@@ -2839,7 +2965,7 @@ PHP_FUNCTION(sqlite_create_aggregate)
 		return;
 	}
 	efree(callable);
-	
+
 	if (!zend_is_callable(zfinal, 0, &callable)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "finalize function `%s' is not callable", callable);
 		efree(callable);
@@ -2847,13 +2973,13 @@ PHP_FUNCTION(sqlite_create_aggregate)
 	}
 	efree(callable);
 
-	
+
 	if (prep_callback_struct(db, 1, funcname, zstep, zfinal, &funcs) == DO_REG) {
 		sqlite_create_aggregate(db->db, funcname, num_args,
 				php_sqlite_agg_step_function_callback,
 				php_sqlite_agg_fini_function_callback, funcs);
 	}
-	
+
 
 }
 /* }}} */
@@ -2869,7 +2995,7 @@ PHP_FUNCTION(sqlite_create_function)
 	struct php_sqlite_agg_functions *funcs;
 	char *callable = NULL;
 	long num_args = -1;
-	
+
 	zval *object = getThis();
 
 	if (object) {
@@ -2890,7 +3016,7 @@ PHP_FUNCTION(sqlite_create_function)
 		return;
 	}
 	efree(callable);
-	
+
 	if (prep_callback_struct(db, 0, funcname, zcall, NULL, &funcs) == DO_REG) {
 		sqlite_create_function(db->db, funcname, num_args, php_sqlite_function_callback, funcs);
 	}
@@ -2915,7 +3041,7 @@ PHP_FUNCTION(sqlite_udf_encode_binary)
 		/* binary string */
 		int enclen;
 		char *ret;
-		
+
 		ret = safe_emalloc(1 + datalen / 254, 257, 3);
 		ret[0] = '\x01';
 		enclen = php_sqlite_encode_binary(data, datalen, ret+1);
@@ -2944,7 +3070,7 @@ PHP_FUNCTION(sqlite_udf_decode_binary)
 		/* encoded string */
 		int enclen;
 		char *ret;
-		
+
 		ret = emalloc(datalen);
 		enclen = php_sqlite_decode_binary(data+1, ret);
 		ret[enclen] = '\0';
