@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,14 +17,13 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: basic_functions.c,v 1.673.2.18 2005/08/21 18:36:33 zeev Exp $ */
+/* $Id: basic_functions.c,v 1.725.2.17 2006/01/04 21:31:29 derick Exp $ */
 
 #include "php.h"
 #include "php_streams.h"
 #include "php_main.h"
 #include "php_globals.h"
 #include "php_ini.h"
-#include "internal_functions_registry.h"
 #include "php_standard.h"
 #include "php_math.h"
 #include "php_http.h"
@@ -51,7 +50,11 @@ typedef struct yy_buffer_state *YY_BUFFER_STATE;
 #include <time.h>
 #include <stdio.h>
 
-#include <netdb.h>
+#ifdef NETWARE
+#include <netinet/in.h>
+#endif
+
+#include<netdb.h>
 
 #if HAVE_ARPA_INET_H
 # include <arpa/inet.h>
@@ -104,6 +107,8 @@ php_basic_globals basic_globals;
 #include "php_fopen_wrappers.h"
 #include "streamsfuncs.h"
 
+static zend_class_entry *incomplete_class_entry = NULL;
+
 static
 	ZEND_BEGIN_ARG_INFO(first_and_second__args_force_ref, 0)
 		ZEND_ARG_PASS_INFO(1)
@@ -139,6 +144,10 @@ static
 		ZEND_ARG_PASS_INFO(1)
 	ZEND_END_ARG_INFO()
 
+static
+	ZEND_BEGIN_ARG_INFO(all_args_prefer_ref, ZEND_SEND_PREFER_REF)
+	ZEND_END_ARG_INFO()
+
 typedef struct _php_shutdown_function_entry {
 	zval **arguments;
 	int arg_count;
@@ -156,36 +165,26 @@ static void user_tick_function_dtor(user_tick_function_entry *tick_function_entr
 
 #undef sprintf
 
-function_entry basic_functions[] = {
+zend_function_entry basic_functions[] = {
 	PHP_FE(constant,														NULL)
 	PHP_FE(bin2hex,															NULL)
 	PHP_FE(sleep,															NULL)
 	PHP_FE(usleep,															NULL)
 #if HAVE_NANOSLEEP
 	PHP_FE(time_nanosleep,														NULL)
-#endif
-	PHP_FE(time,															NULL)
-	PHP_FE(mktime,															NULL)
-	PHP_FE(gmmktime,														NULL)
-
-#if HAVE_STRFTIME
-	PHP_FE(strftime,														NULL)
-	PHP_FE(gmstrftime,														NULL)
+	PHP_FE(time_sleep_until,														NULL)
 #endif
 
-	PHP_FE(strtotime,														NULL)
-	PHP_FE(date,															NULL)
-	PHP_FE(idate,															NULL)
-	PHP_FE(gmdate,															NULL)
-	PHP_FE(getdate,															NULL)
-	PHP_FE(localtime,														NULL)
-	PHP_FE(checkdate,														NULL)
+#if HAVE_STRPTIME
+	PHP_FE(strptime,														NULL)
+#endif
 
 	PHP_FE(flush,															NULL)
 	PHP_FE(wordwrap,														NULL)
 	PHP_FE(htmlspecialchars,												NULL)
 	PHP_FE(htmlentities,													NULL)
 	PHP_FE(html_entity_decode,												NULL)
+	PHP_FE(htmlspecialchars_decode,												NULL)
 	PHP_FE(get_html_translation_table,										NULL)
 	PHP_FE(sha1,															NULL)
 	PHP_FE(sha1_file,														NULL)
@@ -395,9 +394,7 @@ function_entry basic_functions[] = {
 	PHP_FE(log,																NULL)
 	PHP_FE(log10,															NULL)
 	PHP_FE(sqrt,															NULL)
-#ifdef HAVE_HYPOT
 	PHP_FE(hypot,															NULL)
-#endif	
 	PHP_FE(deg2rad,															NULL)
 	PHP_FE(rad2deg,															NULL)
 	PHP_FE(bindec,															NULL)
@@ -409,6 +406,12 @@ function_entry basic_functions[] = {
 	PHP_FE(base_convert,													NULL)
 	PHP_FE(number_format,													NULL)
 	PHP_FE(fmod,															NULL)
+#ifdef HAVE_INET_NTOP
+	PHP_NAMED_FE(inet_ntop,		php_inet_ntop,											NULL)
+#endif
+#ifdef HAVE_INET_PTON
+	PHP_NAMED_FE(inet_pton,		php_inet_pton,											NULL)
+#endif
 	PHP_FE(ip2long,															NULL)
 	PHP_FE(long2ip,															NULL)
 
@@ -450,7 +453,7 @@ function_entry basic_functions[] = {
 	PHP_FE(call_user_func_array,											NULL)
 	PHP_FE(call_user_method,		second_arg_force_ref)
 	PHP_FE(call_user_method_array,	second_arg_force_ref)
-	PHP_FE(serialize,														NULL)															
+	PHP_FE(serialize,														NULL)
 	PHP_FE(unserialize,														NULL)
 
 	PHP_FE(var_dump,														NULL)
@@ -580,17 +583,24 @@ function_entry basic_functions[] = {
 	PHP_FE(stream_context_set_params,										NULL)
 	PHP_FE(stream_context_set_option,										NULL)
 	PHP_FE(stream_context_get_options,										NULL)
+	PHP_FE(stream_context_get_default,										NULL)
 	PHP_FE(stream_filter_prepend,											NULL)
 	PHP_FE(stream_filter_append,											NULL)
+	PHP_FE(stream_filter_remove,											NULL)
 	PHP_FE(stream_socket_client,				 second_and_third_args_force_ref)
 	PHP_FE(stream_socket_server,				 second_and_third_args_force_ref)
 	PHP_FE(stream_socket_accept,				 		   third_arg_force_ref)
 	PHP_FE(stream_socket_get_name,											NULL)
 	PHP_FE(stream_socket_recvfrom,							fourth_arg_force_ref)
 	PHP_FE(stream_socket_sendto,											NULL)
+	PHP_FE(stream_socket_enable_crypto,										NULL)
+#if HAVE_SOCKETPAIR
+	PHP_FE(stream_socket_pair,												NULL)
+#endif
 	PHP_FE(stream_copy_to_stream,											NULL)
 	PHP_FE(stream_get_contents,												NULL)
 	PHP_FE(fgetcsv,															NULL)
+	PHP_FE(fputcsv,															NULL)
 	PHP_FE(flock,											 third_arg_force_ref)
 	PHP_FE(get_meta_tags,													NULL)
 	PHP_FE(stream_set_write_buffer,											NULL)
@@ -604,6 +614,8 @@ function_entry basic_functions[] = {
 	PHP_FE(stream_get_line,												NULL)
 	PHP_FE(stream_wrapper_register,											NULL)
 	PHP_FALIAS(stream_register_wrapper, stream_wrapper_register,			NULL)
+	PHP_FE(stream_wrapper_unregister,										NULL)
+	PHP_FE(stream_wrapper_restore,											NULL)
 	PHP_FE(stream_get_wrappers,												NULL)
 	PHP_FE(stream_get_transports,											NULL)
 	PHP_FE(get_headers,													NULL)
@@ -676,8 +688,16 @@ function_entry basic_functions[] = {
 	PHP_FE(is_link,															NULL)
 	PHP_NAMED_FE(stat,				php_if_stat,							NULL)
 	PHP_NAMED_FE(lstat,				php_if_lstat,							NULL)
+#ifndef NETWARE
 	PHP_FE(chown,															NULL)
 	PHP_FE(chgrp,															NULL)
+#endif
+#if HAVE_LCHOWN
+	PHP_FE(lchown,															NULL)
+#endif
+#if HAVE_LCHOWN
+	PHP_FE(lchgrp,															NULL)
+#endif
 	PHP_FE(chmod,															NULL)
 #if HAVE_UTIME
 	PHP_FE(touch,															NULL)
@@ -742,8 +762,8 @@ function_entry basic_functions[] = {
 	PHP_FE(prev,					first_arg_force_ref)
 	PHP_FE(next,					first_arg_force_ref)
 	PHP_FE(reset,					first_arg_force_ref)
-	PHP_FE(current,					first_arg_force_ref)
-	PHP_FE(key,						first_arg_force_ref)
+	PHP_FE(current,					all_args_prefer_ref)
+	PHP_FE(key,					all_args_prefer_ref)
 	PHP_FE(min,																NULL)
 	PHP_FE(max,																NULL)
 	PHP_FE(in_array,														NULL)
@@ -752,7 +772,7 @@ function_entry basic_functions[] = {
 	PHP_FE(compact,															NULL)
 	PHP_FE(array_fill,														NULL)
 	PHP_FE(range,															NULL)
-	PHP_FE(array_multisort,													NULL)
+	PHP_FE(array_multisort,													all_args_prefer_ref)
 	PHP_FE(array_push,				first_arg_force_ref)
 	PHP_FE(array_pop,				first_arg_force_ref)
 	PHP_FE(array_shift,				first_arg_force_ref)
@@ -772,23 +792,28 @@ function_entry basic_functions[] = {
 	PHP_FE(array_rand,														NULL)
 	PHP_FE(array_unique,													NULL)
 	PHP_FE(array_intersect,													NULL)
+	PHP_FE(array_intersect_key,												NULL)	
+	PHP_FE(array_intersect_ukey,											NULL)
 	PHP_FE(array_uintersect,												NULL)
 	PHP_FE(array_intersect_assoc,											NULL)
 	PHP_FE(array_uintersect_assoc,											NULL)
 	PHP_FE(array_intersect_uassoc,											NULL)
 	PHP_FE(array_uintersect_uassoc,											NULL)
 	PHP_FE(array_diff,														NULL)
+	PHP_FE(array_diff_key,													NULL)
+	PHP_FE(array_diff_ukey,													NULL)
 	PHP_FE(array_udiff,														NULL)
 	PHP_FE(array_diff_assoc,												NULL)
 	PHP_FE(array_udiff_assoc,												NULL)
 	PHP_FE(array_diff_uassoc,												NULL)
 	PHP_FE(array_udiff_uassoc,												NULL)
 	PHP_FE(array_sum,														NULL)
+	PHP_FE(array_product,													NULL)
 	PHP_FE(array_filter,													NULL)
 	PHP_FE(array_map,														NULL)
 	PHP_FE(array_chunk,														NULL)
-	PHP_FE(array_combine,														NULL)
-	PHP_FE(array_key_exists,														NULL)
+	PHP_FE(array_combine,													NULL)
+	PHP_FE(array_key_exists,												NULL)
 
 	/* aliases from array.c */
 	PHP_FALIAS(pos, 				current, 				 first_arg_force_ref)
@@ -817,8 +842,6 @@ function_entry basic_functions[] = {
 
 	PHP_FE(output_add_rewrite_var,											NULL)
 	PHP_FE(output_reset_rewrite_vars,										NULL)
-	PHP_FE(date_sunrise,														NULL)
-	PHP_FE(date_sunset,															NULL)
 
 	{NULL, NULL, NULL}
 };
@@ -856,10 +879,6 @@ static PHP_INI_MH(OnUpdateSafeModeAllowedEnvVars)
 PHP_INI_BEGIN()
 	PHP_INI_ENTRY_EX("safe_mode_protected_env_vars", SAFE_MODE_PROTECTED_ENV_VARS, PHP_INI_SYSTEM, OnUpdateSafeModeProtectedEnvVars, NULL)
 	PHP_INI_ENTRY_EX("safe_mode_allowed_env_vars",   SAFE_MODE_ALLOWED_ENV_VARS,   PHP_INI_SYSTEM, OnUpdateSafeModeAllowedEnvVars,   NULL)
-	PHP_INI_ENTRY("date.default_latitude",           DATE_DEFAULT_LATITUDE,        PHP_INI_ALL, NULL)
-	PHP_INI_ENTRY("date.default_longitude",          DATE_DEFAULT_LONGITUDE,       PHP_INI_ALL, NULL)
-	PHP_INI_ENTRY("date.sunset_zenith",              DATE_SUNSET_ZENITH,           PHP_INI_ALL, NULL)
-	PHP_INI_ENTRY("date.sunrise_zenith",             DATE_SUNRISE_ZENITH,          PHP_INI_ALL, NULL)
 PHP_INI_END()
 
 
@@ -934,8 +953,11 @@ static void basic_globals_ctor(php_basic_globals *basic_globals_p TSRMLS_DC)
 
 	memset(&BG(url_adapt_state), 0, sizeof(BG(url_adapt_state)));
 	memset(&BG(url_adapt_state_ex), 0, sizeof(BG(url_adapt_state_ex)));
-
-	BG(incomplete_class) = php_create_incomplete_class(TSRMLS_C);
+	
+#if defined(_REENTRANT) && defined(HAVE_MBRLEN) && defined(HAVE_MBSTATE_T)
+	memset(&BG(mblen_state), 0, sizeof(BG(mblen_state)));
+#endif
+	BG(incomplete_class) = incomplete_class_entry;
 }
 
 
@@ -997,10 +1019,12 @@ PHP_MINIT_FUNCTION(basic)
 #else
 	basic_globals_ctor(&basic_globals TSRMLS_CC);
 #ifdef PHP_WIN32
-	php_win32_core_globals_ctor(&php_win32_core_globals TSRMLS_CC);
+	php_win32_core_globals_ctor(&the_php_win32_core_globals TSRMLS_CC);
 #endif
 #endif
 
+	BG(incomplete_class) = incomplete_class_entry = php_create_incomplete_class(TSRMLS_C);
+
 	REGISTER_LONG_CONSTANT("CONNECTION_ABORTED", PHP_CONNECTION_ABORTED, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("CONNECTION_NORMAL",  PHP_CONNECTION_NORMAL,  CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("CONNECTION_TIMEOUT", PHP_CONNECTION_TIMEOUT, CONST_CS | CONST_PERSISTENT);
@@ -1010,9 +1034,14 @@ PHP_MINIT_FUNCTION(basic)
 	REGISTER_LONG_CONSTANT("INI_SYSTEM", ZEND_INI_SYSTEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("INI_ALL",    ZEND_INI_ALL,    CONST_CS | CONST_PERSISTENT);
 
-	REGISTER_LONG_CONSTANT("SUNFUNCS_RET_TIMESTAMP", SUNFUNCS_RET_TIMESTAMP, CONST_CS | CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("SUNFUNCS_RET_STRING", SUNFUNCS_RET_STRING, CONST_CS | CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("SUNFUNCS_RET_DOUBLE", SUNFUNCS_RET_DOUBLE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_SCHEME", PHP_URL_SCHEME, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_HOST", PHP_URL_HOST, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_PORT", PHP_URL_PORT, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_USER", PHP_URL_USER, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_PASS", PHP_URL_PASS, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_PATH", PHP_URL_PATH, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_QUERY", PHP_URL_QUERY, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PHP_URL_FRAGMENT", PHP_URL_FRAGMENT, CONST_CS | CONST_PERSISTENT);
 
 #define REGISTER_MATH_CONSTANT(x)  REGISTER_DOUBLE_CONSTANT(#x, x, CONST_CS | CONST_PERSISTENT)
 	REGISTER_MATH_CONSTANT(M_E);
@@ -1194,7 +1223,7 @@ PHP_RSHUTDOWN_FUNCTION(basic)
 	 FG(stream_wrappers) and FG(stream_filters) are destroyed
 	 during php_request_shutdown()
 	 */
-
+	
 	PHP_RSHUTDOWN(filestat)(SHUTDOWN_FUNC_ARGS_PASSTHRU);
 #ifdef HAVE_SYSLOG_H
 	PHP_RSHUTDOWN(syslog)(SHUTDOWN_FUNC_ARGS_PASSTHRU);
@@ -1248,6 +1277,79 @@ PHP_FUNCTION(constant)
 }
 /* }}} */
 
+#ifdef HAVE_INET_NTOP
+/* {{{ proto string inet_ntop(string in_addr)
+   Converts a packed inet address to a human readable IP address string */
+PHP_NAMED_FUNCTION(php_inet_ntop)
+{
+	char *address;
+	int address_len, af = AF_INET;
+	char buffer[40];
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &address, &address_len) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+#ifdef HAVE_IPV6
+	if (address_len == 16) {
+		af = AF_INET6;
+	} else
+#endif
+	if (address_len != 4) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid in_addr value");
+		RETURN_FALSE;
+	}
+
+	if (!inet_ntop(af, address, buffer, sizeof(buffer))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "An unknown error occured");
+		RETURN_FALSE;
+	}
+
+	RETURN_STRING(buffer, 1);
+}
+/* }}} */
+#endif /* HAVE_INET_NTOP */
+
+#ifdef HAVE_INET_PTON
+/* {{{ proto string inet_pton(string ip_address)
+   Converts a human readable IP address to a packed binary string */
+PHP_NAMED_FUNCTION(php_inet_pton)
+{
+	int ret, af = AF_INET;
+	char *address;
+	int address_len;
+	char buffer[17];
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &address, &address_len) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	memset(buffer, 0, sizeof(buffer));
+
+#ifdef HAVE_IPV6
+	if (strchr(address, ':')) {
+		af = AF_INET6;
+	} else 
+#endif
+	if (!strchr(address, '.')) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unrecognized address %s", address);
+		RETURN_FALSE;
+	}
+
+	ret = inet_pton(af, address, buffer);
+
+	if (ret <= 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unrecognized address %s", address);
+		RETURN_FALSE;
+	}
+
+	RETURN_STRINGL(buffer, af == AF_INET ? 4 : 16, 1);
+}
+/* }}} */
+#endif /* HAVE_INET_PTON */
+
+
+
 /* {{{ proto int ip2long(string ip_address)
    Converts a string containing an (IPv4) Internet Protocol dotted address into a proper address */
 PHP_FUNCTION(ip2long)
@@ -1327,7 +1429,7 @@ PHP_FUNCTION(getenv)
    Set the value of an environment variable */
 PHP_FUNCTION(putenv)
 {
-	pval **str;
+	zval **str;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &str) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -1462,6 +1564,7 @@ PHP_FUNCTION(getopt)
 	char *optname;
 	int argc = 0, options_len = 0, o;
 	zval *val, **args = NULL, *p_longopts = NULL;
+	int optname_len = 0;
 #ifdef HARTMUT_0
 	struct option *longopts = NULL;
 	int longindex = 0;
@@ -1559,6 +1662,9 @@ PHP_FUNCTION(getopt)
 	/* Disable getopt()'s error messages. */
 	opterr = 0;
 
+	/* Force reinitialization of getopt() (via optind reset) on every call. */
+	optind = 0;
+
 	/* Invoke getopt(3) on the argument array. */
 #ifdef HARTMUT_0
 	while ((o = getopt_long(argc, argv, options, longopts, &longindex)) != -1) {
@@ -1592,14 +1698,28 @@ PHP_FUNCTION(getopt)
 		}
 
 		/* Add this option / argument pair to the result hash. */
-		if(zend_hash_find(HASH_OF(return_value), optname, strlen(optname)+1, (void **)&args) != FAILURE) {
-			if(Z_TYPE_PP(args) != IS_ARRAY) {
-				convert_to_array_ex(args);
-			} 
- 			zend_hash_next_index_insert(HASH_OF(*args),  (void *)&val, sizeof(zval *), NULL);
+		optname_len = strlen(optname);
+		if (!(optname_len > 1 && optname[0] == '0') && is_numeric_string(optname, optname_len, NULL, NULL, 0) == IS_LONG) {
+			/* numeric string */
+			int optname_int = atoi(optname);
+			if(zend_hash_index_find(HASH_OF(return_value), optname_int, (void **)&args) != FAILURE) {
+				if(Z_TYPE_PP(args) != IS_ARRAY) {
+					convert_to_array_ex(args);
+				} 
+				zend_hash_next_index_insert(HASH_OF(*args),  (void *)&val, sizeof(zval *), NULL);
+			} else {
+				zend_hash_index_update(HASH_OF(return_value), optname_int, &val, sizeof(zval *), NULL);
+			}
 		} else {
-			zend_hash_add(HASH_OF(return_value), optname, strlen(optname)+1, (void *)&val,
-						  sizeof(zval *), NULL);
+			/* other strings */
+			if(zend_hash_find(HASH_OF(return_value), optname, strlen(optname)+1, (void **)&args) != FAILURE) {
+				if(Z_TYPE_PP(args) != IS_ARRAY) {
+					convert_to_array_ex(args);
+				} 
+				zend_hash_next_index_insert(HASH_OF(*args),  (void *)&val, sizeof(zval *), NULL);
+			} else {
+				zend_hash_add(HASH_OF(return_value), optname, strlen(optname)+1, (void *)&val, sizeof(zval *), NULL);
+			}
 		}
 	}
 
@@ -1623,7 +1743,7 @@ PHP_FUNCTION(flush)
    Delay for a given number of seconds */
 PHP_FUNCTION(sleep)
 {
-	pval **num;
+	zval **num;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &num) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -1635,6 +1755,7 @@ PHP_FUNCTION(sleep)
 #else
 	php_sleep(Z_LVAL_PP(num));
 #endif
+
 }
 /* }}} */
 
@@ -1643,7 +1764,7 @@ PHP_FUNCTION(sleep)
 PHP_FUNCTION(usleep)
 {
 #if HAVE_USLEEP
-	pval **num;
+	zval **num;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &num) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -1682,6 +1803,48 @@ PHP_FUNCTION(time_nanosleep)
 	RETURN_FALSE;
 }
 /* }}} */
+
+/* {{{ proto mixed time_sleep_until(float timestamp)
+   Make the script sleep until the specified time */
+PHP_FUNCTION(time_sleep_until)
+{
+	double d_ts, c_ts;
+	struct timeval tm;
+	struct timespec php_req, php_rem;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &d_ts)) {
+		WRONG_PARAM_COUNT;
+	}
+
+	if (gettimeofday((struct timeval *) &tm, NULL) != 0) {
+		RETURN_FALSE;
+	}
+
+	c_ts = (double)(d_ts - tm.tv_sec - tm.tv_usec / 1000000.00);
+	if (c_ts < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Sleep until to time is less then current time.");
+		RETURN_FALSE;
+	}
+
+	php_req.tv_sec = (time_t) c_ts;
+	if (php_req.tv_sec > c_ts) { /* rounding up occurred */
+		php_req.tv_sec--;
+	}
+	/* 1sec = 1000000000 nanoseconds */
+	php_req.tv_nsec = (long) ((c_ts - php_req.tv_sec) * 1000000000.00);
+
+	while (nanosleep(&php_req, &php_rem)) {
+		if (errno == EINTR) {
+			php_req.tv_sec = php_rem.tv_sec;
+			php_req.tv_nsec = php_rem.tv_nsec;
+		} else {
+			RETURN_FALSE;
+		}
+	}
+
+	RETURN_TRUE;
+}
+/* }}} */
 #endif
 
 /* {{{ proto string get_current_user(void)
@@ -1700,7 +1863,7 @@ PHP_FUNCTION(get_current_user)
    Get the value of a PHP configuration option */
 PHP_FUNCTION(get_cfg_var)
 {
-	pval **varname;
+	zval **varname;
 	char *value;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &varname) == FAILURE) {
@@ -1720,7 +1883,7 @@ PHP_FUNCTION(get_cfg_var)
    Set the current active configuration setting of magic_quotes_runtime and return previous */
 PHP_FUNCTION(set_magic_quotes_runtime)
 {
-	pval **new_setting;
+	zval **new_setting;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &new_setting) == FAILURE) {
 		RETURN_FALSE;
@@ -1766,7 +1929,7 @@ error options:
    Send an error message somewhere */
 PHP_FUNCTION(error_log)
 {
-	pval **string, **erropt = NULL, **option = NULL, **emailhead = NULL;
+	zval **string, **erropt = NULL, **option = NULL, **emailhead = NULL;
 	int opt_err = 0;
 	char *message, *opt = NULL, *headers = NULL;
 
@@ -1890,7 +2053,7 @@ PHP_FUNCTION(call_user_func)
 
 	params = safe_emalloc(sizeof(zval **), argc, 0);
 
-	if (zend_get_parameters_array_ex(argc, params) == FAILURE) {
+	if (zend_get_parameters_array_ex(1, params) == FAILURE) {
 		efree(params);
 		RETURN_FALSE;
 	}
@@ -1900,13 +2063,18 @@ PHP_FUNCTION(call_user_func)
 		convert_to_string_ex(params[0]);
 	}
 
-	if (!zend_is_callable(*params[0], IS_CALLABLE_CHECK_NO_ACCESS, &name)) {
+	if (!zend_is_callable(*params[0], 0, &name)) {
 		php_error_docref1(NULL TSRMLS_CC, name, E_WARNING, "First argument is expected to be a valid callback");
 		efree(name);
 		efree(params);
 		RETURN_NULL();
 	}
 
+	if (zend_get_parameters_array_ex(argc, params) == FAILURE) {
+		efree(params);
+		RETURN_FALSE;
+	}
+
 	if (call_user_function_ex(EG(function_table), NULL, *params[0], &retval_ptr, argc-1, params+1, 0, NULL TSRMLS_CC) == SUCCESS) {
 		if (retval_ptr) {
 			COPY_PZVAL_TO_ZVAL(*return_value, retval_ptr);
@@ -1955,7 +2123,7 @@ PHP_FUNCTION(call_user_func_array)
 		convert_to_string_ex(func);
 	}
 
-	if (!zend_is_callable(*func, IS_CALLABLE_CHECK_NO_ACCESS, &name)) {
+	if (!zend_is_callable(*func, 0, &name)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "First argument is expected to be a valid callback, '%s' was given", name);
 		efree(name);
 		RETURN_NULL();
@@ -1964,13 +2132,17 @@ PHP_FUNCTION(call_user_func_array)
 	func_params_ht = Z_ARRVAL_PP(params);
 
 	count = zend_hash_num_elements(func_params_ht);
-	func_params = safe_emalloc(sizeof(zval **), count, 0);
+	if (count) {
+		func_params = safe_emalloc(sizeof(zval **), count, 0);
 
-	for (zend_hash_internal_pointer_reset(func_params_ht);
-		 zend_hash_get_current_data(func_params_ht, (void **) &func_params[current]) == SUCCESS;
-		 zend_hash_move_forward(func_params_ht)
-		) {
-		current++;
+		for (zend_hash_internal_pointer_reset(func_params_ht);
+				zend_hash_get_current_data(func_params_ht, (void **) &func_params[current]) == SUCCESS;
+				zend_hash_move_forward(func_params_ht)
+			) {
+			current++;
+		}
+	} else {
+		func_params = NULL;
 	}
 
 	if (call_user_function_ex(EG(function_table), NULL, *func, &retval_ptr, count, func_params, 0, NULL TSRMLS_CC) == SUCCESS) {
@@ -1982,7 +2154,9 @@ PHP_FUNCTION(call_user_func_array)
 	}
 
 	efree(name);
-	efree(func_params);
+	if (func_params) {
+		efree(func_params);
+	}
 }
 /* }}} */
 
@@ -2217,7 +2391,7 @@ PHP_FUNCTION(register_shutdown_function)
 	}
 	
 	/* Prevent entering of anything but valid callback (syntax check only!) */
-	if (!zend_is_callable(shutdown_function_entry.arguments[0], 1, &function_name)) {
+	if (!zend_is_callable(shutdown_function_entry.arguments[0], 0, &function_name)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid shutdown callback '%s' passed", function_name);
 		efree(shutdown_function_entry.arguments);
 		RETVAL_FALSE;
@@ -2370,7 +2544,7 @@ PHP_FUNCTION(highlight_string)
    Get a configuration option */
 PHP_FUNCTION(ini_get)
 {
-	pval **varname;
+	zval **varname;
 	char *str;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &varname) == FAILURE) {
@@ -2466,7 +2640,7 @@ static int php_ini_check_path(char *option_name, int option_len, char *new_optio
    Set a configuration option, returns false on error and the old value of the configuration option on success */
 PHP_FUNCTION(ini_set)
 {
-	pval **varname, **new_value;
+	zval **varname, **new_value;
 	char *old_value;
 
 	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &varname, &new_value) == FAILURE) {
@@ -2529,7 +2703,7 @@ PHP_FUNCTION(ini_set)
    Restore the value of a configuration option specified by varname */
 PHP_FUNCTION(ini_restore)
 {
-	pval **varname;
+	zval **varname;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &varname) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -2546,7 +2720,7 @@ PHP_FUNCTION(ini_restore)
 
 PHP_FUNCTION(set_include_path)
 {
-	pval **new_value;
+	zval **new_value;
 	char *old_value;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &new_value) == FAILURE) {
@@ -2651,7 +2825,7 @@ PHP_FUNCTION(connection_status)
    Set whether we want to ignore a user abort event or not */
 PHP_FUNCTION(ignore_user_abort)
 {
-	pval **arg;
+	zval **arg;
 	int old_setting;
 
 	old_setting = PG(ignore_user_abort);
@@ -2681,7 +2855,7 @@ PHP_FUNCTION(ignore_user_abort)
    Returns port associated with service. Protocol must be "tcp" or "udp" */
 PHP_FUNCTION(getservbyname)
 {
-	pval **name, **proto;
+	zval **name, **proto;
 	struct servent *serv;
 
 	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &name, &proto) == FAILURE) {
@@ -2706,7 +2880,7 @@ PHP_FUNCTION(getservbyname)
    Returns service name associated with port. Protocol must be "tcp" or "udp" */
 PHP_FUNCTION(getservbyport)
 {
-	pval **port, **proto;
+	zval **port, **proto;
 	struct servent *serv;
 
 	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &port, &proto) == FAILURE) {
@@ -2731,7 +2905,7 @@ PHP_FUNCTION(getservbyport)
    Returns protocol number associated with name as per /etc/protocols */
 PHP_FUNCTION(getprotobyname)
 {
-	pval **name;
+	zval **name;
 	struct protoent *ent;
 
 	if (ZEND_NUM_ARGS() != 1
@@ -2759,7 +2933,7 @@ PHP_FUNCTION(getprotobyname)
    Returns protocol name associated with protocol number proto */
 PHP_FUNCTION(getprotobynumber)
 {
-	pval **proto;
+	zval **proto;
 	struct protoent *ent;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &proto) == FAILURE) {
@@ -3076,13 +3250,27 @@ static int copy_request_variable(void *pDest, int num_args, va_list args, zend_h
 	prefix = va_arg(args, char *);
 	prefix_len = va_arg(args, uint);
 
-	new_key_len = prefix_len + hash_key->nKeyLength;
-	new_key = (char *) emalloc(new_key_len);
+	if (!prefix_len) {
+		if (!hash_key->nKeyLength) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Numeric key detected - possible security hazard.");
+			return 0;
+		} else if (!strcmp(hash_key->arKey, "GLOBALS")) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Attempted GLOBALS variable overwrite.");
+			return 0; 
+		}
+	}
 
-	memcpy(new_key, prefix, prefix_len);
-	memcpy(new_key+prefix_len, hash_key->arKey, hash_key->nKeyLength);
+	if (hash_key->nKeyLength) {
+		new_key_len = prefix_len + hash_key->nKeyLength;
+		new_key = (char *) emalloc(new_key_len);
 
-	zend_hash_del(&EG(symbol_table), new_key, new_key_len);
+		memcpy(new_key, prefix, prefix_len);
+		memcpy(new_key+prefix_len, hash_key->arKey, hash_key->nKeyLength);
+	} else {
+		new_key_len = spprintf(&new_key, 0, "%s%ld", prefix, hash_key->h);
+	}
+
+	zend_delete_global_variable(new_key, new_key_len-1 TSRMLS_CC);
 	ZEND_SET_SYMBOL_WITH_LENGTH(&EG(symbol_table), new_key, new_key_len, *var, (*var)->refcount+1, 0);
 
 	efree(new_key);
