@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
+/* $Id: dir.c,v 1.147.2.2 2006/01/01 12:50:14 sniper Exp $ */
 
 /* {{{ includes/startup/misc */
 
@@ -24,9 +24,7 @@
 #include "fopen_wrappers.h"
 #include "file.h"
 #include "php_dir.h"
-#include "php_string.h"
 #include "php_scandir.h"
-#include "basic_functions.h"
 
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
@@ -75,9 +73,6 @@ static int le_dirp;
 static zend_class_entry *dir_class_entry_ptr;
 
 #define FETCH_DIRP() \
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &id) == FAILURE) { \
-		return; \
-	} \
 	if (ZEND_NUM_ARGS() == 0) { \
 		myself = getThis(); \
 		if (myself) { \
@@ -89,22 +84,18 @@ static zend_class_entry *dir_class_entry_ptr;
 		} else { \
 			ZEND_FETCH_RESOURCE(dirp, php_stream *, 0, DIRG(default_dir), "Directory", php_file_le_stream()); \
 		} \
+	} else if ((ZEND_NUM_ARGS() != 1) || zend_get_parameters_ex(1, &id) == FAILURE) { \
+		WRONG_PARAM_COUNT; \
 	} else { \
-		dirp = (php_stream *) zend_fetch_resource(&id TSRMLS_CC, -1, "Directory", NULL, 1, php_file_le_stream()); \
+		dirp = (php_stream *) zend_fetch_resource(id TSRMLS_CC, -1, "Directory", NULL, 1, php_file_le_stream()); \
 		if (!dirp) \
 			RETURN_FALSE; \
-	}
+	} 
 
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dir, 0, 0, 0)
-	ZEND_ARG_INFO(0, dir_handle)
-ZEND_END_ARG_INFO()
-/* }}} */
-
-static const zend_function_entry php_dir_class_functions[] = {
-	PHP_FALIAS(close,	closedir,		arginfo_dir)
-	PHP_FALIAS(rewind,	rewinddir,		arginfo_dir)
-	PHP_NAMED_FE(read,  php_if_readdir, arginfo_dir)
+static zend_function_entry php_dir_class_functions[] = {
+	PHP_FALIAS(close,	closedir,	NULL)
+	PHP_FALIAS(rewind,	rewinddir,	NULL)
+	PHP_NAMED_FE(read,  php_if_readdir, NULL)
 	{NULL, NULL, NULL}
 };
 
@@ -118,7 +109,7 @@ static void php_set_default_dir(int id TSRMLS_DC)
 	if (id != -1) {
 		zend_list_addref(id);
 	}
-
+	
 	DIRG(default_dir) = id;
 }
 
@@ -148,61 +139,35 @@ PHP_MINIT_FUNCTION(dir)
 	pathsep_str[1] = '\0';
 	REGISTER_STRING_CONSTANT("PATH_SEPARATOR", pathsep_str, CONST_CS|CONST_PERSISTENT);
 
-	REGISTER_LONG_CONSTANT("SCANDIR_SORT_ASCENDING",  PHP_SCANDIR_SORT_ASCENDING,  CONST_CS | CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("SCANDIR_SORT_DESCENDING", PHP_SCANDIR_SORT_DESCENDING, CONST_CS | CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("SCANDIR_SORT_NONE",       PHP_SCANDIR_SORT_NONE,       CONST_CS | CONST_PERSISTENT);
-
 #ifdef HAVE_GLOB
-
 #ifdef GLOB_BRACE
 	REGISTER_LONG_CONSTANT("GLOB_BRACE", GLOB_BRACE, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_BRACE 0
 #endif
-
 #ifdef GLOB_MARK
 	REGISTER_LONG_CONSTANT("GLOB_MARK", GLOB_MARK, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_MARK 0
 #endif
-
 #ifdef GLOB_NOSORT
 	REGISTER_LONG_CONSTANT("GLOB_NOSORT", GLOB_NOSORT, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_NOSORT 0
 #endif
-
 #ifdef GLOB_NOCHECK
 	REGISTER_LONG_CONSTANT("GLOB_NOCHECK", GLOB_NOCHECK, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_NOCHECK 0
 #endif
-
 #ifdef GLOB_NOESCAPE
 	REGISTER_LONG_CONSTANT("GLOB_NOESCAPE", GLOB_NOESCAPE, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_NOESCAPE 0
 #endif
-
 #ifdef GLOB_ERR
 	REGISTER_LONG_CONSTANT("GLOB_ERR", GLOB_ERR, CONST_CS | CONST_PERSISTENT);
-#else
-# define GLOB_ERR 0
 #endif
 
 #ifndef GLOB_ONLYDIR
-# define GLOB_ONLYDIR (1<<30)
-# define GLOB_EMULATE_ONLYDIR
-# define GLOB_FLAGMASK (~GLOB_ONLYDIR)
+#define GLOB_ONLYDIR (1<<30)
+#define GLOB_EMULATE_ONLYDIR
+#define GLOB_FLAGMASK (~GLOB_ONLYDIR)
 #else
-# define GLOB_FLAGMASK (~0)
+#define GLOB_FLAGMASK (~0)
 #endif
 
-/* This is used for checking validity of passed flags (passing invalid flags causes segfault in glob()!! */
-#define GLOB_AVAILABLE_FLAGS (0 | GLOB_BRACE | GLOB_MARK | GLOB_NOSORT | GLOB_NOCHECK | GLOB_NOESCAPE | GLOB_ERR | GLOB_ONLYDIR)
-
 	REGISTER_LONG_CONSTANT("GLOB_ONLYDIR", GLOB_ONLYDIR, CONST_CS | CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("GLOB_AVAILABLE_FLAGS", GLOB_AVAILABLE_FLAGS, CONST_CS | CONST_PERSISTENT);
 
 #endif /* HAVE_GLOB */
 
@@ -219,20 +184,20 @@ static void _php_do_opendir(INTERNAL_FUNCTION_PARAMETERS, int createobject)
 	php_stream_context *context = NULL;
 	php_stream *dirp;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|r", &dirname, &dir_len, &zcontext) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|r", &dirname, &dir_len, &zcontext) == FAILURE) {
 		RETURN_NULL();
 	}
 
-	context = php_stream_context_from_zval(zcontext, 0);
-
-	dirp = php_stream_opendir(dirname, REPORT_ERRORS, context);
+	if (zcontext) {
+		context = php_stream_context_from_zval(zcontext, 0);
+	}
+	
+	dirp = php_stream_opendir(dirname, ENFORCE_SAFE_MODE|REPORT_ERRORS, context);
 
 	if (dirp == NULL) {
 		RETURN_FALSE;
 	}
-
-	dirp->flags |= PHP_STREAM_FLAG_NO_FCLOSE;
-
+		
 	php_set_default_dir(dirp->rsrc_id TSRMLS_CC);
 
 	if (createobject) {
@@ -266,23 +231,16 @@ PHP_FUNCTION(getdir)
    Close directory connection identified by the dir_handle */
 PHP_FUNCTION(closedir)
 {
-	zval *id = NULL, **tmp, *myself;
+	zval **id, **tmp, *myself;
 	php_stream *dirp;
-	int rsrc_id;
 
 	FETCH_DIRP();
 
-	if (!(dirp->flags & PHP_STREAM_FLAG_IS_DIR)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%d is not a valid Directory resource", dirp->rsrc_id);
-		RETURN_FALSE;
-	}
-
-	rsrc_id = dirp->rsrc_id;
-	zend_list_delete(dirp->rsrc_id);
-
-	if (rsrc_id == DIRG(default_dir)) {
+	if (dirp->rsrc_id == DIRG(default_dir)) {
 		php_set_default_dir(-1 TSRMLS_CC);
 	}
+
+	zend_list_delete(dirp->rsrc_id);
 }
 /* }}} */
 
@@ -293,21 +251,20 @@ PHP_FUNCTION(chroot)
 {
 	char *str;
 	int ret, str_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p", &str, &str_len) == FAILURE) {
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
 		RETURN_FALSE;
 	}
-
+	
 	ret = chroot(str);
+	
 	if (ret != 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s (errno %d)", strerror(errno), errno);
 		RETURN_FALSE;
 	}
 
-	php_clear_stat_cache(1, NULL, 0 TSRMLS_CC);
-
 	ret = chdir("/");
-
+	
 	if (ret != 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s (errno %d)", strerror(errno), errno);
 		RETURN_FALSE;
@@ -324,30 +281,21 @@ PHP_FUNCTION(chdir)
 {
 	char *str;
 	int ret, str_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p", &str, &str_len) == FAILURE) {
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(str TSRMLS_CC)) {
+	if (PG(safe_mode) && !php_checkuid(str, NULL, CHECKUID_CHECK_FILE_AND_DIR)) {
 		RETURN_FALSE;
 	}
 	ret = VCWD_CHDIR(str);
-
+	
 	if (ret != 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s (errno %d)", strerror(errno), errno);
 		RETURN_FALSE;
 	}
 
-	if (BG(CurrentStatFile) && !IS_ABSOLUTE_PATH(BG(CurrentStatFile), strlen(BG(CurrentStatFile)))) {
-		efree(BG(CurrentStatFile));
-		BG(CurrentStatFile) = NULL;
-	}
-	if (BG(CurrentLStatFile) && !IS_ABSOLUTE_PATH(BG(CurrentLStatFile), strlen(BG(CurrentLStatFile)))) {
-		efree(BG(CurrentLStatFile));
-		BG(CurrentLStatFile) = NULL;
-	}
-
 	RETURN_TRUE;
 }
 /* }}} */
@@ -358,9 +306,9 @@ PHP_FUNCTION(getcwd)
 {
 	char path[MAXPATHLEN];
 	char *ret=NULL;
-
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	
+	if (ZEND_NUM_ARGS() != 0) {
+		WRONG_PARAM_COUNT;
 	}
 
 #if HAVE_GETCWD
@@ -381,16 +329,11 @@ PHP_FUNCTION(getcwd)
    Rewind dir_handle back to the start */
 PHP_FUNCTION(rewinddir)
 {
-	zval *id = NULL, **tmp, *myself;
+	zval **id, **tmp, *myself;
 	php_stream *dirp;
-
+	
 	FETCH_DIRP();
 
-	if (!(dirp->flags & PHP_STREAM_FLAG_IS_DIR)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%d is not a valid Directory resource", dirp->rsrc_id);
-		RETURN_FALSE;
-	}
-
 	php_stream_rewinddir(dirp);
 }
 /* }}} */
@@ -399,17 +342,12 @@ PHP_FUNCTION(rewinddir)
    Read directory entry from dir_handle */
 PHP_NAMED_FUNCTION(php_if_readdir)
 {
-	zval *id = NULL, **tmp, *myself;
+	zval **id, **tmp, *myself;
 	php_stream *dirp;
 	php_stream_dirent entry;
 
 	FETCH_DIRP();
 
-	if (!(dirp->flags & PHP_STREAM_FLAG_IS_DIR)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%d is not a valid Directory resource", dirp->rsrc_id);
-		RETURN_FALSE;
-	}
-
 	if (php_stream_readdir(dirp, &entry)) {
 		RETURN_STRINGL(entry.d_name, strlen(entry.d_name), 1);
 	}
@@ -422,9 +360,9 @@ PHP_NAMED_FUNCTION(php_if_readdir)
    Find pathnames matching a pattern */
 PHP_FUNCTION(glob)
 {
+	char cwd[MAXPATHLEN];
 	int cwd_skip = 0;
 #ifdef ZTS
-	char cwd[MAXPATHLEN];
 	char work_pattern[MAXPATHLEN];
 	char *result;
 #endif
@@ -432,27 +370,15 @@ PHP_FUNCTION(glob)
 	int pattern_len;
 	long flags = 0;
 	glob_t globbuf;
-	int n;
+	unsigned int n;
 	int ret;
-	zend_bool basedir_limit = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|l", &pattern, &pattern_len, &flags) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &pattern, &pattern_len, &flags) == FAILURE) 
 		return;
-	}
 
-	if (pattern_len >= MAXPATHLEN) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Pattern exceeds the maximum allowed length of %d characters", MAXPATHLEN);
-		RETURN_FALSE;
-	}
-
-	if ((GLOB_AVAILABLE_FLAGS & flags) != flags) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "At least one of the passed flags is invalid or not supported on this platform");
-		RETURN_FALSE;
-	}
-
-#ifdef ZTS
+#ifdef ZTS 
 	if (!IS_ABSOLUTE_PATH(pattern, pattern_len)) {
-		result = VCWD_GETCWD(cwd, MAXPATHLEN);
+		result = VCWD_GETCWD(cwd, MAXPATHLEN);	
 		if (!result) {
 			cwd[0] = '\0';
 		}
@@ -465,11 +391,9 @@ PHP_FUNCTION(glob)
 
 		snprintf(work_pattern, MAXPATHLEN, "%s%c%s", cwd, DEFAULT_SLASH, pattern);
 		pattern = work_pattern;
-	}
+	} 
 #endif
 
-
-	memset(&globbuf, 0, sizeof(glob_t));
 	globbuf.gl_offs = 0;
 	if (0 != (ret = glob(pattern, flags & GLOB_FLAGMASK, NULL, &globbuf))) {
 #ifdef GLOB_NOMATCH
@@ -477,12 +401,13 @@ PHP_FUNCTION(glob)
 			/* Some glob implementation simply return no data if no matches
 			   were found, others return the GLOB_NOMATCH error code.
 			   We don't want to treat GLOB_NOMATCH as an error condition
-			   so that PHP glob() behaves the same on both types of
+			   so that PHP glob() behaves the same on both types of 
 			   implementations and so that 'foreach (glob() as ...'
 			   can be used for simple glob() calls without further error
 			   checking.
 			*/
-			goto no_results;
+			array_init(return_value);
+			return;
 		}
 #endif
 		RETURN_FALSE;
@@ -490,38 +415,29 @@ PHP_FUNCTION(glob)
 
 	/* now catch the FreeBSD style of "no matches" */
 	if (!globbuf.gl_pathc || !globbuf.gl_pathv) {
-no_results:
-#ifndef PHP_WIN32
-		/* Paths containing '*', '?' and some other chars are
-		illegal on Windows but legit on other platforms. For
-		this reason the direct basedir check against the glob
-		query is senseless on windows. For instance while *.txt
-		is a pretty valid filename on EXT3, it's invalid on NTFS. */
-		if (PG(open_basedir) && *PG(open_basedir)) {
-			if (php_check_open_basedir_ex(pattern, 0 TSRMLS_CC)) {
-				RETURN_FALSE;
-			}
-		}
-#endif
 		array_init(return_value);
 		return;
 	}
 
+	/* we assume that any glob pattern will match files from one directory only
+	   so checking the dirname of the first match should be sufficient */
+	strncpy(cwd, globbuf.gl_pathv[0], MAXPATHLEN);
+	if (PG(safe_mode) && (!php_checkuid(cwd, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+		RETURN_FALSE;
+	}
+	if (php_check_open_basedir(cwd TSRMLS_CC)) {
+		RETURN_FALSE;
+	}
+
 	array_init(return_value);
 	for (n = 0; n < globbuf.gl_pathc; n++) {
-		if (PG(open_basedir) && *PG(open_basedir)) {
-			if (php_check_open_basedir_ex(globbuf.gl_pathv[n], 0 TSRMLS_CC)) {
-				basedir_limit = 1;
-				continue;
-			}
-		}
 		/* we need to do this everytime since GLOB_ONLYDIR does not guarantee that
 		 * all directories will be filtered. GNU libc documentation states the
-		 * following:
-		 * If the information about the type of the file is easily available
-		 * non-directories will be rejected but no extra work will be done to
-		 * determine the information for each file. I.e., the caller must still be
-		 * able to filter directories out.
+		 * following: 
+		 * If the information about the type of the file is easily available 
+		 * non-directories will be rejected but no extra work will be done to 
+		 * determine the information for each file. I.e., the caller must still be 
+		 * able to filter directories out. 
 		 */
 		if (flags & GLOB_ONLYDIR) {
 			struct stat s;
@@ -538,14 +454,9 @@ no_results:
 	}
 
 	globfree(&globbuf);
-
-	if (basedir_limit && !zend_hash_num_elements(Z_ARRVAL_P(return_value))) {
-		zval_dtor(return_value);
-		RETURN_FALSE;
-	}
 }
 /* }}} */
-#endif
+#endif 
 
 /* {{{ proto array scandir(string dir [, int sorting_order [, resource context]])
    List files & directories inside the specified path */
@@ -559,23 +470,16 @@ PHP_FUNCTION(scandir)
 	zval *zcontext = NULL;
 	php_stream_context *context = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|lr", &dirn, &dirn_len, &flags, &zcontext) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lr", &dirn, &dirn_len, &flags, &zcontext) == FAILURE) {
 		return;
 	}
 
-	if (dirn_len < 1) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Directory name cannot be empty");
-		RETURN_FALSE;
-	}
-
 	if (zcontext) {
 		context = php_stream_context_from_zval(zcontext, 0);
 	}
 
-	if (flags == PHP_SCANDIR_SORT_ASCENDING) {
+	if (!flags) {
 		n = php_stream_scandir(dirn, &namelist, context, (void *) php_stream_dirent_alphasort);
-	} else if (flags == PHP_SCANDIR_SORT_NONE) {
-		n = php_stream_scandir(dirn, &namelist, context, NULL);
 	} else {
 		n = php_stream_scandir(dirn, &namelist, context, (void *) php_stream_dirent_alphasortr);
 	}
@@ -583,7 +487,7 @@ PHP_FUNCTION(scandir)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "(errno %d): %s", errno, strerror(errno));
 		RETURN_FALSE;
 	}
-
+	
 	array_init(return_value);
 
 	for (i = 0; i < n; i++) {
