@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    |         Ilia Alshanetsky <iliaa@php.net>                             |
    +----------------------------------------------------------------------+
  */
-/* $Id: exec.c,v 1.113.2.3.2.1.2.17 2009/04/30 15:25:37 pajoye Exp $ */
+/* $Id: exec.c,v 1.113.2.2 2006/01/01 12:50:14 sniper Exp $ */
 
 #include <stdio.h>
 #include "php.h"
@@ -25,7 +25,6 @@
 #include "safe_mode.h"
 #include "ext/standard/head.h"
 #include "ext/standard/file.h"
-#include "basic_functions.h"
 #include "exec.h"
 #include "php_globals.h"
 #include "SAPI.h"
@@ -58,7 +57,7 @@
  * If type==3, output will be printed binary, no lines will be saved or returned (passthru)
  *
  */
-PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_DC)
+int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_DC)
 {
 	FILE *fp;
 	char *buf, *tmp=NULL;
@@ -79,16 +78,7 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "No '..' components allowed in path");
 			goto err;
 		}
-		
 		b = strrchr(cmd, PHP_DIR_SEPARATOR);
-
-#ifdef PHP_WIN32
-		if (b && *b == '\\' && b == cmd) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid absolute path.");
-			goto err;
-		}
-#endif
-
 		spprintf(&d, 0, "%s%s%s%s%s", PG(safe_mode_exec_dir), (b ? "" : "/"), (b ? b : cmd), (c ? " " : ""), (c ? c : ""));
 		if (c) {
 			*(c - 1) = ' ';
@@ -121,7 +111,7 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 
 	if (type != 3) {
 		b = buf;
-
+		
 		while (php_stream_get_line(stream, b, EXEC_INPUT_BUF, &bufl)) {
 			/* no new line found, let's read some more */
 			if (b[bufl - 1] != '\n' && !php_stream_eof(stream)) {
@@ -140,14 +130,12 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 
 			if (type == 1) {
 				PHPWRITE(buf, bufl);
-				if (OG(ob_nesting_level) < 1) {
-					sapi_flush(TSRMLS_C);
-				}
+				sapi_flush(TSRMLS_C);
 			} else if (type == 2) {
-				/* strip trailing whitespaces */
+				/* strip trailing whitespaces */	
 				l = bufl;
 				while (l-- && isspace(((unsigned char *)buf)[l]));
-				if (l != (int)(bufl - 1)) {
+				if (l != (bufl - 1)) {
 					bufl = l + 1;
 					buf[bufl] = '\0';
 				}
@@ -156,11 +144,11 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 			b = buf;
 		}
 		if (bufl) {
-			/* strip trailing whitespaces if we have not done so already */
+			/* strip trailing whitespaces if we have not done so already */	
 			if (type != 2) {
 				l = bufl;
 				while (l-- && isspace(((unsigned char *)buf)[l]));
-				if (l != (int)(bufl - 1)) {
+				if (l != (bufl - 1)) {
 					bufl = l + 1;
 					buf[bufl] = '\0';
 				}
@@ -169,7 +157,7 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 			/* Return last line from the shell command */
 			if (PG(magic_quotes_runtime)) {
 				int len;
-
+	
 				tmp = php_addslashes(buf, bufl, &len, 0 TSRMLS_CC);
 				RETVAL_STRINGL(tmp, len, 0);
 			} else {
@@ -184,7 +172,7 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 		}
 	}
 
-	pclose_return = php_stream_close(stream);
+	pclose_return = php_stream_close(stream); 
 	efree(buf);
 
 done:
@@ -203,7 +191,7 @@ err:
 }
 /* }}} */
 
-static void php_exec_ex(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
+static void php_exec_ex(INTERNAL_FUNCTION_PARAMETERS, int mode)
 {
 	char *cmd;
 	int cmd_len;
@@ -238,7 +226,6 @@ static void php_exec_ex(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
 		ZVAL_LONG(ret_code, ret);
 	}
 }
-/* }}} */
 
 /* {{{ proto string exec(string command [, array &output [, int &return_value]])
    Execute an external program */
@@ -246,6 +233,7 @@ PHP_FUNCTION(exec)
 {
 	php_exec_ex(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
+
 /* }}} */
 
 /* {{{ proto int system(string command [, int &return_value])
@@ -273,34 +261,19 @@ PHP_FUNCTION(passthru)
 
    *NOT* safe for binary strings
 */
-PHPAPI char *php_escape_shell_cmd(char *str)
-{
-	register int x, y, l = strlen(str);
+char *php_escape_shell_cmd(char *str) {
+	register int x, y, l;
 	char *cmd;
 	char *p = NULL;
-	size_t estimate = (2 * l) + 1;
-
-	TSRMLS_FETCH();
 
+	l = strlen(str);
 	cmd = safe_emalloc(2, l, 1);
-
+	
 	for (x = 0, y = 0; x < l; x++) {
-		int mb_len = php_mblen(str + x, (l - x));
-
-		/* skip non-valid multibyte characters */
-		if (mb_len < 0) {
-			continue;
-		} else if (mb_len > 1) {
-			memcpy(cmd + y, str + x, mb_len);
-			y += mb_len;
-			x += mb_len - 1;
-			continue;
-		}
-
 		switch (str[x]) {
-#ifndef PHP_WIN32
 			case '"':
 			case '\'':
+#ifndef PHP_WIN32
 				if (!p && (p = memchr(str + x + 1, str[x], l - x - 1))) {
 					/* noop */
 				} else if (p && *p == str[x]) {
@@ -310,13 +283,6 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 				}
 				cmd[y++] = str[x];
 				break;
-#else
-			/* % is Windows specific for enviromental variables, ^%PATH% will 
-				output PATH whil ^%PATH^% not. escapeshellcmd will escape all %.
-			*/
-			case '%':
-			case '"':
-			case '\'':
 #endif
 			case '#': /* This is character-set independent */
 			case '&':
@@ -340,10 +306,12 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 			case '\x0A': /* excluding these two */
 			case '\xFF':
 #ifdef PHP_WIN32
-				cmd[y++] = '^';
-#else
-				cmd[y++] = '\\';
+			/* since Windows does not allow us to escape these chars, just remove them */
+			case '%':
+				cmd[y++] = ' ';
+				break;
 #endif
+				cmd[y++] = '\\';
 				/* fall-through */
 			default:
 				cmd[y++] = str[x];
@@ -351,29 +319,21 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 		}
 	}
 	cmd[y] = '\0';
-
-	if ((estimate - y) > 4096) {
-		/* realloc if the estimate was way overill
-		 * Arbitrary cutoff point of 4096 */
-		cmd = erealloc(cmd, y + 1);
-	}
-
 	return cmd;
 }
 /* }}} */
 
 /* {{{ php_escape_shell_arg
  */
-PHPAPI char *php_escape_shell_arg(char *str)
-{
-	int x, y = 0, l = strlen(str);
+char *php_escape_shell_arg(char *str) {
+	int x, y, l;
 	char *cmd;
-	size_t estimate = (4 * l) + 3;
-
-	TSRMLS_FETCH();
 
+	y = 0;
+	l = strlen(str);
+	
 	cmd = safe_emalloc(4, l, 3); /* worst case */
-
+	
 #ifdef PHP_WIN32
 	cmd[y++] = '"';
 #else
@@ -381,18 +341,6 @@ PHPAPI char *php_escape_shell_arg(char *str)
 #endif
 
 	for (x = 0; x < l; x++) {
-		int mb_len = php_mblen(str + x, (l - x));
-
-		/* skip non-valid multibyte characters */
-		if (mb_len < 0) {
-			continue;
-		} else if (mb_len > 1) {
-			memcpy(cmd + y, str + x, mb_len);
-			y += mb_len;
-			x += mb_len - 1;
-			continue;
-		}
-
 		switch (str[x]) {
 #ifdef PHP_WIN32
 		case '"':
@@ -416,12 +364,6 @@ PHPAPI char *php_escape_shell_arg(char *str)
 	cmd[y++] = '\'';
 #endif
 	cmd[y] = '\0';
-
-	if ((estimate - y) > 4096) {
-		/* realloc if the estimate was way overill
-		 * Arbitrary cutoff point of 4096 */
-		cmd = erealloc(cmd, y + 1);
-	}
 	return cmd;
 }
 /* }}} */
@@ -430,19 +372,18 @@ PHPAPI char *php_escape_shell_arg(char *str)
    Escape shell metacharacters */
 PHP_FUNCTION(escapeshellcmd)
 {
-	char *command;
-	int command_len;
+	zval **arg1;
 	char *cmd = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &command, &command_len) == FAILURE) {
-		return;
+	if (zend_get_parameters_ex(1, &arg1) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	if (command_len) {
-		cmd = php_escape_shell_cmd(command);
-		RETVAL_STRING(cmd, 0);
-	} else {
-		RETVAL_EMPTY_STRING();
+	
+	convert_to_string_ex(arg1);
+	if (Z_STRLEN_PP(arg1)) {
+		cmd = php_escape_shell_cmd(Z_STRVAL_PP(arg1));
+		RETVAL_STRING(cmd, 1);
+		efree(cmd);
 	}
 }
 /* }}} */
@@ -451,17 +392,18 @@ PHP_FUNCTION(escapeshellcmd)
    Quote and escape an argument for use in a shell command */
 PHP_FUNCTION(escapeshellarg)
 {
-	char *argument;
-	int argument_len;
+	zval **arg1;
 	char *cmd = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &argument, &argument_len) == FAILURE) {
-		return;
+	if (zend_get_parameters_ex(1, &arg1) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	if (argument) {
-		cmd = php_escape_shell_arg(argument);
-		RETVAL_STRING(cmd, 0);
+	
+	convert_to_string_ex(arg1);
+	if (Z_STRLEN_PP(arg1)) {
+		cmd = php_escape_shell_arg(Z_STRVAL_PP(arg1));
+		RETVAL_STRING(cmd, 1);
+		efree(cmd);
 	}
 }
 /* }}} */
@@ -472,35 +414,37 @@ PHP_FUNCTION(shell_exec)
 {
 	FILE *in;
 	size_t total_readbytes;
-	char *command;
-	int command_len;
+	zval **cmd;
 	char *ret;
 	php_stream *stream;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &command, &command_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &cmd)==FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
+	
 	if (PG(safe_mode)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot execute using backquotes in Safe Mode");
 		RETURN_FALSE;
 	}
 
+	convert_to_string_ex(cmd);
 #ifdef PHP_WIN32
-	if ((in=VCWD_POPEN(command, "rt"))==NULL) {
+	if ((in=VCWD_POPEN(Z_STRVAL_PP(cmd), "rt"))==NULL) {
 #else
-	if ((in=VCWD_POPEN(command, "r"))==NULL) {
+	if ((in=VCWD_POPEN(Z_STRVAL_PP(cmd), "r"))==NULL) {
 #endif
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to execute '%s'", command);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to execute '%s'", Z_STRVAL_PP(cmd));
 		RETURN_FALSE;
 	}
 
 	stream = php_stream_fopen_from_pipe(in, "rb");
 	total_readbytes = php_stream_copy_to_mem(stream, &ret, PHP_STREAM_COPY_ALL, 0);
-	php_stream_close(stream);
-
+	php_stream_close(stream); 
+	
 	if (total_readbytes > 0) {
-		RETVAL_STRINGL(ret, total_readbytes, 0);
+		RETURN_STRINGL(ret, total_readbytes, 0);
+	} else {
+		RETURN_NULL();	
 	}
 }
 /* }}} */
@@ -519,10 +463,10 @@ PHP_FUNCTION(proc_nice)
 	errno = 0;
 	nice(pri);
 	if (errno) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Only a super user may attempt to increase the priority of a process");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Only a super user may attempt to increase the priority of a process.");
 		RETURN_FALSE;
 	}
-
+	
 	RETURN_TRUE;
 }
 /* }}} */
