@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: filestat.c,v 1.136.2.8.2.14.2.9 2008/12/31 11:15:45 sebastian Exp $ */
+/* $Id: filestat.c,v 1.136.2.3 2006/01/01 12:50:14 sniper Exp $ */
 
 #include "php.h"
 #include "safe_mode.h"
@@ -79,10 +79,6 @@
 # endif
 #endif
 
-#ifdef PHP_WIN32
-#include "win32/winutil.h"
-#endif
-
 #include "basic_functions.h"
 #include "php_filestat.h"
 
@@ -98,15 +94,14 @@
 
 #define S_IXROOT ( S_IXUSR | S_IXGRP | S_IXOTH )
 
-PHP_RINIT_FUNCTION(filestat) /* {{{ */
+PHP_RINIT_FUNCTION(filestat)
 {
 	BG(CurrentStatFile)=NULL;
 	BG(CurrentLStatFile)=NULL;
 	return SUCCESS;
 }
-/* }}} */
 
-PHP_RSHUTDOWN_FUNCTION(filestat) /* {{{ */
+PHP_RSHUTDOWN_FUNCTION(filestat) 
 {
 	if (BG(CurrentStatFile)) {
 		efree (BG(CurrentStatFile));
@@ -118,12 +113,15 @@ PHP_RSHUTDOWN_FUNCTION(filestat) /* {{{ */
 	}
 	return SUCCESS;
 }
-/* }}} */
 
-static int php_disk_total_space(char *path, double *space TSRMLS_DC) /* {{{ */
-#if defined(WINDOWS) /* {{{ */
+/* {{{ proto float disk_total_space(string path)
+   Get total disk space for filesystem that path is on */
+PHP_FUNCTION(disk_total_space)
 {
-	double bytestotal = 0;
+	zval **path;
+#ifdef WINDOWS
+	double bytestotal;
+
 	HINSTANCE kernel32;
 	FARPROC gdfse;
 	typedef BOOL (WINAPI *gdfse_func)(LPCTSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER);
@@ -140,6 +138,26 @@ static int php_disk_total_space(char *path, double *space TSRMLS_DC) /* {{{ */
 	DWORD NumberOfFreeClusters;
 	DWORD TotalNumberOfClusters;
 
+#else /* not - WINDOWS */
+#if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
+	struct statvfs buf;
+#elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
+	struct statfs buf;
+#endif
+	double bytestotal = 0;
+#endif /* WINDOWS */
+
+	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &path)==FAILURE) {
+		WRONG_PARAM_COUNT;
+	}
+
+	convert_to_string_ex(path);
+
+	if (php_check_open_basedir(Z_STRVAL_PP(path) TSRMLS_CC)) {
+		RETURN_FALSE;
+	}
+
+#ifdef WINDOWS
 	/* GetDiskFreeSpaceEx is only available in NT and Win95 post-OSR2,
 	   so we have to jump through some hoops to see if the function
 	   exists. */
@@ -149,64 +167,40 @@ static int php_disk_total_space(char *path, double *space TSRMLS_DC) /* {{{ */
 		/* It's available, so we can call it. */
 		if (gdfse) {
 			func = (gdfse_func)gdfse;
-			if (func(path,
-						&FreeBytesAvailableToCaller,
-						&TotalNumberOfBytes,
-						&TotalNumberOfFreeBytes) == 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", php_win_err());
-				return FAILURE;
-			}
+			if (func(Z_STRVAL_PP(path),
+				&FreeBytesAvailableToCaller,
+				&TotalNumberOfBytes,
+				&TotalNumberOfFreeBytes) == 0) RETURN_FALSE;
 
 			/* i know - this is ugly, but i works <thies@thieso.net> */
 			bytestotal  = TotalNumberOfBytes.HighPart *
 				(double) (((unsigned long)1) << 31) * 2.0 +
 				TotalNumberOfBytes.LowPart;
-		} else { /* If it's not available, we just use GetDiskFreeSpace */
-			if (GetDiskFreeSpace(path,
-						&SectorsPerCluster, &BytesPerSector,
-						&NumberOfFreeClusters, &TotalNumberOfClusters) == 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", php_win_err());
-				return FAILURE;
-			}
+		}
+		/* If it's not available, we just use GetDiskFreeSpace */
+		else {
+			if (GetDiskFreeSpace(Z_STRVAL_PP(path),
+				&SectorsPerCluster, &BytesPerSector,
+				&NumberOfFreeClusters, &TotalNumberOfClusters) == 0) RETURN_FALSE;
 			bytestotal = (double)TotalNumberOfClusters * (double)SectorsPerCluster * (double)BytesPerSector;
 		}
-	} else {
+	}
+	else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to load kernel32.dll");
-		return FAILURE;
+		RETURN_FALSE;
 	}
 
-	*space = bytestotal;
-	return SUCCESS;
-}
-/* }}} */
-#elif defined(OS2) /* {{{ */
-{
-	double bytestotal = 0;
-	FSALLOCATE fsinfo;
-	char drive = path[0] & 95;
+#elif defined(OS2)
+	{
+		FSALLOCATE fsinfo;
+		char drive = Z_STRVAL_PP(path)[0] & 95;
 
-	if (DosQueryFSInfo( drive ? drive - 64 : 0, FSIL_ALLOC, &fsinfo, sizeof( fsinfo ) ) == 0) {
-		bytestotal = (double)fsinfo.cbSector * fsinfo.cSectorUnit * fsinfo.cUnit;
-		*space = bytestotal;
-		return SUCCESS;
+		if (DosQueryFSInfo( drive ? drive - 64 : 0, FSIL_ALLOC, &fsinfo, sizeof( fsinfo ) ) == 0)
+			bytestotal = (double)fsinfo.cbSector * fsinfo.cSectorUnit * fsinfo.cUnit;
 	}
-	return FAILURE;
-}
-/* }}} */
-#else /* {{{ if !defined(OS2) && !defined(WINDOWS) */
-{
-	double bytestotal = 0;
+#else /* WINDOWS, OS/2 */
 #if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
-	struct statvfs buf;
-#elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
-	struct statfs buf;
-#endif
-
-#if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
-	if (statvfs(path, &buf)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
-		return FAILURE;
-	}
+	if (statvfs(Z_STRVAL_PP(path), &buf)) RETURN_FALSE;
 	if (buf.f_frsize) {
 		bytestotal = (((double)buf.f_blocks) * ((double)buf.f_frsize));
 	} else {
@@ -214,47 +208,22 @@ static int php_disk_total_space(char *path, double *space TSRMLS_DC) /* {{{ */
 	}
 
 #elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
-	if (statfs(path, &buf)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
-		return FAILURE;
-	}
+	if (statfs(Z_STRVAL_PP(path), &buf)) RETURN_FALSE;
 	bytestotal = (((double)buf.f_bsize) * ((double)buf.f_blocks));
 #endif
+#endif /* WINDOWS */
 
-	*space = bytestotal;
-	return SUCCESS;
-}
-#endif
-/* }}} */
-/* }}} */
-
-/* {{{ proto float disk_total_space(string path)
-   Get total disk space for filesystem that path is on */
-PHP_FUNCTION(disk_total_space)
-{
-	double bytestotal;
-	char *path;
-	int path_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &path, &path_len) == FAILURE) {
-		return;
-	}
-
-	if (php_check_open_basedir(path TSRMLS_CC)) {
-		RETURN_FALSE;
-	}
-
-	if (php_disk_total_space(path, &bytestotal TSRMLS_CC) == SUCCESS) {
-		RETURN_DOUBLE(bytestotal);
-	}
-	RETURN_FALSE;
+	RETURN_DOUBLE(bytestotal);
 }
 /* }}} */
 
-static int php_disk_free_space(char *path, double *space TSRMLS_DC) /* {{{ */
-#if defined(WINDOWS) /* {{{ */
+/* {{{ proto float disk_free_space(string path)
+   Get free disk space for filesystem that path is on */
+PHP_FUNCTION(disk_free_space)
 {
-	double bytesfree = 0;
+	zval **path;
+#ifdef WINDOWS
+	double bytesfree;
 
 	HINSTANCE kernel32;
 	FARPROC gdfse;
@@ -272,6 +241,26 @@ static int php_disk_free_space(char *path, double *space TSRMLS_DC) /* {{{ */
 	DWORD NumberOfFreeClusters;
 	DWORD TotalNumberOfClusters;
 
+#else /* not - WINDOWS */
+#if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
+	struct statvfs buf;
+#elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
+	struct statfs buf;
+#endif
+	double bytesfree = 0;
+#endif /* WINDOWS */
+
+	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &path)==FAILURE) {
+		WRONG_PARAM_COUNT;
+	}
+
+	convert_to_string_ex(path);
+
+	if (php_check_open_basedir(Z_STRVAL_PP(path) TSRMLS_CC)) {
+		RETURN_FALSE;
+	}
+
+#ifdef WINDOWS
 	/* GetDiskFreeSpaceEx is only available in NT and Win95 post-OSR2,
 	   so we have to jump through some hoops to see if the function
 	   exists. */
@@ -281,360 +270,189 @@ static int php_disk_free_space(char *path, double *space TSRMLS_DC) /* {{{ */
 		/* It's available, so we can call it. */
 		if (gdfse) {
 			func = (gdfse_func)gdfse;
-			if (func(path,
-						&FreeBytesAvailableToCaller,
-						&TotalNumberOfBytes,
-						&TotalNumberOfFreeBytes) == 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", php_win_err());
-				return FAILURE;
-			}
+			if (func(Z_STRVAL_PP(path),
+				&FreeBytesAvailableToCaller,
+				&TotalNumberOfBytes,
+				&TotalNumberOfFreeBytes) == 0) RETURN_FALSE;
 
 			/* i know - this is ugly, but i works <thies@thieso.net> */
 			bytesfree  = FreeBytesAvailableToCaller.HighPart *
 				(double) (((unsigned long)1) << 31) * 2.0 +
 				FreeBytesAvailableToCaller.LowPart;
-		} else { /* If it's not available, we just use GetDiskFreeSpace */
-			if (GetDiskFreeSpace(path,
-						&SectorsPerCluster, &BytesPerSector,
-						&NumberOfFreeClusters, &TotalNumberOfClusters) == 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", php_win_err());
-				return FAILURE;
-			}
+		}
+		/* If it's not available, we just use GetDiskFreeSpace */
+		else {
+			if (GetDiskFreeSpace(Z_STRVAL_PP(path),
+				&SectorsPerCluster, &BytesPerSector,
+				&NumberOfFreeClusters, &TotalNumberOfClusters) == 0) RETURN_FALSE;
 			bytesfree = (double)NumberOfFreeClusters * (double)SectorsPerCluster * (double)BytesPerSector;
 		}
-	} else {
+	}
+	else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to load kernel32.dll");
-		return FAILURE;
+		RETURN_FALSE;
 	}
 
-	*space = bytesfree;
-	return SUCCESS;
-}
-/* }}} */
-#elif defined(OS2) /* {{{ */
-{
-	double bytesfree = 0;
-	FSALLOCATE fsinfo;
-	char drive = path[0] & 95;
+#elif defined(OS2)
+	{
+		FSALLOCATE fsinfo;
+		char drive = Z_STRVAL_PP(path)[0] & 95;
 
-	if (DosQueryFSInfo( drive ? drive - 64 : 0, FSIL_ALLOC, &fsinfo, sizeof( fsinfo ) ) == 0) {
-		bytesfree = (double)fsinfo.cbSector * fsinfo.cSectorUnit * fsinfo.cUnitAvail;
-		*space = bytesfree;
-		return SUCCESS;
+		if (DosQueryFSInfo( drive ? drive - 64 : 0, FSIL_ALLOC, &fsinfo, sizeof( fsinfo ) ) == 0)
+			bytesfree = (double)fsinfo.cbSector * fsinfo.cSectorUnit * fsinfo.cUnitAvail;
 	}
-	return FAILURE;
-}
-/* }}} */
-#else /* {{{ if !defined(OS2) && !defined(WINDOWS) */
-{
-	double bytesfree = 0;
+#else /* WINDOWS, OS/2 */
 #if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
-	struct statvfs buf;
-#elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
-	struct statfs buf;
-#endif
-
-#if defined(HAVE_SYS_STATVFS_H) && defined(HAVE_STATVFS)
-	if (statvfs(path, &buf)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
-		return FAILURE;
-	}
+	if (statvfs(Z_STRVAL_PP(path), &buf)) RETURN_FALSE;
 	if (buf.f_frsize) {
 		bytesfree = (((double)buf.f_bavail) * ((double)buf.f_frsize));
 	} else {
 		bytesfree = (((double)buf.f_bavail) * ((double)buf.f_bsize));
 	}
 #elif (defined(HAVE_SYS_STATFS_H) || defined(HAVE_SYS_MOUNT_H)) && defined(HAVE_STATFS)
-	if (statfs(path, &buf)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
-		return FAILURE;
-	}
+	if (statfs(Z_STRVAL_PP(path), &buf)) RETURN_FALSE;
 #ifdef NETWARE
 	bytesfree = (((double)buf.f_bsize) * ((double)buf.f_bfree));
 #else
 	bytesfree = (((double)buf.f_bsize) * ((double)buf.f_bavail));
 #endif
 #endif
+#endif /* WINDOWS */
 
-	*space = bytesfree;
-	return SUCCESS;
+	RETURN_DOUBLE(bytesfree);
 }
-#endif
-/* }}} */
 /* }}} */
 
-/* {{{ proto float disk_free_space(string path)
-   Get free disk space for filesystem that path is on */
-PHP_FUNCTION(disk_free_space)
+/* {{{ proto bool chgrp(string filename, mixed group)
+   Change file group */
+#ifndef NETWARE
+PHP_FUNCTION(chgrp)
 {
-	double bytesfree;
-	char *path;
-	int path_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &path, &path_len) == FAILURE) {
-		return;
-	}
-
-	if (php_check_open_basedir(path TSRMLS_CC)) {
-		RETURN_FALSE;
-	}
-
-	if (php_disk_free_space(path, &bytesfree TSRMLS_CC) == SUCCESS) {
-		RETURN_DOUBLE(bytesfree);
-	}
-	RETURN_FALSE;
-}
-/* }}} */
-
 #if !defined(WINDOWS)
-static void php_do_chgrp(INTERNAL_FUNCTION_PARAMETERS, int do_lchgrp) /* {{{ */
-{
-	char *filename;
-	int filename_len;
-	zval *group;
+	zval **filename, **group;
 	gid_t gid;
+	struct group *gr=NULL;
 	int ret;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz/", &filename, &filename_len, &group) == FAILURE) {
-		RETURN_FALSE;
+	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &filename, &group)==FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	if (Z_TYPE_P(group) == IS_LONG) {
-		gid = (gid_t)Z_LVAL_P(group);
-	} else if (Z_TYPE_P(group) == IS_STRING) {
-#if defined(ZTS) && defined(HAVE_GETGRNAM_R) && defined(_SC_GETGR_R_SIZE_MAX)
-		struct group gr;
-		struct group *retgrptr;
-		long grbuflen = sysconf(_SC_GETGR_R_SIZE_MAX);
-		char *grbuf;
-
-		if (grbuflen < 1) {
-			RETURN_FALSE;
-		}
-
-		grbuf = emalloc(grbuflen);
-		if (getgrnam_r(Z_STRVAL_P(group), &gr, grbuf, grbuflen, &retgrptr) != 0 || retgrptr == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find gid for %s", Z_STRVAL_P(group));
-			efree(grbuf);
-			RETURN_FALSE;
-		}
-		efree(grbuf);
-		gid = gr.gr_gid;
-#else
-		struct group *gr = getgrnam(Z_STRVAL_P(group));
-
+	convert_to_string_ex(filename);
+	if (Z_TYPE_PP(group) == IS_STRING) {
+		gr = getgrnam(Z_STRVAL_PP(group));
 		if (!gr) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find gid for %s", Z_STRVAL_P(group));
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find gid for %s",
+					   Z_STRVAL_PP(group));
 			RETURN_FALSE;
 		}
 		gid = gr->gr_gid;
-#endif
 	} else {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "parameter 2 should be string or integer, %s given", zend_zval_type_name(group));
-		RETURN_FALSE;
+		convert_to_long_ex(group);
+		gid = Z_LVAL_PP(group);
 	}
 
-	if (PG(safe_mode) &&(!php_checkuid(filename, NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
+	if (PG(safe_mode) &&(!php_checkuid(Z_STRVAL_PP(filename), NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
 		RETURN_FALSE;
 	}
 
 	/* Check the basedir */
-	if (php_check_open_basedir(filename TSRMLS_CC)) {
+	if (php_check_open_basedir(Z_STRVAL_PP(filename) TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
-	if (do_lchgrp) {
-#if HAVE_LCHOWN
-		ret = VCWD_LCHOWN(filename, -1, gid);
-#endif
-	} else {
-		ret = VCWD_CHOWN(filename, -1, gid);
-	}
+	ret = VCWD_CHOWN(Z_STRVAL_PP(filename), -1, gid);
 	if (ret == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
 		RETURN_FALSE;
 	}
 	RETURN_TRUE;
-}
-/* }}} */
+#else
+	RETURN_FALSE;
 #endif
+}
+#endif
+/* }}} */
 
+/* {{{ proto bool chown (string filename, mixed user)
+   Change file owner */
 #ifndef NETWARE
-/* {{{ proto bool chgrp(string filename, mixed group)
-   Change file group */
-PHP_FUNCTION(chgrp)
+PHP_FUNCTION(chown)
 {
 #if !defined(WINDOWS)
-	php_do_chgrp(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
-#else
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto bool lchgrp(string filename, mixed group)
-   Change symlink group */
-#if HAVE_LCHOWN
-PHP_FUNCTION(lchgrp)
-{
-# if !defined(WINDOWS)
-	php_do_chgrp(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
-# else
-	RETURN_FALSE;
-# endif
-}
-#endif
-/* }}} */
-#endif /* !NETWARE */
-
-#if !defined(WINDOWS)
-static void php_do_chown(INTERNAL_FUNCTION_PARAMETERS, int do_lchown) /* {{{ */
-{
-	char *filename;
-	int filename_len;
-	zval *user;
-	uid_t uid;
+	zval **filename, **user;
 	int ret;
+	uid_t uid;
+	struct passwd *pw = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz/", &filename, &filename_len, &user) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &filename, &user)==FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	if (Z_TYPE_P(user) == IS_LONG) {
-		uid = (uid_t)Z_LVAL_P(user);
-	} else if (Z_TYPE_P(user) == IS_STRING) {
-#if defined(ZTS) && defined(_SC_GETPW_R_SIZE_MAX) && defined(HAVE_GETPWNAM_R)
-		struct passwd pw;
-		struct passwd *retpwptr = NULL;
-		long pwbuflen = sysconf(_SC_GETPW_R_SIZE_MAX);
-		char *pwbuf;
-
-		if (pwbuflen < 1) {
-			RETURN_FALSE;
-		}
-
-		pwbuf = emalloc(pwbuflen);
-		if (getpwnam_r(Z_STRVAL_P(user), &pw, pwbuf, pwbuflen, &retpwptr) != 0 || retpwptr == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find uid for %s", Z_STRVAL_P(user));
-			efree(pwbuf);
-			RETURN_FALSE;
-		}
-		efree(pwbuf);
-		uid = pw.pw_uid;
-#else
-		struct passwd *pw = getpwnam(Z_STRVAL_P(user));
-
+	convert_to_string_ex(filename);
+	if (Z_TYPE_PP(user) == IS_STRING) {
+		pw = getpwnam(Z_STRVAL_PP(user));
 		if (!pw) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find uid for %s", Z_STRVAL_P(user));
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find uid for %s",
+					   Z_STRVAL_PP(user));
 			RETURN_FALSE;
 		}
 		uid = pw->pw_uid;
-#endif
 	} else {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "parameter 2 should be string or integer, %s given", zend_zval_type_name(user));
-		RETURN_FALSE;
+		convert_to_long_ex(user);
+		uid = Z_LVAL_PP(user);
 	}
 
-	if (PG(safe_mode) && (!php_checkuid(filename, NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
+	if (PG(safe_mode) &&(!php_checkuid(Z_STRVAL_PP(filename), NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
 		RETURN_FALSE;
 	}
 
 	/* Check the basedir */
-	if (php_check_open_basedir(filename TSRMLS_CC)) {
+	if (php_check_open_basedir(Z_STRVAL_PP(filename) TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
-	if (do_lchown) {
-#if HAVE_LCHOWN
-		ret = VCWD_LCHOWN(filename, uid, -1);
-#endif
-	} else {
-		ret = VCWD_CHOWN(filename, uid, -1);
-	}
+	ret = VCWD_CHOWN(Z_STRVAL_PP(filename), uid, -1);
 	if (ret == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
 		RETURN_FALSE;
 	}
-}
-/* }}} */
 #endif
-
-#ifndef NETWARE
-/* {{{ proto bool chown (string filename, mixed user)
-   Change file owner */
-PHP_FUNCTION(chown)
-{
-#if !defined(WINDOWS)
-	RETVAL_TRUE;
-	php_do_chown(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
-#else
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto bool chown (string filename, mixed user)
-   Change file owner */
-#if HAVE_LCHOWN
-PHP_FUNCTION(lchown)
-{
-# if !defined(WINDOWS)
-	RETVAL_TRUE;
-	php_do_chown(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
-# else
-	RETURN_FALSE;
-# endif
+	RETURN_TRUE;
 }
 #endif
 /* }}} */
-#endif /* !NETWARE */
 
 /* {{{ proto bool chmod(string filename, int mode)
    Change file mode */
 PHP_FUNCTION(chmod)
 {
-	char *filename;
-	int filename_len;
-	long mode;
+	zval **filename, **mode;
 	int ret;
 	mode_t imode;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &filename, &filename_len, &mode) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &filename, &mode)==FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
+	convert_to_string_ex(filename);
+	convert_to_long_ex(mode);
 
-	if (PG(safe_mode) &&(!php_checkuid(filename, NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
+	if (PG(safe_mode) &&(!php_checkuid(Z_STRVAL_PP(filename), NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS))) {
 		RETURN_FALSE;
 	}
 
 	/* Check the basedir */
-	if (php_check_open_basedir(filename TSRMLS_CC)) {
+	if (php_check_open_basedir(Z_STRVAL_PP(filename) TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
-	imode = (mode_t) mode;
-	/* In safe mode, do not allow to setuid files.
-	 * Setuiding files could allow users to gain privileges
-	 * that safe mode doesn't give them. */
+	imode = (mode_t) Z_LVAL_PP(mode);
+	/* in safe mode, do not allow to setuid files.
+	   Setuiding files could allow users to gain privileges
+	   that safe mode doesn't give them.
+	*/
+	if(PG(safe_mode))
+		imode &= 0777;
 
-	if (PG(safe_mode)) {
-		php_stream_statbuf ssb;
-		if (php_stream_stat_path_ex(filename, 0, &ssb, NULL)) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "stat failed for %s", filename);
-			RETURN_FALSE;
-		}
-		if ((imode & 04000) != 0 && (ssb.sb.st_mode & 04000) == 0) {
-			imode ^= 04000;
-		}
-		if ((imode & 02000) != 0 && (ssb.sb.st_mode & 02000) == 0) {
-			imode ^= 02000;
-		}
-		if ((imode & 01000) != 0 && (ssb.sb.st_mode & 01000) == 0) {
-			imode ^= 01000;
-		}
-	}
-
-	ret = VCWD_CHMOD(filename, imode);
+	ret = VCWD_CHMOD(Z_STRVAL_PP(filename), imode);
 	if (ret == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
 		RETURN_FALSE;
@@ -648,59 +466,56 @@ PHP_FUNCTION(chmod)
    Set modification time of file */
 PHP_FUNCTION(touch)
 {
-	char *filename;
-	int filename_len;
-	long filetime = 0, fileatime = 0;
-	int ret, argc = ZEND_NUM_ARGS();
+	zval **filename, **filetime, **fileatime;
+	int ret;
+	struct stat sb;
 	FILE *file;
 	struct utimbuf newtimebuf;
-	struct utimbuf *newtime = &newtimebuf;
+	struct utimbuf *newtime = NULL;
+	int ac = ZEND_NUM_ARGS();
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "s|ll", &filename, &filename_len, &filetime, &fileatime) == FAILURE) {
-		return;
-	}
 
-	switch (argc) {
-		case 1:
-#ifdef HAVE_UTIME_NULL
-			newtime = NULL;
-#else
-			newtime->modtime = newtime->actime = time(NULL);
+	if (ac == 1 && zend_get_parameters_ex(1, &filename) != FAILURE) {
+#ifndef HAVE_UTIME_NULL
+		newtime = &newtimebuf;
+		newtime->modtime = newtime->actime = time(NULL);
 #endif
-			break;
-		case 2:
-			newtime->modtime = newtime->actime = filetime;
-			break;
-		case 3:
-			newtime->modtime = filetime;
-			newtime->actime = fileatime;
-			break;
-		default:
-			/* Never reached */
-			WRONG_PARAM_COUNT;
+	} else if (ac == 2 && zend_get_parameters_ex(2, &filename, &filetime) != FAILURE) {
+		convert_to_long_ex(filetime);
+		newtime = &newtimebuf;
+		newtime->modtime = newtime->actime = Z_LVAL_PP(filetime);
+	} else if (ac == 3 && zend_get_parameters_ex(3, &filename, &filetime, &fileatime) != FAILURE) {
+		convert_to_long_ex(fileatime);
+		convert_to_long_ex(filetime);
+		newtime = &newtimebuf;
+		newtime->actime = Z_LVAL_PP(fileatime);
+		newtime->modtime = Z_LVAL_PP(filetime);
+	} else {
+		WRONG_PARAM_COUNT;
 	}
+	convert_to_string_ex(filename);
 
-	/* Safe-mode */
-	if (PG(safe_mode) && (!php_checkuid(filename, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+	if (PG(safe_mode) &&(!php_checkuid(Z_STRVAL_PP(filename), NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 		RETURN_FALSE;
 	}
 
 	/* Check the basedir */
-	if (php_check_open_basedir(filename TSRMLS_CC)) {
+	if (php_check_open_basedir(Z_STRVAL_PP(filename) TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
 	/* create the file if it doesn't exist already */
-	if (VCWD_ACCESS(filename, F_OK) != 0) {
-		file = VCWD_FOPEN(filename, "w");
+	ret = VCWD_STAT(Z_STRVAL_PP(filename), &sb);
+	if (ret == -1) {
+		file = VCWD_FOPEN(Z_STRVAL_PP(filename), "w");
 		if (file == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to create file %s because %s", filename, strerror(errno));
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to create file %s because %s", Z_STRVAL_PP(filename), strerror(errno));
 			RETURN_FALSE;
 		}
 		fclose(file);
 	}
 
-	ret = VCWD_UTIME(filename, newtime);
+	ret = VCWD_UTIME(Z_STRVAL_PP(filename), newtime);
 	if (ret == -1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Utime failed: %s", strerror(errno));
 		RETURN_FALSE;
@@ -710,13 +525,10 @@ PHP_FUNCTION(touch)
 /* }}} */
 #endif
 
-/* {{{ php_clear_stat_cache()
-*/
-PHPAPI void php_clear_stat_cache(zend_bool clear_realpath_cache, const char *filename, int filename_len TSRMLS_DC)
+/* {{{ proto void clearstatcache(void)
+   Clear file stat cache */
+PHP_FUNCTION(clearstatcache)
 {
-	/* always clear CurrentStatFile and CurrentLStatFile even if filename is not NULL
-	 * as it may contains outdated data (e.g. "nlink" for a directory when deleting a file
-	 * in this directory, as shown by lstat_stat_variation9.phpt) */
 	if (BG(CurrentStatFile)) {
 		efree(BG(CurrentStatFile));
 		BG(CurrentStatFile) = NULL;
@@ -725,29 +537,6 @@ PHPAPI void php_clear_stat_cache(zend_bool clear_realpath_cache, const char *fil
 		efree(BG(CurrentLStatFile));
 		BG(CurrentLStatFile) = NULL;
 	}
-	if (clear_realpath_cache) {
-		if (filename != NULL) {
-			realpath_cache_del(filename, filename_len TSRMLS_CC);
-		} else {
-			realpath_cache_clean(TSRMLS_C);
-		}
-	}
-}
-/* }}} */
-
-/* {{{ proto void clearstatcache([bool clear_realpath_cache[, string filename]])
-   Clear file stat cache */
-PHP_FUNCTION(clearstatcache)
-{
-	zend_bool  clear_realpath_cache = 0;
-	char      *filename             = NULL;
-	int        filename_len         = 0;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|bs", &clear_realpath_cache, &filename, &filename_len) == FAILURE) {
-		return;
-	}
-
-	php_clear_stat_cache(clear_realpath_cache, filename, filename_len TSRMLS_CC);
 }
 /* }}} */
 
@@ -761,43 +550,21 @@ PHP_FUNCTION(clearstatcache)
 PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int type, zval *return_value TSRMLS_DC)
 {
 	zval *stat_dev, *stat_ino, *stat_mode, *stat_nlink, *stat_uid, *stat_gid, *stat_rdev,
-		 *stat_size, *stat_atime, *stat_mtime, *stat_ctime, *stat_blksize, *stat_blocks;
+	 	*stat_size, *stat_atime, *stat_mtime, *stat_ctime, *stat_blksize, *stat_blocks;
 	struct stat *stat_sb;
 	php_stream_statbuf ssb;
 	int flags = 0, rmask=S_IROTH, wmask=S_IWOTH, xmask=S_IXOTH; /* access rights defaults to other */
-	char *stat_sb_names[13] = {
-		"dev", "ino", "mode", "nlink", "uid", "gid", "rdev",
-		"size", "atime", "mtime", "ctime", "blksize", "blocks"
-	};
-	char *local;
-	php_stream_wrapper *wrapper;
-	char safe_mode_buf[MAXPATHLEN];
+	char *stat_sb_names[13]={"dev", "ino", "mode", "nlink", "uid", "gid", "rdev",
+			      "size", "atime", "mtime", "ctime", "blksize", "blocks"};
 
 	if (!filename_length) {
 		RETURN_FALSE;
 	}
 
-	if ((wrapper = php_stream_locate_url_wrapper(filename, &local, 0 TSRMLS_CC)) == &php_plain_files_wrapper) {
-		if (php_check_open_basedir(local TSRMLS_CC)) {
-			RETURN_FALSE;
-		} else if (PG(safe_mode)) {
-			if (type == FS_IS_X) {
-				if (strstr(local, "..")) {
-					RETURN_FALSE;
-				} else {
-					char *b = strrchr(local, PHP_DIR_SEPARATOR);
-					snprintf(safe_mode_buf, MAXPATHLEN, "%s%s%s", PG(safe_mode_exec_dir), (b ? "" : "/"), (b ? b : local));
-					local = (char *)&safe_mode_buf;
-				}
-			} else if (!php_checkuid_ex(local, NULL, CHECKUID_ALLOW_FILE_NOT_EXISTS, CHECKUID_NO_ERRORS)) {
-				RETURN_FALSE;
-			}
-		}
-	}
-
 	if (IS_ACCESS_CHECK(type)) {
-		if (wrapper == &php_plain_files_wrapper) {
+		char *local;
 
+		if (php_stream_locate_url_wrapper(filename, &local, 0 TSRMLS_CC) == &php_plain_files_wrapper) {
 			switch (type) {
 #ifdef F_OK
 				case FS_EXISTS:
@@ -856,7 +623,7 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 			gid_t *gids;
 
 			groups = getgroups(0, NULL);
-			if(groups > 0) {
+			if(groups) {
 				gids=(gid_t *)safe_emalloc(groups, sizeof(gid_t), 0);
 				n=getgroups(groups, gids);
 				for(i=0;i<n;i++){
@@ -875,8 +642,11 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 
 #ifndef NETWARE
 	if (IS_ABLE_CHECK(type) && getuid() == 0) {
-		/* root has special perms on plain_wrapper
+		/* root has special perms on plain_wrapper 
 		   But we don't know about root under Netware */
+		php_stream_wrapper *wrapper;
+
+		wrapper = php_stream_locate_url_wrapper(filename, NULL, 0 TSRMLS_CC);
 		if (wrapper == &php_plain_files_wrapper) {
 			if (type == FS_IS_X) {
 				xmask = S_IXROOT;
@@ -958,9 +728,9 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 		MAKE_LONG_ZVAL_INCREF(stat_uid, stat_sb->st_uid);
 		MAKE_LONG_ZVAL_INCREF(stat_gid, stat_sb->st_gid);
 #ifdef HAVE_ST_RDEV
-		MAKE_LONG_ZVAL_INCREF(stat_rdev, stat_sb->st_rdev);
+		MAKE_LONG_ZVAL_INCREF(stat_rdev, stat_sb->st_rdev); 
 #else
-		MAKE_LONG_ZVAL_INCREF(stat_rdev, -1);
+		MAKE_LONG_ZVAL_INCREF(stat_rdev, -1); 
 #endif
 		MAKE_LONG_ZVAL_INCREF(stat_size, stat_sb->st_size);
 #ifdef NETWARE
@@ -973,7 +743,7 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 		MAKE_LONG_ZVAL_INCREF(stat_ctime, stat_sb->st_ctime);
 #endif
 #ifdef HAVE_ST_BLKSIZE
-		MAKE_LONG_ZVAL_INCREF(stat_blksize, stat_sb->st_blksize);
+		MAKE_LONG_ZVAL_INCREF(stat_blksize, stat_sb->st_blksize); 
 #else
 		MAKE_LONG_ZVAL_INCREF(stat_blksize,-1);
 #endif
@@ -989,7 +759,7 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_nlink, sizeof(zval *), NULL);
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_uid, sizeof(zval *), NULL);
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_gid, sizeof(zval *), NULL);
-
+        
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_rdev, sizeof(zval *), NULL);
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_size, sizeof(zval *), NULL);
 		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_atime, sizeof(zval *), NULL);
@@ -1021,19 +791,15 @@ PHPAPI void php_stat(const char *filename, php_stat_len filename_length, int typ
 /* }}} */
 
 /* another quickie macro to make defining similar functions easier */
-/* {{{ FileFunction(name, funcnum) */
 #define FileFunction(name, funcnum) \
 void name(INTERNAL_FUNCTION_PARAMETERS) { \
-	char *filename; \
-	int filename_len; \
-	\
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) { \
-		return; \
+	zval **filename; \
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) { \
+		WRONG_PARAM_COUNT; \
 	} \
-	\
-	php_stat(filename, (php_stat_len) filename_len, funcnum, return_value TSRMLS_CC); \
+	convert_to_string_ex(filename); \
+	php_stat(Z_STRVAL_PP(filename), (php_stat_len) Z_STRLEN_PP(filename), funcnum, return_value TSRMLS_CC); \
 }
-/* }}} */
 
 /* {{{ proto int fileperms(string filename)
    Get file permissions */
