@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,13 +14,10 @@
    +----------------------------------------------------------------------+
    | Authors:                                                             |
    | Wez Furlong (wez@thebrainroom.com)                                   |
-   | Sara Golemon (pollita@php.net)                                       |
-   | Moriyoshi Koizumi (moriyoshi@php.net)                                |
-   | Marcus Boerger (helly@php.net)                                       |
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: filters.c,v 1.44.2.3 2006/01/01 12:50:14 sniper Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -105,7 +102,7 @@ static php_stream_filter_status_t strfilter_toupper_filter(
 	if (bytes_consumed) {
 		*bytes_consumed = consumed;
 	}
-
+	
 	return PSFS_PASS_ON;
 }
 
@@ -133,7 +130,7 @@ static php_stream_filter_status_t strfilter_tolower_filter(
 	if (bytes_consumed) {
 		*bytes_consumed = consumed;
 	}
-
+	
 	return PSFS_PASS_ON;
 }
 
@@ -470,6 +467,7 @@ static php_conv_err_t php_conv_base64_encode_convert(php_conv_base64_encode *ins
 	register size_t ocnt, icnt;
 	register unsigned char *ps, *pd;
 	register unsigned int line_ccnt;
+	size_t nbytes_written;
 
 	if (in_pp == NULL || in_left_p == NULL) { 
 		return php_conv_base64_encode_flush(inst, in_pp, in_left_p, out_pp, out_left_p);
@@ -480,6 +478,7 @@ static php_conv_err_t php_conv_base64_encode_convert(php_conv_base64_encode *ins
 	ps = (unsigned char *)(*in_pp);
 	icnt = *in_left_p;
 	line_ccnt = inst->line_ccnt;
+	nbytes_written = 0;
 
 	/* consume the remainder first */
 	switch (inst->erem_len) {
@@ -639,7 +638,7 @@ static php_conv_err_t php_conv_base64_decode_convert(php_conv_base64_decode *ins
 	size_t icnt, ocnt;
 	unsigned int ustat;
 
-	static const unsigned int nbitsof_pack = 8;
+	const static unsigned int nbitsof_pack = 8;
 
 	if (in_pp == NULL || in_left_p == NULL) {
 		if (inst->eos || inst->urem_nbits == 0) { 
@@ -771,7 +770,7 @@ static void php_conv_qprint_encode_dtor(php_conv_qprint_encode *inst)
 }
 
 #define NEXT_CHAR(ps, icnt, lb_ptr, lb_cnt, lbchars) \
-	((lb_ptr) < (lb_cnt) ? (lbchars)[(lb_ptr)] : *(ps))
+	((lb_ptr) < (lb_cnt) ? (lbchars)[(lb_ptr)] : *(ps)) 
 
 #define CONSUME_CHAR(ps, icnt, lb_ptr, lb_cnt) \
 	if ((lb_ptr) < (lb_cnt)) { \
@@ -791,7 +790,6 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 	unsigned int line_ccnt;
 	unsigned int lb_ptr;
 	unsigned int lb_cnt;
-	unsigned int trail_ws;
 	int opts;
 	static char qp_digits[] = "0123456789ABCDEF";
 
@@ -808,7 +806,6 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 	icnt = *in_left_p;
 	pd = (unsigned char *)(*out_pp);
 	ocnt = *out_left_p;
-	trail_ws = 0;
 
 	for (;;) {
 		if (!(opts & PHP_CONV_QPRINT_OPT_BINARY) && inst->lbchars != NULL && inst->lbchars_len > 0) {
@@ -841,13 +838,11 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 
 		if (lb_ptr >= lb_cnt && icnt <= 0) {
 			break;
-		}
+		} 
 
 		c = NEXT_CHAR(ps, icnt, lb_ptr, lb_cnt, inst->lbchars);
 
-		if (!(opts & PHP_CONV_QPRINT_OPT_BINARY) &&
-			(trail_ws == 0) &&
-			(c == '\t' || c == ' ')) {
+		if (!(opts & PHP_CONV_QPRINT_OPT_BINARY) && (c == '\t' || c == ' ')) {
 			if (line_ccnt < 2 && inst->lbchars != NULL) {
 				if (ocnt < inst->lbchars_len + 1) {
 					err = PHP_CONV_ERR_TOO_BIG;
@@ -867,44 +862,12 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 					err = PHP_CONV_ERR_TOO_BIG;
 					break;
 				}
-
-				/* Check to see if this is EOL whitespace. */
-				if (inst->lbchars != NULL) {
-					unsigned char *ps2;
-					unsigned int j, lb_cnt2;
-
-					lb_cnt2 = 0;
-					ps2 = ps;
-					trail_ws = 1;
-
-					for (j = icnt - 1; j > 0; j--, ps2++) {
-						if (*ps2 == inst->lbchars[lb_cnt2]) {
-							lb_cnt2++;
-							if (lb_cnt2 >= inst->lbchars_len) {
-								/* Found trailing ws. Reset to top of main
-								 * for loop to allow for code to do necessary
-								 * wrapping/encoding. */
-								break;
-							}
-						} else if (lb_cnt2 != 0 || (*ps2 != '\t' && *ps2 != ' ')) {
-							/* At least one non-EOL character following, so
-							 * don't need to encode ws. */
-							trail_ws = 0;
-							break;
-						} else {
-							trail_ws++;
-						}
-					}
-				}
-
-				if (trail_ws == 0) {
-					*(pd++) = c;
-					ocnt--;
-					line_ccnt--;
-					CONSUME_CHAR(ps, icnt, lb_ptr, lb_cnt);
-				}
+				*(pd++) = c;
+				ocnt--;
+				line_ccnt--;
+				CONSUME_CHAR(ps, icnt, lb_ptr, lb_cnt);
 			}
-		} else if ((!(opts & PHP_CONV_QPRINT_OPT_FORCE_ENCODE_FIRST) || line_ccnt < inst->line_len) && ((c >= 33 && c <= 60) || (c >= 62 && c <= 126))) {
+		} else if ((!(opts & PHP_CONV_QPRINT_OPT_FORCE_ENCODE_FIRST) || line_ccnt < inst->line_len) && ((c >= 33 && c <= 60) || (c >= 62 && c <= 126))) { 
 			if (line_ccnt < 2 && inst->lbchars != NULL) {
 				if (ocnt < inst->lbchars_len + 1) {
 					err = PHP_CONV_ERR_TOO_BIG;
@@ -948,12 +911,9 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 			}
 			*(pd++) = '=';
 			*(pd++) = qp_digits[(c >> 4)];
-			*(pd++) = qp_digits[(c & 0x0f)];
+			*(pd++) = qp_digits[(c & 0x0f)]; 
 			ocnt -= 3;
 			line_ccnt -= 3;
-			if (trail_ws > 0) {
-				trail_ws--;
-			}
 			CONSUME_CHAR(ps, icnt, lb_ptr, lb_cnt);
 		}
 	}
@@ -961,7 +921,7 @@ static php_conv_err_t php_conv_qprint_encode_convert(php_conv_qprint_encode *ins
 	*in_pp = (const char *)ps;
 	*in_left_p = icnt;
 	*out_pp = (char *)pd;
-	*out_left_p = ocnt;
+	*out_left_p = ocnt; 
 	inst->line_ccnt = line_ccnt;
 	inst->lb_ptr = lb_ptr;
 	inst->lb_cnt = lb_cnt;
@@ -1068,18 +1028,6 @@ static php_conv_err_t php_conv_qprint_decode_convert(php_conv_qprint_decode *ins
 					scan_stat = 4;
 					ps++, icnt--;
 					break;
-				} else if (!inst->lbchars && lb_cnt == 0 && *ps == '\r') {
-					/* auto-detect line endings, looks like network line ending \r\n (could be mac \r) */
-					lb_cnt++;
-					scan_stat = 5;
-					ps++, icnt--;
-					break;
-				} else if (!inst->lbchars && lb_cnt == 0 && *ps == '\n') {
-					/* auto-detect line endings, looks like unix-lineendings, not to spec, but it is seem in the wild, a lot */
-					lb_cnt = lb_ptr = 0;
-					scan_stat = 0;
-					ps++, icnt--;
-					break;
 				} else if (lb_cnt < inst->lbchars_len &&
 							*ps == (unsigned char)inst->lbchars[lb_cnt]) {
 					lb_cnt++;
@@ -1089,16 +1037,20 @@ static php_conv_err_t php_conv_qprint_decode_convert(php_conv_qprint_decode *ins
 				}
 			} /* break is missing intentionally */
 
-			case 2: {	
+			case 2: {
+				unsigned int nbl;
+	
 				if (icnt <= 0) {
 					goto out;
 				}
+				nbl = (*ps >= 'A' ? *ps - 0x37 : *ps - 0x30);
 
-				if (!isxdigit((int) *ps)) {
+				if (nbl > 15) {
 					err = PHP_CONV_ERR_INVALID_SEQ;
 					goto out;
 				}
-				next_char = (next_char << 4) | (*ps >= 'A' ? *ps - 0x37 : *ps - 0x30);
+				next_char = (next_char << 4) | nbl;
+
 				scan_stat++;
 				ps++, icnt--;
 				if (scan_stat != 3) {
@@ -1133,16 +1085,7 @@ static php_conv_err_t php_conv_qprint_decode_convert(php_conv_qprint_decode *ins
 			} break;
 
 			case 5: {
-				if (!inst->lbchars && lb_cnt == 1 && *ps == '\n') {
-					/* auto-detect soft line breaks, found network line break */
-					lb_cnt = lb_ptr = 0;
-					scan_stat = 0;
-					ps++, icnt--; /* consume \n */
-				} else if (!inst->lbchars && lb_cnt > 0) {
-					/* auto-detect soft line breaks, found mac line break */
-					lb_cnt = lb_ptr = 0;
-					scan_stat = 0;
-				} else if (lb_cnt >= inst->lbchars_len) {
+				if (lb_cnt >= inst->lbchars_len) {
 					/* soft line break */
 					lb_cnt = lb_ptr = 0;
 					scan_stat = 0;
@@ -1462,10 +1405,12 @@ static php_conv *php_conv_open(int conv_mode, const HashTable *options, int pers
 			size_t lbchars_len;
 
 			if (options != NULL) {
-				/* If line-break-chars are not specified, filter will attempt to detect line endings (\r, \n, or \r\n) */
 				GET_STR_PROP(options, lbchars, lbchars_len, "line-break-chars", 0);
+				if (lbchars == NULL) {
+					lbchars = pestrdup("\r\n", 0);
+					lbchars_len = 2;
+				}
 			}
-
 			retval = pemalloc(sizeof(php_conv_qprint_decode), persistent);
 			if (lbchars != NULL) {
 				if (php_conv_qprint_decode_ctor((php_conv_qprint_decode *)retval, lbchars, lbchars_len, 1, persistent)) {
@@ -1850,295 +1795,6 @@ static php_stream_filter_factory strfilter_convert_factory = {
 };
 /* }}} */
 
-/* {{{ consumed filter implementation */
-typedef struct _php_consumed_filter_data {
-	int persistent;
-	size_t consumed;
-	off_t offset;
-} php_consumed_filter_data;
-
-static php_stream_filter_status_t consumed_filter_filter(
-	php_stream *stream,
-	php_stream_filter *thisfilter,
-	php_stream_bucket_brigade *buckets_in,
-	php_stream_bucket_brigade *buckets_out,
-	size_t *bytes_consumed,
-	int flags
-	TSRMLS_DC)
-{
-	php_consumed_filter_data *data = (php_consumed_filter_data *)(thisfilter->abstract);
-	php_stream_bucket *bucket;
-	size_t consumed = 0;
-
-	if (data->offset == ~0) {
-		data->offset = php_stream_tell(stream);
-	}
-	while ((bucket = buckets_in->head) != NULL) {
-		php_stream_bucket_unlink(bucket TSRMLS_CC);
-		consumed += bucket->buflen;
-		php_stream_bucket_append(buckets_out, bucket TSRMLS_CC);
-	}
-	if (bytes_consumed) {
-		*bytes_consumed = consumed;
-	}
-	if (flags & PSFS_FLAG_FLUSH_CLOSE) {
-		php_stream_seek(stream, data->offset + data->consumed, SEEK_SET);
-	}
-	data->consumed += consumed;
-	
-	return PSFS_PASS_ON;
-}
-
-static void consumed_filter_dtor(php_stream_filter *thisfilter TSRMLS_DC)
-{
-	if (thisfilter && thisfilter->abstract) {
-		php_consumed_filter_data *data = (php_consumed_filter_data*)thisfilter->abstract;
-		pefree(data, data->persistent);
-	}
-}
-
-static php_stream_filter_ops consumed_filter_ops = {
-	consumed_filter_filter,
-	consumed_filter_dtor,
-	"consumed"
-};
-
-static php_stream_filter *consumed_filter_create(const char *filtername, zval *filterparams, int persistent TSRMLS_DC)
-{
-	php_stream_filter_ops *fops = NULL;
-	php_consumed_filter_data *data;
-
-	if (strcasecmp(filtername, "consumed")) {
-		return NULL;
-	}
-
-	/* Create this filter */
-	data = pecalloc(1, sizeof(php_consumed_filter_data), persistent);
-	if (!data) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed allocating %zd bytes", sizeof(php_consumed_filter_data));
-		return NULL;
-	}
-	data->persistent = persistent;
-	data->consumed = 0;
-	data->offset = ~0;
-	fops = &consumed_filter_ops;
-
-	return php_stream_filter_alloc(fops, data, persistent);
-}
-
-php_stream_filter_factory consumed_filter_factory = {
-	consumed_filter_create
-};
-
-/* }}} */
-
-/* {{{ chunked filter implementation */
-typedef enum _php_chunked_filter_state {
-	CHUNK_SIZE_START,
-	CHUNK_SIZE,
-	CHUNK_SIZE_EXT,
-	CHUNK_SIZE_CR,
-	CHUNK_SIZE_LF,
-	CHUNK_BODY,
-	CHUNK_BODY_CR,
-	CHUNK_BODY_LF,
-	CHUNK_TRAILER,
-	CHUNK_ERROR
-} php_chunked_filter_state;
-
-typedef struct _php_chunked_filter_data {
-	php_chunked_filter_state state;
-	size_t chunk_size;
-	int persistent;
-} php_chunked_filter_data;
-
-static int php_dechunk(char *buf, int len, php_chunked_filter_data *data)
-{
-	char *p = buf;
-	char *end = p + len;
-	char *out = buf;
-	int out_len = 0;
-
-	while (p < end) {
-		switch (data->state) {
-			case CHUNK_SIZE_START:
-				data->chunk_size = 0;
-			case CHUNK_SIZE:
-				while (p < end) {
-					if (*p >= '0' && *p <= '9') {
-						data->chunk_size = (data->chunk_size * 16) + (*p - '0');
-					} else if (*p >= 'A' && *p <= 'F') {
-						data->chunk_size = (data->chunk_size * 16) + (*p - 'A' + 10);
-					} else if (*p >= 'a' && *p <= 'f') {
-						data->chunk_size = (data->chunk_size * 16) + (*p - 'a' + 10);
-					} else if (data->state == CHUNK_SIZE_START) {
-						data->state = CHUNK_ERROR;
-						break;
-					} else {
-						data->state = CHUNK_SIZE_EXT;
-						break;
-					}
-					data->state = CHUNK_SIZE;
-					p++;
-				}
-				if (data->state == CHUNK_ERROR) {
-					continue;
-				} else if (p == end) {
-					return out_len;
-				}
-			case CHUNK_SIZE_EXT:
-				/* skip extension */
-				while (p < end && *p != '\r' && *p != '\n') {
-					p++;
-				}
-				if (p == end) {
-					return out_len;
-				}
-			case CHUNK_SIZE_CR:
-				if (*p == '\r') {
-					p++;
-					if (p == end) {
-						data->state = CHUNK_SIZE_LF;
-						return out_len;
-					}
-				}
-			case CHUNK_SIZE_LF:
-				if (*p == '\n') {
-					p++;
-					if (data->chunk_size == 0) {
-						/* last chunk */
-						data->state = CHUNK_TRAILER;
-						continue;
-					} else if (p == end) {
-						data->state = CHUNK_BODY;
-						return out_len;
-					}
-				} else {
-					data->state = CHUNK_ERROR;
-					continue;
-				}
-			case CHUNK_BODY:
-				if ((size_t) (end - p) >= data->chunk_size) {
-					if (p != out) {
-						memmove(out, p, data->chunk_size);
-					}
-					out += data->chunk_size;
-					out_len += data->chunk_size;
-					p += data->chunk_size;
-					if (p == end) {
-						data->state = CHUNK_BODY_CR;
-						return out_len;
-					}
-				} else {
-					if (p != out) {
-						memmove(out, p, end - p);
-					}
-					data->chunk_size -= end - p;
-					data->state=CHUNK_BODY;
-					out_len += end - p;
-					return out_len;
-				}
-			case CHUNK_BODY_CR:
-				if (*p == '\r') {
-					p++;
-					if (p == end) {
-						data->state = CHUNK_BODY_LF;
-						return out_len;
-					}
-				}
-			case CHUNK_BODY_LF:
-				if (*p == '\n') {
-					p++;
-					data->state = CHUNK_SIZE_START;
-					continue;
-				} else {
-					data->state = CHUNK_ERROR;
-					continue;
-				}
-			case CHUNK_TRAILER:
-				/* ignore trailer */
-				p = end;
-				continue;
-			case CHUNK_ERROR:
-				if (p != out) {
-					memmove(out, p, end - p);
-				}
-				out_len += end - p;
-				return out_len;	
-		}
-	}
-	return out_len;
-}
-
-static php_stream_filter_status_t php_chunked_filter(
-	php_stream *stream,
-	php_stream_filter *thisfilter,
-	php_stream_bucket_brigade *buckets_in,
-	php_stream_bucket_brigade *buckets_out,
-	size_t *bytes_consumed,
-	int flags
-	TSRMLS_DC)
-{
-	php_stream_bucket *bucket;
-	size_t consumed = 0;
-	php_chunked_filter_data *data = (php_chunked_filter_data *) thisfilter->abstract;
-
-	while (buckets_in->head) {
-		bucket = php_stream_bucket_make_writeable(buckets_in->head TSRMLS_CC);
-		consumed += bucket->buflen;
-		bucket->buflen = php_dechunk(bucket->buf, bucket->buflen, data);	
-		php_stream_bucket_append(buckets_out, bucket TSRMLS_CC);
-	}
-
-	if (bytes_consumed) {
-		*bytes_consumed = consumed;
-	}
-	
-	return PSFS_PASS_ON;
-}
-
-static void php_chunked_dtor(php_stream_filter *thisfilter TSRMLS_DC)
-{
-	if (thisfilter && thisfilter->abstract) {
-		php_chunked_filter_data *data = (php_chunked_filter_data *) thisfilter->abstract;
-		pefree(data, data->persistent);
-	}
-}
-
-static php_stream_filter_ops chunked_filter_ops = {
-	php_chunked_filter,
-	php_chunked_dtor,
-	"dechunk"
-};
-
-static php_stream_filter *chunked_filter_create(const char *filtername, zval *filterparams, int persistent TSRMLS_DC)
-{
-	php_stream_filter_ops *fops = NULL;
-	php_chunked_filter_data *data;
-
-	if (strcasecmp(filtername, "dechunk")) {
-		return NULL;
-	}
-
-	/* Create this filter */
-	data = (php_chunked_filter_data *)pecalloc(1, sizeof(php_chunked_filter_data), persistent);
-	if (!data) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed allocating %zd bytes", sizeof(php_chunked_filter_data));
-		return NULL;
-	}
-	data->state = CHUNK_SIZE_START;
-	data->chunk_size = 0;
-	data->persistent = persistent;
-	fops = &chunked_filter_ops;
-
-	return php_stream_filter_alloc(fops, data, persistent);
-}
-
-static php_stream_filter_factory chunked_filter_factory = {
-	chunked_filter_create
-};
-/* }}} */
-
 static const struct {
 	php_stream_filter_ops *ops;
 	php_stream_filter_factory *factory;
@@ -2148,8 +1804,6 @@ static const struct {
 	{ &strfilter_tolower_ops, &strfilter_tolower_factory },
 	{ &strfilter_strip_tags_ops, &strfilter_strip_tags_factory },
 	{ &strfilter_convert_ops, &strfilter_convert_factory },
-	{ &consumed_filter_ops, &consumed_filter_factory },
-	{ &chunked_filter_ops, &chunked_filter_factory },
 	/* additional filters to go here */
 	{ NULL, NULL }
 };
