@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: html.c,v 1.97.2.10 2005/05/11 14:58:34 jorton Exp $ */
+/* $Id: html.c,v 1.111.2.1 2006/01/01 12:50:14 sniper Exp $ */
 
 /*
  * HTML entity resources:
@@ -465,6 +465,12 @@ static const struct {
 	{ 0, NULL, 0, 0 }
 };
 	
+struct basic_entities_dec {
+	unsigned short charcode;
+	char entity[8];
+	int entitylen;	
+};
+	
 #define MB_RETURN { \
 			*newpos = pos;       \
 		  	mbseq[mbpos] = '\0'; \
@@ -1206,6 +1212,72 @@ PHP_FUNCTION(htmlspecialchars)
 }
 /* }}} */
 
+/* {{{ proto string htmlspecialchars(string string [, int quote_style])
+   Convert special HTML entities back to characters */
+PHP_FUNCTION(htmlspecialchars_decode)
+{
+	char *str, *new_str, *e, *p;
+	int len, j, i, new_len;
+	long quote_style = ENT_COMPAT;
+	struct basic_entities_dec basic_entities_dec[8];
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &str, &len, &quote_style) == FAILURE) {
+		return;
+	}
+
+	new_str = estrndup(str, len);
+	new_len = len;
+	e = new_str + new_len;
+
+	if (!(p = memchr(new_str, '&', new_len))) {
+		RETURN_STRINGL(new_str, new_len, 0);
+	}
+
+	for (j = 0, i = 0; basic_entities[i].charcode != 0; i++) {
+		if (basic_entities[i].flags && !(quote_style & basic_entities[i].flags)) {
+			continue;
+		}
+		basic_entities_dec[j].charcode = basic_entities[i].charcode;
+		memcpy(basic_entities_dec[j].entity, basic_entities[i].entity, basic_entities[i].entitylen + 1);
+		basic_entities_dec[j].entitylen = basic_entities[i].entitylen;
+		j++;
+	}
+	basic_entities_dec[j].charcode = '&';
+	basic_entities_dec[j].entitylen = sizeof("&amp;") - 1;
+	memcpy(basic_entities_dec[j].entity, "&amp;", sizeof("&amp;"));
+	i = j + 1;
+	
+	do {
+		int l = e - p;
+	
+		for (j = 0; j < i; j++) {
+			if (basic_entities_dec[j].entitylen > l) {
+				continue;
+			}
+			if (!memcmp(p, basic_entities_dec[j].entity, basic_entities_dec[j].entitylen)) {
+				int e_len = basic_entities_dec[j].entitylen - 1;
+		
+				*p++ = basic_entities_dec[j].charcode;
+				memmove(p, p + e_len, (e - p - e_len));
+				e -= e_len;
+				goto done;
+			}
+		}
+		p++;
+
+done:
+		if (p >= e) {
+			break;
+		}
+	} while ((p = memchr(p, '&', (e - p))));
+
+	new_len = e - new_str;
+
+	new_str[new_len] = '\0';
+	RETURN_STRINGL(new_str, new_len, 0);
+}
+/* }}} */
+
 /* {{{ proto string html_entity_decode(string string [, int quote_style][, string charset])
    Convert all HTML entities to their applicable characters */
 PHP_FUNCTION(html_entity_decode)
