@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: http.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id: http.c,v 1.14.2.3 2006/01/01 12:50:14 sniper Exp $ */
 
 #include "php_http.h"
 #include "php_ini.h"
@@ -29,12 +29,10 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 				const char *num_prefix, int num_prefix_len,
 				const char *key_prefix, int key_prefix_len,
 				const char *key_suffix, int key_suffix_len,
-			  zval *type, char *arg_sep, int enc_type TSRMLS_DC)
+				zval *type, char *arg_sep TSRMLS_DC)
 {
-	char *key = NULL;
-	char *ekey, *newprefix, *p;
-	int arg_sep_len, ekey_len, key_type, newprefix_len;
-	uint key_len;
+	char *key = NULL, *ekey, *newprefix, *p;
+	int arg_sep_len, key_len, ekey_len, key_type, newprefix_len;
 	ulong idx;
 	zval **zdata = NULL, *copyzval;
 
@@ -66,29 +64,25 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 
 		/* handling for private & protected object properties */
 		if (key && *key == '\0' && type != NULL) {
-			const char *tmp;
+			char *tmp;
 
 			zend_object *zobj = zend_objects_get_address(type TSRMLS_CC);
-			if (zend_check_property_access(zobj, key, key_len-1 TSRMLS_CC) != SUCCESS) {
+			if (zend_check_property_access(zobj, key TSRMLS_CC) != SUCCESS) {
 				/* private or protected property access outside of the class */
 				continue;
 			}
-			zend_unmangle_property_name(key, key_len-1, &tmp, (const char**)&key);
+			zend_unmangle_property_name_ex(key, key_len, &tmp, &key);
 			key_len = strlen(key);		
 		}
 
 		if (zend_hash_get_current_data_ex(ht, (void **)&zdata, NULL) == FAILURE || !zdata || !(*zdata)) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error traversing form data array");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error traversing form data array.");
 			return FAILURE;
 		}
 		if (Z_TYPE_PP(zdata) == IS_ARRAY || Z_TYPE_PP(zdata) == IS_OBJECT) {
 			if (key_type == HASH_KEY_IS_STRING) {
-				if (enc_type == PHP_QUERY_RFC3986) {
-					ekey = php_raw_url_encode(key, key_len, &ekey_len);
-				} else {
-					ekey = php_url_encode(key, key_len, &ekey_len);
-				}
-				newprefix_len = key_suffix_len + ekey_len + key_prefix_len + 3 /* %5B */;
+				ekey = php_url_encode(key, key_len, &ekey_len);
+				newprefix_len = key_suffix_len + ekey_len + key_prefix_len + 1;
 				newprefix = emalloc(newprefix_len + 1);
 				p = newprefix;
 
@@ -105,14 +99,13 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 					memcpy(p, key_suffix, key_suffix_len);
 					p += key_suffix_len;
 				}
-				*(p++) = '%';
-				*(p++) = '5';
-				*(p++) = 'B';
+
+				*(p++) = '[';
 				*p = '\0';
 			} else {
 				/* Is an integer key */
-				ekey_len = spprintf(&ekey, 0, "%ld", idx);
-				newprefix_len = key_prefix_len + num_prefix_len + ekey_len + key_suffix_len + 3 /* %5B */;
+				ekey_len = spprintf(&ekey, 12, "%ld", idx);
+				newprefix_len = key_prefix_len + num_prefix_len + ekey_len + key_suffix_len + 1;
 				newprefix = emalloc(newprefix_len + 1);
 				p = newprefix;
 
@@ -132,13 +125,11 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 					memcpy(p, key_suffix, key_suffix_len);
 					p += key_suffix_len;
 				}
-				*(p++) = '%';
-				*(p++) = '5';
-				*(p++) = 'B';
+				*(p++) = '[';
 				*p = '\0';
 			}
 			ht->nApplyCount++;
-			php_url_encode_hash_ex(HASH_OF(*zdata), formstr, NULL, 0, newprefix, newprefix_len, "%5D", 3, (Z_TYPE_PP(zdata) == IS_OBJECT ? *zdata : NULL), arg_sep, enc_type TSRMLS_CC);
+			php_url_encode_hash_ex(HASH_OF(*zdata), formstr, NULL, 0, newprefix, newprefix_len, "]", 1, (Z_TYPE_PP(zdata) == IS_OBJECT ? *zdata : NULL), arg_sep TSRMLS_CC);
 			ht->nApplyCount--;
 			efree(newprefix);
 		} else if (Z_TYPE_PP(zdata) == IS_NULL || Z_TYPE_PP(zdata) == IS_RESOURCE) {
@@ -151,11 +142,7 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 			/* Simple key=value */
 			smart_str_appendl(formstr, key_prefix, key_prefix_len);
 			if (key_type == HASH_KEY_IS_STRING) {
-				if (enc_type == PHP_QUERY_RFC3986) {
-					ekey = php_raw_url_encode(key, key_len, &ekey_len);
-				} else {
-					ekey = php_url_encode(key, key_len, &ekey_len);
-				}
+				ekey = php_url_encode(key, key_len, &ekey_len);
 				smart_str_appendl(formstr, ekey, ekey_len);
 				efree(ekey);
 			} else {
@@ -163,7 +150,7 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 				if (num_prefix) {
 					smart_str_appendl(formstr, num_prefix, num_prefix_len);
 				}
-				ekey_len = spprintf(&ekey, 0, "%ld", idx);
+				ekey_len = spprintf(&ekey, 12, "%ld", idx);
 				smart_str_appendl(formstr, ekey, ekey_len);
 				efree(ekey);
 			}
@@ -171,18 +158,14 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 			smart_str_appendl(formstr, "=", 1);
 			switch (Z_TYPE_PP(zdata)) {
 				case IS_STRING:
-					if (enc_type == PHP_QUERY_RFC3986) {
-						ekey = php_raw_url_encode(Z_STRVAL_PP(zdata), Z_STRLEN_PP(zdata), &ekey_len);
-					} else {
-						ekey = php_url_encode(Z_STRVAL_PP(zdata), Z_STRLEN_PP(zdata), &ekey_len);						
-					}
+					ekey = php_url_encode(Z_STRVAL_PP(zdata), Z_STRLEN_PP(zdata), &ekey_len);
 					break;
 				case IS_LONG:
 				case IS_BOOL:
-					ekey_len = spprintf(&ekey, 0, "%ld", Z_LVAL_PP(zdata));
+					ekey_len = spprintf(&ekey, 12, "%ld", Z_LVAL_PP(zdata));
 					break;
 				case IS_DOUBLE:
-					ekey_len = spprintf(&ekey, 0, "%.*G", (int) EG(precision), Z_DVAL_PP(zdata));
+					ekey_len = spprintf(&ekey, 48, "%.*G", (int) EG(precision), Z_DVAL_PP(zdata));
 					break;
 				default:
 					/* fall back on convert to string */
@@ -190,11 +173,7 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 					*copyzval = **zdata;
 					zval_copy_ctor(copyzval);
 					convert_to_string_ex(&copyzval);
-					if (enc_type == PHP_QUERY_RFC3986) {
-						ekey = php_raw_url_encode(Z_STRVAL_P(copyzval), Z_STRLEN_P(copyzval), &ekey_len);
-					} else {
-						ekey = php_url_encode(Z_STRVAL_P(copyzval), Z_STRLEN_P(copyzval), &ekey_len);
-					}
+					ekey = php_url_encode(Z_STRVAL_P(copyzval), Z_STRLEN_P(copyzval), &ekey_len);
 					zval_ptr_dtor(&copyzval);
 			}
 			smart_str_appendl(formstr, ekey, ekey_len);
@@ -206,26 +185,26 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 }
 /* }}} */
 
-/* {{{ proto string http_build_query(mixed formdata [, string prefix [, string arg_separator [, int enc_type]]])
+/* {{{ proto string http_build_query(mixed formdata [, string prefix [, string arg_separator]])
    Generates a form-encoded query string from an associative array or object. */
 PHP_FUNCTION(http_build_query)
 {
 	zval *formdata;
 	char *prefix = NULL, *arg_sep=NULL;
-	int arg_sep_len = 0, prefix_len = 0;
+	int arg_sep_len, prefix_len = 0;
 	smart_str formstr = {0};
-	long enc_type = PHP_QUERY_RFC1738;
+	
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|ssl", &formdata, &prefix, &prefix_len, &arg_sep, &arg_sep_len, &enc_type) != SUCCESS) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|ss", &formdata, &prefix, &prefix_len, &arg_sep, &arg_sep_len) != SUCCESS) {
 		RETURN_FALSE;
 	}
 
 	if (Z_TYPE_P(formdata) != IS_ARRAY && Z_TYPE_P(formdata) != IS_OBJECT) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Parameter 1 expected to be Array or Object.  Incorrect value given");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Parameter 1 expected to be Array or Object.  Incorrect value given.");
 		RETURN_FALSE;
 	}
 
-	if (php_url_encode_hash_ex(HASH_OF(formdata), &formstr, prefix, prefix_len, NULL, 0, NULL, 0, (Z_TYPE_P(formdata) == IS_OBJECT ? formdata : NULL), arg_sep, enc_type TSRMLS_CC) == FAILURE) {
+	if (php_url_encode_hash_ex(HASH_OF(formdata), &formstr, prefix, prefix_len, NULL, 0, NULL, 0, (Z_TYPE_P(formdata) == IS_OBJECT ? formdata : NULL), arg_sep TSRMLS_CC) == FAILURE) {
 		if (formstr.c) {
 			efree(formstr.c);
 		}
@@ -233,7 +212,7 @@ PHP_FUNCTION(http_build_query)
 	}
 
 	if (!formstr.c) {
-		RETURN_EMPTY_STRING();
+		RETURN_NULL();
 	}
 
 	smart_str_0(&formstr);
