@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,9 +14,10 @@
    +----------------------------------------------------------------------+
    | Author:  Sascha Schumann <sascha@schumann.cx>                        |
    +----------------------------------------------------------------------+
-*/
+ */
 
-/* $Id: incomplete_class.c,v 1.28.2.2.2.2.2.3 2008/12/31 11:15:45 sebastian Exp $ */
+
+/* $Id: incomplete_class.c,v 1.28.2.1 2006/01/01 12:50:15 sniper Exp $ */
 
 #include "php.h"
 #include "basic_functions.h"
@@ -30,6 +31,7 @@
 		"unserialize() gets called or provide a __autoload() function " \
 		"to load the class definition "
 
+
 static zend_object_handlers php_incomplete_object_handlers;
 
 /* {{{ incomplete_class_message
@@ -37,72 +39,57 @@ static zend_object_handlers php_incomplete_object_handlers;
 static void incomplete_class_message(zval *object, int error_type TSRMLS_DC)
 {
 	char *class_name;
-	zend_bool class_name_alloced = 1;
 
 	class_name = php_lookup_class_name(object, NULL);
 	
 	if (!class_name) {
-		class_name_alloced = 0;
 		class_name = "unknown";
 	}
 	
 	php_error_docref(NULL TSRMLS_CC, error_type, INCOMPLETE_CLASS_MSG, class_name);
-
-	if (class_name_alloced) {
-		efree(class_name);
-	}
 }
 /* }}} */
 
-static zval *incomplete_class_get_property(zval *object, zval *member, int type TSRMLS_DC) /* {{{ */
+static zval *incomplete_class_get_property(zval *object, zval *member, int type TSRMLS_DC)
 {
 	incomplete_class_message(object, E_NOTICE TSRMLS_CC);
-	
-	if (type == BP_VAR_W || type == BP_VAR_RW) {
+	if(type == BP_VAR_W || type == BP_VAR_RW) {
 		return EG(error_zval_ptr);
 	} else {
 		return EG(uninitialized_zval_ptr);
 	}
 }
-/* }}} */
 
-static void incomplete_class_write_property(zval *object, zval *member, zval *value TSRMLS_DC) /* {{{ */
+static void incomplete_class_write_property(zval *object, zval *member, zval *value TSRMLS_DC)
 {
 	incomplete_class_message(object, E_NOTICE TSRMLS_CC);
 }
-/* }}} */
 	
-static zval **incomplete_class_get_property_ptr_ptr(zval *object, zval *member TSRMLS_DC) /* {{{ */
+static zval **incomplete_class_get_property_ptr_ptr(zval *object, zval *member TSRMLS_DC)
 {
 	incomplete_class_message(object, E_NOTICE TSRMLS_CC);
 	return &EG(error_zval_ptr);
 }
-/* }}} */
 
-static void incomplete_class_unset_property(zval *object, zval *member TSRMLS_DC) /* {{{ */
+static void incomplete_class_unset_property(zval *object, zval *member TSRMLS_DC)
 {
 	incomplete_class_message(object, E_NOTICE TSRMLS_CC);
 }
-/* }}} */
 
-static int incomplete_class_has_property(zval *object, zval *member, int check_empty TSRMLS_DC) /* {{{ */
+static int incomplete_class_has_property(zval *object, zval *member, int check_empty TSRMLS_DC)
 {
 	incomplete_class_message(object, E_NOTICE TSRMLS_CC);
 	return 0;
 }
-/* }}} */
 
-static union _zend_function *incomplete_class_get_method(zval **object, char *method, int method_len TSRMLS_DC) /* {{{ */
-{
+static union _zend_function *incomplete_class_get_method(zval **object, char *method, int method_len TSRMLS_DC) {
 	incomplete_class_message(*object, E_ERROR TSRMLS_CC);
 	return NULL;
 }
-/* }}} */
 
 /* {{{ php_create_incomplete_class
  */
-static zend_object_value php_create_incomplete_object(zend_class_entry *class_type TSRMLS_DC)
-{
+static zend_object_value php_create_incomplete_object(zend_class_entry *class_type TSRMLS_DC) {
 	zend_object *object;
 	zend_object_value value;
 	
@@ -114,7 +101,7 @@ static zend_object_value php_create_incomplete_object(zend_class_entry *class_ty
 	return value;
 }
 
-PHPAPI zend_class_entry *php_create_incomplete_class(TSRMLS_D)
+zend_class_entry *php_create_incomplete_class(TSRMLS_D)
 {
 	zend_class_entry incomplete_class;
 
@@ -135,7 +122,7 @@ PHPAPI zend_class_entry *php_create_incomplete_class(TSRMLS_D)
 
 /* {{{ php_lookup_class_name
  */
-PHPAPI char *php_lookup_class_name(zval *object, zend_uint *nlen)
+char *php_lookup_class_name(zval *object, zend_uint *nlen)
 {
 	zval **val;
 	char *retval = NULL;
@@ -147,18 +134,17 @@ PHPAPI char *php_lookup_class_name(zval *object, zend_uint *nlen)
 	if (zend_hash_find(object_properties, MAGIC_MEMBER, sizeof(MAGIC_MEMBER), (void **) &val) == SUCCESS) {
 		retval = estrndup(Z_STRVAL_PP(val), Z_STRLEN_PP(val));
 
-		if (nlen) {
+		if (nlen)
 			*nlen = Z_STRLEN_PP(val);
-		}
 	}
 
-	return retval;
+	return (retval);
 }
 /* }}} */
 
 /* {{{ php_store_class_name
  */
-PHPAPI void php_store_class_name(zval *object, const char *name, zend_uint len)
+void php_store_class_name(zval *object, const char *name, zend_uint len)
 {
 	zval *val;
 	TSRMLS_FETCH();
