@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    |          Hartmut Holzgraefe <hholzgra@php.net>                       |
    +----------------------------------------------------------------------+
  */
-/* $Id: php_fopen_wrapper.c,v 1.45.2.4.2.7.2.7 2008/12/31 11:15:45 sebastian Exp $ */
+/* $Id: php_fopen_wrapper.c,v 1.45.2.3 2006/01/01 12:50:15 sniper Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,11 +48,17 @@ static int php_stream_output_close(php_stream *stream, int close_handle TSRMLS_D
 	return 0;
 }
 
+static int php_stream_output_flush(php_stream *stream TSRMLS_DC)
+{
+	sapi_flush(TSRMLS_C);
+	return 0;
+}
+
 php_stream_ops php_stream_output_ops = {
 	php_stream_output_write,
 	php_stream_output_read,
 	php_stream_output_close,
-	NULL, /* flush */
+	php_stream_output_flush,
 	"Output",
 	NULL, /* seek */
 	NULL, /* cast */
@@ -127,7 +133,6 @@ static void php_stream_apply_filter_list(php_stream *stream, char *filterlist, i
 
 	p = php_strtok_r(filterlist, "|", &token);
 	while (p) {
-		php_url_decode(p, strlen(p));
 		if (read_chain) {
 			if ((temp_filter = php_stream_filter_create(p, NULL, php_stream_is_persistent(stream) TSRMLS_CC))) {
 				php_stream_filter_append(&stream->readfilters, temp_filter);
@@ -153,100 +158,24 @@ php_stream * php_stream_url_wrap_php(php_stream_wrapper *wrapper, char *path, ch
 	int mode_rw = 0;
 	php_stream * stream = NULL;
 	char *p, *token, *pathdup;
-	long max_memory;
-	FILE *file = NULL;
 
-	if (!strncasecmp(path, "php://", 6)) {
+	if (!strncasecmp(path, "php://", 6))
 		path += 6;
-	}
-	
-	if (!strncasecmp(path, "temp", 4)) {
-		path += 4;
-		max_memory = PHP_STREAM_MAX_MEM;
-		if (!strncasecmp(path, "/maxmemory:", 11)) {
-			path += 11;
-			max_memory = strtol(path, NULL, 10);
-			if (max_memory < 0) {
-				php_error_docref(NULL TSRMLS_CC, E_RECOVERABLE_ERROR, "Max memory must be >= 0");
-				return NULL;
-			}
-		}
-		if (strpbrk(mode, "wa+")) {
-			mode_rw = TEMP_STREAM_DEFAULT;
-		} else {
-			mode_rw = TEMP_STREAM_READONLY;
-		}
-		return php_stream_temp_create(mode_rw, max_memory);		
-	}
-	
-	if (!strcasecmp(path, "memory")) {
-		if (strpbrk(mode, "wa+")) {
-			mode_rw = TEMP_STREAM_DEFAULT;
-		} else {
-			mode_rw = TEMP_STREAM_READONLY;
-		}
-		return php_stream_memory_create(mode_rw);
-	}
 	
 	if (!strcasecmp(path, "output")) {
 		return php_stream_alloc(&php_stream_output_ops, NULL, 0, "wb");
 	}
 	
 	if (!strcasecmp(path, "input")) {
-		if ((options & STREAM_OPEN_FOR_INCLUDE) && !PG(allow_url_include) ) {
-			if (options & REPORT_ERRORS) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "URL file-access is disabled in the server configuration");
-			}
-			return NULL;
-		}
 		return php_stream_alloc(&php_stream_input_ops, ecalloc(1, sizeof(off_t)), 0, "rb");
-	}
+	}  
 	
 	if (!strcasecmp(path, "stdin")) {
-		if ((options & STREAM_OPEN_FOR_INCLUDE) && !PG(allow_url_include) ) {
-			if (options & REPORT_ERRORS) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "URL file-access is disabled in the server configuration");
-			}
-			return NULL;
-		}
-		if (!strcmp(sapi_module.name, "cli")) {
-			static int cli_in = 0;
-			fd = STDIN_FILENO;
-			if (cli_in) {
-				fd = dup(fd);
-			} else {
-				cli_in = 1;
-				file = stdin;
-			}
-		} else {
-			fd = dup(STDIN_FILENO);
-		}
+		fd = !strcmp(sapi_module.name, "cli") ? STDIN_FILENO : dup(STDIN_FILENO);
 	} else if (!strcasecmp(path, "stdout")) {
-		if (!strcmp(sapi_module.name, "cli")) {
-			static int cli_out = 0;
-			fd = STDOUT_FILENO;
-			if (cli_out++) {
-				fd = dup(fd);
-			} else {
-				cli_out = 1;
-				file = stdout;
-			}
-		} else {
-			fd = dup(STDOUT_FILENO);
-		}
+		fd = !strcmp(sapi_module.name, "cli") ? STDOUT_FILENO : dup(STDOUT_FILENO);
 	} else if (!strcasecmp(path, "stderr")) {
-		if (!strcmp(sapi_module.name, "cli")) {
-			static int cli_err = 0;
-			fd = STDERR_FILENO;
-			if (cli_err++) {
-				fd = dup(fd);
-			} else {
-				cli_err = 1;
-				file = stderr;
-			}
-		} else {
-			fd = dup(STDERR_FILENO);
-		}
+		fd = !strcmp(sapi_module.name, "cli") ? STDERR_FILENO : dup(STDERR_FILENO);
 	} else if (!strncasecmp(path, "filter/", 7)) {
 		/* Save time/memory when chain isn't specified */
 		if (strchr(mode, 'r') || strchr(mode, '+')) {
@@ -258,7 +187,7 @@ php_stream * php_stream_url_wrap_php(php_stream_wrapper *wrapper, char *path, ch
 		pathdup = estrndup(path + 6, strlen(path + 6));
 		p = strstr(pathdup, "/resource=");
 		if (!p) {
-			php_error_docref(NULL TSRMLS_CC, E_RECOVERABLE_ERROR, "No URL resource specified");
+			php_error_docref(NULL TSRMLS_CC, E_ERROR, "No URL resource specified.");
 			efree(pathdup);
 			return NULL;
 		}
@@ -285,7 +214,6 @@ php_stream * php_stream_url_wrap_php(php_stream_wrapper *wrapper, char *path, ch
 		return stream;
  	} else {
 		/* invalid php://thingy */
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid php:// URL specified");
 		return NULL;
 	}
 	
@@ -294,28 +222,10 @@ php_stream * php_stream_url_wrap_php(php_stream_wrapper *wrapper, char *path, ch
 		/* failed to dup */
 		return NULL;
 	}
-
-#if defined(S_IFSOCK) && !defined(WIN32) && !defined(__BEOS__)
-	do {
-		struct stat st;
-		memset(&st, 0, sizeof(st));
-		if (fstat(fd, &st) == 0 && (st.st_mode & S_IFMT) == S_IFSOCK) {
-			stream = php_stream_sock_open_from_socket(fd, NULL);
-			if (stream) {
-				stream->ops = &php_stream_socket_ops;
-				return stream;
-			}
-		}
-	} while (0);
-#endif
-
-	if (file) {
-		stream = php_stream_fopen_from_file(file, mode);
-	} else {
-		stream = php_stream_fopen_from_fd(fd, mode, NULL);
-		if (stream == NULL) {
-			close(fd);
-		}
+	
+	stream = php_stream_fopen_from_fd(fd, mode, NULL);
+	if (stream == NULL) {
+		close(fd);
 	}
  
 	return stream;
