@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2008 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    |          Jaakko Hyvtti <jaakko@hyvatti.iki.fi>                      | 
    +----------------------------------------------------------------------+
  */
-/* $Id: reg.c,v 1.66.2.8.2.4 2007/12/31 07:22:52 sebastian Exp $ */
+/* $Id: reg.c,v 1.82.2.2 2006/01/01 12:50:15 sniper Exp $ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -184,7 +184,7 @@ static void php_reg_eprint(int err, regex_t *re) {
  */
 static void php_ereg(INTERNAL_FUNCTION_PARAMETERS, int icase)
 {
-	pval **regex,			/* Regular expression */
+	zval **regex,			/* Regular expression */
 		**findin,		/* String to apply expression to */
 		**array = NULL;		/* Optional register array */
 	regex_t re;
@@ -380,7 +380,7 @@ PHPAPI char *php_reg_replace(const char *pattern, const char *replace, const cha
 			walkbuf = &buf[tmp + subs[0].rm_so];
 			walk = replace;
 			while (*walk) {
-				if ('\\' == *walk && isdigit(walk[1]) && walk[1] - '0' <= re.re_nsub) {
+				if ('\\' == *walk && isdigit(walk[1]) && walk[1] - '0' <= (int)re.re_nsub) {
 					if (subs[walk[1] - '0'].rm_so > -1 && subs[walk[1] - '0'].rm_eo > -1
 						/* this next case shouldn't happen. it does. */
 						&& subs[walk[1] - '0'].rm_so <= subs[walk[1] - '0'].rm_eo) {
@@ -442,7 +442,7 @@ PHPAPI char *php_reg_replace(const char *pattern, const char *replace, const cha
  */
 static void php_ereg_replace(INTERNAL_FUNCTION_PARAMETERS, int icase)
 {
-	pval **arg_pattern,
+	zval **arg_pattern,
 		**arg_replace,
 		**arg_string;
 	char *pattern;
@@ -459,7 +459,7 @@ static void php_ereg_replace(INTERNAL_FUNCTION_PARAMETERS, int icase)
 		if (Z_STRVAL_PP(arg_pattern) && Z_STRLEN_PP(arg_pattern))
 			pattern = estrndup(Z_STRVAL_PP(arg_pattern), Z_STRLEN_PP(arg_pattern));
 		else
-			pattern = empty_string;
+			pattern = STR_EMPTY_ALLOC();
 	} else {
 		convert_to_long_ex(arg_pattern);
 		pattern = emalloc(2);
@@ -471,7 +471,7 @@ static void php_ereg_replace(INTERNAL_FUNCTION_PARAMETERS, int icase)
 		if (Z_STRVAL_PP(arg_replace) && Z_STRLEN_PP(arg_replace))
 			replace = estrndup(Z_STRVAL_PP(arg_replace), Z_STRLEN_PP(arg_replace));
 		else
-			replace = empty_string;
+			replace = STR_EMPTY_ALLOC();
 	} else {
 		convert_to_long_ex(arg_replace);
 		replace = emalloc(2);
@@ -483,7 +483,7 @@ static void php_ereg_replace(INTERNAL_FUNCTION_PARAMETERS, int icase)
 	if (Z_STRVAL_PP(arg_string) && Z_STRLEN_PP(arg_string))
 		string = estrndup(Z_STRVAL_PP(arg_string), Z_STRLEN_PP(arg_string));
 	else
-		string = empty_string;
+		string = STR_EMPTY_ALLOC();
 
 	/* do the actual work */
 	ret = php_reg_replace(pattern, replace, string, icase, 1);
@@ -558,7 +558,7 @@ static void php_split(INTERNAL_FUNCTION_PARAMETERS, int icase)
 	while ((count == -1 || count > 1) && !(err = regexec(&re, strp, 1, subs, 0))) {
 		if (subs[0].rm_so == 0 && subs[0].rm_eo) {
 			/* match is at start of string, return empty string */
-			add_next_index_stringl(return_value, empty_string, 0, 1);
+			add_next_index_stringl(return_value, "", 0, 1);
 			/* skip ahead the length of the regex match */
 			strp += subs[0].rm_eo;
 		} else if (subs[0].rm_so == 0 && subs[0].rm_eo == 0) {
