@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: streamsfuncs.c 293995 2010-01-25 15:57:24Z johannes $ */
+/* $Id: streamsfuncs.c,v 1.58.2.1 2006/01/01 12:50:15 sniper Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -33,11 +33,8 @@
 
 #ifndef PHP_WIN32
 #define php_select(m, r, w, e, t)	select(m, r, w, e, t)
-typedef unsigned long long php_timeout_ull;
 #else
 #include "win32/select.h"
-#include "win32/sockets.h"
-typedef unsigned __int64 php_timeout_ull;
 #endif
 
 static php_stream_context *decode_context_param(zval *contextresource TSRMLS_DC);
@@ -70,18 +67,13 @@ PHP_FUNCTION(stream_socket_pair)
 	s1 = php_stream_sock_open_from_socket(pair[0], 0);
 	s2 = php_stream_sock_open_from_socket(pair[1], 0);
 
-	/* set the __exposed flag. 
-	 * php_stream_to_zval() does, add_next_index_resource() does not */
-	php_stream_auto_cleanup(s1);
-	php_stream_auto_cleanup(s2);
-
 	add_next_index_resource(return_value, php_stream_get_resource_id(s1));
 	add_next_index_resource(return_value, php_stream_get_resource_id(s2));
 }
 /* }}} */
 #endif
 
-/* {{{ proto resource stream_socket_client(string remoteaddress [, long &errcode [, string &errstring [, double timeout [, long flags [, resource context]]]]])
+/* {{{ proto resource stream_socket_client(string remoteaddress [, long &errcode, string &errstring, double timeout, long flags, resource context])
    Open a client connection to a remote address */
 PHP_FUNCTION(stream_socket_client)
 {
@@ -89,7 +81,7 @@ PHP_FUNCTION(stream_socket_client)
 	int host_len;
 	zval *zerrno = NULL, *zerrstr = NULL, *zcontext = NULL;
 	double timeout = FG(default_socket_timeout);
-	php_timeout_ull conv;
+	unsigned long conv;
 	struct timeval tv;
 	char *hashkey = NULL;
 	php_stream *stream = NULL;
@@ -100,22 +92,18 @@ PHP_FUNCTION(stream_socket_client)
 
 	RETVAL_FALSE;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zzdlr", &host, &host_len, &zerrno, &zerrstr, &timeout, &flags, &zcontext) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zzd!lr", &host, &host_len, &zerrno, &zerrstr, &timeout, &flags, &zcontext) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
 	context = php_stream_context_from_zval(zcontext, flags & PHP_FILE_NO_DEFAULT_CONTEXT);
 
-	if (context) {
-		zend_list_addref(context->rsrc_id);
-	}
-
 	if (flags & PHP_STREAM_CLIENT_PERSISTENT) {
 		spprintf(&hashkey, 0, "stream_socket_client__%s", host);
 	}
 	
 	/* prepare the timeout value for use */
-	conv = (php_timeout_ull) (timeout * 1000000.0);
+	conv = (unsigned long) (timeout * 1000000.0);
 	tv.tv_sec = conv / 1000000;
 	tv.tv_usec = conv % 1000000;
 
@@ -132,7 +120,6 @@ PHP_FUNCTION(stream_socket_client)
 			STREAM_XPORT_CLIENT | (flags & PHP_STREAM_CLIENT_CONNECT ? STREAM_XPORT_CONNECT : 0) |
 			(flags & PHP_STREAM_CLIENT_ASYNC_CONNECT ? STREAM_XPORT_CONNECT_ASYNC : 0),
 			hashkey, &tv, context, &errstr, &err);
-		
 
 	if (stream == NULL) {
 		/* host might contain binary characters */
@@ -166,11 +153,14 @@ PHP_FUNCTION(stream_socket_client)
 	}
 	
 	php_stream_to_zval(stream, return_value);
-	
+
+	if (zcontext) {
+		zend_list_addref(Z_RESVAL_P(zcontext));
+	}
 }
 /* }}} */
 
-/* {{{ proto resource stream_socket_server(string localaddress [, long &errcode [, string &errstring [, long flags [, resource context]]]])
+/* {{{ proto resource stream_socket_server(string localaddress [, long &errcode, string &errstring, long flags, resource context])
    Create a server socket bound to localaddress */
 PHP_FUNCTION(stream_socket_server)
 {
@@ -190,10 +180,6 @@ PHP_FUNCTION(stream_socket_server)
 	}
 	
 	context = php_stream_context_from_zval(zcontext, flags & PHP_FILE_NO_DEFAULT_CONTEXT);
-	
-	if (context) {
-		zend_list_addref(context->rsrc_id);
-	}
 
 	if (zerrno)	{
 		zval_dtor(zerrno);
@@ -207,7 +193,7 @@ PHP_FUNCTION(stream_socket_server)
 	stream = php_stream_xport_create(host, host_len, ENFORCE_SAFE_MODE | REPORT_ERRORS,
 			STREAM_XPORT_SERVER | flags,
 			NULL, NULL, context, &errstr, &err);
-			
+
 	if (stream == NULL) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to connect to %s (%s)", host, errstr == NULL ? "Unknown error" : errstr);
 	}
@@ -221,8 +207,6 @@ PHP_FUNCTION(stream_socket_server)
 			/* no need to dup; we need to efree buf anyway */
 			zval_dtor(zerrstr);
 			ZVAL_STRING(zerrstr, errstr, 0);
-		} else if (errstr) {
-			efree(errstr);
 		}
 		RETURN_FALSE;
 	}
@@ -232,49 +216,51 @@ PHP_FUNCTION(stream_socket_server)
 	}
 	
 	php_stream_to_zval(stream, return_value);
+
+	if (zcontext) {
+		zend_list_addref(Z_RESVAL_P(zcontext));
+	}
 }
 /* }}} */
 
-/* {{{ proto resource stream_socket_accept(resource serverstream, [ double timeout [, string &peername ]])
+/* {{{ proto resource stream_socket_accept(resource serverstream, [ double timeout, string &peername ])
    Accept a client connection from a server socket */
 PHP_FUNCTION(stream_socket_accept)
 {
 	double timeout = FG(default_socket_timeout);
-	zval *zpeername = NULL;
-	char *peername = NULL;
-	int peername_len;
-	php_timeout_ull conv;
+	zval *peername = NULL;
+	unsigned long conv;
 	struct timeval tv;
 	php_stream *stream = NULL, *clistream = NULL;
 	zval *zstream;
 
 	char *errstr = NULL;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|dz", &zstream, &timeout, &zpeername) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|dz", &zstream, &timeout, &peername) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
 	php_stream_from_zval(stream, &zstream);
 	
 	/* prepare the timeout value for use */
-	conv = (php_timeout_ull) (timeout * 1000000.0);
+	conv = (unsigned long) (timeout * 1000000.0);
 	tv.tv_sec = conv / 1000000;
 	tv.tv_usec = conv % 1000000;
 
-	if (zpeername) {
-		zval_dtor(zpeername);
-		ZVAL_NULL(zpeername);
+	if (peername) {
+		zval_dtor(peername);
+		ZVAL_NULL(peername);
 	}
 
 	if (0 == php_stream_xport_accept(stream, &clistream,
-				zpeername ? &peername : NULL,
-				zpeername ? &peername_len : NULL,
+				peername ? &Z_STRVAL_P(peername) : NULL,
+				peername ? &Z_STRLEN_P(peername) : NULL,
 				NULL, NULL,
 				&tv, &errstr
 				TSRMLS_CC) && clistream) {
-		
+
 		if (peername) {
-			ZVAL_STRINGL(zpeername, peername, peername_len, 0);
+			Z_TYPE_P(peername) = IS_STRING;
 		}
 		php_stream_to_zval(clistream, return_value);
 	} else {
@@ -295,8 +281,6 @@ PHP_FUNCTION(stream_socket_get_name)
 	php_stream *stream;
 	zval *zstream;
 	zend_bool want_peer;
-	char *name = NULL;
-	int name_len;
 	
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rb", &zstream, &want_peer) == FAILURE) {
 		RETURN_FALSE;
@@ -304,15 +288,15 @@ PHP_FUNCTION(stream_socket_get_name)
 	
 	php_stream_from_zval(stream, &zstream);
 
+	Z_TYPE_P(return_value) = IS_STRING;
+	
 	if (0 != php_stream_xport_get_name(stream, want_peer,
-				&name,
-				&name_len,
+				&Z_STRVAL_P(return_value),
+				&Z_STRLEN_P(return_value),
 				NULL, NULL
 				TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
-
-	RETURN_STRINGL(name, name_len, 0);
 }
 /* }}} */
 
@@ -323,7 +307,7 @@ PHP_FUNCTION(stream_socket_sendto)
 	php_stream *stream;
 	zval *zstream;
 	long flags = 0;
-	char *data, *target_addr = NULL;
+	char *data, *target_addr;
 	int datalen, target_addr_len = 0;
 	php_sockaddr_storage sa;
 	socklen_t sl = 0;
@@ -351,8 +335,6 @@ PHP_FUNCTION(stream_socket_recvfrom)
 {
 	php_stream *stream;
 	zval *zstream, *zremote = NULL;
-	char *remote_addr = NULL;
-	int remote_addr_len;
 	long to_read = 0;
 	char *read_buf;
 	long flags = 0;
@@ -367,29 +349,31 @@ PHP_FUNCTION(stream_socket_recvfrom)
 	if (zremote) {
 		zval_dtor(zremote);
 		ZVAL_NULL(zremote);
+		Z_STRLEN_P(zremote) = 0;
 	}
 
 	if (to_read <= 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0.");
 		RETURN_FALSE;
 	}
 	
-	read_buf = safe_emalloc(1, to_read, 1);
+	read_buf = emalloc(to_read + 1);
 	
 	recvd = php_stream_xport_recvfrom(stream, read_buf, to_read, flags, NULL, NULL,
-			zremote ? &remote_addr : NULL,
-			zremote ? &remote_addr_len : NULL
+			zremote ? &Z_STRVAL_P(zremote) : NULL,
+			zremote ? &Z_STRLEN_P(zremote) : NULL
 			TSRMLS_CC);
 
 	if (recvd >= 0) {
-		if (zremote) {
-			ZVAL_STRINGL(zremote, remote_addr, remote_addr_len, 0);
+		if (zremote && Z_STRLEN_P(zremote)) {
+			Z_TYPE_P(zremote) = IS_STRING;
 		}
 		read_buf[recvd] = '\0';
 
 		if (PG(magic_quotes_runtime)) {
 			Z_TYPE_P(return_value) = IS_STRING;
-			Z_STRVAL_P(return_value) = php_addslashes(Z_STRVAL_P(return_value), Z_STRLEN_P(return_value), &Z_STRLEN_P(return_value), 1 TSRMLS_CC);
+			Z_STRVAL_P(return_value) = php_addslashes(Z_STRVAL_P(return_value),
+					Z_STRLEN_P(return_value), &Z_STRLEN_P(return_value), 1 TSRMLS_CC);
 			return;
 		} else {
 			RETURN_STRINGL(read_buf, recvd, 0);
@@ -401,7 +385,7 @@ PHP_FUNCTION(stream_socket_recvfrom)
 }
 /* }}} */
 
-/* {{{ proto string stream_get_contents(resource source [, long maxlen [, long offset]])
+/* {{{ proto long stream_get_contents(resource source [, long maxlen [, long offset]])
    Reads all remaining bytes (or up to maxlen bytes) from a stream and returns them as a string. */
 PHP_FUNCTION(stream_get_contents)
 {
@@ -417,21 +401,23 @@ PHP_FUNCTION(stream_get_contents)
 
 	php_stream_from_zval(stream, &zsrc);
 
-	if ((pos > 0 || (pos == 0 && ZEND_NUM_ARGS() > 2)) && php_stream_seek(stream, pos, SEEK_SET) < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to position %ld in the stream", pos);
+	if (pos > 0 && php_stream_seek(stream, pos, SEEK_SET) < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to %ld position in the stream.", pos);
 		RETURN_FALSE;
 	}
 
-	len = php_stream_copy_to_mem(stream, &contents, maxlen, 0);
-	
-	if (contents) {
-		if (len && PG(magic_quotes_runtime)) {
+	if ((len = php_stream_copy_to_mem(stream, &contents, maxlen, 0)) > 0) {
+		
+		if (PG(magic_quotes_runtime)) {
 			contents = php_addslashes(contents, len, &newlen, 1 TSRMLS_CC); /* 1 = free source string */
 			len = newlen;
 		}
+
 		RETVAL_STRINGL(contents, len, 0);
-	} else {
+	} else if (len == 0) {
 		RETVAL_EMPTY_STRING();
+	} else {
+		RETVAL_FALSE;
 	}
 }
 /* }}} */
@@ -443,8 +429,6 @@ PHP_FUNCTION(stream_copy_to_stream)
 	php_stream *src, *dest;
 	zval *zsrc, *zdest;
 	long maxlen = PHP_STREAM_COPY_ALL, pos = 0;
-	size_t len;
-	int ret;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr|ll", &zsrc, &zdest, &maxlen, &pos) == FAILURE) {
 		RETURN_FALSE;
@@ -454,37 +438,34 @@ PHP_FUNCTION(stream_copy_to_stream)
 	php_stream_from_zval(dest, &zdest);
 
 	if (pos > 0 && php_stream_seek(src, pos, SEEK_SET) < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to position %ld in the stream", pos);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to %ld position in the stream.", pos);
 		RETURN_FALSE;
 	}
 
-	ret = php_stream_copy_to_stream_ex(src, dest, maxlen, &len);
-
-	if (ret != SUCCESS) {
-		RETURN_FALSE;
-	}
-	RETURN_LONG(len);
+	RETURN_LONG(php_stream_copy_to_stream(src, dest, maxlen));
 }
 /* }}} */
 
-/* {{{ proto array stream_get_meta_data(resource fp)
+/* {{{ proto resource stream_get_meta_data(resource fp)
     Retrieves header/meta data from streams/file pointers */
 PHP_FUNCTION(stream_get_meta_data)
 {
-	zval *arg1;
+	zval **arg1;
 	php_stream *stream;
 	zval *newval;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &arg1) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg1) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	php_stream_from_zval(stream, &arg1);
+	php_stream_from_zval(stream, arg1);
 
 	array_init(return_value);
 	
 	if (stream->wrapperdata) {
 		MAKE_STD_ZVAL(newval);
-		MAKE_COPY_ZVAL(&stream->wrapperdata, newval);
+		*newval = *(stream->wrapperdata);
+		zval_copy_ctor(newval);
+		INIT_PZVAL(newval);
 
 		add_assoc_zval(return_value, "wrapper_data", newval);
 	}
@@ -535,19 +516,18 @@ PHP_FUNCTION(stream_get_transports)
 	int stream_xport_len;
 	ulong num_key;
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 0) {
+		WRONG_PARAM_COUNT;
 	}
 
 	if ((stream_xport_hash = php_stream_xport_get_hash())) {
-		HashPosition pos;
 		array_init(return_value);
-		zend_hash_internal_pointer_reset_ex(stream_xport_hash, &pos);
+		zend_hash_internal_pointer_reset(stream_xport_hash);
 		while (zend_hash_get_current_key_ex(stream_xport_hash,
 					&stream_xport, &stream_xport_len,
-					&num_key, 0, &pos) == HASH_KEY_IS_STRING) {
-			add_next_index_stringl(return_value, stream_xport, stream_xport_len - 1, 1);
-			zend_hash_move_forward_ex(stream_xport_hash, &pos);
+					&num_key, 0, NULL) == HASH_KEY_IS_STRING) {
+			add_next_index_stringl(return_value, stream_xport, stream_xport_len, 1);
+			zend_hash_move_forward(stream_xport_hash);
 		}
 	} else {
 		RETURN_FALSE;
@@ -564,18 +544,17 @@ PHP_FUNCTION(stream_get_wrappers)
 	int key_flags, stream_protocol_len = 0;
 	ulong num_key;
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 0) {
+		WRONG_PARAM_COUNT;
 	}
 
 	if ((url_stream_wrappers_hash = php_stream_get_url_stream_wrappers_hash())) {
-		HashPosition pos;
 		array_init(return_value);
-		for (zend_hash_internal_pointer_reset_ex(url_stream_wrappers_hash, &pos);
-			(key_flags = zend_hash_get_current_key_ex(url_stream_wrappers_hash, &stream_protocol, &stream_protocol_len, &num_key, 0, &pos)) != HASH_KEY_NON_EXISTANT;
-			zend_hash_move_forward_ex(url_stream_wrappers_hash, &pos)) {
+		for(zend_hash_internal_pointer_reset(url_stream_wrappers_hash);
+			(key_flags = zend_hash_get_current_key_ex(url_stream_wrappers_hash, &stream_protocol, &stream_protocol_len, &num_key, 0, NULL)) != HASH_KEY_NON_EXISTANT;
+			zend_hash_move_forward(url_stream_wrappers_hash)) {
 				if (key_flags == HASH_KEY_IS_STRING) {
-					add_next_index_stringl(return_value, stream_protocol, stream_protocol_len - 1, 1);
+					add_next_index_stringl(return_value, stream_protocol, stream_protocol_len, 1);
 				}
 		}
 	} else {
@@ -591,7 +570,6 @@ static int stream_array_to_fd_set(zval *stream_array, fd_set *fds, php_socket_t
 	zval **elem;
 	php_stream *stream;
 	php_socket_t this_fd;
-	int cnt = 0;
 
 	if (Z_TYPE_P(stream_array) != IS_ARRAY) {
 		return 0;
@@ -616,10 +594,9 @@ static int stream_array_to_fd_set(zval *stream_array, fd_set *fds, php_socket_t
 			if (this_fd > *max_fd) {
 				*max_fd = this_fd;
 			}
-			cnt++;
 		}
 	}
-	return cnt ? 1 : 0;
+	return 1;
 }
 
 static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
@@ -627,14 +604,13 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 	zval **elem, **dest_elem;
 	php_stream *stream;
 	HashTable *new_hash;
-	php_socket_t this_fd;
-	int ret = 0;
+	int this_fd, ret = 0;
 
 	if (Z_TYPE_P(stream_array) != IS_ARRAY) {
 		return 0;
 	}
 	ALLOC_HASHTABLE(new_hash);
-	zend_hash_init(new_hash, zend_hash_num_elements(Z_ARRVAL_P(stream_array)), NULL, ZVAL_PTR_DTOR, 0);
+	zend_hash_init(new_hash, 0, NULL, ZVAL_PTR_DTOR, 0);
 	
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
 		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
@@ -682,7 +658,7 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
 		return 0;
 	}
 	ALLOC_HASHTABLE(new_hash);
-	zend_hash_init(new_hash, zend_hash_num_elements(Z_ARRVAL_P(stream_array)), NULL, ZVAL_PTR_DTOR, 0);
+	zend_hash_init(new_hash, 0, NULL, ZVAL_PTR_DTOR, 0);
 	
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
 		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
@@ -728,16 +704,16 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
    Runs the select() system call on the sets of streams with a timeout specified by tv_sec and tv_usec */
 PHP_FUNCTION(stream_select)
 {
-	zval			*r_array, *w_array, *e_array, **sec = NULL;
+	zval			*r_array, *w_array, *e_array, *sec = NULL;
 	struct timeval	tv;
 	struct timeval *tv_p = NULL;
 	fd_set			rfds, wfds, efds;
-	php_socket_t	max_fd = 0;
+	int				max_fd = 0;
 	int				retval, sets = 0;
 	long			usec = 0;
 	int				set_count, max_set_count = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a!a!a!Z!|l", &r_array, &w_array, &e_array, &sec, &usec) == FAILURE)
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a!a!a!z!|l", &r_array, &w_array, &e_array, &sec, &usec) == FAILURE)
 		return;
 
 	FD_ZERO(&rfds);
@@ -774,22 +750,14 @@ PHP_FUNCTION(stream_select)
 
 	/* If seconds is not set to null, build the timeval, else we wait indefinitely */
 	if (sec != NULL) {
-		convert_to_long_ex(sec);
-
-		if (Z_LVAL_PP(sec) < 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "The seconds parameter must be greater than 0");
-			RETURN_FALSE;
-		} else if (usec < 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "The microseconds parameter must be greater than 0");
-			RETURN_FALSE;
-		}
+		convert_to_long_ex(&sec);
 
 		/* Solaris + BSD do not like microsecond values which are >= 1 sec */
 		if (usec > 999999) {
-			tv.tv_sec = Z_LVAL_PP(sec) + (usec / 1000000);
+			tv.tv_sec = Z_LVAL_P(sec) + (usec / 1000000);
 			tv.tv_usec = usec % 1000000;			
 		} else {
-			tv.tv_sec = Z_LVAL_PP(sec);
+			tv.tv_sec = Z_LVAL_P(sec);
 			tv.tv_usec = usec;
 		}
 
@@ -803,12 +771,6 @@ PHP_FUNCTION(stream_select)
 
 		retval = stream_array_emulate_read_fd_set(r_array TSRMLS_CC);
 		if (retval > 0) {
-			if (w_array != NULL) {
-				zend_hash_clean(Z_ARRVAL_P(w_array));
-			}
-			if (e_array != NULL) {
-				zend_hash_clean(Z_ARRVAL_P(e_array));
-			}
 			RETURN_LONG(retval);
 		}
 	}
@@ -844,13 +806,12 @@ static void user_space_stream_notifier(php_stream_context *context, int notifyco
 		INIT_ZVAL(zvs[i]);
 		ps[i] = &zvs[i];
 		ptps[i] = &ps[i];
-		MAKE_STD_ZVAL(ps[i]);
 	}
 		
 	ZVAL_LONG(ps[0], notifycode);
 	ZVAL_LONG(ps[1], severity);
 	if (xmsg) {
-		ZVAL_STRING(ps[2], xmsg, 1);
+		ZVAL_STRING(ps[2], xmsg, 0);
 	} else {
 		ZVAL_NULL(ps[2]);
 	}
@@ -861,9 +822,6 @@ static void user_space_stream_notifier(php_stream_context *context, int notifyco
 	if (FAILURE == call_user_function_ex(EG(function_table), NULL, callback, &retval, 6, ptps, 0, NULL TSRMLS_CC)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to call user notifier");
 	}
-	for (i = 0; i < 6; i++) {
-		zval_ptr_dtor(&ps[i]);
-	}
 	if (retval) {
 		zval_ptr_dtor(&retval);
 	}
@@ -877,7 +835,7 @@ static void user_space_stream_notifier_dtor(php_stream_notifier *notifier)
 	}
 }
 
-static int parse_context_options(php_stream_context *context, zval *options TSRMLS_DC)
+static int parse_context_options(php_stream_context *context, zval *options)
 {
 	HashPosition pos, opos;
 	zval **wval, **oval;
@@ -901,7 +859,7 @@ static int parse_context_options(php_stream_context *context, zval *options TSRM
 			}
 
 		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "options should have the form [\"wrappername\"][\"optionname\"] = $value");
+			zend_error(E_WARNING, "options should have the form [\"wrappername\"][\"optionname\"] = $value");
 		}
 		zend_hash_move_forward_ex(Z_ARRVAL_P(options), &pos);
 	}
@@ -909,7 +867,7 @@ static int parse_context_options(php_stream_context *context, zval *options TSRM
 	return ret;
 }
 
-static int parse_context_params(php_stream_context *context, zval *params TSRMLS_DC)
+static int parse_context_params(php_stream_context *context, zval *params)
 {
 	int ret = SUCCESS;
 	zval **tmp;
@@ -924,15 +882,11 @@ static int parse_context_params(php_stream_context *context, zval *params TSRMLS
 		context->notifier = php_stream_notification_alloc();
 		context->notifier->func = user_space_stream_notifier;
 		context->notifier->ptr = *tmp;
-		Z_ADDREF_P(*tmp);
+		ZVAL_ADDREF(*tmp);
 		context->notifier->dtor = user_space_stream_notifier_dtor;
 	}
 	if (SUCCESS == zend_hash_find(Z_ARRVAL_P(params), "options", sizeof("options"), (void**)&tmp)) {
-		if (Z_TYPE_PP(tmp) == IS_ARRAY) {
-			parse_context_options(context, *tmp TSRMLS_CC);
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
-		}
+		parse_context_options(context, *tmp);
 	}
 	
 	return ret;
@@ -979,7 +933,7 @@ PHP_FUNCTION(stream_context_get_options)
 	}
 	context = decode_context_param(zcontext TSRMLS_CC);
 	if (!context) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter.");
 		RETURN_FALSE;
 	}
 
@@ -1009,13 +963,13 @@ PHP_FUNCTION(stream_context_set_option)
 	/* figure out where the context is coming from exactly */
 	context = decode_context_param(zcontext TSRMLS_CC);
 	if (!context) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter.");
 		RETURN_FALSE;
 	}
 
 	if (options) {
 		/* handle the array syntax */
-		RETVAL_BOOL(parse_context_options(context, options TSRMLS_CC) == SUCCESS);
+		RETVAL_BOOL(parse_context_options(context, options) == SUCCESS);
 	} else {
 		php_stream_context_set_option(context, wrappername, optionname, zvalue);
 		RETVAL_TRUE;
@@ -1036,39 +990,11 @@ PHP_FUNCTION(stream_context_set_params)
 
 	context = decode_context_param(zcontext TSRMLS_CC);
 	if (!context) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter.");
 		RETURN_FALSE;
 	}
 
-	RETVAL_BOOL(parse_context_params(context, params TSRMLS_CC) == SUCCESS);
-}
-/* }}} */
-
-/* {{{ proto array stream_context_get_params(resource context|resource stream)
-   Get parameters of a file context */
-PHP_FUNCTION(stream_context_get_params)
-{
-	zval *zcontext, *options;
-	php_stream_context *context;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zcontext) == FAILURE) {
-		RETURN_FALSE;
-	}
-
-	context = decode_context_param(zcontext TSRMLS_CC);
-	if (!context) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
-		RETURN_FALSE;
-	}
-
-	array_init(return_value);
-	if (context->notifier && context->notifier->ptr && context->notifier->func == user_space_stream_notifier) {
-		add_assoc_zval_ex(return_value, ZEND_STRS("notification"), context->notifier->ptr);
-		Z_ADDREF_P(context->notifier->ptr);
-	}
-	ALLOC_INIT_ZVAL(options);
-	ZVAL_ZVAL(options, context->options, 1, 0);
-	add_assoc_zval_ex(return_value, ZEND_STRS("options"), options);
+	RETVAL_BOOL(parse_context_params(context, params) == SUCCESS);
 }
 /* }}} */
 
@@ -1087,59 +1013,33 @@ PHP_FUNCTION(stream_context_get_default)
 		FG(default_context) = php_stream_context_alloc();
 	}
 	context = FG(default_context);
-
+	
 	if (params) {
-		parse_context_options(context, params TSRMLS_CC);
+		parse_context_options(context, params);
 	}
 	
 	php_stream_context_to_zval(context, return_value);
 }
 /* }}} */
 
-/* {{{ proto resource stream_context_set_default(array options)
-   Set default file/stream context, returns the context as a resource */
-PHP_FUNCTION(stream_context_set_default)
-{
-	zval *options = NULL;
-	php_stream_context *context;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &options) == FAILURE) {
-		return;
-	}
-
-	if (FG(default_context) == NULL) {
-		FG(default_context) = php_stream_context_alloc();
-	}
-	context = FG(default_context);
-
-	parse_context_options(context, options TSRMLS_CC);
-
-	php_stream_context_to_zval(context, return_value);
-}
-/* }}} */
-
-/* {{{ proto resource stream_context_create([array options[, array params]])
+/* {{{ proto resource stream_context_create([array options])
    Create a file context and optionally set parameters */
 PHP_FUNCTION(stream_context_create)
 {
-	zval *options = NULL, *params = NULL;
+	zval *params = NULL;
 	php_stream_context *context;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a!a!", &options, &params) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a", &params) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
 	context = php_stream_context_alloc();
 	
-	if (options) {
-		parse_context_options(context, options TSRMLS_CC);
-	}
-
 	if (params) {
-		parse_context_params(context, params TSRMLS_CC);
+		parse_context_options(context, params);
 	}
 	
-	RETURN_RESOURCE(context->rsrc_id);
+	php_stream_context_to_zval(context, return_value);
 }
 /* }}} */
 
@@ -1153,7 +1053,6 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 	long read_write = 0;
 	zval *filterparams = NULL;
 	php_stream_filter *filter = NULL;
-	int ret;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|lz", &zstream,
 				&filtername, &filternamelen, &read_write, &filterparams) == FAILURE) {
@@ -1183,13 +1082,9 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 		}
 
 		if (append) { 
-			ret = php_stream_filter_append_ex(&stream->readfilters, filter TSRMLS_CC);
+			php_stream_filter_append(&stream->readfilters, filter);
 		} else {
-			ret = php_stream_filter_prepend_ex(&stream->readfilters, filter TSRMLS_CC);
-		}
-		if (ret != SUCCESS) {
-			php_stream_filter_remove(filter, 1 TSRMLS_CC);
-			RETURN_FALSE;
+			php_stream_filter_prepend(&stream->readfilters, filter);
 		}
 	}
 
@@ -1200,13 +1095,9 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 		}
 
 		if (append) { 
-			ret = php_stream_filter_append_ex(&stream->writefilters, filter TSRMLS_CC);
+			php_stream_filter_append(&stream->writefilters, filter);
 		} else {
-			ret = php_stream_filter_prepend_ex(&stream->writefilters, filter TSRMLS_CC);
-		}
-		if (ret != SUCCESS) {
-			php_stream_filter_remove(filter, 1 TSRMLS_CC);
-			RETURN_FALSE;
+			php_stream_filter_prepend(&stream->writefilters, filter);
 		}
 	}
 
@@ -1216,7 +1107,6 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 		RETURN_FALSE;
 	}
 }
-/* }}} */
 
 /* {{{ proto resource stream_filter_prepend(resource stream, string filtername[, int read_write[, string filterparams]])
    Prepend a filter to a stream */
@@ -1271,7 +1161,7 @@ PHP_FUNCTION(stream_filter_remove)
 PHP_FUNCTION(stream_get_line)
 {
 	char *str = NULL;
-	int str_len = 0;
+	int str_len;
 	long max_length;
 	zval *zstream;
 	char *buf;
@@ -1283,7 +1173,7 @@ PHP_FUNCTION(stream_get_line)
 	}
 
 	if (max_length < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "The maximum allowed length must be greater than or equal to zero");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "The maximum allowed length must be greater then or equal to zero.");
 		RETURN_FALSE;
 	}
 	if (!max_length) {
@@ -1305,53 +1195,61 @@ PHP_FUNCTION(stream_get_line)
    Set blocking/non-blocking mode on a socket or stream */
 PHP_FUNCTION(stream_set_blocking)
 {
-	zval *arg1;
+	zval **arg1, **arg2;
 	int block;
-	long arg2;
 	php_stream *stream;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &arg1, &arg2) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &arg1, &arg2) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, &arg1);
+	php_stream_from_zval(stream, arg1);
 
-	block = arg2;
+	convert_to_long_ex(arg2);
+	block = Z_LVAL_PP(arg2);
 
-	if (php_stream_set_option(stream, PHP_STREAM_OPTION_BLOCKING, block == 0 ? 0 : 1, NULL) == -1) {
+	if (php_stream_set_option(stream, PHP_STREAM_OPTION_BLOCKING, block == 0 ? 0 : 1, NULL) == -1)
 		RETURN_FALSE;
-	}
-
 	RETURN_TRUE;
 }
 
 /* }}} */
 
-/* {{{ proto bool stream_set_timeout(resource stream, int seconds [, int microseconds])
+/* {{{ proto bool set_socket_blocking(resource socket, int mode)
+   Set blocking/non-blocking mode on a socket */
+PHP_FUNCTION(set_socket_blocking)
+{
+	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "This function is deprecated, use stream_set_blocking() instead");
+	PHP_FN(stream_set_blocking)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
+}
+/* }}} */
+
+/* {{{ proto bool stream_set_timeout(resource stream, int seconds, int microseconds)
    Set timeout on stream read to seconds + microseonds */
 #if HAVE_SYS_TIME_H || defined(PHP_WIN32)
 PHP_FUNCTION(stream_set_timeout)
 {
-	zval *socket;
-	long seconds, microseconds = 0;
+	zval **socket, **seconds, **microseconds;
 	struct timeval t;
 	php_stream *stream;
-	int argc = ZEND_NUM_ARGS();
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "rl|l", &socket, &seconds, &microseconds) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() < 2 || ZEND_NUM_ARGS() > 3 ||
+		zend_get_parameters_ex(ZEND_NUM_ARGS(), &socket, &seconds, &microseconds)==FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, &socket);
+	php_stream_from_zval(stream, socket);
 
-	t.tv_sec = seconds;
+	convert_to_long_ex(seconds);
+	t.tv_sec = Z_LVAL_PP(seconds);
 
-	if (argc == 3) {
-		t.tv_usec = microseconds % 1000000;
-		t.tv_sec += microseconds / 1000000;
-	} else {
+	if (ZEND_NUM_ARGS() == 3) {
+		convert_to_long_ex(microseconds);
+		t.tv_usec = Z_LVAL_PP(microseconds) % 1000000;
+		t.tv_sec += Z_LVAL_PP(microseconds) / 1000000;
+	}
+	else
 		t.tv_usec = 0;
-	}
 
 	if (PHP_STREAM_OPTION_RETURN_OK == php_stream_set_option(stream, PHP_STREAM_OPTION_READ_TIMEOUT, 0, &t)) {
 		RETURN_TRUE;
@@ -1366,15 +1264,14 @@ PHP_FUNCTION(stream_set_timeout)
    Set file write buffer */
 PHP_FUNCTION(stream_set_write_buffer)
 {
-	zval *arg1;
+	zval **arg1, **arg2;
 	int ret;
-	long arg2;
 	size_t buff;
 	php_stream *stream;
 
 	switch (ZEND_NUM_ARGS()) {
 	case 2:
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &arg1, &arg2) == FAILURE) {
+		if (zend_get_parameters_ex(2, &arg1, &arg2)==FAILURE) {
 			RETURN_FALSE;
 		}
 		break;
@@ -1384,9 +1281,10 @@ PHP_FUNCTION(stream_set_write_buffer)
 		break;
 	}
 	
-	php_stream_from_zval(stream, &arg1);
-
-	buff = arg2;
+	php_stream_from_zval(stream, arg1);
+	
+	convert_to_long_ex(arg2);
+	buff = Z_LVAL_PP(arg2);
 
 	/* if buff is 0 then set to non-buffered */
 	if (buff == 0) {
@@ -1399,11 +1297,11 @@ PHP_FUNCTION(stream_set_write_buffer)
 }
 /* }}} */
 
-/* {{{ proto int stream_socket_enable_crypto(resource stream, bool enable [, int cryptokind [, resource sessionstream]])
+/* {{{ proto int stream_socket_enable_crypto(resource stream, bool enable [, int cryptokind, resource sessionstream])
    Enable or disable a specific kind of crypto on the stream */
 PHP_FUNCTION(stream_socket_enable_crypto)
 {
-	long cryptokind = 0;
+	long cryptokind;
 	zval *zstream, *zsessstream = NULL;
 	php_stream *stream, *sessstream = NULL;
 	zend_bool enable;
@@ -1423,9 +1321,6 @@ PHP_FUNCTION(stream_socket_enable_crypto)
 		if (php_stream_xport_crypto_setup(stream, cryptokind, sessstream TSRMLS_CC) < 0) {
 			RETURN_FALSE;
 		}
-	} else if (enable) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "When enabling encryption you must specify the crypto type");
-		RETURN_FALSE;
 	}
 
 	ret = php_stream_xport_crypto_enable(stream, enable TSRMLS_CC);
@@ -1442,108 +1337,6 @@ PHP_FUNCTION(stream_socket_enable_crypto)
 }
 /* }}} */
 
-/* {{{ proto string stream_resolve_include_path(string filename)
-Determine what file will be opened by calls to fopen() with a relative path */
-PHP_FUNCTION(stream_resolve_include_path)
-{
-	char *filename, *resolved_path; 
-	int filename_len;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
-		return;
-	}
-
-	resolved_path = zend_resolve_path(filename, filename_len TSRMLS_CC);
-
-	if (resolved_path) {
-		RETURN_STRING(resolved_path, 0);
-	}
-	RETURN_FALSE;
-}
-/* }}} */
-
-/* {{{ proto bool stream_is_local(resource stream|string url) U
-*/
-PHP_FUNCTION(stream_is_local)
-{
-	zval **zstream;
-	php_stream *stream = NULL;
-	php_stream_wrapper *wrapper = NULL;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &zstream) == FAILURE) {
-		RETURN_FALSE;
-	}
-
-	if (Z_TYPE_PP(zstream) == IS_RESOURCE) {
-		php_stream_from_zval(stream, zstream);
-		if (stream == NULL) {
-			RETURN_FALSE;
-		}
-		wrapper = stream->wrapper;
-	} else {
-		convert_to_string_ex(zstream);
-		
-		wrapper = php_stream_locate_url_wrapper(Z_STRVAL_PP(zstream), NULL, 0 TSRMLS_CC);
-	}
-
-	if (!wrapper) {
-		RETURN_FALSE;
-	}
-
-	RETURN_BOOL(wrapper->is_url==0);
-}
-/* }}} */
-
-/* {{{ proto bool stream_supports_lock(resource stream)
-   Tells wether the stream supports locking through flock(). */
-PHP_FUNCTION(stream_supports_lock)
-{
-	php_stream *stream;
-	zval *zsrc;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zsrc) == FAILURE) {
-		RETURN_FALSE;
-	}
-
-	php_stream_from_zval(stream, &zsrc);
-
-	if (!php_stream_supports_lock(stream)) {
-		RETURN_FALSE;
-	}
-
-	RETURN_TRUE;
-}
-
-#ifdef HAVE_SHUTDOWN
-/* {{{ proto int stream_socket_shutdown(resource stream, int how)
-	causes all or part of a full-duplex connection on the socket associated
-	with stream to be shut down.  If how is SHUT_RD,  further receptions will
-	be disallowed. If how is SHUT_WR, further transmissions will be disallowed.
-	If how is SHUT_RDWR,  further  receptions and transmissions will be
-	disallowed. */
-PHP_FUNCTION(stream_socket_shutdown)
-{
-	long how;
-	zval *zstream;
-	php_stream *stream;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &zstream, &how) == FAILURE) {
-		RETURN_FALSE;
-	}
-	
-	if (how != STREAM_SHUT_RD &&
-	    how != STREAM_SHUT_WR &&
-	    how != STREAM_SHUT_RDWR) {
-		RETURN_FALSE;
-	}
-
-	php_stream_from_zval(stream, &zstream);
-
-	RETURN_BOOL(php_stream_xport_shutdown(stream, (stream_shutdown_t)how TSRMLS_CC) == 0);
-}
-/* }}} */
-#endif
-
 /*
  * Local variables:
  * tab-width: 4
