@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2007 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: tidy.c,v 1.66.2.8.2.22 2007/02/11 16:07:30 nlopess Exp $ */
+/* $Id: tidy.c,v 1.66.2.5 2006/01/01 12:50:16 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,21 +31,29 @@
 #include "ext/standard/info.h"
 #include "safe_mode.h"
 
+#include "Zend/zend_exceptions.h"
+#include "Zend/zend_object_handlers.h"
+
 #include "tidy.h"
 #include "buffio.h"
 
-/* compatibility with older versions of libtidy */
-#ifndef TIDY_CALL
-#define TIDY_CALL
-#endif
-
 #define PHP_TIDY_MODULE_VERSION	"2.0"
 
 /* {{{ ext/tidy macros
 */
-#define TIDY_SET_CONTEXT \
-    zval *object = getThis();
+#define REMOVE_NEWLINE(_z) _z->value.str.val[_z->value.str.len-1] = '\0'; _z->value.str.len--;
 
+#define TIDYDOC_FROM_OBJECT(tdoc, object) \
+	{ \
+		PHPTidyObj *obj = (PHPTidyObj*) zend_object_store_get_object(object TSRMLS_CC); \
+		tdoc = obj->ptdoc; \
+	}
+
+#define TIDY_SET_CONTEXT \
+    zval *object; \
+    TG(inst) = getThis(); \
+    object = TG(inst)
+   
 #define TIDY_FETCH_OBJECT	\
 	PHPTidyObj *obj;	\
 	TIDY_SET_CONTEXT; \
@@ -68,20 +76,19 @@
 	}	\
 	obj = (PHPTidyObj *) zend_object_store_get_object(object TSRMLS_CC);	\
 
+
+#define Z_OBJ_P(zval_p) zend_objects_get_address(zval_p TSRMLS_CC)
+
 #define TIDY_APPLY_CONFIG_ZVAL(_doc, _val) \
     if(_val) { \
-        if(Z_TYPE_PP(_val) == IS_ARRAY) { \
-            _php_tidy_apply_config_array(_doc, HASH_OF(*_val) TSRMLS_CC); \
+        if(Z_TYPE_P(_val) == IS_ARRAY) { \
+            _php_tidy_apply_config_array(_doc, HASH_OF(_val) TSRMLS_CC); \
         } else { \
-            convert_to_string_ex(_val); \
-            TIDY_SAFE_MODE_CHECK(Z_STRVAL_PP(_val)); \
-            switch (tidyLoadConfig(_doc, Z_STRVAL_PP(_val))) { \
-              case -1: \
-                php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not load configuration file '%s'", Z_STRVAL_PP(_val)); \
-                break; \
-              case 1: \
-                php_error_docref(NULL TSRMLS_CC, E_NOTICE, "There were errors while parsing the configuration file '%s'", Z_STRVAL_PP(_val)); \
-                break; \
+            convert_to_string_ex(&_val); \
+            TIDY_SAFE_MODE_CHECK(Z_STRVAL_P(_val)); \
+            if (tidyLoadConfig(_doc, Z_STRVAL_P(_val)) < 0) { \
+                php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not load configuration file '%s'", Z_STRVAL_P(_val)); \
+                RETURN_FALSE; \
             } \
         } \
     }
@@ -176,7 +183,8 @@ typedef struct _PHPTidyObj PHPTidyObj;
 
 typedef enum {
 	is_node,
-	is_doc
+	is_doc,
+	is_exception
 } tidy_obj_type;
 
 typedef enum {
@@ -206,11 +214,12 @@ static char *php_tidy_file_to_mem(char *, zend_bool, int * TSRMLS_DC);
 static void tidy_object_free_storage(void * TSRMLS_DC);
 static zend_object_value tidy_object_new_node(zend_class_entry * TSRMLS_DC);
 static zend_object_value tidy_object_new_doc(zend_class_entry * TSRMLS_DC);
+static zend_object_value tidy_object_new_exception(zend_class_entry * TSRMLS_DC);
 static zend_class_entry *tidy_get_ce_node(zval * TSRMLS_DC);
 static zend_class_entry *tidy_get_ce_doc(zval * TSRMLS_DC);
 static zval * tidy_instanciate(zend_class_entry *, zval * TSRMLS_DC);
-static int tidy_doc_cast_handler(zval *, zval *, int TSRMLS_DC);
-static int tidy_node_cast_handler(zval *, zval *, int TSRMLS_DC);
+static int tidy_doc_cast_handler(zval *, zval *, int, int TSRMLS_DC);
+static int tidy_node_cast_handler(zval *, zval *, int, int TSRMLS_DC);
 static void tidy_doc_update_properties(PHPTidyObj * TSRMLS_DC);
 static void tidy_add_default_properties(PHPTidyObj *, tidy_obj_type TSRMLS_DC);
 static void *php_tidy_get_opt_val(PHPTidyDoc *, TidyOption, TidyOptionType * TSRMLS_DC);
@@ -219,55 +228,6 @@ static int _php_tidy_set_tidy_opt(TidyDoc, char *, zval * TSRMLS_DC);
 static int _php_tidy_apply_config_array(TidyDoc doc, HashTable *ht_options TSRMLS_DC);
 static void _php_tidy_register_nodetypes(INIT_FUNC_ARGS);
 static void _php_tidy_register_tags(INIT_FUNC_ARGS);
-
-static PHP_MINIT_FUNCTION(tidy);
-static PHP_MSHUTDOWN_FUNCTION(tidy);
-static PHP_RINIT_FUNCTION(tidy);
-static PHP_MINFO_FUNCTION(tidy);
-
-static PHP_FUNCTION(tidy_getopt);
-static PHP_FUNCTION(tidy_parse_string);
-static PHP_FUNCTION(tidy_parse_file);
-static PHP_FUNCTION(tidy_clean_repair);
-static PHP_FUNCTION(tidy_repair_string);
-static PHP_FUNCTION(tidy_repair_file);
-static PHP_FUNCTION(tidy_diagnose);
-static PHP_FUNCTION(tidy_get_output);
-static PHP_FUNCTION(tidy_get_error_buffer);
-static PHP_FUNCTION(tidy_get_release);
-static PHP_FUNCTION(tidy_get_config);
-static PHP_FUNCTION(tidy_get_status);
-static PHP_FUNCTION(tidy_get_html_ver);
-#if HAVE_TIDYOPTGETDOC
-static PHP_FUNCTION(tidy_get_opt_doc);
-#endif
-static PHP_FUNCTION(tidy_is_xhtml);
-static PHP_FUNCTION(tidy_is_xml);
-static PHP_FUNCTION(tidy_error_count);
-static PHP_FUNCTION(tidy_warning_count);
-static PHP_FUNCTION(tidy_access_count);
-static PHP_FUNCTION(tidy_config_count);
-
-static PHP_FUNCTION(ob_tidyhandler);
-
-static PHP_FUNCTION(tidy_get_root);
-static PHP_FUNCTION(tidy_get_html);
-static PHP_FUNCTION(tidy_get_head);
-static PHP_FUNCTION(tidy_get_body);
-
-static TIDY_DOC_METHOD(__construct);
-static TIDY_DOC_METHOD(parseFile);
-static TIDY_DOC_METHOD(parseString);
-
-static TIDY_NODE_METHOD(hasChildren);
-static TIDY_NODE_METHOD(hasSiblings);
-static TIDY_NODE_METHOD(isComment);
-static TIDY_NODE_METHOD(isHtml);
-static TIDY_NODE_METHOD(isText);
-static TIDY_NODE_METHOD(isJste);
-static TIDY_NODE_METHOD(isAsp);
-static TIDY_NODE_METHOD(isPhp);
-static TIDY_NODE_METHOD(getParent);
 /* }}} */
 
 ZEND_DECLARE_MODULE_GLOBALS(tidy)
@@ -277,7 +237,7 @@ STD_PHP_INI_ENTRY("tidy.default_config",	"",	PHP_INI_SYSTEM,		OnUpdateString,		d
 PHP_INI_ENTRY("tidy.clean_output",     "0",    PHP_INI_PERDIR,         NULL)
 PHP_INI_END()
 
-static zend_function_entry tidy_functions[] = {
+zend_function_entry tidy_functions[] = {
 	PHP_FE(tidy_getopt,             NULL)
 	PHP_FE(tidy_parse_string,       NULL)
 	PHP_FE(tidy_parse_file,         NULL)
@@ -308,7 +268,7 @@ static zend_function_entry tidy_functions[] = {
 	{NULL, NULL, NULL}
 };
 
-static zend_function_entry tidy_funcs_doc[] = {
+zend_function_entry tidy_funcs_doc[] = {
 	TIDY_METHOD_MAP(getOpt, tidy_getopt, NULL)
 	TIDY_METHOD_MAP(cleanRepair, tidy_clean_repair, NULL)
 	TIDY_DOC_ME(parseFile, NULL)
@@ -333,7 +293,8 @@ static zend_function_entry tidy_funcs_doc[] = {
 	{NULL, NULL, NULL}
 };
 
-static zend_function_entry tidy_funcs_node[] = {
+zend_function_entry tidy_funcs_node[] = {
+	TIDY_NODE_ME(__construct, NULL)
 	TIDY_NODE_ME(hasChildren, NULL)
 	TIDY_NODE_ME(hasSiblings, NULL)
 	TIDY_NODE_ME(isComment, NULL)
@@ -342,52 +303,52 @@ static zend_function_entry tidy_funcs_node[] = {
 	TIDY_NODE_ME(isJste, NULL)
 	TIDY_NODE_ME(isAsp, NULL)
 	TIDY_NODE_ME(isPhp, NULL)
-	TIDY_NODE_ME(getParent, NULL)
 	{NULL, NULL, NULL}
 };
 
-static zend_class_entry *tidy_ce_doc, *tidy_ce_node;
+zend_function_entry tidy_funcs_exception[] = {
+	{NULL, NULL, NULL}
+};
+
+zend_class_entry *tidy_ce_doc, *tidy_ce_node, *tidy_ce_exception;
 
 static zend_object_handlers tidy_object_handlers_doc;
 static zend_object_handlers tidy_object_handlers_node;
+static zend_object_handlers tidy_object_handlers_exception;
 
 zend_module_entry tidy_module_entry = {
 	STANDARD_MODULE_HEADER,
 	"tidy",
 	tidy_functions,
 	PHP_MINIT(tidy),
-	PHP_MSHUTDOWN(tidy),
+	NULL,
 	PHP_RINIT(tidy),
 	NULL,
 	PHP_MINFO(tidy),
 	PHP_TIDY_MODULE_VERSION,
-	PHP_MODULE_GLOBALS(tidy),
-	NULL,
-	NULL,
-	NULL,
-	STANDARD_MODULE_PROPERTIES_EX
+	STANDARD_MODULE_PROPERTIES
 };
 
 #ifdef COMPILE_DL_TIDY
 ZEND_GET_MODULE(tidy)
 #endif
 
-static void* TIDY_CALL php_tidy_malloc(size_t len)
+void *php_tidy_malloc(size_t len)
 {
 	return emalloc(len);
 }
 
-static void* TIDY_CALL php_tidy_realloc(void *buf, size_t len)
+void *php_tidy_realloc(void *buf, size_t len)
 {
 	return erealloc(buf, len);
 }
 
-static void TIDY_CALL php_tidy_free(void *buf)
+void php_tidy_free(void *buf)
 {
 	efree(buf);
 }
 
-static void TIDY_CALL php_tidy_panic(ctmbstr msg)
+void php_tidy_panic(ctmbstr msg)
 {
 	TSRMLS_FETCH();
 	php_error_docref(NULL TSRMLS_CC, E_ERROR, "Could not allocate memory for tidy! (Reason: %s)", (char *)msg);
@@ -460,9 +421,11 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, zend_bool is_fil
 	zend_bool use_include_path = 0;
 	TidyDoc doc;
 	TidyBuffer *errbuf;
-	zval **config;
+	zval *config;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|Zsb", &arg1, &arg1_len, &config, &enc, &enc_len, &use_include_path) == FAILURE) {
+	TIDY_SET_CONTEXT;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zsb", &arg1, &arg1_len, &config, &enc, &enc_len, &use_include_path) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
@@ -491,8 +454,19 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, zend_bool is_fil
 	
 	TIDY_SET_DEFAULT_CONFIG(doc);
 	
+	/* We can't use TIDY_APPLY_CONFIG_ZVAL() here, it uses RETURN_FALSE */
+
 	if (ZEND_NUM_ARGS() > 1) {
-		TIDY_APPLY_CONFIG_ZVAL(doc, config);
+		if(Z_TYPE_P(config) == IS_ARRAY) {
+			_php_tidy_apply_config_array(doc, HASH_OF(config) TSRMLS_CC);
+		} else {
+			convert_to_string_ex(&config);
+			TIDY_SAFE_MODE_CHECK(Z_STRVAL_P(config));
+			if (tidyLoadConfig(doc, Z_STRVAL_P(config)) < 0) {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not load configuration file '%s'", Z_STRVAL_P(config));
+				RETVAL_FALSE;
+			}
+		}
 	}
 
 	if(enc_len) {
@@ -503,21 +477,20 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, zend_bool is_fil
 	}
 	
 	if (data) {
-		TidyBuffer buf;
-
+		TidyBuffer buf = {0};
+		
 		tidyBufInit(&buf);
 		tidyBufAppend(&buf, data, data_len);
-
+		
 		if (tidyParseBuffer(doc, &buf) < 0) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errbuf->bp);
 			RETVAL_FALSE;
 		} else {
 			if (tidyCleanAndRepair(doc) >= 0) {
-				TidyBuffer output;
-				tidyBufInit(&output);
+				TidyBuffer output = {0};
 
 				tidySaveBuffer (doc, &output);
-				RETVAL_STRINGL(output.bp, output.size ? output.size-1 : 0, 1);
+				RETVAL_STRINGL(output.bp, output.size-1, 1);
 				tidyBufFree(&output);
 			} else {
 				RETVAL_FALSE;
@@ -557,7 +530,8 @@ static void tidy_object_free_storage(void *object TSRMLS_DC)
 {
 	PHPTidyObj *intern = (PHPTidyObj *)object;
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_hash_destroy(intern->std.properties);
+	FREE_HASHTABLE(intern->std.properties);
 
 	if (intern->ptdoc) {
 		intern->ptdoc->ref_count--;
@@ -581,8 +555,10 @@ static void tidy_object_new(zend_class_entry *class_type, zend_object_handlers *
 
 	intern = emalloc(sizeof(PHPTidyObj));
 	memset(intern, 0, sizeof(PHPTidyObj));
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
-	
+	intern->std.ce = class_type;
+
+	ALLOC_HASHTABLE(intern->std.properties);
+	zend_hash_init(intern->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
 	zend_hash_copy(intern->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 
 	switch(objtype) {
@@ -640,6 +616,13 @@ static zend_object_value tidy_object_new_doc(zend_class_entry *class_type TSRMLS
 	return retval;
 }
 
+static zend_object_value tidy_object_new_exception(zend_class_entry *class_type TSRMLS_DC)
+{
+	zend_object_value retval;
+	tidy_object_new(class_type, &tidy_object_handlers_exception, &retval, is_exception TSRMLS_CC);
+	return retval;
+}
+
 static zend_class_entry *tidy_get_ce_node(zval *object TSRMLS_DC)
 {
 	return tidy_ce_node;
@@ -663,9 +646,9 @@ static zval * tidy_instanciate(zend_class_entry *pce, zval *object TSRMLS_DC)
 	return object;
 }
 
-static int tidy_doc_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
+static int tidy_doc_cast_handler(zval *in, zval *out, int type, int free TSRMLS_DC)
 {
-	TidyBuffer output;
+	TidyBuffer output = {0};
 	PHPTidyObj *obj;
 
 	switch(type) {
@@ -683,9 +666,8 @@ static int tidy_doc_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
 
 		case IS_STRING:
 			obj = (PHPTidyObj *)zend_object_store_get_object(in TSRMLS_CC);
-			tidyBufInit(&output);
 			tidySaveBuffer (obj->ptdoc->doc, &output);
-			ZVAL_STRINGL(out, output.bp, output.size ? output.size-1 : 0, TRUE);
+			ZVAL_STRINGL(out, output.bp, output.size-1, TRUE);
 			tidyBufFree(&output);
 			break;
 
@@ -696,9 +678,9 @@ static int tidy_doc_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
 	return SUCCESS;
 }
 
-static int tidy_node_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
+static int tidy_node_cast_handler(zval *in, zval *out, int type, int free TSRMLS_DC)
 {
-	TidyBuffer buf;
+	TidyBuffer buf = {0};
 	PHPTidyObj *obj;
 
 	switch(type) {
@@ -716,11 +698,8 @@ static int tidy_node_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
 
 		case IS_STRING:
 			obj = (PHPTidyObj *)zend_object_store_get_object(in TSRMLS_CC);
-			tidyBufInit(&buf);
-			if (obj->ptdoc) {
-				tidyNodeGetText(obj->ptdoc->doc, obj->node, &buf);
-			}
-			ZVAL_STRINGL(out, buf.bp, buf.size ? buf.size-1 : 0, TRUE);
+			tidyNodeGetText(obj->ptdoc->doc, obj->node, &buf);
+			ZVAL_STRINGL(out, buf.bp, buf.size-1, TRUE);
 			tidyBufFree(&buf);
 			break;
 
@@ -734,10 +713,9 @@ static int tidy_node_cast_handler(zval *in, zval *out, int type TSRMLS_DC)
 static void tidy_doc_update_properties(PHPTidyObj *obj TSRMLS_DC)
 {
 
-	TidyBuffer output;
+	TidyBuffer output = {0};
 	zval *temp;
 
-	tidyBufInit(&output);
 	tidySaveBuffer (obj->ptdoc->doc, &output);
 	
 	if (output.size) {
@@ -767,7 +745,8 @@ static void tidy_add_default_properties(PHPTidyObj *obj, tidy_obj_type type TSRM
 	switch(type) {
 
 		case is_node:
-			tidyBufInit(&buf);
+
+			memset(&buf, 0, sizeof(buf));
 			tidyNodeGetText(obj->ptdoc->doc, obj->node, &buf);
 			ADD_PROPERTY_STRINGL(obj->std.properties, value, buf.bp, buf.size-1);
 			tidyBufFree(&buf);
@@ -775,8 +754,8 @@ static void tidy_add_default_properties(PHPTidyObj *obj, tidy_obj_type type TSRM
 			ADD_PROPERTY_STRING(obj->std.properties, name, tidyNodeGetName(obj->node));
 			ADD_PROPERTY_LONG(obj->std.properties, type, tidyNodeGetType(obj->node));
 			ADD_PROPERTY_LONG(obj->std.properties, line, tidyNodeLine(obj->node));
-			ADD_PROPERTY_LONG(obj->std.properties, column, tidyNodeColumn(obj->node));
-			ADD_PROPERTY_BOOL(obj->std.properties, proprietary, tidyNodeIsProp(obj->ptdoc->doc, obj->node));
+            ADD_PROPERTY_LONG(obj->std.properties, column, tidyNodeColumn(obj->node));
+            ADD_PROPERTY_BOOL(obj->std.properties, proprietary, tidyNodeIsProp(obj->ptdoc->doc, obj->node));
 
 			switch(tidyNodeGetType(obj->node)) {
 				case TidyNode_Root:
@@ -839,6 +818,10 @@ static void tidy_add_default_properties(PHPTidyObj *obj, tidy_obj_type type TSRM
 			ADD_PROPERTY_NULL(obj->std.properties, errorBuffer);
 			ADD_PROPERTY_NULL(obj->std.properties, value);
 			break;
+
+		case is_exception:
+		default:
+			break;
 	}
 }
 
@@ -929,7 +912,7 @@ static int _php_tidy_apply_config_array(TidyDoc doc, HashTable *ht_options TSRML
 
 static int php_tidy_parse_string(PHPTidyObj *obj, char *string, int len, char *enc TSRMLS_DC)
 {
-	TidyBuffer buf;
+	TidyBuffer buf = {0};
 	
 	if(enc) {
 		if (tidySetCharEncoding(obj->ptdoc->doc, enc) < 0) {
@@ -952,12 +935,26 @@ static int php_tidy_parse_string(PHPTidyObj *obj, char *string, int len, char *e
 	return SUCCESS;
 }
 
-static PHP_MINIT_FUNCTION(tidy)
+static void tidy_globals_ctor(void *global TSRMLS_DC)
 {
+
+}
+
+static void tidy_globals_dtor(void *global TSRMLS_DC)
+{
+
+}
+
+PHP_MINIT_FUNCTION(tidy)
+{
+	ZEND_INIT_MODULE_GLOBALS(tidy, tidy_globals_ctor, tidy_globals_dtor);
+
 	REGISTER_INI_ENTRIES();
 	REGISTER_TIDY_CLASS(tidy, doc,	NULL, 0);
 	REGISTER_TIDY_CLASS(tidyNode, node,	NULL, ZEND_ACC_FINAL_CLASS);
-
+	/* no exceptions for now..
+	REGISTER_TIDY_CLASS(tidyException, exception,	zend_exception_get_default());
+	*/
 	tidy_object_handlers_doc.get_class_entry = tidy_get_ce_doc;
 	tidy_object_handlers_node.get_class_entry = tidy_get_ce_node;
 	
@@ -970,7 +967,7 @@ static PHP_MINIT_FUNCTION(tidy)
 	return SUCCESS;
 }
 
-static PHP_RINIT_FUNCTION(tidy)
+PHP_RINIT_FUNCTION(tidy)
 {
 	if (INI_BOOL("tidy.clean_output") == TRUE) {
 		if (php_start_ob_buffer_named("ob_tidyhandler", 0, 1 TSRMLS_CC) == FAILURE) {
@@ -981,33 +978,27 @@ static PHP_RINIT_FUNCTION(tidy)
 	return SUCCESS;
 }
 
-static PHP_MSHUTDOWN_FUNCTION(tidy)
-{
-	UNREGISTER_INI_ENTRIES();
-	return SUCCESS;
-}
-
-static PHP_MINFO_FUNCTION(tidy)
+PHP_MINFO_FUNCTION(tidy)
 {
 	php_info_print_table_start();
 	php_info_print_table_header(2, "Tidy support", "enabled");
 	php_info_print_table_row(2, "libTidy Release", (char *)tidyReleaseDate());
-	php_info_print_table_row(2, "Extension Version", PHP_TIDY_MODULE_VERSION " ($Id: tidy.c,v 1.66.2.8.2.22 2007/02/11 16:07:30 nlopess Exp $)");
+	php_info_print_table_row(2, "Extension Version", PHP_TIDY_MODULE_VERSION " ($Id: tidy.c,v 1.66.2.5 2006/01/01 12:50:16 sniper Exp $)");
 	php_info_print_table_end();
 
 	DISPLAY_INI_ENTRIES();
 }
 
-static PHP_FUNCTION(ob_tidyhandler)
+PHP_FUNCTION(ob_tidyhandler)
 {
 	char *input;
 	int input_len;
 	long mode;
-	TidyBuffer errbuf;
+	TidyBuffer errbuf = {0};
 	TidyDoc doc;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &input, &input_len, &mode) == FAILURE) {
-		return;
+		WRONG_PARAM_COUNT;
 	}
 
 	doc = tidyCreate();
@@ -1026,7 +1017,7 @@ static PHP_FUNCTION(ob_tidyhandler)
 	TIDY_SET_DEFAULT_CONFIG(doc);
 
 	if (input_len > 1) {
-		TidyBuffer buf;
+		TidyBuffer buf = {0};
 		
 		tidyBufInit(&buf);
 		tidyBufAppend(&buf, input, input_len);
@@ -1035,11 +1026,11 @@ static PHP_FUNCTION(ob_tidyhandler)
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errbuf.bp);
 			RETVAL_NULL();
 		} else {
-			TidyBuffer output;
+			TidyBuffer output = {0};
 			tidyBufInit(&output);
 
 			tidySaveBuffer(doc, &output);
-			RETVAL_STRINGL(output.bp, output.size ? output.size-1 : 0, 1);
+			RETVAL_STRINGL(output.bp, output.size-1, 1);
 
 			tidyBufFree(&output);
 		}
@@ -1055,15 +1046,17 @@ static PHP_FUNCTION(ob_tidyhandler)
 
 /* {{{ proto bool tidy_parse_string(string input [, mixed config_options [, string encoding]])
    Parse a document stored in a string */
-static PHP_FUNCTION(tidy_parse_string)
+PHP_FUNCTION(tidy_parse_string)
 {
 	char *input, *enc = NULL;
 	int input_len, enc_len = 0;
-	zval **options = NULL;
+	zval *options = NULL;
 	
 	PHPTidyObj *obj;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|Zs", &input, &input_len, &options, &enc, &enc_len) == FAILURE) {
+	TIDY_SET_CONTEXT;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zs", &input, &input_len, &options, &enc, &enc_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
@@ -1083,7 +1076,7 @@ static PHP_FUNCTION(tidy_parse_string)
 
 /* {{{ proto string tidy_get_error_buffer([boolean detailed])
    Return warnings and errors which occured parsing the specified document*/
-static PHP_FUNCTION(tidy_get_error_buffer)
+PHP_FUNCTION(tidy_get_error_buffer)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1097,15 +1090,14 @@ static PHP_FUNCTION(tidy_get_error_buffer)
 
 /* {{{ proto string tidy_get_output()
    Return a string representing the parsed tidy markup */
-static PHP_FUNCTION(tidy_get_output)
+PHP_FUNCTION(tidy_get_output)
 {
-	TidyBuffer output;
+	TidyBuffer output = {0};
 	TIDY_FETCH_OBJECT;
 
-	tidyBufInit(&output);
 	tidySaveBuffer(obj->ptdoc->doc, &output);
 
-	RETVAL_STRINGL(output.bp, output.size ? output.size-1 : 0, 1);
+	RETVAL_STRINGL(output.bp, output.size-1, 1);
 
 	tidyBufFree(&output);
 }
@@ -1113,17 +1105,18 @@ static PHP_FUNCTION(tidy_get_output)
 
 /* {{{ proto boolean tidy_parse_file(string file [, mixed config_options [, string encoding [, bool use_include_path]]])
    Parse markup in file or URI */
-static PHP_FUNCTION(tidy_parse_file)
+PHP_FUNCTION(tidy_parse_file)
 {
 	char *inputfile, *enc = NULL;
 	int input_len, contents_len, enc_len = 0;
 	zend_bool use_include_path = 0;
 	char *contents;
-	zval **options = NULL;
+	zval *options = NULL;
 	
 	PHPTidyObj *obj;
+	TIDY_SET_CONTEXT;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|Zsb", &inputfile, &input_len,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zsb", &inputfile, &input_len,
 							  &options, &enc, &enc_len, &use_include_path) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -1150,7 +1143,7 @@ static PHP_FUNCTION(tidy_parse_file)
 
 /* {{{ proto boolean tidy_clean_repair()
    Execute configured cleanup and repair operations on parsed markup */
-static PHP_FUNCTION(tidy_clean_repair)
+PHP_FUNCTION(tidy_clean_repair)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1165,23 +1158,25 @@ static PHP_FUNCTION(tidy_clean_repair)
 
 /* {{{ proto boolean tidy_repair_string(string data [, mixed config_file [, string encoding]])
    Repair a string using an optionally provided configuration file */
-static PHP_FUNCTION(tidy_repair_string)
+PHP_FUNCTION(tidy_repair_string)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_quick_repair(INTERNAL_FUNCTION_PARAM_PASSTHRU, FALSE);
 }
 /* }}} */
 
 /* {{{ proto boolean tidy_repair_file(string filename [, mixed config_file [, string encoding [, bool use_include_path]]])
    Repair a file using an optionally provided configuration file */
-static PHP_FUNCTION(tidy_repair_file)
+PHP_FUNCTION(tidy_repair_file)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_quick_repair(INTERNAL_FUNCTION_PARAM_PASSTHRU, TRUE);
 }
 /* }}} */
 
 /* {{{ proto boolean tidy_diagnose()
    Run configured diagnostics on parsed and repaired markup. */
-static PHP_FUNCTION(tidy_diagnose)
+PHP_FUNCTION(tidy_diagnose)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1196,8 +1191,10 @@ static PHP_FUNCTION(tidy_diagnose)
 
 /* {{{ proto string tidy_get_release()
    Get release date (version) for Tidy library */
-static PHP_FUNCTION(tidy_get_release)
+PHP_FUNCTION(tidy_get_release)
 {
+	TIDY_SET_CONTEXT;
+
 	if (ZEND_NUM_ARGS()) {
 		WRONG_PARAM_COUNT;
 	}
@@ -1210,13 +1207,13 @@ static PHP_FUNCTION(tidy_get_release)
 #if HAVE_TIDYOPTGETDOC
 /* {{{ proto string tidy_get_opt_doc(tidy resource, string optname)
    Returns the documentation for the given option name */
-static PHP_FUNCTION(tidy_get_opt_doc)
+PHP_FUNCTION(tidy_get_opt_doc)
 {
 	PHPTidyObj *obj;
 	char *optname, *optval;
 	int optname_len;
 	TidyOption opt;
-
+	
 	TIDY_SET_CONTEXT;
 
 	if (object) {
@@ -1250,7 +1247,7 @@ static PHP_FUNCTION(tidy_get_opt_doc)
 
 /* {{{ proto array tidy_get_config()
    Get current Tidy configuarion */
-static PHP_FUNCTION(tidy_get_config)
+PHP_FUNCTION(tidy_get_config)
 {
 	TidyIterator itOpt;
 	char *opt_name;
@@ -1289,7 +1286,7 @@ static PHP_FUNCTION(tidy_get_config)
 
 /* {{{ proto int tidy_get_status()
    Get status of specfied document. */
-static PHP_FUNCTION(tidy_get_status)
+PHP_FUNCTION(tidy_get_status)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1299,7 +1296,7 @@ static PHP_FUNCTION(tidy_get_status)
 
 /* {{{ proto int tidy_get_html_ver()
    Get the Detected HTML version for the specified document. */
-static PHP_FUNCTION(tidy_get_html_ver)
+PHP_FUNCTION(tidy_get_html_ver)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1309,7 +1306,7 @@ static PHP_FUNCTION(tidy_get_html_ver)
 
 /* {{{ proto boolean tidy_is_xhtml()
    Indicates if the document is a XHTML document. */
-static PHP_FUNCTION(tidy_is_xhtml)
+PHP_FUNCTION(tidy_is_xhtml)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1317,9 +1314,9 @@ static PHP_FUNCTION(tidy_is_xhtml)
 }
 /* }}} */
 
-/* {{{ proto boolean tidy_is_xml()
+/* {{{ proto boolean tidy_is_xhtml()
    Indicates if the document is a generic (non HTML/XHTML) XML document. */
-static PHP_FUNCTION(tidy_is_xml)
+PHP_FUNCTION(tidy_is_xml)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1329,7 +1326,7 @@ static PHP_FUNCTION(tidy_is_xml)
 
 /* {{{ proto int tidy_error_count()
    Returns the Number of Tidy errors encountered for specified document. */
-static PHP_FUNCTION(tidy_error_count)
+PHP_FUNCTION(tidy_error_count)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1339,7 +1336,7 @@ static PHP_FUNCTION(tidy_error_count)
 
 /* {{{ proto int tidy_warning_count()
    Returns the Number of Tidy warnings encountered for specified document. */
-static PHP_FUNCTION(tidy_warning_count)
+PHP_FUNCTION(tidy_warning_count)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1349,7 +1346,7 @@ static PHP_FUNCTION(tidy_warning_count)
 
 /* {{{ proto int tidy_access_count()
    Returns the Number of Tidy accessibility warnings encountered for specified document. */
-static PHP_FUNCTION(tidy_access_count)
+PHP_FUNCTION(tidy_access_count)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1359,7 +1356,7 @@ static PHP_FUNCTION(tidy_access_count)
 
 /* {{{ proto int tidy_config_count()
    Returns the Number of Tidy configuration errors encountered for specified document. */
-static PHP_FUNCTION(tidy_config_count)
+PHP_FUNCTION(tidy_config_count)
 {
 	TIDY_FETCH_OBJECT;
 
@@ -1369,7 +1366,7 @@ static PHP_FUNCTION(tidy_config_count)
 
 /* {{{ proto mixed tidy_getopt(string option)
    Returns the value of the specified configuration option for the tidy document. */
-static PHP_FUNCTION(tidy_getopt)
+PHP_FUNCTION(tidy_getopt)
 {  
 	PHPTidyObj *obj;
 	char *optname;
@@ -1426,18 +1423,18 @@ static PHP_FUNCTION(tidy_getopt)
 }
 /* }}} */
 
-static TIDY_DOC_METHOD(__construct)
+TIDY_DOC_METHOD(__construct)
 {
 	char *inputfile = NULL, *enc = NULL;
 	int input_len = 0, enc_len = 0, contents_len = 0;
 	zend_bool use_include_path = 0;
 	char *contents;
-	zval **options = NULL;
+	zval *options = NULL;
 	
 	PHPTidyObj *obj;
 	TIDY_SET_CONTEXT;	
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|sZsb", &inputfile, &input_len,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|szsb", &inputfile, &input_len,
 							  &options, &enc, &enc_len, &use_include_path) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -1458,20 +1455,20 @@ static TIDY_DOC_METHOD(__construct)
 	}
 }
 
-static TIDY_DOC_METHOD(parseFile)
+TIDY_DOC_METHOD(parseFile)
 {
 	char *inputfile, *enc = NULL;
 	int input_len, enc_len = 0, contents_len = 0;
 	zend_bool use_include_path = 0;
 	char *contents;
-	zval **options = NULL;
+	zval *options = NULL;
 	PHPTidyObj *obj;
 
 	TIDY_SET_CONTEXT;
 
 	obj = (PHPTidyObj *)zend_object_store_get_object(object TSRMLS_CC);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|Zsb", &inputfile, &input_len,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zsb", &inputfile, &input_len,
 							  &options, &enc, &enc_len, &use_include_path) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -1492,16 +1489,16 @@ static TIDY_DOC_METHOD(parseFile)
 	efree(contents);
 }
 
-static TIDY_DOC_METHOD(parseString)
+TIDY_DOC_METHOD(parseString)
 {
 	char *input, *enc = NULL;
 	int input_len, enc_len = 0;
-	zval **options = NULL;
+	zval *options = NULL;
 	PHPTidyObj *obj;
 
 	TIDY_SET_CONTEXT;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|Zs", &input, &input_len, &options, &enc, &enc_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zs", &input, &input_len, &options, &enc, &enc_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
@@ -1519,39 +1516,50 @@ static TIDY_DOC_METHOD(parseString)
 
 /* {{{ proto TidyNode tidy_get_root()
    Returns a TidyNode Object representing the root of the tidy parse tree */
-static PHP_FUNCTION(tidy_get_root)
+PHP_FUNCTION(tidy_get_root)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_create_node(INTERNAL_FUNCTION_PARAM_PASSTHRU, is_root_node);
 }
 /* }}} */
 
 /* {{{ proto TidyNode tidy_get_html()
    Returns a TidyNode Object starting from the <HTML> tag of the tidy parse tree */
-static PHP_FUNCTION(tidy_get_html)
+PHP_FUNCTION(tidy_get_html)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_create_node(INTERNAL_FUNCTION_PARAM_PASSTHRU, is_html_node);
 }
 /* }}} */
 
 /* {{{ proto TidyNode tidy_get_head()
    Returns a TidyNode Object starting from the <HEAD> tag of the tidy parse tree */
-static PHP_FUNCTION(tidy_get_head)
+PHP_FUNCTION(tidy_get_head)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_create_node(INTERNAL_FUNCTION_PARAM_PASSTHRU, is_head_node);
 }
 /* }}} */
 
 /* {{{ proto TidyNode tidy_get_body(resource tidy)
    Returns a TidyNode Object starting from the <BODY> tag of the tidy parse tree */
-static PHP_FUNCTION(tidy_get_body)
+PHP_FUNCTION(tidy_get_body)
 {
+	TIDY_SET_CONTEXT;
 	php_tidy_create_node(INTERNAL_FUNCTION_PARAM_PASSTHRU, is_body_node);
 }
 /* }}} */
 
+/* {{{ proto tidyNode::tidyNode()
+   Constructor. */
+TIDY_NODE_METHOD(__construct)
+{	
+}
+/* }}} */
+
 /* {{{ proto boolean tidyNode::hasChildren()
    Returns true if this node has children */
-static TIDY_NODE_METHOD(hasChildren)
+TIDY_NODE_METHOD(hasChildren)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1565,11 +1573,11 @@ static TIDY_NODE_METHOD(hasChildren)
 
 /* {{{ proto boolean tidyNode::hasSiblings()
    Returns true if this node has siblings */
-static TIDY_NODE_METHOD(hasSiblings)
+TIDY_NODE_METHOD(hasSiblings)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
-	if (obj->node && tidyGetNext(obj->node)) {
+	if (tidyGetNext(obj->node)) {
 		RETURN_TRUE;
 	} else {
 		RETURN_FALSE;
@@ -1579,7 +1587,7 @@ static TIDY_NODE_METHOD(hasSiblings)
 
 /* {{{ proto boolean tidyNode::isComment()
    Returns true if this node represents a comment */
-static TIDY_NODE_METHOD(isComment)
+TIDY_NODE_METHOD(isComment)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1593,7 +1601,7 @@ static TIDY_NODE_METHOD(isComment)
 
 /* {{{ proto boolean tidyNode::isHtml()
    Returns true if this node is part of a HTML document */
-static TIDY_NODE_METHOD(isHtml)
+TIDY_NODE_METHOD(isHtml)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1605,9 +1613,37 @@ static TIDY_NODE_METHOD(isHtml)
 }
 /* }}} */
 
+/* {{{ proto boolean tidyNode::isXhtml()
+   Returns true if this node is part of a XHTML document */
+TIDY_NODE_METHOD(isXhtml)
+{
+	TIDY_FETCH_ONLY_OBJECT;
+
+	if (tidyDetectedXhtml(obj->ptdoc->doc)) {
+		RETURN_TRUE;
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
+/* {{{ proto boolean tidyNode::isXml()
+   Returns true if this node is part of a XML document */
+TIDY_NODE_METHOD(isXml)
+{
+	TIDY_FETCH_ONLY_OBJECT;
+
+	if (tidyDetectedGenericXml(obj->ptdoc->doc)) {
+		RETURN_TRUE;
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
 /* {{{ proto boolean tidyNode::isText()
    Returns true if this node represents text (no markup) */
-static TIDY_NODE_METHOD(isText)
+TIDY_NODE_METHOD(isText)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1621,7 +1657,7 @@ static TIDY_NODE_METHOD(isText)
 
 /* {{{ proto boolean tidyNode::isJste()
    Returns true if this node is JSTE */
-static TIDY_NODE_METHOD(isJste)
+TIDY_NODE_METHOD(isJste)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1635,7 +1671,7 @@ static TIDY_NODE_METHOD(isJste)
 
 /* {{{ proto boolean tidyNode::isAsp()
    Returns true if this node is ASP */
-static TIDY_NODE_METHOD(isAsp)
+TIDY_NODE_METHOD(isAsp)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1649,7 +1685,7 @@ static TIDY_NODE_METHOD(isAsp)
 
 /* {{{ proto boolean tidyNode::isPhp()
    Returns true if this node is PHP */
-static TIDY_NODE_METHOD(isPhp)
+TIDY_NODE_METHOD(isPhp)
 {
 	TIDY_FETCH_ONLY_OBJECT;
 
@@ -1661,29 +1697,6 @@ static TIDY_NODE_METHOD(isPhp)
 }
 /* }}} */
 
-/* {{{ proto tidyNode tidyNode::getParent()
-   Returns the parent node if available or NULL */
-static TIDY_NODE_METHOD(getParent)
-{
-	TidyNode	parent_node;
-	PHPTidyObj *newobj;
-	TIDY_FETCH_ONLY_OBJECT;
-
-	parent_node = tidyGetParent(obj->node);
-	if(parent_node) {
-		tidy_instanciate(tidy_ce_node, return_value TSRMLS_CC);
-		newobj = (PHPTidyObj *) zend_object_store_get_object(return_value TSRMLS_CC);
-		newobj->node = parent_node;
-		newobj->type = is_node;
-		newobj->ptdoc = obj->ptdoc;
-		newobj->ptdoc->ref_count++;
-		tidy_add_default_properties(newobj, is_node TSRMLS_CC);
-	} else {
-		ZVAL_NULL(return_value);
-	}
-}
-/* }}} */
-
 static void _php_tidy_register_nodetypes(INIT_FUNC_ARGS)
 {
 	TIDY_NODE_CONST(ROOT, Root);
