@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,11 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: wddx.c,v 1.119.2.10.2.15 2007/03/04 04:38:43 stas Exp $ */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+/* $Id: wddx.c,v 1.119.2.8 2006/01/01 12:50:16 sniper Exp $ */
 
 #include "php.h"
 
@@ -39,7 +35,6 @@
 #include "ext/standard/html.h"
 #include "ext/standard/php_string.h"
 #include "ext/date/php_date.h"
-#include "zend_globals.h"
 
 #define WDDX_BUF_LEN			256
 #define PHP_CLASS_NAME_VAR		"php_class_name"
@@ -284,7 +279,7 @@ PS_SERIALIZER_DECODE_FUNC(wddx)
 
 			switch (hash_type) {
 				case HASH_KEY_IS_LONG:
-					key_length = slprintf(tmp, sizeof(tmp), "%ld", idx) + 1;
+					sprintf(tmp, "%ld", idx);
 					key = tmp;
 					/* fallthru */
 				case HASH_KEY_IS_STRING:
@@ -366,22 +361,53 @@ void php_wddx_packet_end(wddx_packet *packet)
 
 /* {{{ php_wddx_serialize_string
  */
-static void php_wddx_serialize_string(wddx_packet *packet, zval *var TSRMLS_DC)
+static void php_wddx_serialize_string(wddx_packet *packet, zval *var)
 {
+	char *buf,
+		 *p,
+		 *vend,
+		 control_buf[WDDX_BUF_LEN];
+	int l;
+
 	php_wddx_add_chunk_static(packet, WDDX_STRING_S);
 
 	if (Z_STRLEN_P(var) > 0) {
-		char *buf, *enc;
-		int buf_len, enc_len;
+		l = 0;
+		vend = Z_STRVAL_P(var) + Z_STRLEN_P(var);
+		buf = (char *)emalloc(Z_STRLEN_P(var) + 1);
 
-		buf = php_escape_html_entities(Z_STRVAL_P(var), Z_STRLEN_P(var), &buf_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
-		enc = xml_utf8_encode(buf, buf_len, &enc_len, "ISO-8859-1");
+		for(p = Z_STRVAL_P(var); p != vend; p++) {
+			switch (*p) {
+				case '<':
+					FLUSH_BUF();
+					php_wddx_add_chunk_static(packet, "&lt;");
+					break;
 
-		php_wddx_add_chunk_ex(packet, enc, enc_len);
+				case '&':
+					FLUSH_BUF();
+					php_wddx_add_chunk_static(packet, "&amp;");
+					break;
 
+				case '>':
+					FLUSH_BUF();
+					php_wddx_add_chunk_static(packet, "&gt;");
+					break;
+
+				default:
+					if (iscntrl((int)*(unsigned char *)p)) {
+						FLUSH_BUF();
+						sprintf(control_buf, WDDX_CHAR, *p);
+						php_wddx_add_chunk(packet, control_buf);
+					} else
+						buf[l++] = *p;
+					break;
+			}
+		}
+
+		FLUSH_BUF();
 		efree(buf);
-		efree(enc);
 	}
+	
 	php_wddx_add_chunk_static(packet, WDDX_STRING_E);
 }
 /* }}} */
@@ -409,7 +435,7 @@ static void php_wddx_serialize_boolean(wddx_packet *packet, zval *var)
 {
 	char tmp_buf[WDDX_BUF_LEN];
 
-	snprintf(tmp_buf, sizeof(tmp_buf), WDDX_BOOLEAN, Z_LVAL_P(var) ? "true" : "false");
+	sprintf(tmp_buf, WDDX_BOOLEAN, Z_LVAL_P(var) ? "true" : "false");
 	php_wddx_add_chunk(packet, tmp_buf);
 }
 /* }}} */
@@ -448,7 +474,7 @@ static void php_wddx_serialize_object(wddx_packet *packet, zval *obj)
 			PHP_SET_CLASS_ATTRIBUTES(obj);
 
 			php_wddx_add_chunk_static(packet, WDDX_STRUCT_S);
-			snprintf(tmp_buf, WDDX_BUF_LEN, WDDX_VAR_S, PHP_CLASS_NAME_VAR);
+			sprintf(tmp_buf, WDDX_VAR_S, PHP_CLASS_NAME_VAR);
 			php_wddx_add_chunk(packet, tmp_buf);
 			php_wddx_add_chunk_static(packet, WDDX_STRING_S);
 			php_wddx_add_chunk_ex(packet, class_name, name_len);
@@ -480,7 +506,7 @@ static void php_wddx_serialize_object(wddx_packet *packet, zval *obj)
 		PHP_SET_CLASS_ATTRIBUTES(obj);
 
 		php_wddx_add_chunk_static(packet, WDDX_STRUCT_S);
-		snprintf(tmp_buf, WDDX_BUF_LEN, WDDX_VAR_S, PHP_CLASS_NAME_VAR);
+		sprintf(tmp_buf, WDDX_VAR_S, PHP_CLASS_NAME_VAR);
 		php_wddx_add_chunk(packet, tmp_buf);
 		php_wddx_add_chunk_static(packet, WDDX_STRING_S);
 		php_wddx_add_chunk_ex(packet, class_name, name_len);
@@ -498,10 +524,10 @@ static void php_wddx_serialize_object(wddx_packet *packet, zval *obj)
 			if (zend_hash_get_current_key_ex(HASH_OF(obj), &key, &key_len, &idx, 0, NULL) == HASH_KEY_IS_STRING) {
 				char *class_name, *prop_name;
 				
-				zend_unmangle_property_name(key, key_len-1, &class_name, &prop_name);
+				zend_unmangle_property_name_ex(key, key_len, &class_name, &prop_name);
 				php_wddx_serialize_var(packet, *ent, prop_name, strlen(prop_name)+1 TSRMLS_CC);
 			} else {
-				key_len = slprintf(tmp_buf, sizeof(tmp_buf), "%ld", idx);
+				key_len = sprintf(tmp_buf, "%ld", idx);
 				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len TSRMLS_CC);
 			}
 		}
@@ -556,7 +582,7 @@ static void php_wddx_serialize_array(wddx_packet *packet, zval *arr)
 	if (is_struct) {
 		php_wddx_add_chunk_static(packet, WDDX_STRUCT_S);
 	} else {
-		snprintf(tmp_buf, sizeof(tmp_buf), WDDX_ARRAY_S, zend_hash_num_elements(target_hash));
+		sprintf(tmp_buf, WDDX_ARRAY_S, zend_hash_num_elements(target_hash));
 		php_wddx_add_chunk(packet, tmp_buf);
 	}
 
@@ -572,7 +598,7 @@ static void php_wddx_serialize_array(wddx_packet *packet, zval *arr)
 			if (ent_type == HASH_KEY_IS_STRING) {
 				php_wddx_serialize_var(packet, *ent, key, key_len TSRMLS_CC);
 			} else {
-				key_len = slprintf(tmp_buf, sizeof(tmp_buf), "%ld", idx);
+				key_len = sprintf(tmp_buf, "%ld", idx);
 				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len TSRMLS_CC);
 			}
 		} else
@@ -607,7 +633,7 @@ void php_wddx_serialize_var(wddx_packet *packet, zval *var, char *name, int name
 	
 	switch(Z_TYPE_P(var)) {
 		case IS_STRING:
-			php_wddx_serialize_string(packet, var TSRMLS_CC);
+			php_wddx_serialize_string(packet, var);
 			break;
 			
 		case IS_LONG:
@@ -626,7 +652,7 @@ void php_wddx_serialize_var(wddx_packet *packet, zval *var, char *name, int name
 		case IS_ARRAY:
 			ht = Z_ARRVAL_P(var);
 			if (ht->nApplyCount > 1) {
-				php_error_docref(NULL TSRMLS_CC, E_RECOVERABLE_ERROR, "WDDX doesn't support circular references");
+				php_error_docref(NULL TSRMLS_CC, E_ERROR, "WDDX doesn't support circular references");
 				return;
 			}
 			ht->nApplyCount++;															
@@ -637,7 +663,7 @@ void php_wddx_serialize_var(wddx_packet *packet, zval *var, char *name, int name
 		case IS_OBJECT:
 			ht = Z_OBJPROP_P(var);
 			if (ht->nApplyCount > 1) {
-				php_error_docref(NULL TSRMLS_CC, E_RECOVERABLE_ERROR, "WDDX doesn't support circular references");
+				php_error_docref(NULL TSRMLS_CC, E_ERROR, "WDDX doesn't support circular references");
 				return;
 			}
 			ht->nApplyCount++;
@@ -692,7 +718,7 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 	if (!strcmp(name, EL_PACKET)) {
 		int i;
 		
-		if (atts) for (i=0; atts[i]; i++) {
+		for (i=0; atts[i]; i++) {
 			if (!strcmp(atts[i], EL_VERSION)) {
 				/* nothing for now */
 			}
@@ -720,11 +746,11 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 	} else if (!strcmp(name, EL_CHAR)) {
 		int i;
 		
-		if (atts) for (i = 0; atts[i]; i++) {
+		for (i = 0; atts[i]; i++) {
 			if (!strcmp(atts[i], EL_CHAR_CODE) && atts[++i] && atts[i][0]) {
 				char tmp_buf[2];
 
-				snprintf(tmp_buf, sizeof(tmp_buf), "%c", (char)strtol(atts[i], NULL, 16));
+				sprintf(tmp_buf, "%c", (char)strtol(atts[i], NULL, 16));
 				php_wddx_process_data(user_data, tmp_buf, strlen(tmp_buf));
 				break;
 			}
@@ -740,7 +766,7 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 	} else if (!strcmp(name, EL_BOOLEAN)) {
 		int i;
 
-		if (atts) for (i = 0; atts[i]; i++) {
+		for (i = 0; atts[i]; i++) {
 			if (!strcmp(atts[i], EL_VALUE) && atts[++i] && atts[i][0]) {
 				ent.type = ST_BOOLEAN;
 				SET_STACK_VARNAME;
@@ -781,7 +807,7 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 	} else if (!strcmp(name, EL_VAR)) {
 		int i;
 		
-		if (atts) for (i = 0; atts[i]; i++) {
+		for (i = 0; atts[i]; i++) {
 			if (!strcmp(atts[i], EL_NAME) && atts[++i] && atts[i][0]) {
 				char *decoded;
 				int decoded_len;
@@ -798,7 +824,7 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 		MAKE_STD_ZVAL(ent.data);
 		array_init(ent.data);
 
-		if (atts) for (i = 0; atts[i]; i++) {
+		for (i = 0; atts[i]; i++) {
 			if (!strcmp(atts[i], "fieldNames") && atts[++i] && atts[i][0]) {
 				zval *tmp;
 				char *key;
@@ -838,7 +864,7 @@ static void php_wddx_push_element(void *user_data, const XML_Char *name, const X
 		ent.varname = NULL;
 		ent.data = NULL;
 
-		if (atts) for (i = 0; atts[i]; i++) {
+		for (i = 0; atts[i]; i++) {
 			if (!strcmp(atts[i], EL_NAME) && atts[++i] && atts[i][0]) {
 				char *decoded;
 				int decoded_len;
@@ -974,11 +1000,10 @@ static void php_wddx_pop_element(void *user_data, const XML_Char *name)
 						add_property_zval(ent2->data, ent1->varname, ent1->data);
 						EG(scope) = old_scope;
 					} else {
-						long l;
+						long l;  
 						double d;
-						int varname_len = strlen(ent1->varname);
 				
-						switch (is_numeric_string(ent1->varname, varname_len, &l, &d, 0)) {
+						switch (is_numeric_string(ent1->varname, strlen(ent1->varname), &l, &d, 0)) {
 							case IS_DOUBLE:
 								if (d > INT_MAX) {
 									goto bigint;
@@ -989,7 +1014,7 @@ static void php_wddx_pop_element(void *user_data, const XML_Char *name)
 								break;
 							default:
 bigint:
-								zend_hash_update(target_hash,ent1->varname, varname_len + 1, &ent1->data, sizeof(zval *), NULL);
+								zend_hash_update(target_hash,ent1->varname, strlen(ent1->varname)+1, &ent1->data, sizeof(zval *), NULL);
 						}
 					}
 					efree(ent1->varname);
@@ -1034,8 +1059,9 @@ static void php_wddx_process_data(void *user_data, const XML_Char *s, int len)
 					Z_STRVAL_P(ent->data) = estrndup(decoded, decoded_len);
 					Z_STRLEN_P(ent->data) = decoded_len;
 				} else {
-					Z_STRVAL_P(ent->data) = erealloc(Z_STRVAL_P(ent->data), Z_STRLEN_P(ent->data) + decoded_len + 1);
-					memcpy(Z_STRVAL_P(ent->data) + Z_STRLEN_P(ent->data), decoded, decoded_len);
+					Z_STRVAL_P(ent->data) = erealloc(Z_STRVAL_P(ent->data),
+							Z_STRLEN_P(ent->data) + decoded_len + 1);
+					strncpy(Z_STRVAL_P(ent->data)+Z_STRLEN_P(ent->data), decoded, decoded_len);
 					Z_STRLEN_P(ent->data) += decoded_len;
 					Z_STRVAL_P(ent->data)[Z_STRLEN_P(ent->data)] = '\0';
 				}
