@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
+/* $Id: xml.c,v 1.157.2.3 2006/01/01 12:50:16 sniper Exp $ */
 
 #define IS_EXT_MODULE
 
@@ -32,7 +32,6 @@
 #include "zend_variables.h"
 #include "ext/standard/php_string.h"
 #include "ext/standard/info.h"
-#include "ext/standard/html.h"
 
 #if HAVE_XML
 
@@ -58,18 +57,24 @@
  * - Weird things happen with <![CDATA[]]> sections.
  */
 
-ZEND_DECLARE_MODULE_GLOBALS(xml)
+#ifdef ZTS
+int xml_globals_id;
+#else
+PHP_XML_API php_xml_globals xml_globals;
+#endif
 
 /* {{{ dynamically loadable module stuff */
 #ifdef COMPILE_DL_XML
 ZEND_GET_MODULE(xml)
+# ifdef PHP_WIN32
+# include "zend_arg_defs.c"
+# endif
 #endif /* COMPILE_DL_XML */
 /* }}} */
 
 /* {{{ function prototypes */
 PHP_MINIT_FUNCTION(xml);
 PHP_MINFO_FUNCTION(xml);
-static PHP_GINIT_FUNCTION(xml);
 
 static void xml_parser_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC);
 static void xml_set_handler(zval **, zval **);
@@ -77,6 +82,7 @@ inline static unsigned short xml_encode_iso_8859_1(unsigned char);
 inline static char xml_decode_iso_8859_1(unsigned short);
 inline static unsigned short xml_encode_us_ascii(unsigned char);
 inline static char xml_decode_us_ascii(unsigned short);
+static XML_Char *xml_utf8_encode(const char *, int, int *, const XML_Char *);
 static zval *xml_call_handler(xml_parser *, zval *, zend_function *, int, zval **);
 static zval *_xml_xmlchar_zval(const XML_Char *, int, const XML_Char *);
 static int _xml_xmlcharlen(const XML_Char *);
@@ -97,154 +103,46 @@ void _xml_endNamespaceDeclHandler(void *, const XML_Char *);
 /* }}} */
 
 /* {{{ extension definition structures */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_create, 0, 0, 0)
-	ZEND_ARG_INFO(0, encoding)
-ZEND_END_ARG_INFO()
+static
+	ZEND_BEGIN_ARG_INFO(third_and_fourth_args_force_ref, 0)
+		ZEND_ARG_PASS_INFO(0)
+		ZEND_ARG_PASS_INFO(0)
+		ZEND_ARG_PASS_INFO(1)
+		ZEND_ARG_PASS_INFO(1)
+	ZEND_END_ARG_INFO();
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_create_ns, 0, 0, 0)
-	ZEND_ARG_INFO(0, encoding)
-	ZEND_ARG_INFO(0, sep)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_object, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(1, obj)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_element_handler, 0, 0, 3)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, shdl)
-	ZEND_ARG_INFO(0, ehdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_character_data_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_processing_instruction_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_default_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_unparsed_entity_decl_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_notation_decl_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_external_entity_ref_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_start_namespace_decl_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_end_namespace_decl_handler, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, hdl)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parse, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, data)
-	ZEND_ARG_INFO(0, isfinal)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parse_into_struct, 0, 0, 3)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, data)
-	ZEND_ARG_INFO(1, values)
-	ZEND_ARG_INFO(1, index)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_error_code, 0, 0, 1)
-	ZEND_ARG_INFO(0, parser)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_error_string, 0, 0, 1)
-	ZEND_ARG_INFO(0, code)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_line_number, 0, 0, 1)
-	ZEND_ARG_INFO(0, parser)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_column_number, 0, 0, 1)
-	ZEND_ARG_INFO(0, parser)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_byte_index, 0, 0, 1)
-	ZEND_ARG_INFO(0, parser)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_free, 0, 0, 1)
-	ZEND_ARG_INFO(0, parser)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_set_option, 0, 0, 3)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, option)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_get_option, 0, 0, 2)
-	ZEND_ARG_INFO(0, parser)
-	ZEND_ARG_INFO(0, option)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_utf8_encode, 0, 0, 1)
-	ZEND_ARG_INFO(0, data)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_utf8_decode, 0, 0, 1)
-	ZEND_ARG_INFO(0, data)
-ZEND_END_ARG_INFO()
-
-const zend_function_entry xml_functions[] = {
-	PHP_FE(xml_parser_create,					arginfo_xml_parser_create)
-	PHP_FE(xml_parser_create_ns,				arginfo_xml_parser_create_ns)
-	PHP_FE(xml_set_object, 						arginfo_xml_set_object)
-	PHP_FE(xml_set_element_handler,				arginfo_xml_set_element_handler)
-	PHP_FE(xml_set_character_data_handler,		arginfo_xml_set_character_data_handler)
-	PHP_FE(xml_set_processing_instruction_handler, 	arginfo_xml_set_processing_instruction_handler)
-	PHP_FE(xml_set_default_handler, 				arginfo_xml_set_default_handler)
-	PHP_FE(xml_set_unparsed_entity_decl_handler,arginfo_xml_set_unparsed_entity_decl_handler)
-	PHP_FE(xml_set_notation_decl_handler,		arginfo_xml_set_notation_decl_handler)
-	PHP_FE(xml_set_external_entity_ref_handler,	arginfo_xml_set_external_entity_ref_handler)
-	PHP_FE(xml_set_start_namespace_decl_handler,arginfo_xml_set_start_namespace_decl_handler)
-	PHP_FE(xml_set_end_namespace_decl_handler,	arginfo_xml_set_end_namespace_decl_handler)
-	PHP_FE(xml_parse,							arginfo_xml_parse)
-	PHP_FE(xml_parse_into_struct, 				arginfo_xml_parse_into_struct)
-	PHP_FE(xml_get_error_code,					arginfo_xml_get_error_code)
-	PHP_FE(xml_error_string,					arginfo_xml_error_string)
-	PHP_FE(xml_get_current_line_number,			arginfo_xml_get_current_line_number)
-	PHP_FE(xml_get_current_column_number,		arginfo_xml_get_current_column_number)
-	PHP_FE(xml_get_current_byte_index,			arginfo_xml_get_current_byte_index)
-	PHP_FE(xml_parser_free, 					arginfo_xml_parser_free)
-	PHP_FE(xml_parser_set_option, 				arginfo_xml_parser_set_option)
-	PHP_FE(xml_parser_get_option,				arginfo_xml_parser_get_option)
-	PHP_FE(utf8_encode, 						arginfo_utf8_encode)
-	PHP_FE(utf8_decode, 						arginfo_utf8_decode)
-	PHP_FE_END
+zend_function_entry xml_functions[] = {
+	PHP_FE(xml_parser_create, NULL)
+	PHP_FE(xml_parser_create_ns, NULL)
+	PHP_FE(xml_set_object, second_arg_force_ref)
+	PHP_FE(xml_set_element_handler, NULL)
+	PHP_FE(xml_set_character_data_handler, NULL)
+	PHP_FE(xml_set_processing_instruction_handler, NULL)
+	PHP_FE(xml_set_default_handler, NULL)
+	PHP_FE(xml_set_unparsed_entity_decl_handler, NULL)
+	PHP_FE(xml_set_notation_decl_handler, NULL)
+	PHP_FE(xml_set_external_entity_ref_handler, NULL)
+	PHP_FE(xml_set_start_namespace_decl_handler, NULL)
+	PHP_FE(xml_set_end_namespace_decl_handler, NULL)
+	PHP_FE(xml_parse, NULL)
+	PHP_FE(xml_parse_into_struct, third_and_fourth_args_force_ref)
+	PHP_FE(xml_get_error_code, NULL)
+	PHP_FE(xml_error_string, NULL)
+	PHP_FE(xml_get_current_line_number, NULL)
+	PHP_FE(xml_get_current_column_number, NULL)
+	PHP_FE(xml_get_current_byte_index, NULL)
+	PHP_FE(xml_parser_free, NULL)
+	PHP_FE(xml_parser_set_option, NULL)
+	PHP_FE(xml_parser_get_option, NULL)
+	PHP_FE(utf8_encode, NULL)
+	PHP_FE(utf8_decode, NULL)
+	{NULL, NULL, NULL}
 };
 
 #ifdef LIBXML_EXPAT_COMPAT
-static const zend_module_dep xml_deps[] = {
+static zend_module_dep xml_deps[] = {
 	ZEND_MOD_REQUIRED("libxml")
-	ZEND_MOD_END
+	{NULL, NULL, NULL}
 };
 #endif
 
@@ -263,21 +161,17 @@ zend_module_entry xml_module_entry = {
 	NULL,                 /* per-request shutdown function */
 	PHP_MINFO(xml),       /* information function */
     NO_VERSION_YET,
-    PHP_MODULE_GLOBALS(xml), /* globals descriptor */
-    PHP_GINIT(xml),          /* globals ctor */
-    NULL,                    /* globals dtor */
-    NULL,                    /* post deactivate */
-	STANDARD_MODULE_PROPERTIES_EX
+	STANDARD_MODULE_PROPERTIES
 };
 
 /* All the encoding functions are set to NULL right now, since all
  * the encoding is currently done internally by expat/xmltok.
  */
 xml_encoding xml_encodings[] = {
-	{ (XML_Char *)"ISO-8859-1", xml_decode_iso_8859_1, xml_encode_iso_8859_1 },
-	{ (XML_Char *)"US-ASCII",   xml_decode_us_ascii,   xml_encode_us_ascii   },
-	{ (XML_Char *)"UTF-8",      NULL,                  NULL                  },
-	{ (XML_Char *)NULL,         NULL,                  NULL                  }
+	{ "ISO-8859-1", xml_decode_iso_8859_1, xml_encode_iso_8859_1 },
+	{ "US-ASCII",   xml_decode_us_ascii,   xml_encode_us_ascii   },
+	{ "UTF-8",      NULL,                  NULL                  },
+	{ NULL,         NULL,                  NULL                  }
 };
 
 static XML_Memory_Handling_Suite php_xml_mem_hdlrs;
@@ -288,10 +182,12 @@ static int le_xml_parser;
 /* }}} */
 
 /* {{{ startup, shutdown and info functions */
-static PHP_GINIT_FUNCTION(xml)
+#ifdef ZTS
+static void php_xml_init_globals(php_xml_globals *xml_globals_p TSRMLS_DC)
 {
-	xml_globals->default_encoding = "UTF-8";
+	XML(default_encoding) = "UTF-8";
 }
+#endif
 
 static void *php_xml_malloc_wrapper(size_t sz)
 {
@@ -314,6 +210,12 @@ PHP_MINIT_FUNCTION(xml)
 {
 	le_xml_parser =	zend_register_list_destructors_ex(xml_parser_dtor, NULL, "xml", module_number);
 
+#ifdef ZTS
+	ts_allocate_id(&xml_globals_id, sizeof(php_xml_globals), (ts_allocate_ctor) php_xml_init_globals, NULL);
+#else
+	XML(default_encoding) = "UTF-8";
+#endif
+
 	REGISTER_LONG_CONSTANT("XML_ERROR_NONE", XML_ERROR_NONE, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XML_ERROR_NO_MEMORY", XML_ERROR_NO_MEMORY, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XML_ERROR_SYNTAX", XML_ERROR_SYNTAX, CONST_CS|CONST_PERSISTENT);
@@ -428,7 +330,7 @@ static void xml_parser_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 	}
 	if (parser->ltags) {
 		int inx;
-		for (inx = 0; ((inx < parser->level) && (inx < XML_MAXLEVEL)); inx++)
+		for (inx = 0; inx < parser->level; inx++)
 			efree(parser->ltags[ inx ]);
 		efree(parser->ltags);
 	}
@@ -485,13 +387,8 @@ static void xml_set_handler(zval **handler, zval **data)
 	}
 
 	/* IS_ARRAY might indicate that we're using array($obj, 'method') syntax */
-	if (Z_TYPE_PP(data) != IS_ARRAY && Z_TYPE_PP(data) != IS_OBJECT) {
-
+	if (Z_TYPE_PP(data) != IS_ARRAY) {
 		convert_to_string_ex(data);
-		if (Z_STRLEN_PP(data) == 0) {
-			*handler = NULL;
-			return;
-		}
 	}
 
 	zval_add_ref(data);
@@ -521,7 +418,7 @@ static zval *xml_call_handler(xml_parser *parser, zval *handler, zend_function *
 		fci.function_table = EG(function_table);
 		fci.function_name = handler;
 		fci.symbol_table = NULL;
-		fci.object_ptr = parser->object;
+		fci.object_pp = &parser->object;
 		fci.retval_ptr_ptr = &retval;
 		fci.param_count = argc;
 		fci.params = args;
@@ -607,7 +504,7 @@ static xml_encoding *xml_get_encoding(const XML_Char *name)
 /* }}} */
 
 /* {{{ xml_utf8_encode */
-PHPAPI char *xml_utf8_encode(const char *s, int len, int *newlen, const XML_Char *encoding)
+static XML_Char *xml_utf8_encode(const char *s, int len, int *newlen, const XML_Char *encoding)
 {
 	int pos = len;
 	char *newbuf;
@@ -663,9 +560,9 @@ PHPAPI char *xml_utf8_encode(const char *s, int len, int *newlen, const XML_Char
 /* {{{ xml_utf8_decode */
 PHPAPI char *xml_utf8_decode(const XML_Char *s, int len, int *newlen, const XML_Char *encoding)
 {
-	size_t pos = 0;
+	int pos = len;
 	char *newbuf = emalloc(len + 1);
-	unsigned int c;
+	unsigned short c;
 	char (*decoder)(unsigned short) = NULL;
 	xml_encoding *enc = xml_get_encoding(encoding);
 
@@ -682,15 +579,24 @@ PHPAPI char *xml_utf8_decode(const XML_Char *s, int len, int *newlen, const XML_
 		newbuf[*newlen] = '\0';
 		return newbuf;
 	}
-
-	while (pos < (size_t)len) {
-		int status = FAILURE;
-		c = php_next_utf8_char((const unsigned char*)s, (size_t) len, &pos, &status);
-
-		if (status == FAILURE || c > 0xFFU) {
-			c = '?';
+	while (pos > 0) {
+		c = (unsigned char)(*s);
+		if (c >= 0xf0) { /* four bytes encoded, 21 bits */
+			c = ((s[0]&7)<<18) | ((s[1]&63)<<12) | ((s[2]&63)<<6) | (s[3]&63);
+			s += 4;
+			pos -= 4;
+		} else if (c >= 0xe0) { /* three bytes encoded, 16 bits */
+			c = ((s[0]&63)<<12) | ((s[1]&63)<<6) | (s[2]&63);
+			s += 3;
+			pos -= 3;
+		} else if (c >= 0xc0) { /* two bytes encoded, 11 bits */
+			c = ((s[0]&63)<<6) | (s[1]&63);
+			s += 2;
+			pos -= 2;
+		} else {
+			s++;
+			pos--;
 		}
-
 		newbuf[*newlen] = decoder ? decoder(c) : c;
 		++*newlen;
 	}
@@ -784,7 +690,7 @@ void _xml_startElementHandler(void *userData, const XML_Char *name, const XML_Ch
 
 		if (parser->startElementHandler) {
 			args[0] = _xml_resource_zval(parser->index);
-			args[1] = _xml_string_zval(((char *) tag_name) + parser->toffset);
+			args[1] = _xml_string_zval(tag_name);
 			MAKE_STD_ZVAL(args[2]);
 			array_init(args[2]);
 
@@ -805,50 +711,45 @@ void _xml_startElementHandler(void *userData, const XML_Char *name, const XML_Ch
 		} 
 
 		if (parser->data) {
-			if (parser->level <= XML_MAXLEVEL)  {
-				zval *tag, *atr;
-				int atcnt = 0;
+			zval *tag, *atr;
+			int atcnt = 0;
 
-				MAKE_STD_ZVAL(tag);
-				MAKE_STD_ZVAL(atr);
+			MAKE_STD_ZVAL(tag);
+			MAKE_STD_ZVAL(atr);
 
-				array_init(tag);
-				array_init(atr);
+			array_init(tag);
+			array_init(atr);
 
-				_xml_add_to_info(parser,((char *) tag_name) + parser->toffset);
+			_xml_add_to_info(parser,((char *) tag_name) + parser->toffset);
 
-				add_assoc_string(tag,"tag",((char *) tag_name) + parser->toffset,1); /* cast to avoid gcc-warning */
-				add_assoc_string(tag,"type","open",1);
-				add_assoc_long(tag,"level",parser->level);
+			add_assoc_string(tag,"tag",((char *) tag_name) + parser->toffset,1); /* cast to avoid gcc-warning */
+			add_assoc_string(tag,"type","open",1);
+			add_assoc_long(tag,"level",parser->level);
 
-				parser->ltags[parser->level-1] = estrdup(tag_name);
-				parser->lastwasopen = 1;
+			parser->ltags[parser->level-1] = estrdup(tag_name);
+			parser->lastwasopen = 1;
 
-				attributes = (const XML_Char **) attrs;
+			attributes = (const XML_Char **) attrs;
 
-				while (attributes && *attributes) {
-					att = _xml_decode_tag(parser, attributes[0]);
-					val = xml_utf8_decode(attributes[1], strlen(attributes[1]), &val_len, parser->target_encoding);
+			while (attributes && *attributes) {
+				att = _xml_decode_tag(parser, attributes[0]);
+				val = xml_utf8_decode(attributes[1], strlen(attributes[1]), &val_len, parser->target_encoding);
+				
+				add_assoc_stringl(atr,att,val,val_len,0);
 
-					add_assoc_stringl(atr,att,val,val_len,0);
+				atcnt++;
+				attributes += 2;
 
-					atcnt++;
-					attributes += 2;
-
-					efree(att);
-				}
-
-				if (atcnt) {
-					zend_hash_add(Z_ARRVAL_P(tag),"attributes",sizeof("attributes"),&atr,sizeof(zval*),NULL);
-				} else {
-					zval_ptr_dtor(&atr);
-				}
-
-				zend_hash_next_index_insert(Z_ARRVAL_P(parser->data),&tag,sizeof(zval*),(void *) &parser->ctag);
-			} else if (parser->level == (XML_MAXLEVEL + 1)) {
-				TSRMLS_FETCH();
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Maximum depth exceeded - Results truncated");
+				efree(att);
 			}
+
+			if (atcnt) {
+				zend_hash_add(Z_ARRVAL_P(tag),"attributes",sizeof("attributes"),&atr,sizeof(zval*),NULL);
+			} else {
+				zval_ptr_dtor(&atr);
+			}
+
+			zend_hash_next_index_insert(Z_ARRVAL_P(parser->data),&tag,sizeof(zval*),(void *) &parser->ctag);
 		}
 
 		efree(tag_name);
@@ -869,7 +770,7 @@ void _xml_endElementHandler(void *userData, const XML_Char *name)
 
 		if (parser->endElementHandler) {
 			args[0] = _xml_resource_zval(parser->index);
-			args[1] = _xml_string_zval(((char *) tag_name) + parser->toffset);
+			args[1] = _xml_string_zval(tag_name);
 
 			if ((retval = xml_call_handler(parser, parser->endElementHandler, parser->endElementPtr, 2, args))) {
 				zval_ptr_dtor(&retval);
@@ -900,7 +801,7 @@ void _xml_endElementHandler(void *userData, const XML_Char *name)
 
 		efree(tag_name);
 
-		if ((parser->ltags) && (parser->level <= XML_MAXLEVEL)) {
+		if (parser->ltags) {
 			efree(parser->ltags[parser->level-1]);
 		}
 
@@ -955,7 +856,7 @@ void _xml_characterDataHandler(void *userData, const XML_Char *s, int len)
 					if (zend_hash_find(Z_ARRVAL_PP(parser->ctag),"value",sizeof("value"),(void **) &myval) == SUCCESS) {
 						int newlen = Z_STRLEN_PP(myval) + decoded_len;
 						Z_STRVAL_PP(myval) = erealloc(Z_STRVAL_PP(myval),newlen+1);
-						strncpy(Z_STRVAL_PP(myval) + Z_STRLEN_PP(myval), decoded_value, decoded_len + 1);
+						strcpy(Z_STRVAL_PP(myval) + Z_STRLEN_PP(myval),decoded_value);
 						Z_STRLEN_PP(myval) += decoded_len;
 						efree(decoded_value);
 					} else {
@@ -964,43 +865,19 @@ void _xml_characterDataHandler(void *userData, const XML_Char *s, int len)
 					
 				} else {
 					zval *tag;
-					zval **curtag, **mytype, **myval;
-					HashPosition hpos=NULL;
 
-					zend_hash_internal_pointer_end_ex(Z_ARRVAL_P(parser->data), &hpos);
+					MAKE_STD_ZVAL(tag);
+					
+					array_init(tag);
+					
+					_xml_add_to_info(parser,parser->ltags[parser->level-1] + parser->toffset);
 
-					if (hpos && (zend_hash_get_current_data_ex(Z_ARRVAL_P(parser->data), (void **) &curtag, &hpos) == SUCCESS)) {
-						if (zend_hash_find(Z_ARRVAL_PP(curtag),"type",sizeof("type"),(void **) &mytype) == SUCCESS) {
-							if (!strcmp(Z_STRVAL_PP(mytype), "cdata")) {
-								if (zend_hash_find(Z_ARRVAL_PP(curtag),"value",sizeof("value"),(void **) &myval) == SUCCESS) {
-									int newlen = Z_STRLEN_PP(myval) + decoded_len;
-									Z_STRVAL_PP(myval) = erealloc(Z_STRVAL_PP(myval),newlen+1);
-									strncpy(Z_STRVAL_PP(myval) + Z_STRLEN_PP(myval), decoded_value, decoded_len + 1);
-									Z_STRLEN_PP(myval) += decoded_len;
-									efree(decoded_value);
-									return;
-								}
-							}
-						}
-					}
+					add_assoc_string(tag,"tag",parser->ltags[parser->level-1] + parser->toffset,1);
+					add_assoc_string(tag,"value",decoded_value,0);
+					add_assoc_string(tag,"type","cdata",1);
+					add_assoc_long(tag,"level",parser->level);
 
-					if (parser->level <= XML_MAXLEVEL) {
-						MAKE_STD_ZVAL(tag);
-
-						array_init(tag);
-
-						_xml_add_to_info(parser,parser->ltags[parser->level-1] + parser->toffset);
-
-						add_assoc_string(tag,"tag",parser->ltags[parser->level-1] + parser->toffset,1);
-						add_assoc_string(tag,"value",decoded_value,0);
-						add_assoc_string(tag,"type","cdata",1);
-						add_assoc_long(tag,"level",parser->level);
-
-						zend_hash_next_index_insert(Z_ARRVAL_P(parser->data),&tag,sizeof(zval*),NULL);
-					} else if (parser->level == (XML_MAXLEVEL + 1)) {
-						TSRMLS_FETCH();
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Maximum depth exceeded - Results truncated");
-					}
+					zend_hash_next_index_insert(Z_ARRVAL_P(parser->data),&tag,sizeof(zval*),NULL);
 				}
 			} else {
 				efree(decoded_value);
@@ -1162,7 +1039,7 @@ void _xml_endNamespaceDeclHandler(void *userData, const XML_Char *prefix)
 
 /************************* EXTENSION FUNCTIONS *************************/
 
-static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_support) /* {{{ */
+static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_support)
 {
 	xml_parser *parser;
 	int auto_detect = 0;
@@ -1204,7 +1081,7 @@ static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_supp
 		ns_param = ":";
 	}
 
-	parser = ecalloc(1, sizeof(xml_parser));
+	parser = ecalloc(sizeof(xml_parser), 1);
 	parser->parser = XML_ParserCreate_MM((auto_detect ? NULL : encoding),
                                          &php_xml_mem_hdlrs, ns_param);
 
@@ -1218,7 +1095,6 @@ static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_supp
 	ZEND_REGISTER_RESOURCE(return_value, parser,le_xml_parser);
 	parser->index = Z_LVAL_P(return_value);
 }
-/* }}} */
 
 /* {{{ proto resource xml_parser_create([string encoding]) 
    Create an XML parser */
@@ -1241,13 +1117,19 @@ PHP_FUNCTION(xml_parser_create_ns)
 PHP_FUNCTION(xml_set_object)
 {
 	xml_parser *parser;
-	zval *pind, *mythis;
+	zval **pind, **mythis;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ro", &pind, &mythis) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 ||
+		zend_get_parameters_ex(2, &pind, &mythis) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	if (Z_TYPE_PP(mythis) != IS_OBJECT) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Argument 2 has wrong type");
+		RETURN_FALSE;
+	}
+
+	ZEND_FETCH_RESOURCE(parser,xml_parser *,pind, -1, "XML Parser", le_xml_parser);
 
 	/* please leave this commented - or ask thies@thieso.net before doing it (again) */
 	if (parser->object) {
@@ -1260,7 +1142,9 @@ PHP_FUNCTION(xml_set_object)
 #endif */
 
 	ALLOC_ZVAL(parser->object);
-	MAKE_COPY_ZVAL(&mythis, parser->object);
+	*parser->object = **mythis;
+	zval_copy_ctor(parser->object);
+	INIT_PZVAL(parser->object);
 
 	RETVAL_TRUE;
 }
@@ -1271,13 +1155,14 @@ PHP_FUNCTION(xml_set_object)
 PHP_FUNCTION(xml_set_element_handler)
 {
 	xml_parser *parser;
-	zval *pind, **shdl, **ehdl;
+	zval **pind, **shdl, **ehdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZZ", &pind, &shdl, &ehdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 3 ||
+		zend_get_parameters_ex(3, &pind, &shdl, &ehdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *,pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->startElementHandler, shdl);
 	xml_set_handler(&parser->endElementHandler, ehdl);
@@ -1291,13 +1176,13 @@ PHP_FUNCTION(xml_set_element_handler)
 PHP_FUNCTION(xml_set_character_data_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->characterDataHandler, hdl);
 	XML_SetCharacterDataHandler(parser->parser, _xml_characterDataHandler);
@@ -1310,13 +1195,13 @@ PHP_FUNCTION(xml_set_character_data_handler)
 PHP_FUNCTION(xml_set_processing_instruction_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->processingInstructionHandler, hdl);
 	XML_SetProcessingInstructionHandler(parser->parser, _xml_processingInstructionHandler);
@@ -1329,12 +1214,12 @@ PHP_FUNCTION(xml_set_processing_instruction_handler)
 PHP_FUNCTION(xml_set_default_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->defaultHandler, hdl);
 	XML_SetDefaultHandler(parser->parser, _xml_defaultHandler);
@@ -1347,13 +1232,13 @@ PHP_FUNCTION(xml_set_default_handler)
 PHP_FUNCTION(xml_set_unparsed_entity_decl_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->unparsedEntityDeclHandler, hdl);
 	XML_SetUnparsedEntityDeclHandler(parser->parser, _xml_unparsedEntityDeclHandler);
@@ -1366,12 +1251,12 @@ PHP_FUNCTION(xml_set_unparsed_entity_decl_handler)
 PHP_FUNCTION(xml_set_notation_decl_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->notationDeclHandler, hdl);
 	XML_SetNotationDeclHandler(parser->parser, _xml_notationDeclHandler);
@@ -1384,12 +1269,12 @@ PHP_FUNCTION(xml_set_notation_decl_handler)
 PHP_FUNCTION(xml_set_external_entity_ref_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->externalEntityRefHandler, hdl);
 	XML_SetExternalEntityRefHandler(parser->parser, (void *) _xml_externalEntityRefHandler);
@@ -1402,13 +1287,13 @@ PHP_FUNCTION(xml_set_external_entity_ref_handler)
 PHP_FUNCTION(xml_set_start_namespace_decl_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->startNamespaceDeclHandler, hdl);
 	XML_SetStartNamespaceDeclHandler(parser->parser, _xml_startNamespaceDeclHandler);
@@ -1421,13 +1306,13 @@ PHP_FUNCTION(xml_set_start_namespace_decl_handler)
 PHP_FUNCTION(xml_set_end_namespace_decl_handler)
 {
 	xml_parser *parser;
-	zval *pind, **hdl;
+	zval **pind, **hdl;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->endNamespaceDeclHandler, hdl);
 	XML_SetEndNamespaceDeclHandler(parser->parser, _xml_endNamespaceDeclHandler);
@@ -1440,54 +1325,58 @@ PHP_FUNCTION(xml_set_end_namespace_decl_handler)
 PHP_FUNCTION(xml_parse)
 {
 	xml_parser *parser;
-	zval *pind;
-	char *data;
-	int data_len, ret;
-	long isFinal = 0;
+	zval **pind, **data, **final;
+	int argc, isFinal, ret;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &pind, &data, &data_len, &isFinal) == FAILURE) {
-		return;
+	argc = ZEND_NUM_ARGS();
+	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &pind, &data, &final) == FAILURE) {
+		WRONG_PARAM_COUNT;
+	}
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+
+	convert_to_string_ex(data);
+
+	if (argc == 3) {
+		convert_to_long_ex(final);
+		isFinal = Z_LVAL_PP(final);
+	} else {
+		isFinal = 0;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	parser->isparsing = 1;
-	ret = XML_Parse(parser->parser, data, data_len, isFinal);
+	ret = XML_Parse(parser->parser, Z_STRVAL_PP(data), Z_STRLEN_PP(data), isFinal);
 	parser->isparsing = 0;
 	RETVAL_LONG(ret);
 }
 
 /* }}} */
 
-/* {{{ proto int xml_parse_into_struct(resource parser, string data, array &values [, array &index ])
+/* {{{ proto int xml_parse_into_struct(resource parser, string data, array &struct, array &index) 
    Parsing a XML document */
 
 PHP_FUNCTION(xml_parse_into_struct)
 {
 	xml_parser *parser;
-	zval *pind, **xdata, **info = NULL;
-	char *data;
-	int data_len, ret;
+	zval **pind, **data, **xdata, **info = 0;
+	int argc, ret;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsZ|Z", &pind, &data, &data_len, &xdata, &info) == FAILURE) {
-		return;
-	}
-	
-	if (info) {	
+	argc = ZEND_NUM_ARGS();
+	if (zend_get_parameters_ex(4, &pind, &data, &xdata,&info) == SUCCESS) {
 		zval_dtor(*info);
 		array_init(*info);
+	} else if (zend_get_parameters_ex(3, &pind, &data, &xdata) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
+	convert_to_string_ex(data);
 	zval_dtor(*xdata);
 	array_init(*xdata);
 
 	parser->data = *xdata;
-	
-	if (info) {
-		parser->info = *info;
-	}
-	
+	if (info)
+	  parser->info = *info;
 	parser->level = 0;
 	parser->ltags = safe_emalloc(XML_MAXLEVEL, sizeof(char *), 0);
 
@@ -1496,7 +1385,7 @@ PHP_FUNCTION(xml_parse_into_struct)
 	XML_SetCharacterDataHandler(parser->parser, _xml_characterDataHandler);
 
 	parser->isparsing = 1;
-	ret = XML_Parse(parser->parser, data, data_len, 1);
+	ret = XML_Parse(parser->parser, Z_STRVAL_PP(data), Z_STRLEN_PP(data), 1);
 	parser->isparsing = 0;
 
 	RETVAL_LONG(ret);
@@ -1508,12 +1397,12 @@ PHP_FUNCTION(xml_parse_into_struct)
 PHP_FUNCTION(xml_get_error_code)
 {
 	xml_parser *parser;
-	zval *pind;
+	zval **pind;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG((long)XML_GetErrorCode(parser->parser));
 }
@@ -1523,14 +1412,14 @@ PHP_FUNCTION(xml_get_error_code)
    Get XML parser error string */
 PHP_FUNCTION(xml_error_string)
 {
-	long code;
+	zval **code;
 	char *str;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &code) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &code) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	str = (char *)XML_ErrorString((int)code);
+	convert_to_long_ex(code);
+	str = (char *)XML_ErrorString((int)Z_LVAL_PP(code));
 	if (str) {
 		RETVAL_STRING(str, 1);
 	}
@@ -1542,12 +1431,12 @@ PHP_FUNCTION(xml_error_string)
 PHP_FUNCTION(xml_get_current_line_number)
 {
 	xml_parser *parser;
-	zval *pind;
+	zval **pind;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentLineNumber(parser->parser));
 }
@@ -1558,12 +1447,12 @@ PHP_FUNCTION(xml_get_current_line_number)
 PHP_FUNCTION(xml_get_current_column_number)
 {
 	xml_parser *parser;
-	zval *pind;
+	zval **pind;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentColumnNumber(parser->parser));
 }
@@ -1574,12 +1463,12 @@ PHP_FUNCTION(xml_get_current_column_number)
 PHP_FUNCTION(xml_get_current_byte_index)
 {
 	xml_parser *parser;
-	zval *pind;
+	zval **pind;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentByteIndex(parser->parser));
 }
@@ -1589,14 +1478,14 @@ PHP_FUNCTION(xml_get_current_byte_index)
    Free an XML parser */
 PHP_FUNCTION(xml_parser_free)
 {
-	zval *pind;
+	zval **pind;
 	xml_parser *parser;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
 	if (parser->isparsing == 1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Parser cannot be freed while it is parsing.");
@@ -1616,15 +1505,16 @@ PHP_FUNCTION(xml_parser_free)
 PHP_FUNCTION(xml_parser_set_option)
 {
 	xml_parser *parser;
-	zval *pind, **val;
-	long opt;
+	zval **pind, **opt, **val;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rlZ", &pind, &opt, &val) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 3 || zend_get_parameters_ex(3, &pind, &opt, &val) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
-	switch (opt) {
+	convert_to_long_ex(opt);
+
+	switch (Z_LVAL_PP(opt)) {
 		case PHP_XML_OPTION_CASE_FOLDING:
 			convert_to_long_ex(val);
 			parser->case_folding = Z_LVAL_PP(val);
@@ -1662,15 +1552,16 @@ PHP_FUNCTION(xml_parser_set_option)
 PHP_FUNCTION(xml_parser_get_option)
 {
 	xml_parser *parser;
-	zval *pind;
-	long opt;
+	zval **pind, **opt;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &pind, &opt) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &opt) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
 
-	switch (opt) {
+	convert_to_long_ex(opt);
+
+	switch (Z_LVAL_PP(opt)) {
 		case PHP_XML_OPTION_CASE_FOLDING:
 			RETURN_LONG(parser->case_folding);
 			break;
@@ -1691,15 +1582,15 @@ PHP_FUNCTION(xml_parser_get_option)
    Encodes an ISO-8859-1 string to UTF-8 */
 PHP_FUNCTION(utf8_encode)
 {
-	char *arg;
+	zval **arg;
 	XML_Char *encoded;
-	int arg_len, len;
+	int len;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	encoded = xml_utf8_encode(arg, arg_len, &len, "ISO-8859-1");
+	convert_to_string_ex(arg);
+	encoded = xml_utf8_encode(Z_STRVAL_PP(arg), Z_STRLEN_PP(arg), &len, "ISO-8859-1");
 	if (encoded == NULL) {
 		RETURN_FALSE;
 	}
@@ -1711,15 +1602,15 @@ PHP_FUNCTION(utf8_encode)
    Converts a UTF-8 encoded string to ISO-8859-1 */
 PHP_FUNCTION(utf8_decode)
 {
-	char *arg;
+	zval **arg;
 	XML_Char *decoded;
-	int arg_len, len;
+	int len;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
-
-	decoded = xml_utf8_decode(arg, arg_len, &len, "ISO-8859-1");
+	convert_to_string_ex(arg);
+	decoded = xml_utf8_decode(Z_STRVAL_PP(arg), Z_STRLEN_PP(arg), &len, "ISO-8859-1");
 	if (decoded == NULL) {
 		RETURN_FALSE;
 	}
