@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: php_xmlreader.c,v 1.13.2.8 2006/01/01 12:50:16 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -30,7 +30,6 @@
 #ifdef HAVE_DOM
 #include "ext/dom/xml_common.h"
 #endif
-#include <libxml/xmlreader.h>
 #include <libxml/uri.h>
 
 zend_class_entry *xmlreader_class_entry;
@@ -41,23 +40,21 @@ static HashTable xmlreader_prop_handlers;
 
 typedef int (*xmlreader_read_int_t)(xmlTextReaderPtr reader);
 typedef unsigned char *(*xmlreader_read_char_t)(xmlTextReaderPtr reader);
-typedef const unsigned char *(*xmlreader_read_const_char_t)(xmlTextReaderPtr reader);
 typedef int (*xmlreader_write_t)(xmlreader_object *obj, zval *newval TSRMLS_DC);
 
 typedef unsigned char *(*xmlreader_read_one_char_t)(xmlTextReaderPtr reader, const unsigned char *);
 
 typedef struct _xmlreader_prop_handler {
 	xmlreader_read_int_t read_int_func;
-	xmlreader_read_const_char_t read_char_func;
+	xmlreader_read_char_t read_char_func;
 	xmlreader_write_t write_func;
 	int type;
 } xmlreader_prop_handler;
 
 #define XMLREADER_LOAD_STRING 0
 #define XMLREADER_LOAD_FILE 1
-
 /* {{{ xmlreader_register_prop_handler */
-static void xmlreader_register_prop_handler(HashTable *prop_handler, char *name, xmlreader_read_int_t read_int_func, xmlreader_read_const_char_t read_char_func, int rettype TSRMLS_DC)
+static void xmlreader_register_prop_handler(HashTable *prop_handler, char *name, xmlreader_read_int_t read_int_func, xmlreader_read_char_t read_char_func, int rettype TSRMLS_DC)
 {
 	xmlreader_prop_handler hnd;
 	
@@ -71,7 +68,7 @@ static void xmlreader_register_prop_handler(HashTable *prop_handler, char *name,
 /* {{{ xmlreader_property_reader */
 static int xmlreader_property_reader(xmlreader_object *obj, xmlreader_prop_handler *hnd, zval **retval TSRMLS_DC)
 {
-	const xmlChar *retchar = NULL;
+	char *retchar = NULL;
 	int retint = 0;
 
 	if (obj->ptr != NULL) {
@@ -93,7 +90,8 @@ static int xmlreader_property_reader(xmlreader_object *obj, xmlreader_prop_handl
 	switch (hnd->type) {
 		case IS_STRING:
 			if (retchar) {
-				ZVAL_STRING(*retval, (char *) retchar, 1);
+				ZVAL_STRING(*retval, retchar, 1);
+				xmlFree(retchar);
 			} else {
 				ZVAL_EMPTY_STRING(*retval);
 			}
@@ -112,42 +110,8 @@ static int xmlreader_property_reader(xmlreader_object *obj, xmlreader_prop_handl
 }
 /* }}} */
 
-/* {{{ xmlreader_get_property_ptr_ptr */
-zval **xmlreader_get_property_ptr_ptr(zval *object, zval *member, const zend_literal *key TSRMLS_DC)
-{
-	xmlreader_object *obj;
-	zval tmp_member;
-	zval **retval = NULL;
-	xmlreader_prop_handler *hnd;
-	zend_object_handlers *std_hnd;
-	int ret = FAILURE;
-
- 	if (member->type != IS_STRING) {
-		tmp_member = *member;
-		zval_copy_ctor(&tmp_member);
-		convert_to_string(&tmp_member);
-		member = &tmp_member;
-	}
-
-	obj = (xmlreader_object *)zend_objects_get_address(object TSRMLS_CC);
-
-	if (obj->prop_handler != NULL) {
-		ret = zend_hash_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, (void **) &hnd);
-	}
-	if (ret == FAILURE) {
-		std_hnd = zend_get_std_object_handlers();
-		retval = std_hnd->get_property_ptr_ptr(object, member, key TSRMLS_CC);
-	}
-
-	if (member == &tmp_member) {
-		zval_dtor(member);
-	}
-	return retval;
-}
-/* }}} */
-
 /* {{{ xmlreader_read_property */
-zval *xmlreader_read_property(zval *object, zval *member, int type, const zend_literal *key TSRMLS_DC)
+zval *xmlreader_read_property(zval *object, zval *member, int type TSRMLS_DC)
 {
 	xmlreader_object *obj;
 	zval tmp_member;
@@ -173,13 +137,13 @@ zval *xmlreader_read_property(zval *object, zval *member, int type, const zend_l
 		ret = xmlreader_property_reader(obj, hnd, &retval TSRMLS_CC);
 		if (ret == SUCCESS) {
 			/* ensure we're creating a temporary variable */
-			Z_SET_REFCOUNT_P(retval, 0);
+			retval->refcount = 0;
 		} else {
 			retval = EG(uninitialized_zval_ptr);
 		}
 	} else {
 		std_hnd = zend_get_std_object_handlers();
-		retval = std_hnd->read_property(object, member, type, key TSRMLS_CC);
+		retval = std_hnd->read_property(object, member, type TSRMLS_CC);
 	}
 
 	if (member == &tmp_member) {
@@ -190,7 +154,7 @@ zval *xmlreader_read_property(zval *object, zval *member, int type, const zend_l
 /* }}} */
 
 /* {{{ xmlreader_write_property */
-void xmlreader_write_property(zval *object, zval *member, zval *value, const zend_literal *key TSRMLS_DC)
+void xmlreader_write_property(zval *object, zval *member, zval *value TSRMLS_DC)
 {
 	xmlreader_object *obj;
 	zval tmp_member;
@@ -215,7 +179,7 @@ void xmlreader_write_property(zval *object, zval *member, zval *value, const zen
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot write to read-only property");
 	} else {
 		std_hnd = zend_get_std_object_handlers();
-		std_hnd->write_property(object, member, value, key TSRMLS_CC);
+		std_hnd->write_property(object, member, value TSRMLS_CC);
 	}
 
 	if (member == &tmp_member) {
@@ -234,8 +198,8 @@ char *_xmlreader_get_valid_file_path(char *source, char *resolved_path, int reso
 	int isFileUri = 0;
 
 	uri = xmlCreateURI();
-	escsource = xmlURIEscapeStr((xmlChar *)source, (xmlChar *)":");
-	xmlParseURIReference(uri, (const char *)escsource);
+	escsource = xmlURIEscapeStr(source, ":");
+	xmlParseURIReference(uri, escsource);
 	xmlFree(escsource);
 
 	if (uri->scheme != NULL) {
@@ -260,9 +224,9 @@ char *_xmlreader_get_valid_file_path(char *source, char *resolved_path, int reso
 	file_dest = source;
 
 	if ((uri->scheme == NULL || isFileUri)) {
-		if (!VCWD_REALPATH(source, resolved_path) && !expand_filepath(source, resolved_path TSRMLS_CC)) {
-			xmlFreeURI(uri);
-			return NULL;
+		/* XXX possible buffer overflow if VCWD_REALPATH does not know size of resolved_path */
+		if (! VCWD_REALPATH(source, resolved_path)) {
+			expand_filepath(source, resolved_path TSRMLS_CC);
 		}
 		file_dest = resolved_path;
 	}
@@ -319,9 +283,9 @@ static xmlRelaxNGPtr _xmlreader_get_relaxNG(char *source, int source_len, int ty
 /* }}} */
 #endif
 
-static const zend_module_dep xmlreader_deps[] = {
+static zend_module_dep xmlreader_deps[] = {
 	ZEND_MOD_REQUIRED("libxml")
-	ZEND_MOD_END
+	{NULL, NULL, NULL}
 };
 
 /* {{{ xmlreader_module_entry
@@ -379,7 +343,8 @@ void xmlreader_objects_free_storage(void *object TSRMLS_DC)
 {
 	xmlreader_object *intern = (xmlreader_object *)object;
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_hash_destroy(intern->std.properties);
+	FREE_HASHTABLE(intern->std.properties);
 	
 	xmlreader_free_resources(intern);
 
@@ -392,16 +357,19 @@ zend_object_value xmlreader_objects_new(zend_class_entry *class_type TSRMLS_DC)
 {
 	zend_object_value retval;
 	xmlreader_object *intern;
+	zval *tmp;
 
 	intern = emalloc(sizeof(xmlreader_object));
-	memset(&intern->std, 0, sizeof(zend_object));
+	intern->std.ce = class_type;
+	intern->std.guards = NULL;
 	intern->ptr = NULL;
 	intern->input = NULL;
 	intern->schema = NULL;
 	intern->prop_handler = &xmlreader_prop_handlers;
 
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
-	object_properties_init(&intern->std, class_type);
+	ALLOC_HASHTABLE(intern->std.properties);
+	zend_hash_init(intern->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
+	zend_hash_copy(intern->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 	retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t) xmlreader_objects_free_storage, xmlreader_objects_clone TSRMLS_CC);
 	intern->handle = retval.handle;
 	retval.handlers = &xmlreader_object_handlers;
@@ -430,14 +398,14 @@ static void php_xmlreader_string_arg(INTERNAL_FUNCTION_PARAMETERS, xmlreader_rea
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retchar = (char *)internal_function(intern->ptr, (const unsigned char *)name);
+		retchar = internal_function(intern->ptr, name);
 	}
 	if (retchar) {
 		RETVAL_STRING(retchar, 1);
 		xmlFree(retchar);
 		return;
 	} else {
-		RETVAL_NULL();
+		RETVAL_EMPTY_STRING();
 	}
 }
 /* }}} */
@@ -462,8 +430,7 @@ static void php_xmlreader_no_arg(INTERNAL_FUNCTION_PARAMETERS, xmlreader_read_in
 }
 /* }}} */
 
-#if LIBXML_VERSION >= 20620
-/* {{{ php_xmlreader_no_arg_string */
+/* This function not yet needed until some additional functions are implemented in libxml
 static void php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAMETERS, xmlreader_read_char_t internal_function) {
 	zval *id;
 	char *retchar = NULL;
@@ -473,7 +440,7 @@ static void php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAMETERS, xmlreader_
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retchar = (char *)internal_function(intern->ptr);
+		retchar = xmlTextReaderReadString(intern->ptr);
 	}
 	if (retchar) {
 		RETVAL_STRING(retchar, 1);
@@ -483,8 +450,7 @@ static void php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAMETERS, xmlreader_
 		RETVAL_EMPTY_STRING();
 	}
 }
-/* }}} */
-#endif
+*/
 
 /* {{{ php_xmlreader_set_relaxng_schema */
 static void php_xmlreader_set_relaxng_schema(INTERNAL_FUNCTION_PARAMETERS, int type) {
@@ -495,7 +461,7 @@ static void php_xmlreader_set_relaxng_schema(INTERNAL_FUNCTION_PARAMETERS, int t
 	xmlRelaxNGPtr schema = NULL;
 	char *source;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p!", &source, &source_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s!", &source, &source_len) == FAILURE) {
 		return;
 	}
 
@@ -571,7 +537,7 @@ Get value of an attribute at index from current element */
 PHP_METHOD(xmlreader, getAttributeNo)
 {
 	zval *id;
-	long attr_pos;
+	int attr_pos;
 	char *retchar = NULL;
 	xmlreader_object *intern;
 
@@ -583,7 +549,7 @@ PHP_METHOD(xmlreader, getAttributeNo)
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retchar = (char *)xmlTextReaderGetAttributeNo(intern->ptr, attr_pos);
+		retchar = xmlTextReaderGetAttributeNo(intern->ptr,attr_pos);
 	}
 	if (retchar) {
 		RETVAL_STRING(retchar, 1);
@@ -617,7 +583,7 @@ PHP_METHOD(xmlreader, getAttributeNs)
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retchar = (char *)xmlTextReaderGetAttributeNs(intern->ptr, (xmlChar *)name, (xmlChar *)ns_uri);
+		retchar = xmlTextReaderGetAttributeNs(intern->ptr, name, ns_uri);
 	}
 	if (retchar) {
 		RETVAL_STRING(retchar, 1);
@@ -634,8 +600,7 @@ Indicates whether given property (one of the parser option constants) is set or
 PHP_METHOD(xmlreader, getParserProperty)
 {
 	zval *id;
-	long property;
-	int retval = -1;
+	int property, retval = -1;
 	xmlreader_object *intern;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &property) == FAILURE) {
@@ -697,7 +662,7 @@ PHP_METHOD(xmlreader, moveToAttribute)
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retval = xmlTextReaderMoveToAttribute(intern->ptr, (xmlChar *)name);
+		retval = xmlTextReaderMoveToAttribute(intern->ptr, name);
 		if (retval == 1) {
 			RETURN_TRUE;
 		}
@@ -708,13 +673,12 @@ PHP_METHOD(xmlreader, moveToAttribute)
 /* }}} */
 
 /* {{{ proto boolean XMLReader::moveToAttributeNo(int index)
-Positions reader at attribute at specified index.
+Positions reader at attribute at spcecified index.
 Returns TRUE on success and FALSE on failure */
 PHP_METHOD(xmlreader, moveToAttributeNo)
 {
 	zval *id;
-	long attr_pos;
-	int retval;
+	int attr_pos, retval;
 	xmlreader_object *intern;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &attr_pos) == FAILURE) {
@@ -758,7 +722,7 @@ PHP_METHOD(xmlreader, moveToAttributeNs)
 
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 	if (intern && intern->ptr) {
-		retval = xmlTextReaderMoveToAttributeNs(intern->ptr, (xmlChar *)name, (xmlChar *)ns_uri);
+		retval = xmlTextReaderMoveToAttributeNs(intern->ptr, name, ns_uri);
 		if (retval == 1) {
 			RETURN_TRUE;
 		}
@@ -805,6 +769,7 @@ PHP_METHOD(xmlreader, read)
 	if (intern != NULL && intern->ptr != NULL) {
 		retval = xmlTextReaderRead(intern->ptr);
 		if (retval == -1) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "An Error Occured while reading");
 			RETURN_FALSE;
 		} else {
 			RETURN_BOOL(retval);
@@ -840,12 +805,13 @@ PHP_METHOD(xmlreader, next)
 #endif
 		retval = xmlTextReaderNext(intern->ptr);
 		while (name != NULL && retval == 1) {
-			if (xmlStrEqual(xmlTextReaderConstLocalName(intern->ptr), (xmlChar *)name)) {
+			if (xmlStrEqual(xmlTextReaderConstLocalName(intern->ptr), name)) {
 				RETURN_TRUE;
 			}
 			retval = xmlTextReaderNext(intern->ptr); 
 		}
 		if (retval == -1) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "An Error Occured while reading");
 			RETURN_FALSE;
 		} else {
 			RETURN_BOOL(retval);
@@ -857,20 +823,18 @@ PHP_METHOD(xmlreader, next)
 }
 /* }}} */
 
-/* {{{ proto boolean XMLReader::open(string URI [, string encoding [, int options]])
-Sets the URI that the XMLReader will parse. */
+/* {{{ proto boolean XMLReader::open(string URI)
+Sets the URI that the the XMLReader will parse. */
 PHP_METHOD(xmlreader, open)
 {
 	zval *id;
-	int source_len = 0, encoding_len = 0;
-	long options = 0;
-	xmlreader_object *intern = NULL;
+	int source_len = 0;
+	xmlreader_object *intern;
 	char *source, *valid_file = NULL;
-	char *encoding = NULL;
 	char resolved_path[MAXPATHLEN + 1];
 	xmlTextReaderPtr reader = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|s!l", &source, &source_len, &encoding, &encoding_len, &options) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &source, &source_len) == FAILURE) {
 		return;
 	}
 
@@ -892,7 +856,7 @@ PHP_METHOD(xmlreader, open)
 	valid_file = _xmlreader_get_valid_file_path(source, resolved_path, MAXPATHLEN  TSRMLS_CC);
 
 	if (valid_file) {
-		reader = xmlReaderForFile(valid_file, encoding, options);
+		reader = xmlNewTextReaderFilename(valid_file);
 	}
 
 	if (reader == NULL) {
@@ -915,87 +879,34 @@ PHP_METHOD(xmlreader, open)
 /* }}} */
 
 /* Not Yet Implemented in libxml - functions exist just not coded
+PHP_METHOD(xmlreader, readInnerXml)
+{
+	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadInnerXml);
+}
+
+PHP_METHOD(xmlreader, readOuterXml)
+{
+	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadInnerXml);
+}
+
+PHP_METHOD(xmlreader, readString)
+{
+	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadString);
+}
+
 PHP_METHOD(xmlreader, resetState)
 {
 
 }
 */
 
-#if LIBXML_VERSION >= 20620
-/* {{{ proto string XMLReader::readInnerXml()
-Reads the contents of the current node, including child nodes and markup. */
-PHP_METHOD(xmlreader, readInnerXml)
-{
-	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadInnerXml);
-}
-/* }}} */
-
-/* {{{ proto boolean XMLReader::readOuterXml()
-Reads the contents of the current node, including child nodes and markup. */
-PHP_METHOD(xmlreader, readOuterXml)
-{
-	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadOuterXml);
-}
-/* }}} */
-
-/* {{{ proto boolean XMLReader::readString()
-Reads the contents of an element or a text node as a string. */
-PHP_METHOD(xmlreader, readString)
-{
-	php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAM_PASSTHRU, xmlTextReaderReadString);
-}
-/* }}} */
-
-/* {{{ proto boolean XMLReader::setSchema(string filename)
-Use W3C XSD schema to validate the document as it is processed. Activation is only possible before the first Read(). */
-PHP_METHOD(xmlreader, setSchema)
-{
-#ifdef LIBXML_SCHEMAS_ENABLED
-	zval *id;
-	int source_len = 0, retval = -1;
-	xmlreader_object *intern;
-	char *source;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p!", &source, &source_len) == FAILURE) {
-		return;
-	}
-
-	if (source != NULL && !source_len) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Schema data source is required");
-		RETURN_FALSE;
-	}
-
-	id = getThis();
-
-	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
-	if (intern && intern->ptr) {
-		retval = xmlTextReaderSchemaValidate(intern->ptr, source);
-
-		if (retval == 0) {
-			RETURN_TRUE;
-		}
-	}
-	
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to set schema. This must be set prior to reading or schema contains errors.");
-
-	RETURN_FALSE;
-#else
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "No Schema support built into libxml.");
-
-	RETURN_FALSE;
-#endif
-}
-/* }}} */
-#endif
-
 /* {{{ proto boolean XMLReader::setParserProperty(int property, boolean value)
 Sets parser property (one of the parser option constants).
 Properties must be set after open() or XML() and before the first read() is called */
 PHP_METHOD(xmlreader, setParserProperty)
 {
 	zval *id;
-	long property;
-	int retval = -1;
+	int property, retval = -1;
 	zend_bool value;
 	xmlreader_object *intern;
 
@@ -1018,8 +929,8 @@ PHP_METHOD(xmlreader, setParserProperty)
 }
 /* }}} */
 
-/* {{{ proto boolean XMLReader::setRelaxNGSchema(string filename)
-Sets the string that the XMLReader will parse. */
+/* {{{ proto boolean XMLReader::setRelaxNGSchemaSource(string filename)
+Sets the string that the the XMLReader will parse. */
 PHP_METHOD(xmlreader, setRelaxNGSchema)
 {
 	php_xmlreader_set_relaxng_schema(INTERNAL_FUNCTION_PARAM_PASSTHRU, XMLREADER_LOAD_FILE);
@@ -1027,34 +938,27 @@ PHP_METHOD(xmlreader, setRelaxNGSchema)
 /* }}} */
 
 /* {{{ proto boolean XMLReader::setRelaxNGSchemaSource(string source)
-Sets the string that the XMLReader will parse. */
+Sets the string that the the XMLReader will parse. */
 PHP_METHOD(xmlreader, setRelaxNGSchemaSource)
 {
 	php_xmlreader_set_relaxng_schema(INTERNAL_FUNCTION_PARAM_PASSTHRU, XMLREADER_LOAD_STRING);
 }
 /* }}} */
 
-/* TODO
-XMLPUBFUN int XMLCALL		
-		    xmlTextReaderSetSchema	(xmlTextReaderPtr reader,
-		    				 xmlSchemaPtr schema);
-*/
-
-/* {{{ proto boolean XMLReader::XML(string source [, string encoding [, int options]])
-Sets the string that the XMLReader will parse. */
+/* {{{ proto boolean XMLReader::XML(string source)
+Sets the string that the the XMLReader will parse. */
 PHP_METHOD(xmlreader, XML)
 {
 	zval *id;
-	int source_len = 0, encoding_len = 0;
-	long options = 0;
-	xmlreader_object *intern = NULL;
-	char *source, *uri = NULL, *encoding = NULL;
-	int resolved_path_len, ret = 0;
+	int source_len = 0;
+	xmlreader_object *intern;
+	char *source, *uri = NULL;
+	int resolved_path_len;
 	char *directory=NULL, resolved_path[MAXPATHLEN];
 	xmlParserInputBufferPtr inputbfr;
 	xmlTextReaderPtr reader;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s!l", &source, &source_len, &encoding, &encoding_len, &options) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &source, &source_len) == FAILURE) {
 		return;
 	}
 
@@ -1090,34 +994,25 @@ PHP_METHOD(xmlreader, XML)
 			uri = (char *) xmlCanonicPath((const xmlChar *) resolved_path);
 		}
 		reader = xmlNewTextReader(inputbfr, uri);
-
+		if (uri) {
+			xmlFree(uri);
+		}
 		if (reader != NULL) {
-#if LIBXML_VERSION >= 20628
-			ret = xmlTextReaderSetup(reader, NULL, uri, encoding, options);
-#endif
-			if (ret == 0) {
-				if (id == NULL) {
-					object_init_ex(return_value, xmlreader_class_entry);
-					intern = (xmlreader_object *)zend_objects_get_address(return_value TSRMLS_CC);
-				} else {
-					RETVAL_TRUE;
-				}
+			if (id == NULL) {
+				object_init_ex(return_value, xmlreader_class_entry);
+				intern = (xmlreader_object *)zend_objects_get_address(return_value TSRMLS_CC);
 				intern->input = inputbfr;
 				intern->ptr = reader;
-
-				if (uri) {
-					xmlFree(uri);
-				}
-
 				return;
+			} else {
+				intern->input = inputbfr;
+				intern->ptr = reader;
+				RETURN_TRUE;
+
 			}
 		}
 	}
 
-	if (uri) {
-		xmlFree(uri);
-	}
-
 	if (inputbfr) {
 		xmlFreeParserInputBuffer(inputbfr);
 	}
@@ -1131,38 +1026,23 @@ Moves the position of the current instance to the next node in the stream. */
 PHP_METHOD(xmlreader, expand)
 {
 #ifdef HAVE_DOM
-	zval *id, *basenode = NULL;
+	zval *id, *rv = NULL;
 	int ret;
 	xmlreader_object *intern;
 	xmlNode *node, *nodec;
-	xmlDocPtr docp = NULL;
-	php_libxml_node_object *domobj = NULL;
-
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|O!", &id, xmlreader_class_entry, &basenode, dom_node_class_entry) == FAILURE) {
-		return;
-	}
-	
-	if (basenode != NULL) {
-		NODE_GET_OBJ(node, basenode, xmlNodePtr, domobj);
-		docp = node->doc;
-	}
 
+	id = getThis();
 	intern = (xmlreader_object *)zend_object_store_get_object(id TSRMLS_CC);
 
 	if (intern && intern->ptr) {
 		node = xmlTextReaderExpand(intern->ptr);
 		
 		if (node == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "An Error Occurred while expanding ");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "An Error Occured while expanding ");
 			RETURN_FALSE;
 		} else {
-			nodec = xmlDocCopyNode(node, docp, 1);
-			if (nodec == NULL) {
-				php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Cannot expand this node type");
-				RETURN_FALSE;
-			} else {
-				DOM_RET_OBJ(nodec, &ret, (dom_object *)domobj);
-			}
+			nodec = xmlCopyNode(node, 1);
+			DOM_RET_OBJ(rv, nodec, &ret, NULL);
 		}
 	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Load Data before trying to expand");
@@ -1174,137 +1054,36 @@ PHP_METHOD(xmlreader, expand)
 #endif
 }
 /* }}} */
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_close, 0)
-ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_getAttribute, 0)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_getAttributeNo, 0)
-	ZEND_ARG_INFO(0, index)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_getAttributeNs, 0)
-	ZEND_ARG_INFO(0, name)
-	ZEND_ARG_INFO(0, namespaceURI)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_getParserProperty, 0)
-	ZEND_ARG_INFO(0, property)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_isValid, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_lookupNamespace, 0)
-ZEND_ARG_INFO(0, prefix)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToAttribute, 0)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToAttributeNo, 0)
-	ZEND_ARG_INFO(0, index)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToAttributeNs, 0)
-	ZEND_ARG_INFO(0, name)
-	ZEND_ARG_INFO(0, namespaceURI)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToElement, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToFirstAttribute, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_moveToNextAttribute, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_read, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlreader_next, 0, 0, 0)
-	ZEND_ARG_INFO(0, localname)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlreader_open, 0, 0, 1)
-	ZEND_ARG_INFO(0, URI)
-	ZEND_ARG_INFO(0, encoding)
-	ZEND_ARG_INFO(0, options)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_readInnerXml, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_readOuterXml, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_readString, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_setSchema, 0)
-	ZEND_ARG_INFO(0, filename)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_setParserProperty, 0)
-	ZEND_ARG_INFO(0, property)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_setRelaxNGSchema, 0)
-	ZEND_ARG_INFO(0, filename)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_setRelaxNGSchemaSource, 0)
-	ZEND_ARG_INFO(0, source)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlreader_XML, 0, 0, 1)
-	ZEND_ARG_INFO(0, source)
-	ZEND_ARG_INFO(0, encoding)
-	ZEND_ARG_INFO(0, options)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_xmlreader_expand, 0)
-ZEND_END_ARG_INFO()
-/* }}} */
-
-static const zend_function_entry xmlreader_functions[] = {
-	PHP_ME(xmlreader, close, arginfo_xmlreader_close, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, getAttribute, arginfo_xmlreader_getAttribute, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, getAttributeNo, arginfo_xmlreader_getAttributeNo, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, getAttributeNs, arginfo_xmlreader_getAttributeNs, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, getParserProperty, arginfo_xmlreader_getParserProperty, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, isValid, arginfo_xmlreader_isValid, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, lookupNamespace, arginfo_xmlreader_lookupNamespace, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToAttributeNo, arginfo_xmlreader_moveToAttributeNo, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToAttribute, arginfo_xmlreader_moveToAttribute, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToAttributeNs, arginfo_xmlreader_moveToAttributeNs, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToElement, arginfo_xmlreader_moveToElement, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToFirstAttribute, arginfo_xmlreader_moveToFirstAttribute, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, moveToNextAttribute, arginfo_xmlreader_moveToNextAttribute, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, open, arginfo_xmlreader_open, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(xmlreader, read, arginfo_xmlreader_read, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, next, arginfo_xmlreader_next, ZEND_ACC_PUBLIC)
-#if LIBXML_VERSION >= 20620
-	PHP_ME(xmlreader, readInnerXml, arginfo_xmlreader_readInnerXml, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, readOuterXml, arginfo_xmlreader_readOuterXml, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, readString, arginfo_xmlreader_readString, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, setSchema, arginfo_xmlreader_setSchema, ZEND_ACC_PUBLIC)
-#endif
+static zend_function_entry xmlreader_functions[] = {
+	PHP_ME(xmlreader, close, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, getAttribute, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, getAttributeNo, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, getAttributeNs, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, getParserProperty, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, isValid, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, lookupNamespace, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToAttributeNo, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToAttribute, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToAttributeNs, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToElement, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToFirstAttribute, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, moveToNextAttribute, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, open, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(xmlreader, read, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, next, NULL, ZEND_ACC_PUBLIC)
 /* Not Yet Implemented though defined in libxml as of 2.6.9dev
+	PHP_ME(xmlreader, readInnerXml, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, readOuterXml, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, readString, NULL, ZEND_ACC_PUBLIC)
 	PHP_ME(xmlreader, resetState, NULL, ZEND_ACC_PUBLIC)
 */
-	PHP_ME(xmlreader, setParserProperty, arginfo_xmlreader_setParserProperty, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, setRelaxNGSchema, arginfo_xmlreader_setRelaxNGSchema, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, setRelaxNGSchemaSource, arginfo_xmlreader_setRelaxNGSchemaSource, ZEND_ACC_PUBLIC)
-	PHP_ME(xmlreader, XML, arginfo_xmlreader_XML, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(xmlreader, expand, arginfo_xmlreader_expand, ZEND_ACC_PUBLIC)
-	PHP_FE_END
+	PHP_ME(xmlreader, setParserProperty, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, setRelaxNGSchema, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, setRelaxNGSchemaSource, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(xmlreader, XML, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(xmlreader, expand, NULL, ZEND_ACC_PUBLIC)
+	{NULL, NULL, NULL}
 };
 
 /* {{{ PHP_MINIT_FUNCTION
@@ -1317,8 +1096,7 @@ PHP_MINIT_FUNCTION(xmlreader)
 	memcpy(&xmlreader_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
 	xmlreader_object_handlers.read_property = xmlreader_read_property;
 	xmlreader_object_handlers.write_property = xmlreader_write_property;
-	xmlreader_object_handlers.get_property_ptr_ptr = xmlreader_get_property_ptr_ptr;
-	xmlreader_object_handlers.clone_obj = NULL;
+	xmlreader_object_handlers.get_property_ptr_ptr = NULL;
 
 	INIT_CLASS_ENTRY(ce, "XMLReader", xmlreader_functions);
 	ce.create_object = xmlreader_objects_new;
@@ -1326,19 +1104,19 @@ PHP_MINIT_FUNCTION(xmlreader)
 
 	zend_hash_init(&xmlreader_prop_handlers, 0, NULL, NULL, 1);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "attributeCount", xmlTextReaderAttributeCount, NULL, IS_LONG TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "baseURI", NULL, xmlTextReaderConstBaseUri, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "baseURI", NULL, xmlTextReaderBaseUri, IS_STRING TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "depth", xmlTextReaderDepth, NULL, IS_LONG TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasAttributes", xmlTextReaderHasAttributes, NULL, IS_BOOL TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasValue", xmlTextReaderHasValue, NULL, IS_BOOL TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isDefault", xmlTextReaderIsDefault, NULL, IS_BOOL TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isEmptyElement", xmlTextReaderIsEmptyElement, NULL, IS_BOOL TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "localName", NULL, xmlTextReaderConstLocalName, IS_STRING TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "name", NULL, xmlTextReaderConstName, IS_STRING TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "namespaceURI", NULL, xmlTextReaderConstNamespaceUri, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "localName", NULL, xmlTextReaderLocalName, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "name", NULL, xmlTextReaderName, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "namespaceURI", NULL, xmlTextReaderNamespaceUri, IS_STRING TSRMLS_CC);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "nodeType", xmlTextReaderNodeType, NULL, IS_LONG TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "prefix", NULL, xmlTextReaderConstPrefix, IS_STRING TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "value", NULL, xmlTextReaderConstValue, IS_STRING TSRMLS_CC);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "xmlLang", NULL, xmlTextReaderConstXmlLang, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "prefix", NULL, xmlTextReaderPrefix, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "value", NULL, xmlTextReaderValue, IS_STRING TSRMLS_CC);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "xmlLang", NULL, xmlTextReaderXmlLang, IS_STRING TSRMLS_CC);
 
 	/* Constants for NodeType - cannot define common types to share with dom as there are differences in these types */
 
@@ -1378,6 +1156,7 @@ PHP_MINIT_FUNCTION(xmlreader)
 }
 /* }}} */
 
+
 /* {{{ PHP_MSHUTDOWN_FUNCTION
  */
 PHP_MSHUTDOWN_FUNCTION(xmlreader)
@@ -1396,6 +1175,10 @@ PHP_MINFO_FUNCTION(xmlreader)
 		php_info_print_table_row(2, "XMLReader", "enabled");
 	}
 	php_info_print_table_end();
+
+	/* Remove comments if you have entries in php.ini
+	DISPLAY_INI_ENTRIES();
+	*/
 }
 /* }}} */
 
