@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: xsltprocessor.c,v 1.39.2.2.2.16 2008/12/31 11:17:47 sebastian Exp $ */
+/* $Id: xsltprocessor.c,v 1.39.2.2 2006/01/01 12:50:17 sniper Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,57 +27,6 @@
 #include "php_xsl.h"
 #include "ext/libxml/php_libxml.h"
 
-/* {{{ arginfo */
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_import_stylesheet, 0, 0, 1)
-	ZEND_ARG_INFO(0, doc)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_transform_to_doc, 0, 0, 1)
-	ZEND_ARG_INFO(0, doc)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_transform_to_uri, 0, 0, 2)
-	ZEND_ARG_INFO(0, doc)
-	ZEND_ARG_INFO(0, uri)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_transform_to_xml, 0, 0, 1)
-	ZEND_ARG_INFO(0, doc)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_set_parameter, 0, 0, 2)
-	ZEND_ARG_INFO(0, namespace)
-	ZEND_ARG_INFO(0, name)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_get_parameter, 0, 0, 2)
-	ZEND_ARG_INFO(0, namespace)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_remove_parameter, 0, 0, 2)
-	ZEND_ARG_INFO(0, namespace)
-	ZEND_ARG_INFO(0, name)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_has_exslt_support, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-static
-ZEND_BEGIN_ARG_INFO_EX(arginfo_xsl_xsltprocessor_register_php_functions, 0, 0, 0)
-	ZEND_ARG_INFO(0, restrict)
-ZEND_END_ARG_INFO();
-/* }}} */
-
 /*
 * class xsl_xsltprocessor 
 *
@@ -86,18 +35,19 @@ ZEND_END_ARG_INFO();
 */
 
 zend_function_entry php_xsl_xsltprocessor_class_functions[] = {
-	PHP_FALIAS(importStylesheet, xsl_xsltprocessor_import_stylesheet, arginfo_xsl_xsltprocessor_import_stylesheet)
-	PHP_FALIAS(transformToDoc, xsl_xsltprocessor_transform_to_doc, arginfo_xsl_xsltprocessor_transform_to_doc)
-	PHP_FALIAS(transformToUri, xsl_xsltprocessor_transform_to_uri, arginfo_xsl_xsltprocessor_transform_to_uri)
-	PHP_FALIAS(transformToXml, xsl_xsltprocessor_transform_to_xml, arginfo_xsl_xsltprocessor_transform_to_xml)
-	PHP_FALIAS(setParameter, xsl_xsltprocessor_set_parameter, arginfo_xsl_xsltprocessor_set_parameter)
-	PHP_FALIAS(getParameter, xsl_xsltprocessor_get_parameter, arginfo_xsl_xsltprocessor_get_parameter)
-	PHP_FALIAS(removeParameter, xsl_xsltprocessor_remove_parameter, arginfo_xsl_xsltprocessor_remove_parameter)
-	PHP_FALIAS(hasExsltSupport, xsl_xsltprocessor_has_exslt_support, arginfo_xsl_xsltprocessor_has_exslt_support)
-	PHP_FALIAS(registerPHPFunctions, xsl_xsltprocessor_register_php_functions, arginfo_xsl_xsltprocessor_register_php_functions)
+	PHP_FALIAS(importStylesheet, xsl_xsltprocessor_import_stylesheet, NULL)
+	PHP_FALIAS(transformToDoc, xsl_xsltprocessor_transform_to_doc, NULL)
+	PHP_FALIAS(transformToUri, xsl_xsltprocessor_transform_to_uri, NULL)
+	PHP_FALIAS(transformToXml, xsl_xsltprocessor_transform_to_xml, NULL)
+	PHP_FALIAS(setParameter, xsl_xsltprocessor_set_parameter, NULL)
+	PHP_FALIAS(getParameter, xsl_xsltprocessor_get_parameter, NULL)
+	PHP_FALIAS(removeParameter, xsl_xsltprocessor_remove_parameter, NULL)
+	PHP_FALIAS(hasExsltSupport, xsl_xsltprocessor_has_exslt_support, NULL)
+	PHP_FALIAS(registerPHPFunctions, xsl_xsltprocessor_register_php_functions, NULL)
 	{NULL, NULL, NULL}
 };
 
+/* {{{ attribute protos, not implemented yet */
 /* {{{ php_xsl_xslt_string_to_xpathexpr()
    Translates a string to a XPath Expression */
 static char *php_xsl_xslt_string_to_xpathexpr(const char *str TSRMLS_DC)
@@ -122,7 +72,7 @@ static char *php_xsl_xslt_string_to_xpathexpr(const char *str TSRMLS_DC)
 	}
 	return (char *) value;
 }
-/* }}} */
+
 
 /* {{{ php_xsl_xslt_make_params()
    Translates a PHP array to a libxslt parameters array */
@@ -137,7 +87,7 @@ static char **php_xsl_xslt_make_params(HashTable *parht, int xpath_params TSRMLS
 	int i = 0;
 
 	parsize = (2 * zend_hash_num_elements(parht) + 1) * sizeof(char *);
-	params = (char **)safe_emalloc((2 * zend_hash_num_elements(parht) + 1), sizeof(char *), 0);
+	params = (char **)emalloc(parsize);
 	memset((char *)params, 0, parsize);
 
 	for (zend_hash_internal_pointer_reset(parht);
@@ -172,7 +122,8 @@ static char **php_xsl_xslt_make_params(HashTable *parht, int xpath_params TSRMLS
 }
 /* }}} */
 
-static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int type) /* {{{ */
+
+static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int type)
 {
 	xsltTransformContextPtr tctxt;
 	zval **args;
@@ -280,9 +231,8 @@ static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int t
 				}
 				break;
 			default:
-				str = xmlXPathCastToString(obj);
-				ZVAL_STRING(args[i], str, 1);
-				xmlFree(str);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php:function object type %d is not supported yet", obj->type);
+			ZVAL_STRING(args[i], "", 0);
 		}
 		xmlXPathFreeObject(obj);
 		fci.params[i] = &args[i];
@@ -319,7 +269,7 @@ static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int t
 		
 	} else if ( intern->registerPhpFunctions == 2 && zend_hash_exists(intern->registered_phpfunctions, callable, strlen(callable) + 1) == 0) { 
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Not allowed to call handler '%s()'.", callable);
-		/* Push an empty string, so that we at least have an xslt result... */
+		// Push an empty string, so that we at least have an xslt result...
 		valuePush(ctxt, xmlXPathNewString(""));
 	} else {
 		result = zend_call_function(&fci, NULL TSRMLS_CC);
@@ -364,19 +314,17 @@ static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int t
 		efree(fci.params);
 	}
 }
-/* }}} */
 
-void xsl_ext_function_string_php(xmlXPathParserContextPtr ctxt, int nargs) /* {{{ */
+void xsl_ext_function_string_php(xmlXPathParserContextPtr ctxt, int nargs)
 {
 	xsl_ext_function_php(ctxt, nargs, 1);
 }
-/* }}} */
 
-void xsl_ext_function_object_php(xmlXPathParserContextPtr ctxt, int nargs) /* {{{ */
+void xsl_ext_function_object_php(xmlXPathParserContextPtr ctxt, int nargs)
 {
 	xsl_ext_function_php(ctxt, nargs, 2);
 }
-/* }}} */
+
 
 /* {{{ proto void xsl_xsltprocessor_import_stylesheet(domdocument doc);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#
@@ -404,7 +352,7 @@ PHP_FUNCTION(xsl_xsltprocessor_import_stylesheet)
 	}
 	if (doc == NULL) {
 		php_error(E_WARNING, "Invalid Document");
-		RETURN_FALSE;
+		RETURN_NULL();
 	}
 
 	/* libxslt uses _private, so we must copy the imported 
@@ -437,20 +385,18 @@ PHP_FUNCTION(xsl_xsltprocessor_import_stylesheet)
 	efree(member);
 	if (clone_docu == 0) {
 		/* check if the stylesheet is using xsl:key, if yes, we have to clone the document _always_ before a transformation */
-		nodep = xmlDocGetRootElement(sheetp->doc);
-		if (nodep && (nodep = nodep->children)) {
-			while (nodep) {
-				if (nodep->type == XML_ELEMENT_NODE && xmlStrEqual(nodep->name, "key") && xmlStrEqual(nodep->ns->href, XSLT_NAMESPACE)) {
-					intern->hasKeys = 1;
-					break;
-				}
-				nodep = nodep->next;
+		nodep = xmlDocGetRootElement(sheetp->doc)->children;
+		while (nodep) {
+			if (nodep->type == XML_ELEMENT_NODE && xmlStrEqual(nodep->name, "key") && xmlStrEqual(nodep->ns->href, XSLT_NAMESPACE)) {
+				intern->hasKeys = 1;
+				break;
 			}
+			nodep = nodep->next;
 		}
 	} else {
 		intern->hasKeys = clone_docu;
 	}
-
+	
 	if ((oldsheetp = (xsltStylesheetPtr)intern->ptr)) { 
 		/* free wrapper */
 		if (((xsltStylesheetPtr) intern->ptr)->_private != NULL) {
@@ -461,11 +407,11 @@ PHP_FUNCTION(xsl_xsltprocessor_import_stylesheet)
 	}
 
 	php_xsl_set_object(id, sheetp TSRMLS_CC);
-	RETVAL_TRUE;
 }
 /* }}} end xsl_xsltprocessor_import_stylesheet */
 
-static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStylesheetPtr style, zval *docp TSRMLS_DC) /* {{{ */
+
+static xmlDocPtr php_xsl_apply_stylesheet(xsl_object *intern, xsltStylesheetPtr style, zval *docp TSRMLS_DC)
 {
 	xmlDocPtr newdocp;
 	xmlDocPtr doc = NULL;
@@ -474,8 +420,6 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	php_libxml_node_object *object;
 	char **params = NULL;
 	int clone;
-	zval *doXInclude, *member;
-	zend_object_handlers *std_hnd;
 
 	node = php_libxml_import_node(docp TSRMLS_CC);
 	
@@ -509,18 +453,7 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 
 	ctxt = xsltNewTransformContext(style, doc);
 	ctxt->_private = (void *) intern;
-
-	std_hnd = zend_get_std_object_handlers();
-
-	MAKE_STD_ZVAL(member);
-	ZVAL_STRING(member, "doXInclude", 0);
-	doXInclude = std_hnd->read_property(id, member, BP_VAR_IS TSRMLS_CC);
-	if (Z_TYPE_P(doXInclude) != IS_NULL) {
-		convert_to_long(doXInclude);
-		ctxt->xinclude = Z_LVAL_P(doXInclude);
-	}
-	efree(member);
-
+	
 	newdocp = xsltApplyStylesheetUser(style, doc, (const char**) params, NULL, NULL, ctxt);
 
 	xsltFreeTransformContext(ctxt);
@@ -547,7 +480,6 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	return newdocp;
 
 }
-/* }}} */
 
 /* {{{ proto domdocument xsl_xsltprocessor_transform_to_doc(domnode doc);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#
@@ -558,49 +490,21 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_doc)
 	zval *id, *rv = NULL, *docp = NULL;
 	xmlDoc *newdocp;
 	xsltStylesheetPtr sheetp;
-	int ret, ret_class_len=0;
-	char *ret_class = NULL;
+	int ret;
 	xsl_object *intern;
-
+	
 	id = getThis();
 	intern = (xsl_object *)zend_object_store_get_object(id TSRMLS_CC);
 	sheetp = (xsltStylesheetPtr) intern->ptr;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "o|s!", &docp, &ret_class, &ret_class_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "o", &docp) == FAILURE) {
 		RETURN_FALSE;
 	}
 
-	newdocp = php_xsl_apply_stylesheet(id, intern, sheetp, docp TSRMLS_CC);
+	newdocp = php_xsl_apply_stylesheet(intern, sheetp, docp TSRMLS_CC);
 
 	if (newdocp) {
-		if (ret_class) {
-			int found;
-			char *curclass_name;
-			zend_class_entry *curce, **ce;
-			php_libxml_node_object *interndoc;
-
-			curce = Z_OBJCE_P(docp);
-			curclass_name = curce->name;
-			while (curce->parent != NULL) {
-				curce = curce->parent;
-			}
-
-			found = zend_lookup_class(ret_class, ret_class_len, &ce TSRMLS_CC);
-			if ((found != SUCCESS) || !instanceof_function(*ce, curce TSRMLS_CC)) {
-				xmlFreeDoc(newdocp);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, 
-					"Expecting class compatible with %s, '%s' given", curclass_name, ret_class);
-				RETURN_FALSE;
-			}
-
-			object_init_ex(return_value, *ce);
-		
-			interndoc = (php_libxml_node_object *)zend_objects_get_address(return_value TSRMLS_CC);
-			php_libxml_increment_doc_ref(interndoc, newdocp TSRMLS_CC);
-			php_libxml_increment_node_ptr(interndoc, (xmlNodePtr)newdocp, (void *)interndoc TSRMLS_CC);
-		} else {
-			DOM_RET_OBJ(rv, (xmlNodePtr) newdocp, &ret, NULL);
-		}
+		DOM_RET_OBJ(rv, (xmlNodePtr) newdocp, &ret, NULL);
 	} else {
 		RETURN_FALSE;
 	}
@@ -608,6 +512,7 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_doc)
 }
 /* }}} end xsl_xsltprocessor_transform_to_doc */
 
+
 /* {{{ proto int xsl_xsltprocessor_transform_to_uri(domdocument doc, string uri);
 */
 PHP_FUNCTION(xsl_xsltprocessor_transform_to_uri)
@@ -627,7 +532,7 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_uri)
 		RETURN_FALSE;
 	}
 
-	newdocp = php_xsl_apply_stylesheet(id, intern, sheetp, docp TSRMLS_CC);
+	newdocp = php_xsl_apply_stylesheet(intern, sheetp, docp TSRMLS_CC);
 
 	ret = -1;
 	if (newdocp) {
@@ -639,6 +544,7 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_uri)
 }
 /* }}} end xsl_xsltprocessor_transform_to_uri */
 
+
 /* {{{ proto string xsl_xsltprocessor_transform_to_xml(domdocument doc);
 */
 PHP_FUNCTION(xsl_xsltprocessor_transform_to_xml)
@@ -659,7 +565,7 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_xml)
 		RETURN_FALSE;
 	}
 
-	newdocp = php_xsl_apply_stylesheet(id, intern, sheetp, docp TSRMLS_CC);
+	newdocp = php_xsl_apply_stylesheet(intern, sheetp, docp TSRMLS_CC);
 
 	ret = -1;
 	if (newdocp) {
@@ -677,6 +583,7 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_xml)
 }
 /* }}} end xsl_xsltprocessor_transform_to_xml */
 
+
 /* {{{ proto bool xsl_xsltprocessor_set_parameter(string namespace, mixed name [, string value]);
 */
 PHP_FUNCTION(xsl_xsltprocessor_set_parameter)
@@ -776,7 +683,7 @@ PHP_FUNCTION(xsl_xsltprocessor_remove_parameter)
 }
 /* }}} end xsl_xsltprocessor_remove_parameter */
 
-/* {{{ proto void xsl_xsltprocessor_register_php_functions([mixed $restrict]);
+/* {{{ proto void xsl_xsltprocessor_register_php_functions();
 */
 PHP_FUNCTION(xsl_xsltprocessor_register_php_functions)
 {
@@ -834,11 +741,3 @@ PHP_FUNCTION(xsl_xsltprocessor_has_exslt_support)
 }
 /* }}} end xsl_xsltprocessor_has_exslt_support(); */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
