@@ -2,7 +2,7 @@
 	+----------------------------------------------------------------------+
 	| PHP Version 5                                                        |
 	+----------------------------------------------------------------------+
-	| Copyright (c) 1997-2009 The PHP Group                                |
+	| Copyright (c) 1997-2006 The PHP Group                                |
 	+----------------------------------------------------------------------+
 	| This source file is subject to version 3.01 of the PHP license,      |
 	| that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 	+----------------------------------------------------------------------+
 */
 
-/* $Id: internal_functions_win32.c,v 1.87.2.1.2.3.2.3 2008/12/31 11:15:47 sebastian Exp $ */
+/* $Id: internal_functions_win32.c,v 1.87.2.1 2006/01/01 12:50:17 sniper Exp $ */
 
 /* {{{ includes
  */
@@ -48,7 +48,6 @@
 #include "ext/standard/php_lcg.h"
 #include "ext/standard/php_array.h"
 #include "ext/standard/php_assert.h"
-#include "ext/reflection/php_reflection.h"
 #if HAVE_BCMATH
 #include "ext/bcmath/php_bcmath.h"
 #endif
@@ -58,15 +57,9 @@
 #if HAVE_CTYPE
 #include "ext/ctype/php_ctype.h"
 #endif
-#if HAVE_DATE
-#include "ext/date/php_date.h"
-#endif
 #if HAVE_FTP
 #include "ext/ftp/php_ftp.h"
 #endif
-#if HAVE_ICONV
-#include "ext/iconv/php_iconv.h"
-#endif
 #include "ext/standard/reg.h"
 #if HAVE_PCRE || HAVE_BUNDLED_PCRE
 #include "ext/pcre/php_pcre.h"
@@ -105,20 +98,11 @@
 #include "ext/sqlite/php_sqlite.h"
 #endif
 #include "ext/com_dotnet/php_com_dotnet.h"
-#ifdef HAVE_SPL
-#include "ext/spl/php_spl.h"
-#endif
-#if HAVE_XML && HAVE_XMLREADER
-#include "ext/xmlreader/php_xmlreader.h"
-#endif
-#if HAVE_XML && HAVE_XMLWRITER
-#include "ext/xmlwriter/php_xmlwriter.h"
-#endif
 /* }}} */
 
 /* {{{ php_builtin_extensions[]
  */
-static zend_module_entry *php_builtin_extensions[] = {
+zend_module_entry *php_builtin_extensions[] = {
 	phpext_standard_ptr
 #if HAVE_BCMATH
 	,phpext_bcmath_ptr
@@ -130,18 +114,9 @@ static zend_module_entry *php_builtin_extensions[] = {
 #if HAVE_CTYPE
 	,phpext_ctype_ptr
 #endif
-#if HAVE_DATE
-	,phpext_date_ptr
-#endif
 #if HAVE_FTP
 	,phpext_ftp_ptr
 #endif
-#if HAVE_HASH
-	,phpext_hash_ptr
-#endif
-#if HAVE_ICONV
-	,phpext_iconv_ptr
-#endif
 #if HAVE_MBSTRING
 	,phpext_mbstring_ptr
 #endif
@@ -151,7 +126,6 @@ static zend_module_entry *php_builtin_extensions[] = {
 #if HAVE_PCRE || HAVE_BUNDLED_PCRE
 	,phpext_pcre_ptr
 #endif
-	,phpext_reflection_ptr
 #if HAVE_PHP_SESSION
 	,phpext_session_ptr
 #endif
@@ -179,21 +153,12 @@ static zend_module_entry *php_builtin_extensions[] = {
 #if HAVE_SQLITE
 	,phpext_sqlite_ptr
 #endif
-#if HAVE_SPL
-	,phpext_spl_ptr
-#endif
-#if HAVE_XML && HAVE_XMLREADER
-	,phpext_xmlreader_ptr
-#endif
-#if HAVE_XML && HAVE_XMLWRITER
-	,phpext_xmlwriter_ptr
-#endif
 };
 /* }}} */
 
 #define EXTCOUNT (sizeof(php_builtin_extensions)/sizeof(zend_module_entry *))
 	
-PHPAPI int php_register_internal_extensions(TSRMLS_D)
+int php_register_internal_extensions(TSRMLS_D)
 {
 	return php_register_extensions(php_builtin_extensions, EXTCOUNT TSRMLS_CC);
 }
