@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: network.c,v 1.118.2.2.2.6.2.17 2009/05/04 14:44:46 tony2001 Exp $ */
+/* $Id: network.c,v 1.118.2.1 2006/01/01 12:50:17 sniper Exp $ */
 
 /*#define DEBUG_MAIN_NETWORK 1*/
 
@@ -26,9 +26,8 @@
 #include <stddef.h>
 
 #ifdef PHP_WIN32
-# include "win32/inet.h"
-# define O_RDONLY _O_RDONLY
-# include "win32/param.h"
+#define O_RDONLY _O_RDONLY
+#include "win32/param.h"
 #elif defined(NETWARE)
 #include <sys/timeval.h>
 #include <sys/param.h>
@@ -91,11 +90,6 @@ int inet_aton(const char *, struct in_addr *);
 # define SOCK_ERR INVALID_SOCKET
 # define SOCK_CONN_ERR SOCKET_ERROR
 # define PHP_TIMEOUT_ERROR_VALUE		WSAETIMEDOUT
-
-#if HAVE_IPV6
-const struct in6_addr in6addr_any = {0}; /* IN6ADDR_ANY_INIT; */
-#endif
-
 #else
 # define SOCK_ERR -1
 # define SOCK_CONN_ERR -1
@@ -109,7 +103,7 @@ const struct in6_addr in6addr_any = {0}; /* IN6ADDR_ANY_INIT; */
 #  define PHP_GAI_STRERROR(x) (php_gai_strerror(x))
 /* {{{ php_gai_strerror
  */
-static const char *php_gai_strerror(int code)
+static char *php_gai_strerror(int code)
 {
         static struct {
                 int code;
@@ -168,9 +162,7 @@ static int php_network_getaddresses(const char *host, int socktype, struct socka
 	struct sockaddr **sap;
 	int n;
 #if HAVE_GETADDRINFO
-# if HAVE_IPV6
 	static int ipv6_borked = -1; /* the way this is used *is* thread safe */
-# endif
 	struct addrinfo hints, *res, *sai;
 #else
 	struct hostent *host_info;
@@ -208,20 +200,10 @@ static int php_network_getaddresses(const char *host, int socktype, struct socka
 # endif
 		
 	if ((n = getaddrinfo(host, NULL, &hints, &res))) {
-		if (error_string) {
-			spprintf(error_string, 0, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
-		}
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
 		return 0;
 	} else if (res == NULL) {
-		if (error_string) {
-			spprintf(error_string, 0, "php_network_getaddresses: getaddrinfo failed (null result pointer) errno=%d", errno);
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed (null result pointer)");
-		}
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed (null result pointer)");
 		return 0;
 	}
 
@@ -245,12 +227,7 @@ static int php_network_getaddresses(const char *host, int socktype, struct socka
 		/* XXX NOT THREAD SAFE (is safe under win32) */
 		host_info = gethostbyname(host);
 		if (host_info == NULL) {
-			if (error_string) {
-				spprintf(error_string, 0, "php_network_getaddresses: gethostbyname failed. errno=%d", errno);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
-			} else {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: gethostbyname failed");
-			}
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: gethostbyname failed");
 			return 0;
 		}
 		in = *((struct in_addr *) host_info->h_addr);
@@ -464,7 +441,7 @@ php_socket_t php_network_bind_socket_to_local_addr(const char *host, unsigned po
 			err = php_socket_errno();
 		}
 
-		closesocket(sock);
+		close(sock);
 	}
 	sock = -1;
 
@@ -647,7 +624,6 @@ PHPAPI int php_network_get_peer_name(php_socket_t sock,
 {
 	php_sockaddr_storage sa;
 	socklen_t sl = sizeof(sa);
-	memset(&sa, 0, sizeof(sa));
 	
 	if (getpeername(sock, (struct sockaddr*)&sa, &sl) == 0) {
 		php_network_populate_name_from_sockaddr((struct sockaddr*)&sa, sl,
@@ -667,7 +643,6 @@ PHPAPI int php_network_get_sock_name(php_socket_t sock,
 {
 	php_sockaddr_storage sa;
 	socklen_t sl = sizeof(sa);
-	memset(&sa, 0, sizeof(sa));
 	
 	if (getsockname(sock, (struct sockaddr*)&sa, &sl) == 0) {
 		php_network_populate_name_from_sockaddr((struct sockaddr*)&sa, sl,
@@ -792,14 +767,9 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 		switch (sa->sa_family) {
 #if HAVE_GETADDRINFO && HAVE_IPV6
 			case AF_INET6:
-				if (bindto && strchr(bindto, ':')) {
-					((struct sockaddr_in6 *)sa)->sin6_family = sa->sa_family;
-					((struct sockaddr_in6 *)sa)->sin6_port = htons(port);
-					socklen = sizeof(struct sockaddr_in6);
-				} else {
-					socklen = 0;
-					sa = NULL;
-				}
+				((struct sockaddr_in6 *)sa)->sin6_family = sa->sa_family;
+				((struct sockaddr_in6 *)sa)->sin6_port = htons(port);
+				socklen = sizeof(struct sockaddr_in6);
 				break;
 #endif
 			case AF_INET:
@@ -817,46 +787,37 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 			/* make a connection attempt */
 
 			if (bindto) {
-				struct sockaddr *local_address = NULL;
-				int local_address_len = 0;
-
+				struct sockaddr local_address;
+			
 				if (sa->sa_family == AF_INET) {
-					struct sockaddr_in *in4 = emalloc(sizeof(struct sockaddr_in));
-
-					local_address = (struct sockaddr*)in4;
-					local_address_len = sizeof(struct sockaddr_in);
+					struct sockaddr_in *in4 = (struct sockaddr_in*)&local_address;
 				
 					in4->sin_family = sa->sa_family;
 					in4->sin_port = htons(bindport);
 					if (!inet_aton(bindto, &in4->sin_addr)) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid IP Address: %s", bindto);
-						goto skip_bind;
+						goto bad_ip;
 					}
 					memset(&(in4->sin_zero), 0, sizeof(in4->sin_zero));
 				}
 #if HAVE_IPV6 && HAVE_INET_PTON
 				 else { /* IPV6 */
-					struct sockaddr_in6 *in6 = emalloc(sizeof(struct sockaddr_in6));
-
-					local_address = (struct sockaddr*)in6;
-					local_address_len = sizeof(struct sockaddr_in6);
+					struct sockaddr_in6 *in6 = (struct sockaddr_in6*)&local_address;
 				
 					in6->sin6_family = sa->sa_family;
 					in6->sin6_port = htons(bindport);
 					if (inet_pton(AF_INET6, bindto, &in6->sin6_addr) < 1) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid IP Address: %s", bindto);
-						goto skip_bind;
+						goto bad_ip;
 					}
 				}
 #endif
-				if (!local_address || bind(sock, local_address, local_address_len)) {
+				if (bind(sock, &local_address, sizeof(struct sockaddr))) {
 					php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to bind to '%s:%d', system said: %s", bindto, bindport, strerror(errno));
 				}
-skip_bind:
-				if (local_address) {
-					efree(local_address);
-				}
+				goto bind_done;
+bad_ip:
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid IP Address: %s", bindto);
 			}
+bind_done:
 			/* free error string recieved during previous iteration (if any) */
 			if (error_string && *error_string) {
 				efree(*error_string);
@@ -900,7 +861,7 @@ skip_bind:
 #endif
 		}
 
-		closesocket(sock);
+		close(sock);
 	}
 	sock = -1;
 
@@ -1027,12 +988,10 @@ PHPAPI php_stream *_php_stream_sock_open_from_socket(php_socket_t socket, const
 	sock->socket = socket;
 
 	stream = php_stream_alloc_rel(&php_stream_generic_socket_ops, sock, persistent_id, "r+");
+	stream->flags |= PHP_STREAM_FLAG_AVOID_BLOCKING;
 
-	if (stream == NULL) {
+	if (stream == NULL)	
 		pefree(sock, persistent_id ? 1 : 0);
-	} else {
-		stream->flags |= PHP_STREAM_FLAG_AVOID_BLOCKING;
-	}
 
 	return stream;
 }
@@ -1056,36 +1015,32 @@ PHPAPI php_stream *_php_stream_sock_open_host(const char *host, unsigned short p
 
 PHPAPI int php_set_sock_blocking(int socketd, int block TSRMLS_DC)
 {
-	int ret = SUCCESS;
-	int flags;
-	int myflag = 0;
+      int ret = SUCCESS;
+      int flags;
+      int myflag = 0;
 
 #ifdef PHP_WIN32
-	/* with ioctlsocket, a non-zero sets nonblocking, a zero sets blocking */
-	flags = !block;
-	if (ioctlsocket(socketd, FIONBIO, &flags) == SOCKET_ERROR) {
-		char *error_string;
-		
-		error_string = php_socket_strerror(WSAGetLastError(), NULL, 0);
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", error_string);
-		efree(error_string);
-		ret = FAILURE;
-	}
+      /* with ioctlsocket, a non-zero sets nonblocking, a zero sets blocking */
+	  flags = !block;
+	  if (ioctlsocket(socketd, FIONBIO, &flags)==SOCKET_ERROR){
+		  php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", WSAGetLastError());
+		  ret = FALSE;
+      }
 #else
-	flags = fcntl(socketd, F_GETFL);
+      flags = fcntl(socketd, F_GETFL);
 #ifdef O_NONBLOCK
-	myflag = O_NONBLOCK; /* POSIX version */
+      myflag = O_NONBLOCK; /* POSIX version */
 #elif defined(O_NDELAY)
-	myflag = O_NDELAY;   /* old non-POSIX version */
+      myflag = O_NDELAY;   /* old non-POSIX version */
 #endif
-	if (!block) {
-		flags |= myflag;
-	} else {
+      if (!block) {
+              flags |= myflag;
+      } else {
 		flags &= ~myflag;
-	}
-	fcntl(socketd, F_SETFL, flags);
+      }
+      fcntl(socketd, F_SETFL, flags);
 #endif
-	return ret;
+      return ret;
 }
 
 PHPAPI void _php_emit_fd_setsize_warning(int max_fd)
@@ -1152,12 +1107,6 @@ PHPAPI int php_poll2(php_pollfd *ufds, unsigned int nfds, int timeout)
 		tv.tv_sec = timeout / 1000;
 		tv.tv_usec = (timeout - (tv.tv_sec * 1000)) * 1000;
 	}
-/* Reseting/initializing */
-#ifdef PHP_WIN32
-	WSASetLastError(0);
-#else
-	errno = 0;
-#endif
 	n = select(max_fd + 1, &rset, &wset, &eset, timeout >= 0 ? &tv : NULL);
 
 	if (n >= 0) {
