@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: plain_wrapper.c,v 1.52.2.6.2.14 2007/01/01 09:36:12 sebastian Exp $ */
+/* $Id: plain_wrapper.c,v 1.52.2.4 2006/01/01 12:50:18 sniper Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -39,11 +39,6 @@
 
 #include "php_streams_int.h"
 
-#define php_stream_fopen_from_fd_int(fd, mode, persistent_id)	_php_stream_fopen_from_fd_int((fd), (mode), (persistent_id) STREAMS_CC TSRMLS_CC)
-#define php_stream_fopen_from_fd_int_rel(fd, mode, persistent_id)	 _php_stream_fopen_from_fd_int((fd), (mode), (persistent_id) STREAMS_REL_CC TSRMLS_CC)
-#define php_stream_fopen_from_file_int(file, mode)	_php_stream_fopen_from_file_int((file), (mode) STREAMS_CC TSRMLS_CC)
-#define php_stream_fopen_from_file_int_rel(file, mode)	 _php_stream_fopen_from_file_int((file), (mode) STREAMS_REL_CC TSRMLS_CC)
-
 /* parse standard "fopen" modes into open() flags */
 PHPAPI int php_stream_parse_fopen_modes(const char *mode, int *open_flags)
 {
@@ -133,44 +128,12 @@ static int do_fstat(php_stdio_stream_data *d, int force)
 	return 0;
 }
 
-static php_stream *_php_stream_fopen_from_fd_int(int fd, const char *mode, const char *persistent_id STREAMS_DC TSRMLS_DC)
-{
-	php_stdio_stream_data *self;
-	
-	self = pemalloc_rel_orig(sizeof(*self), persistent_id);
-	memset(self, 0, sizeof(*self));
-	self->file = NULL;
-	self->is_pipe = 0;
-	self->lock_flag = LOCK_UN;
-	self->is_process_pipe = 0;
-	self->temp_file_name = NULL;
-	self->fd = fd;
-	
-	return php_stream_alloc_rel(&php_stream_stdio_ops, self, persistent_id, mode);
-}
-
-static php_stream *_php_stream_fopen_from_file_int(FILE *file, const char *mode STREAMS_DC TSRMLS_DC)
-{
-	php_stdio_stream_data *self;
-	
-	self = emalloc_rel_orig(sizeof(*self));
-	memset(self, 0, sizeof(*self));
-	self->file = file;
-	self->is_pipe = 0;
-	self->lock_flag = LOCK_UN;
-	self->is_process_pipe = 0;
-	self->temp_file_name = NULL;
-	self->fd = fileno(file);
-
-	return php_stream_alloc_rel(&php_stream_stdio_ops, self, 0, mode);
-}
-
 PHPAPI php_stream *_php_stream_fopen_temporary_file(const char *dir, const char *pfx, char **opened_path STREAMS_DC TSRMLS_DC)
 {
 	int fd = php_open_temporary_fd(dir, pfx, opened_path TSRMLS_CC);
 
 	if (fd != -1)	{
-		php_stream *stream = php_stream_fopen_from_fd_int_rel(fd, "r+b", NULL);
+		php_stream *stream = php_stream_fopen_from_fd_rel(fd, "r+b", NULL);
 		if (stream) {
 			return stream;
 		}
@@ -189,7 +152,7 @@ PHPAPI php_stream *_php_stream_fopen_tmpfile(int dummy STREAMS_DC TSRMLS_DC)
 	int fd = php_open_temporary_fd(NULL, "php", &opened_path TSRMLS_CC);
 
 	if (fd != -1)	{
-		php_stream *stream = php_stream_fopen_from_fd_int_rel(fd, "r+b", NULL);
+		php_stream *stream = php_stream_fopen_from_fd_rel(fd, "r+b", NULL);
 		if (stream) {
 			php_stdio_stream_data *self = (php_stdio_stream_data*)stream->abstract;
 			stream->wrapper = &php_plain_files_wrapper;
@@ -211,26 +174,37 @@ PHPAPI php_stream *_php_stream_fopen_tmpfile(int dummy STREAMS_DC TSRMLS_DC)
 
 PHPAPI php_stream *_php_stream_fopen_from_fd(int fd, const char *mode, const char *persistent_id STREAMS_DC TSRMLS_DC)
 {
-	php_stream *stream = php_stream_fopen_from_fd_int_rel(fd, mode, persistent_id);
-
-	if (stream) {
-		php_stdio_stream_data *self = (php_stdio_stream_data*)stream->abstract;
+	php_stdio_stream_data *self;
+	php_stream *stream;
+	
+	self = pemalloc_rel_orig(sizeof(*self), persistent_id);
+	memset(self, 0, sizeof(*self));
+	self->file = NULL;
+	self->is_pipe = 0;
+	self->lock_flag = LOCK_UN;
+	self->is_process_pipe = 0;
+	self->temp_file_name = NULL;
+	self->fd = fd;
 
 #ifdef S_ISFIFO
-		/* detect if this is a pipe */
-		if (self->fd >= 0) {
-			self->is_pipe = (do_fstat(self, 0) == 0 && S_ISFIFO(self->sb.st_mode)) ? 1 : 0;
-		}
+	/* detect if this is a pipe */
+	if (self->fd >= 0) {
+		self->is_pipe = (do_fstat(self, 0) == 0 && S_ISFIFO(self->sb.st_mode)) ? 1 : 0;
+	}
 #elif defined(PHP_WIN32)
-		{
-			long handle = _get_osfhandle(self->fd);
+	{
+		long handle = _get_osfhandle(self->fd);
+		DWORD in_buf_size, out_buf_size;
 
-			if (handle != 0xFFFFFFFF) {
-				self->is_pipe = GetFileType((HANDLE)handle) == FILE_TYPE_PIPE;
-			}
+		if (handle != 0xFFFFFFFF) {
+			self->is_pipe = GetNamedPipeInfo((HANDLE)handle, NULL, &out_buf_size, &in_buf_size, NULL);
 		}
+	}
 #endif
 	
+	stream = php_stream_alloc_rel(&php_stream_stdio_ops, self, persistent_id, mode);
+
+	if (stream) {
 		if (self->is_pipe) {
 			stream->flags |= PHP_STREAM_FLAG_NO_SEEK;
 		} else {
@@ -249,26 +223,37 @@ PHPAPI php_stream *_php_stream_fopen_from_fd(int fd, const char *mode, const cha
 
 PHPAPI php_stream *_php_stream_fopen_from_file(FILE *file, const char *mode STREAMS_DC TSRMLS_DC)
 {
-	php_stream *stream = php_stream_fopen_from_file_int_rel(file, mode);
-
-	if (stream) {
-		php_stdio_stream_data *self = (php_stdio_stream_data*)stream->abstract;
+	php_stdio_stream_data *self;
+	php_stream *stream;
+	
+	self = emalloc_rel_orig(sizeof(*self));
+	memset(self, 0, sizeof(*self));
+	self->file = file;
+	self->is_pipe = 0;
+	self->lock_flag = LOCK_UN;
+	self->is_process_pipe = 0;
+	self->temp_file_name = NULL;
+	self->fd = fileno(file);
 
 #ifdef S_ISFIFO
-		/* detect if this is a pipe */
-		if (self->fd >= 0) {
-			self->is_pipe = (do_fstat(self, 0) == 0 && S_ISFIFO(self->sb.st_mode)) ? 1 : 0;
-		}
+	/* detect if this is a pipe */
+	if (self->fd >= 0) {
+		self->is_pipe = (do_fstat(self, 0) == 0 && S_ISFIFO(self->sb.st_mode)) ? 1 : 0;
+	}
 #elif defined(PHP_WIN32)
-		{
-			long handle = _get_osfhandle(self->fd);
+	{
+		long handle = _get_osfhandle(self->fd);
+		DWORD in_buf_size, out_buf_size;
 
-			if (handle != 0xFFFFFFFF) {
-				self->is_pipe = GetFileType((HANDLE)handle) == FILE_TYPE_PIPE;
-			}
+		if (handle != 0xFFFFFFFF) {
+			self->is_pipe = GetNamedPipeInfo((HANDLE)handle, NULL, &out_buf_size, &in_buf_size, NULL);
 		}
+	}
 #endif
 	
+	stream = php_stream_alloc_rel(&php_stream_stdio_ops, self, 0, mode);
+
+	if (stream) {
 		if (self->is_pipe) {
 			stream->flags |= PHP_STREAM_FLAG_NO_SEEK;
 		} else {
@@ -329,9 +314,6 @@ static size_t php_stdiop_read(php_stream *stream, char *buf, size_t count TSRMLS
 	assert(data != NULL);
 
 	if (data->fd >= 0) {
-		if (stream->eof && !data->is_pipe) {
-			return 0;
-		}
 		ret = read(data->fd, buf, count);
 		
 		stream->eof = (ret == 0 || (ret == (size_t)-1 && errno != EWOULDBLOCK));
@@ -392,7 +374,7 @@ static int php_stdiop_close(php_stream *stream, int close_handle TSRMLS_DC)
 				data->file = NULL;
 			}
 		} else if (data->fd != -1) {
-#if PHP_DEBUG
+#ifdef PHP_DEBUG
 			if ((data->fd == 1 || data->fd == 2) && 0 == strcmp(sapi_module.name, "cli")) {
 				/* don't close stdout or stderr in CLI in DEBUG mode, as we want to see any leaks */
 				ret = 0;
@@ -627,16 +609,9 @@ static int php_stdiop_set_option(php_stream *stream, int option, int value, void
 
 					case PHP_STREAM_MMAP_MAP_RANGE:
 						do_fstat(data, 1);
-						if (range->length == 0 && range->offset > 0 && range->offset < data->sb.st_size) {
-							range->length = data->sb.st_size - range->offset;
-						}
 						if (range->length == 0 || range->length > data->sb.st_size) {
 							range->length = data->sb.st_size;
 						}
-						if (range->offset >= data->sb.st_size) {
-							range->offset = data->sb.st_size;
-							range->length = 0;
-						}
 						switch (range->mode) {
 							case PHP_STREAM_MAP_MODE_READONLY:
 								prot = PROT_READ;
@@ -731,7 +706,7 @@ static int php_stdiop_set_option(php_stream *stream, int option, int value, void
 							delta = range->offset - loffs;
 						}
 
-						data->last_mapped_addr = MapViewOfFile(data->file_mapping, acc, 0, loffs, range->length + delta);
+						data->last_mapped_addr = MapViewOfFile(data->file_mapping, acc, 0, loffs, range->length);
 
 						if (data->last_mapped_addr) {
 							/* give them back the address of the start offset they requested */
@@ -768,7 +743,7 @@ static int php_stdiop_set_option(php_stream *stream, int option, int value, void
 					return fd == -1 ? PHP_STREAM_OPTION_RETURN_ERR : PHP_STREAM_OPTION_RETURN_OK;
 
 				case PHP_STREAM_TRUNCATE_SET_SIZE:
-					return ftruncate(fd, *(ptrdiff_t*)ptrparam) == 0 ? PHP_STREAM_OPTION_RETURN_OK : PHP_STREAM_OPTION_RETURN_ERR;
+					return ftruncate(fd, *(size_t*)ptrparam) == 0 ? PHP_STREAM_OPTION_RETURN_OK : PHP_STREAM_OPTION_RETURN_ERR;
 			}
 			
 		default:
@@ -904,11 +879,7 @@ PHPAPI php_stream *_php_stream_fopen(const char *filename, const char *mode, cha
 
 	if (fd != -1)	{
 
-		if (options & STREAM_OPEN_FOR_INCLUDE) {
-			ret = php_stream_fopen_from_fd_int_rel(fd, mode, persistent_id);
-		} else {
-			ret = php_stream_fopen_from_fd_rel(fd, mode, persistent_id);
-		}
+		ret = php_stream_fopen_from_fd_rel(fd, mode, persistent_id);
 
 		if (ret)	{
 			if (opened_path) {
@@ -922,8 +893,6 @@ PHPAPI php_stream *_php_stream_fopen(const char *filename, const char *mode, cha
 				efree(persistent_id);
 			}
 
-			/* WIN32 always set ISREG flag */
-#ifndef PHP_WIN32
 			/* sanity checks for include/require.
 			 * We check these after opening the stream, so that we save
 			 * on fstat() syscalls */
@@ -932,12 +901,15 @@ PHPAPI php_stream *_php_stream_fopen(const char *filename, const char *mode, cha
 				int r;
 
 				r = do_fstat(self, 0);
-				if ((r == 0 && !S_ISREG(self->sb.st_mode))) {
+				if (
+#ifndef PHP_WIN32
+						(r != 0) || /* it is OK for fstat to fail under win32 */
+#endif
+						(r == 0 && !S_ISREG(self->sb.st_mode))) {
 					php_stream_close(ret);
 					return NULL;
 				}
 			}
-#endif
 
 			return ret;
 		}
@@ -1117,46 +1089,18 @@ static int php_plain_files_mkdir(php_stream_wrapper *wrapper, char *dir, int mod
 		int offset = 0;
 
 		buf = estrndup(dir, dir_len);
-
-#ifdef PHP_WIN32
-		e = buf;
-		while (*e) {
-			if (*e == '/') {
-				*e = DEFAULT_SLASH;
-			}
-			e++;
-		}
-#else
 		e = buf + dir_len;
-#endif
 
 		if ((p = memchr(buf, DEFAULT_SLASH, dir_len))) {
 			offset = p - buf + 1;
 		}
 
-		if (p && dir_len == 1) {
-			/* buf == "DEFAULT_SLASH" */	
-		}
-		else {
-			/* find a top level directory we need to create */
-			while ( (p = strrchr(buf + offset, DEFAULT_SLASH)) || ( offset !=1 && (p = strrchr(buf, DEFAULT_SLASH))) ) {
-				int n = 0;
-
-				*p = '\0';
-				while (p > buf && *(p-1) == DEFAULT_SLASH) {
-					++n;
-					--p;
-					*p = '\0';
-				}
-				if (VCWD_STAT(buf, &sb) == 0) {
-					while (1) {
-						*p = DEFAULT_SLASH;
-						if (!n) break;
-						--n;
-						++p;
-					}
-					break;
-				}
+		/* find a top level directory we need to create */
+		while ((p = strrchr(buf + offset, DEFAULT_SLASH))) {
+			*p = '\0';
+			if (VCWD_STAT(buf, &sb) == 0) {
+				*p = DEFAULT_SLASH;
+				break;
 			}
 		}
 
@@ -1168,13 +1112,10 @@ static int php_plain_files_mkdir(php_stream_wrapper *wrapper, char *dir, int mod
 			}
 			/* create any needed directories if the creation of the 1st directory worked */
 			while (++p != e) {
-				if (*p == '\0') {
+				if (*p == '\0' && *(p + 1) != '\0') {
 					*p = DEFAULT_SLASH;
-					if ((*(p+1) != '\0') &&
-					    (ret = VCWD_MKDIR(buf, (mode_t)mode)) < 0) {
-						if (options & REPORT_ERRORS) {
-							php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
-						}
+					if ((ret = VCWD_MKDIR(buf, (mode_t)mode)) < 0) {
+						php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
 						break;
 					}
 				}
@@ -1324,6 +1265,11 @@ not_relative_path:
 #endif
 
 	if (!path || (path && !*path)) {
+
+		if (((options & STREAM_DISABLE_OPEN_BASEDIR) == 0) && php_check_open_basedir(path TSRMLS_CC)) {
+			return NULL;
+		}
+
 		if (PG(safe_mode) && (!php_checkuid(filename, mode, CHECKUID_CHECK_MODE_PARAM))) {
 			return NULL;
 		}
@@ -1363,13 +1309,11 @@ not_relative_path:
 			*end = '\0';
 			end++;
 		}
-		if (*ptr == '\0') {
-			goto stream_skip;
-		}
 		snprintf(trypath, MAXPATHLEN, "%s/%s", ptr, filename);
 
 		if (((options & STREAM_DISABLE_OPEN_BASEDIR) == 0) && php_check_open_basedir_ex(trypath, 0 TSRMLS_CC)) {
-			goto stream_skip;
+			ptr = end;
+			continue;
 		}
 		
 		if (PG(safe_mode)) {
@@ -1382,7 +1326,8 @@ not_relative_path:
 					goto stream_done;
 				}
 			}
-			goto stream_skip;
+			ptr = end;
+			continue;
 		}
 		stream = php_stream_fopen_rel(trypath, mode, opened_path, options);
 		if (stream) {
@@ -1390,7 +1335,6 @@ stream_done:
 			efree(pathbuf);
 			return stream;
 		}
-stream_skip:
 		ptr = end;
 	} /* end provided path */
 
