@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,15 +17,11 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: userspace.c,v 1.31.2.3.2.7.2.10 2009/05/08 11:35:12 bjori Exp $ */
+/* $Id: userspace.c,v 1.31.2.2 2006/01/01 12:50:18 sniper Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
 #include "ext/standard/file.h"
-#include "ext/standard/flock_compat.h"
-#ifdef HAVE_SYS_FILE_H
-#include <sys/file.h>
-#endif
 
 static int le_protocols;
 
@@ -85,20 +81,6 @@ PHP_MINIT_FUNCTION(user_streams)
 	REGISTER_LONG_CONSTANT("STREAM_URL_STAT_QUIET", 	PHP_STREAM_URL_STAT_QUIET,		CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_MKDIR_RECURSIVE",	PHP_STREAM_MKDIR_RECURSIVE,		CONST_CS|CONST_PERSISTENT);
 
-	REGISTER_LONG_CONSTANT("STREAM_IS_URL",	PHP_STREAM_IS_URL,		CONST_CS|CONST_PERSISTENT);
-
-	REGISTER_LONG_CONSTANT("STREAM_OPTION_BLOCKING",	PHP_STREAM_OPTION_BLOCKING,		CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_OPTION_READ_TIMEOUT",	PHP_STREAM_OPTION_READ_TIMEOUT,		CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_OPTION_READ_BUFFER",	PHP_STREAM_OPTION_READ_BUFFER,		CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_OPTION_WRITE_BUFFER",	PHP_STREAM_OPTION_WRITE_BUFFER,		CONST_CS|CONST_PERSISTENT);
-
-	REGISTER_LONG_CONSTANT("STREAM_BUFFER_NONE",		PHP_STREAM_BUFFER_NONE,			CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_BUFFER_LINE",		PHP_STREAM_BUFFER_LINE,			CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_BUFFER_FULL",		PHP_STREAM_BUFFER_FULL,			CONST_CS|CONST_PERSISTENT);
-
-	REGISTER_LONG_CONSTANT("STREAM_CAST_AS_STREAM",		PHP_STREAM_AS_STDIO,			CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_CAST_FOR_SELECT",	PHP_STREAM_AS_FD_FOR_SELECT,		CONST_CS|CONST_PERSISTENT);
-
 	return SUCCESS;
 }
 
@@ -128,8 +110,6 @@ typedef struct _php_userstream_data php_userstream_data_t;
 #define USERSTREAM_DIR_REWIND	"dir_rewinddir"
 #define USERSTREAM_DIR_CLOSE	"dir_closedir"
 #define USERSTREAM_LOCK     "stream_lock"
-#define USERSTREAM_CAST		"stream_cast"
-#define USERSTREAM_SET_OPTION	"stream_set_option"
 
 /* {{{ class should have methods like these:
  
@@ -179,33 +159,6 @@ typedef struct _php_userstream_data php_userstream_data_t;
 		return array( just like that returned by fstat() );
 	}
 
-	function stream_cast($castas)
-	{
-		if ($castas == STREAM_CAST_FOR_SELECT) {
-			return $this->underlying_stream;
-		}
-		return false;
-	}
-
-	function stream_set_option($option, $arg1, $arg2)
-	{
-		switch($option) {
-		case STREAM_OPTION_BLOCKING:
-			$blocking = $arg1;
-			...
-		case STREAM_OPTION_READ_TIMEOUT:
-			$sec = $arg1;
-			$usec = $arg2;
-			...
-		case STREAM_OPTION_WRITE_BUFFER:
-			$mode = $arg1;
-			$size = $arg2;
-			...
-		default:
-			return false;
-		}
-	}
-
 	function url_stat(string $url, int $flags)
 	{
 		return array( just like that returned by stat() );
@@ -250,11 +203,6 @@ typedef struct _php_userstream_data php_userstream_data_t;
 	{
 		reset to start of dir list;
 	}
-
-	function stream_lock($operation)
-	{
-		return true / false;
-	}
   
 	}}} **/
 
@@ -266,7 +214,7 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	zval **args[4];	
 	int call_result;
 	php_stream *stream = NULL;
-	zend_bool old_in_user_include;
+	zval *zcontext = NULL;
 
 	/* Try to catch bad usage without preventing flexibility */
 	if (FG(user_stream_current_filename) != NULL && strcmp(filename, FG(user_stream_current_filename)) == 0) {
@@ -275,65 +223,22 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	}
 	FG(user_stream_current_filename) = filename;
 	
-	/* if the user stream was registered as local and we are in include context,
-		we add allow_url_include restrictions to allow_url_fopen ones */
-	/* we need only is_url == 0 here since if is_url == 1 and remote wrappers
-		were restricted we wouldn't get here */
-	old_in_user_include = PG(in_user_include);
-	if(uwrap->wrapper.is_url == 0 && 
-		(options & STREAM_OPEN_FOR_INCLUDE) &&
-		!PG(allow_url_include)) {
-		PG(in_user_include) = 1;
-	}
-
 	us = emalloc(sizeof(*us));
 	us->wrapper = uwrap;	
 
 	/* create an instance of our class */
 	ALLOC_ZVAL(us->object);
 	object_init_ex(us->object, uwrap->ce);
-	Z_SET_REFCOUNT_P(us->object, 1);
-	Z_SET_ISREF_P(us->object);
-	
-	if (uwrap->ce->constructor) {
-		zend_fcall_info fci;
-		zend_fcall_info_cache fcc;
-		zval *retval_ptr;
-		
-		fci.size = sizeof(fci);
-		fci.function_table = &uwrap->ce->function_table;
-		fci.function_name = NULL;
-		fci.symbol_table = NULL;
-		fci.object_ptr = us->object;
-		fci.retval_ptr_ptr = &retval_ptr;
-		fci.param_count = 0;
-		fci.params = NULL;
-		fci.no_separation = 1;
-		
-		fcc.initialized = 1;
-		fcc.function_handler = uwrap->ce->constructor;
-		fcc.calling_scope = EG(scope);
-		fcc.called_scope = Z_OBJCE_P(us->object);
-		fcc.object_ptr = us->object;
-
-		if (zend_call_function(&fci, &fcc TSRMLS_CC) == FAILURE) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not execute %s::%s()", uwrap->ce->name, uwrap->ce->constructor->common.function_name);
-			zval_dtor(us->object);
-			FREE_ZVAL(us->object);
-			efree(us);
-			FG(user_stream_current_filename) = NULL;
-			PG(in_user_include) = old_in_user_include;
-			return NULL;
-		} else {
-			if (retval_ptr) {
-				zval_ptr_dtor(&retval_ptr);
-			}
-		}
-	}
+	ZVAL_REFCOUNT(us->object) = 1;
+	PZVAL_IS_REF(us->object) = 1;
 
 	if (context) {
-		add_property_resource(us->object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(us->object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(us->object, "context");
 	}
@@ -352,8 +257,8 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	args[2] = &zoptions;
 
 	MAKE_STD_ZVAL(zopened);
-	Z_SET_REFCOUNT_P(zopened, 1);
-	Z_SET_ISREF_P(zopened);
+	ZVAL_REFCOUNT(zopened) = 1;
+	PZVAL_IS_REF(zopened) = 1;
 	ZVAL_NULL(zopened);
 	args[3] = &zopened;
 
@@ -400,7 +305,6 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 
 	FG(user_stream_current_filename) = NULL;
 		
-	PG(in_user_include) = old_in_user_include;
 	return stream;
 }
 
@@ -409,7 +313,7 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
 	php_userstream_data_t *us;
-	zval *zfilename, *zoptions, *zretval = NULL, *zfuncname;
+	zval *zfilename, *zoptions, *zretval = NULL, *zfuncname, *zcontext;
 	zval **args[2];	
 	int call_result;
 	php_stream *stream = NULL;
@@ -427,12 +331,16 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 	/* create an instance of our class */
 	ALLOC_ZVAL(us->object);
 	object_init_ex(us->object, uwrap->ce);
-	Z_SET_REFCOUNT_P(us->object, 1);
-	Z_SET_ISREF_P(us->object);
+	ZVAL_REFCOUNT(us->object) = 1;
+	PZVAL_IS_REF(us->object) = 1;
 
 	if (context) {
-		add_property_resource(us->object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(us->object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(us->object, "context");
 	}
@@ -486,7 +394,7 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 }
 
 
-/* {{{ proto bool stream_wrapper_register(string protocol, string classname[, integer flags])
+/* {{{ proto bool stream_wrapper_register(string protocol, string classname)
    Registers a custom URL protocol handler class */
 PHP_FUNCTION(stream_wrapper_register)
 {
@@ -494,9 +402,8 @@ PHP_FUNCTION(stream_wrapper_register)
 	int protocol_len, classname_len;
 	struct php_user_stream_wrapper * uwrap;
 	int rsrc_id;
-	long flags = 0;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|l", &protocol, &protocol_len, &classname, &classname_len, &flags) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &protocol, &protocol_len, &classname, &classname_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 	
@@ -505,7 +412,6 @@ PHP_FUNCTION(stream_wrapper_register)
 	uwrap->classname = estrndup(classname, classname_len);
 	uwrap->wrapper.wops = &user_stream_wops;
 	uwrap->wrapper.abstract = uwrap;
-	uwrap->wrapper.is_url = ((flags & PHP_STREAM_IS_URL) != 0);
 
 	rsrc_id = ZEND_REGISTER_RESOURCE(NULL, uwrap, le_protocols);
 
@@ -515,11 +421,11 @@ PHP_FUNCTION(stream_wrapper_register)
 			RETURN_TRUE;
 		} else {
 			/* We failed.  But why? */
-			if (zend_hash_exists(php_stream_get_url_stream_wrappers_hash(), protocol, protocol_len + 1)) {
+			if (zend_hash_exists(php_stream_get_url_stream_wrappers_hash(), protocol, protocol_len)) {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Protocol %s:// is already defined.", protocol);
 			} else {
-				/* Hash doesn't exist so it must have been an invalid protocol scheme */
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid protocol scheme specified. Unable to register wrapper class %s to %s://", classname, protocol);
+				/* Should never happen */
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to register wrapper class %s to %s://", classname, protocol);
 			}
 		}
 	} else {
@@ -558,7 +464,7 @@ PHP_FUNCTION(stream_wrapper_restore)
 {
 	char *protocol;
 	int protocol_len;
-	php_stream_wrapper **wrapperpp = NULL, *wrapper;
+	php_stream_wrapper *wrapper = NULL;
 	HashTable *global_wrapper_hash;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &protocol, &protocol_len) == FAILURE) {
@@ -571,14 +477,11 @@ PHP_FUNCTION(stream_wrapper_restore)
 		RETURN_TRUE;
 	}
 
-	if ((zend_hash_find(global_wrapper_hash, protocol, protocol_len + 1, (void**)&wrapperpp) == FAILURE) || !wrapperpp) {
+	if ((zend_hash_find(global_wrapper_hash, protocol, protocol_len, (void**)&wrapper) == FAILURE) || !wrapper) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s:// never existed, nothing to restore", protocol);
 		RETURN_FALSE;
 	}
 
-	/* next line might delete the pointer that wrapperpp points at, so deref it now */
-	wrapper = *wrapperpp;
-
 	/* A failure here could be okay given that the protocol might have been merely unregistered */
 	php_unregister_url_stream_wrapper_volatile(protocol TSRMLS_CC);
 
@@ -819,10 +722,6 @@ static int php_userstreamop_seek(php_stream *stream, off_t offset, int whence, o
 		retval = NULL;
 	}
 
-	if (ret) {
-		return ret;
-	}
-
 	/* now determine where we are */
 	ZVAL_STRINGL(&func_name, USERSTREAM_TELL, sizeof(USERSTREAM_TELL)-1, 0);
 
@@ -832,20 +731,16 @@ static int php_userstreamop_seek(php_stream *stream, off_t offset, int whence, o
 		&retval,
 		0, NULL, 0, NULL TSRMLS_CC);
 
-	if (call_result == SUCCESS && retval != NULL && Z_TYPE_P(retval) == IS_LONG) {
+	if (call_result == SUCCESS && retval != NULL && Z_TYPE_P(retval) == IS_LONG)
 		*newoffs = Z_LVAL_P(retval);
-		ret = 0;
-	} else if (call_result == FAILURE) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_TELL " is not implemented!", us->wrapper->classname);
-		ret = -1;
-	} else {
-		ret = -1;
-	}
+	else
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_TELL " is not implemented!",
+				us->wrapper->classname);
 
-	if (retval) {
+	if (retval)
 		zval_ptr_dtor(&retval);
-	}
-	return ret;
+	
+	return 0;
 }
 
 /* parse the return value from one of the stat functions and store the
@@ -933,7 +828,7 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
 	int ret = -1;
 	zval *zvalue = NULL;
-	zval **args[3];
+	zval **args[1];
 
 	switch (option) {
 	case PHP_STREAM_OPTION_CHECK_LIVENESS:
@@ -951,23 +846,7 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 
 	case PHP_STREAM_OPTION_LOCKING:
 		MAKE_STD_ZVAL(zvalue);
-		ZVAL_LONG(zvalue, 0);
-
-		if (value & LOCK_NB) {
-			Z_LVAL_P(zvalue) |= PHP_LOCK_NB;
-		}
-		switch(value & ~LOCK_NB) {
-		case LOCK_SH:
-			Z_LVAL_P(zvalue) |= PHP_LOCK_SH;
-			break;
-		case LOCK_EX:
-			Z_LVAL_P(zvalue) |= PHP_LOCK_EX;
-			break;
-		case LOCK_UN:
-			Z_LVAL_P(zvalue) |= PHP_LOCK_UN;
-			break;
-		}
-
+		ZVAL_LONG(zvalue, value);
 		args[0] = &zvalue;
 		
 		/* TODO wouldblock */
@@ -992,75 +871,6 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 		}
 
 		break;
-	
-	case PHP_STREAM_OPTION_READ_BUFFER:
-	case PHP_STREAM_OPTION_WRITE_BUFFER:
-	case PHP_STREAM_OPTION_READ_TIMEOUT:
-	case PHP_STREAM_OPTION_BLOCKING: {
-		zval *zoption = NULL;
-		zval *zptrparam = NULL;
-		
-		ZVAL_STRINGL(&func_name, USERSTREAM_SET_OPTION, sizeof(USERSTREAM_SET_OPTION)-1, 0);
-
-		ALLOC_INIT_ZVAL(zoption);
-		ZVAL_LONG(zoption, option);
-
-		ALLOC_INIT_ZVAL(zvalue);
-		ALLOC_INIT_ZVAL(zptrparam);
-
-		args[0] = &zoption;
-		args[1] = &zvalue;
-		args[2] = &zptrparam;
-
-		switch(option) {
-		case PHP_STREAM_OPTION_READ_BUFFER:
-		case PHP_STREAM_OPTION_WRITE_BUFFER:
-			ZVAL_LONG(zvalue, value);
-			if (ptrparam) {
-				ZVAL_LONG(zptrparam, *(long *)ptrparam);
-			} else {
-				ZVAL_LONG(zptrparam, BUFSIZ);
-			}
-			break;
-		case PHP_STREAM_OPTION_READ_TIMEOUT: {
-			struct timeval tv = *(struct timeval*)ptrparam;
-			ZVAL_LONG(zvalue, tv.tv_sec);
-			ZVAL_LONG(zptrparam, tv.tv_usec);
-			break;
-			}
-		case PHP_STREAM_OPTION_BLOCKING:
-			ZVAL_LONG(zvalue, value);
-			break;
-		default:
-			break;
-		}
-
-		call_result = call_user_function_ex(NULL,
-			&us->object,
-			&func_name,
-			&retval,
-			3, args, 0, NULL TSRMLS_CC);
-	
-		do {
-			if (call_result == FAILURE) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_SET_OPTION " is not implemented!",
-						us->wrapper->classname);
-				break;
-			}
-			if (retval && zend_is_true(retval)) {
-				ret = PHP_STREAM_OPTION_RETURN_OK;
-			}
-		} while (0);
-
-		if (zoption) {
-			zval_ptr_dtor(&zoption);
-		}
-		if (zptrparam) {
-			zval_ptr_dtor(&zptrparam);
-		}
-
-		break;
-		}
 	}
 
 	/* clean up */
@@ -1080,7 +890,7 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zfuncname, *zretval;
+	zval *zfilename, *zfuncname, *zretval, *zcontext;
 	zval **args[1];
 	int call_result;
 	zval *object;
@@ -1089,12 +899,16 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 	/* create an instance of our class */
 	ALLOC_ZVAL(object);
 	object_init_ex(object, uwrap->ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	ZVAL_REFCOUNT(object) = 1;
+	PZVAL_IS_REF(object) = 1;
 
 	if (context) {
-		add_property_resource(object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(object, "context");
 	}
@@ -1134,7 +948,7 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char *url_to, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zold_name, *znew_name, *zfuncname, *zretval;
+	zval *zold_name, *znew_name, *zfuncname, *zretval, *zcontext;
 	zval **args[2];
 	int call_result;
 	zval *object;
@@ -1143,12 +957,16 @@ static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char
 	/* create an instance of our class */
 	ALLOC_ZVAL(object);
 	object_init_ex(object, uwrap->ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	ZVAL_REFCOUNT(object) = 1;
+	PZVAL_IS_REF(object) = 1;
 
 	if (context) {
-		add_property_resource(object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(object, "context");
 	}
@@ -1193,7 +1011,7 @@ static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char
 static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zmode, *zoptions, *zfuncname, *zretval;
+	zval *zfilename, *zmode, *zoptions, *zfuncname, *zretval, *zcontext;
 	zval **args[3];
 	int call_result;
 	zval *object;
@@ -1202,17 +1020,21 @@ static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode,
 	/* create an instance of our class */
 	ALLOC_ZVAL(object);
 	object_init_ex(object, uwrap->ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	ZVAL_REFCOUNT(object) = 1;
+	PZVAL_IS_REF(object) = 1;
 
 	if (context) {
-		add_property_resource(object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(object, "context");
 	}
 
-	/* call the mkdir method */
+	/* call the unlink method */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1258,7 +1080,7 @@ static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode,
 static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zoptions, *zfuncname, *zretval;
+	zval *zfilename, *zoptions, *zfuncname, *zretval, *zcontext;
 	zval **args[3];
 	int call_result;
 	zval *object;
@@ -1267,17 +1089,21 @@ static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int option
 	/* create an instance of our class */
 	ALLOC_ZVAL(object);
 	object_init_ex(object, uwrap->ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	ZVAL_REFCOUNT(object) = 1;
+	PZVAL_IS_REF(object) = 1;
 
 	if (context) {
-		add_property_resource(object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(object, "context");
 	}
 
-	/* call the rmdir method */
+	/* call the unlink method */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1318,7 +1144,7 @@ static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int option
 static int user_wrapper_stat_url(php_stream_wrapper *wrapper, char *url, int flags, php_stream_statbuf *ssb, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zfuncname, *zretval, *zflags;
+	zval *zfilename, *zfuncname, *zretval, *zflags, *zcontext;
 	zval **args[2];	
 	int call_result;
 	zval *object;
@@ -1327,17 +1153,23 @@ static int user_wrapper_stat_url(php_stream_wrapper *wrapper, char *url, int fla
 	/* create an instance of our class */
 	ALLOC_ZVAL(object);
 	object_init_ex(object, uwrap->ce);
-	Z_SET_REFCOUNT_P(object, 1);
-	Z_SET_ISREF_P(object);
+	ZVAL_REFCOUNT(object) = 1;
+	PZVAL_IS_REF(object) = 1;
 
 	if (context) {
-		add_property_resource(object, "context", context->rsrc_id);
-		zend_list_addref(context->rsrc_id);
+		MAKE_STD_ZVAL(zcontext);
+		php_stream_context_to_zval(context, zcontext);
+		add_property_zval(object, "context", zcontext);
+		/* The object property should be the only reference,
+		   'get rid' of our local reference. */
+		zval_ptr_dtor(&zcontext);
 	} else {
 		add_property_null(object, "context");
 	}
 
-	/* call it's stat_url method - set up params first */
+	/* call the stat_url method */
+	
+	/* call it's stream_open method - set up params first */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1465,76 +1297,12 @@ static int php_userstreamop_rewinddir(php_stream *stream, off_t offset, int when
 
 }
 
-static int php_userstreamop_cast(php_stream *stream, int castas, void **retptr TSRMLS_DC)
-{
-	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
-	zval func_name;
-	zval *retval = NULL;
-	zval *zcastas = NULL;
-	zval **args[1];
-	php_stream * intstream = NULL;
-	int call_result;
-	int ret = FAILURE;
-
-	ZVAL_STRINGL(&func_name, USERSTREAM_CAST, sizeof(USERSTREAM_CAST)-1, 0);
-
-	ALLOC_INIT_ZVAL(zcastas);
-	switch(castas) {
-	case PHP_STREAM_AS_FD_FOR_SELECT:
-		ZVAL_LONG(zcastas, PHP_STREAM_AS_FD_FOR_SELECT);
-		break;
-	default:
-		ZVAL_LONG(zcastas, PHP_STREAM_AS_STDIO);
-		break;
-	}
-	args[0] = &zcastas;
-
-	call_result = call_user_function_ex(NULL,
-			&us->object,
-			&func_name,
-			&retval,
-			1, args, 0, NULL TSRMLS_CC);
-
-	do {
-		if (call_result == FAILURE) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " is not implemented!",
-					us->wrapper->classname);
-			break;
-		}
-		if (retval == NULL || !zend_is_true(retval)) {
-			break;
-		}
-		php_stream_from_zval_no_verify(intstream, &retval);
-		if (!intstream) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " must return a stream resource",
-					us->wrapper->classname);
-			break;
-		}
-		if (intstream == stream) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " must not return itself",
-					us->wrapper->classname);
-			intstream = NULL;
-			break;
-		}
-		ret = php_stream_cast(intstream, castas, retptr, 1);
-	} while (0);
-
-	if (retval) {
-		zval_ptr_dtor(&retval);
-	}
-	if (zcastas) {
-		zval_ptr_dtor(&zcastas);
-	}
-
-	return ret;
-}
-
 php_stream_ops php_stream_userspace_ops = {
 	php_userstreamop_write, php_userstreamop_read,
 	php_userstreamop_close, php_userstreamop_flush,
 	"user-space",
 	php_userstreamop_seek,
-	php_userstreamop_cast,
+	NULL, /* cast */
 	php_userstreamop_stat, 
 	php_userstreamop_set_option,
 };
