@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2009 The PHP Group                                |
+  | Copyright (c) 1997-2006 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: xp_socket.c,v 1.33.2.2.2.6.2.10 2009/02/09 02:55:45 iliaa Exp $ */
+/* $Id: xp_socket.c,v 1.33.2.1 2006/01/01 12:50:18 sniper Exp $ */
 
 #include "php.h"
 #include "ext/standard/file.h"
@@ -31,14 +31,6 @@
 #include <sys/un.h>
 #endif
 
-#ifndef MSG_DONTWAIT
-# define MSG_DONTWAIT 0
-#endif
-
-#ifndef MSG_PEEK
-# define MSG_PEEK 0
-#endif
-
 php_stream_ops php_stream_generic_socket_ops;
 PHPAPI php_stream_ops php_stream_socket_ops;
 php_stream_ops php_stream_udp_socket_ops;
@@ -67,7 +59,7 @@ static size_t php_sockop_write(php_stream *stream, const char *buf, size_t count
 		ptimeout = &sock->timeout;
 
 retry:
-	didwrite = send(sock->socket, buf, count, (sock->is_blocked && ptimeout) ? MSG_DONTWAIT : 0);
+	didwrite = send(sock->socket, buf, count, 0);
 
 	if (didwrite <= 0) {
 		long err = php_socket_errno();
@@ -156,7 +148,7 @@ static size_t php_sockop_read(php_stream *stream, char *buf, size_t count TSRMLS
 			return 0;
 	}
 
-	nr_bytes = recv(sock->socket, buf, count, (sock->is_blocked && sock->timeout.tv_sec != -1) ? MSG_DONTWAIT : 0);
+	nr_bytes = recv(sock->socket, buf, count, 0);
 
 	stream->eof = (nr_bytes == 0 || (nr_bytes == -1 && php_socket_errno() != EWOULDBLOCK));
 
@@ -281,7 +273,7 @@ static int php_sockop_set_option(php_stream *stream, int option, int value, void
 				if (sock->socket == -1) {
 					alive = 0;
 				} else if (php_pollfd_for(sock->socket, PHP_POLLREADABLE|POLLPRI, &tv) > 0) {
-					if (0 >= recv(sock->socket, &buf, sizeof(buf), MSG_PEEK) && php_socket_errno() != EWOULDBLOCK) {
+					if (0 == recv(sock->socket, &buf, sizeof(buf), MSG_PEEK) && php_socket_errno() != EAGAIN) {
 						alive = 0;
 					}
 				}
@@ -289,11 +281,18 @@ static int php_sockop_set_option(php_stream *stream, int option, int value, void
 			}
 			
 		case PHP_STREAM_OPTION_BLOCKING:
+	
 			oldmode = sock->is_blocked;
+	
+			/* no need to change anything */
+			if (value == oldmode)
+				return oldmode;
+	
 			if (SUCCESS == php_set_sock_blocking(sock->socket, value TSRMLS_CC)) {
 				sock->is_blocked = value;
 				return oldmode;
 			}
+
 			return PHP_STREAM_OPTION_RETURN_ERR;
 
 		case PHP_STREAM_OPTION_READ_TIMEOUT:
@@ -370,24 +369,6 @@ static int php_sockop_set_option(php_stream *stream, int option, int value, void
 					return PHP_STREAM_OPTION_RETURN_OK;
 
 
-#ifdef HAVE_SHUTDOWN
-# ifndef SHUT_RD
-#  define SHUT_RD 0
-# endif
-# ifndef SHUT_WR
-#  define SHUT_WR 1
-# endif
-# ifndef SHUT_RDWR
-#  define SHUT_RDWR 2
-# endif
-				case STREAM_XPORT_OP_SHUTDOWN: {
-					static const int shutdown_how[] = {SHUT_RD, SHUT_WR, SHUT_RDWR};
-
-					xparam->outputs.returncode = shutdown(sock->socket, shutdown_how[xparam->how]);
-					return PHP_STREAM_OPTION_RETURN_OK;
-				}
-#endif
-				
 				default:
 					return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 			}
@@ -514,7 +495,7 @@ static inline char *parse_ip_address_ex(const char *str, int str_len, int *portn
 #ifdef HAVE_IPV6
 	char *p;
 
-	if (*(str) == '[' && str_len > 1) {
+	if (*(str) == '[') {
 		/* IPV6 notation to specify raw address with port (i.e. [fe80::1]:80) */
 		p = memchr(str + 1, ']', str_len - 2);
 		if (!p || *(p + 1) != ':') {
@@ -527,11 +508,8 @@ static inline char *parse_ip_address_ex(const char *str, int str_len, int *portn
 		return estrndup(str + 1, p - str - 1);
 	}
 #endif
-	if (str_len) {
-		colon = memchr(str, ':', str_len - 1);
-	} else {
-		colon = NULL;
-	}
+
+	colon = memchr(str, ':', str_len - 1);
 	if (colon) {
 		*portno = atoi(colon + 1);
 		host = estrndup(str, colon - str);
@@ -600,8 +578,8 @@ static inline int php_tcp_sockop_connect(php_stream *stream, php_netstream_data_
 		php_stream_xport_param *xparam TSRMLS_DC)
 {
 	char *host = NULL, *bindto = NULL;
-	int portno, bindport = 0;
-	int err = 0;
+	int portno, bindport;
+	int err;
 	int ret;
 	zval **tmpzval = NULL;
 
@@ -621,7 +599,7 @@ static inline int php_tcp_sockop_connect(php_stream *stream, php_netstream_data_
 		parse_unix_address(xparam, &unix_addr TSRMLS_CC);
 
 		ret = php_network_connect_socket(sock->socket,
-				(const struct sockaddr *)&unix_addr, (socklen_t) XtOffsetOf(struct sockaddr_un, sun_path) + xparam->inputs.namelen,
+				(const struct sockaddr *)&unix_addr, (socklen_t)sizeof(unix_addr),
 				xparam->op == STREAM_XPORT_OP_CONNECT_ASYNC, xparam->inputs.timeout,
 				xparam->want_errortext ? &xparam->outputs.error_text : NULL,
 				&err);
@@ -643,7 +621,6 @@ static inline int php_tcp_sockop_connect(php_stream *stream, php_netstream_data_
 			if (xparam->want_errortext) {
 				spprintf(&xparam->outputs.error_text, 0, "local_addr context option is not a string.");
 			}
-			efree(host);
 			return -1;
 		}
 		bindto = parse_ip_address_ex(Z_STRVAL_PP(tmpzval), Z_STRLEN_PP(tmpzval), &bindport, xparam->want_errortext, &xparam->outputs.error_text TSRMLS_CC);
@@ -716,10 +693,8 @@ static inline int php_tcp_sockop_accept(php_stream *stream, php_netstream_data_t
 
 			xparam->outputs.client = php_stream_alloc_rel(stream->ops, clisockdata, NULL, "r+");
 			if (xparam->outputs.client) {
+				/* TODO: addref ? */
 				xparam->outputs.client->context = stream->context;
-				if (stream->context) {
-					zend_list_addref(stream->context->rsrc_id);
-				}
 			}
 		}
 	}
@@ -754,8 +729,11 @@ static int php_tcp_sockop_set_option(php_stream *stream, int option, int value,
 					/* fall through */
 					;
 			}
+			
+			/* fall through */
+		default:
+			return php_sockop_set_option(stream, option, value, ptrparam TSRMLS_CC);
 	}
-	return php_sockop_set_option(stream, option, value, ptrparam TSRMLS_CC);
 }
 
 
