@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    | PHP 4.0 patches by Zeev Suraski <zeev@zend.com>                      |
    +----------------------------------------------------------------------+
  */
-/* $Id: mod_php5.c,v 1.10.2.4 2005/08/01 08:12:42 dmitry Exp $ */
+/* $Id: mod_php5.c,v 1.19.2.5 2006/01/06 18:06:38 rasmus Exp $ */
 
 #include "php_apache_http.h"
 #include "http_conf_globals.h"
@@ -220,7 +220,7 @@ static int sapi_apache_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 		send_http_header(r);
 	}   
 	return SAPI_HEADER_SENT_SUCCESSFULLY;
-} 
+}
 /* }}} */
 
 /* {{{ sapi_apache_register_server_variables
@@ -239,7 +239,7 @@ static void sapi_apache_register_server_variables(zval *track_vars_array TSRMLS_
 		if (elts[i].val) {
 			val = elts[i].val;
 		} else {
-			val = empty_string;
+			val = "";
 		}
 		php_register_variable(elts[i].key, val, track_vars_array  TSRMLS_CC);
 	}
@@ -409,6 +409,14 @@ static int sapi_apache_get_target_gid(gid_t *obj TSRMLS_DC)
 }
 /* }}} */
 
+/* {{{ php_apache_get_request_time
+ */
+static time_t php_apache_get_request_time(TSRMLS_D)
+{
+	return ((request_rec *)SG(server_context))->request_time;
+}
+/* }}} */
+
 /* {{{ sapi_module_struct apache_sapi_module
  */
 static sapi_module_struct apache_sapi_module = {
@@ -437,6 +445,7 @@ static sapi_module_struct apache_sapi_module = {
 
 	sapi_apache_register_server_variables,		/* register server variables */
 	php_apache_log_message,			/* Log message */
+	php_apache_get_request_time,	/* Get request time */
 
 	NULL,							/* php.ini path override */
 
@@ -483,28 +492,33 @@ static void init_request_info(TSRMLS_D)
 	SG(request_info).content_type = (char *) table_get(r->subprocess_env, "CONTENT_TYPE");
 	SG(request_info).content_length = (content_length ? atoi(content_length) : 0);
 	SG(sapi_headers).http_response_code = r->status;
+	SG(request_info).proto_num = r->proto_num;
 
 	if (r->headers_in) {
 		authorization = table_get(r->headers_in, "Authorization");
 	}
-	if (authorization
-		&& (!PG(safe_mode) || (PG(safe_mode) && !auth_type(r)))
-		&& !strcasecmp(getword(r->pool, &authorization, ' '), "Basic")) {
-		tmp = uudecode(r->pool, authorization);
-		SG(request_info).auth_user = NULL;
-		tmp_user = getword_nulls_nc(r->pool, &tmp, ':');
-		if (tmp_user) {
-			r->connection->user = pstrdup(r->connection->pool, tmp_user);
-			r->connection->ap_auth_type = "Basic";
-			SG(request_info).auth_user = estrdup(tmp_user);
+
+	SG(request_info).auth_user = NULL;
+	SG(request_info).auth_password = NULL;
+	SG(request_info).auth_digest = NULL;
+
+	if (authorization && (!PG(safe_mode) || (PG(safe_mode) && !auth_type(r)))) {
+		char *p = getword(r->pool, &authorization, ' ');
+		if (!strcasecmp(p, "Basic")) {
+			tmp = uudecode(r->pool, authorization);
+			tmp_user = getword_nulls_nc(r->pool, &tmp, ':');
+			if (tmp_user) {
+				r->connection->user = pstrdup(r->connection->pool, tmp_user);
+				r->connection->ap_auth_type = "Basic";
+				SG(request_info).auth_user = estrdup(tmp_user);
+			}
+			if (tmp) {
+				SG(request_info).auth_password = estrdup(tmp);
+			}
+		} else if (!strcasecmp(p, "Digest")) {
+			r->connection->ap_auth_type = "Digest";
+			SG(request_info).auth_digest = estrdup(authorization);
 		}
-		SG(request_info).auth_password = NULL;
-		if (tmp) {
-			SG(request_info).auth_password = estrdup(tmp);
-		}
-	} else {
-		SG(request_info).auth_user = NULL;
-		SG(request_info).auth_password = NULL;
 	}
 }
 /* }}} */
@@ -579,7 +593,6 @@ static int send_php(request_rec *r, int display_source_mode, char *filename)
 		 */
 		if (!AP(engine)) {
 			r->content_type = php_apache_get_default_mimetype(r TSRMLS_CC);
-			r->allowed |= (1 << METHODS) - 1;
 			zend_try {
 				zend_ini_deactivate(TSRMLS_C);
 			} zend_end_try();
@@ -591,7 +604,7 @@ static int send_php(request_rec *r, int display_source_mode, char *filename)
 
 		/* Apache 1.2 has a more complex mechanism for reading POST data */
 #if MODULE_MAGIC_NUMBER > 19961007
-		if ((retval = setup_client_block(r, REQUEST_CHUNKED_ERROR))) {
+		if ((retval = setup_client_block(r, REQUEST_CHUNKED_DECHUNK))) {
 			zend_try {
 				zend_ini_deactivate(TSRMLS_C);
 			} zend_end_try();
@@ -837,7 +850,6 @@ static int php_xbithack_handler(request_rec * r)
 	TSRMLS_FETCH();
 
 	if (!(r->finfo.st_mode & S_IXUSR)) {
-		r->allowed |= (1 << METHODS) - 1;
 		return DECLINED;
 	}
 	per_dir_conf = (HashTable *) get_module_config(r->per_dir_config, &php5_module);
@@ -845,7 +857,6 @@ static int php_xbithack_handler(request_rec * r)
 		zend_hash_apply((HashTable *) per_dir_conf, (apply_func_t) php_apache_alter_ini_entries TSRMLS_CC);
 	}
 	if(!AP(xbithack)) {
-		r->allowed |= (1 << METHODS) - 1;
 		zend_try {
 			zend_ini_deactivate(TSRMLS_C);
 		} zend_end_try();
