@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    |          David Sklar <sklar@student.net>                             |
    +----------------------------------------------------------------------+
  */
-/* $Id: php_apache.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php_apache.c,v 1.89.2.3 2006/01/01 12:50:18 sniper Exp $ */
 
 #include "php_apache_http.h"
 
@@ -34,13 +34,8 @@ php_apache_info_struct php_apache_info;
 
 #define SECTION(name)  PUTS("<h2>" name "</h2>\n")
 
-#ifndef PHP_WIN32
 extern module *top_module;
 extern module **ap_loaded_modules;
-#else
-extern  __declspec(dllimport) module *top_module;
-extern  __declspec(dllimport) module **ap_loaded_modules;
-#endif
 
 PHP_FUNCTION(virtual);
 PHP_FUNCTION(apache_request_headers);
@@ -56,57 +51,17 @@ PHP_FUNCTION(apache_reset_timeout);
 
 PHP_MINFO_FUNCTION(apache);
 
-ZEND_BEGIN_ARG_INFO(arginfo_apache_child_terminate, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_apache_note, 0, 0, 1)
-	ZEND_ARG_INFO(0, note_name)
-	ZEND_ARG_INFO(0, note_value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_apache_virtual, 0, 0, 1)
-	ZEND_ARG_INFO(0, filename)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_apache_request_headers, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_apache_response_headers, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_apache_setenv, 0, 0, 2)
-	ZEND_ARG_INFO(0, variable)
-	ZEND_ARG_INFO(0, value)
-	ZEND_ARG_INFO(0, walk_to_top)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_apache_lookup_uri, 0, 0, 1)
-	ZEND_ARG_INFO(0, uri)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_apache_get_version, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_apache_get_modules, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_apache_reset_timeout, 0)
-ZEND_END_ARG_INFO()
-
-
-
-const zend_function_entry apache_functions[] = {
-	PHP_FE(virtual,									arginfo_apache_virtual)
-	PHP_FE(apache_request_headers,					arginfo_apache_request_headers)
-	PHP_FE(apache_note,								arginfo_apache_note)
-	PHP_FE(apache_lookup_uri,						arginfo_apache_lookup_uri)
-	PHP_FE(apache_child_terminate,					arginfo_apache_child_terminate)
-	PHP_FE(apache_setenv,							arginfo_apache_setenv)
-	PHP_FE(apache_response_headers,					arginfo_apache_response_headers)
-	PHP_FE(apache_get_version,						arginfo_apache_get_version)
-	PHP_FE(apache_get_modules,						arginfo_apache_get_modules)
-	PHP_FE(apache_reset_timeout,					arginfo_apache_reset_timeout)
-	PHP_FALIAS(getallheaders, apache_request_headers, arginfo_apache_request_headers)
+zend_function_entry apache_functions[] = {
+	PHP_FE(virtual,									NULL)
+	PHP_FE(apache_request_headers,					NULL)
+	PHP_FE(apache_note,								NULL)
+	PHP_FE(apache_lookup_uri,						NULL)
+	PHP_FE(apache_child_terminate,					NULL)
+	PHP_FE(apache_setenv,							NULL)
+	PHP_FE(apache_response_headers,					NULL)
+	PHP_FE(apache_get_version,					NULL)
+	PHP_FE(apache_get_modules,					NULL)
+	PHP_FALIAS(getallheaders, apache_request_headers, NULL)
 	{NULL, NULL, NULL}
 };
 
@@ -157,6 +112,54 @@ zend_module_entry apache_module_entry = {
 	STANDARD_MODULE_PROPERTIES
 };
 
+/* {{{ proto bool apache_child_terminate(void)
+   Terminate apache process after this request */
+PHP_FUNCTION(apache_child_terminate)
+{
+#ifndef MULTITHREAD
+	if (AP(terminate_child)) {
+		ap_child_terminate( ((request_rec *)SG(server_context)) );
+		RETURN_TRUE;
+	} else { /* tell them to get lost! */
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "This function is disabled");
+		RETURN_FALSE;
+	}
+#else
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "This function is not supported in this build");
+		RETURN_FALSE;
+#endif
+}
+/* }}} */
+
+/* {{{ proto string apache_note(string note_name [, string note_value])
+   Get and set Apache request notes */
+PHP_FUNCTION(apache_note)
+{
+	zval **arg_name, **arg_val;
+	char *note_val;
+	int arg_count = ZEND_NUM_ARGS();
+
+	if (arg_count<1 || arg_count>2 ||
+		zend_get_parameters_ex(arg_count, &arg_name, &arg_val) ==FAILURE ) {
+		WRONG_PARAM_COUNT;
+	}
+	
+	convert_to_string_ex(arg_name);
+	note_val = (char *) table_get(((request_rec *)SG(server_context))->notes, (*arg_name)->value.str.val);
+	
+	if (arg_count == 2) {
+		convert_to_string_ex(arg_val);
+		table_set(((request_rec *)SG(server_context))->notes, (*arg_name)->value.str.val, (*arg_val)->value.str.val);
+	}
+
+	if (note_val) {
+		RETURN_STRING(note_val, 1);
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
 /* {{{ PHP_MINFO_FUNCTION
  */
 PHP_MINFO_FUNCTION(apache)
@@ -199,20 +202,20 @@ PHP_MINFO_FUNCTION(apache)
 	} 
 
 #ifdef APACHE_RELEASE
-	snprintf(output_buf, sizeof(output_buf), "%d", APACHE_RELEASE);
+	sprintf(output_buf, "%d", APACHE_RELEASE);
 	php_info_print_table_row(2, "Apache Release", output_buf);
 #endif
-	snprintf(output_buf, sizeof(output_buf), "%d", MODULE_MAGIC_NUMBER);
+	sprintf(output_buf, "%d", MODULE_MAGIC_NUMBER);
 	php_info_print_table_row(2, "Apache API Version", output_buf);
-	snprintf(output_buf, sizeof(output_buf), "%s:%u", serv->server_hostname, serv->port);
+	sprintf(output_buf, "%s:%u", serv->server_hostname, serv->port);
 	php_info_print_table_row(2, "Hostname:Port", output_buf);
 #if !defined(WIN32) && !defined(WINNT)
-	snprintf(output_buf, sizeof(output_buf), "%s(%d)/%d", user_name, (int)user_id, (int)group_id);
+	sprintf(output_buf, "%s(%d)/%d", user_name, (int)user_id, (int)group_id);
 	php_info_print_table_row(2, "User/Group", output_buf);
-	snprintf(output_buf, sizeof(output_buf), "Per Child: %d - Keep Alive: %s - Max Per Connection: %d", max_requests_per_child, serv->keep_alive ? "on":"off", serv->keep_alive_max);
+	sprintf(output_buf, "Per Child: %d - Keep Alive: %s - Max Per Connection: %d", max_requests_per_child, serv->keep_alive ? "on":"off", serv->keep_alive_max);
 	php_info_print_table_row(2, "Max Requests", output_buf);
 #endif
-	snprintf(output_buf, sizeof(output_buf), "Connection: %d - Keep-Alive: %d", serv->timeout, serv->keep_alive_timeout);
+	sprintf(output_buf, "Connection: %d - Keep-Alive: %d", serv->timeout, serv->keep_alive_timeout);
 	php_info_print_table_row(2, "Timeouts", output_buf);
 #if !defined(WIN32) && !defined(WINNT)
 /*
@@ -227,9 +230,9 @@ PHP_MINFO_FUNCTION(apache)
 		if ((p = strrchr(name, '.'))) {
 			*p='\0'; /* Cut off ugly .c extensions on module names */
 		}
-		strlcat(modulenames, name, sizeof(modulenames));
+		strcat(modulenames, name);
 		if (modp->next) {
-			strlcat(modulenames, ", ", sizeof(modulenames));
+			strcat(modulenames, ", ");
 		}
 	}
 	php_info_print_table_row(2, "Loaded Modules", modulenames);
@@ -289,51 +292,6 @@ PHP_MINFO_FUNCTION(apache)
 }
 /* }}} */
 
-/* {{{ proto bool apache_child_terminate(void)
-   Terminate apache process after this request */
-PHP_FUNCTION(apache_child_terminate)
-{
-#ifndef MULTITHREAD
-	if (AP(terminate_child)) {
-		ap_child_terminate( ((request_rec *)SG(server_context)) );
-		RETURN_TRUE;
-	} else { /* tell them to get lost! */
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "This function is disabled");
-		RETURN_FALSE;
-	}
-#else
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "This function is not supported in this build");
-		RETURN_FALSE;
-#endif
-}
-/* }}} */
-
-/* {{{ proto string apache_note(string note_name [, string note_value])
-   Get and set Apache request notes */
-PHP_FUNCTION(apache_note)
-{
-	char *note_name, *note_val = NULL;
-	int note_name_len, note_val_len;
-	char *old_val;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s", &note_name, &note_name_len, &note_val, &note_val_len) == FAILURE) {
-		return;
-	}
-
-	old_val = (char *) table_get(((request_rec *)SG(server_context))->notes, note_name);
-
-	if (note_val) {
-		table_set(((request_rec *)SG(server_context))->notes, note_name, note_val);
-	}
-
-	if (old_val) {
-		RETURN_STRING(old_val, 1);
-	}
-
-	RETURN_FALSE;
-}
-/* }}} */
-
 /* {{{ proto bool virtual(string filename)
    Perform an Apache sub-request */
 /* This function is equivalent to <!--#include virtual...-->
@@ -346,25 +304,23 @@ PHP_FUNCTION(apache_note)
  */
 PHP_FUNCTION(virtual)
 {
-	char *filename;
-	int filename_len;
+	zval **filename;
 	request_rec *rr = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
+	convert_to_string_ex(filename);
 	
-	if (!(rr = sub_req_lookup_uri (filename, ((request_rec *) SG(server_context))))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", filename);
-		if (rr)
-			destroy_sub_req (rr);
+	if (!(rr = sub_req_lookup_uri ((*filename)->value.str.val, ((request_rec *) SG(server_context))))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", (*filename)->value.str.val);
+		if (rr) destroy_sub_req (rr);
 		RETURN_FALSE;
 	}
 
 	if (rr->status != 200) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", filename);
-		if (rr)
-			destroy_sub_req (rr);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", (*filename)->value.str.val);
+		if (rr) destroy_sub_req (rr);
 		RETURN_FALSE;
 	}
 
@@ -372,35 +328,31 @@ PHP_FUNCTION(virtual)
 	php_header(TSRMLS_C);
 
 	if (run_sub_req(rr)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - request execution failed", filename);
-		if (rr)
-			destroy_sub_req (rr);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - request execution failed", (*filename)->value.str.val);
+		if (rr) destroy_sub_req (rr);
 		RETURN_FALSE;
+	} else {
+		if (rr) destroy_sub_req (rr);
+		RETURN_TRUE;
 	}
-
-	if (rr)
-		destroy_sub_req (rr);
-
-	RETURN_TRUE;
 }
 /* }}} */
 
 /* {{{ proto array getallheaders(void)
    Alias for apache_request_headers() */
 /* }}} */
-
 /* {{{ proto array apache_request_headers(void)
    Fetch all HTTP request headers */
 PHP_FUNCTION(apache_request_headers)
 {
-	array_header *env_arr;
-	table_entry *tenv;
-	int i;
-
-	array_init(return_value);
-	env_arr = table_elts(((request_rec *) SG(server_context))->headers_in);
-	tenv = (table_entry *)env_arr->elts;
-	for (i = 0; i < env_arr->nelts; ++i) {
+    array_header *env_arr;
+    table_entry *tenv;
+    int i;
+	
+    array_init(return_value);
+    env_arr = table_elts(((request_rec *) SG(server_context))->headers_in);
+    tenv = (table_entry *)env_arr->elts;
+    for (i = 0; i < env_arr->nelts; ++i) {
 		if (!tenv[i].key ||
 			(PG(safe_mode) &&
 			 !strncasecmp(tenv[i].key, "authorization", 13))) {
@@ -417,14 +369,14 @@ PHP_FUNCTION(apache_request_headers)
    Fetch all HTTP response headers */
 PHP_FUNCTION(apache_response_headers)
 {
-	array_header *env_arr;
-	table_entry *tenv;
-	int i;
+    array_header *env_arr;
+    table_entry *tenv;
+    int i;
 
-	array_init(return_value);
-	env_arr = table_elts(((request_rec *) SG(server_context))->headers_out);
-	tenv = (table_entry *)env_arr->elts;
-	for (i = 0; i < env_arr->nelts; ++i) {
+    array_init(return_value);
+    env_arr = table_elts(((request_rec *) SG(server_context))->headers_out);
+    tenv = (table_entry *)env_arr->elts;
+    for (i = 0; i < env_arr->nelts; ++i) {
 		if (!tenv[i].key) continue;
 		if (add_assoc_string(return_value, tenv[i].key, (tenv[i].val==NULL) ? "" : tenv[i].val, 1)==FAILURE) {
 			RETURN_FALSE;
@@ -437,13 +389,12 @@ PHP_FUNCTION(apache_response_headers)
    Set an Apache subprocess_env variable */
 PHP_FUNCTION(apache_setenv)
 {
-	int var_len, val_len;
-	zend_bool top=0;
+	int var_len, val_len, top=0;
 	char *var = NULL, *val = NULL;
 	request_rec *r = (request_rec *) SG(server_context);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|b", &var, &var_len, &val, &val_len, &top) == FAILURE) {
-		return;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|b", &var, &var_len, &val, &val_len, &top) == FAILURE) {
+        RETURN_FALSE;
 	}
 
 	while(top) {
@@ -460,22 +411,20 @@ PHP_FUNCTION(apache_setenv)
    Perform a partial request of the given URI to obtain information about it */
 PHP_FUNCTION(apache_lookup_uri)
 {
-	char *filename;
-	int filename_len;
+	zval **filename;
 	request_rec *rr=NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
+	convert_to_string_ex(filename);
 
-	if (!(rr = sub_req_lookup_uri(filename, ((request_rec *) SG(server_context))))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "URI lookup failed '%s'", filename);
+	if(!(rr = sub_req_lookup_uri((*filename)->value.str.val, ((request_rec *) SG(server_context))))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "URI lookup failed '%s'", (*filename)->value.str.val);
 		RETURN_FALSE;
 	}
-
 	object_init(return_value);
 	add_property_long(return_value,"status", rr->status);
-
 	if (rr->the_request) {
 		add_property_string(return_value,"the_request", rr->the_request, 1);
 	}
@@ -506,7 +455,6 @@ PHP_FUNCTION(apache_lookup_uri)
 	if (rr->boundary) {
 		add_property_string(return_value,"boundary", rr->boundary, 1);
 	}
-
 	add_property_long(return_value,"no_cache", rr->no_cache);
 	add_property_long(return_value,"no_local_copy", rr->no_local_copy);
 	add_property_long(return_value,"allowed", rr->allowed);
@@ -537,20 +485,19 @@ This function is most likely a bad idea.  Just playing with it for now.
 
 PHP_FUNCTION(apache_exec_uri)
 {
-	char *filename;
-	int filename_len;
+	zval **filename;
 	request_rec *rr=NULL;
 	TSRMLS_FETCH();
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
-		return;
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) {
+		WRONG_PARAM_COUNT;
 	}
+	convert_to_string_ex(filename);
 
-	if(!(rr = ap_sub_req_lookup_uri(filename, ((request_rec *) SG(server_context))))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "URI lookup failed", filename);
+	if(!(rr = ap_sub_req_lookup_uri((*filename)->value.str.val, ((request_rec *) SG(server_context))))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "URI lookup failed", (*filename)->value.str.val);
 		RETURN_FALSE;
 	}
-
 	RETVAL_LONG(ap_run_sub_req(rr));
 	ap_destroy_sub_req(rr);
 }
@@ -564,9 +511,9 @@ PHP_FUNCTION(apache_get_version)
 
 	if (apv && *apv) {
 		RETURN_STRING(apv, 1);
+	} else {
+		RETURN_FALSE;
 	}
-
-	RETURN_FALSE;
 }
 /* }}} */
 
