@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,6 +16,7 @@
    |          Ben Mansell <ben@zeus.com> (Zeus Support)                   |
    +----------------------------------------------------------------------+
  */
+/* $Id: php5isapi.c,v 1.8.2.1 2006/01/01 12:50:19 sniper Exp $ */
 
 #include "php.h"
 #include <httpext.h>
@@ -56,7 +57,7 @@ exception trapping when running under a debugger
 #endif
 */
 
-#define MAX_STATUS_LENGTH sizeof("xxxx LONGEST STATUS DESCRIPTION")
+#define MAX_STATUS_LENGTH sizeof("xxxx LONGEST POSSIBLE STATUS DESCRIPTION")
 #define ISAPI_SERVER_VAR_BUF_SIZE 1024
 #define ISAPI_POST_DATA_BUF 1024
 
@@ -244,8 +245,8 @@ static int sapi_isapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 	char *combined_headers, *combined_headers_ptr;
 	LPEXTENSION_CONTROL_BLOCK lpECB = (LPEXTENSION_CONTROL_BLOCK) SG(server_context);
 	HSE_SEND_HEADER_EX_INFO header_info;
-	char status_buf[MAX_STATUS_LENGTH];
 	sapi_header_struct default_content_type;
+	char *status_buf = NULL;
 
 	/* Obtain headers length */
 	if (SG(sapi_headers).send_default_content_type) {
@@ -276,10 +277,21 @@ static int sapi_isapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 		case 401:
 			header_info.pszStatus = "401 Authorization Required";
 			break;
-		default:
-			snprintf(status_buf, MAX_STATUS_LENGTH, "%d Undescribed", SG(sapi_headers).http_response_code);
+		default: {
+			const char *sline = SG(sapi_headers).http_status_line;
+			
+			status_buf = emalloc(MAX_STATUS_LENGTH + 1);
+			
+			/* httpd requires that r->status_line is set to the first digit of
+			 * the status-code: */
+			if (sline && strlen(sline) > 12 && strncmp(sline, "HTTP/1.", 7) == 0 && sline[8] == ' ') {
+				status_buf = estrndup(sline + 9, MAX_STATUS_LENGTH);
+			} else {
+				snprintf(status_buf, MAX_STATUS_LENGTH, "%d Undescribed", SG(sapi_headers).http_response_code);
+			}
 			header_info.pszStatus = status_buf;
 			break;
+		}
 	}
 	header_info.cchStatus = strlen(header_info.pszStatus);
 	header_info.pszHeader = combined_headers;
@@ -290,6 +302,9 @@ static int sapi_isapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 	lpECB->ServerSupportFunction(lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER_EX, &header_info, NULL, NULL);
 
 	efree(combined_headers);
+	if (status_buf) {
+		efree(status_buf);
+	}
 	return SAPI_HEADER_SENT_SUCCESSFULLY;
 }
 
@@ -305,7 +320,6 @@ static int php_isapi_startup(sapi_module_struct *sapi_module)
 }
 
 
-
 static int sapi_isapi_read_post(char *buffer, uint count_bytes TSRMLS_DC)
 {
 	LPEXTENSION_CONTROL_BLOCK lpECB = (LPEXTENSION_CONTROL_BLOCK) SG(server_context);
@@ -354,7 +368,7 @@ static char *sapi_isapi_read_cookies(TSRMLS_D)
 			efree(tmp_variable_buf);
 		}
 	}
-	return estrndup("", sizeof("")-1);
+	return STR_EMPTY_ALLOC();
 }
 
 
@@ -668,7 +682,8 @@ static sapi_module_struct isapi_sapi_module = {
 	sapi_isapi_read_cookies,		/* read Cookies */
 
 	sapi_isapi_register_server_variables,	/* register server variables */
-	NULL,									/* Log message */
+	NULL,							/* Log message */
+	NULL,							/* Get request time */
 
 	STANDARD_SAPI_MODULE_PROPERTIES
 };
