@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -27,7 +27,7 @@
 
 #include <unixlib/local.h>
 
-#define WEBJAMES_SAPI_VERSION "1.0.2"
+#define WEBJAMES_SAPI_VERSION "1.0.0"
 
 typedef struct {
 	struct connection *conn; /*structure holding all the details of the current request*/
@@ -42,44 +42,24 @@ static php_webjames_globals webjames_globals;
 static int sapi_webjames_ub_write(const char *str, uint str_length TSRMLS_DC)
 /*unbuffered write - send data straight out to socket*/
 {
-	int totalbytes = 0;
-
-	do {
-		int bytes;
-		bytes = webjames_writebuffer(WG(conn),str,str_length);
-		if (bytes<0) {
-			PG(connection_status) = PHP_CONNECTION_ABORTED;
-			if (!PG(ignore_user_abort)) {
-				zend_bailout();
-			}
-			return bytes;
+	int bytes;
+	
+	bytes = webjames_writebuffer(WG(conn),str,str_length);
+	if (bytes<0) {
+		PG(connection_status) = PHP_CONNECTION_ABORTED;
+		if (!PG(ignore_user_abort)) {
+			zend_bailout();
 		}
-		str += bytes;
-		str_length -= bytes;
-		totalbytes += bytes;
-	} while (str_length);
-	return totalbytes;
+	}
+	return bytes;
 }
 
 static void sapi_webjames_send_header(sapi_header_struct *sapi_header, void *server_context TSRMLS_DC)
 /*send an HTTP header*/
 {
-	char *header = sapi_header->header;
-	int len = sapi_header->header_len;
 	if (WG(conn)->flags.outputheaders) {
-		while (sapi_header && len > 0) {
-			int bytes;
-			bytes = webjames_writebuffer(WG(conn), header, len);
-			if (bytes<0) {
-				PG(connection_status) = PHP_CONNECTION_ABORTED;
-				if (!PG(ignore_user_abort)) {
-					zend_bailout();
-				}
-				return;
-			}
-			header += bytes;
-			len -= bytes;
-		}
+		if (sapi_header)
+			webjames_writebuffer(WG(conn), sapi_header->header, sapi_header->header_len);
 		webjames_writestring(WG(conn), "\r\n");
 	}
 }
@@ -116,7 +96,6 @@ static char *sapi_webjames_read_cookies(TSRMLS_D)
 static void sapi_webjames_register_variables(zval *track_vars_array TSRMLS_DC)
 {
 	char buf[BUF_SIZE + 1];
-	char *docroot;
 
 	buf[BUF_SIZE] = '\0';
 
@@ -126,9 +105,7 @@ static void sapi_webjames_register_variables(zval *track_vars_array TSRMLS_DC)
 	ADD_NUM("SERVER_PORT", port);
 	ADD_STRING("SERVER_ADMIN",configuration.webmaster);
 	ADD_STRING("GATEWAY_INTERFACE", "CGI/1.1");
-
-	docroot = __unixify(WG(conn)->homedir,0,NULL,1024,0);
-	if (docroot) ADD_STRING("DOCUMENT_ROOT", docroot);
+	ADD_STRING("DOCUMENT_ROOT", configuration.site);
 
 	ADD_FIELD("REQUEST_METHOD", methodstr);
 	ADD_FIELD("REQUEST_URI", requesturi);
@@ -170,7 +147,7 @@ static void webjames_module_main(TSRMLS_D)
 	char *path;
 
 	/* Convert filename to Unix format*/
-	__riscosify_control|=__RISCOSIFY_STRICT_UNIX_SPECS;
+	__riscosify_control|=__RISCOSIFY_DONT_CHECK_DIR;
 	path = __unixify(WG(conn)->filename,0,NULL,1024,0);
 	if (path) SG(request_info).path_translated = estrdup(path);
 
@@ -301,7 +278,6 @@ static sapi_module_struct sapi_module = {
 	sapi_webjames_register_variables,       /* register server variables */
 	NULL,									/* Log message */
 	NULL,									/* Get request time */
-	NULL,									/* Child terminate */
 
 	STANDARD_SAPI_MODULE_PROPERTIES
 };
